(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   5113278,     162949]*)
(*NotebookOutlinePosition[   5113952,     162973]*)
(*  CellTagsIndexPosition[   5113908,     162969]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[BoxData[
    \(<< FeynArts`\)], "Input",
  Background->GrayLevel[0.900008]],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"FeynArts 3"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"by Hagen Eck, Sepp Kueblbeck, and Thomas Hahn"\)], 
  "Print"],

Cell[BoxData[
    \(TraditionalForm\`"last revised 19 Oct 01"\)], "Print"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< FormCalc\\\\FormCalc.m\)], "Input",
  Background->GrayLevel[0.900008]],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"FormCalc 3.1"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"by Thomas Hahn"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"last revised 4 Feb 02"\)], "Print"]
}, Open  ]],

Cell[BoxData[
    \(<< FormCalc\\\\tools\\\\ScanGraphics.m\)], "Input",
  Background->GrayLevel[0.900008]],

Cell[CellGroupData[{

Cell[BoxData[
    \(time1\  = \ SessionTime[]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`1.70300000000000011`\)], "Output"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CKM\  = \ IndexDelta\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`IndexDelta\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(FortranDir\  := \ "\<fortran_mssm\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SetOptions[Paint, \ PaintLevel\  -> \ {Classes}, \ 
      ColumnsXRows\  -> \ {4, \ 5}]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`{PaintLevel \[Rule] {Classes}, ColumnsXRows \[Rule] {4, 5}, 
      AutoEdit \[Rule] True, SheetHeader \[Rule] Automatic, 
      Numbering \[Rule] Full, FieldNumbers \[Rule] False, 
      DisplayFunction \[RuleDelayed] $DisplayFunction}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \( (*\ take\ the\ comments\ out\ if\ you\ want\ the\ diagrams\ painted\n
      DoPaint[diags_, \ file_]\  := \ 
        \((\n\ \ If[\ FileType["\<diagrams\>"]\  =!= \ Directory, \n\ \ \ \ 
            CreateDirectory["\<diagrams\>"]\ ]; \n\ \ 
          Paint[diags, \n\ \ \ \ 
            DisplayFunction\  -> \ 
              \((Display["\<diagrams/\>"\ <>\ file\ <>\ "\<.ps\>", \ #]&)\)]
            \n)\)\n*) \)], "Input"],

Cell[BoxData[
    \(\( (*\ 
      Tweeked\ the\ Paint\  - \ 
        Turned\ off\ .ps\ out\ and\ added\ the\ Topology\ Editor\ switch\ *) 
      \nDoPaint[diags_, \ file_]\  := \ 
      \((\n\ \ If[\ FileType["\<diagrams\>"]\  =!= \ Directory, \n\ \ \ \ 
          CreateDirectory["\<diagrams\>"]\ ]; \n\ \ 
        Paint[diags, \ \ AutoEdit\  -> \ True]\n)\)\n\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Small[ME]\  = \ \(Small[ME2]\  = \ 0\)\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`0\)], "Output"],

Cell[CellGroupData[{

Cell[BoxData[
    \(eeHH\  = 
      \ {\(-F[2, \ {1}]\), \ F[2, \ {1}]}\  -> \ {\(-S[5]\), \ S[5]}\)], 
  "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`{\(-\(F(2, {1})\)\), F(2, {1})} \[Rule] {\(-\(S(5)\)\), S(5)}\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SetOptions[InsertFields, \n\ \ Model\  -> \ "\<MSSM\>", \ 
      Restrictions\  -> \ NoLightFHCoupling]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`{Model \[Rule] "MSSM", GenericModel \[Rule] "Lorentz", 
      InsertionLevel \[Rule] Classes, ExcludeParticles \[Rule] {}, 
      LastSelections \[Rule] {}, ExcludeFieldPoints \[Rule] {}, 
      Restrictions \[Rule] NoLightFHCoupling}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(inss\  := \ \(ins\  = \ InsertFields[tops, \ eeHH]\)\)], "Input"],

Cell[BoxData[
    \(\(tops\  = \ 
      CreateCTTopologies[1, \ 2\  -> \ 2, \n\ \ 
        ExcludeTopologies\  -> \ {TadpoleCTs, \ WFCorrectionCTs}]; \)\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\t (*\ 
      this\ is\ because\ there\ are\ no\ counter\ terms\ in\ MSSM.mod\ 
        \(yet : \)\ *) \n
    ins\  = \ InsertFields[tops, \ eeHH\  /. \ S[5]\  -> \ S[3], \ 
        Model\  -> \ "\<SMc\>"]; \nTheLabel[S[3]]\  = \ "\<H\>"; \)\)], 
  "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("initializing generic model file \
"\[InvisibleSpace]"C:\\PROGRA~1\\WOLFRA~1\\MATHEM~1\\3.0\\AddOns\\\
Applications\\Models\\Lorentz.gen"\),
        SequenceForm[ 
        "initializing generic model file ", 
          "C:\\PROGRA~1\\WOLFRA~1\\MATHEM~1\\3.0\\AddOns\\Applications\\\
Models\\Lorentz.gen"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"> $SVMixing is OFF"\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("generic model "\[InvisibleSpace]"Lorentz"\[InvisibleSpace]" \
initialized"\),
        SequenceForm[ "generic model ", "Lorentz", " initialized"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("initializing classes model file \
"\[InvisibleSpace]"C:\\PROGRA~1\\WOLFRA~1\\MATHEM~1\\3.0\\AddOns\\\
Applications\\Models\\SMc.mod"\),
        SequenceForm[ 
        "initializing classes model file ", 
          "C:\\PROGRA~1\\WOLFRA~1\\MATHEM~1\\3.0\\AddOns\\Applications\\\
Models\\SMc.mod"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]46
          \[InvisibleSpace]" particles (incl. antiparticles) in "\
\[InvisibleSpace]16\[InvisibleSpace]" classes"\),
        SequenceForm[ 
        "> ", 46, " particles (incl. antiparticles) in ", 16, " classes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"> $CounterTerms are ON"\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]88\[InvisibleSpace]" vertices"\),
        SequenceForm[ "> ", 88, " vertices"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]114
          \[InvisibleSpace]" counter terms of order 1"\),
        SequenceForm[ "> ", 114, " counter terms of order 1"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]6
          \[InvisibleSpace]" counter terms of order 2"\),
        SequenceForm[ "> ", 6, " counter terms of order 2"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("classes model "\[InvisibleSpace]"SMc"\[InvisibleSpace]" \
initialized"\),
        SequenceForm[ "classes model ", "SMc", " initialized"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 2 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "1 Generic, 2 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 3, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 4, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]5
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 5, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]6
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 6, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]7
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 2 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 7, ": ", "1 Generic, 2 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]8
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 4 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 8, ": ", "1 Generic, 4 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]9
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 9, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]10
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 10, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"3 Generic, 8 Classes insertions"\),
        SequenceForm[ "in total: ", "3 Generic, 8 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DoPaint[ins, \ "\<counter\>"]; \)\)], "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`Shape::"wait" \( : \ \) 
      "Starting Java and the topology editor. This may take a moment."\)], 
  "Message"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 diagrams"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"2 diagrams"\),
        SequenceForm[ "> Top. ", 2, ": ", "2 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"4 diagrams"\),
        SequenceForm[ "> Top. ", 3, ": ", "4 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.68182 .5 m
.67727 .50393 L
.67273 .50738 L
.66818 .50991 L
.66363 .51122 L
.65909 .51114 L
.65454 .50968 L
.65 .50703 L
.64545 .50351 L
.64091 .49955 L
.63636 .49564 L
.63182 .49228 L
.62727 .48987 L
.62273 .48871 L
.61818 .48894 L
.61364 .49054 L
.60909 .49331 L
.60455 .4969 L
.6 .50088 L
.59545 .50474 L
.59091 .50802 L
.58636 .51031 L
.58182 .51131 L
.57727 .51092 L
.57273 .50918 L
.56818 .5063 L
.56364 .50264 L
.55909 .49865 L
.55455 .49482 L
.55 .49164 L
.54546 .48948 L
.54091 .48863 L
.53636 .48918 L
.53182 .49106 L
.52727 .49404 L
.52273 .49777 L
.51818 .50176 L
.51364 .50553 L
.50909 .50862 L
.50455 .51064 L
.5 .51135 L
.49545 .51064 L
.49091 .50862 L
.48636 .50553 L
.48182 .50176 L
.47727 .49777 L
.47273 .49404 L
.46818 .49106 L
.46364 .48918 L
.45909 .48863 L
Mistroke
.45454 .48948 L
.45 .49164 L
.44545 .49482 L
.44091 .49865 L
.43636 .50264 L
.43182 .5063 L
.42727 .50918 L
.42273 .51092 L
.41818 .51131 L
.41364 .51031 L
.40909 .50802 L
.40455 .50474 L
.4 .50088 L
.39545 .4969 L
.39091 .49331 L
.38636 .49054 L
.38182 .48894 L
.37727 .48871 L
.37273 .48987 L
.36818 .49228 L
.36364 .49564 L
.35909 .49955 L
.35455 .50351 L
.35 .50703 L
.34546 .50968 L
.34091 .51114 L
.33637 .51122 L
.33182 .50991 L
.32727 .50738 L
.32273 .50393 L
.31818 .5 L
Mfstroke
gsave
.5 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.015 w
.65909 .47727 m
.70455 .52273 L
s
.65909 .52273 m
.70455 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N1) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.68182 .5 m
.67727 .50393 L
.67273 .50738 L
.66818 .50991 L
.66363 .51122 L
.65909 .51114 L
.65454 .50968 L
.65 .50703 L
.64545 .50351 L
.64091 .49955 L
.63636 .49564 L
.63182 .49228 L
.62727 .48987 L
.62273 .48871 L
.61818 .48894 L
.61364 .49054 L
.60909 .49331 L
.60455 .4969 L
.6 .50088 L
.59545 .50474 L
.59091 .50802 L
.58636 .51031 L
.58182 .51131 L
.57727 .51092 L
.57273 .50918 L
.56818 .5063 L
.56364 .50264 L
.55909 .49865 L
.55455 .49482 L
.55 .49164 L
.54546 .48948 L
.54091 .48863 L
.53636 .48918 L
.53182 .49106 L
.52727 .49404 L
.52273 .49777 L
.51818 .50176 L
.51364 .50553 L
.50909 .50862 L
.50455 .51064 L
.5 .51135 L
.49545 .51064 L
.49091 .50862 L
.48636 .50553 L
.48182 .50176 L
.47727 .49777 L
.47273 .49404 L
.46818 .49106 L
.46364 .48918 L
.45909 .48863 L
Mistroke
.45454 .48948 L
.45 .49164 L
.44545 .49482 L
.44091 .49865 L
.43636 .50264 L
.43182 .5063 L
.42727 .50918 L
.42273 .51092 L
.41818 .51131 L
.41364 .51031 L
.40909 .50802 L
.40455 .50474 L
.4 .50088 L
.39545 .4969 L
.39091 .49331 L
.38636 .49054 L
.38182 .48894 L
.37727 .48871 L
.37273 .48987 L
.36818 .49228 L
.36364 .49564 L
.35909 .49955 L
.35455 .50351 L
.35 .50703 L
.34546 .50968 L
.34091 .51114 L
.33637 .51122 L
.33182 .50991 L
.32727 .50738 L
.32273 .50393 L
.31818 .5 L
Mfstroke
gsave
.5 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.015 w
.65909 .47727 m
.70455 .52273 L
s
.65909 .52273 m
.70455 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N2) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.68182 .5 m
.67727 .50393 L
.67273 .50738 L
.66818 .50991 L
.66363 .51122 L
.65909 .51114 L
.65454 .50968 L
.65 .50703 L
.64545 .50351 L
.64091 .49955 L
.63636 .49564 L
.63182 .49228 L
.62727 .48987 L
.62273 .48871 L
.61818 .48894 L
.61364 .49054 L
.60909 .49331 L
.60455 .4969 L
.6 .50088 L
.59545 .50474 L
.59091 .50802 L
.58636 .51031 L
.58182 .51131 L
.57727 .51092 L
.57273 .50918 L
.56818 .5063 L
.56364 .50264 L
.55909 .49865 L
.55455 .49482 L
.55 .49164 L
.54546 .48948 L
.54091 .48863 L
.53636 .48918 L
.53182 .49106 L
.52727 .49404 L
.52273 .49777 L
.51818 .50176 L
.51364 .50553 L
.50909 .50862 L
.50455 .51064 L
.5 .51135 L
.49545 .51064 L
.49091 .50862 L
.48636 .50553 L
.48182 .50176 L
.47727 .49777 L
.47273 .49404 L
.46818 .49106 L
.46364 .48918 L
.45909 .48863 L
Mistroke
.45454 .48948 L
.45 .49164 L
.44545 .49482 L
.44091 .49865 L
.43636 .50264 L
.43182 .5063 L
.42727 .50918 L
.42273 .51092 L
.41818 .51131 L
.41364 .51031 L
.40909 .50802 L
.40455 .50474 L
.4 .50088 L
.39545 .4969 L
.39091 .49331 L
.38636 .49054 L
.38182 .48894 L
.37727 .48871 L
.37273 .48987 L
.36818 .49228 L
.36364 .49564 L
.35909 .49955 L
.35455 .50351 L
.35 .50703 L
.34546 .50968 L
.34091 .51114 L
.33637 .51122 L
.33182 .50991 L
.32727 .50738 L
.32273 .50393 L
.31818 .5 L
Mfstroke
gsave
.5 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.68182 .5 Mdot
.015 w
.29545 .47727 m
.34091 .52273 L
s
.29545 .52273 m
.34091 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N3) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.68182 .5 m
.67727 .50393 L
.67273 .50738 L
.66818 .50991 L
.66363 .51122 L
.65909 .51114 L
.65454 .50968 L
.65 .50703 L
.64545 .50351 L
.64091 .49955 L
.63636 .49564 L
.63182 .49228 L
.62727 .48987 L
.62273 .48871 L
.61818 .48894 L
.61364 .49054 L
.60909 .49331 L
.60455 .4969 L
.6 .50088 L
.59545 .50474 L
.59091 .50802 L
.58636 .51031 L
.58182 .51131 L
.57727 .51092 L
.57273 .50918 L
.56818 .5063 L
.56364 .50264 L
.55909 .49865 L
.55455 .49482 L
.55 .49164 L
.54546 .48948 L
.54091 .48863 L
.53636 .48918 L
.53182 .49106 L
.52727 .49404 L
.52273 .49777 L
.51818 .50176 L
.51364 .50553 L
.50909 .50862 L
.50455 .51064 L
.5 .51135 L
.49545 .51064 L
.49091 .50862 L
.48636 .50553 L
.48182 .50176 L
.47727 .49777 L
.47273 .49404 L
.46818 .49106 L
.46364 .48918 L
.45909 .48863 L
Mistroke
.45454 .48948 L
.45 .49164 L
.44545 .49482 L
.44091 .49865 L
.43636 .50264 L
.43182 .5063 L
.42727 .50918 L
.42273 .51092 L
.41818 .51131 L
.41364 .51031 L
.40909 .50802 L
.40455 .50474 L
.4 .50088 L
.39545 .4969 L
.39091 .49331 L
.38636 .49054 L
.38182 .48894 L
.37727 .48871 L
.37273 .48987 L
.36818 .49228 L
.36364 .49564 L
.35909 .49955 L
.35455 .50351 L
.35 .50703 L
.34546 .50968 L
.34091 .51114 L
.33637 .51122 L
.33182 .50991 L
.32727 .50738 L
.32273 .50393 L
.31818 .5 L
Mfstroke
gsave
.5 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.68182 .5 Mdot
.015 w
.29545 .47727 m
.34091 .52273 L
s
.29545 .52273 m
.34091 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N4) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .5 m
.32273 .49565 L
.32727 .49197 L
.33182 .4895 L
.33636 .48864 L
.34091 .4895 L
.34546 .49197 L
.35 .49565 L
.35455 .5 L
.35909 .50435 L
.36364 .50804 L
.36818 .5105 L
.37273 .51137 L
.37727 .5105 L
.38182 .50804 L
.38636 .50435 L
.39091 .5 L
.39545 .49566 L
.4 .49197 L
.40455 .48951 L
.40909 .48864 L
.41364 .48951 L
.41818 .49197 L
.42273 .49566 L
.42727 .5 L
.43182 .50435 L
.43636 .50804 L
.44091 .5105 L
.44546 .51137 L
.45 .5105 L
.45455 .50804 L
.45909 .50435 L
.46364 .5 L
.46818 .49565 L
.47273 .49197 L
.47727 .4895 L
.48182 .48864 L
.48636 .4895 L
.49091 .49197 L
.49545 .49565 L
.5 .5 L
s
gsave
.40909 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .5 m
.67727 .50435 L
.67273 .50803 L
.66818 .5105 L
.66364 .51136 L
.65909 .5105 L
.65454 .50803 L
.65 .50435 L
.64545 .5 L
.64091 .49565 L
.63636 .49196 L
.63182 .4895 L
.62727 .48863 L
.62273 .4895 L
.61818 .49196 L
.61364 .49565 L
.60909 .5 L
.60455 .50434 L
.6 .50803 L
.59545 .51049 L
.59091 .51136 L
.58636 .51049 L
.58182 .50803 L
.57727 .50434 L
.57273 .5 L
.56818 .49565 L
.56364 .49196 L
.55909 .4895 L
.55454 .48863 L
.55 .4895 L
.54545 .49196 L
.54091 .49565 L
.53636 .5 L
.53182 .50435 L
.52727 .50803 L
.52273 .5105 L
.51818 .51136 L
.51364 .5105 L
.50909 .50803 L
.50455 .50435 L
.5 .5 L
s
gsave
.59091 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.015 w
.47727 .47727 m
.52273 .52273 L
s
.47727 .52273 m
.52273 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N5) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .5 m
.32273 .49565 L
.32727 .49197 L
.33182 .4895 L
.33636 .48864 L
.34091 .4895 L
.34546 .49197 L
.35 .49565 L
.35455 .5 L
.35909 .50435 L
.36364 .50804 L
.36818 .5105 L
.37273 .51137 L
.37727 .5105 L
.38182 .50804 L
.38636 .50435 L
.39091 .5 L
.39545 .49566 L
.4 .49197 L
.40455 .48951 L
.40909 .48864 L
.41364 .48951 L
.41818 .49197 L
.42273 .49566 L
.42727 .5 L
.43182 .50435 L
.43636 .50804 L
.44091 .5105 L
.44546 .51137 L
.45 .5105 L
.45455 .50804 L
.45909 .50435 L
.46364 .5 L
.46818 .49565 L
.47273 .49197 L
.47727 .4895 L
.48182 .48864 L
.48636 .4895 L
.49091 .49197 L
.49545 .49565 L
.5 .5 L
s
gsave
.40909 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .5 m
.67727 .50435 L
.67273 .50803 L
.66818 .5105 L
.66364 .51136 L
.65909 .5105 L
.65454 .50803 L
.65 .50435 L
.64545 .5 L
.64091 .49565 L
.63636 .49196 L
.63182 .4895 L
.62727 .48863 L
.62273 .4895 L
.61818 .49196 L
.61364 .49565 L
.60909 .5 L
.60455 .50434 L
.6 .50803 L
.59545 .51049 L
.59091 .51136 L
.58636 .51049 L
.58182 .50803 L
.57727 .50434 L
.57273 .5 L
.56818 .49565 L
.56364 .49196 L
.55909 .4895 L
.55454 .48863 L
.55 .4895 L
.54545 .49196 L
.54091 .49565 L
.53636 .5 L
.53182 .50435 L
.52727 .50803 L
.52273 .5105 L
.51818 .51136 L
.51364 .5105 L
.50909 .50803 L
.50455 .50435 L
.5 .5 L
s
gsave
.59091 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.015 w
.47727 .47727 m
.52273 .52273 L
s
.47727 .52273 m
.52273 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N6) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .5 m
.32273 .49565 L
.32727 .49197 L
.33182 .4895 L
.33636 .48864 L
.34091 .4895 L
.34546 .49197 L
.35 .49565 L
.35455 .5 L
.35909 .50435 L
.36364 .50804 L
.36818 .5105 L
.37273 .51137 L
.37727 .5105 L
.38182 .50804 L
.38636 .50435 L
.39091 .5 L
.39545 .49566 L
.4 .49197 L
.40455 .48951 L
.40909 .48864 L
.41364 .48951 L
.41818 .49197 L
.42273 .49566 L
.42727 .5 L
.43182 .50435 L
.43636 .50804 L
.44091 .5105 L
.44546 .51137 L
.45 .5105 L
.45455 .50804 L
.45909 .50435 L
.46364 .5 L
.46818 .49565 L
.47273 .49197 L
.47727 .4895 L
.48182 .48864 L
.48636 .4895 L
.49091 .49197 L
.49545 .49565 L
.5 .5 L
s
gsave
.40909 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67727 .50435 L
.67273 .50803 L
.66818 .5105 L
.66364 .51136 L
.65909 .5105 L
.65454 .50803 L
.65 .50435 L
.64545 .5 L
.64091 .49565 L
.63636 .49196 L
.63182 .4895 L
.62727 .48863 L
.62273 .4895 L
.61818 .49196 L
.61364 .49565 L
.60909 .5 L
.60455 .50434 L
.6 .50803 L
.59545 .51049 L
.59091 .51136 L
.58636 .51049 L
.58182 .50803 L
.57727 .50434 L
.57273 .5 L
.56818 .49565 L
.56364 .49196 L
.55909 .4895 L
.55454 .48863 L
.55 .4895 L
.54545 .49196 L
.54091 .49565 L
.53636 .5 L
.53182 .50435 L
.52727 .50803 L
.52273 .5105 L
.51818 .51136 L
.51364 .5105 L
.50909 .50803 L
.50455 .50435 L
.5 .5 L
s
gsave
.59091 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.015 w
.47727 .47727 m
.52273 .52273 L
s
.47727 .52273 m
.52273 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N7) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .5 m
.32273 .49565 L
.32727 .49197 L
.33182 .4895 L
.33636 .48864 L
.34091 .4895 L
.34546 .49197 L
.35 .49565 L
.35455 .5 L
.35909 .50435 L
.36364 .50804 L
.36818 .5105 L
.37273 .51137 L
.37727 .5105 L
.38182 .50804 L
.38636 .50435 L
.39091 .5 L
.39545 .49566 L
.4 .49197 L
.40455 .48951 L
.40909 .48864 L
.41364 .48951 L
.41818 .49197 L
.42273 .49566 L
.42727 .5 L
.43182 .50435 L
.43636 .50804 L
.44091 .5105 L
.44546 .51137 L
.45 .5105 L
.45455 .50804 L
.45909 .50435 L
.46364 .5 L
.46818 .49565 L
.47273 .49197 L
.47727 .4895 L
.48182 .48864 L
.48636 .4895 L
.49091 .49197 L
.49545 .49565 L
.5 .5 L
s
gsave
.40909 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67727 .50435 L
.67273 .50803 L
.66818 .5105 L
.66364 .51136 L
.65909 .5105 L
.65454 .50803 L
.65 .50435 L
.64545 .5 L
.64091 .49565 L
.63636 .49196 L
.63182 .4895 L
.62727 .48863 L
.62273 .4895 L
.61818 .49196 L
.61364 .49565 L
.60909 .5 L
.60455 .50434 L
.6 .50803 L
.59545 .51049 L
.59091 .51136 L
.58636 .51049 L
.58182 .50803 L
.57727 .50434 L
.57273 .5 L
.56818 .49565 L
.56364 .49196 L
.55909 .4895 L
.55454 .48863 L
.55 .4895 L
.54545 .49196 L
.54091 .49565 L
.53636 .5 L
.53182 .50435 L
.52727 .50803 L
.52273 .5105 L
.51818 .51136 L
.51364 .5105 L
.50909 .50803 L
.50455 .50435 L
.5 .5 L
s
gsave
.59091 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.015 w
.47727 .47727 m
.52273 .52273 L
s
.47727 .52273 m
.52273 .47727 L
s
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N8) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000A0?ooo`040000003oool0
000000000080oooo0`0000020?ooo`060000003oool000000?ooo`000000oooo0P00000S0?ooo`04
0000003oool0000000000080oooo0`000000203oool000000000003oool000000?ooo`000000oooo
0P00000S0?ooo`040000003oool0000000000080oooo0`0000001`3oool000000000003oool00000
0?ooo`0000009P3oool010000000oooo0000000000020?ooo`<000001@3oool010000000oooo0000
003oool200000180oooo000A0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0000000200000080oooo0P00000R0?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002000000040?ooo`00000000000000
0280oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
0000008000000P3oool00`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool0oooo0P0000020?ooo`<0000000<0oooo0000000000004P3oool00140oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0000000800000
00<0oooo0000003oool08`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0
oooo100000000`3oool000000?ooo`0S0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool08@3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool3000000030?ooo`00
000000000180oooo000<0?ooo`8000000P3oool3000000030?ooo`000000oooo0080oooo0P000004
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0703oool200000080oooo0`000000
0`3oool000000?ooo`020?ooo`8000000P3oool00`000000oooo000000040?ooo`030000003oool0
oooo01`0oooo0P0000020?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<00000
0?ooo`0000000`3oool3000001d0oooo0P0000020?ooo`<0000000<0oooo0000003oool00P3oool2
000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`00303oool30000
03<0oooo0`00000c0?ooo`<00000<`3oool3000002P0oooo000=0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo02L0
oooo003I0?ooo`007P3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo04`0oooo0002
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo01T0oooo0P00000C0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool06@3oool2000001<0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0000000B0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0000000B0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0080oooo00040?ooo`80
0000603oool300000100oooo00<000000?ooo`3oool01P3oool2000001T0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`060?ooo`800000603oool300000100oooo00<000000?ooo`3o
ool01P3oool2000001T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`00
1P3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0E0?ooo`<00000
3P3oool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool0
5@3oool3000000l0oooo00<000000?ooo`3oool0103oool000L0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2
000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2
000002<0oooo00<000000?ooo`3oool01`3oool000P0oooo0`00000O0?ooo`<000004@3oool30000
01l0oooo0`00000A0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000002P3oool00080
oooo0P0000070?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000
oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0
oooo0P0000060?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo
0`0000070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00`0oooo000>0?ooo`8000005`3oool00`000000oooo0?ooo`0J
0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000oooo0?ooo`0J
0?ooo`8000005`3oool00`000000oooo0?ooo`0=0?ooo`00403oool300000080oooo0P0000020?oo
o`80000000@0oooo00000000000000000P3oool200000080oooo0P00000O0?ooo`030000003oool0
00000080oooo0P0000020?ooo`80000000@0oooo00000000000000000P3oool200000080oooo0P00
000O0?ooo`030000003oool000000080oooo0P0000020?ooo`80000000@0oooo0000000000000000
0P3oool200000080oooo0P00000O0?ooo`030000003oool0000000<0oooo00H000000?ooo`3oool0
oooo0000003oool300000080oooo0P0000030?ooo`030000003oool0oooo00h0oooo000A0?oooa@0
00008P3ooolD00000280oooo5000000R0?oooaD000004@3oool00100oooo00<000000?ooo`3oool0
0P0000020?ooo`<0000000<0oooo0000000000000P3oool200000080oooo0`0000090?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool00P0000020?ooo`<0000000<0oooo000000000000
0P3oool200000080oooo0`0000090?ooo`030000003oool0oooo01@0oooo00D000000?ooo`3oool0
oooo000000020?ooo`<0000000<0oooo0000000000000P3oool200000080oooo0`0000090?ooo`03
0000003oool0oooo01@0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool00000
008000000P3oool200000080oooo0P0000000`3oool000000?ooo`070?ooo`030000003oool0oooo
00D0oooo000>0?ooo`8000005P3oool2000000L0oooo0`00000B0?ooo`8000005P3oool2000000L0
oooo0`00000B0?ooo`8000005P3oool2000000L0oooo0`00000B0?ooo`8000005`3oool00`000000
oooo0?ooo`050?ooo`<000001@3oool000d0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`040?ooo`030000003oool0000000D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`002P3oool2000001h0oooo00<00000
0?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<00000
0?ooo`3oool04`3oool2000001l0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?oo
o`8000004P3oool200000200oooo0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?oo
o`8000002@3oool000P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0E0?ooo`80
0000403oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01D0oooo0P00000@0?ooo`03
0000003oool0oooo0240oooo0P00000@0?ooo`030000003oool0oooo0240oooo0P0000090?ooo`00
1`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0:0?ooo`800000603oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00X0oooo0P00000H0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`0000006@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`0000006@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`040?ooo`00103oool2
000000h0oooo0`00000I0?ooo`800000203oool2000000h0oooo0`00000I0?ooo`800000203oool2
000000l0oooo00<000000?ooo`3oool0603oool2000000P0oooo0P00000?0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3oool0;03oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0>0?ooo`<000006`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00h0oooo0`00000K0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo02h0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0
oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool002@0oooo0`00
000c0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo0000000c
0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000@
0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000A0?ooo`040000003oool0oooo00000080oooo
0`0000020?ooo`050000003oool000000?ooo`0000000P3oool00`000000oooo0?ooo`0Q0?ooo`04
0000003oool0oooo00000080oooo0`000000203oool000000000003oool000000?ooo`000000oooo
0P00000S0?ooo`040000003oool0000000000080oooo0`0000020?ooo`060000003oool000000?oo
o`000000oooo0P00000S0?ooo`040000003oool0000000000080oooo0`0000000`3oool000000000
00020?ooo`030000003oool0000001D0oooo000A0?ooo`040000003oool0oooo00000080oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool08@3o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`000000
0P0000020?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`020?ooo`030000003oool00000008000000P3oool00`000000oooo0?ooo`0Q0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030000
00030?ooo`00000000000180oooo000A0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool08@3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0oooo100000030?ooo`030000003oool0
oooo0200oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`000000
0P0000020?ooo`030000003oool0oooo0240oooo00L000000?ooo`3oool0oooo0000003oool00000
00D0oooo00<000000?ooo`0000000P0000000`3oool000000000000B0?ooo`00303oool200000080
oooo0`0000050?ooo`800000103oool00`000000oooo0?ooo`0Q0?ooo`8000000P3oool3000000D0
oooo0P0000020?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool0703oool200000080
oooo0`0000000`3oool000000?ooo`020?ooo`800000103oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo01d0oooo0P0000001P3oool00000000000000000oooo000000<0oooo0P000002
0?ooo`040000003oool0oooo000001L0oooo000<0?ooo`<00000<`3oool3000003<0oooo0`00000d
0?ooo`<000009`3oool000d0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo03@0oooo00<000000?ooo`3oool09P3oool00=T0oooo000I0?ooo`030000
003oool0oooo06T0oooo00<000000?ooo`3oool0D@3oool00080oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0503o
ool2000001P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0D0?ooo`800000603o
ool00`000000oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
000001L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
000001L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000C0?ooo`<000005@3oool00`00
0000oooo0?ooo`060?ooo`800000503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00H0oooo0P00000C0?ooo`<000005@3oool00`000000oooo0?ooo`060?ooo`800000503oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo00<0oooo00060?ooo`030000003oool0oooo02H0
oooo0P00000;0?ooo`030000003oool0oooo0100oooo0`00000C0?ooo`8000002`3oool00`000000
oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0@0?ooo`<00000503oool00`000000
oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00H0oooo
00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?oo
o`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?oo
o`070?ooo`00203oool3000001l0oooo0`00000A0?ooo`<000007`3oool300000140oooo0`00000O
0?ooo`<000004@3oool3000001l0oooo0`00000:0?ooo`000P3oool2000000L0oooo00<000000?oo
o`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?oo
o`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool3000000L0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`00
0`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
303oool000h0oooo0P00000G0?ooo`030000003oool0oooo01X0oooo0P00000G0?ooo`030000003o
ool0oooo01X0oooo0P00000G0?ooo`030000003oool0oooo01X0oooo0P00000G0?ooo`030000003o
ool0oooo00d0oooo000@0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?oo
o`020?ooo`080000003oool0oooo0?ooo`000000oooo0000003oool2000001l0oooo00D000000?oo
o`3oool0oooo000000030?ooo`030000003oool0oooo0080oooo00P000000?ooo`3oool0oooo0000
003oool000000?ooo`8000007`3oool01@000000oooo0000003oool0000000<0oooo00<000000?oo
o`3oool00P3oool01@000000oooo0?ooo`3oool0000000<0oooo0P00000O0?ooo`050000003oool0
00000?ooo`000000103oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0>0?ooo`004@3oool5000000030?ooo`000000000000H0000000<0
oooo0000000000001000000Q0?ooo`D0000000<0oooo0000000000001P0000000`3oool000000000
000400000240oooo00<000000?ooo`0000000P0000000`3oool0000000000006000000030?ooo`00
0000000000<000008P3oool00`000000oooo0000000;000000030?ooo`000000000000@000004@3o
ool00100oooo0`0000030?ooo`8000000P3oool200000080oooo0P0000030?ooo`@00000203oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P0000020?ooo`8000000`3oool2
00000080oooo100000080?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool2
00000080oooo0P0000030?ooo`8000000P3oool3000000T0oooo00<000000?ooo`3oool0503oool0
0`000000oooo000000030?ooo`8000000P3oool2000000<0oooo0P0000020?ooo`80000000<0oooo
0000003oool01`3oool00`000000oooo0?ooo`050?ooo`003P3oool2000001H0oooo0P0000070?oo
o`<000004P3oool2000001H0oooo0P0000070?ooo`<000004P3oool2000001H0oooo0P0000070?oo
o`<000004P3oool2000001L0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0103oool00`000000oooo000000050?ooo`00
303oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
1@3oool000/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`002@3oool2
00000200oooo0P00000B0?ooo`800000803oool300000140oooo0P00000P0?ooo`<000004@3oool2
00000240oooo0P0000090?ooo`00203oool00`000000oooo0?ooo`0Q0?ooo`800000403oool00`00
0000oooo0?ooo`0Q0?ooo`800000403oool00`000000oooo0?ooo`0Q0?ooo`800000403oool00`00
0000oooo0?ooo`0Q0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000
oooo0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00@0oooo
00040?ooo`800000:P3oool2000000P0oooo0P00000Z0?ooo`800000203oool2000002X0oooo0P00
00080?ooo`800000:`3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0/
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/
0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo02h0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<000000?oo
o`3oool00@3oool002@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool002@0oooo0`00000c0?ooo`<00000
<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003o
ool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000@0?ooo`00903oool0
0`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`030000003o
ool0oooo00@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F0?ooo`00
E03oool3000000P0oooo0`0000080?ooo`D000001@3oool010000000oooo0?ooo`0000070?ooo`04
0000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3000000/0oooo00<000000?ooo`3o
ool0103oool4000000L0oooo1000001F0?ooo`00M03oool010000000oooo0?ooo`0000070?ooo`04
0000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00001
\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(counter\  = \ CreateFeynAmp[ins]\  /. \ SW\  -> \ \(-SW\)\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 2 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "1 Generic, 2 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 2 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "1 Generic, 2 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 4 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 3, ": ", "1 Generic, 4 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"3 Generic, 8 Classes amplitudes"\),
        SequenceForm[ "in total: ", "3 Generic, 8 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"FeynAmpList", "(", 
            RowBox[{
            \(Model \[Rule] "SMc"\), ",", \(GenericModel \[Rule] "Lorentz"\), 
              ",", \(InsertionLevel \[Rule] Classes\), ",", 
              \(ExcludeParticles \[Rule] {}\), ",", 
              \(LastSelections \[Rule] {}\), ",", 
              \(ExcludeFieldPoints \[Rule] {}\), ",", 
              \(Restrictions \[Rule] 
                \(ExcludeFieldPoints 
                  \[Rule] {\((FieldPoint(_))\) (\(\(-\(F(2)\)\), F(2), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(2)\)\), F(1), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(3, {1, ___})\)\), 
                      F(3, {1, ___}), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(3, {2, ___})\)\), 
                      F(3, {2, ___}), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(4)\)\), F(4), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(4)\)\), F(3, {1, ___}), S
                      \)), \((FieldPoint(_))\) (\(\(-\(F(4)\)\), 
                      F(3, {2, ___}), S\))}\)\), ",", 
              RowBox[{"Process", "\[Rule]", 
                RowBox[{
                  RowBox[{"(", GridBox[{
                        {\(-\(F(2, {1})\)\), 
                          FormBox[\("p"\[InvisibleSpace]1\),
                            "TraditionalForm"], "ME"},
                        {\(F(2, {1})\), 
                          FormBox[\("p"\[InvisibleSpace]2\),
                            "TraditionalForm"], "ME"}
                        }], ")"}], "\[Rule]", 
                  RowBox[{"(", GridBox[{
                        {\(-\(S(3)\)\), 
                          FormBox[\("k"\[InvisibleSpace]1\),
                            "TraditionalForm"], "MW"},
                        {\(S(3)\), 
                          FormBox[\("k"\[InvisibleSpace]2\),
                            "TraditionalForm"], "MW"}
                        }], ")"}]}]}]}], ")"}], ")"}], "(", 
        RowBox[{
          RowBox[{"FeynAmp", "(", 
            RowBox[{
            \(GraphID(Topology == 1, Generic == 1)\), ",", \(Integral()\), 
              ",", 
              RowBox[{"RelativeCF", " ", 
                FormBox[
                  RowBox[{
                    RowBox[{\("v"\&"_"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]1\),
                            SequenceForm[ "p", 1],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}], ".", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "-"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "-"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}], "+", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "+"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "+"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}]}], ")"}], ".", 
                    RowBox[{\("u"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]2\),
                            SequenceForm[ "p", 2],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}]}],
                  "TraditionalForm"], " ", 
                FormBox[
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]2\),
                            SequenceForm[ "k", 2],
                            Editable->False],
                          "TraditionalForm"], "-", 
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]1\),
                            SequenceForm[ "k", 1],
                            Editable->False],
                          "TraditionalForm"]}], ")"}], "[", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"], "]"}],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                        SequenceForm[ "Lor", 1],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]1\),
                                SequenceForm[ "k", 1],
                                Editable->False],
                              "TraditionalForm"], "+", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                                SequenceForm[ "Gen", 5],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[
                    TagBox[\(G\_SSV\%\((1)\)\),
                      DisplayForm],
                    "TraditionalForm"], "(", 
                  FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                            SequenceForm[ "Gen", 5],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"-"\),
                                    Subscript[ "-"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "-"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"+"\),
                                    Subscript[ "+"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "+"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_SSV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                        "TraditionalForm"], ")"}], ",", "RelativeCF"}], "}"}],
                 "\[Rule]", 
                \(\((Insertions(Classes))\) (
                  \({0, \[ImaginaryI]\ EL, \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ EL\ \((
                      dZAA1\/2 + dZe1 + dZGp1 - 
                        \(dZZA1\ \((SW2 - CW2)\)\)\/\(4\ CW\ SW\))\), 1}, {
                    MZ, \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - 1\/2)\)\)\/\(CW\ SW
                            \)\)\), \(-\(\(\[ImaginaryI]\ EL\ SW\)\/CW\)\), 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\ 
                            \((\(-\(dSW1\/\(CW2\ SW\ \((SW2 - CW2)\)\)\)\) + 
                                dZe1 + dZGp1 + dZZZ1\/2 - 
                                \(CW\ dZAZ1\ SW\)\/\(SW2 - CW2\))\)\)\/\(2\ CW
                            \ SW\)\)\), 1}\))\)}]}], ")"}], ",", 
          RowBox[{"FeynAmp", "(", 
            RowBox[{
            \(GraphID(Topology == 2, Generic == 1)\), ",", \(Integral()\), 
              ",", 
              RowBox[{"RelativeCF", " ", 
                FormBox[
                  RowBox[{
                    RowBox[{\("v"\&"_"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]1\),
                            SequenceForm[ "p", 1],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}], ".", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "-"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((1)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "-"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}], "+", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "+"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((1)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "+"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}]}], ")"}], ".", 
                    RowBox[{\("u"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]2\),
                            SequenceForm[ "p", 2],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}]}],
                  "TraditionalForm"], " ", 
                FormBox[
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]2\),
                            SequenceForm[ "k", 2],
                            Editable->False],
                          "TraditionalForm"], "-", 
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]1\),
                            SequenceForm[ "k", 1],
                            Editable->False],
                          "TraditionalForm"]}], ")"}], "[", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"], "]"}],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                        SequenceForm[ "Lor", 1],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]1\),
                                SequenceForm[ "k", 1],
                                Editable->False],
                              "TraditionalForm"], "+", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                                SequenceForm[ "Gen", 5],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[
                    TagBox[\(G\_SSV\%\((0)\)\),
                      DisplayForm],
                    "TraditionalForm"], "(", 
                  FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                            SequenceForm[ "Gen", 5],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_SSV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"-"\),
                                    Subscript[ "-"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "-"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"+"\),
                                    Subscript[ "+"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "+"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", "RelativeCF"}], "}"}],
                 "\[Rule]", 
                RowBox[{\((Insertions(Classes))\), "(", 
                  RowBox[{
                    RowBox[{"{", 
                      RowBox[{"0", ",", \(\[ImaginaryI]\ EL\), ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAA1\/2\), "+", "dZe1", "-", 
                              \(\(dZZA1\ \((SW2 - 1\/2)\)\)\/\(2\ CW\ SW\)\), 
                              "+", 
                              RowBox[{\(1\/2\), " ", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{
                                      \(dZfL1(2, 1, 1)\), "\[InvisibleSpace]",
                                         
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                      SequenceForm[ 
                                        dZfL1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "+", 
                              \(1\/2\ \(dZfL1(2, 1, 1)\)\)}], ")"}]}], ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAA1\/2\), "+", "dZe1", "-", 
                              \(\(dZZA1\ SW\)\/\(2\ CW\)\), "+", 
                              RowBox[{\(1\/2\), " ", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{
                                      \(dZfR1(2, 1, 1)\), "\[InvisibleSpace]",
                                         
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                      SequenceForm[ 
                                        dZfR1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "+", 
                              \(1\/2\ \(dZfR1(2, 1, 1)\)\)}], ")"}]}], ",", 
                        "1"}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{
                      "MZ", ",", 
                        \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\)\/\(2\ CW\ 
                                SW\)\)\), ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAZ1\/2\), "-", 
                              \(\(\((dZe1 - dSW1\/\(CW2\ SW\))\)\ SW\)\/CW\), 
                              "-", \(\(dZZZ1\ \((SW2 - 1\/2)\)\)\/\(2\ CW\ SW
                                    \)\), "+", 
                              \(\(dZe1 - 
                                    \(dSW1\ \((SW2 - CW2)\)\)\/\(CW2\ SW
                                        \)\)\/\(2\ CW\ SW\)\), "-", 
                              FractionBox[
                                RowBox[{\((SW2 - 1\/2)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{\(1\/2\), " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(dZfL1(2, 1, 1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        dZfL1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "+", 
                                      \(1\/2\ \(dZfL1(2, 1, 1)\)\)}], ")"}]}],
                                 \(CW\ SW\)]}], ")"}]}], ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAZ1\/2\), "-", \(\(dZZZ1\ SW\)\/\(2\ CW\)\), 
                              "-", \(\(\((dZe1 - dSW1\/\(CW2\ SW\))\)\ SW
                                    \)\/CW\), "-", 
                              FractionBox[
                                RowBox[{"SW", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{\(1\/2\), " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(dZfR1(2, 1, 1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        dZfR1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "+", 
                                      \(1\/2\ \(dZfR1(2, 1, 1)\)\)}], ")"}]}],
                                 "CW"]}], ")"}]}], ",", "1"}], "}"}]}], 
                  ")"}]}]}], ")"}], ",", 
          RowBox[{"FeynAmp", "(", 
            RowBox[{
            \(GraphID(Topology == 3, Generic == 1)\), ",", \(Integral()\), 
              ",", 
              RowBox[{\(-\[ImaginaryI]\), " ", "RelativeCF", " ", 
                FormBox[
                  RowBox[{
                    RowBox[{\("v"\&"_"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]1\),
                            SequenceForm[ "p", 1],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}], ".", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]2\),
                                    SequenceForm[ "Lor", 2],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "-"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "-"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}], "+", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]2\),
                                    SequenceForm[ "Lor", 2],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "+"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "+"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}]}], ")"}], ".", 
                    RowBox[{\("u"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]2\),
                            SequenceForm[ "p", 2],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}]}],
                  "TraditionalForm"], " ", 
                FormBox[
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]2\),
                            SequenceForm[ "k", 2],
                            Editable->False],
                          "TraditionalForm"], "-", 
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]1\),
                            SequenceForm[ "k", 1],
                            Editable->False],
                          "TraditionalForm"]}], ")"}], "[", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]4\),
                        SequenceForm[ "Lor", 4],
                        Editable->False],
                      "TraditionalForm"], "]"}],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                        SequenceForm[ "Lor", 1],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                        SequenceForm[ "Lor", 3],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]4\),
                        SequenceForm[ "Lor", 4],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              RowBox[{"(", 
                                FormBox[
                                  
                                  InterpretationBox[\("k"\[InvisibleSpace]1\),
                                    
                                    SequenceForm[ "k", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}], "-", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                                SequenceForm[ "Gen", 5],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]1\),
                                SequenceForm[ "k", 1],
                                Editable->False],
                              "TraditionalForm"], "+", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]6\),
                                SequenceForm[ "Gen", 6],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[
                    TagBox[\(G\_SSV\%\((0)\)\),
                      DisplayForm],
                    "TraditionalForm"], "(", 
                  FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      FormBox[
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{"-", 
                                RowBox[{"(", 
                                  FormBox[
                                    
                                    InterpretationBox[
                                      \("k"\[InvisibleSpace]1\),
                                      SequenceForm[ "k", 1],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "-", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}], "[", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                              SequenceForm[ "Lor", 1],
                              Editable->False],
                            "TraditionalForm"], "]"}],
                        "TraditionalForm"], " ", 
                      FormBox[
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]1\),
                                  SequenceForm[ "k", 1],
                                  Editable->False],
                                "TraditionalForm"], "+", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}], "[", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                              SequenceForm[ "Lor", 3],
                              Editable->False],
                            "TraditionalForm"], "]"}],
                        "TraditionalForm"], " ", 
                      RowBox[{
                        FormBox[
                          TagBox[\(G\_VV\%\((1)\)\),
                            DisplayForm],
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[\(\((Mom(1))\) (\(KI1(2)\))\),
                            "TraditionalForm"], " ", 
                          FormBox[\(\((Mom(2))\) (\(KI1(1)\))\),
                            "TraditionalForm"]}], ")"}]}], "+", 
                    RowBox[{
                      RowBox[{
                        FormBox[\("g"\),
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                              SequenceForm[ "Lor", 1],
                              Editable->False],
                            "TraditionalForm"], ",", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                              SequenceForm[ "Lor", 3],
                              Editable->False],
                            "TraditionalForm"]}], ")"}], " ", 
                      RowBox[{
                        FormBox[
                          TagBox[\(G\_VV\%\((1)\)\),
                            DisplayForm],
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[\("g"\),
                            "TraditionalForm"], "(", \(KI1(1), KI1(2)\), 
                          ")"}], ")"}]}], "+", 
                    RowBox[{
                      RowBox[{
                        FormBox[\("g"\),
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                              SequenceForm[ "Lor", 1],
                              Editable->False],
                            "TraditionalForm"], ",", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                              SequenceForm[ "Lor", 3],
                              Editable->False],
                            "TraditionalForm"]}], ")"}], " ", 
                      FormBox[
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{"-", 
                                RowBox[{"(", 
                                  FormBox[
                                    
                                    InterpretationBox[
                                      \("k"\[InvisibleSpace]1\),
                                      SequenceForm[ "k", 1],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "-", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}], ".", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]1\),
                                  SequenceForm[ "k", 1],
                                  Editable->False],
                                "TraditionalForm"], "+", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}]}],
                        "TraditionalForm"], " ", 
                      RowBox[{
                        FormBox[
                          TagBox[\(G\_VV\%\((1)\)\),
                            DisplayForm],
                          "TraditionalForm"], "(", 
                        RowBox[{
                          RowBox[{
                            FormBox[\("g"\),
                              "TraditionalForm"], "(", \(KI1(1), KI1(2)\), 
                            ")"}], " ", 
                          FormBox[\(\(Mom(1)\).\(Mom(2)\)\),
                            "TraditionalForm"]}], ")"}]}]}], ")"}]}], ",", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                            SequenceForm[ "Gen", 5],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]6\),
                            SequenceForm[ "Gen", 6],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"-"\),
                                    Subscript[ "-"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "-"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"+"\),
                                    Subscript[ "+"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "+"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_SSV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_VV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      RowBox[{
                        FormBox[\(\((Mom(1))\) (\(KI1(2)\))\),
                          "TraditionalForm"], " ", 
                        FormBox[\(\((Mom(2))\) (\(KI1(1)\))\),
                          "TraditionalForm"]}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_VV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      RowBox[{
                        FormBox[\("g"\),
                          "TraditionalForm"], "(", \(KI1(1), KI1(2)\), ")"}], 
                      ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_VV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      RowBox[{
                        RowBox[{
                          FormBox[\("g"\),
                            "TraditionalForm"], "(", \(KI1(1), KI1(2)\), 
                          ")"}], " ", 
                        FormBox[\(\(Mom(1)\).\(Mom(2)\)\),
                          "TraditionalForm"]}], ")"}], ",", "RelativeCF"}], 
                  "}"}], "\[Rule]", 
                \(\((Insertions(Classes))\) (
                  \({0, 0, \[ImaginaryI]\ EL, \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ EL, \(-\[ImaginaryI]\)\ dZAA1, 0, 
                    \[ImaginaryI]\ dZAA1, 1}, {0, MZ, \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ EL, 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\)\/\(2\ CW\ SW
                            \)\)\), 
                    \[ImaginaryI]\ \((\(-\(dZAZ1\/2\)\) - dZZA1\/2)\), 
                    \(\[ImaginaryI]\ dZZA1\ MZ2\)\/2, 
                    \[ImaginaryI]\ \((dZAZ1\/2 + dZZA1\/2)\), 1}, {MZ, 0, 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - 1\/2)\)\)\/\(CW\ SW
                            \)\)\), \(-\(\(\[ImaginaryI]\ EL\ SW\)\/CW\)\), 
                    \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ \((\(-\(dZAZ1\/2\)\) - dZZA1\/2)\), 
                    \(\[ImaginaryI]\ dZZA1\ MZ2\)\/2, 
                    \[ImaginaryI]\ \((dZAZ1\/2 + dZZA1\/2)\), 1}, {MZ, MZ, 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - 1\/2)\)\)\/\(CW\ SW
                            \)\)\), \(-\(\(\[ImaginaryI]\ EL\ SW\)\/CW\)\), 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\)\/\(2\ CW\ SW
                            \)\)\), \(-\[ImaginaryI]\)\ dZZZ1, 
                    \[ImaginaryI]\ \((dMZsq1 + dZZZ1\ MZ2)\), 
                    \[ImaginaryI]\ dZZZ1, 1}\))\)}]}], ")"}]}], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(tops\  = \ CreateTopologies[0, \ 2\  -> \ 2]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DoPaint[inss, \ "\<born\>"]; \)\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("initializing classes model file \
"\[InvisibleSpace]"C:\\PROGRA~1\\WOLFRA~1\\MATHEM~1\\3.0\\AddOns\\\
Applications\\Models\\MSSM.mod"\),
        SequenceForm[ 
        "initializing classes model file ", 
          "C:\\PROGRA~1\\WOLFRA~1\\MATHEM~1\\3.0\\AddOns\\Applications\\\
Models\\MSSM.mod"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]94
          \[InvisibleSpace]" particles (incl. antiparticles) in "\
\[InvisibleSpace]23\[InvisibleSpace]" classes"\),
        SequenceForm[ 
        "> ", 94, " particles (incl. antiparticles) in ", 23, " classes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"> $CounterTerms are ON"\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]383\[InvisibleSpace]" vertices"\),
        SequenceForm[ "> ", 383, " vertices"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("classes model "\[InvisibleSpace]"MSSM"\[InvisibleSpace]" \
initialized"\),
        SequenceForm[ "classes model ", "MSSM", " initialized"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 2 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "1 Generic, 2 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 3, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 4, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"1 Generic, 2 Classes insertions"\),
        SequenceForm[ "in total: ", "1 Generic, 2 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 diagrams"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.68182 .5 m
.67727 .50393 L
.67273 .50738 L
.66818 .50991 L
.66363 .51122 L
.65909 .51114 L
.65454 .50968 L
.65 .50703 L
.64545 .50351 L
.64091 .49955 L
.63636 .49564 L
.63182 .49228 L
.62727 .48987 L
.62273 .48871 L
.61818 .48894 L
.61364 .49054 L
.60909 .49331 L
.60455 .4969 L
.6 .50088 L
.59545 .50474 L
.59091 .50802 L
.58636 .51031 L
.58182 .51131 L
.57727 .51092 L
.57273 .50918 L
.56818 .5063 L
.56364 .50264 L
.55909 .49865 L
.55455 .49482 L
.55 .49164 L
.54546 .48948 L
.54091 .48863 L
.53636 .48918 L
.53182 .49106 L
.52727 .49404 L
.52273 .49777 L
.51818 .50176 L
.51364 .50553 L
.50909 .50862 L
.50455 .51064 L
.5 .51135 L
.49545 .51064 L
.49091 .50862 L
.48636 .50553 L
.48182 .50176 L
.47727 .49777 L
.47273 .49404 L
.46818 .49106 L
.46364 .48918 L
.45909 .48863 L
Mistroke
.45454 .48948 L
.45 .49164 L
.44545 .49482 L
.44091 .49865 L
.43636 .50264 L
.43182 .5063 L
.42727 .50918 L
.42273 .51092 L
.41818 .51131 L
.41364 .51031 L
.40909 .50802 L
.40455 .50474 L
.4 .50088 L
.39545 .4969 L
.39091 .49331 L
.38636 .49054 L
.38182 .48894 L
.37727 .48871 L
.37273 .48987 L
.36818 .49228 L
.36364 .49564 L
.35909 .49955 L
.35455 .50351 L
.35 .50703 L
.34546 .50968 L
.34091 .51114 L
.33637 .51122 L
.33182 .50991 L
.32727 .50738 L
.32273 .50393 L
.31818 .5 L
Mfstroke
gsave
.5 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N1) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.68182 .5 m
.67727 .50393 L
.67273 .50738 L
.66818 .50991 L
.66363 .51122 L
.65909 .51114 L
.65454 .50968 L
.65 .50703 L
.64545 .50351 L
.64091 .49955 L
.63636 .49564 L
.63182 .49228 L
.62727 .48987 L
.62273 .48871 L
.61818 .48894 L
.61364 .49054 L
.60909 .49331 L
.60455 .4969 L
.6 .50088 L
.59545 .50474 L
.59091 .50802 L
.58636 .51031 L
.58182 .51131 L
.57727 .51092 L
.57273 .50918 L
.56818 .5063 L
.56364 .50264 L
.55909 .49865 L
.55455 .49482 L
.55 .49164 L
.54546 .48948 L
.54091 .48863 L
.53636 .48918 L
.53182 .49106 L
.52727 .49404 L
.52273 .49777 L
.51818 .50176 L
.51364 .50553 L
.50909 .50862 L
.50455 .51064 L
.5 .51135 L
.49545 .51064 L
.49091 .50862 L
.48636 .50553 L
.48182 .50176 L
.47727 .49777 L
.47273 .49404 L
.46818 .49106 L
.46364 .48918 L
.45909 .48863 L
Mistroke
.45454 .48948 L
.45 .49164 L
.44545 .49482 L
.44091 .49865 L
.43636 .50264 L
.43182 .5063 L
.42727 .50918 L
.42273 .51092 L
.41818 .51131 L
.41364 .51031 L
.40909 .50802 L
.40455 .50474 L
.4 .50088 L
.39545 .4969 L
.39091 .49331 L
.38636 .49054 L
.38182 .48894 L
.37727 .48871 L
.37273 .48987 L
.36818 .49228 L
.36364 .49564 L
.35909 .49955 L
.35455 .50351 L
.35 .50703 L
.34546 .50968 L
.34091 .51114 L
.33637 .51122 L
.33182 .50991 L
.32727 .50738 L
.32273 .50393 L
.31818 .5 L
Mfstroke
gsave
.5 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N2) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
000A0?ooo`040000003oool0oooo00000080oooo0`0000020?ooo`050000003oool000000?ooo`00
00000P3oool00`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo00000080oooo0`000000203o
ool000000000003oool000000?ooo`000000oooo0P00001o0?ooo`004@3oool010000000oooo0?oo
o`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`030000
003oool0oooo0240oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool00000008000000P3oool00`000000oooo0?ooo`1m0?ooo`004@3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`03
0000003oool0oooo0240oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?oo
o`@000000`3oool00`000000oooo0?ooo`1l0?ooo`00303oool200000080oooo0`0000050?ooo`80
0000103oool00`000000oooo0?ooo`0Q0?ooo`8000000P3oool3000000D0oooo0P0000020?ooo`03
0000003oool0000000@0oooo00<000000?ooo`3oool0O@3oool000`0oooo0`00000c0?ooo`<00000
U03oool000d0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`2C0?ooo`00f@3oool0
01T0oooo00<000000?ooo`3oool0_@3oool00080oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0503oool2000001P0
oooo00<000000?ooo`3oool0K@3oool000<0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0000000G0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo06h0oooo00040?ooo`8000004`3oool3000001D0oooo00<0
00000?ooo`3oool01P3oool2000001@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`1_0?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0@
0?ooo`<000004`3oool200000780oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0780oooo00080?ooo`80
00008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`1c0?ooo`00
203oool3000001l0oooo0`00000A0?ooo`<000007`3oool3000007H0oooo00020?ooo`8000001`3o
ool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703o
ool2000007L0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0M`3oool0
00<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
07P0oooo000>0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000
oooo0?ooo`1i0?ooo`00403oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3o
ool00P3oool01@000000oooo0?ooo`3oool0000000<0oooo0P00000O0?ooo`050000003oool0oooo
0?ooo`0000000`3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000`3o
ool2000007`0oooo000A0?ooo`D0000000<0oooo0000000000001P0000000`3oool0000000000003
00000280oooo1@0000000`3oool0000000000006000000030?ooo`000000000000<00000OP3oool0
0100oooo0`0000030?ooo`8000000P3oool200000080oooo0P0000030?ooo`<000002@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P0000020?ooo`8000000`3oool20000
0080oooo0`0000090?ooo`030000003oool0oooo0740oooo000>0?ooo`8000005P3oool2000000L0
oooo0`00000B0?ooo`8000005P3oool2000000L0oooo0`00001a0?ooo`003@3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`000000L@3oool000`0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0740oooo000;
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo07H0oooo00090?ooo`800000803oool200000180oooo0P00000P0?oo
o`<00000M@3oool000P0oooo00<000000?ooo`3oool08@3oool200000100oooo00<000000?ooo`3o
ool08@3oool2000007D0oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0780oooo00060?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?ooo`030000003o
ool0oooo0740oooo00040?ooo`800000:P3oool2000000P0oooo0P00000Z0?ooo`800000L@3oool0
00<0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
02`0oooo00<000000?ooo`3oool0KP3oool00080oooo00<000000?ooo`3oool0;P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0K@3oool002@0oooo
00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`1l0?ooo`00903oool3000003<0oooo0`00
001l0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003oool0000007`0oooo000T0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0O03oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool005@0oooo0`000008
0?ooo`<000002`3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo000000L0oooo00@0
00000?ooo`3oool00000EP3oool005@0oooo0`0000080?ooo`<00000203oool5000000D0oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo
0`00000;0?ooo`030000003oool0oooo00@0oooo100000070?ooo`@00000EP3oool007@0oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(born\  = \ CalcFeynAmp[bornamp\  = \ CreateFeynAmp[ins]]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 2 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "1 Generic, 2 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"1 Generic, 2 Classes amplitudes"\),
        SequenceForm[ "in total: ", "1 Generic, 2 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]1
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 1, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]0
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 0, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell["running FORM... ", "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm
    \`\((Amp({ME, ME} \[Rule] {MHp, MHp}))\) (
      \(\((8\ Alfa\ \[Pi]\ \(Den(S, 0)\) + 
              \((\(4\ \[Pi]\ SW2\ Alfa\)\/CW2 + \(2\ \[Pi]\ Alfa\)\/SW2 - 
                    \(2\ \[Pi]\ Alfa\)\/CW2 - 4\ \[Pi]\ Alfa)\)\ 
                \(Den(S, MZ2)\))\)\ \(Mat(F1)\) + 
        \((8\ Alfa\ \[Pi]\ \(Den(S, 0)\) + 
              \((\(4\ Alfa\ \[Pi]\ SW2\)\/CW2 - 4\ Alfa\ \[Pi])\)\ 
                \(Den(S, MZ2)\))\)\ \(Mat(F2)\)\))\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(counter\  = \ Head[bornamp]@@\ counter\)], "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"FeynAmpList", "(", 
            RowBox[{
            \(Model \[Rule] "MSSM"\), ",", \(GenericModel \[Rule] "Lorentz"\),
               ",", \(InsertionLevel \[Rule] Classes\), ",", 
              \(ExcludeParticles \[Rule] {}\), ",", 
              \(LastSelections \[Rule] {}\), ",", 
              \(ExcludeFieldPoints \[Rule] {}\), ",", 
              \(Restrictions \[Rule] 
                \(ExcludeFieldPoints 
                  \[Rule] {\((FieldPoint(_))\) (\(\(-\(F(2)\)\), F(2), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(2)\)\), F(1), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(3, {1, ___})\)\), 
                      F(3, {1, ___}), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(3, {2, ___})\)\), 
                      F(3, {2, ___}), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(4)\)\), F(4), S\)), 
                    \((FieldPoint(_))\) (\(\(-\(F(4)\)\), F(3, {1, ___}), S
                      \)), \((FieldPoint(_))\) (\(\(-\(F(4)\)\), 
                      F(3, {2, ___}), S\))}\)\), ",", 
              RowBox[{"Process", "\[Rule]", 
                RowBox[{
                  RowBox[{"(", GridBox[{
                        {\(-\(F(2, {1})\)\), 
                          FormBox[\("p"\[InvisibleSpace]1\),
                            "TraditionalForm"], "ME"},
                        {\(F(2, {1})\), 
                          FormBox[\("p"\[InvisibleSpace]2\),
                            "TraditionalForm"], "ME"}
                        }], ")"}], "\[Rule]", 
                  RowBox[{"(", GridBox[{
                        {\(-\(S(5)\)\), 
                          FormBox[\("k"\[InvisibleSpace]1\),
                            "TraditionalForm"], "MHp"},
                        {\(S(5)\), 
                          FormBox[\("k"\[InvisibleSpace]2\),
                            "TraditionalForm"], "MHp"}
                        }], ")"}]}]}]}], ")"}], ")"}], "(", 
        RowBox[{
          RowBox[{"FeynAmp", "(", 
            RowBox[{
            \(GraphID(Topology == 1, Generic == 1)\), ",", \(Integral()\), 
              ",", 
              RowBox[{"RelativeCF", " ", 
                FormBox[
                  RowBox[{
                    RowBox[{\("v"\&"_"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]1\),
                            SequenceForm[ "p", 1],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}], ".", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "-"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "-"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}], "+", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "+"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "+"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}]}], ")"}], ".", 
                    RowBox[{\("u"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]2\),
                            SequenceForm[ "p", 2],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}]}],
                  "TraditionalForm"], " ", 
                FormBox[
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]2\),
                            SequenceForm[ "k", 2],
                            Editable->False],
                          "TraditionalForm"], "-", 
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]1\),
                            SequenceForm[ "k", 1],
                            Editable->False],
                          "TraditionalForm"]}], ")"}], "[", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"], "]"}],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                        SequenceForm[ "Lor", 1],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]1\),
                                SequenceForm[ "k", 1],
                                Editable->False],
                              "TraditionalForm"], "+", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                                SequenceForm[ "Gen", 5],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[
                    TagBox[\(G\_SSV\%\((1)\)\),
                      DisplayForm],
                    "TraditionalForm"], "(", 
                  FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                            SequenceForm[ "Gen", 5],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"-"\),
                                    Subscript[ "-"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "-"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"+"\),
                                    Subscript[ "+"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "+"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_SSV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                        "TraditionalForm"], ")"}], ",", "RelativeCF"}], "}"}],
                 "\[Rule]", 
                \(\((Insertions(Classes))\) (
                  \({0, \[ImaginaryI]\ EL, \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ EL\ \((
                      dZAA1\/2 + dZe1 + dZGp1 - 
                        \(dZZA1\ \((SW2 - CW2)\)\)\/\(4\ CW\ SW\))\), 1}, {
                    MZ, \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - 1\/2)\)\)\/\(CW\ SW
                            \)\)\), \(-\(\(\[ImaginaryI]\ EL\ SW\)\/CW\)\), 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\ 
                            \((\(-\(dSW1\/\(CW2\ SW\ \((SW2 - CW2)\)\)\)\) + 
                                dZe1 + dZGp1 + dZZZ1\/2 - 
                                \(CW\ dZAZ1\ SW\)\/\(SW2 - CW2\))\)\)\/\(2\ CW
                            \ SW\)\)\), 1}\))\)}]}], ")"}], ",", 
          RowBox[{"FeynAmp", "(", 
            RowBox[{
            \(GraphID(Topology == 2, Generic == 1)\), ",", \(Integral()\), 
              ",", 
              RowBox[{"RelativeCF", " ", 
                FormBox[
                  RowBox[{
                    RowBox[{\("v"\&"_"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]1\),
                            SequenceForm[ "p", 1],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}], ".", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "-"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((1)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "-"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}], "+", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]1\),
                                    SequenceForm[ "Lor", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "+"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((1)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "+"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}]}], ")"}], ".", 
                    RowBox[{\("u"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]2\),
                            SequenceForm[ "p", 2],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}]}],
                  "TraditionalForm"], " ", 
                FormBox[
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]2\),
                            SequenceForm[ "k", 2],
                            Editable->False],
                          "TraditionalForm"], "-", 
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]1\),
                            SequenceForm[ "k", 1],
                            Editable->False],
                          "TraditionalForm"]}], ")"}], "[", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"], "]"}],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                        SequenceForm[ "Lor", 1],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]1\),
                                SequenceForm[ "k", 1],
                                Editable->False],
                              "TraditionalForm"], "+", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                                SequenceForm[ "Gen", 5],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[
                    TagBox[\(G\_SSV\%\((0)\)\),
                      DisplayForm],
                    "TraditionalForm"], "(", 
                  FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                    "TraditionalForm"], ")"}]}], ",", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                            SequenceForm[ "Gen", 5],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_SSV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"-"\),
                                    Subscript[ "-"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "-"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"+"\),
                                    Subscript[ "+"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "+"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", "RelativeCF"}], "}"}],
                 "\[Rule]", 
                RowBox[{\((Insertions(Classes))\), "(", 
                  RowBox[{
                    RowBox[{"{", 
                      RowBox[{"0", ",", \(\[ImaginaryI]\ EL\), ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAA1\/2\), "+", "dZe1", "-", 
                              \(\(dZZA1\ \((SW2 - 1\/2)\)\)\/\(2\ CW\ SW\)\), 
                              "+", 
                              RowBox[{\(1\/2\), " ", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{
                                      \(dZfL1(2, 1, 1)\), "\[InvisibleSpace]",
                                         
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                      SequenceForm[ 
                                        dZfL1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "+", 
                              \(1\/2\ \(dZfL1(2, 1, 1)\)\)}], ")"}]}], ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAA1\/2\), "+", "dZe1", "-", 
                              \(\(dZZA1\ SW\)\/\(2\ CW\)\), "+", 
                              RowBox[{\(1\/2\), " ", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{
                                      \(dZfR1(2, 1, 1)\), "\[InvisibleSpace]",
                                         
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                      SequenceForm[ 
                                        dZfR1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "+", 
                              \(1\/2\ \(dZfR1(2, 1, 1)\)\)}], ")"}]}], ",", 
                        "1"}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{
                      "MZ", ",", 
                        \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\)\/\(2\ CW\ 
                                SW\)\)\), ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAZ1\/2\), "-", 
                              \(\(\((dZe1 - dSW1\/\(CW2\ SW\))\)\ SW\)\/CW\), 
                              "-", \(\(dZZZ1\ \((SW2 - 1\/2)\)\)\/\(2\ CW\ SW
                                    \)\), "+", 
                              \(\(dZe1 - 
                                    \(dSW1\ \((SW2 - CW2)\)\)\/\(CW2\ SW
                                        \)\)\/\(2\ CW\ SW\)\), "-", 
                              FractionBox[
                                RowBox[{\((SW2 - 1\/2)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{\(1\/2\), " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(dZfL1(2, 1, 1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        dZfL1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "+", 
                                      \(1\/2\ \(dZfL1(2, 1, 1)\)\)}], ")"}]}],
                                 \(CW\ SW\)]}], ")"}]}], ",", 
                        RowBox[{"\[ImaginaryI]", " ", "EL", " ", 
                          RowBox[{"(", 
                            RowBox[{
                            \(dZAZ1\/2\), "-", \(\(dZZZ1\ SW\)\/\(2\ CW\)\), 
                              "-", \(\(\((dZe1 - dSW1\/\(CW2\ SW\))\)\ SW
                                    \)\/CW\), "-", 
                              FractionBox[
                                RowBox[{"SW", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{\(1\/2\), " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(dZfR1(2, 1, 1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        dZfR1[ 2, 1, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "+", 
                                      \(1\/2\ \(dZfR1(2, 1, 1)\)\)}], ")"}]}],
                                 "CW"]}], ")"}]}], ",", "1"}], "}"}]}], 
                  ")"}]}]}], ")"}], ",", 
          RowBox[{"FeynAmp", "(", 
            RowBox[{
            \(GraphID(Topology == 3, Generic == 1)\), ",", \(Integral()\), 
              ",", 
              RowBox[{\(-\[ImaginaryI]\), " ", "RelativeCF", " ", 
                FormBox[
                  RowBox[{
                    RowBox[{\("v"\&"_"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]1\),
                            SequenceForm[ "p", 1],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}], ".", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]2\),
                                    SequenceForm[ "Lor", 2],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "-"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"-"\),
                                        Subscript[ "-"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "-"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}], "+", 
                        RowBox[{
                          FormBox[
                            RowBox[{
                              RowBox[{
                                FormBox[\("ga"\),
                                  "TraditionalForm"], "(", 
                                FormBox[
                                  
                                  InterpretationBox[
                                    \("Lor"\[InvisibleSpace]2\),
                                    SequenceForm[ "Lor", 2],
                                    Editable->False],
                                  "TraditionalForm"], ")"}], ".", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{\("om"\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                    SequenceForm[ "om", 
                                      Subscript[ "+"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}],
                            "TraditionalForm"], " ", 
                          RowBox[{
                            FormBox[
                              TagBox[\(G\_FFV\%\((0)\)\),
                                DisplayForm],
                              "TraditionalForm"], "(", 
                            FormBox[
                              RowBox[{
                                RowBox[{
                                  FormBox[\("ga"\),
                                    "TraditionalForm"], "(", \(KI1(3)\), 
                                  ")"}], ".", 
                                RowBox[{"(", 
                                  FormBox[
                                    InterpretationBox[
                                      RowBox[{\("om"\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\_"+"\),
                                        Subscript[ "+"],
                                        Editable->False]}],
                                      SequenceForm[ "om", 
                                        Subscript[ "+"]],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}],
                              "TraditionalForm"], ")"}]}]}], ")"}], ".", 
                    RowBox[{\("u"\), "[", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("p"\[InvisibleSpace]2\),
                            SequenceForm[ "p", 2],
                            Editable->False],
                          "TraditionalForm"], ",", "ME"}], "]"}]}],
                  "TraditionalForm"], " ", 
                FormBox[
                  RowBox[{
                    RowBox[{"(", 
                      RowBox[{
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]2\),
                            SequenceForm[ "k", 2],
                            Editable->False],
                          "TraditionalForm"], "-", 
                        FormBox[
                          InterpretationBox[\("k"\[InvisibleSpace]1\),
                            SequenceForm[ "k", 1],
                            Editable->False],
                          "TraditionalForm"]}], ")"}], "[", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]4\),
                        SequenceForm[ "Lor", 4],
                        Editable->False],
                      "TraditionalForm"], "]"}],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                        SequenceForm[ "Lor", 1],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]2\),
                        SequenceForm[ "Lor", 2],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                RowBox[{
                  FormBox[\("g"\),
                    "TraditionalForm"], "(", 
                  RowBox[{
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                        SequenceForm[ "Lor", 3],
                        Editable->False],
                      "TraditionalForm"], ",", 
                    FormBox[
                      InterpretationBox[\("Lor"\[InvisibleSpace]4\),
                        SequenceForm[ "Lor", 4],
                        Editable->False],
                      "TraditionalForm"]}], ")"}], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              RowBox[{"(", 
                                FormBox[
                                  
                                  InterpretationBox[\("k"\[InvisibleSpace]1\),
                                    
                                    SequenceForm[ "k", 1],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}], "-", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                                SequenceForm[ "Gen", 5],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                FormBox[
                  FractionBox["1", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]1\),
                                SequenceForm[ "k", 1],
                                Editable->False],
                              "TraditionalForm"], "+", 
                            FormBox[
                              InterpretationBox[\("k"\[InvisibleSpace]2\),
                                SequenceForm[ "k", 2],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Mass", "(", 
                          RowBox[{"V", "(", 
                            FormBox[
                              InterpretationBox[\("Gen"\[InvisibleSpace]6\),
                                SequenceForm[ "Gen", 6],
                                Editable->False],
                              "TraditionalForm"], ")"}], ")"}], "2"]}]],
                  "TraditionalForm"], " ", 
                RowBox[{
                  FormBox[
                    TagBox[\(G\_SSV\%\((0)\)\),
                      DisplayForm],
                    "TraditionalForm"], "(", 
                  FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                    "TraditionalForm"], ")"}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      FormBox[
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{"-", 
                                RowBox[{"(", 
                                  FormBox[
                                    
                                    InterpretationBox[
                                      \("k"\[InvisibleSpace]1\),
                                      SequenceForm[ "k", 1],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "-", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}], "[", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                              SequenceForm[ "Lor", 1],
                              Editable->False],
                            "TraditionalForm"], "]"}],
                        "TraditionalForm"], " ", 
                      FormBox[
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]1\),
                                  SequenceForm[ "k", 1],
                                  Editable->False],
                                "TraditionalForm"], "+", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}], "[", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                              SequenceForm[ "Lor", 3],
                              Editable->False],
                            "TraditionalForm"], "]"}],
                        "TraditionalForm"], " ", 
                      RowBox[{
                        FormBox[
                          TagBox[\(G\_VV\%\((1)\)\),
                            DisplayForm],
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[\(\((Mom(1))\) (\(KI1(2)\))\),
                            "TraditionalForm"], " ", 
                          FormBox[\(\((Mom(2))\) (\(KI1(1)\))\),
                            "TraditionalForm"]}], ")"}]}], "+", 
                    RowBox[{
                      RowBox[{
                        FormBox[\("g"\),
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                              SequenceForm[ "Lor", 1],
                              Editable->False],
                            "TraditionalForm"], ",", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                              SequenceForm[ "Lor", 3],
                              Editable->False],
                            "TraditionalForm"]}], ")"}], " ", 
                      RowBox[{
                        FormBox[
                          TagBox[\(G\_VV\%\((1)\)\),
                            DisplayForm],
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[\("g"\),
                            "TraditionalForm"], "(", \(KI1(1), KI1(2)\), 
                          ")"}], ")"}]}], "+", 
                    RowBox[{
                      RowBox[{
                        FormBox[\("g"\),
                          "TraditionalForm"], "(", 
                        RowBox[{
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]1\),
                              SequenceForm[ "Lor", 1],
                              Editable->False],
                            "TraditionalForm"], ",", 
                          FormBox[
                            InterpretationBox[\("Lor"\[InvisibleSpace]3\),
                              SequenceForm[ "Lor", 3],
                              Editable->False],
                            "TraditionalForm"]}], ")"}], " ", 
                      FormBox[
                        RowBox[{
                          RowBox[{"(", 
                            RowBox[{
                              RowBox[{"-", 
                                RowBox[{"(", 
                                  FormBox[
                                    
                                    InterpretationBox[
                                      \("k"\[InvisibleSpace]1\),
                                      SequenceForm[ "k", 1],
                                      Editable->False],
                                    "TraditionalForm"], ")"}]}], "-", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}], ".", 
                          RowBox[{"(", 
                            RowBox[{
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]1\),
                                  SequenceForm[ "k", 1],
                                  Editable->False],
                                "TraditionalForm"], "+", 
                              FormBox[
                                InterpretationBox[\("k"\[InvisibleSpace]2\),
                                  SequenceForm[ "k", 2],
                                  Editable->False],
                                "TraditionalForm"]}], ")"}]}],
                        "TraditionalForm"], " ", 
                      RowBox[{
                        FormBox[
                          TagBox[\(G\_VV\%\((1)\)\),
                            DisplayForm],
                          "TraditionalForm"], "(", 
                        RowBox[{
                          RowBox[{
                            FormBox[\("g"\),
                              "TraditionalForm"], "(", \(KI1(1), KI1(2)\), 
                            ")"}], " ", 
                          FormBox[\(\(Mom(1)\).\(Mom(2)\)\),
                            "TraditionalForm"]}], ")"}]}]}], ")"}]}], ",", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]5\),
                            SequenceForm[ "Gen", 5],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{"Mass", "(", 
                      RowBox[{"V", "(", 
                        FormBox[
                          InterpretationBox[\("Gen"\[InvisibleSpace]6\),
                            SequenceForm[ "Gen", 6],
                            Editable->False],
                          "TraditionalForm"], ")"}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"-"\),
                                    Subscript[ "-"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "-"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_FFV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[
                        RowBox[{
                          RowBox[{
                            FormBox[\("ga"\),
                              "TraditionalForm"], "(", \(KI1(3)\), ")"}], ".",
                           
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{\("om"\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\_"+"\),
                                    Subscript[ "+"],
                                    Editable->False]}],
                                SequenceForm[ "om", 
                                  Subscript[ "+"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}],
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_SSV\%\((0)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      FormBox[\(\((Mom(1) - Mom(2))\)[KI1(3)]\),
                        "TraditionalForm"], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_VV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      RowBox[{
                        FormBox[\(\((Mom(1))\) (\(KI1(2)\))\),
                          "TraditionalForm"], " ", 
                        FormBox[\(\((Mom(2))\) (\(KI1(1)\))\),
                          "TraditionalForm"]}], ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_VV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      RowBox[{
                        FormBox[\("g"\),
                          "TraditionalForm"], "(", \(KI1(1), KI1(2)\), ")"}], 
                      ")"}], ",", 
                    RowBox[{
                      FormBox[
                        TagBox[\(G\_VV\%\((1)\)\),
                          DisplayForm],
                        "TraditionalForm"], "(", 
                      RowBox[{
                        RowBox[{
                          FormBox[\("g"\),
                            "TraditionalForm"], "(", \(KI1(1), KI1(2)\), 
                          ")"}], " ", 
                        FormBox[\(\(Mom(1)\).\(Mom(2)\)\),
                          "TraditionalForm"]}], ")"}], ",", "RelativeCF"}], 
                  "}"}], "\[Rule]", 
                \(\((Insertions(Classes))\) (
                  \({0, 0, \[ImaginaryI]\ EL, \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ EL, \(-\[ImaginaryI]\)\ dZAA1, 0, 
                    \[ImaginaryI]\ dZAA1, 1}, {0, MZ, \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ EL, 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\)\/\(2\ CW\ SW
                            \)\)\), 
                    \[ImaginaryI]\ \((\(-\(dZAZ1\/2\)\) - dZZA1\/2)\), 
                    \(\[ImaginaryI]\ dZZA1\ MZ2\)\/2, 
                    \[ImaginaryI]\ \((dZAZ1\/2 + dZZA1\/2)\), 1}, {MZ, 0, 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - 1\/2)\)\)\/\(CW\ SW
                            \)\)\), \(-\(\(\[ImaginaryI]\ EL\ SW\)\/CW\)\), 
                    \[ImaginaryI]\ EL, 
                    \[ImaginaryI]\ \((\(-\(dZAZ1\/2\)\) - dZZA1\/2)\), 
                    \(\[ImaginaryI]\ dZZA1\ MZ2\)\/2, 
                    \[ImaginaryI]\ \((dZAZ1\/2 + dZZA1\/2)\), 1}, {MZ, MZ, 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - 1\/2)\)\)\/\(CW\ SW
                            \)\)\), \(-\(\(\[ImaginaryI]\ EL\ SW\)\/CW\)\), 
                    \(-\(\(\[ImaginaryI]\ EL\ \((SW2 - CW2)\)\)\/\(2\ CW\ SW
                            \)\)\), \(-\[ImaginaryI]\)\ dZZZ1, 
                    \[ImaginaryI]\ \((dMZsq1 + dZZZ1\ MZ2)\), 
                    \[ImaginaryI]\ dZZZ1, 1}\))\)}]}], ")"}]}], ")"}], 
      TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(tops\  = \ 
      CreateTopologies[1, \ 2\  -> \ 2, \n\ \ 
        ExcludeTopologies\  -> 
          \ {Tadpoles, \ WFCorrections, \ Triangles, \ AllBoxes}]; \)\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DoPaint[inss, \ "\<self\>"]; \)\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 29 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 29 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 3, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"10 Generic, 117 Classes \
insertions"\),
        SequenceForm[ 
        "> Top. ", 4, ": ", "10 Generic, 117 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]5
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 5, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]6
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 6, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"12 Generic, 146 Classes insertions"\),
        
        SequenceForm[ "in total: ", "12 Generic, 146 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"29 diagrams"\),
        SequenceForm[ "> Top. ", 1, ": ", "29 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"117 diagrams"\),
        SequenceForm[ "> Top. ", 2, ": ", "117 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N1) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N2) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N3) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N4) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N5) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N6) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N7) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N8) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N9) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N10) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C11) show
80.562500 14.625000 moveto
(N11) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C12) show
80.562500 14.625000 moveto
(N12) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C13) show
80.562500 14.625000 moveto
(N13) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C14) show
80.562500 14.625000 moveto
(N14) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C15) show
80.562500 14.625000 moveto
(N15) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
gsave
.5 .71342 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C16) show
80.562500 14.625000 moveto
(N16) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
gsave
.5 .71342 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C17) show
80.562500 14.625000 moveto
(N17) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
gsave
.5 .71342 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C18) show
80.562500 14.625000 moveto
(N18) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
gsave
.5 .71342 -67 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C19) show
80.562500 14.625000 moveto
(N19) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C20) show
80.562500 14.625000 moveto
(N20) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000h0oooo00@000000?ooo`3o
ool000000P3oool300000080oooo00<000000?ooo`3oool00`3oool00`000000oooo000000020?oo
o`030000003oool0oooo01d0oooo00@000000?ooo`3oool000000P3oool300000080oooo00@00000
0?ooo`00000000000P3oool00`000000oooo000000020?ooo`040000003oool00000000001`0oooo
00@000000?ooo`3oool000000P3oool300000080oooo00@000000?ooo`00000000000P3oool00`00
0000oooo000000020?ooo`040000003oool00000000001`0oooo00@000000?ooo`3oool000000P3o
ool3000000040?ooo`00000000000?ooo`8000000P3oool010000000oooo0000003oool200000003
0?ooo`000000000000l0oooo000>0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool00P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool010000000oooo0?ooo`00000L
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo0000
000000020?ooo`<000000P3oool010000000oooo00000000000L0?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02000000040?ooo`0000000000
00000080oooo00<000000?ooo`3oool00P00000K0?ooo`040000003oool0oooo00000080oooo00<0
00000?ooo`3oool00P3oool01P000000oooo0000003oool000000?ooo`<000000P3oool01@000000
oooo0000003oool0000000h0oooo000>0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool010000000oooo0?ooo`00
000L0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo
0000000000020?ooo`<000000P3oool010000000oooo00000000000L0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool00P3oool01P000000oooo0000003oool000000?ooo`<00000
0P3oool01@000000oooo0000003oool0000001/0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`030?ooo`<000000P3oool3000000<0oooo0`00000?0?ooo`003@3oool3000000D0
oooo0P0000040?ooo`<0000000<0oooo0000003oool01@3oool3000001X0oooo0`0000050?ooo`80
00001@3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo01T0oooo0`0000050?oo
o`8000001@3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo01T0oooo0`000005
0?ooo`8000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@0
00000?ooo`3oool000003`3oool000`0oooo0P00000d0?ooo`800000=03oool2000003D0oooo0P00
000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03oool3000002L0oooo000=0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0d0?oo
o`030000003oool0oooo02H0oooo003I0?ooo`000P3oool00`000000oooo0?ooo`0^0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000
003oool0oooo0040oooo00030?ooo`030000003oool0oooo00h0oooo0P00000L0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool03P3oool2000001`0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0>0?ooo`800000703oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00h0oooo0P00000L0?ooo`030000003oool0oooo0080oooo00040?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`020?ooo`001@3oool20000
00h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`070?ooo`8000003P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00L0oooo0P00000>0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool01`3oool2000000h0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`030?ooo`001`3oool00`000000oooo0?ooo`0:0?ooo`<00000603oool2000000`0
oooo00<000000?ooo`3oool02P3oool3000001P0oooo0P00000<0?ooo`030000003oool0oooo00X0
oooo0`00000H0?ooo`800000303oool00`000000oooo0?ooo`0:0?ooo`<000006@3oool00`000000
oooo0?ooo`040?ooo`00203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00H0oooo
00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?oo
o`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?oo
o`070?ooo`002@3oool2000001l0oooo0`00000C0?ooo`030000003oool0oooo01d0oooo0`00000C
0?ooo`030000003oool0oooo01d0oooo0`00000C0?ooo`030000003oool0oooo01d0oooo0`00000:
0?ooo`000P3oool2000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?oo
o`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?oo
o`8000003@3oool2000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?oo
o`000P3oool3000000L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<0
00001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`800000
6@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo0P00000I0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2000001T0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`8000006@3oool00`000000oooo0?ooo`0<0?ooo`003`3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000003oool0
oooo008000000P3oool200000080oooo0P0000020?ooo`8000000P3oool4000001l0oooo00<00000
0?ooo`3oool00P0000020?ooo`8000000P3oool200000080oooo0P0000020?ooo`@000007`3oool0
0`000000oooo0?ooo`0200000080oooo0P0000020?ooo`8000000P3oool200000080oooo1000000O
0?ooo`030000003oool0oooo008000000P3oool200000080oooo0P0000020?ooo`8000000P3oool2
000000030?ooo`000000oooo00l0oooo000A0?oooa@000002P3oool00`000000oooo0?ooo`0E0?oo
oa@000002P3oool00`000000oooo0?ooo`0E0?ooo`<0000000<0oooo0000000000003P00000:0?oo
o`030000003oool0oooo01D0oooo0`0000000`3oool000000000000?000000T0oooo00<000000?oo
o`3oool01@3oool00100oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`80
00001P3oool00`000000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`<0
0000503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`<0
00001@3oool000h0oooo0P0000060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
103oool2000000L0oooo00<000000?ooo`0000004P3oool2000000H0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`040?ooo`8000001`3oool00`000000oooo0000000B0?ooo`800000
1P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00@0oooo0P0000070?ooo`030000
003oool000000180oooo0P0000060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`050?ooo`030000003oool0000000D0oooo000=0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00D0oooo000<0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00/0oooo000:0?ooo`8000002P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04`3oool20000
00X0oooo00<000000?ooo`3oool01P3oool2000000T0oooo00<000000?ooo`3oool04`3oool20000
00X0oooo00<000000?ooo`3oool01P3oool2000000T0oooo00<000000?ooo`3oool04`3oool20000
00X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00090?ooo`8000002`3oool00`000000oooo0?ooo`0B0?ooo`8000004P3oool2000000/0
oooo00<000000?ooo`3oool04P3oool300000140oooo0P00000;0?ooo`030000003oool0oooo0180
oooo0`00000A0?ooo`8000002`3oool00`000000oooo0?ooo`0C0?ooo`8000002@3oool000P0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`050?ooo`800000303oool200000100oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`050?ooo`800000303oool200000100oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`800000303oool200000100oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`800000303oool2000000T0oooo
00070?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103oool2000000h0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo0P00
000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`040?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0103oool2000000h0oooo00<000000?ooo`3oool01P3oool000H0oooo00<00000
0?ooo`3oool03`3oool2000000060?ooo`00000000000000003oool000003`3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00l0oooo0P0000001P3oool00000000000000000oooo0000
00l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`80000000H0oooo0000
0000000000000?ooo`00000?0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03`3o
ool2000000060?ooo`00000000000000003oool00000403oool00`000000oooo0?ooo`040?ooo`00
103oool2000001@0oooo0P0000020?ooo`<000003`3oool2000000P0oooo0P00000D0?ooo`800000
0P3oool3000000l0oooo0P0000080?ooo`800000503oool200000080oooo0`00000?0?ooo`800000
203oool2000001<0oooo0`0000020?ooo`<00000403oool00`000000oooo0?ooo`030?ooo`000`3o
ool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool002@0
oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo
0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo
0000000@0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`005`3oool00`000000oooo0?ooo`0c
0?ooo`800000=03oool4000003<0oooo00<000000?ooo`3oool0703oool001L0oooo0`00000c0?oo
o`030000003oool0000003<0oooo00@000000?ooo`0000000000<`3oool3000001`0oooo000G0?oo
o`040000003oool000000?ooo`800000<03oool200000080oooo0P00000`0?ooo`040000003oool0
00000?ooo`800000<@3oool00`000000oooo0000000L0?ooo`005`3oool010000000oooo0?ooo`3o
ool200000340oooo00<000000?ooo`3oool00P00000a0?ooo`80000000<0oooo000000000000<@3o
ool00`000000oooo0?ooo`0L0?ooo`006`3oool2000003@0oooo0P00000d0?ooo`800000D03oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`003P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool01000
0000oooo0000000000020?ooo`030000003oool000000080oooo00@000000?ooo`0000000000703o
ool010000000oooo0?ooo`0000020?ooo`<000000P3oool00`000000oooo0?ooo`030?ooo`030000
003oool000000080oooo00<000000?ooo`3oool07@3oool010000000oooo0?ooo`0000020?ooo`<0
00000P3oool010000000oooo0000000000020?ooo`030000003oool000000080oooo00@000000?oo
o`0000000000703oool010000000oooo0?ooo`0000020?ooo`<000000P3oool010000000oooo0000
000000020?ooo`030000003oool000000080oooo00@000000?ooo`00000000003`3oool000h0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo0000
0080oooo0`0000020?ooo`040000003oool0oooo000001`0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`020?ooo`040000003oool0000000000080oooo0`0000020?ooo`040000
003oool00000000001`0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0080000000@0oooo00000000000000000P3oool00`000000oooo0?ooo`02
000001/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool000000080000000@0oooo00000000000000000P3oool00`000000oooo00000002000000h0oooo
000>0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo
0?ooo`0000020?ooo`<000000P3oool010000000oooo0?ooo`00000L0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool00P3oool010000000oooo0000000000020?ooo`<000000P3o
ool010000000oooo00000000000L0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo000000030?ooo`<000000P3oool00`000000oooo0000000M0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo000000030?oo
o`<000000P3oool00`000000oooo0000000@0?ooo`003@3oool3000000D0oooo0P0000050?ooo`04
0000003oool0oooo000000L0oooo00<000000?ooo`3oool06@3oool3000000D0oooo0P0000080?oo
o`030000003oool0oooo0240oooo0`0000050?ooo`8000001@3oool010000000oooo0?ooo`000007
0?ooo`030000003oool0oooo01T0oooo0`0000050?ooo`8000001@3oool010000000oooo0?ooo`00
00070?ooo`030000003oool0oooo00d0oooo000<0?ooo`800000=03oool2000003@0oooo0P00000e
0?ooo`800000:03oool000`0oooo0`00000c0?ooo`<00000<`3oool3000003@0oooo0`00000W0?oo
o`003@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0=03oool00`000000oooo0?ooo`0V0?ooo`00f@3oool00=T0oooo00020?ooo`030000003oool0
oooo00l0oooo0P00000M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool2
000001d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`8000007@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo0P00000M0?ooo`030000003oool0
oooo0040oooo00030?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`020?ooo`00103oool2000000l0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`060?ooo`8000003`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00H0
oooo0P00000?0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool01P3oool2000000l0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo
0?ooo`0;0?ooo`<00000603oool2000000/0oooo00<000000?ooo`3oool02`3oool3000001P0oooo
0P00000;0?ooo`030000003oool0oooo00/0oooo0`00000H0?ooo`8000002`3oool00`000000oooo
0?ooo`0;0?ooo`<000006@3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>
0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>
0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`00203oool3000001l0oooo0`00000A0?oo
o`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000:0?oo
o`000P3oool2000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`03
0000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`80
00003@3oool2000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`00
0P3oool3000000L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<00000
1P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0303oool000h0oooo0P00000G0?ooo`030000003oool0
oooo01X0oooo0P00000G0?ooo`030000003oool0oooo01X0oooo0P00000G0?ooo`030000003oool0
oooo01X0oooo0P00000G0?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000003oool0oooo
008000000P3oool200000080oooo0P0000020?ooo`8000000P3oool4000001l0oooo00<000000?oo
o`3oool00P0000020?ooo`8000000P3oool200000080oooo0P0000020?ooo`@000007`3oool00`00
0000oooo0?ooo`0200000080oooo0P0000020?ooo`8000000P3oool200000080oooo1000000O0?oo
o`030000003oool0oooo008000000P3oool200000080oooo0P0000020?ooo`8000000P3oool20000
00030?ooo`000000oooo00l0oooo000A0?oooa@000008P3ooolD00000280oooo0`0000000`3oool0
00000000000>00000280oooo0`0000000`3oool000000000000?00000140oooo000@0?ooo`030000
003oool0oooo00@0oooo0P0000040?ooo`8000001P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0103oool2000000D0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
103oool2000000D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3o
ool2000000H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`8000001`3o
ool300000180oooo0P0000060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0103o
ool2000000L0oooo0`00000B0?ooo`8000001P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00@0oooo0P0000070?ooo`<000004P3oool2000000H0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo0`0000050?ooo`003@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0000000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool0
0`000000oooo000000050?ooo`00303oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00D0oooo000:0?ooo`8000002P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool04`3oool2000000X0oooo00<000000?ooo`3oool01P3oool2000000T0
oooo00<000000?ooo`3oool04`3oool2000000X0oooo00<000000?ooo`3oool01P3oool2000000T0
oooo00<000000?ooo`3oool04`3oool2000000X0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00090?ooo`8000002`3oool00`000000oooo
0?ooo`0B0?ooo`8000004P3oool2000000/0oooo00<000000?ooo`3oool04P3oool300000140oooo
0P00000;0?ooo`030000003oool0oooo0180oooo0`00000A0?ooo`8000002`3oool00`000000oooo
0?ooo`0C0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`050?ooo`800000303oool200000100oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`050?ooo`800000303oool200000100oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`040?ooo`800000303oool200000100oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00X0oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00@0oooo0P00000>0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooo`8000003P3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103oool2000000h0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo0P00000>
0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003oool0oooo00l0oooo0P0000020?oo
o`80000000<0oooo0000003oool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00l0oooo0P0000020?ooo`80000000<0oooo0000003oool03P3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00l0oooo0P0000030?ooo`030000003oool0000000l0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`8000000`3oool00`000000oooo0000000@0?oo
o`030000003oool0oooo00@0oooo00040?ooo`8000004`3oool300000080oooo0`00000?0?ooo`80
0000203oool2000001<0oooo0`0000020?ooo`<000003`3oool2000000P0oooo0P00000C0?ooo`@0
000000@0oooo00000000000000003`3oool2000000P0oooo0P00000D0?ooo`<000000P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01P0
oooo0`00000A0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo02h0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo
00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool002@0oooo0`00000c
0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo0000000c0?oo
o`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000@0?oo
o`006`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0@0?ooo`00603oool2000000030?ooo`000000oooo0340oooo
0P0000000`3oool000000?ooo`0a0?ooo`80000000<0oooo0000003oool0<@3oool00`000000oooo
0?ooo`0L0?ooo`00603oool400000380oooo0P0000000`3oool000000?ooo`0a0?ooo`80000000<0
oooo0000003oool0<@3oool2000001d0oooo000I0?ooo`030000003oool0oooo0380oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`0b0?ooo`80000000<0oooo0000000000006P3oool0
0;X0oooo00<000000?ooo`3oool00P00000J0?ooo`00603oool2000003@0oooo0P00001]0?ooo`80
00006P3oool001P0oooo0P0000000`3oool000000000000a0?ooo`80000000<0oooo000000000000
<@3oool2000005<0oooo000K0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<03o
ool2000005<0oooo00270?ooo`800000D03oool008L0oooo00<000000?ooo`3oool0C`3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000A0?ooo`040000003o
ool0oooo00000080oooo0`000000203oool000000000003oool000000?ooo`000000oooo0P00000P
0?ooo`040000003oool0oooo00000080oooo0`0000020?ooo`040000003oool0000000000080oooo
00<000000?ooo`0000000P3oool010000000oooo00000000000L0?ooo`040000003oool0oooo0000
0080oooo0`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`0000000P3oool0
10000000oooo0?ooo`00000L0?ooo`040000003oool0oooo00000080oooo0`0000020?ooo`040000
003oool0000000000080oooo00<000000?ooo`0000000P3oool010000000oooo00000000000?0?oo
o`004@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo1@0000020?oo
o`8000007`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00H00000
0?ooo`000000oooo0000003oool300000080oooo00D000000?ooo`000000oooo0000000K0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`000002
0?ooo`<000000P3oool010000000oooo0?ooo`00000L0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool010000000
oooo0?ooo`00000?0?ooo`004@3oool010000000oooo0?ooo`0000020?ooo`060000003oool0oooo
0?ooo`000000oooo10000000103oool000000?ooo`00000O0?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool00P3oool010000000oooo0000000000020?ooo`<000000P3oool01000
0000oooo00000000000L0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3o
ool010000000oooo0?ooo`0000020?ooo`<000000P3oool010000000oooo0?ooo`00000L0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool01P000000oooo0?ooo`3oool0
00000?ooo`<000000P3oool01@000000oooo0?ooo`3oool0000000h0oooo000@0?ooo`<000001@3o
ool200000080oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`0M0?ooo`<000001@3o
ool2000000D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`0I0?ooo`<00000
1@3oool2000000P0oooo00<000000?ooo`3oool08@3oool3000000D0oooo0P0000050?ooo`040000
003oool0oooo000000L0oooo00<000000?ooo`3oool03@3oool000`0oooo0P00000d0?ooo`800000
=03oool2000003D0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03o
ool3000002L0oooo000=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0d0?ooo`030000003oool0oooo02H0oooo003I0?ooo`00503oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo08`0oooo00020?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo
0P00000M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool2000001d0oooo
00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0000
000L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0000
000L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo0080oooo00040?ooo`8000003P3oool3000001X0oooo00<00000
0?ooo`3oool01P3oool2000000h0oooo0`00000J0?ooo`030000003oool0oooo00H0oooo0P00000?
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool01P3oool2000000l0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`0V0?oo
o`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0;0?oo
o`<00000603oool2000000/0oooo00<000000?ooo`3oool02`3oool3000001T0oooo00<000000?oo
o`3oool0103oool000L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`060?ooo`00
203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0
3P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0
1`3oool000P0oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3o
ool300000140oooo0`00000O0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000003oool0
oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2
000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00`0
oooo000>0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000oooo
0?ooo`0J0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000oooo
0?ooo`0=0?ooo`00403oool00`000000oooo0?ooo`0200000080oooo0P0000020?ooo`8000000P3o
ool200000080oooo1000000O0?ooo`030000003oool0oooo008000000P3oool200000080oooo0P00
00020?ooo`8000000P3oool4000001l0oooo00<000000?ooo`3oool00P0000020?ooo`8000000P3o
ool200000080oooo0P0000020?ooo`@000007`3oool00`000000oooo0?ooo`0200000080oooo0P00
00020?ooo`8000000P3oool200000080oooo0P0000000`3oool000000?ooo`0?0?ooo`004@3ooolD
00000280oooo5000000R0?ooo`<0000000<0oooo0000000000003P00000R0?ooo`<0000000D0oooo
00000000000000000?ooo`0=00000140oooo000@0?ooo`030000003oool0000000<0oooo00D00000
0?ooo`000000oooo000000020?ooo`8000000P3oool01@000000oooo0?ooo`3oool0000000T0oooo
00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`050000003oool000000?ooo`00
00000`3oool010000000oooo0?ooo`0000030?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0503oool00`000000oooo000000030?ooo`050000003oool000000?ooo`0000000`3oool0
1@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0000000<0oooo00D000000?ooo`000000oooo000000030?ooo`050000
003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0
oooo000>0?ooo`8000001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo
0P0000070?ooo`<000004P3oool2000000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`040?ooo`8000001`3oool300000180oooo0P0000070?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0103oool2000000L0oooo0`00000B0?ooo`8000001`3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool3000000D0oooo
000=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool00`000000oooo0000
000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`040?ooo`030000003oool0000000D0oooo000<0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool01@3oool000X0oooo0P00000:0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0C0?ooo`8000002P3oool00`000000oooo0?ooo`060?oo
o`8000002@3oool00`000000oooo0?ooo`0C0?ooo`8000002P3oool00`000000oooo0?ooo`060?oo
o`8000002@3oool00`000000oooo0?ooo`0C0?ooo`8000002P3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000;0?ooo`03
0000003oool0oooo0180oooo0P00000B0?ooo`8000002`3oool00`000000oooo0?ooo`0B0?ooo`<0
00004@3oool2000000/0oooo00<000000?ooo`3oool04P3oool300000140oooo0P00000;0?ooo`03
0000003oool0oooo01<0oooo0P0000090?ooo`00203oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0103oool2000000`0oooo0P00000@0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0103oool2000000`0oooo0P0000090?ooo`001`3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo0P00000>0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooo`8000003P3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103o
ool2000000h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00@0oooo0P00000>0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003oool0
oooo00l0oooo0P0000020?ooo`80000000<0oooo0000003oool03P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00l0oooo0P0000020?ooo`80000000<0oooo0000003oool03P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00l0oooo0P0000030?ooo`030000003oool0
000000l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`8000000`3oool0
0`000000oooo0000000@0?ooo`030000003oool0oooo00@0oooo00040?ooo`8000004`3oool30000
00<0oooo00<000000?ooo`3oool03P3oool2000000P0oooo0P00000C0?ooo`<000000`3oool00`00
0000oooo0?ooo`0>0?ooo`800000203oool2000001<0oooo10000000103oool0000000000000000?
0?ooo`800000203oool2000001<0oooo10000000103oool0000000000000000@0?ooo`030000003o
ool0oooo00<0oooo00030?ooo`030000003oool0oooo01P0oooo0`00000A0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0603oool300000140oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03oool0
0`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
0040oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo000T0?ooo`<00000<`3oool3000003<0oooo
0`00000c0?ooo`<00000403oool002@0oooo00<000000?ooo`000000<`3oool00`000000oooo0000
000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000403oool001/0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo000H0?oo
o`80000000<0oooo0000003oool0<@3oool2000000030?ooo`000000oooo0340oooo00<000000?oo
o`3oool0<`3oool4000001/0oooo000H0?ooo`80000000<0oooo0000003oool0<@3oool200000003
0?ooo`000000oooo0340oooo0`00000c0?ooo`040000003oool00000000001/0oooo000H0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0000000c0?ooo`03
0000003oool0000001`0oooo00240?ooo`030000003oool0oooo03@0oooo0P00000L0?ooo`00603o
ool200000;l0oooo000H0?ooo`80000000<0oooo000000000000<@3oool2000008T0oooo000K0?oo
o`030000003oool0oooo0300oooo0P0000290?ooo`00D@3oool2000008H0oooo001A0?ooo`030000
003oool0oooo08D0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00140oooo00@000000?ooo`3oool000000P3oool3000000080?ooo`0000000000
0?ooo`000000oooo0000003oool2000002<0oooo00@000000?ooo`3oool000000P3oool300000008
0?ooo`00000000000?ooo`000000oooo0000003oool2000002<0oooo00@000000?ooo`3oool00000
0P3oool3000000@0oooo00<000000?ooo`0000009P3oool010000000oooo0?ooo`0000020?ooo`<0
000000<0oooo0000000000000P3oool010000000oooo0000003oool200000180oooo000A0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool500000080oooo0P00000R0?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo1P000000103oool000000000
0000000R0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000
oooo0000000200000080oooo00<000000?ooo`3oool08@3oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo0?ooo`8000000P3oool3000000030?ooo`00000000000180oooo000A0?oo
o`040000003oool0oooo00000080oooo00D000000?ooo`3oool0oooo000000020?ooo`<0000000<0
oooo0000003oool08`3oool010000000oooo0?ooo`0000020?ooo`050000003oool0oooo0?ooo`00
00000P3oool3000000030?ooo`000000oooo02<0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool00000008000000P3oool00`000000oooo0?ooo`0Q0?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<0000000<0
oooo0000000000004P3oool000`0oooo0P0000020?ooo`<000001@3oool200000080oooo00<00000
0?ooo`000000103oool00`000000oooo0?ooo`0L0?ooo`8000000P3oool3000000D0oooo0P000002
0?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool0703oool200000080oooo0`000005
0?ooo`80000000<0oooo0000000000000P0000030?ooo`<000007@3oool200000080oooo0`000005
0?ooo`8000000P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0100oooo000<
0?ooo`<00000<`3oool3000003<0oooo0`00000c0?ooo`<00000:03oool000d0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool09`3oool00=T0oooo000D0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0200oooo00020?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`010?ooo`000`3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0000001`0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0000001`0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0000001`0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0000001`0oooo00<000000?ooo`3oool00P3oool0
00@0oooo0P00000>0?ooo`<000006P3oool00`000000oooo0?ooo`060?ooo`8000003P3oool30000
01X0oooo00<000000?ooo`3oool01P3oool2000000h0oooo0`00000J0?ooo`030000003oool0oooo
00H0oooo0P00000>0?ooo`<000006P3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000
oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000
oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00@0
oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00
000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00
000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo0008
0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A
0?ooo`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?oo
o`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?oo
o`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<0
00001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0<0?ooo`003P3o
ool2000001L0oooo00<000000?ooo`3oool06P3oool2000001L0oooo00<000000?ooo`3oool06P3o
ool2000001L0oooo00<000000?ooo`3oool06P3oool2000001L0oooo00<000000?ooo`3oool03@3o
ool00100oooo00<000000?ooo`3oool00P0000020?ooo`8000000P3oool200000080oooo0P000002
0?ooo`@000007`3oool00`000000oooo0?ooo`0200000080oooo0P0000020?ooo`8000000P3oool2
00000080oooo1000000O0?ooo`050000003oool0oooo0?ooo`0000000P3oool200000080oooo0P00
00020?ooo`8000000P3oool4000001l0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000
003oool0oooo008000000P3oool200000080oooo0P0000000`3oool000000?ooo`0?0?ooo`004@3o
oolD00000280oooo5000000R0?oooa@000008P3ooolE00000140oooo000@0?ooo`<000000P3oool2
000000040?ooo`000000000000000080oooo0P0000020?ooo`8000000P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000P3oool2000000040?ooo`00
0000000000000080oooo0P0000020?ooo`8000000P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`0000000P3oool2000000040?ooo`000000000000000080
oooo0P0000020?ooo`8000000P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`0000000`3oool01@000000oooo0000003oool000000080oooo0P0000020?oo
o`8000000`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000>0?ooo`80
00001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo0P0000070?ooo`<0
00004P3oool2000000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`80
00001`3oool300000180oooo0P0000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0103oool2000000L0oooo0`00000B0?ooo`8000001`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000
003oool000000140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000
003oool0000000D0oooo000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000:0?ooo`8000002P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04`3oool2
000000X0oooo00<000000?ooo`3oool01P3oool2000000T0oooo00<000000?ooo`3oool04`3oool2
000000X0oooo00<000000?ooo`3oool01P3oool2000000T0oooo00<000000?ooo`3oool04`3oool2
000000X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00T0oooo00090?ooo`8000002`3oool00`000000oooo0?ooo`0B0?ooo`8000004P3oool20000
00/0oooo00<000000?ooo`3oool04P3oool300000140oooo0P00000;0?ooo`030000003oool0oooo
0180oooo0`00000A0?ooo`8000002`3oool00`000000oooo0?ooo`0C0?ooo`8000002@3oool000P0
oooo00<000000?ooo`3oool04`3oool2000000`0oooo0P00000@0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0:0?ooo`800000403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00X0
oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0<0?ooo`8000001P3oool2000000h0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo
0P00000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`040?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0103oool2000000h0oooo00<000000?ooo`3oool01P3oool000H0oooo00<0
00000?ooo`3oool03`3oool010000000oooo0?ooo`3oool2000000030?ooo`000000oooo00h0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`040000003oool0oooo0?ooo`80
000000<0oooo0000003oool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000?0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000003oool000000100
oooo00<000000?ooo`3oool0103oool000@0oooo0P00000C0?ooo`<000000P3oool3000000l0oooo
0P0000080?ooo`8000004`3oool3000000<0oooo00<000000?ooo`3oool03P3oool2000000P0oooo
0P00000C0?ooo`@000000P3oool00`000000oooo0?ooo`0>0?ooo`800000203oool2000001<0oooo
100000020?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool00`3oool000<0oooo00<0
00000?ooo`3oool0;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01P0oooo0`00
000A0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0603oool300000140oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0H0?ooo`<000004@3oool00`000000oooo0?oo
o`020?ooo`000P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0040oooo000T0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo0100oooo000T0?ooo`<00000<`3oool3000003<0oooo0`00000c0?ooo`<0
0000403oool002@0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000
003oool0000003<0oooo00<000000?ooo`000000403oool001/0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0100oooo000K0?ooo`030000003oool0
oooo09l0oooo00<000000?ooo`3oool06@3oool001P0oooo0P0000000`3oool000000?ooo`0a0?oo
o`030000003oool0oooo03<0oooo1000000b0?ooo`80000000<0oooo0000003oool06P3oool001P0
oooo0P00000d0?ooo`<00000<`3oool010000000oooo00000000000b0?ooo`<00000703oool001T0
oooo00<000000?ooo`3oool0<P3oool00`000000oooo0000000c0?ooo`030000003oool0000003@0
oooo00<000000?ooo`3oool06`3oool004h0oooo00<000000?ooo`3oool0=03oool2000003<0oooo
0P00000M0?ooo`00603oool200000:00oooo0P00000M0?ooo`00603oool200000:<0oooo0P00000J
0?ooo`006`3oool200000:00oooo00<000000?ooo`3oool06@3oool001/0oooo00<000000?ooo`3o
ool0^`3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`004@3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool01@000000oooo0000003o
ool000000080oooo00<000000?ooo`3oool08@3oool010000000oooo0?ooo`0000020?ooo`<00000
00P0oooo000000000000oooo0000003oool000000?ooo`8000008`3oool010000000oooo0?ooo`00
00020?ooo`<0000000P0oooo000000000000oooo0000003oool000000?ooo`8000008`3oool01000
0000oooo0?ooo`0000020?ooo`<000001@3oool00`000000oooo0000000E0?ooo`004@3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P000002
0?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`020?ooo`030000003oool00000008000000P3oool00`000000oooo0?ooo`0Q0?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000200000080oooo
00<000000?ooo`3oool08@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?oo
o`8000000P3oool3000000030?ooo`00000000000180oooo000A0?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?oo
o`3oool08@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0oooo10000003
0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`020?ooo`030000003oool00000008000000P3oool00`000000oooo0?ooo`0Q0?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<0000000<0oooo00000000
00004P3oool000`0oooo0P0000020?ooo`<000001@3oool2000000@0oooo00<000000?ooo`3oool0
8@3oool200000080oooo0`0000050?ooo`8000000P3oool00`000000oooo000000040?ooo`030000
003oool0oooo01`0oooo0P0000020?ooo`<000001@3oool200000080oooo00<000000?ooo`000000
103oool00`000000oooo0?ooo`0M0?ooo`80000000@0oooo00000000000000001@3oool2000000D0
oooo00<000000?ooo`3oool05@3oool000`0oooo0`00000c0?ooo`<00000<`3oool3000003@0oooo
0`00000W0?ooo`003@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0=03oool00`000000oooo0?ooo`0V0?ooo`00f@3oool001@0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0803oool00080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`000000
703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`000000
703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`000000
703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`000000
703oool00`000000oooo0?ooo`020?ooo`00103oool2000000h0oooo0`00000J0?ooo`030000003o
ool0oooo00H0oooo0P00000>0?ooo`<000006P3oool00`000000oooo0?ooo`060?ooo`8000003P3o
ool3000001X0oooo00<000000?ooo`3oool01P3oool2000000h0oooo0`00000J0?ooo`030000003o
ool0oooo00<0oooo00060?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0
oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0
oooo02L0oooo00<000000?ooo`3oool0103oool000L0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0
oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0
oooo00<000000?ooo`3oool01`3oool000P0oooo0`00000O0?ooo`<000004@3oool3000001l0oooo
0`00000A0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000002P3oool00080oooo0P00
00070?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?oo
o`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P00
00060?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo0`000007
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo00`0oooo000>0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`80
00005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`80
00005`3oool00`000000oooo0?ooo`0=0?ooo`00403oool01@000000oooo0?ooo`3oool0000000<0
oooo00D000000?ooo`3oool0oooo000000020?ooo`060000003oool0oooo0?ooo`000000oooo0P00
000O0?ooo`050000003oool0oooo0?ooo`0000000`3oool01@000000oooo0?ooo`3oool0000000<0
oooo00D000000?ooo`3oool000000?ooo`02000001l0oooo00D000000?ooo`3oool0oooo00000003
0?ooo`050000003oool0oooo0?ooo`0000000`3oool010000000oooo0?ooo`3oool3000001l0oooo
00D000000?ooo`3oool0oooo000000030?ooo`050000003oool0oooo0?ooo`0000000`3oool01`00
0000oooo0?ooo`3oool000000?ooo`000000403oool00140oooo5000000R0?oooa@000008P3oool0
0`000000oooo0000000A00000280oooo00<000000?ooo`0000000P0000000`3oool000000000000=
00000140oooo000@0?ooo`<000000P3oool2000000040?ooo`000000000000000080oooo0P000002
0?ooo`8000000P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo0`000002
0?ooo`80000000@0oooo00000000000000000P3oool200000080oooo0P0000020?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0503oool300000080oooo0P000000103oool000000000
000000020?ooo`8000000P3oool200000080oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0D0?ooo`<000000P3oool2000000040?ooo`000000000000000080oooo0P0000020?ooo`80
00000`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000>0?ooo`800000
1`3oool00`000000oooo0?ooo`040?ooo`8000001P3oool2000000L0oooo0`00000B0?ooo`800000
1`3oool00`000000oooo0?ooo`040?ooo`8000001P3oool2000000L0oooo0`00000B0?ooo`800000
1`3oool00`000000oooo0?ooo`040?ooo`8000001P3oool2000000L0oooo0`00000B0?ooo`800000
1`3oool00`000000oooo0?ooo`040?ooo`8000001`3oool00`000000oooo0?ooo`050?ooo`<00000
1@3oool000d0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool000000140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool0
00/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00H0oooo0P0000090?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`060?ooo`8000002@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00090?ooo`8000002`3oool00`000000oooo0?ooo`0B0?ooo`8000004P3oool2000000/0
oooo00<000000?ooo`3oool04P3oool300000140oooo0P00000;0?ooo`030000003oool0oooo0180
oooo0`00000A0?ooo`8000002`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00T0
oooo0P0000090?ooo`00203oool00`000000oooo0?ooo`0C0?ooo`800000303oool200000100oooo
00<000000?ooo`3oool04`3oool2000000`0oooo0P00000@0?ooo`030000003oool0oooo01<0oooo
0P00000<0?ooo`800000403oool00`000000oooo0?ooo`0C0?ooo`800000303oool2000000T0oooo
00070?ooo`030000003oool0oooo00`0oooo0P0000060?ooo`8000003P3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00`0oooo0P0000060?ooo`8000003P3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103oool2000000h0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo0P00
000>0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0000000?0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000003oool0000000l0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`0000003`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0000000@0?ooo`030000003oool0oooo00@0oooo00040?oo
o`8000004`3oool4000000040?ooo`0000000000000000l0oooo0P0000080?ooo`8000004`3oool4
000000040?ooo`0000000000000000l0oooo0P0000080?ooo`8000004`3oool8000000l0oooo0P00
00080?ooo`8000004`3oool800000100oooo00<000000?ooo`3oool00`3oool000<0oooo00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo0040oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo000T0?ooo`<00000
<`3oool3000003<0oooo0`00000c0?ooo`<00000403oool002@0oooo00<000000?ooo`000000<`3o
ool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000403o
ool002@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0100oooo00270?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool06@3oool001P0
oooo00<000000?ooo`3oool0<`3oool400000380oooo0P0000000`3oool000000?ooo`0a0?ooo`80
000000<0oooo0000003oool06P3oool001P0oooo0`00000c0?ooo`040000003oool0000000000380
oooo0`00000c0?ooo`8000007@3oool001P0oooo00<000000?ooo`000000<`3oool00`000000oooo
0000000d0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0703oool001P0oooo00<0
00000?ooo`3oool0=03oool2000003<0oooo0P00000d0?ooo`8000007@3oool008@0oooo0P00000d
0?ooo`8000007@3oool008L0oooo0P00000d0?ooo`8000006P3oool008L0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool005@0oooo0`0000080?ooo`<00000
2`3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3o
ool00000EP3oool005@0oooo0`0000080?ooo`<00000203oool5000000D0oooo00@000000?ooo`3o
ool000001`3oool010000000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`00000;0?oo
o`030000003oool0oooo00@0oooo100000070?ooo`@00000EP3oool007@0oooo00@000000?ooo`3o
ool000001`3oool010000000oooo0?ooo`00001F0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C21) show
80.562500 14.625000 moveto
(N21) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C22) show
80.562500 14.625000 moveto
(N22) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C23) show
80.562500 14.625000 moveto
(N23) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C24) show
80.562500 14.625000 moveto
(N24) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .59091 .09091 270 630 arc
s
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C25) show
80.562500 14.625000 moveto
(N25) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.49562 .50376 L
.49157 .50733 L
.48776 .51036 L
.48404 .51258 L
.48021 .51384 L
.47612 .51414 L
.47162 .51362 L
.46666 .51252 L
.46125 .51119 L
.45552 .51 L
.44965 .50932 L
.4439 .50947 L
.43856 .51066 L
.43389 .51301 L
.43013 .5165 L
.42739 .52099 L
.42572 .52625 L
.42502 .53199 L
.42509 .53791 L
.42563 .54371 L
.4263 .54918 L
.42673 .55415 L
.4266 .5586 L
.42568 .56256 L
.42382 .56616 L
.42103 .56957 L
.41746 .573 L
.41336 .57663 L
.4091 .58058 L
.4051 .58494 L
.40175 .58967 L
.39942 .5947 L
.39838 .59987 L
.39877 .60499 L
.4006 .60987 L
.40372 .61434 L
.40786 .61831 L
.41266 .62173 L
.41774 .62468 L
.4227 .62729 L
.42719 .62976 L
.43096 .63233 L
.43389 .63523 L
.43598 .63866 L
.43735 .64274 L
.43821 .64749 L
.43887 .65281 L
.43964 .65851 L
.44082 .66431 L
Mistroke
.44264 .66986 L
.44526 .67482 L
.44873 .67886 L
.453 .68173 L
.45793 .68331 L
.46331 .68358 L
.46891 .68266 L
.47449 .68082 L
.47987 .67839 L
.48491 .67578 L
.48958 .67338 L
.49391 .67156 L
.498 .67058 L
.502 .67058 L
.50609 .67156 L
.51042 .67338 L
.51509 .67578 L
.52013 .67839 L
.52551 .68082 L
.53109 .68266 L
.53669 .68358 L
.54207 .68331 L
.547 .68173 L
.55127 .67886 L
.55474 .67482 L
.55736 .66986 L
.55918 .66431 L
.56036 .65851 L
.56113 .65281 L
.56179 .64749 L
.56265 .64274 L
.56402 .63866 L
.56611 .63523 L
.56904 .63233 L
.57281 .62976 L
.5773 .62729 L
.58226 .62468 L
.58734 .62173 L
.59214 .61831 L
.59628 .61434 L
.5994 .60987 L
.60122 .60499 L
.60162 .59987 L
.60058 .5947 L
.59825 .58967 L
.5949 .58494 L
.5909 .58058 L
.58664 .57663 L
.58254 .573 L
.57897 .56957 L
Mistroke
.57618 .56616 L
.57432 .56256 L
.5734 .5586 L
.57327 .55415 L
.57371 .54917 L
.57437 .54371 L
.57491 .53791 L
.57498 .53199 L
.57428 .52625 L
.5726 .52099 L
.56987 .5165 L
.56611 .51302 L
.56144 .51066 L
.5561 .50947 L
.55035 .50932 L
.54448 .51 L
.53875 .51119 L
.53334 .51252 L
.52838 .51361 L
.52388 .51414 L
.51979 .51384 L
.51596 .51258 L
.51224 .51036 L
.50843 .50733 L
.50438 .50376 L
.5 .5 L
Mfstroke
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N26) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.49562 .50376 L
.49157 .50733 L
.48776 .51036 L
.48404 .51258 L
.48021 .51384 L
.47612 .51414 L
.47162 .51362 L
.46666 .51252 L
.46125 .51119 L
.45552 .51 L
.44965 .50932 L
.4439 .50947 L
.43856 .51066 L
.43389 .51301 L
.43013 .5165 L
.42739 .52099 L
.42572 .52625 L
.42502 .53199 L
.42509 .53791 L
.42563 .54371 L
.4263 .54918 L
.42673 .55415 L
.4266 .5586 L
.42568 .56256 L
.42382 .56616 L
.42103 .56957 L
.41746 .573 L
.41336 .57663 L
.4091 .58058 L
.4051 .58494 L
.40175 .58967 L
.39942 .5947 L
.39838 .59987 L
.39877 .60499 L
.4006 .60987 L
.40372 .61434 L
.40786 .61831 L
.41266 .62173 L
.41774 .62468 L
.4227 .62729 L
.42719 .62976 L
.43096 .63233 L
.43389 .63523 L
.43598 .63866 L
.43735 .64274 L
.43821 .64749 L
.43887 .65281 L
.43964 .65851 L
.44082 .66431 L
Mistroke
.44264 .66986 L
.44526 .67482 L
.44873 .67886 L
.453 .68173 L
.45793 .68331 L
.46331 .68358 L
.46891 .68266 L
.47449 .68082 L
.47987 .67839 L
.48491 .67578 L
.48958 .67338 L
.49391 .67156 L
.498 .67058 L
.502 .67058 L
.50609 .67156 L
.51042 .67338 L
.51509 .67578 L
.52013 .67839 L
.52551 .68082 L
.53109 .68266 L
.53669 .68358 L
.54207 .68331 L
.547 .68173 L
.55127 .67886 L
.55474 .67482 L
.55736 .66986 L
.55918 .66431 L
.56036 .65851 L
.56113 .65281 L
.56179 .64749 L
.56265 .64274 L
.56402 .63866 L
.56611 .63523 L
.56904 .63233 L
.57281 .62976 L
.5773 .62729 L
.58226 .62468 L
.58734 .62173 L
.59214 .61831 L
.59628 .61434 L
.5994 .60987 L
.60122 .60499 L
.60162 .59987 L
.60058 .5947 L
.59825 .58967 L
.5949 .58494 L
.5909 .58058 L
.58664 .57663 L
.58254 .573 L
.57897 .56957 L
Mistroke
.57618 .56616 L
.57432 .56256 L
.5734 .5586 L
.57327 .55415 L
.57371 .54917 L
.57437 .54371 L
.57491 .53791 L
.57498 .53199 L
.57428 .52625 L
.5726 .52099 L
.56987 .5165 L
.56611 .51302 L
.56144 .51066 L
.5561 .50947 L
.55035 .50932 L
.54448 .51 L
.53875 .51119 L
.53334 .51252 L
.52838 .51361 L
.52388 .51414 L
.51979 .51384 L
.51596 .51258 L
.51224 .51036 L
.50843 .50733 L
.50438 .50376 L
.5 .5 L
Mfstroke
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N27) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.5 .5 m
.49562 .50376 L
.49157 .50733 L
.48776 .51036 L
.48404 .51258 L
.48021 .51384 L
.47612 .51414 L
.47162 .51362 L
.46666 .51252 L
.46125 .51119 L
.45552 .51 L
.44965 .50932 L
.4439 .50947 L
.43856 .51066 L
.43389 .51301 L
.43013 .5165 L
.42739 .52099 L
.42572 .52625 L
.42502 .53199 L
.42509 .53791 L
.42563 .54371 L
.4263 .54918 L
.42673 .55415 L
.4266 .5586 L
.42568 .56256 L
.42382 .56616 L
.42103 .56957 L
.41746 .573 L
.41336 .57663 L
.4091 .58058 L
.4051 .58494 L
.40175 .58967 L
.39942 .5947 L
.39838 .59987 L
.39877 .60499 L
.4006 .60987 L
.40372 .61434 L
.40786 .61831 L
.41266 .62173 L
.41774 .62468 L
.4227 .62729 L
.42719 .62976 L
.43096 .63233 L
.43389 .63523 L
.43598 .63866 L
.43735 .64274 L
.43821 .64749 L
.43887 .65281 L
.43964 .65851 L
.44082 .66431 L
Mistroke
.44264 .66986 L
.44526 .67482 L
.44873 .67886 L
.453 .68173 L
.45793 .68331 L
.46331 .68358 L
.46891 .68266 L
.47449 .68082 L
.47987 .67839 L
.48491 .67578 L
.48958 .67338 L
.49391 .67156 L
.498 .67058 L
.502 .67058 L
.50609 .67156 L
.51042 .67338 L
.51509 .67578 L
.52013 .67839 L
.52551 .68082 L
.53109 .68266 L
.53669 .68358 L
.54207 .68331 L
.547 .68173 L
.55127 .67886 L
.55474 .67482 L
.55736 .66986 L
.55918 .66431 L
.56036 .65851 L
.56113 .65281 L
.56179 .64749 L
.56265 .64274 L
.56402 .63866 L
.56611 .63523 L
.56904 .63233 L
.57281 .62976 L
.5773 .62729 L
.58226 .62468 L
.58734 .62173 L
.59214 .61831 L
.59628 .61434 L
.5994 .60987 L
.60122 .60499 L
.60162 .59987 L
.60058 .5947 L
.59825 .58967 L
.5949 .58494 L
.5909 .58058 L
.58664 .57663 L
.58254 .573 L
.57897 .56957 L
Mistroke
.57618 .56616 L
.57432 .56256 L
.5734 .5586 L
.57327 .55415 L
.57371 .54917 L
.57437 .54371 L
.57491 .53791 L
.57498 .53199 L
.57428 .52625 L
.5726 .52099 L
.56987 .5165 L
.56611 .51302 L
.56144 .51066 L
.5561 .50947 L
.55035 .50932 L
.54448 .51 L
.53875 .51119 L
.53334 .51252 L
.52838 .51361 L
.52388 .51414 L
.51979 .51384 L
.51596 .51258 L
.51224 .51036 L
.50843 .50733 L
.50438 .50376 L
.5 .5 L
Mfstroke
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N28) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.5 .5 m
.49545 .50435 L
.49091 .50803 L
.48636 .5105 L
.48182 .51136 L
.47727 .5105 L
.47273 .50803 L
.46818 .50435 L
.46364 .5 L
.45909 .49565 L
.45455 .49196 L
.45 .4895 L
.44546 .48863 L
.44091 .4895 L
.43636 .49196 L
.43182 .49565 L
.42727 .5 L
.42273 .50434 L
.41818 .50803 L
.41364 .51049 L
.40909 .51136 L
.40455 .51049 L
.4 .50803 L
.39545 .50434 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34546 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.40909 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.50455 .49565 L
.50909 .49197 L
.51364 .4895 L
.51818 .48864 L
.52273 .4895 L
.52727 .49197 L
.53182 .49565 L
.53636 .5 L
.54091 .50435 L
.54545 .50804 L
.55 .5105 L
.55454 .51137 L
.55909 .5105 L
.56364 .50804 L
.56818 .50435 L
.57273 .5 L
.57727 .49566 L
.58182 .49197 L
.58636 .48951 L
.59091 .48864 L
.59545 .48951 L
.6 .49197 L
.60455 .49566 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65454 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.59091 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.5 .5 m
.49562 .50376 L
.49157 .50733 L
.48776 .51036 L
.48404 .51258 L
.48021 .51384 L
.47612 .51414 L
.47162 .51362 L
.46666 .51252 L
.46125 .51119 L
.45552 .51 L
.44965 .50932 L
.4439 .50947 L
.43856 .51066 L
.43389 .51301 L
.43013 .5165 L
.42739 .52099 L
.42572 .52625 L
.42502 .53199 L
.42509 .53791 L
.42563 .54371 L
.4263 .54918 L
.42673 .55415 L
.4266 .5586 L
.42568 .56256 L
.42382 .56616 L
.42103 .56957 L
.41746 .573 L
.41336 .57663 L
.4091 .58058 L
.4051 .58494 L
.40175 .58967 L
.39942 .5947 L
.39838 .59987 L
.39877 .60499 L
.4006 .60987 L
.40372 .61434 L
.40786 .61831 L
.41266 .62173 L
.41774 .62468 L
.4227 .62729 L
.42719 .62976 L
.43096 .63233 L
.43389 .63523 L
.43598 .63866 L
.43735 .64274 L
.43821 .64749 L
.43887 .65281 L
.43964 .65851 L
.44082 .66431 L
Mistroke
.44264 .66986 L
.44526 .67482 L
.44873 .67886 L
.453 .68173 L
.45793 .68331 L
.46331 .68358 L
.46891 .68266 L
.47449 .68082 L
.47987 .67839 L
.48491 .67578 L
.48958 .67338 L
.49391 .67156 L
.498 .67058 L
.502 .67058 L
.50609 .67156 L
.51042 .67338 L
.51509 .67578 L
.52013 .67839 L
.52551 .68082 L
.53109 .68266 L
.53669 .68358 L
.54207 .68331 L
.547 .68173 L
.55127 .67886 L
.55474 .67482 L
.55736 .66986 L
.55918 .66431 L
.56036 .65851 L
.56113 .65281 L
.56179 .64749 L
.56265 .64274 L
.56402 .63866 L
.56611 .63523 L
.56904 .63233 L
.57281 .62976 L
.5773 .62729 L
.58226 .62468 L
.58734 .62173 L
.59214 .61831 L
.59628 .61434 L
.5994 .60987 L
.60122 .60499 L
.60162 .59987 L
.60058 .5947 L
.59825 .58967 L
.5949 .58494 L
.5909 .58058 L
.58664 .57663 L
.58254 .573 L
.57897 .56957 L
Mistroke
.57618 .56616 L
.57432 .56256 L
.5734 .5586 L
.57327 .55415 L
.57371 .54917 L
.57437 .54371 L
.57491 .53791 L
.57498 .53199 L
.57428 .52625 L
.5726 .52099 L
.56987 .5165 L
.56611 .51302 L
.56144 .51066 L
.5561 .50947 L
.55035 .50932 L
.54448 .51 L
.53875 .51119 L
.53334 .51252 L
.52838 .51361 L
.52388 .51414 L
.51979 .51384 L
.51596 .51258 L
.51224 .51036 L
.50843 .50733 L
.50438 .50376 L
.5 .5 L
Mfstroke
.52727 .68182 m
.47273 .7 L
.47273 .66364 L
F
gsave
.5 .72478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.5 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N29) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N30) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N31) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N32) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N33) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N34) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N35) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N36) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N37) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N38) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N39) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N40) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00
000000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2000002<0oooo
00@000000?ooo`00000000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003o
ool2000000030?ooo`000000000001h0oooo00@000000?ooo`00000000000P3oool300000080oooo
00@000000?ooo`00000000000P3oool010000000oooo0000003oool2000000030?ooo`0000000000
01h0oooo00@000000?ooo`00000000000P3oool300000080oooo00H000000?ooo`3oool000000?oo
o`0000040?ooo`800000403oool00100oooo00@000000?ooo`3oool000000P3oool010000000oooo
0?ooo`3oool2000000040?ooo`000000000000000080oooo00@000000?ooo`3oool00000803oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo1@0000020?ooo`040000003o
ool00000000001h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`06
0000003oool000000?ooo`000000oooo0`0000020?ooo`030000003oool0oooo008000007@3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`00
00001`3oool000000000003oool000000?ooo`0000003`3oool00100oooo00L000000?ooo`3oool0
oooo0000003oool0000000<0oooo0P000000103oool000000000000000020?ooo`040000003oool0
oooo00000200oooo00L000000?ooo`3oool0oooo0000003oool0000000<0oooo00<000000?ooo`00
00000`0000020?ooo`040000003oool00000000001h0oooo00L000000?ooo`3oool0oooo0000003o
ool0000000@0oooo00@000000?ooo`00000000000P3oool300000080oooo00D000000?ooo`000000
oooo0000000M0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0
oooo00<0000000@0oooo000000000000oooo0P00000@0?ooo`003`3oool3000000030?ooo`000000
oooo0080oooo0P0000020?ooo`030000003oool0000000@0oooo00<000000?ooo`0000000P00000N
0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<000000?ooo`000000103oool0
10000000oooo0?ooo`00000M0?ooo`<0000000<0oooo0000003oool00P3oool2000000D0oooo00@0
00000?ooo`3oool00000103oool010000000oooo0?ooo`00000M0?ooo`<0000000<0oooo0000003o
ool00P3oool2000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0>0?ooo`00
303oool2000003@0oooo0P00000d0?ooo`8000003P3oool00`000000oooo0?ooo`0T0?ooo`800000
:03oool000`0oooo0`00000<0?ooo`030000003oool0oooo02@0oooo0`0000090?ooo`040000003o
ool00000000002H0oooo0`0000090?ooo`050000003oool000000?ooo`0000009P3oool3000002L0
oooo000=0?ooo`030000003oool0oooo00P0oooo0P0000000`3oool000000?ooo`0V0?ooo`030000
003oool0oooo00T0oooo0`00000W0?ooo`030000003oool0oooo00T0oooo0P0000000`3oool00000
0?ooo`0V0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0703oool001P0oooo0P00
00000`3oool000000?ooo`0c0?ooo`800000=03oool200000380oooo0`00000L0?ooo`000P3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0D0?ooo`800000603oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo01@0oooo0P00000H0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0000001P0oooo00<000000?ooo`3oool00@3oool0
00<0oooo00<000000?ooo`3oool0303oool2000001h0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0<0?ooo`800000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0<0?ooo`800000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`020?ooo`00103oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0080
oooo0P00000H0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`020?ooo`800000603oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`0000007P3oool00`000000oooo0?ooo`020?ooo`001@3oool2000000`0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`070?ooo`800000303oool00`000000oooo
0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`070?ooo`800000303oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo00L0oooo0P00000;0?ooo`<00000703oool00`000000oooo
0?ooo`030?ooo`001`3oool00`000000oooo0?ooo`080?ooo`<000006P3oool2000000`0oooo00<0
00000?ooo`3oool0203oool3000000H0oooo0P00000B0?ooo`800000303oool00`000000oooo0?oo
o`080?ooo`<000006P3oool2000000`0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`040?ooo`00203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0403oool200000140oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`06
0?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool01`3oool000T0oooo0P00000O0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<0
00004`3oool00`000000oooo0?ooo`0M0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<0
00002P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`800000
1P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0
703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
2@3oool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3o
ool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool01`3oool2
000000X0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2
000000/0oooo0P00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2
000000X0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2
000000X0oooo1000000;0?ooo`030000003oool0oooo00`0oooo000?0?ooo`030000003oool0oooo
00H0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00H0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00H0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00d0oooo000@0?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00D0oooo0P00000O0?ooo`030000003oool0oooo008000000P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo0080
00000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo0P00000O0?ooo`03
0000003oool0oooo008000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool03P3oool00100oooo200000060?ooo`@0000000<0oooo000000000000
2P3oool00`000000oooo0?ooo`0D0?ooo`P000001P3oool4000000030?ooo`000000000000X0oooo
00<000000?ooo`3oool0503oool4000000040?ooo`0000000000000000H0oooo100000000`3oool0
00000000000:0?ooo`030000003oool0oooo01D0oooo0`000000103oool000000000000000070?oo
o`<0000000@0oooo00000000000000002@3oool00`000000oooo0?ooo`050?ooo`00403oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`<00000503oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`<000001@3oool000h0
oooo0P0000080?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool00`000000
oooo0000000B0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01P3oool2000000L0
oooo00<000000?ooo`0000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0
oooo0P0000070?ooo`030000003oool000000180oooo0P0000080?ooo`8000000`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000001@3oool000d0oooo
00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00X0oooo0`00000;0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool02@3oool4000000/0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`050?ooo`00303oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00/0oooo000:0?ooo`8000007P3oool00`000000oooo0?ooo`0C
0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C
0?ooo`8000007`3oool00`000000oooo0?ooo`090?ooo`002@3oool200000200oooo0P00000B0?oo
o`800000803oool300000140oooo0P00000P0?ooo`<000004@3oool200000240oooo0P0000090?oo
o`00203oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`03
0000003oool0oooo0100oooo00@000000?ooo`3oool000003@3oool200000100oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo00X0oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0D0?ooo`<0
00003@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool00`00000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04P3oool00`000000
oooo00000002000000d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0E0?ooo`80
00003@3oool00`000000oooo0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0E0?ooo`040000
003oool000000?ooo`8000002`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00l0
oooo00@000000?ooo`00000000000P3oool010000000oooo0000003oool2000000/0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0C0?ooo`060000003oool000000?ooo`000000oooo
0P00000;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05P3oool2000000030?oo
o`000000000000`0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000E0?ooo`040000003o
ool0oooo000000<0oooo0P00000<0?ooo`800000203oool2000001<0oooo0`0000020?ooo`040000
003oool0oooo0?ooo`800000303oool2000000P0oooo0P00000B0?ooo`070000003oool000000?oo
o`000000oooo000000030?ooo`800000303oool2000000P0oooo0P00000I0?ooo`030000003oool0
oooo008000003@3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0B0?oo
o`80000000<0oooo0000003oool01@3oool2000000h0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0D0?ooo`8000001P3oool2000000h0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0C0?ooo`80000000<0oooo0000003oool0103oool2000000h0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0P00000>0?oo
o`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo01<0oooo0P0000000`3oool0
00000?ooo`0F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0503oool2000001P0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0E0?ooo`8000005`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0`00000H0?ooo`030000003oool0oooo0040
oooo000H0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool09`3oool2000000X0oooo
00<000000?ooo`3oool0:03oool2000000T0oooo00<000000?ooo`3oool09`3oool00`000000oooo
000000090?ooo`030000003oool0oooo0100oooo000T0?ooo`<000009`3oool2000000X0oooo0`00
000W0?ooo`8000002P3oool3000002L0oooo00<000000?ooo`3oool02@3oool300000100oooo000T
0?ooo`030000003oool0000002X0oooo0P0000070?ooo`030000003oool0000002L0oooo0P00000:
0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000403oool002@0oooo00<000000?oo
o`3oool0:P3oool2000000L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo0100oooo001A0?ooo`800000QP3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00000000000P3oool3000000@0
oooo00@000000?ooo`000000oooo0P0000000`3oool000000000000P0?ooo`040000003oool00000
00000080oooo0`000000203oool000000000003oool000000?ooo`000000oooo0P00000S0?ooo`04
0000003oool0000000000080oooo0`000000203oool000000000003oool000000?ooo`000000oooo
0P0000000`3oool000000000000P0?ooo`040000003oool0000000000080oooo0`0000050?ooo`04
0000003oool000000?ooo`80000000<0oooo000000000000403oool00100oooo00@000000?ooo`3o
ool000000P3oool010000000oooo0?ooo`3oool2000000040?ooo`000000000000000080oooo00@0
00000?ooo`3oool00000803oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080
oooo1@0000020?ooo`040000003oool0000000000200oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`3oool600000080oooo00<000000?ooo`3oool00P00000O0?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0300000080oooo
00<000000?ooo`0000000P00000?0?ooo`00403oool01`000000oooo0?ooo`3oool000000?ooo`00
00000`3oool2000000040?ooo`000000000000000080oooo00@000000?ooo`3oool00000803oool0
1`000000oooo0?ooo`3oool000000?ooo`0000000`3oool00`000000oooo0?ooo`0300000080oooo
00@000000?ooo`0000000000803oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool0
0`000000oooo0?ooo`0300000080oooo00<000000?ooo`0000008@3oool01`000000oooo0?ooo`3o
ool000000?ooo`000000103oool00`000000oooo0?ooo`0300000080oooo00<000000?ooo`000000
4@3oool000l0oooo0`0000000`3oool000000?ooo`020?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`040000003oool0oooo000001l0oooo0`0000000`3oool000000?ooo`020?ooo`800000
0P3oool00`000000oooo000000040?ooo`030000003oool0oooo0200oooo0`0000000`3oool00000
0?ooo`020?ooo`8000000P3oool00`000000oooo000000040?ooo`040000003oool0oooo000001l0
oooo0`0000000`3oool000000?ooo`020?ooo`80000000H0oooo00000000000000000?ooo`000004
0?ooo`040000003oool0oooo00000100oooo000<0?ooo`8000003P3oool00`000000oooo0?ooo`0S
0?ooo`800000=03oool2000003D0oooo0P00000=0?ooo`030000003oool0oooo01P0oooo000<0?oo
o`<000002@3oool01@000000oooo0000003oool0000002D0oooo0`00000<0?ooo`030000003oool0
oooo02@0oooo0`0000090?ooo`040000003oool00000000002L0oooo0`0000080?ooo`050000003o
ool000000?ooo`0000006P3oool000d0oooo00<000000?ooo`3oool02@3oool2000000030?ooo`00
0000oooo02D0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02H0oooo00<0
00000?ooo`3oool02@3oool3000002P0oooo00<000000?ooo`3oool0203oool2000000030?ooo`00
0000oooo01T0oooo000J0?ooo`800000<P3oool2000000030?ooo`000000oooo03<0oooo0P00000d
0?ooo`8000006`3oool00180oooo00<000000?ooo`3oool0103oool2000003<0oooo00<000000?oo
o`3oool0=03oool2000003@0oooo0P00000L0?ooo`000P3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo0080oooo0P00000I0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool03@3oool2000001l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`80
0000103oool2000001T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`80
00001@3oool2000001P0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo000000040?ooo`800000603oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool00P3oool2000001P0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo0P00000G0?ooo`030000
003oool0oooo0080oooo00040?ooo`800000303oool3000001`0oooo00<000000?ooo`3oool01P3o
ool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`060?ooo`8000003@3o
ool00`000000oooo0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`060?ooo`8000003@3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00<0oooo00060?ooo`030000003oool0
oooo02H0oooo0P00000;0?ooo`030000003oool0oooo00T0oooo0`00000J0?ooo`8000002`3oool0
0`000000oooo0?ooo`090?ooo`<000001P3oool200000180oooo0P00000;0?ooo`030000003oool0
oooo00T0oooo0`00000K0?ooo`030000003oool0oooo00@0oooo00070?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool04@3oool200000140oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000
oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000
oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`00203oool3000001l0oooo
0`00000A0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo
0`00000:0?ooo`000P3oool2000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P00
00060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?oo
o`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`090?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=
0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`000006
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool02P3oool2000000`0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<0?oo
o`030000003oool0oooo00`0oooo000>0?ooo`800000203oool200000080oooo0P0000090?ooo`03
0000003oool0oooo01X0oooo0P0000080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3o
ool06P3oool2000000P0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0J0?ooo`80
0000203oool200000080oooo0P0000090?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000
003oool0oooo008000000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo
0P00000O0?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo008000000P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo00800000
0P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
3P3oool00100oooo200000060?ooo`@0000000<0oooo0000000000008@3oool8000000H0oooo1000
00000`3oool000000000000Q0?ooo`@0000000@0oooo00000000000000001P3oool4000000030?oo
o`00000000000240oooo10000000103oool000000000000000060?ooo`@0000000@0oooo00000000
000000004@3oool00100oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`00
3P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`<00000
4P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`<00000
4P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`<00000
4P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1@3oool3000000D0oooo000=0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`090?ooo`@000002`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool02P3o
ool3000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003o
ool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000001@3o
ool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00D0oooo000:0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`00
0000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007`3oool00`00
0000oooo0?ooo`090?ooo`002@3oool200000200oooo0P00000B0?ooo`800000803oool300000140
oooo0P00000P0?ooo`<000004@3oool200000240oooo0P0000090?ooo`00203oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool02`3oool200000100oooo00<000000?ooo`3oool0403oool01@000000oooo
0?ooo`3oool0000000`0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool02P3oool2000000T0oooo00070?ooo`030000003oool0oooo01@0oooo0`00000=0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0503oool2000000h0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0A0?ooo`040000003oool0oooo0?ooo`8000003@3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P00000=0?ooo`03
0000003oool0oooo00H0oooo00060?ooo`030000003oool0oooo01<0oooo00H000000?ooo`000000
oooo0000003oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?oo
o`8000000P3oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?oo
o`040000003oool00000000000<0oooo0P0000000`3oool000000000000;0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`02000000030?ooo`000000
000000`0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000B0?ooo`070000003oool00000
0?ooo`000000oooo000000030?ooo`800000303oool2000000P0oooo0P00000E0?ooo`040000003o
ool0oooo000000<0oooo0P00000<0?ooo`800000203oool2000001<0oooo0`0000030?ooo`030000
003oool0oooo00800000303oool2000000P0oooo0P00000B0?ooo`050000003oool000000?ooo`00
00000P3oool00`000000oooo0?ooo`02000000d0oooo00<000000?ooo`3oool00`3oool000<0oooo
00<000000?ooo`3oool04`3oool2000000030?ooo`000000oooo00@0oooo0P00000>0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000oooo00D0oooo
0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool2000000H0oooo
0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04`3oool2000000030?oo
o`000000oooo00@0oooo0P00000>0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003o
ool0oooo01D0oooo0P00000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3o
ool2000000030?ooo`000000oooo01H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0D0?ooo`800000603oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01D0oooo0P00
000G0?ooo`030000003oool0oooo0040oooo000I0?ooo`8000002@3oool00`000000oooo0?ooo`0W
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool09`3oool2000000X0oooo00<00000
0?ooo`3oool0:03oool2000000T0oooo00<000000?ooo`3oool0403oool001P0oooo0P00000:0?oo
o`<00000<`3oool3000002L0oooo0P00000:0?ooo`<00000:03oool2000000T0oooo0`00000@0?oo
o`00603oool2000000X0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000Z0?ooo`80
00001`3oool00`000000oooo0000000X0?ooo`8000002@3oool00`000000oooo0000000@0?ooo`00
903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo02X0oooo0P0000070?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0403oool008L0oooo0P00001@0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00403oool010000000oooo0?ooo`00
00020?ooo`<00000103oool010000000oooo0000003oool2000000030?ooo`00000000000200oooo
00@000000?ooo`00000000000P3oool300000080oooo00H000000?ooo`000000oooo0000003oool2
000000030?ooo`00000000000200oooo00@000000?ooo`00000000000P3oool3000000080?ooo`00
000000000?ooo`000000oooo0000003oool200000080oooo00<000000?ooo`3oool07P3oool01000
0000oooo0000000000020?ooo`<0000000<0oooo0000000000000P3oool010000000oooo0000003o
ool2000000030?ooo`00000000000100oooo000@0?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool0oooo0P000000103oool000000000000000020?ooo`030000003oool0oooo0080
00007`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?oo
o`0000000P0000020?ooo`050000003oool000000?ooo`0000007`3oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`040000003o
ool0oooo00000200oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00<000000P3oool010000000oooo0?ooo`00000@0?ooo`00403oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80000000@0oooo00000000000000000`3o
ool2000000030?ooo`000000oooo01h0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0
oooo00<000000?ooo`0000000P0000020?ooo`040000003oool0000000000200oooo00L000000?oo
o`3oool0oooo0000003oool0000000D0oooo100000020?ooo`040000003oool0oooo00000200oooo
00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`3oool00`0000020?oo
o`050000003oool0oooo0?ooo`0000003`3oool000l0oooo0`0000050?ooo`800000103oool00`00
0000oooo0?ooo`020?ooo`040000003oool0oooo000001l0oooo0`0000000`3oool000000?ooo`02
0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000001l0oooo0`00
00000`3oool000000?ooo`020?ooo`8000000P3oool00`000000oooo000000040?ooo`030000003o
ool0oooo0200oooo0`0000000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?oo
o`0000040?ooo`040000003oool0oooo00000100oooo000<0?ooo`800000=03oool2000003@0oooo
0P00000>0?ooo`030000003oool0oooo02@0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00
000<0?ooo`030000003oool0oooo02@0oooo0`0000090?ooo`050000003oool000000?ooo`000000
9P3oool3000000`0oooo00<000000?ooo`3oool0603oool000d0oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`080?ooo`80000000<0oooo0000003oool09P3oool00`000000oooo0?oo
o`090?ooo`80000000<0oooo0000003oool09P3oool00`000000oooo0?ooo`070?ooo`8000000P3o
ool00`000000oooo0?ooo`0H0?ooo`00CP3oool2000000030?ooo`000000oooo03<0oooo0P00000b
0?ooo`8000000P3oool00`000000oooo0?ooo`0H0?ooo`00B03oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02d0oooo00<000000?ooo`3oool0103oool2000002d0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0L0?ooo`000P3oool00`000000oooo0?ooo`0?0?ooo`80
00007@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool00P3oool2000001T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo000000040?ooo`800000603oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`0000007P3oool00`000000oooo0?ooo`020?ooo`00103o
ool2000000l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`060?ooo`800000303o
ool3000001`0oooo00<000000?ooo`3oool01P3oool2000000`0oooo0`00000L0?ooo`030000003o
ool0oooo00H0oooo0P00000<0?ooo`<00000703oool00`000000oooo0?ooo`030?ooo`001P3oool0
0`000000oooo0?ooo`0;0?ooo`<00000603oool2000000/0oooo00<000000?ooo`3oool09P3oool2
000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000
oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000
oooo0?ooo`070?ooo`00203oool3000001l0oooo0`00000A0?ooo`<000007`3oool300000140oooo
0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000:0?ooo`000P3oool2000000L0oooo00<0
00000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00
000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool3000000L0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0;
0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?oo
o`<00000303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool02@3oool3000000`0oooo00<000000?ooo`3oool0303oool000h0oooo0P00000G0?ooo`03
0000003oool0oooo01X0oooo0P0000080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3o
ool06P3oool2000000P0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0J0?ooo`80
0000203oool200000080oooo0P0000090?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000
003oool0oooo008000000P3oool200000080oooo0P0000020?ooo`8000000P3oool4000001l0oooo
00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
103oool2000001l0oooo00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0103oool2000001l0oooo00<000000?ooo`3oool00P0000020?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0>0?ooo`00403o
oolE00000240oooo200000060?ooo`@0000000<0oooo0000000000008@3oool4000000040?ooo`00
00000000000000H0oooo100000000`3oool000000000000Q0?ooo`@0000000@0oooo000000000000
00001P3oool4000000040?ooo`000000000000000140oooo000@0?ooo`030000003oool0000000<0
oooo2P0000000`3oool000000?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0503oool00`000000oooo000000030?ooo`8000001P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`80
00001P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0503oool00`000000oooo000000030?ooo`8000001P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool000h0oooo0P0000060?ooo`800000
1`3oool00`000000oooo0?ooo`040?ooo`8000001`3oool300000180oooo0P0000080?ooo`800000
0`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo0P0000080?ooo`800000
0`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo0P0000080?ooo`800000
0`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo0`0000050?ooo`003@3o
ool00`000000oooo0?ooo`060?ooo`8000001P3oool2000000P0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`090?ooo`@000002`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool02@3o
ool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo000000050?ooo`00303oool0
0`000000oooo0?ooo`060?ooo`800000203oool2000000P0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`050?ooo`002P3oool2000000T0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo01<0oooo0P00000N0?ooo`030000003oool0oooo
01<0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000O0?ooo`030000003oool0oooo
00T0oooo00090?ooo`8000002P3oool2000000P0oooo0P00000:0?ooo`8000004P3oool200000200
oooo0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo
00<000000?ooo`3oool02`3oool2000000H0oooo0P00000<0?ooo`800000403oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0;0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`040?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01@0oooo0P00
000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05@3oool2000000d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0D0?ooo`<000003@3oool00`000000oooo0?oo
o`060?ooo`001P3oool00`000000oooo0?ooo`0>0?ooo`80000000@0oooo000000000000oooo0`00
000?0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05@3oool200000080oooo0P00
000;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`02000000030?ooo`000000000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0E0?ooo`040000003oool000000?ooo`800000303oool00`000000oooo0?ooo`040?ooo`00103o
ool200000180oooo1P0000000`3oool000000?ooo`0?0?ooo`800000203oool2000001D0oooo00@0
00000?ooo`3oool000000`3oool2000000`0oooo0P0000080?ooo`8000004P3oool01@000000oooo
0000003oool000000080oooo00<000000?ooo`3oool00P00000<0?ooo`800000203oool2000001H0
oooo00<000000?ooo`0000000`3oool2000000d0oooo00<000000?ooo`3oool00`3oool000<0oooo
00<000000?ooo`3oool0603oool300000140oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0B0?ooo`80000000<0oooo0000003oool01@3oool2000000h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo0000003oool0103oool2000000h0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`8000000P3oool00`000000oooo
0?ooo`030?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?oo
o`0^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool2000000030?ooo`00
0000oooo01H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0E0?ooo`8000005`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0P0000020?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo02P0oooo0P0000090?ooo`030000003oool0
oooo02L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`00903oool30000
03<0oooo0`00000W0?ooo`8000002P3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000
oooo0000000c0?ooo`030000003oool0000002L0oooo0P00000:0?ooo`030000003oool0000003<0
oooo00<000000?ooo`000000403oool002@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool001T0oooo00<0
00000?ooo`000000_@3oool001P0oooo1000002m0?ooo`00603oool400000;d0oooo000H0?ooo`05
0000003oool000000?ooo`000000_03oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000>
0?ooo`040000003oool0oooo00000080oooo0`000000103oool000000000003oool200000080oooo
00@000000?ooo`000000oooo0P0000000`3oool000000000000N0?ooo`040000003oool0oooo0000
0080oooo0`0000020?ooo`060000003oool000000?ooo`000000oooo0P0000000`3oool000000000
000P0?ooo`040000003oool0oooo00000080oooo0`000000203oool000000000003oool000000?oo
o`000000oooo0P00000S0?ooo`040000003oool0oooo00000080oooo0`0000000`3oool000000000
00020?ooo`040000003oool000000?ooo`80000000<0oooo000000000000403oool000h0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00800000
00@0oooo00000000000000000P3oool00`000000oooo0?ooo`02000001d0oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool00000008000000P3oool00`00
0000oooo00000002000001l0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool00000008000000P3oool010000000oooo0?ooo`00000P0?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0300000080oooo
00@000000?ooo`0000000000403oool000h0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`030?ooo`8000000`3oool3000000<0oooo0P00000O0?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002000000<0oooo0P00000Q0?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool00`3oool4000000<0oooo00<00000
0?ooo`000000803oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00`0000030?ooo`<00000403oool000`0oooo100000050?ooo`8000000P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000
6P3oool2000000040?ooo`0000000000000000D0oooo0P0000040?ooo`030000003oool0oooo0080
oooo00@000000?ooo`3oool00000703oool2000000040?ooo`0000000000000000D0oooo0P000002
0?ooo`030000003oool0000000@0oooo00<000000?ooo`0000000P00000K0?ooo`80000000@0oooo
00000000000000001@3oool200000080oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`00000@0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000<`3oool3000002P0oooo
000=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo02L0oooo003I0?ooo`00BP3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo05H0oooo00020?ooo`030000003oool0oooo00l0oooo0P00000M0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool2000001d0oooo00<000000?ooo`3o
ool00@3oool000<0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0000000L0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0000000L0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo0080oooo00040?ooo`8000003`3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo00H0oooo0P00000>0?ooo`<000006P3oool00`000000oooo0?ooo`060?ooo`8000003P3o
ool3000001X0oooo00<000000?ooo`3oool01P3oool2000000l0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`0;0?ooo`<00000603oool2
000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3oool2
000000/0oooo00<000000?ooo`3oool02`3oool3000001T0oooo00<000000?ooo`3oool0103oool0
00L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0
oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0
oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool000P0oooo
0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool300000140oooo
0`00000O0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0oooo0P00
000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<0
00000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=
0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`000006
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00`0oooo000>0?ooo`80
00005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000oooo0?ooo`0J0?ooo`80
00005`3oool00`000000oooo0?ooo`0J0?ooo`8000005`3oool00`000000oooo0?ooo`0=0?ooo`00
403oool00`000000oooo0?ooo`0200000080oooo0P0000020?ooo`8000000P3oool200000080oooo
1000000O0?ooo`030000003oool0oooo008000000P3oool200000080oooo0P0000020?ooo`800000
0P3oool4000001l0oooo00D000000?ooo`3oool0oooo000000020?ooo`8000000P3oool200000080
oooo0P0000020?ooo`@000007`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?oo
o`3oool00P0000020?ooo`8000000P3oool2000000030?ooo`000000oooo00l0oooo000A0?oooa@0
00008P3ooolD00000280oooo5000000R0?oooaD000004@3oool00100oooo0`0000020?ooo`800000
00@0oooo00000000000000000P3oool200000080oooo0P0000020?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0503oool00`000000oooo000000020?ooo`/0000000@0oooo00000000
0000oooo0P0000090?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000P3oool;0000
00040?ooo`00000000000?ooo`8000002@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
000000<0oooo2P0000001`3oool000000000003oool000000?ooo`000000203oool00`000000oooo
0?ooo`050?ooo`003P3oool2000000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`040?ooo`8000001`3oool300000180oooo0P0000070?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0103oool2000000L0oooo0`00000B0?ooo`8000001`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00@0oooo0P0000070?ooo`<000004P3oool2000000L0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo0`00
00050?ooo`003@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`0000004@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0103oool00`000000oooo000000050?ooo`00303oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool2000000P0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00L0oooo0P0000070?oo
o`800000203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool01`3oool2000000L0oooo0P0000080?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool01@3oool000X0oooo0P00000:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`0C0?ooo`8000002@3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool04`3oool2000000T0oooo0P0000090?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool04`3oool2000000T0oooo0P0000090?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000;0?ooo`030000
003oool0oooo0180oooo0P00000B0?ooo`8000002P3oool2000000T0oooo00<000000?ooo`3oool0
203oool300000140oooo0P00000:0?ooo`8000002@3oool00`000000oooo0?ooo`080?ooo`<00000
4@3oool2000000X0oooo0P0000090?ooo`030000003oool0oooo00T0oooo0P0000090?ooo`00203o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000003o
ool0oooo00/0oooo0P0000060?ooo`800000303oool200000100oooo00<000000?ooo`3oool02`3o
ool2000000H0oooo0P00000<0?ooo`800000403oool00`000000oooo0?ooo`0;0?ooo`8000001P3o
ool2000000`0oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0<0?ooo`8000001P3oool2
000000h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00@0oooo0P00000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`040?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0103oool2000000h0oooo00<000000?ooo`3oool01P3oool0
00H0oooo00<000000?ooo`3oool03`3oool010000000oooo0?ooo`3oool2000000030?ooo`000000
oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?ooo`80000000@0oooo
000000000000oooo0`00000?0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03P3o
ool2000000030?ooo`00000000000080oooo0P00000?0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool03P3oool2000000040?ooo`00000000000?ooo`<00000403oool00`000000oooo
0?ooo`040?ooo`00103oool2000001<0oooo0`0000030?ooo`030000003oool0oooo00h0oooo0P00
00080?ooo`8000004P3oool9000000l0oooo0P0000080?ooo`8000004P3oool6000000030?ooo`00
0000oooo00l0oooo0P0000080?ooo`8000004P3oool900000100oooo00<000000?ooo`3oool00`3o
ool000<0oooo00<000000?ooo`3oool0603oool300000140oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0603oool3
00000140oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0
oooo0080oooo00020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00@3oool002@0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0403oool002@0oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo
0`00000@0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<0
00000?ooo`000000<`3oool00`000000oooo0000000@0?ooo`006`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo0100oooo000K0?ooo`030000003oool0oooo0;/0oooo000H0?ooo`80
000000<0oooo0000003oool0<P3oool00`000000oooo0000000c0?ooo`030000003oool0000003<0
oooo00<000000?ooo`0000006`3oool001P0oooo0P00000d0?ooo`@00000<P3oool400000380oooo
1000000K0?ooo`006@3oool00`000000oooo0?ooo`0b0?ooo`@00000<P3oool400000380oooo1000
000K0?ooo`00CP3oool01@000000oooo0000003oool000000340oooo00D000000?ooo`000000oooo
0000000a0?ooo`050000003oool000000?ooo`0000006P3oool001P0oooo0P00002o0?ooo`00603o
ool200000;l0oooo000K0?ooo`800000_03oool001/0oooo00<000000?ooo`3oool0^`3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`003P3oool0
10000000oooo0?ooo`0000020?ooo`<0000000<0oooo0000000000000P3oool01`000000oooo0?oo
o`000000oooo0000003oool00P0000020?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3o
ool000000P3oool3000000040?ooo`00000000000?ooo`8000000P3oool010000000oooo0000003o
ool2000000030?ooo`000000000001`0oooo00@000000?ooo`3oool000000P3oool3000000040?oo
o`00000000000?ooo`8000000P3oool010000000oooo0000003oool2000000030?ooo`0000000000
01`0oooo00@000000?ooo`3oool000000P3oool3000000030?ooo`000000000000D0oooo00@00000
0?ooo`000000oooo0P00000B0?ooo`003P3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool000000P3oool300000080oooo00@000000?ooo`3oool0
0000703oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool000000P3oool300000080oooo00@000000?ooo`3oool00000703oool010000000oooo0?oo
o`0000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000000P3oool300000080
oooo00@000000?ooo`3oool00000703oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`00000000000P3oool300000080oooo00@000000?ooo`0000000000
3`3oool000h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?ooo`030000
003oool000000080oooo0`0000030?ooo`030000003oool0000001`0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0000003oool00`0000030?oo
o`040000003oool0oooo000001/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`030?ooo`030000003oool000000080oooo0`0000030?ooo`030000003oool0000001`0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?ooo`<000000P3oool3000000<0oooo
0`00000?0?ooo`00303oool4000000D0oooo0P0000020?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`@000001@3oool200000080oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000001X0
oooo100000050?ooo`8000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo00@000000?ooo`3oool000006`3oool3000000D0oooo0P0000020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`00303oool30000
03<0oooo0`00000c0?ooo`<00000=03oool3000002L0oooo000=0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo02H0
oooo003I0?ooo`00f@3oool00080oooo00<000000?ooo`3oool03`3oool2000001d0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`8000007@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00l0oooo0P00000M0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool03`3oool2000001d0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00040?ooo`80
00003`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00H0oooo0P00000?0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool01P3oool2000000l0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`060?ooo`8000003`3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo00<0oooo00060?ooo`030000003oool0oooo00/0oooo0`00000H0?ooo`800000
2`3oool00`000000oooo0?ooo`0;0?ooo`<00000603oool2000000/0oooo00<000000?ooo`3oool0
2`3oool3000001P0oooo0P00000;0?ooo`030000003oool0oooo00/0oooo0`00000I0?ooo`030000
003oool0oooo00@0oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3o
ool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00L0oooo00080?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool30000
01l0oooo0`00000A0?ooo`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000
oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0
oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo
00030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0<0?ooo`003P3oool2000001L0oooo00<000000?ooo`3oool06P3oool2000001L0oooo00<00000
0?ooo`3oool06P3oool2000001L0oooo00<000000?ooo`3oool06P3oool2000001L0oooo00<00000
0?ooo`3oool03@3oool00100oooo00D000000?ooo`3oool0oooo000000030?ooo`050000003oool0
oooo0?ooo`0000000P3oool01P000000oooo0?ooo`3oool000000?ooo`8000007`3oool01@000000
oooo0?ooo`3oool0000000<0oooo00D000000?ooo`3oool0oooo000000030?ooo`050000003oool0
oooo0000003oool00P00000O0?ooo`050000003oool0oooo0?ooo`0000000`3oool01@000000oooo
0?ooo`3oool0000000<0oooo00@000000?ooo`3oool0oooo0`00000O0?ooo`050000003oool0oooo
0?ooo`0000000`3oool01@000000oooo0?ooo`3oool0000000<0oooo00L000000?ooo`3oool0oooo
0000003oool000000100oooo000A0?oooa@000008P3ooolD00000280oooo00<000000?ooo`000000
4@00000R0?ooo`030000003oool000000080000000<0oooo0000000000003@00000A0?ooo`00403o
ool300000080oooo0P000000103oool000000000000000020?ooo`8000000P3oool200000080oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`<000000P3oool2000000040?oo
o`000000000000000080oooo0P0000020?ooo`8000000P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo01@0oooo0`0000020?ooo`80000000@0oooo00000000000000000P3oool20000
0080oooo0P0000020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0503oool30000
0080oooo0P000000103oool000000000000000020?ooo`8000000P3oool2000000<0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000L0oooo00<000000?oo
o`3oool0103oool2000000H0oooo0P0000070?ooo`<000004P3oool2000000L0oooo00<000000?oo
o`3oool0103oool2000000H0oooo0P0000070?ooo`<000004P3oool2000000L0oooo00<000000?oo
o`3oool0103oool2000000H0oooo0P0000070?ooo`<000004P3oool2000000L0oooo00<000000?oo
o`3oool0103oool2000000L0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool000000140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0000000D0oooo000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000;0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?ooo`8000002@3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01P3o
ool2000000T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`002@3oool2
000000/0oooo00<000000?ooo`3oool04P3oool200000180oooo0P00000;0?ooo`030000003oool0
oooo0180oooo0`00000A0?ooo`8000002`3oool00`000000oooo0?ooo`0B0?ooo`<000004@3oool2
000000/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`8000002@3oool0
00P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0:0?ooo`800000403oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0:0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool0303oool2000000H0
oooo0P00000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool2000000H0
oooo0P00000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`040?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool0103oool2000000h0oooo00<000000?ooo`3oool01P3oool000H0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000003oool0000000l0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`0000003`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0000000?0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000003oool000000100oooo
00<000000?ooo`3oool0103oool000@0oooo0P00000C0?ooo`@000000P3oool00`000000oooo0?oo
o`0>0?ooo`800000203oool2000001<0oooo100000020?ooo`030000003oool0oooo00h0oooo0P00
00080?ooo`8000004`3oool5000000030?ooo`000000oooo00l0oooo0P0000080?ooo`8000004`3o
ool5000000030?ooo`000000oooo0100oooo00<000000?ooo`3oool00`3oool000<0oooo00<00000
0?ooo`3oool0603oool300000140oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0H
0?ooo`<000004@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01P0oooo0`00000A
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0603oool300000140oooo00<00000
0?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`010?oo
o`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0@0?ooo`00903oool3000003<0oooo0`00000c0?ooo`<00000
<`3oool300000100oooo000T0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3o
ool00`000000oooo0000000c0?ooo`030000003oool000000100oooo000T0?ooo`030000003oool0
oooo03<0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0@0?ooo`00D@3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool06@3oool001P0
oooo1000000c0?ooo`030000003oool000000380oooo0P0000000`3oool000000?ooo`0a0?ooo`80
000000<0oooo0000003oool06P3oool001P0oooo00@000000?ooo`0000000000<`3oool00`000000
oooo0000000b0?ooo`<00000<`3oool2000001d0oooo000H0?ooo`030000003oool0000003<0oooo
0`00000d0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0703oool001T0oooo0P00
000c0?ooo`800000=03oool2000003@0oooo0P00000M0?ooo`00CP3oool2000003@0oooo0P00000d
0?ooo`8000007@3oool008L0oooo0P00000d0?ooo`8000006P3oool008L0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool005@0oooo0`0000080?ooo`<00000
2`3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3o
ool00000EP3oool005@0oooo0`0000080?ooo`<00000203oool5000000D0oooo00@000000?ooo`3o
ool000001`3oool010000000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`00000;0?oo
o`030000003oool0oooo00@0oooo100000070?ooo`@00000EP3oool007@0oooo00@000000?ooo`3o
ool000001`3oool010000000oooo0?ooo`00001F0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N41) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N42) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N43) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N44) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N45) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N46) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N47) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N48) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N49) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C11) show
80.562500 14.625000 moveto
(N50) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C12) show
80.562500 14.625000 moveto
(N51) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C13) show
80.562500 14.625000 moveto
(N52) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C14) show
80.562500 14.625000 moveto
(N53) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C15) show
80.562500 14.625000 moveto
(N54) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C16) show
80.562500 14.625000 moveto
(N55) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C17) show
80.562500 14.625000 moveto
(N56) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C18) show
80.562500 14.625000 moveto
(N57) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C19) show
80.562500 14.625000 moveto
(N58) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C20) show
80.562500 14.625000 moveto
(N59) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C21) show
80.562500 14.625000 moveto
(N60) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000h0oooo00@000000?ooo`00
000000000P3oool300000080oooo00@000000?ooo`00000000000P3oool010000000oooo0000003o
ool2000001l0oooo00@000000?ooo`00000000000P3oool300000080oooo00@000000?ooo`000000
00000P3oool010000000oooo0000003oool2000000030?ooo`000000000001`0oooo00@000000?oo
o`00000000000P3oool3000000040?ooo`00000000000?ooo`8000000P3oool010000000oooo0000
003oool2000000030?ooo`000000000001`0oooo00@000000?ooo`00000000000P3oool300000003
0?ooo`00000000000080oooo00L000000?ooo`3oool000000?ooo`000000oooo0080000000<0oooo
0000000000003`3oool000h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0000000000080oooo0`0000020?ooo`80000000<0oooo0000003oool06`3o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0P000000103oool000000000000000020?ooo`@00000703oool010000000oooo0?ooo`0000020?oo
o`030000003oool0oooo0080oooo00H000000?ooo`000000oooo0000003oool300000080oooo0P00
00000`3oool000000000000K0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
0P3oool010000000oooo0?ooo`0000020?ooo`<0000000<0oooo0000000000000P0000000`3oool0
00000?ooo`0=0?ooo`003P3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool01000
0000oooo0000000000020?ooo`<0000000<0oooo0000003oool00P3oool00`000000oooo0?ooo`0J
0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`060000003oool000000?ooo`00
0000oooo0`000000103oool000000?ooo`3oool2000001`0oooo00L000000?ooo`3oool0oooo0000
003oool0000000D0oooo0`0000020?ooo`<0000000L0oooo0000003oool0oooo0000003oool00000
01/0oooo00L000000?ooo`3oool0oooo0000003oool0000000D0oooo00<000000?ooo`0000000P3o
ool3000000040?ooo`000000oooo0?ooo`8000003`3oool000d0oooo0`0000000`3oool000000?oo
o`020?ooo`8000001@3oool010000000oooo0?ooo`0000040?ooo`030000003oool0000000800000
6P3oool3000000030?ooo`000000oooo0080oooo0P0000050?ooo`040000003oool0oooo000000@0
oooo00@000000?ooo`3oool000006`3oool3000000030?ooo`000000oooo0080oooo0P0000020?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00
000K0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000l0oooo000<0?ooo`800000=03o
ool2000003@0oooo0P00000=0?ooo`030000003oool0oooo02D0oooo0P00000<0?ooo`030000003o
ool0oooo01T0oooo000<0?ooo`<00000<`3oool3000000`0oooo00<000000?ooo`3oool0903oool3
000000`0oooo00<000000?ooo`3oool09@3oool3000000/0oooo00<000000?ooo`3oool06@3oool0
00d0oooo00<000000?ooo`3oool0203oool4000002L0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0000000W0?ooo`030000003oool0oooo00P0oooo0P0000000`3oool000000?ooo`0W0?oo
o`030000003oool0oooo00L0oooo0P0000000`3oool000000?ooo`0J0?ooo`00603oool010000000
oooo00000000000c0?ooo`030000003oool000000380oooo0`00000c0?ooo`8000007@3oool00080
oooo00<000000?ooo`3oool04`3oool00`000000oooo0000000H0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool04`3oool3000001P0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool00@3oool000<0oooo
00<000000?ooo`3oool0303oool2000000D0oooo0P00000G0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0303oool2000000@0oooo0P00000H0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0303oool2000000@0oooo0P00000H0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0303oool2000000@0oooo0P00000H0?ooo`030000003oool0oooo0080oooo
00040?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool00P3oool2000001P0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0080oooo0P00
000H0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`020?ooo`800000603oool00`000000oooo0?ooo`020?ooo`001@3oool2000000`0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`070?ooo`800000303oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo00L0oooo0P00000<0?ooo`030000003oool0oooo00D0oooo0P00000D
0?ooo`030000003oool0oooo00L0oooo0P00000<0?ooo`030000003oool0oooo00D0oooo0P00000D
0?ooo`030000003oool0oooo00<0oooo00070?ooo`030000003oool0oooo00P0oooo0`00000J0?oo
o`800000303oool00`000000oooo0?ooo`080?ooo`<000006P3oool2000000`0oooo00<000000?oo
o`3oool0203oool3000000H0oooo00<000000?ooo`3oool04@3oool2000000`0oooo00<000000?oo
o`3oool0203oool3000000H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`040?oo
o`00203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00H0oooo00080?ooo`800000
8`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`800000
8`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`002@3o
ool2000001l0oooo0`00000C0?ooo`030000003oool0oooo01d0oooo0`00000C0?ooo`030000003o
ool0oooo01d0oooo0`00000C0?ooo`030000003oool0oooo01d0oooo0`00000:0?ooo`000P3oool2
000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2
000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool30000
00L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`8000002P3oool4000000/0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool4000000/0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool4000000/0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool4000000/0
oooo00<000000?ooo`3oool0303oool000l0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool00100oooo00<0
00000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103o
ool2000001l0oooo00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0103oool2000001l0oooo00<000000?ooo`3oool00P0000020?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0103oool2000001l0oooo00<000000?ooo`3oool00P00
00020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`0>0?ooo`004@3oool7000000H0oooo100000000`3oool000000000000:0?ooo`030000003oool0
oooo01D0oooo1`0000070?ooo`<0000000<0oooo0000000000002P3oool00`000000oooo0?ooo`0E
0?ooo`<0000000@0oooo00000000000000001`3oool3000000030?ooo`000000000000X0oooo00<0
00000?ooo`3oool05@3oool3000000040?ooo`0000000000000000L0oooo0`000000103oool00000
0000000000090?ooo`030000003oool0oooo00D0oooo000@0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo
0`00000D0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00L0oooo0`00000D0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo
0`00000D0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00H0oooo0`0000050?ooo`003P3oool2000000P0oooo0P00
00020?ooo`030000003oool0oooo00L0oooo0P0000070?ooo`030000003oool000000180oooo0P00
00080?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool00`000000oooo0000
000B0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01P3oool2000000L0oooo00<0
00000?ooo`0000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo000000050?ooo`003@3oool00`000000oooo0?ooo`09
0?ooo`<00000303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool02`3oool000X0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000N0?ooo`03
0000003oool0oooo01<0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000O0?ooo`03
0000003oool0oooo00T0oooo00090?ooo`800000803oool200000180oooo0P00000P0?ooo`<00000
4@3oool200000200oooo0`00000A0?ooo`8000008@3oool2000000T0oooo00080?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool02`3oool200000100oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02`3o
ool2000000T0oooo00070?ooo`030000003oool0oooo01@0oooo0`00000=0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0D0?ooo`<000003@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01@0oooo0`00000=0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003oool0oooo
01D0oooo00@000000?ooo`000000oooo0P00000;0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool05@3oool010000000oooo0000003oool2000000/0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0B0?ooo`070000003oool0oooo0000003oool000000?ooo`02000000/0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0C0?ooo`060000003oool000000?oo
o`000000oooo0P00000<0?ooo`030000003oool0oooo00@0oooo00040?ooo`800000603oool01000
0000oooo0?ooo`3oool2000000`0oooo0P0000080?ooo`8000005@3oool010000000oooo0?ooo`00
00030?ooo`800000303oool2000000P0oooo0P00000E0?ooo`040000003oool0oooo000000<0oooo
0P00000<0?ooo`800000203oool2000001H0oooo00<000000?ooo`0000000`3oool2000000d0oooo
00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3oool04P3oool4000000H0oooo0P00
000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0000
00060?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo0P00
00000`3oool000000?ooo`050?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0180oooo0P0000020?ooo`030000003oool0oooo00<0oooo0P00000>0?ooo`030000003o
ool0oooo0080oooo00020?ooo`030000003oool0oooo01<0oooo00@000000?ooo`00000000005`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000005`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0`00000H0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool04`3oool2000001T0oooo00<000000?ooo`3oool00@3o
ool001P0oooo00<000000?ooo`0000002@3oool00`000000oooo0?ooo`0W0?ooo`<000002@3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0100oooo000I0?ooo`8000002@3oool30000
02L0oooo0P00000:0?ooo`<00000:03oool2000000T0oooo0`00000X0?ooo`8000002@3oool30000
0100oooo000T0?ooo`030000003oool0000002L0oooo0P00000:0?ooo`030000003oool0000002P0
oooo0P0000090?ooo`030000003oool0000002P0oooo0P0000090?ooo`030000003oool000000100
oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0:P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0@0?ooo`00Q`3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01P0oooo0027
0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0603oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`003P3oool010000000oooo0000000000020?ooo`<000000P3o
ool00`000000oooo0?ooo`030?ooo`040000003oool000000?ooo`80000000<0oooo000000000000
703oool010000000oooo0000000000020?ooo`<000000P3oool010000000oooo0000000000020?oo
o`040000003oool000000?ooo`8000007`3oool010000000oooo0000000000020?ooo`<000000P3o
ool010000000oooo0000000000020?ooo`040000003oool000000?ooo`80000000<0oooo00000000
0000703oool010000000oooo0000000000020?ooo`<000000P3oool00`000000oooo0?ooo`030?oo
o`040000003oool000000?ooo`80000000<0oooo0000000000003`3oool000h0oooo00@000000?oo
o`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0000000000080oooo0`00
00020?ooo`80000000<0oooo0000003oool06`3oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool00P000000103oool000000000000000020?ooo`@0
0000703oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?oo
o`0000000P000000103oool000000000000000020?ooo`80000000<0oooo0000000000006`3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool00000
0P3oool300000080oooo1@00000>0?ooo`003P3oool01`000000oooo0?ooo`3oool000000?ooo`00
0000103oool010000000oooo0000000000020?ooo`<0000000<0oooo0000003oool00P3oool00`00
0000oooo0?ooo`0J0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003o
ool0000000<0oooo0`000000103oool000000?ooo`3oool2000001`0oooo00L000000?ooo`3oool0
oooo0000003oool0000000@0oooo00<000000?ooo`0000000`3oool3000000050?ooo`000000oooo
0?ooo`0000007@3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool010000000oooo
0?ooo`0000020?ooo`<0000000D0oooo0000003oool0oooo0000000@0?ooo`003@3oool300000003
0?ooo`000000oooo0080oooo0P0000080?ooo`030000003oool0oooo0080oooo00@000000?ooo`3o
ool000006`3oool3000000030?ooo`000000oooo0080oooo0P0000050?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool0703oool3000000030?ooo`000000oooo0080oooo0P000005
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool000006`3oool3000000030?oo
o`000000oooo0080oooo0P0000040?ooo`<0000000<0oooo0000003oool00`3oool010000000oooo
0?ooo`00000?0?ooo`00303oool2000000d0oooo00<000000?ooo`3oool0903oool2000000d0oooo
00<000000?ooo`3oool0903oool2000000d0oooo00<000000?ooo`3oool09@3oool2000002P0oooo
000<0?ooo`<00000303oool00`000000oooo0?ooo`0T0?ooo`<00000303oool00`000000oooo0?oo
o`0T0?ooo`<00000303oool00`000000oooo0?ooo`0U0?ooo`<000009`3oool000d0oooo00<00000
0?ooo`3oool0203oool2000000030?ooo`000000oooo02H0oooo00<000000?ooo`3oool0203oool2
000000030?ooo`000000oooo02H0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000
oooo02L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0L0?ooo`00603oool30000
03<0oooo0P00000d0?ooo`800000=03oool3000001`0oooo000I0?ooo`030000003oool0oooo0380
oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0b0?ooo`030000003oool0000001`0
oooo00020?ooo`030000003oool0oooo00d0oooo0P0000040?ooo`8000006@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00d0oooo0P0000040?ooo`8000006@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00d0oooo0P00000O0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool03@3oool2000000@0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0080oooo0P00
000H0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`020?ooo`800000603oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool00P3oool2000001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000=
0?ooo`030000003oool0oooo00D0oooo0P00000D0?ooo`030000003oool0oooo00H0oooo0P00000=
0?ooo`030000003oool0oooo00D0oooo0P00000D0?ooo`030000003oool0oooo00H0oooo0P00000=
0?ooo`030000003oool0oooo0080oooo0P00000G0?ooo`030000003oool0oooo00H0oooo0P00000=
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00`3oool000H0oooo00<000000?oo
o`3oool02@3oool3000000H0oooo00<000000?ooo`3oool04@3oool2000000/0oooo00<000000?oo
o`3oool02@3oool3000000H0oooo00<000000?ooo`3oool04@3oool2000000/0oooo00<000000?oo
o`3oool02@3oool3000000H0oooo0P00000B0?ooo`8000002`3oool00`000000oooo0?ooo`090?oo
o`<000006`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`030000
003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000
003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000007`3o
ool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3o
ool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2
000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool30000
00H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool4000000/0
oooo00<000000?ooo`3oool0303oool000h0oooo0P00000=0?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool06P3oool2000000d0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0J0?ooo`8000003@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo
0P0000080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0=0?ooo`00403oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`040?ooo`8000007`3oool00`000000oooo0?ooo`02000000<0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`8000007`3oool00`000000oooo0?oo
o`02000000<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`8000007`3o
ool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00h0oooo000A0?ooo`L000001P3oool4000000030?ooo`000000
00000280oooo1`0000070?ooo`<0000000<0oooo0000000000008P3oool3000000040?ooo`000000
0000000000L0oooo0`0000000`3oool000000000000R0?ooo`<0000000@0oooo0000000000000000
1P3oool4000000040?ooo`000000000000000140oooo000@0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01@3oool000h0oooo0P0000080?ooo`8000000P3oool00`000000oooo0?oo
o`070?ooo`8000001`3oool300000180oooo0P0000080?ooo`8000000`3oool00`000000oooo0?oo
o`060?ooo`8000001`3oool300000180oooo0P0000080?ooo`8000000`3oool00`000000oooo0?oo
o`060?ooo`8000001`3oool300000180oooo0P0000080?ooo`8000000`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00D0oooo0`0000050?ooo`003@3oool00`000000oooo0?ooo`09
0?ooo`<00000303oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<00000
0?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A
0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`0000004@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0000000D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`050?ooo`002P3oool2000001h0oooo00<000000?ooo`3o
ool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3o
ool04`3oool2000001l0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`800000
4P3oool200000200oooo0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`800000
2@3oool000P0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0;0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool0503oool2
000000h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0D0?ooo`8000003P3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01D0oooo0P00000=0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3oool01P3oool0
00H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0200000080oooo0P00000;0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`020000
0080oooo0P00000;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3oool01000
0000oooo0?ooo`3oool2000000030?ooo`000000000000/0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0E0?ooo`040000003oool000000?ooo`800000303oool00`000000oooo0?oo
o`040?ooo`00103oool2000001D0oooo00@000000?ooo`3oool000000`3oool2000000`0oooo0P00
00080?ooo`8000005@3oool010000000oooo0?ooo`0000030?ooo`800000303oool2000000P0oooo
0P00000E0?ooo`050000003oool0oooo0?ooo`0000000P3oool2000000`0oooo0P0000080?ooo`80
0000603oool010000000oooo0?ooo`3oool2000000d0oooo00<000000?ooo`3oool00`3oool000<0
oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000oooo00D0oooo0P00000>0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000oooo00D0
oooo0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool200000003
0?ooo`000000oooo00D0oooo0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`070?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`00
0P3oool00`000000oooo0?ooo`0C0?ooo`<00000603oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01<0oooo0P00000I0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
4`3oool2000001T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`<00000
603oool00`000000oooo0?ooo`010?ooo`006@3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo02L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0X0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool09`3oool00`000000oooo000000090?ooo`030000003o
ool0oooo0100oooo000H0?ooo`8000002P3oool3000002L0oooo0P00000:0?ooo`<00000<`3oool3
000002L0oooo00<000000?ooo`3oool02@3oool300000100oooo000H0?ooo`8000002P3oool00`00
0000oooo0000000W0?ooo`8000002P3oool00`000000oooo0000000X0?ooo`8000002@3oool00`00
0000oooo0000000c0?ooo`030000003oool000000100oooo000K0?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02P0
oooo0P0000090?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool001/0oooo
00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo04l0oooo
000K0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`1?0?ooo`00Q`3oool00`000000oooo0?ooo`1?0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool000h0oooo00@000000?ooo`00000000000P3oool300000080oooo00@000000?ooo`000000
00000P3oool00`000000oooo000000040?ooo`800000703oool010000000oooo0000000000020?oo
o`<000000P3oool010000000oooo0?ooo`0000020?ooo`040000003oool000000?ooo`80000000<0
oooo000000000000703oool010000000oooo0000000000020?ooo`<000000P3oool010000000oooo
0000000000020?ooo`040000003oool000000?ooo`8000000P3oool00`000000oooo0?ooo`0J0?oo
o`040000003oool0000000000080oooo0`0000020?ooo`040000003oool0000000000080oooo00@0
00000?ooo`000000oooo0P0000000`3oool000000000000?0?ooo`003P3oool010000000oooo0?oo
o`0000020?ooo`030000003oool0oooo0080oooo00H000000?ooo`000000oooo0000003oool30000
00030?ooo`00000000000080oooo0P00000K0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool3000000030?ooo`00
0000oooo01X0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000
003oool0oooo00000080oooo0`0000020?ooo`80000000<0oooo0000003oool06`3oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000000P3oool3
00000080oooo0P0000000`3oool000000?ooo`0>0?ooo`003P3oool01`000000oooo0?ooo`3oool0
00000?ooo`000000103oool010000000oooo0000000000020?ooo`<0000000L0oooo000000000000
oooo0000003oool0000001/0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00@0
00000?ooo`3oool000000P3oool3000000040?ooo`000000oooo0?ooo`800000703oool01`000000
oooo0?ooo`3oool000000?ooo`000000103oool01P000000oooo0?ooo`3oool000000?ooo`<00000
00<0oooo0000003oool00P3oool00`000000oooo0?ooo`0J0?ooo`070000003oool0oooo0?ooo`00
0000oooo000000040?ooo`040000003oool0oooo00000080oooo0`0000000`3oool000000?ooo`03
0?ooo`030000003oool0oooo00`0oooo000=0?ooo`<0000000<0oooo0000003oool00P3oool20000
00D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`0I0?ooo`<0000000<0oooo
0000003oool00P3oool2000000P0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00
000K0?ooo`<0000000<0oooo0000003oool00P3oool2000000D0oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`0L0?ooo`<0000000<0oooo0000003oool00P3oool2000000D0oooo
00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00000?0?ooo`00303oool2000000d0
oooo00<000000?ooo`3oool0903oool2000003@0oooo0P00000e0?ooo`800000:03oool000`0oooo
0`00000<0?ooo`030000003oool0oooo02@0oooo0`00000c0?ooo`<00000=03oool3000000/0oooo
00<000000?ooo`3oool06@3oool000d0oooo00<000000?ooo`3oool0203oool2000000030?ooo`00
0000oooo02H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0X0?ooo`030000003o
ool0oooo00P0oooo1000000X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`0000006`3o
ool001P0oooo0P00000d0?ooo`<00000<`3oool010000000oooo00000000000c0?ooo`030000003o
ool0000001/0oooo000B0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0000000c0?ooo`030000003oool0000003<0oooo0`00000L0?ooo`000P3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0=0?ooo`800000103oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool03@3oool2000000D0oooo0P00000H0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool03@3oool2000000@0oooo0P00000I0?ooo`030000003oool0oooo
0040oooo00030?ooo`030000003oool0oooo00/0oooo00<000000?ooo`000000103oool2000001P0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0080
oooo0P00000H0?ooo`030000003oool0oooo0080oooo00040?ooo`800000303oool3000000<0oooo
0P00000G0?ooo`030000003oool0oooo00H0oooo0P00000=0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool01P3oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00<0oooo
00060?ooo`030000003oool0oooo0180oooo0P00000B0?ooo`8000002`3oool00`000000oooo0?oo
o`090?ooo`<000006P3oool2000000/0oooo00<000000?ooo`3oool02@3oool3000001X0oooo0P00
000;0?ooo`030000003oool0oooo00T0oooo0`00000K0?ooo`030000003oool0oooo00@0oooo0007
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`06
0?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool01`3oool000P0oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<0
00007`3oool300000140oooo0`00000O0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000
003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`800000
3@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3o
ool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`@000002`3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool4
000000/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00T0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`0<0?ooo`003P3oool20000
00d0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0J0?ooo`8000003@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P00000=0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool06P3oool2000000h0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`0=0?ooo`00403oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`050?ooo`8000007`3oool00`000000oooo0?ooo`02000000<0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`8000007`3oool00`000000
oooo0?ooo`02000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`80
00007`3oool00`000000oooo0?ooo`02000000<0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00h0oooo000A0?ooo`L000001P3oool4000000030?oo
o`00000000000280oooo1`0000060?ooo`@0000000<0oooo0000000000008P3oool3000000040?oo
o`0000000000000000H0oooo100000000`3oool000000000000R0?ooo`<0000000@0oooo00000000
000000001P3oool4000000040?ooo`000000000000000140oooo000@0?ooo`030000003oool00000
00<0oooo0P0000060?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P0000060?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool00000
00<0oooo0P0000060?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P0000060?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0
oooo00D000000?ooo`3oool0oooo000000090?ooo`8000001`3oool300000180oooo0P0000080?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool01P3oool2000000L0oooo0`00000B0?oo
o`800000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0oooo0P0000070?oo
o`<000004P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00D0oooo0`0000050?ooo`003@3oool00`000000oooo0?ooo`090?ooo`<0
0000303oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3o
ool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`03
0000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`00
00004@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0000000D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`050?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3o
ool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3o
ool2000001l0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool2
00000200oooo0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool0
00P0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0:0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool0503oool3000000d0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0D0?ooo`8000003P3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool05@3oool2000000d0oooo00<000000?ooo`3oool01P3oool000H0oooo
00<000000?ooo`3oool04P3oool01`000000oooo0?ooo`000000oooo0000003oool00P00000;0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool05@3oool200000080oooo0P00000;0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool05P3oool2000000030?ooo`0000000000
00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F0?ooo`80000000<0oooo0000
00000000303oool00`000000oooo0?ooo`040?ooo`00103oool2000001D0oooo00@000000?ooo`3o
ool000000`3oool2000000`0oooo0P0000080?ooo`800000603oool010000000oooo0?ooo`3oool2
000000`0oooo0P0000080?ooo`8000006@3oool00`000000oooo0?ooo`02000000`0oooo0P000008
0?ooo`8000005@3oool01@000000oooo0?ooo`3oool000000080oooo0P00000=0?ooo`030000003o
ool0oooo00<0oooo00030?ooo`030000003oool0oooo0180oooo0P0000000`3oool000000?ooo`05
0?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool01`3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B
0?ooo`@000001P3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0000000H0oooo0P00000>0?ooo`030000003oool0oooo0080oooo00020?oo
o`030000003oool0oooo01<0oooo0P00000I0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool04`3oool3000001P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?oo
o`040000003oool00000000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0000001L0oooo00<000000?ooo`3oool00@3oool001T0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0W0?ooo`030000003oool0000000T0oooo00<000000?oo
o`3oool09`3oool00`000000oooo000000090?ooo`030000003oool0oooo02L0oooo0`0000090?oo
o`030000003oool0oooo0100oooo000T0?ooo`<000009`3oool00`000000oooo0?ooo`090?ooo`<0
0000:03oool2000000T0oooo0`00000W0?ooo`8000002P3oool300000100oooo000H0?ooo`800000
2P3oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000
9`3oool2000000X0oooo00<000000?ooo`000000403oool001P0oooo0P00000:0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo0100oooo000K0?ooo`800000_03oool001/0oooo00<000000?ooo`3oool0^`3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00
000000000P3oool3000000070?ooo`00000000000?ooo`000000oooo000000040?ooo`800000803o
ool010000000oooo0000000000020?ooo`<00000103oool00`000000oooo000000040?ooo`800000
803oool010000000oooo0000000000020?ooo`<0000000L0oooo000000000000oooo0000003oool0
000002H0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo00<0
00000?ooo`000000103oool200000100oooo000@0?ooo`040000003oool0oooo00000080oooo00@0
00000?ooo`3oool0oooo1P0000000`3oool00000000000020?ooo`8000007`3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P000000103oool0
00000000003oool3000001l0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3o
ool2000000050?ooo`00000000000000003oool00P0000020?ooo`030000003oool0oooo01h0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`80000000D0oooo00000000
000000000?ooo`02000000030?ooo`00000000000100oooo000@0?ooo`070000003oool0oooo0?oo
o`000000oooo000000030?ooo`030000003oool0oooo00<0000000D0oooo000000000000oooo0000
000Q0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool000000080
000000D0oooo000000000000oooo0000000Q0?ooo`070000003oool0oooo0?ooo`000000oooo0000
00030?ooo`80000000D0oooo00000000000000000?ooo`0200000080oooo00<000000?ooo`3oool0
7P3oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool010000000oooo0000003oool3
000000040?ooo`00000000000?ooo`800000403oool000`0oooo0P0000001P3oool0000000000000
0000oooo000000<0oooo0P0000020?ooo`030000003oool0000000L0oooo00<000000?ooo`3oool0
6P3oool2000000060?ooo`00000000000000003oool000000`3oool2000000030?ooo`0000000000
008000001`3oool00`000000oooo0?ooo`0J0?ooo`80000000H0oooo00000000000000000?ooo`00
00030?ooo`8000000P3oool00`000000oooo000000060?ooo`<000006`3oool2000000060?ooo`00
000000000000003oool000000`3oool200000080oooo00@000000?ooo`3oool000001`3oool00`00
0000oooo0?ooo`0>0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000303oool00`000000
oooo0?ooo`0T0?ooo`<00000303oool00`000000oooo0?ooo`0I0?ooo`003@3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool4000002L0oooo
00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02H0oooo00<000000?ooo`3oool0
203oool2000000030?ooo`000000oooo01X0oooo000H0?ooo`<00000<`3oool010000000oooo0000
0000000b0?ooo`@00000<P3oool2000000030?ooo`000000oooo01X0oooo000B0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`000000;@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0000002d0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0703oool00080oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo0P00000H0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`8000006@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00P3oool2
000001T0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo000000040?ooo`800000603oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`000000103oool2000001P0oooo00<000000?ooo`3oool00P3oool000@0
oooo0P00000<0?ooo`<00000703oool00`000000oooo0?ooo`060?ooo`800000303oool3000001`0
oooo00<000000?ooo`3oool01P3oool2000000`0oooo0`0000060?ooo`800000503oool00`000000
oooo0?ooo`060?ooo`800000303oool3000000H0oooo0P00000D0?ooo`030000003oool0oooo00<0
oooo00060?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo
0P00000;0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3oool2000000/0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00@0oooo
00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S
0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S
0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?oo
o`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?oo
o`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`80
00003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`03
0000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
3@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<00000
1P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`@000002`3o
ool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3o
ool4000000/0oooo00<000000?ooo`3oool0303oool000h0oooo0P0000090?ooo`050000003oool0
oooo0?ooo`0000002@3oool00`000000oooo0?ooo`0J0?ooo`8000002@3oool01@000000oooo0?oo
o`3oool0000000T0oooo00<000000?ooo`3oool06P3oool2000000T0oooo00D000000?ooo`3oool0
oooo000000090?ooo`030000003oool0oooo01X0oooo0P0000090?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0=0?ooo`00403oool00`000000oooo
0?ooo`0200000080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`800000
7`3oool00`000000oooo0?ooo`02000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`050?ooo`8000007`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`050?ooo`8000007`3oool01@000000oooo0?ooo`3oool00000
00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00h0oooo000A0?ooo`L000001P3oool4000000030?ooo`00000000000280oooo1`0000060?ooo`@0
000000<0oooo0000000000008P3oool7000000H0oooo100000000`3oool000000000000R0?ooo`L0
00001P3oool4000000040?ooo`000000000000000140oooo000@0?ooo`<000000P3oool3000000H0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`0000000P3oool3000000H0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000P3oool3000000H0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`0000000`3oool2000000H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000>0?ooo`800000203oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo
0P0000080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01P3oool2000000L0oooo
0`00000B0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0oooo00T0oooo1000
000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?oo
o`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<0
00000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000
000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00D0oooo000:0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?oo
o`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?oo
o`8000007`3oool00`000000oooo0?ooo`090?ooo`002@3oool200000200oooo0P00000B0?ooo`80
0000803oool300000140oooo0P00000P0?ooo`<000004@3oool200000240oooo0P0000090?ooo`00
203oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool02`3oool200000100oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo00/0oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0D0?ooo`<000003@3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01@0oooo0`00000=0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0D0?ooo`<000003@3oool00`000000oooo0?ooo`060?ooo`001P3oool0
0`000000oooo0?ooo`0E0?ooo`040000003oool000000?ooo`8000002`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo01D0oooo00@000000?ooo`000000oooo0P00000;0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool05@3oool010000000oooo0000003oool2000000/0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool000000?oo
o`800000303oool00`000000oooo0?ooo`040?ooo`00103oool2000001P0oooo00@000000?ooo`3o
ool0oooo0P00000<0?ooo`800000203oool2000001P0oooo00@000000?ooo`3oool0oooo0P00000<
0?ooo`800000203oool2000001D0oooo00@000000?ooo`3oool000000`3oool2000000`0oooo0P00
00080?ooo`8000005@3oool010000000oooo0?ooo`0000030?ooo`8000003@3oool00`000000oooo
0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00L0oooo0P00
000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool4000000H0oooo0P00
000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`00
0000oooo00D0oooo0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3o
ool2000000030?ooo`000000oooo00D0oooo0P00000>0?ooo`030000003oool0oooo0080oooo0002
0?ooo`030000003oool0oooo01<0oooo0`00000H0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool04`3oool010000000oooo00000000000G0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0C0?ooo`80000000<0oooo0000003oool05P3oool00`000000oooo0?ooo`010?ooo`00603oool0
0`000000oooo000000090?ooo`030000003oool0oooo02L0oooo00<000000?ooo`0000002@3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0100oooo000H0?ooo`030000003oool0oooo
00T0oooo0`00000X0?ooo`8000002@3oool3000002L0oooo0P00000:0?ooo`<000009`3oool20000
00X0oooo0`00000@0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003oool0000002L0
oooo0P00000:0?ooo`030000003oool0000002L0oooo0P00000:0?ooo`030000003oool000000100
oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0:P3oool2000000L0oooo
00<000000?ooo`3oool0:P3oool2000000L0oooo00<000000?ooo`3oool0403oool008L0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00000000000P3oool3
000000070?ooo`00000000000?ooo`000000oooo000000050?ooo`030000003oool0oooo01h0oooo
00@000000?ooo`00000000000P3oool3000000070?ooo`00000000000?ooo`000000oooo00000004
0?ooo`800000803oool010000000oooo0000000000020?ooo`<00000103oool00`000000oooo0000
00040?ooo`800000803oool010000000oooo0000000000020?ooo`<0000000<0oooo000000000000
0P3oool00`000000oooo0000000F0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`0000000P0000000`3oool00000000000020?ooo`030000
003oool0oooo01h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool000000080000000<0oooo0000000000000P3oool00`000000oooo0?ooo`0N0?ooo`04
0000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000001@3oool0000000000000
0000oooo008000000P3oool00`000000oooo0?ooo`0N0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00P3oool2000000050?ooo`00000000000000003oool00P0000000`3oool0
00000000000@0?ooo`00403oool01`000000oooo0?ooo`3oool000000?ooo`0000001@3oool40000
00030?ooo`00000000000080oooo00<000000?ooo`3oool07P3oool01`000000oooo0?ooo`3oool0
00000?ooo`000000103oool00`000000oooo00000002000000030?ooo`000000000000<0oooo00<0
00000?ooo`3oool07@3oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool200000005
0?ooo`00000000000000003oool00P0000020?ooo`030000003oool0oooo01h0oooo00L000000?oo
o`3oool0oooo0000003oool0000000<0oooo00@000000?ooo`3oool0oooo0`000000103oool00000
0000003oool200000100oooo000<0?ooo`80000000H0oooo00000000000000000?ooo`0000030?oo
o`8000000P3oool00`000000oooo0000000T0?ooo`80000000H0oooo00000000000000000?ooo`00
00030?ooo`8000000P3oool00`000000oooo000000070?ooo`030000003oool0oooo01X0oooo0P00
00001P3oool00000000000000000oooo000000<0oooo0P0000040?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool06`3oool5000000030?ooo`000000oooo0080oooo0P0000020?ooo`04
0000003oool0oooo000001P0oooo000<0?ooo`<00000<`3oool3000000`0oooo00<000000?ooo`3o
ool0903oool3000000`0oooo00<000000?ooo`3oool09@3oool3000000/0oooo00<000000?ooo`3o
ool06@3oool000d0oooo00<000000?ooo`3oool0203oool4000002L0oooo00<000000?ooo`3oool0
203oool2000000030?ooo`000000oooo02H0oooo00<000000?ooo`3oool0203oool2000000030?oo
o`000000oooo02L0oooo00<000000?ooo`3oool01`3oool2000000030?ooo`000000oooo01X0oooo
000H0?ooo`040000003oool0000000000380oooo1000000b0?ooo`80000000<0oooo0000003oool0
<@3oool2000000030?ooo`000000oooo01X0oooo000B0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`000000;@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool06`3oool00080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`03
0?ooo`800000603oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool00P3oool2000001T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo0080oooo0P00000I0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0040oooo00030?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`0000007P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`000000103oool2000001P0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000000@0oooo0P00000H0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool00`000000oooo000000040?oo
o`800000603oool00`000000oooo0?ooo`020?ooo`00103oool2000000`0oooo0`00000L0?ooo`03
0000003oool0oooo00H0oooo0P00000<0?ooo`<000001P3oool2000001@0oooo00<000000?ooo`3o
ool01P3oool2000000`0oooo0`0000060?ooo`800000503oool00`000000oooo0?ooo`060?ooo`80
0000303oool3000000<0oooo0P00000G0?ooo`030000003oool0oooo00<0oooo00060?ooo`030000
003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4@3oool2000000/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`800000
2`3oool00`000000oooo0?ooo`0B0?ooo`8000004`3oool00`000000oooo0?ooo`040?ooo`001`3o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3o
ool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00L0oooo00080?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool30000
01l0oooo0`00000A0?ooo`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000
oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0
oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo
00030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00
000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`090?ooo`<00000303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool0303oool000h0oooo0P000009
0?ooo`030000003oool0oooo008000002@3oool00`000000oooo0?ooo`0J0?ooo`8000002@3oool0
0`000000oooo0?ooo`02000000T0oooo00<000000?ooo`3oool06P3oool2000000T0oooo00<00000
0?ooo`3oool00P0000090?ooo`030000003oool0oooo01X0oooo0P0000090?ooo`060000003oool0
oooo0000003oool00000203oool00`000000oooo0?ooo`0=0?ooo`00403oool01@000000oooo0?oo
o`3oool000000080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`800000
7`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`050?ooo`8000007`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`050?ooo`8000007`3oool01@000000oooo0?ooo`3oool0
000000<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo00h0oooo000A0?ooo`L000001P3oool4000000030?ooo`00000000000280oooo1`0000060?oo
o`@0000000<0oooo0000000000008P3oool00`000000oooo00000004000000H0oooo100000000`3o
ool000000000000R0?ooo`030000003oool000000080000000<0oooo0000003oool01@3oool40000
00040?ooo`000000000000000140oooo000@0?ooo`<000000P3oool3000000H0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo0`0000020?oo
o`<000001P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0503oool300000080oooo0`0000060?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`<000000P3oool3000000H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000>0?ooo`80
0000203oool00`000000oooo0?ooo`0;0?ooo`8000001`3oool300000180oooo0P0000080?ooo`03
0000003oool0oooo00/0oooo0P0000070?ooo`<000004P3oool2000000P0oooo00<000000?ooo`3o
ool02`3oool2000000L0oooo0`00000B0?ooo`800000203oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000
003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`000000
4@3oool00`000000oooo0?ooo`090?ooo`D000002P3oool00`000000oooo0?ooo`040?ooo`030000
003oool000000140oooo00<000000?ooo`3oool02@3oool5000000X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1@00000:0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000;0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool20000
0200oooo0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0:0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool0503oool2000000h0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0D0?ooo`8000003P3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool05@3oool2000000d0oooo00<000000?ooo`3oool01P3oool000H0oooo00<0
00000?ooo`3oool05@3oool200000080oooo0P00000;0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool05@3oool200000080oooo0P00000;0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool05P3oool2000000030?ooo`000000000000/0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0F0?ooo`80000000<0oooo000000000000303oool00`000000oooo0?oo
o`040?ooo`00103oool2000001P0oooo00@000000?ooo`3oool0oooo0P00000<0?ooo`800000203o
ool2000001D0oooo00@000000?ooo`3oool000000`3oool2000000`0oooo0P0000080?ooo`800000
5@3oool01@000000oooo0?ooo`3oool000000080oooo0P00000<0?ooo`800000203oool2000001D0
oooo00D000000?ooo`3oool0oooo000000020?ooo`8000003@3oool00`000000oooo0?ooo`030?oo
o`000`3oool00`000000oooo0?ooo`0B0?ooo`@000001P3oool2000000h0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool01@3oool2000000h0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`80000000<0oooo0000003o
ool01@3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`80
000000<0oooo0000003oool01@3oool2000000h0oooo00<000000?ooo`3oool00P3oool00080oooo
00<000000?ooo`3oool04`3oool010000000oooo00000000000G0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0C0?ooo`80000000<0oooo0000003oool05P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo01<0oooo0P0000000`3oool000000?ooo`0F0?ooo`030000003oool0oooo0040
oooo000H0?ooo`030000003oool0000000T0oooo00<000000?ooo`3oool0:03oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0403oool001T0oooo0P00
00090?ooo`<000009`3oool2000000X0oooo0`00000W0?ooo`8000002P3oool3000003<0oooo0`00
000@0?ooo`00903oool00`000000oooo0000000W0?ooo`8000002P3oool00`000000oooo0000000W
0?ooo`8000002P3oool00`000000oooo0000000W0?ooo`8000002P3oool00`000000oooo0000000@
0?ooo`00903oool00`000000oooo0?ooo`0Z0?ooo`8000001`3oool00`000000oooo0?ooo`0Z0?oo
o`8000001`3oool00`000000oooo0?ooo`0W0?ooo`8000002P3oool00`000000oooo0?ooo`0@0?oo
o`00D@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo0P00000J0?ooo`00
_@3oool00`000000oooo0?ooo`0I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool005@0oooo0`0000080?ooo`<000002`3oool00`000000oooo0?ooo`04
0?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool00000EP3oool005@0oooo0`00
00080?ooo`<00000203oool5000000D0oooo00@000000?ooo`3oool000001`3oool010000000oooo
0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`030000003oool0oooo00@0oooo
100000070?ooo`@00000EP3oool007@0oooo00@000000?ooo`3oool000001`3oool010000000oooo
0?ooo`00001F0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C22) show
80.562500 14.625000 moveto
(N61) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N62) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N63) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N64) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N65) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N66) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N67) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N68) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N69) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N70) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N71) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N72) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N73) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N74) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N75) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N76) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N77) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N78) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N79) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N80) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00
000000000P3oool3000000@0oooo00<000000?ooo`0000009P3oool010000000oooo000000000002
0?ooo`<0000000L0oooo000000000000oooo0000003oool0000000@0oooo0P00000P0?ooo`040000
003oool0000000000080oooo0`0000001`3oool000000000003oool000000?ooo`000000103oool2
00000200oooo00@000000?ooo`00000000000P3oool3000000D0oooo00@000000?ooo`000000oooo
0P0000000`3oool000000000000@0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`040000
003oool0oooo0?ooo`80000000@0oooo00000000000000000P3oool010000000oooo0?ooo`00000P
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool500000080oooo00@0
00000?ooo`0000000000803oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?oo
o`H000000P3oool00`000000oooo0?ooo`02000001l0oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0000000L0oooo000000000000oooo0000
003oool0000000l0oooo000@0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`80
000000@0oooo00000000000000000P3oool010000000oooo0?ooo`00000P0?ooo`070000003oool0
oooo0?ooo`000000oooo000000030?ooo`030000003oool0oooo00<000000P3oool010000000oooo
00000000000P0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`030000003oool0
oooo00<000000P3oool01@000000oooo0000003oool0000001l0oooo00L000000?ooo`3oool0oooo
0000003oool0000000@0oooo00<000000?ooo`3oool00`000000103oool000000000003oool20000
0100oooo000?0?ooo`<0000000<0oooo0000003oool00P3oool2000000@0oooo00@000000?ooo`3o
ool0oooo1P00000N0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<000000?oo
o`0000000`3oool3000000030?ooo`000000oooo01h0oooo0`0000000`3oool000000?ooo`020?oo
o`8000000P3oool00`000000oooo000000030?ooo`<0000000<0oooo0000003oool07P3oool30000
00030?ooo`000000oooo0080oooo0P0000001P3oool00000000000000000oooo000000@0oooo00@0
00000?ooo`3oool00000403oool000`0oooo0P00000d0?ooo`800000=03oool2000003D0oooo0P00
000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03oool3000002L0oooo000=0?oo
o`030000003oool0oooo00P0oooo1000000W0?ooo`030000003oool0oooo00P0oooo1000000W0?oo
o`030000003oool0oooo00P0oooo1000000X0?ooo`030000003oool0oooo00L0oooo1000000K0?oo
o`00603oool010000000oooo00000000000b0?ooo`040000003oool0000000000380oooo00@00000
0?ooo`0000000000<P3oool010000000oooo00000000000K0?ooo`000P3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`000000603oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01<0oooo00<000000?ooo`000000603oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01<0oooo00<000000?ooo`000000603oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01<0oooo00<000000?ooo`000000603oool00`000000oooo0?oo
o`010?ooo`000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo0P00000G
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0303oool2000000D0oooo0P00000G
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0303oool2000000D0oooo0P00000G
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0303oool2000000D0oooo0P00000G
0?ooo`030000003oool0oooo0080oooo00040?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`0000007P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`020?ooo`001@3oool2000000/0oooo0`00000L0?ooo`03
0000003oool0oooo00L0oooo0P00000<0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool01`3oool2000000`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`070?ooo`80
0000303oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00<0oooo00070?ooo`030000
003oool0oooo02D0oooo0P00000<0?ooo`030000003oool0oooo00P0oooo0`00000J0?ooo`800000
303oool00`000000oooo0?ooo`080?ooo`<000006P3oool2000000`0oooo00<000000?ooo`3oool0
203oool3000001/0oooo00<000000?ooo`3oool0103oool000P0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2
000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2
000002<0oooo00<000000?ooo`3oool01`3oool000T0oooo0P00000O0?ooo`<000004`3oool00`00
0000oooo0?ooo`0M0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<000004`3oool00`00
0000oooo0?ooo`0M0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0
oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0
oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo
0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<0
00000?ooo`3oool01`3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool01P3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool01P3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool01P3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00`0oooo000?
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000003oool0oooo008000000P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo
008000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo0P00000O0?oo
o`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00@0oooo0P00000O0?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool03P3oool00100oooo200000060?ooo`@0
000000<0oooo0000000000002P3oool00`000000oooo0?ooo`0D0?ooo`T000001@3oool400000003
0?ooo`000000000000X0oooo00<000000?ooo`3oool0503oool4000000030?ooo`00000000000080
00001@3oool4000000030?ooo`000000000000X0oooo00<000000?ooo`3oool0503oool400000003
0?ooo`0000000000008000001@3oool4000000040?ooo`0000000000000000T0oooo00<000000?oo
o`3oool01@3oool00100oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`80
00001P3oool00`000000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`<0
0000503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`<0
00001@3oool000h0oooo0P0000080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
1P3oool2000000L0oooo00<000000?ooo`0000004P3oool2000000P0oooo00<000000?ooo`3oool0
0P3oool2000000L0oooo0P0000070?ooo`030000003oool000000180oooo0P0000080?ooo`030000
003oool0oooo0080oooo0P0000070?ooo`8000001`3oool00`000000oooo0000000B0?ooo`800000
203oool00`000000oooo0?ooo`020?ooo`800000203oool00`000000oooo0?ooo`050?ooo`030000
003oool0000000D0oooo000=0?ooo`030000003oool0oooo00P0oooo1P00000:0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`080?ooo`H000002P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0203o
ool6000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00P0oooo1P00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3o
ool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`002P3oool20000
01h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool20000
01h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo00<000000?ooo`3oool02@3oool000T0
oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?ooo`800000803oool300000140
oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo
0P00000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02P3oool200000100oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`8000002@3oool000L0oooo00<0
00000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0D0?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01D0oooo0P00
000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<0
00000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool05@3oool010000000oooo0000003o
ool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`8000000P3o
ool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F0?ooo`80000000<0
oooo0000000000002`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01D0oooo00@0
00000?ooo`000000oooo0P00000<0?ooo`030000003oool0oooo00@0oooo00040?ooo`800000603o
ool010000000oooo0?ooo`3oool2000000`0oooo0P0000080?ooo`800000603oool010000000oooo
0?ooo`3oool2000000`0oooo0P0000080?ooo`8000006@3oool00`000000oooo0?ooo`02000000`0
oooo0P0000080?ooo`800000603oool010000000oooo0?ooo`3oool2000000d0oooo00<000000?oo
o`3oool00`3oool000<0oooo00<000000?ooo`3oool04`3oool00`000000oooo000000060?ooo`80
00003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo00<000000?ooo`00
00001P3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0000000H0oooo0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo000000060?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`00
0P3oool00`000000oooo0?ooo`0C0?ooo`@000005`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01<0oooo1000000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
4`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`@00000
5`3oool00`000000oooo0?ooo`010?ooo`00603oool4000000P0oooo00<000000?ooo`3oool09`3o
ool4000000P0oooo00<000000?ooo`3oool09`3oool4000000P0oooo00<000000?ooo`3oool09`3o
ool4000000P0oooo00<000000?ooo`3oool0403oool001P0oooo00D000000?ooo`000000oooo0000
00070?ooo`<000009`3oool01@000000oooo0000003oool0000000L0oooo0`00000W0?ooo`050000
003oool000000?ooo`0000001`3oool3000002L0oooo00D000000?ooo`000000oooo000000070?oo
o`<00000403oool002@0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`03
0000003oool0000003<0oooo00<000000?ooo`000000403oool002@0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
403oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000@
0?ooo`040000003oool0000000000080oooo0`0000040?ooo`030000003oool0000000@0oooo0P00
000P0?ooo`040000003oool0000000000080oooo0`0000020?ooo`050000003oool000000?ooo`00
00009P3oool010000000oooo0000000000020?ooo`<0000000L0oooo000000000000oooo0000003o
ool0000000@0oooo0P00000P0?ooo`040000003oool0000000000080oooo0`0000000`3oool00000
000000020?ooo`030000003oool0000000@0oooo0P00000@0?ooo`00403oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo0?ooo`80000000@0oooo00000000000000000P3oool01000
0000oooo0?ooo`00000P0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0000000200000080oooo00@000000?ooo`0000000000803oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`03
0000003oool0oooo008000007`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00`0000020?ooo`030000003oool00000008000003`3oool00100
oooo00L000000?ooo`3oool0oooo0000003oool0000000<0oooo0P000000103oool0000000000000
00020?ooo`040000003oool0oooo00000200oooo00L000000?ooo`3oool0oooo0000003oool00000
00@0oooo00<000000?ooo`0000000P0000020?ooo`040000003oool0000000000200oooo00L00000
0?ooo`3oool0oooo0000003oool0000000D0oooo100000020?ooo`030000003oool000000240oooo
00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`3oool00`0000020?oo
o`030000003oool000000140oooo000?0?ooo`<0000000<0oooo0000003oool00P3oool2000000@0
oooo00@000000?ooo`3oool0oooo0`0000000`3oool000000?ooo`0N0?ooo`<0000000<0oooo0000
003oool00P3oool2000000@0oooo00@000000?ooo`3oool0oooo0`00000Q0?ooo`<0000000<0oooo
0000003oool00P3oool200000080oooo00<000000?ooo`0000000`3oool3000000030?ooo`000000
oooo01h0oooo0`0000000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`00
00030?ooo`<0000000<0oooo0000003oool03`3oool000`0oooo0P00000d0?ooo`800000=03oool2
000003D0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03oool30000
02L0oooo000=0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`0000009`3oool00`000000
oooo0?ooo`080?ooo`@000009`3oool00`000000oooo0?ooo`080?ooo`@00000:03oool00`000000
oooo0?ooo`070?ooo`@000006`3oool001P0oooo1000000b0?ooo`040000003oool0000000000380
oooo00@000000?ooo`0000000000<P3oool010000000oooo00000000000K0?ooo`00603oool40000
02`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0000000]0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`000000;@3oool00`000000oooo0?ooo`030?ooo`030000003oool00000
01`0oooo00020?ooo`030000003oool0oooo00d0oooo0P0000040?ooo`050000003oool000000?oo
o`0000005P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool00`3oool2000001P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo00<0oooo0P00000H0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`030?ooo`800000603oool00`000000oooo0?ooo`010?oo
o`000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3o
ool00P3oool000@0oooo0P00000=0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
1P3oool2000000`0oooo0`00000L0?ooo`030000003oool0oooo00H0oooo0P00000<0?ooo`<00000
703oool00`000000oooo0?ooo`060?ooo`800000303oool3000001`0oooo00<000000?ooo`3oool0
0`3oool000H0oooo00<000000?ooo`3oool02@3oool3000001X0oooo0P00000;0?ooo`030000003o
ool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000003o
ool0oooo02L0oooo00<000000?ooo`3oool0103oool000L0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool20000
02<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool20000
02<0oooo00<000000?ooo`3oool01`3oool000P0oooo0`00000O0?ooo`<000004@3oool3000001l0
oooo0`00000A0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000002P3oool00080oooo
0P0000070?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo
0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo
0P0000060?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo0`00
00070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`08
0?ooo`H000002P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`0<
0?ooo`003P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo01X0oooo0P00000=0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool06P3oool2000000d0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0J0?oo
o`800000203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool03@3oool00100oooo00<000000?ooo`3oool00P0000020?ooo`800000103oool2000000H0
oooo0P00000O0?ooo`030000003oool0oooo008000000`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo008000000`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo0080
00000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool03P3oool00100oooo200000060?ooo`@0000000<0oooo0000000000008@3oool8000000H0oooo
100000000`3oool000000000000Q0?ooo`@0000000@0oooo00000000000000001P3oool400000003
0?ooo`00000000000240oooo100000000`3oool0000000000002000000D0oooo10000000103oool0
000000000000000A0?ooo`00403oool00`000000oooo0?ooo`040?ooo`800000103oool2000000H0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo
0P0000080?ooo`030000003oool0oooo0080oooo0P0000070?ooo`8000001`3oool300000180oooo
0P0000080?ooo`030000003oool0oooo0080oooo0P0000070?ooo`8000001`3oool300000180oooo
0P0000080?ooo`030000003oool0oooo0080oooo0P0000080?ooo`030000003oool0oooo00D0oooo
0`0000050?ooo`003@3oool00`000000oooo0?ooo`080?ooo`H000002P3oool00`000000oooo0?oo
o`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool0203oool6000000X0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00P0oooo1P00
000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?oo
o`080?ooo`H000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000D0oooo000<
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`05
0?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?oo
o`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo00<000000?oo
o`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?ooo`80
0000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
2P3oool200000100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000
2@3oool000L0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0D0?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0503o
ool3000000d0oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool05@3oool0
10000000oooo0000003oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0E0?ooo`8000000P3oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0F0?ooo`80000000<0oooo0000000000002`3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo01D0oooo00@000000?ooo`000000oooo0P00000<0?ooo`030000003oool0oooo00@0oooo
00040?ooo`800000603oool010000000oooo0?ooo`3oool2000000`0oooo0P0000080?ooo`800000
603oool010000000oooo0?ooo`3oool2000000`0oooo0P0000080?ooo`8000006@3oool00`000000
oooo0?ooo`02000000`0oooo0P0000080?ooo`800000603oool010000000oooo0?ooo`3oool20000
00d0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3oool04`3oool00`000000
oooo000000060?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`0000001P3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0000000H0oooo0P00000>0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool04`3oool00`000000oooo000000060?ooo`8000003P3oool00`000000
oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0C0?ooo`@000005`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo01<0oooo1000000G0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0C0?ooo`@000005`3oool00`000000oooo0?ooo`010?ooo`00603oool4000000P0oooo00<0
00000?ooo`3oool09`3oool4000000P0oooo00<000000?ooo`3oool09`3oool4000000P0oooo00<0
00000?ooo`3oool09`3oool4000000P0oooo00<000000?ooo`3oool0403oool001P0oooo00D00000
0?ooo`000000oooo000000070?ooo`<000009`3oool01@000000oooo0000003oool0000000L0oooo
0`00000W0?ooo`050000003oool000000?ooo`0000001`3oool3000002L0oooo00D000000?ooo`00
0000oooo000000070?ooo`<00000403oool002@0oooo00<000000?ooo`000000<`3oool00`000000
oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000403oool002@0oooo
00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool0403oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo000@0?ooo`040000003oool0000000000080oooo0`000000203oool000000000
003oool000000?ooo`000000oooo0P0000000`3oool000000000000P0?ooo`040000003oool00000
00000080oooo0`0000020?ooo`050000003oool000000?ooo`000000103oool200000200oooo00@0
00000?ooo`00000000000P3oool3000000070?ooo`00000000000?ooo`000000oooo000000050?oo
o`030000003oool0oooo01h0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`000000
00000080oooo00<000000?ooo`000000103oool200000100oooo000@0?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool0oooo0P0000001@3oool00000000000000000oooo00<00000
00<0oooo0000000000007`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080
oooo00<000000?ooo`0000000P0000020?ooo`050000003oool000000?ooo`0000007`3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P000002
0?ooo`040000003oool0oooo00000200oooo00@000000?ooo`3oool000000P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00<000000P3oool010000000oooo0?ooo`00000@0?ooo`00
403oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool2000000060?ooo`0000000000
0000003oool000000P3oool00`000000oooo0000000O0?ooo`070000003oool0oooo0?ooo`000000
oooo000000040?ooo`030000003oool00000008000000P3oool010000000oooo00000000000P0?oo
o`070000003oool0oooo0?ooo`000000oooo000000050?ooo`@000000P3oool010000000oooo0?oo
o`00000P0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0oooo
00<000000P3oool01@000000oooo0?ooo`3oool0000000l0oooo000?0?ooo`<0000000<0oooo0000
003oool00P3oool200000080oooo00<000000?ooo`000000103oool010000000oooo0?ooo`00000O
0?ooo`<0000000<0oooo0000003oool00P3oool2000000@0oooo00@000000?ooo`3oool0oooo0`00
00000`3oool000000?ooo`0N0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<0
00000?ooo`0000000`3oool300000240oooo0`0000000`3oool000000?ooo`020?ooo`8000000P3o
ool010000000oooo0?ooo`0000030?ooo`<0000000<0oooo0000003oool03`3oool000`0oooo0P00
000=0?ooo`030000003oool0oooo02@0oooo0P00000d0?ooo`800000=@3oool2000002P0oooo000<
0?ooo`<000002`3oool3000002D0oooo0`00000c0?ooo`<00000=03oool3000002L0oooo000=0?oo
o`030000003oool0oooo00L0oooo0P0000020?ooo`030000003oool0oooo02D0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0000000W0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`000000:03oool00`000000oooo0?ooo`080?ooo`030000003oool0000001/0oooo000G0?ooo`80
0000=@3oool400000380oooo1000000b0?ooo`@000006`3oool001L0oooo00<000000?ooo`3oool0
;P3oool00`000000oooo0?ooo`030?ooo`@00000;03oool00`000000oooo0?ooo`030?ooo`@00000
<P3oool4000001/0oooo00020?ooo`030000003oool0oooo00d0oooo0P00000O0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`050000003o
ool000000?ooo`0000005P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool00P3oool01@000000oooo0000003oool0000001H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0=0?ooo`800000103oool01@000000oooo0000003oool00000
01H0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0L0?ooo`030000003oool0oooo0080oooo00040?ooo`8000003@3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo00H0oooo0P00000<0?ooo`<00000703oool00`000000oooo
0?ooo`060?ooo`800000303oool3000001`0oooo00<000000?ooo`3oool01P3oool2000000d0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?oo
o`090?ooo`<000006P3oool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<0
00000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool02@3oool3000001/0oooo00<0
00000?ooo`3oool0103oool000L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`06
0?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool01`3oool000P0oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<0
00007`3oool300000140oooo0`00000O0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000
003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`800000
3@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3o
ool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`@000002`3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0203oool6
000000X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00P0oooo1P00000:0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`080?ooo`L000002@3oool00`000000oooo0?ooo`0<0?ooo`003P3oool20000
00P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
01X0oooo0P0000080?ooo`030000003oool0oooo0080oooo0P0000080?ooo`030000003oool0oooo
01X0oooo0P0000080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`0J0?ooo`800000203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool03@3oool00100oooo00<000000?ooo`3oool00P0000090?ooo`03
0000003oool0oooo00@0oooo0P00000O0?ooo`030000003oool0oooo008000000P3oool2000000D0
oooo00<000000?ooo`3oool0103oool2000001l0oooo00<000000?ooo`3oool00P0000020?ooo`80
00001@3oool00`000000oooo0?ooo`040?ooo`8000007`3oool00`000000oooo0?ooo`02000000<0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00h0
oooo000A0?ooo`L000001P3oool4000000030?ooo`00000000000240oooo200000060?ooo`@00000
00<0oooo0000000000008@3oool4000000040?ooo`0000000000000000H0oooo100000000`3oool0
00000000000Q0?ooo`@0000000<0oooo0000000000000P0000050?ooo`@0000000@0oooo00000000
000000004@3oool00100oooo00<000000?ooo`0000000`3oool2000000d0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`00
00001@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0503oool00`000000oooo000000030?ooo`030000003oool0000000D0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`00
00000`3oool00`000000oooo000000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo0P0000030?ooo`030000
003oool0oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo00<000000?ooo`3oool0
0P3oool2000000L0oooo0P0000070?ooo`<000004P3oool2000000P0oooo00<000000?ooo`3oool0
0P3oool2000000L0oooo0P0000070?ooo`<000004P3oool2000000P0oooo00<000000?ooo`3oool0
0P3oool2000000P0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003o
ool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3o
ool00`000000oooo0?ooo`080?ooo`H000002P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool000000140oooo00<000000?ooo`3oool0203oool6000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0000000A0?ooo`030000003oool0oooo00P0oooo1P00000:0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000:0?ooo`8000007P3oool00`00
0000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`00
0000oooo0?ooo`0C0?ooo`8000007`3oool00`000000oooo0?ooo`090?ooo`002@3oool200000200
oooo0P00000B0?ooo`800000803oool300000140oooo0P00000P0?ooo`<000004@3oool200000240
oooo0P0000090?ooo`00203oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo
0P00000@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02`3oool200000100oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo00X0oooo0P0000090?ooo`001`3oool00`000000oooo0?oo
o`0D0?ooo`<000003@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01@0oooo0P00
000>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0E0?ooo`8000003@3oool00`000000oooo0?oo
o`060?ooo`001P3oool00`000000oooo0?ooo`0B0?ooo`070000003oool0oooo0000003oool00000
0?ooo`02000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`800000
0P3oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`040000
003oool000000?ooo`8000002`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01H0
oooo0P0000000`3oool000000000000<0?ooo`030000003oool0oooo00@0oooo00040?ooo`800000
503oool3000000030?ooo`000000oooo0080oooo0P00000<0?ooo`800000203oool2000001P0oooo
00@000000?ooo`3oool0oooo0P00000<0?ooo`800000203oool2000001P0oooo00@000000?ooo`3o
ool0oooo0P00000<0?ooo`800000203oool2000001T0oooo00<000000?ooo`3oool00P00000=0?oo
o`030000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo0140oooo0P0000020?ooo`03
0000003oool0oooo00@0oooo0P00000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo000000060?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`0000001P3oool2000000h0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0C0?ooo`030000003oool0000000H0oooo0P00000>0?ooo`03
0000003oool0oooo0080oooo00020?ooo`030000003oool0oooo0180oooo0P00000J0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0C0?ooo`@000005`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01<0oooo1000000G0?ooo`030000003oool0oooo0040oooo000G0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool09`3oool4000000P0oooo00<000000?ooo`3oool09`3o
ool4000000P0oooo00<000000?ooo`3oool09`3oool4000000P0oooo00<000000?ooo`3oool0403o
ool002@0oooo0`00000W0?ooo`050000003oool000000?ooo`0000001`3oool3000002L0oooo00D0
00000?ooo`000000oooo000000070?ooo`<000009`3oool01@000000oooo0000003oool0000000L0
oooo0`00000@0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo
00<000000?ooo`000000<`3oool00`000000oooo0000000@0?ooo`00903oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo000@0?ooo`040000003oool0000000000080oooo0`0000040?ooo`040000003oool00000
0?ooo`80000000<0oooo000000000000803oool010000000oooo0000000000020?ooo`<0000000P0
oooo000000000000oooo0000003oool000000?ooo`80000000<0oooo000000000000803oool01000
0000oooo0000000000020?ooo`<0000000P0oooo000000000000oooo0000003oool000000?ooo`80
00008`3oool010000000oooo0000000000020?ooo`<000001@3oool010000000oooo0000003oool2
000000030?ooo`00000000000100oooo000@0?ooo`040000003oool0oooo00000080oooo00@00000
0?ooo`3oool0oooo0P0000001@3oool00000000000000000oooo00<0000000<0oooo000000000000
7`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo1@0000000`3oool0
000000000004000001l0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool6
000000060?ooo`00000000000000003oool00000803oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00`0000000`3oool000000000000300000100
oooo000@0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`80000000H0oooo0000
0000000000000?ooo`0000020?ooo`030000003oool0oooo01l0oooo00L000000?ooo`3oool0oooo
0000003oool0000000<0oooo00<000000?ooo`3oool00`0000001@3oool000000?ooo`3oool00000
0240oooo00L000000?ooo`3oool0oooo0000003oool0000000<0oooo00<000000?ooo`3oool00`00
00000`3oool000000?ooo`020?ooo`030000003oool0oooo01h0oooo00L000000?ooo`3oool0oooo
0000003oool0000000@0oooo00<000000?ooo`3oool00`000000103oool000000?ooo`3oool20000
0100oooo000<0?ooo`80000000H0oooo00000000000000000?ooo`0000030?ooo`8000000P3oool0
0`000000oooo000000040?ooo`040000003oool0oooo000001`0oooo0P0000001P3oool000000000
00000000oooo000000<0oooo0P0000020?ooo`030000003oool0000000@0oooo00@000000?ooo`3o
ool00000703oool2000000060?ooo`00000000000000003oool000000`3oool200000080oooo00<0
00000?ooo`000000103oool00`000000oooo00000002000001/0oooo0P0000001P3oool000000000
00000000oooo000000<0oooo0P0000001P3oool00000000000000000oooo000000@0oooo00@00000
0?ooo`3oool00000403oool000`0oooo0`00000;0?ooo`<000009@3oool3000000/0oooo0P00000V
0?ooo`<000002`3oool3000002D0oooo0`00000<0?ooo`8000006P3oool000d0oooo00<000000?oo
o`3oool01`3oool200000080oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`070?oo
o`800000:P3oool00`000000oooo0?ooo`070?ooo`8000000P3oool00`000000oooo0?ooo`0U0?oo
o`030000003oool0oooo00L0oooo0P00000N0?ooo`005`3oool2000003@0oooo0P00000d0?ooo`80
0000=03oool2000001h0oooo000B0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
7@3oool00080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool03@3oool2000001l0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0=0?ooo`8000007`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00d0oooo0P00000O0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`0000007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`020?ooo`00103oool20000
00`0oooo0`00000L0?ooo`030000003oool0oooo00H0oooo0P00000=0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool01P3oool2000000d0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00<0oooo00060?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo00T0
oooo0`00000J0?ooo`8000002`3oool00`000000oooo0?ooo`090?ooo`<000006P3oool2000000/0
oooo00<000000?ooo`3oool02@3oool3000001/0oooo00<000000?ooo`3oool0103oool000L0oooo
00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0903oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<0
00000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<0
00000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool000P0oooo0`00000O
0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool300000140oooo0`00000O
0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0oooo0P00000=0?oo
o`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?oo
o`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<0
00001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000
00<0oooo0000003oool02@3oool00`000000oooo0?ooo`0<0?ooo`003P3oool2000000P0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P00
00080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0J0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool06P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000003oool0oooo008000002@3oool0
0`000000oooo0?ooo`040?ooo`8000007`3oool00`000000oooo0?ooo`02000000T0oooo00<00000
0?ooo`3oool0103oool2000001l0oooo00D000000?ooo`3oool0oooo000000090?ooo`030000003o
ool0oooo00@0oooo0P00000O0?ooo`050000003oool0oooo0?ooo`0000002@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00h0oooo000A0?ooo`L000001P3oool4000000030?ooo`00
000000000280oooo1`0000060?ooo`@0000000<0oooo0000000000008P3oool7000000H0oooo1000
00000`3oool000000000000R0?ooo`L000001`3oool3000000040?ooo`000000000000000140oooo
000@0?ooo`<000000P3oool3000000d0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool000000080oooo0`00000=0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0503oool00`000000oooo000000020?ooo`@00000303oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool3000000d0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo0P000009
0?ooo`040000003oool0oooo0?ooo`8000001`3oool2000000L0oooo0`00000B0?ooo`8000002@3o
ool010000000oooo0?ooo`3oool2000000L0oooo0P0000070?ooo`<000004P3oool2000000T0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo
0`0000050?ooo`003@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?oo
o`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000
000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?oo
o`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000D0oooo000<
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`05
0?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?oo
o`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo00<000000?oo
o`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?ooo`80
0000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
2P3oool200000100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000
2@3oool000L0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0D0?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0503o
ool3000000d0oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool04P3oool0
1`000000oooo0?ooo`000000oooo0000003oool00P00000;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool05@3oool200000080oooo0P00000;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool04P3oool010000000oooo0?ooo`3oool2000000030?ooo`000000000000/0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`040000003oool000000?oo
o`800000303oool00`000000oooo0?ooo`040?ooo`00103oool2000001@0oooo0`0000000`3oool0
00000?ooo`020?ooo`800000303oool2000000P0oooo0P00000D0?ooo`8000000P3oool010000000
oooo0?ooo`3oool2000000`0oooo0P0000080?ooo`800000503oool300000080oooo00<000000?oo
o`3oool00P00000<0?ooo`800000203oool2000001D0oooo0P0000000`3oool000000?ooo`020?oo
o`8000003@3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0A0?ooo`80
00000P3oool00`000000oooo0?ooo`040?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0140oooo0P0000090?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0140oooo0P0000020?ooo`030000003oool0oooo00@0oooo0P00000>0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool2000000T0oooo0P00000>0?ooo`03
0000003oool0oooo0080oooo00020?ooo`030000003oool0oooo0180oooo0P00000J0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool04P3oool2000001X0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0B0?ooo`8000006P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0180oooo0P00000J0?ooo`030000003oool0oooo0040oooo000G0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0403oool002@0oooo0`00000c0?ooo`<00000<`3oool3
000003<0oooo0`00000@0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003oool00000
03<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000@0?ooo`00903oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo000>0?ooo`040000003oool0000000000080oooo0`000000103oool000000000
003oool200000080oooo00@000000?ooo`000000oooo0P0000020?ooo`030000003oool0oooo01`0
oooo00@000000?ooo`00000000000P3oool300000080oooo00H000000?ooo`000000oooo0000003o
ool2000000030?ooo`00000000000200oooo00@000000?ooo`00000000000P3oool3000000080?oo
o`00000000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200oooo00@00000
0?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo00@000000?ooo`000000oooo
0P00000C0?ooo`003P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo
00@000000?ooo`3oool000000P3oool3000000060?ooo`00000000000000003oool000007P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P00
00001P3oool00000000000000000oooo00000200oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool000000080000000H0oooo00000000000000000?ooo`00
000P0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`03000000030?ooo`000000000000<00000403oool000h0oooo00L000000?ooo`3oool0oooo
0000003oool0000000D0oooo00D000000?ooo`3oool000000?ooo`03000000030?ooo`000000oooo
0080oooo00<000000?ooo`3oool0703oool01`000000oooo0?ooo`3oool000000?ooo`000000103o
ool00`000000oooo00000002000000030?ooo`000000oooo00<0oooo00<000000?ooo`3oool07@3o
ool01`000000oooo0?ooo`3oool000000?ooo`0000001@3oool4000000030?ooo`000000oooo0080
oooo00<000000?ooo`3oool07P3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool0
0`000000oooo0?ooo`03000000040?ooo`000000oooo0?ooo`800000403oool000`0oooo10000000
0`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool06`3oool2000000060?ooo`00000000000000003oool0
00000`3oool2000000@0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00000L0?oo
o`80000000H0oooo00000000000000000?ooo`0000030?ooo`8000000P3oool00`000000oooo0000
00040?ooo`040000003oool0oooo000001d0oooo1@0000000`3oool000000?ooo`020?ooo`800000
0P3oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo0140oooo000<0?ooo`<00000
303oool00`000000oooo0?ooo`0T0?ooo`<000002`3oool2000002H0oooo0`00000;0?ooo`<00000
9P3oool3000000/0oooo0P00000J0?ooo`003@3oool00`000000oooo0?ooo`080?ooo`80000000<0
oooo0000003oool09P3oool00`000000oooo0?ooo`070?ooo`800000:P3oool00`000000oooo0?oo
o`070?ooo`8000000P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00H0oooo0P00
000N0?ooo`00603oool2000000030?ooo`000000oooo0300oooo0P00000d0?ooo`800000=03oool2
000001h0oooo000I0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo01d0oooo00020?ooo`030000003oool0oooo00d0
oooo0P00000O0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool00P3oool2000001P0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00
000=0?ooo`030000003oool0oooo0080oooo0P00000G0?ooo`030000003oool0oooo00H0oooo0P00
000<0?ooo`<00000703oool00`000000oooo0?ooo`060?ooo`800000303oool3000001`0oooo00<0
00000?ooo`3oool01P3oool2000000`0oooo0`00000L0?ooo`030000003oool0oooo00<0oooo0006
0?ooo`030000003oool0oooo00T0oooo0`0000060?ooo`8000004P3oool2000000/0oooo00<00000
0?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`03
0000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`03
0000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<00000
7`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<00000
7`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3o
ool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003o
ool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3
000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool30000
00`0oooo00<000000?ooo`3oool0303oool000h0oooo0P0000090?ooo`030000003oool0oooo0080
00002@3oool00`000000oooo0?ooo`0J0?ooo`800000203oool200000080oooo0P0000090?ooo`03
0000003oool0oooo01X0oooo0P0000080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3o
ool06P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool03@3oool00100oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool01@3oool2000001l0oooo00D000000?ooo`3oool0oooo000000030?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool2000001l0oooo00D000000?oo
o`3oool0oooo000000030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool2
000001l0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0>0?ooo`004@3oool7000000H0oooo10000000
0`3oool000000000000R0?ooo`L000001P3oool4000000030?ooo`00000000000280oooo00<00000
0?ooo`000000100000060?ooo`@0000000<0oooo0000000000008P3oool00`000000oooo00000002
000000030?ooo`000000oooo00D0oooo10000000103oool0000000000000000A0?ooo`00403oool3
00000080oooo0`0000060?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0D0?ooo`<000000P3oool3000000d0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0D0?ooo`<000000P3oool4000000`0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0D0?ooo`<000000P3oool4000000d0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo00<000000?ooo`3oool02`3oool20000
00L0oooo0`00000B0?ooo`8000003@3oool00`000000oooo0?ooo`060?ooo`8000001`3oool30000
0180oooo0P00000=0?ooo`8000001`3oool2000000L0oooo0`00000B0?ooo`8000003@3oool20000
00P0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0oooo00T0
oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000
oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140
oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000
oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00D0oooo000;0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A
0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo00/0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool02P3oool200000100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?oo
o`8000002@3oool000L0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0D0?ooo`8000003P3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0503oool3000000d0oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool0
5@3oool010000000oooo0000003oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0E0?ooo`8000000P3oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0B0?ooo`040000003oool0oooo0?ooo`80000000<0oooo0000000000002`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo01D0oooo00@000000?ooo`000000oooo0P00000<
0?ooo`030000003oool0oooo00@0oooo00040?ooo`8000005@3oool010000000oooo0?ooo`000003
0?ooo`800000303oool2000000P0oooo0P00000D0?ooo`8000000P3oool010000000oooo0?ooo`3o
ool2000000`0oooo0P0000080?ooo`800000503oool300000080oooo00<000000?ooo`3oool00P00
000<0?ooo`800000203oool2000001D0oooo0P0000000`3oool000000?ooo`020?ooo`8000003@3o
ool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0B0?ooo`80000000<0oooo
0000003oool01@3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A
0?ooo`8000002@3oool2000000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A
0?ooo`8000000P3oool00`000000oooo0?ooo`040?ooo`8000003P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0140oooo0P0000090?ooo`8000003P3oool00`000000oooo0?ooo`02
0?ooo`000P3oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo0000003oool05P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo0P00000J0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool04P3oool2000001X0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0B0?ooo`8000006P3oool00`000000oooo0?ooo`010?ooo`006@3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0100oooo000T0?ooo`<00000<`3oool3000003<0oooo
0`00000c0?ooo`<00000403oool001P0oooo0P00000:0?ooo`030000003oool0000003<0oooo00<0
00000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000003oool000000100oooo000H
0?ooo`8000002P3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`006`3oool00`000000oooo0?ooo`2k0?oo
o`006`3oool00`000000oooo0?ooo`2k0?ooo`006`3oool00`000000oooo0?ooo`2k0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0oooo0`00
000;0?ooo`030000003oool0oooo00@0oooo00@000000?ooo`3oool000001`3oool010000000oooo
0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`0000080?ooo`D000001@3oool010000000oooo
0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3000000/0
oooo00<000000?ooo`3oool0103oool4000000L0oooo1000001F0?ooo`00M03oool010000000oooo
0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .5 m
.57273 .5 L
s
gsave
.62727 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N81) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N82) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N83) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N84) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N85) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N86) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N87) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N88) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N89) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N90) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N91) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C11) show
80.562500 14.625000 moveto
(N92) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C12) show
80.562500 14.625000 moveto
(N93) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C13) show
80.562500 14.625000 moveto
(N94) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C14) show
80.562500 14.625000 moveto
(N95) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C15) show
80.562500 14.625000 moveto
(N96) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C16) show
80.562500 14.625000 moveto
(N97) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C17) show
80.562500 14.625000 moveto
(N98) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C18) show
80.562500 14.625000 moveto
(N99) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N100) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000h0oooo00@000000?ooo`00
000000000P3oool300000080oooo00@000000?ooo`00000000000P3oool010000000oooo0000003o
ool2000001l0oooo00@000000?ooo`00000000000P3oool300000080oooo00<000000?ooo`3oool0
0`3oool010000000oooo0000003oool2000000030?ooo`000000000001`0oooo00@000000?ooo`00
000000000P3oool300000080oooo00@000000?ooo`00000000000P3oool010000000oooo0000003o
ool2000000030?ooo`000000000001`0oooo00@000000?ooo`00000000000P3oool300000080oooo
00H000000?ooo`3oool000000?ooo`0000020?ooo`050000003oool000000000003oool00P00000?
0?ooo`003P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`0000000P000000103oool000000000000000020?ooo`80000000<0oooo0000003oool06`3o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool0
00000P3oool300000080oooo1000000L0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool00P3oool010000000oooo0000000000020?ooo`<000000P3oool2000000030?ooo`000000
000001/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00<000000P3oool010000000oooo0000003oool2000000030?ooo`000000oooo00d0oooo
000>0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0000000<0
oooo0`0000001P3oool000000?ooo`000000oooo000001`0oooo00L000000?ooo`3oool0oooo0000
003oool0000000@0oooo00@000000?ooo`3oool000000P3oool3000000030?ooo`000000oooo00<0
0000703oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool010000000oooo00000000
00020?ooo`<0000000<0oooo0000003oool00P0000000`3oool000000?ooo`0J0?ooo`070000003o
ool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0oooo00<000000P3oool01@000000
oooo000000000000oooo008000003`3oool000d0oooo0`0000000`3oool000000?ooo`020?ooo`80
00001@3oool010000000oooo0?ooo`0000040?ooo`030000003oool00000008000006P3oool30000
00030?ooo`000000oooo0080oooo0P0000040?ooo`<0000000<0oooo0000003oool00`3oool01000
0000oooo0?ooo`00000K0?ooo`<0000000<0oooo0000003oool00P3oool2000000D0oooo00@00000
0?ooo`3oool00000103oool010000000oooo0?ooo`00000K0?ooo`<0000000<0oooo0000003oool0
0P3oool2000000D0oooo00<000000?ooo`3oool01@3oool010000000oooo0?ooo`00000?0?ooo`00
303oool2000003@0oooo0P00000d0?ooo`8000003P3oool00`000000oooo0?ooo`0T0?ooo`800000
:03oool000`0oooo0`00000<0?ooo`030000003oool0oooo02@0oooo0`0000090?ooo`040000003o
ool00000000002H0oooo0`0000090?ooo`050000003oool000000?ooo`0000009P3oool3000002L0
oooo000=0?ooo`030000003oool0oooo00P0oooo0P0000000`3oool000000?ooo`0V0?ooo`030000
003oool0oooo00T0oooo0`00000W0?ooo`030000003oool0oooo00T0oooo0P0000000`3oool00000
0?ooo`0V0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0703oool001P0oooo0P00
00000`3oool000000?ooo`0c0?ooo`800000=03oool200000380oooo0`00000L0?ooo`000P3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0D0?ooo`800000603oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo01@0oooo0P00000H0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0000001P0oooo00<000000?ooo`3oool00@3oool0
00<0oooo00<000000?ooo`3oool0303oool2000001h0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0<0?ooo`800000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0<0?ooo`800000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`020?ooo`00103oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0080
oooo0P00000H0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`020?ooo`800000603oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`0000007P3oool00`000000oooo0?ooo`020?ooo`001@3oool2000000`0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`070?ooo`800000303oool00`000000oooo
0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`070?ooo`800000303oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo00L0oooo0P00000;0?ooo`<00000703oool00`000000oooo
0?ooo`030?ooo`001`3oool00`000000oooo0?ooo`080?ooo`<000006P3oool2000000`0oooo00<0
00000?ooo`3oool0203oool3000000H0oooo0P00000B0?ooo`800000303oool00`000000oooo0?oo
o`080?ooo`<000006P3oool2000000`0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`040?ooo`00203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0403oool200000140oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`06
0?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?oo
o`3oool01`3oool000T0oooo0P00000O0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<0
00004`3oool00`000000oooo0?ooo`0M0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<0
00002P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`800000
1P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0
703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
2@3oool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3o
ool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool01`3oool2
000000X0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2
000000/0oooo0P00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2
000000X0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool2
000000X0oooo1000000;0?ooo`030000003oool0oooo00`0oooo000?0?ooo`030000003oool0oooo
00H0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00H0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00H0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00d0oooo000@0?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`030?ooo`04
0000003oool0000000000080oooo1000000O0?ooo`030000003oool0oooo008000000P3oool00`00
0000oooo0?ooo`040?ooo`<000000P3oool4000001l0oooo00<000000?ooo`3oool00P0000020?oo
o`030000003oool0oooo00@0oooo0`0000020?ooo`@000007`3oool00`000000oooo0?ooo`020000
0080oooo00<000000?ooo`3oool0103oool300000080oooo0P0000000`3oool000000?ooo`0?0?oo
o`00403oool8000000H0oooo1`00000:0?ooo`030000003oool0oooo01@0oooo200000060?ooo`L0
00002P3oool00`000000oooo0?ooo`0D0?ooo`@0000000@0oooo00000000000000001P3oool70000
00X0oooo00<000000?ooo`3oool05@3oool3000000040?ooo`0000000000000000L0oooo1`000009
0?ooo`030000003oool0oooo00D0oooo000@0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo0`00000D0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00L0oooo0`00000D0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo0`00000D0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00H0oooo0`0000050?ooo`003P3oool2000000P0oooo0P0000030?ooo`03
0000003oool0oooo00H0oooo0P0000070?ooo`030000003oool000000180oooo0P0000080?ooo`80
00000`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool00`000000oooo0000000B0?ooo`80
0000203oool2000000<0oooo00<000000?ooo`3oool01P3oool2000000L0oooo00<000000?ooo`00
00004P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo000000050?ooo`003@3oool00`000000oooo0?ooo`090?ooo`@00000
2`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
2P3oool3000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3o
ool000X0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000N0?ooo`030000003oool0
oooo01<0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000O0?ooo`030000003oool0
oooo00T0oooo00090?ooo`800000803oool200000180oooo0P00000P0?ooo`<000004@3oool20000
0200oooo0`00000A0?ooo`8000005@3oool00`000000oooo0?ooo`090?ooo`8000002@3oool000P0
oooo00<000000?ooo`3oool0503oool2000000/0oooo0P00000@0?ooo`030000003oool0oooo0100
oooo00@000000?ooo`3oool0oooo0P00000;0?ooo`800000403oool00`000000oooo0?ooo`0E0?oo
o`8000002P3oool200000100oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`080?oo
o`8000002@3oool000L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`000000303oool00`000000oooo0?ooo`060?ooo`00
1P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0?0?ooo`040000003oool00000000000@0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool05`3oool3000000d0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000E0?ooo`040000
003oool0oooo0?ooo`<000003P3oool2000000P0oooo0P00000C0?ooo`<000000`3oool3000000h0
oooo0P0000080?ooo`8000004P3oool01@000000oooo0000003oool0000000<0oooo0`00000=0?oo
o`800000203oool2000002/0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3o
ool04P3oool2000000030?ooo`000000oooo01D0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0D0?ooo`8000005P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0
oooo0P0000000`3oool000000?ooo`0D0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0080oooo00020?ooo`030000
003oool0oooo01<0oooo0P0000000`3oool000000?ooo`0F0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0503oool2000001P0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0E0?ooo`8000005`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo
0`00000H0?ooo`030000003oool0oooo0040oooo000I0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool09`3oool2000000X0oooo00<000000?ooo`3oool0:03oool2000000T0oooo00<0
00000?ooo`3oool09`3oool00`000000oooo000000090?ooo`030000003oool0oooo0100oooo000T
0?ooo`<000009`3oool2000000X0oooo0`00000W0?ooo`8000002P3oool3000002L0oooo00<00000
0?ooo`3oool02@3oool300000100oooo000T0?ooo`030000003oool0000002X0oooo0P0000070?oo
o`030000003oool0000002L0oooo0P00000:0?ooo`030000003oool0000003<0oooo00<000000?oo
o`000000403oool002@0oooo00<000000?ooo`3oool0:P3oool2000000L0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo001A0?ooo`800000
QP3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000h0oooo00@0
00000?ooo`00000000000P3oool300000080oooo00@000000?ooo`00000000000P3oool010000000
oooo0000003oool2000000030?ooo`000000000001`0oooo00@000000?ooo`00000000000P3oool3
00000080oooo00@000000?ooo`00000000000P3oool010000000oooo0000003oool2000001l0oooo
00@000000?ooo`00000000000P3oool300000080oooo00<000000?ooo`3oool00`3oool010000000
oooo0000003oool2000000030?ooo`000000000001`0oooo00@000000?ooo`00000000000P3oool3
00000080oooo00@000000?ooo`00000000000P3oool010000000oooo0000003oool2000000030?oo
o`000000000000l0oooo000>0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
0P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool2000000030?ooo`000000oooo01/0
oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo
00000080oooo0`0000020?ooo`@00000703oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`00000000000P3oool300000080oooo0P0000000`3oool00000
0000000K0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`02000000040?ooo`000000000000000080oooo1@00000>0?ooo`003P3oool01`000000
oooo0?ooo`3oool000000?ooo`000000103oool01P000000oooo0?ooo`3oool000000?ooo`<00000
00H0oooo0000003oool000000?ooo`00000L0?ooo`070000003oool0oooo0?ooo`000000oooo0000
00040?ooo`040000003oool0oooo00000080oooo0`0000000`3oool000000?ooo`03000001`0oooo
00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00@000000?ooo`00000000000P3oool3
000000030?ooo`000000oooo008000007@3oool01`000000oooo0?ooo`3oool000000?ooo`000000
103oool00`000000oooo000000030?ooo`<0000000<0oooo0000003oool00P00000@0?ooo`003@3o
ool3000000030?ooo`000000oooo0080oooo0P0000050?ooo`040000003oool0oooo000000@0oooo
00@000000?ooo`3oool000006`3oool3000000030?ooo`000000oooo0080oooo0P0000050?ooo`04
0000003oool0oooo000000@0oooo00<000000?ooo`3oool0703oool3000000030?ooo`000000oooo
0080oooo0P0000080?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000006`3oool3
000000030?ooo`000000oooo0080oooo0P0000050?ooo`040000003oool0oooo000000@0oooo00@0
00000?ooo`3oool000003`3oool000`0oooo0P00000>0?ooo`030000003oool0oooo02<0oooo0P00
000d0?ooo`800000=@3oool2000002P0oooo000<0?ooo`<000002@3oool01@000000oooo0000003o
ool0000002D0oooo0`00000<0?ooo`030000003oool0oooo02@0oooo0`00000<0?ooo`030000003o
ool0oooo02D0oooo0`00000<0?ooo`030000003oool0oooo01P0oooo000=0?ooo`030000003oool0
oooo00T0oooo0P0000000`3oool000000?ooo`0U0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`0000009`3oool00`000000oooo0?ooo`080?ooo`80000000<0oooo0000003oool09`3oool0
0`000000oooo0?ooo`070?ooo`8000000P3oool00`000000oooo0?ooo`0H0?ooo`006P3oool20000
03<0oooo00<000000?ooo`000000<P3oool400000380oooo0P0000020?ooo`030000003oool0oooo
01P0oooo000I0?ooo`800000<`3oool3000003@0oooo00<000000?ooo`3oool0<P3oool00`000000
oooo0?ooo`0L0?ooo`000P3oool00`000000oooo0?ooo`0=0?ooo`800000103oool2000001T0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`8000007`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00d0oooo0P00000O0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool03@3oool2000001l0oooo00<000000?ooo`3oool00@3oool000<0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`020?ooo`800000603oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`020?ooo`00103oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`06
0?ooo`8000003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00H0oooo0P00000=
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01P3oool2000000d0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`090?oo
o`<000006P3oool2000000/0oooo00<000000?ooo`3oool02@3oool3000001X0oooo0P00000;0?oo
o`030000003oool0oooo00T0oooo0`00000J0?ooo`8000002`3oool00`000000oooo0?ooo`090?oo
o`<000006`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`800000
8`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`800000
8`3oool00`000000oooo0?ooo`070?ooo`00203oool3000001l0oooo0`00000A0?ooo`<000007`3o
ool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000:0?ooo`000P3oool2
000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2
000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool30000
00L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00T0
oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`090?ooo`<00000303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo00`0
oooo000>0?ooo`800000203oool200000080oooo0P0000090?ooo`030000003oool0oooo01X0oooo
0P0000080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3oool06P3oool2000000P0oooo
0P0000020?ooo`8000002@3oool00`000000oooo0?ooo`0J0?ooo`800000203oool200000080oooo
0P0000090?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000003oool0oooo008000000P3o
ool00`000000oooo0?ooo`030?ooo`040000003oool0000000000080oooo1000000O0?ooo`030000
003oool0oooo008000000P3oool00`000000oooo0?ooo`040?ooo`<000000P3oool4000001l0oooo
00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo0`0000020?ooo`@00000
7`3oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool0103oool300000080oooo
0P0000000`3oool000000?ooo`0?0?ooo`00403oool8000000H0oooo1`00000Q0?ooo`P000001P3o
ool700000240oooo10000000103oool000000000000000060?ooo`L000008@3oool4000000040?oo
o`0000000000000000H0oooo2000000A0?ooo`00403oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00D0oooo000>0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01P3o
ool2000000L0oooo0`00000B0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01P3o
ool2000000L0oooo0`00000B0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01P3o
ool2000000L0oooo0`00000B0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`050?ooo`<000001@3oool000d0oooo00<000000?ooo`3oool02@3oool4
000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0
oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool0
0`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool0
00000140oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool0
0`000000oooo000000050?ooo`00303oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool01@3oool000X0oooo0P00000N0?ooo`030000003oool0oooo01<0
oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000N0?ooo`030000003oool0oooo01<0
oooo0P00000O0?ooo`030000003oool0oooo00T0oooo00090?ooo`800000803oool200000180oooo
0P00000P0?ooo`<000004@3oool200000200oooo0`00000A0?ooo`8000008@3oool2000000T0oooo
00080?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02P3oool200000100oooo00<0
00000?ooo`3oool08@3oool200000100oooo00<000000?ooo`3oool08@3oool200000100oooo00<0
00000?ooo`3oool08@3oool2000000T0oooo00070?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01D0oooo0P00000=
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool2000000`0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`800000303oool00`000000oooo0?ooo`06
0?ooo`001P3oool00`000000oooo0?ooo`0C0?ooo`050000003oool000000?ooo`0000003P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`040?ooo`00103oool200000180oooo00D000000?ooo`000000oooo00000002
0?ooo`<000003P3oool2000000P0oooo0P00000E0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool03@3oool2000000P0oooo0P00000E0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0303oool2000000P0oooo0P00000F0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0C0?oo
o`80000000<0oooo0000003oool0503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`0000000`3oool300000100oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool00`3oool3000000l0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0B0?ooo`8000000P3oool010000000oooo0?ooo`3oool3
000000l0oooo00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool05@3oool20000
01L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool00000
01L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`@000005`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0P0000020?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool00@3oool001T0oooo0P0000090?ooo`030000003oool0oooo02L0
oooo0`0000090?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`0W0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0403oool001P0oooo
0P00000:0?ooo`<00000<`3oool3000003<0oooo0`00000c0?ooo`<00000403oool001P0oooo0P00
000:0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000
000c0?ooo`030000003oool000000100oooo000T0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00403oool010000000
oooo0000000000020?ooo`<0000000P0oooo000000000000oooo0000003oool000000?ooo`800000
00<0oooo000000000000803oool010000000oooo0000000000020?ooo`<0000000P0oooo00000000
0000oooo0000003oool000000?ooo`80000000<0oooo0000000000007P3oool010000000oooo0000
000000020?ooo`<000000P3oool010000000oooo0000000000020?ooo`040000003oool000000?oo
o`8000000P3oool00`000000oooo0?ooo`0J0?ooo`040000003oool0000000000080oooo0`000002
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`000000oooo0P0000000`3oool00000
0000000?0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80
000000D0oooo00000000000000000?ooo`0200000080oooo0P00000O0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool00P3oool500000080oooo0`0000000`3oool000000?ooo`0L
0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool01P000000oooo0000
003oool000000?ooo`<000000P3oool2000000030?ooo`000000oooo01/0oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo0`000002
0?ooo`80000000<0oooo0000003oool03P3oool00100oooo00L000000?ooo`3oool0oooo0000003o
ool0000000<0oooo0P0000001@3oool00000000000000000oooo0080000000@0oooo0000003oool0
00007`3oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool00`000000oooo00000003
000000030?ooo`000000oooo00<000007P3oool01`000000oooo0?ooo`3oool000000?ooo`000000
103oool010000000oooo0000000000020?ooo`<0000000H0oooo0000003oool000000?ooo`00000L
0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`040000003oool0oooo00000080
oooo0`000000103oool000000?ooo`0000020?ooo`030000003oool0oooo00`0oooo000?0?ooo`<0
000000<0oooo0000003oool00P3oool200000080oooo00<000000?ooo`000000103oool010000000
oooo0?ooo`00000O0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<000000?oo
o`000000103oool010000000oooo0?ooo`00000M0?ooo`<0000000<0oooo0000003oool00P3oool2
000000D0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0L0?ooo`<0000000<0
oooo0000003oool00P3oool2000000P0oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`00000?0?ooo`00303oool2000000h0oooo00<000000?ooo`3oool08`3oool2000003@0oooo0P00
000e0?ooo`800000:03oool000`0oooo0`0000090?ooo`050000003oool000000?ooo`0000009@3o
ool3000000`0oooo00<000000?ooo`3oool0903oool3000000`0oooo00<000000?ooo`3oool09@3o
ool3000000`0oooo00<000000?ooo`3oool0603oool000d0oooo00<000000?ooo`3oool02@3oool2
000000030?ooo`000000oooo02D0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000
oooo02H0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02L0oooo00<00000
0?ooo`3oool01`3oool200000080oooo00<000000?ooo`3oool0603oool001X0oooo0P00000b0?oo
o`@00000<P3oool2000000030?ooo`000000oooo0340oooo0P0000020?ooo`030000003oool0oooo
01P0oooo000B0?ooo`030000003oool0oooo00@0oooo0P00000d0?ooo`030000003oool0oooo0380
oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0K0?ooo`000P3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo0080oooo0P00000I0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool03@3oool2000001l0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0=0?ooo`8000007`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0oooo
0P00000O0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`000000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000<
0?ooo`<00000703oool00`000000oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo00H0oooo0P00000=0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool01P3oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`03
0?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`090?oo
o`<000006P3oool2000000/0oooo00<000000?ooo`3oool02@3oool3000001X0oooo0P00000;0?oo
o`030000003oool0oooo00T0oooo0`00000K0?ooo`030000003oool0oooo00@0oooo00070?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`030000
003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000
003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000007`3o
ool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3o
ool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2
000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool30000
00H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool3000000`0
oooo00<000000?ooo`3oool0303oool000h0oooo0P0000080?ooo`8000000P3oool2000000T0oooo
00<000000?ooo`3oool06P3oool2000000P0oooo0P0000020?ooo`8000002@3oool00`000000oooo
0?ooo`0J0?ooo`800000203oool200000080oooo0P0000090?ooo`030000003oool0oooo01X0oooo
0P0000080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3oool03@3oool00100oooo00<0
00000?ooo`3oool00P0000020?ooo`030000003oool0oooo00<0oooo00@000000?ooo`0000000000
0P3oool4000001l0oooo00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo
0`0000020?ooo`@000007`3oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool0
103oool300000080oooo1000000O0?ooo`030000003oool0oooo008000000P3oool00`000000oooo
0?ooo`040?ooo`<000000P3oool2000000030?ooo`000000oooo00l0oooo000@0?ooo`P000001P3o
ool700000240oooo200000060?ooo`L000008@3oool4000000040?ooo`0000000000000000H0oooo
1`00000Q0?ooo`@0000000@0oooo00000000000000001P3oool800000140oooo000@0?ooo`030000
003oool0000000<0oooo0P0000050?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool2000000H0
oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`0000000`3oool2000000H0oooo00D000000?ooo`3oool0oooo00000002
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`000000oooo00000003
0?ooo`8000001P3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo0P0000030?ooo`030000003oool0
oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0
oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0
oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0oooo
00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`00
0000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool00000
0140oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000:0?ooo`8000007P3oool00`000000oooo
0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo
0?ooo`0C0?ooo`8000007`3oool00`000000oooo0?ooo`090?ooo`002@3oool200000200oooo0P00
000B0?ooo`800000803oool300000140oooo0P00000P0?ooo`<000004@3oool200000240oooo0P00
00090?ooo`00203oool00`000000oooo0?ooo`0Q0?ooo`800000403oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool02@3oool200000100oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`09
0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool05@3oool2000000d0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool01P3oool000H0oooo00<000000?ooo`3oool04`3oool01@000000oooo0?ooo`3oool00000
00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool00000
00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool00000
00d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool00000
00h0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000B0?ooo`050000003oool000000?oo
o`0000000`3oool00`000000oooo0?ooo`0=0?ooo`800000203oool2000001D0oooo00@000000?oo
o`3oool0oooo0`00000>0?ooo`800000203oool2000001D0oooo00<000000?ooo`3oool00P3oool3
000000d0oooo0P0000080?ooo`8000005P3oool010000000oooo0?ooo`3oool3000000h0oooo00<0
00000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3oool04`3oool2000000040?ooo`000000
oooo0?ooo`<00000403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo0P00
00000`3oool000000?ooo`0E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3o
ool2000000030?ooo`000000oooo01D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0B0?ooo`8000000P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0080oooo0002
0?ooo`030000003oool0oooo01D0oooo0P00000G0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C
0?ooo`80000000<0oooo0000003oool05P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo01<0oooo0P0000020?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool00@3oool0
01T0oooo0P0000090?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`0W0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0:03oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0100oooo000H0?ooo`8000002P3oool3000003<0
oooo0`00000c0?ooo`<00000<`3oool300000100oooo000H0?ooo`8000002P3oool00`000000oooo
0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo
0000000@0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000@0?ooo`040000003oool000000000
0080oooo0`0000040?ooo`040000003oool000000?ooo`80000000<0oooo000000000000803oool0
10000000oooo0000000000020?ooo`<0000000P0oooo000000000000oooo0000003oool000000?oo
o`80000000<0oooo000000000000803oool010000000oooo0000000000020?ooo`<0000000P0oooo
000000000000oooo0000003oool000000?ooo`8000008`3oool010000000oooo0000000000020?oo
o`<000001@3oool010000000oooo0000003oool2000000030?ooo`00000000000100oooo000@0?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000001@3oool000000000
00000000oooo008000000P3oool2000001l0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`020?ooo`D0000000@0oooo000000000000oooo0`00000O0?ooo`040000003oool0oooo
00000080oooo00@000000?ooo`3oool0oooo1P0000000`3oool00000000000020?ooo`030000003o
ool0oooo01h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00<0000000@0oooo000000000000oooo0P00000@0?ooo`00403oool01`000000oooo
0?ooo`3oool000000?ooo`0000000`3oool2000000050?ooo`00000000000000003oool00P000000
0`3oool000000?ooo`0P0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`030000
003oool0oooo00<0000000D0oooo000000000000oooo0000000Q0?ooo`070000003oool0oooo0?oo
o`000000oooo000000030?ooo`030000003oool0oooo00<0000000<0oooo0000000000000P3oool0
0`000000oooo0?ooo`0N0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000
003oool0oooo00<0000000@0oooo000000000000oooo0P00000@0?ooo`00303oool2000000060?oo
o`00000000000000003oool000000`3oool2000000<0oooo0P0000040?ooo`040000003oool0oooo
000001`0oooo0P0000001P3oool00000000000000000oooo000000<0oooo0P0000020?ooo`030000
003oool0000000@0oooo00@000000?ooo`3oool00000703oool2000000060?ooo`00000000000000
003oool000000`3oool200000080oooo00<000000?ooo`000000103oool00`000000oooo00000002
000001/0oooo0P0000001P3oool00000000000000000oooo000000<0oooo0P0000001P3oool00000
000000000000oooo000000@0oooo00@000000?ooo`3oool00000403oool000`0oooo0`0000090?oo
o`050000003oool000000?ooo`0000009@3oool3000000`0oooo00<000000?ooo`3oool0903oool3
000000`0oooo00<000000?ooo`3oool0903oool3000000d0oooo00<000000?ooo`3oool0603oool0
00d0oooo00<000000?ooo`3oool02@3oool2000000030?ooo`000000oooo02D0oooo00<000000?oo
o`3oool0203oool2000000030?ooo`000000oooo02H0oooo00<000000?ooo`3oool0203oool20000
00030?ooo`000000oooo02H0oooo00<000000?ooo`3oool0203oool200000080oooo00<000000?oo
o`3oool0603oool001X0oooo0P00000b0?ooo`@00000<P3oool2000000030?ooo`000000oooo0340
oooo0P0000020?ooo`030000003oool0oooo01P0oooo000B0?ooo`030000003oool0oooo00@0oooo
0P00000]0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0K0?ooo`000P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0080oooo0P00
000I0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`000000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000<0?oo
o`<00000703oool00`000000oooo0?ooo`060?ooo`800000303oool3000001`0oooo00<000000?oo
o`3oool01P3oool2000000`0oooo0`00000L0?ooo`030000003oool0oooo00H0oooo0P00000<0?oo
o`<00000703oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`80
00002`3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`80
00002`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00@0oooo00070?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`030000003o
ool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003o
ool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000007`3oool3
00000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool3
000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool20000
00H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo
01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0
oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool3000000`0oooo
00<000000?ooo`3oool0303oool000h0oooo0P0000080?ooo`8000000P3oool2000000T0oooo00<0
00000?ooo`3oool06P3oool2000000P0oooo0P0000020?ooo`8000002@3oool00`000000oooo0?oo
o`0J0?ooo`800000203oool200000080oooo0P0000090?ooo`030000003oool0oooo01X0oooo0P00
00080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3oool03@3oool00100oooo00<00000
0?ooo`3oool00P0000020?ooo`030000003oool0oooo00<0oooo00@000000?ooo`00000000000P3o
ool4000001l0oooo00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo0`00
00020?ooo`@000007`3oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
103oool300000080oooo1000000O0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000
oooo0?ooo`040?ooo`<000000P3oool2000000030?ooo`000000oooo00l0oooo000A0?ooo`L00000
1P3oool700000280oooo1`0000070?ooo`H000008P3oool7000000L0oooo1P00000R0?ooo`L00000
1`3oool700000140oooo000@0?ooo`<000000P3oool3000000D0oooo0P0000020?ooo`80000000<0
oooo0000000000002@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool000000080oooo0`00
00060?ooo`030000003oool0oooo0080000000<0oooo0000000000002@3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool000000080oooo0`0000060?ooo`030000003oool0oooo0080000000<0
oooo0000000000002@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P00
00060?ooo`030000003oool0oooo0080000000@0oooo0000003oool00000203oool00`000000oooo
0?ooo`050?ooo`003P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P00
00070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P00
00070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P00
00070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0oooo00T0oooo1000000;
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`09
0?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<00000
0?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A
0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00D0oooo000:0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`80
00007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`80
00007`3oool00`000000oooo0?ooo`090?ooo`002@3oool200000200oooo0P00000B0?ooo`800000
803oool300000140oooo0P00000P0?ooo`<000004@3oool200000240oooo0P0000090?ooo`00203o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000003o
ool0oooo0240oooo0P00000@0?ooo`030000003oool0oooo0240oooo0P00000@0?ooo`030000003o
ool0oooo0240oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0E0?ooo`8000003@3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo0P00000<0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool05P3oool2000000`0oooo00<000000?ooo`3oool01P3oool0
00H0oooo00<000000?ooo`3oool04`3oool01@000000oooo0000003oool0000000h0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0103oool000@0oooo0P00000B0?ooo`050000003oool000000?ooo`0000000P3oool3
000000h0oooo0P0000080?ooo`8000005@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00d0oooo0P0000080?ooo`8000005@3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00`0oooo0P0000080?ooo`8000005P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3oool04`3oool20000
00030?ooo`000000oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?oo
o`80000000<0oooo0000003oool00P3oool300000100oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool00`3oool3000000l0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0B0?ooo`8000000P3oool010000000oooo0?ooo`3oool3
000000l0oooo00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool05@3oool20000
01L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`@000005`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0P0000000`3oool000000?ooo`0F0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool200000080oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`010?ooo`006@3oool2000000T0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool0403oool001P0oooo0P00000:0?ooo`<00000<`3oool3000003<0oooo0`00000c0?ooo`<00000
403oool001P0oooo0P00000:0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3o
ool00`000000oooo0000000c0?ooo`030000003oool000000100oooo000T0?ooo`030000003oool0
oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo0100oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00100oooo00@000000?ooo`00000000000P3oool3000000080?ooo`00000000000?oo
o`000000oooo0000003oool200000080oooo00<000000?ooo`3oool07P3oool010000000oooo0000
000000020?ooo`<000000P3oool01P000000oooo0000003oool000000?ooo`80000000<0oooo0000
00000000803oool010000000oooo0000000000020?ooo`<0000000P0oooo000000000000oooo0000
003oool000000?ooo`80000000<0oooo000000000000803oool010000000oooo0000000000020?oo
o`<0000000<0oooo0000000000000P3oool010000000oooo0000003oool2000001<0oooo000@0?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000001@3oool000000000
00000000oooo008000000P3oool00`000000oooo0?ooo`0N0?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002000000030?ooo`00000000000080
oooo00<000000?ooo`3oool07P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`0000000P0000000`3oool00000000000020?ooo`030000003oool0oooo
01h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<0000000@0oooo000000000000oooo0P00000@0?ooo`00403oool01`000000oooo0?ooo`3o
ool000000?ooo`0000000`3oool2000000050?ooo`00000000000000003oool00P0000020?ooo`03
0000003oool0oooo01h0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<00000
0?ooo`0000000P0000000`3oool00000000000030?ooo`030000003oool0oooo01d0oooo00L00000
0?ooo`3oool0oooo0000003oool0000000D0oooo100000000`3oool00000000000020?ooo`030000
003oool0oooo01h0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?oo
o`3oool00`000000103oool000000000003oool200000100oooo000<0?ooo`80000000H0oooo0000
0000000000000?ooo`0000030?ooo`8000000P3oool00`000000oooo000000040?ooo`030000003o
ool0oooo01d0oooo0P0000001P3oool00000000000000000oooo000000<0oooo0P0000040?ooo`03
0000003oool0oooo0080oooo00@000000?ooo`3oool00000703oool2000000060?ooo`0000000000
0000003oool000000`3oool200000080oooo00<000000?ooo`000000103oool010000000oooo0?oo
o`00000M0?ooo`D0000000<0oooo0000003oool00P3oool200000080oooo00@000000?ooo`3oool0
0000103oool00`000000oooo0?ooo`0A0?ooo`00303oool3000003<0oooo0`00000<0?ooo`030000
003oool0oooo02@0oooo0`00000<0?ooo`030000003oool0oooo02D0oooo0`00000<0?ooo`030000
003oool0oooo01P0oooo000=0?ooo`030000003oool0oooo00P0oooo1000000W0?ooo`030000003o
ool0oooo00P0oooo0P0000000`3oool000000?ooo`0V0?ooo`030000003oool0oooo00P0oooo0P00
00000`3oool000000?ooo`0W0?ooo`030000003oool0oooo00L0oooo0P0000020?ooo`030000003o
ool0oooo01P0oooo000H0?ooo`040000003oool0000000000380oooo1000000b0?ooo`80000000<0
oooo0000003oool0<@3oool200000080oooo00<000000?ooo`3oool0603oool001P0oooo00<00000
0?ooo`000000;@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool06`3oool00080oooo00<000000?ooo`3oool03@3oool2000000D0oooo0P00000H0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo00<000000?ooo`00
00007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo00<000000?ooo`00
00007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo00<000000?ooo`00
00007P3oool00`000000oooo0?ooo`020?ooo`00103oool2000000d0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`060?ooo`800000303oool3000001`0oooo00<000000?ooo`3oool0
1P3oool2000000`0oooo0`00000L0?ooo`030000003oool0oooo00H0oooo0P00000<0?ooo`<00000
703oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`090?ooo`<000006P3o
ool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3o
ool2000000/0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`040?ooo`001`3oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`00
0000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`00
0000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`00203oool3000001l0
oooo0`00000A0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0
oooo0`00000:0?ooo`000P3oool2000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo
0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo
0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`090?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`00
00060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00000<
0?ooo`030000003oool0oooo00`0oooo000>0?ooo`8000002@3oool00`000000oooo0?ooo`020000
00T0oooo00<000000?ooo`3oool06P3oool2000000P0oooo0P0000020?ooo`8000002@3oool00`00
0000oooo0?ooo`0J0?ooo`800000203oool200000080oooo0P0000090?ooo`030000003oool0oooo
01X0oooo0P0000080?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3oool03@3oool00100
oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool01@3oool2000001l0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`0000000P3oool010000000oooo00000000000O0?ooo`050000003o
ool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000<0oooo
0`00000O0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0000000<0oooo00<000000?ooo`000000403oool00140oooo1`0000060?ooo`@00000
00<0oooo0000000000008P3oool7000000L0oooo1P00000R0?ooo`030000003oool0000000@00000
1`3oool600000280oooo00<000000?ooo`0000000P0000000`3oool000000?ooo`060?ooo`L00000
4@3oool00100oooo0`0000020?ooo`@00000103oool2000000D0oooo0P0000090?ooo`030000003o
ool0oooo01@0oooo0`0000020?ooo`<000001P3oool00`000000oooo0?ooo`02000000030?ooo`00
0000000000T0oooo00<000000?ooo`3oool0503oool300000080oooo0`0000060?ooo`030000003o
ool0oooo0080000000<0oooo0000000000002@3oool00`000000oooo0?ooo`0D0?ooo`<000000P3o
ool3000000H0oooo00<000000?ooo`3oool00P000000103oool000000?ooo`0000080?ooo`030000
003oool0oooo00D0oooo000>0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003o
ool0oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003o
ool0oooo00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo0P0000030?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0
oooo00P0oooo1P00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool0
0`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool0
00000140oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000;0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo
0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`800000403oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool02@3oool200000100oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`090?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool0503oool3000000d0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool05@3oool010000000oooo0000003oool2
000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0
000000h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
000000d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
000000h0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000H0?ooo`040000003oool0oooo
0?ooo`800000303oool2000000P0oooo0P00000E0?ooo`040000003oool0oooo0?ooo`<000003P3o
ool2000000P0oooo0P00000E0?ooo`030000003oool0oooo0080oooo0`00000=0?ooo`800000203o
ool2000001H0oooo00@000000?ooo`3oool0oooo0`00000>0?ooo`030000003oool0oooo00<0oooo
00030?ooo`030000003oool0oooo01<0oooo00<000000?ooo`0000001P3oool2000000h0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool05@3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo0P0000000`3oool000000?oo
o`0E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool200000080oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0C
0?ooo`@000005`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo1000000G
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool2000000030?ooo`000000
oooo01H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`8000000P3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0040oooo000H0?ooo`@00000203oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`0@0?ooo`00603oool01@000000oooo0000003oool0000000L0oooo0`00000c0?oo
o`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo0000000c0?ooo`03
0000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000@0?ooo`00
903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`03
0000003oool0oooo00@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F
0?ooo`00E03oool3000000P0oooo0`0000080?ooo`D000001@3oool010000000oooo0?ooo`000007
0?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3000000/0oooo00<00000
0?ooo`3oool0103oool4000000L0oooo1000001F0?ooo`00M03oool010000000oooo0?ooo`000007
0?ooo`040000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N101) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N102) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N103) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N104) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N105) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N106) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N107) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N108) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N109) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C11) show
80.562500 14.625000 moveto
(N110) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C12) show
80.562500 14.625000 moveto
(N111) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C13) show
80.562500 14.625000 moveto
(N112) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C14) show
80.562500 14.625000 moveto
(N113) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C15) show
80.562500 14.625000 moveto
(N114) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C16) show
80.562500 14.625000 moveto
(N115) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C17) show
80.562500 14.625000 moveto
(N116) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -67 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C18) show
80.562500 14.625000 moveto
(N117) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
gsave
.5 .58978 -67 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C19) show
80.562500 14.625000 moveto
(N118) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C20) show
80.562500 14.625000 moveto
(N119) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C21) show
80.562500 14.625000 moveto
(N120) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000d0oooo00@000000?ooo`00
000000000P3oool300000080oooo00P000000?ooo`00000000000?ooo`000000oooo00000080oooo
00@000000?ooo`3oool000007@3oool010000000oooo0000000000020?ooo`<000000P3oool02000
0000oooo000000000000oooo0000003oool000000P3oool01@000000oooo0?ooo`000000oooo0080
00006P3oool010000000oooo0000000000020?ooo`<0000000@0oooo000000000000oooo0P000000
103oool000000?ooo`0000020?ooo`050000003oool0oooo0000003oool00P00000J0?ooo`040000
003oool0000000000080oooo0`0000000`3oool00000000000020?ooo`050000003oool000000?oo
o`0000000P3oool01@000000oooo000000000000oooo008000003P3oool000d0oooo00@000000?oo
o`3oool000000P3oool00`000000oooo0?ooo`020?ooo`050000003oool000000000003oool00`00
00020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0603oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01@0000020?ooo`05
0000003oool0oooo0000003oool00P00000J0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool010000000oooo0000003oool400000080oooo00@000000?ooo`3oool00000
0P3oool2000001T0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`05
0000003oool0oooo0000003oool00`0000020?ooo`080000003oool0oooo0000003oool000000?oo
o`00000=0?ooo`003@3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool01@000000
oooo000000000000oooo00<000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo01P0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00@000000?ooo`000000
oooo100000020?ooo`050000003oool0oooo0000003oool00P00000J0?ooo`070000003oool0oooo
0?ooo`000000oooo000000050?ooo`<0000000@0oooo00000000000000000P3oool020000000oooo
0?ooo`000000oooo0000003oool000006@3oool01`000000oooo0?ooo`3oool000000?ooo`000000
1@3oool010000000oooo0000003oool300000080oooo00@000000?ooo`3oool0oooo0`00000>0?oo
o`00303oool3000000030?ooo`000000oooo0080oooo0P0000050?ooo`030000003oool0000000T0
oooo0`00000H0?ooo`<0000000<0oooo0000003oool00P3oool2000000D0oooo00<000000?ooo`00
00002P3oool00`000000oooo0?ooo`0G0?ooo`<0000000<0oooo0000003oool00P3oool200000080
oooo00H000000?ooo`3oool000000?ooo`00000:0?ooo`030000003oool0oooo01L0oooo0`000000
0`3oool000000?ooo`020?ooo`8000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00D0oooo00@000000?ooo`3oool000003P3oool000`0oooo0P00000d0?ooo`800000=03oool2
000003D0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03oool30000
02L0oooo000=0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`0Y0?ooo`030000003oool0oooo00L0oooo1000000K0?ooo`00603oool2000003<0oooo
0`00000d0?ooo`<00000<`3oool010000000oooo00000000000K0?ooo`000P3oool00`000000oooo
0?ooo`0C0?ooo`8000006@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo
00<000000?ooo`0000006@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`000000603oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`000000603oool00`000000oooo0?ooo`010?ooo`000`3oool00`000000oooo0?oo
o`0<0?ooo`800000103oool00`000000oooo0?ooo`02000001D0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0<0?ooo`8000000`3oool010000000oooo0?ooo`3oool2000001D0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0<0?ooo`800000103oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0303oool2000000D0oooo
0P00000G0?ooo`030000003oool0oooo0080oooo00040?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool01@3oool2000001D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00D0oooo0P00000E0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0080oooo0005
0?ooo`800000303oool00`000000oooo0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`07
0?ooo`800000303oool00`000000oooo0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`07
0?ooo`800000303oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00L0oooo0P00000<
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00`3oool000L0oooo00<000000?oo
o`3oool0203oool3000001X0oooo0P00000<0?ooo`030000003oool0oooo00P0oooo0`00000J0?oo
o`800000303oool00`000000oooo0?ooo`080?ooo`<000006P3oool2000000`0oooo00<000000?oo
o`3oool0203oool3000001/0oooo00<000000?ooo`3oool0103oool000P0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3oool0
3P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0
3P3oool2000002<0oooo00<000000?ooo`3oool01`3oool000T0oooo0P00000O0?ooo`<000004`3o
ool00`000000oooo0?ooo`0M0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<000004`3o
ool00`000000oooo0?ooo`0M0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000003oool0
oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2
000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0
oooo00<000000?ooo`3oool01`3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool2000000X0oooo1000000;0?ooo`030000003oool0oooo00`0
oooo000?0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00d0oooo000@0?ooo`030000003oool0oooo008000000P3o
ool00`000000oooo0?ooo`040?ooo`<000000P3oool4000001l0oooo00<000000?ooo`3oool00P00
00020?ooo`030000003oool0oooo00@0oooo0`0000020?ooo`@000007`3oool00`000000oooo0?oo
o`0200000080oooo00<000000?ooo`3oool0103oool300000080oooo1000000O0?ooo`030000003o
ool0oooo008000000P3oool00`000000oooo0?ooo`040?ooo`<000000P3oool2000000030?ooo`00
0000oooo00l0oooo000A0?ooo`L000001P3oool7000000X0oooo00<000000?ooo`3oool05@3oool7
000000L0oooo1P00000:0?ooo`030000003oool0oooo01D0oooo0`000000103oool0000000000000
00070?ooo`H000002P3oool00`000000oooo0?ooo`0E0?ooo`<0000000@0oooo0000000000000000
1`3oool7000000T0oooo00<000000?ooo`3oool01@3oool00100oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3o
ool3000001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool01`3oool3000001@0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3o
ool3000001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool3000000D0oooo000>0?ooo`800000203oool2
00000080oooo00<000000?ooo`3oool01`3oool2000000L0oooo00<000000?ooo`0000004P3oool2
000000P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`030000003oool0
00000180oooo0P0000080?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool0
0`000000oooo0000000B0?ooo`800000203oool2000000<0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0000000D0oooo000=0?ooo`030000003oool0oooo
00X0oooo0P00000<0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0:0?ooo`<000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool01@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0;0?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo
00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo
00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00
000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<00000
0?ooo`3oool0503oool2000000/0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo0P00000;
0?ooo`800000403oool00`000000oooo0?ooo`0E0?ooo`8000002P3oool200000100oooo00<00000
0?ooo`3oool05@3oool2000000X0oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo00040?ooo`800000
4@3oool4000000@0oooo0`00000>0?ooo`800000203oool200000140oooo100000040?ooo`<00000
3P3oool2000000P0oooo0P00000J0?ooo`<000003@3oool2000000P0oooo0P00000J0?ooo`<00000
3P3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0A0?ooo`040000003o
ool00000000001L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`040000
003oool00000000001L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`@0
00005P3oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0B0?ooo`040000
003oool000000?ooo`8000005P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0180
oooo00@000000?ooo`000000oooo0P00000F0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool04`3oool3000001P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?oo
o`040000003oool00000000001L0oooo00<000000?ooo`3oool00@3oool001P0oooo0P0000000`3o
ool00000000000070?ooo`030000003oool0oooo02L0oooo0P0000000`3oool00000000000070?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`0000002@3oool00`000000oooo0?ooo`0W0?oo
o`030000003oool0000000T0oooo00<000000?ooo`3oool0403oool001/0oooo0P0000070?ooo`<0
0000:P3oool2000000L0oooo0`00000W0?ooo`030000003oool0oooo00T0oooo0`00000X0?ooo`80
00002@3oool300000100oooo000T0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000
<`3oool00`000000oooo0000000c0?ooo`030000003oool000000100oooo000T0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo0100oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`003@3oool010000000oooo0000000000020?ooo`<000000P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool000000080oooo00D000000?ooo`3oool000000?ooo`02000001X0oooo00@0
00000?ooo`00000000000P3oool300000080oooo00P000000?ooo`00000000000?ooo`000000oooo
00000080oooo00@000000?ooo`3oool000007@3oool010000000oooo0000000000020?ooo`<00000
0P3oool020000000oooo000000000000oooo0000003oool000000P3oool01@000000oooo0?ooo`00
0000oooo008000006P3oool010000000oooo0000000000020?ooo`<000000P3oool00`000000oooo
0?ooo`020?ooo`030000003oool000000080oooo00D000000?ooo`3oool000000?ooo`02000000h0
oooo000=0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool01@000000
oooo000000000000oooo00<000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo01P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00D000000P3oool01@000000oooo0?ooo`000000oooo008000006P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000001@0000020?ooo`04
0000003oool0oooo00000080oooo0P00000I0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool01@000000oooo0?ooo`000000oooo00<000000P3oool01@000000oooo0?oo
o`000000oooo00<000003@3oool000d0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0
oooo00D000000?ooo`00000000000?ooo`0300000080oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`0H0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000
003oool000000080oooo0`0000020?ooo`050000003oool0oooo0000003oool00P00000J0?ooo`07
0000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool000000080oooo0`000002
0?ooo`060000003oool0oooo0000003oool000006`3oool01`000000oooo0?ooo`3oool000000?oo
o`000000103oool01@000000oooo0?ooo`000000oooo00<000000P3oool01P000000oooo0?ooo`00
0000oooo000000l0oooo000<0?ooo`<0000000<0oooo0000003oool00P3oool2000000L0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0G0?ooo`<0000000<0oooo0000003oool00P3o
ool2000000D0oooo00<000000?ooo`000000903oool3000000030?ooo`000000oooo0080oooo0P00
00050?ooo`030000003oool0000000X0oooo00<000000?ooo`3oool05`3oool3000000030?ooo`00
0000oooo0080oooo0P0000040?ooo`@000002P3oool00`000000oooo0?ooo`0<0?ooo`00303oool2
000000d0oooo00<000000?ooo`3oool0903oool2000000d0oooo00<000000?ooo`3oool0903oool2
000003D0oooo0P00000X0?ooo`00303oool3000000`0oooo00<000000?ooo`3oool0903oool30000
00`0oooo00<000000?ooo`3oool0903oool3000003@0oooo0`00000W0?ooo`003@3oool00`000000
oooo0?ooo`080?ooo`80000000<0oooo0000003oool09P3oool00`000000oooo0?ooo`080?ooo`80
000000<0oooo0000003oool09P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02T0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0M0?ooo`00603oool2000003@0oooo
0P00000d0?ooo`800000<`3oool3000001d0oooo000H0?ooo`030000003oool0oooo03@0oooo00<0
00000?ooo`3oool0<P3oool2000003<0oooo00<000000?ooo`0000007@3oool00080oooo00<00000
0?ooo`3oool03@3oool2000000@0oooo0P00000I0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool03@3oool2000001l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=
0?ooo`800000103oool00`000000oooo0?ooo`02000001H0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0=0?ooo`8000000`3oool00`000000oooo0?ooo`020?ooo`8000005@3oool0
0`000000oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
0080oooo0P00000H0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`020?ooo`800000603oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool01@3oool2000001D0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo0P00000D0?ooo`030000003oool0oooo
0080oooo00040?ooo`8000003@3oool00`000000oooo0?ooo`050?ooo`800000503oool00`000000
oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`020?ooo`8000005`3oool00`000000
oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`050?ooo`800000503oool00`000000
oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`060?ooo`8000004`3oool00`000000
oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`090?ooo`<000001P3oool00`000000oooo
0?ooo`0A0?ooo`8000002`3oool00`000000oooo0?ooo`090?ooo`<000001P3oool200000180oooo
0P00000;0?ooo`030000003oool0oooo00T0oooo0`00000J0?ooo`8000002`3oool00`000000oooo
0?ooo`090?ooo`<000006`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S
0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S
0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?oo
o`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?oo
o`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`80
00003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`03
0000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
3@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<00000
1P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`@000002`3o
ool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3o
ool4000000/0oooo00<000000?ooo`3oool0303oool000h0oooo0P00000=0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool06P3oool2000000d0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0J0?ooo`8000003@3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo01X0oooo0P0000080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0=0?ooo`00403oool00`000000oooo0?ooo`0200000080oooo00<000000?oo
o`3oool0103oool300000080oooo1000000O0?ooo`030000003oool0oooo008000000`3oool00`00
0000oooo0?ooo`030?ooo`<000000P3oool4000001l0oooo00<000000?ooo`3oool00P0000030?oo
o`030000003oool0oooo00<0oooo0`0000020?ooo`@000007`3oool00`000000oooo0?ooo`020000
0080oooo00<000000?ooo`3oool0103oool300000080oooo0P0000000`3oool000000?ooo`0?0?oo
o`004@3oool7000000H0oooo1`00000R0?ooo`L000001`3oool600000280oooo0`000000103oool0
00000000000000070?ooo`H000008P3oool3000000040?ooo`0000000000000000H0oooo2000000A
0?ooo`00403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo000>0?ooo`80
0000203oool200000080oooo00<000000?ooo`3oool01`3oool2000000L0oooo0`00000B0?ooo`80
0000203oool2000000<0oooo00<000000?ooo`3oool01P3oool2000000L0oooo0`00000B0?ooo`80
0000203oool2000000<0oooo00<000000?ooo`3oool01P3oool2000000L0oooo0`00000B0?ooo`80
0000203oool2000000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`<0
00001@3oool000d0oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool0
103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`0:0?ooo`<00000
2`3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool0
2P3oool3000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo000000050?ooo`00303o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3o
ool000X0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000N0?ooo`030000003oool0
oooo01<0oooo0P00000N0?ooo`030000003oool0oooo01<0oooo0P00000O0?ooo`030000003oool0
oooo00T0oooo00090?ooo`800000803oool200000180oooo0P00000P0?ooo`<000004@3oool20000
0200oooo0`00000A0?ooo`8000008@3oool2000000T0oooo00080?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool02P3oool200000100oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0:0?ooo`800000403oool00`000000oooo0?ooo`0Q0?ooo`800000403oool00`000000
oooo0?ooo`0Q0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool2000000`0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`800000303oool00`000000oooo
0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0B0?ooo`060000003oool0oooo0000003oool0
00003P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0180oooo00H000000?ooo`3o
ool000000?ooo`00000>0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo00040?ooo`8000005@3oool01000
0000oooo0?ooo`3oool3000000h0oooo0P0000080?ooo`8000005@3oool010000000oooo0?ooo`3o
ool3000000h0oooo0P0000080?ooo`8000004@3oool2000000P0oooo00<000000?ooo`3oool0303o
ool2000000P0oooo0P00000A0?ooo`800000203oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00<0oooo00030?ooo`030000003oool0oooo0180oooo0P0000000`3oool000000?ooo`0E
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000
oooo01D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
000000H0oooo0`00000?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo000000060?ooo`<000003`3oool00`000000oooo0?ooo`020?ooo`000P3oool00`00
0000oooo0?ooo`0C0?ooo`8000006@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01<0oooo0P00000I0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool20000
0080oooo0P00000F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool20000
0080oooo0P00000F0?ooo`030000003oool0oooo0040oooo000H0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02L0
oooo00<000000?ooo`3oool00P0000070?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool00P0000070?ooo`030000003oool0oooo0100oooo000H0?ooo`8000002P3oool3000003<0oooo
0`00000Z0?ooo`8000001`3oool3000002X0oooo0P0000070?ooo`<00000403oool001P0oooo0P00
000:0?ooo`030000003oool0000002L0oooo0P00000:0?ooo`030000003oool0000003<0oooo00<0
00000?ooo`000000<`3oool00`000000oooo0000000@0?ooo`006`3oool2000000L0oooo00<00000
0?ooo`3oool09`3oool2000000X0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo0100oooo000K0?ooo`030000003oool0oooo03<0oooo0P0000260?oo
o`00D@3oool00`000000oooo0?ooo`250?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo000=0?ooo`040000003oool0000000000080oooo0`0000020?ooo`080000003oool00000
0000003oool000000?ooo`0000020?ooo`050000003oool000000000003oool00P00000J0?ooo`04
0000003oool0000000000080oooo0`0000020?ooo`080000003oool0oooo0000003oool000000?oo
o`0000020?ooo`050000003oool0oooo0000003oool00P00000J0?ooo`040000003oool000000000
0080oooo0`0000020?ooo`080000003oool000000000003oool000000?ooo`0000020?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool0603oool010000000oooo0000000000020?oo
o`<000000P3oool020000000oooo000000000000oooo0000003oool000000P3oool01@000000oooo
0?ooo`000000oooo008000003P3oool000d0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`020?ooo`040000003oool000000?ooo`@000000P3oool01P000000oooo0000003oool0
00000?ooo`8000006@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo
00D000000?ooo`3oool000000?ooo`0300000080oooo00P000000?ooo`3oool000000?ooo`000000
oooo000001T0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`050000
003oool0oooo0000003oool00`0000020?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool0603oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00D00000
0?ooo`3oool000000?ooo`0300000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo
0?ooo`0<0?ooo`003@3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool01@000000
oooo000000000000oooo00<000000P3oool020000000oooo000000000000oooo0000003oool00000
6@3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool01@000000oooo0?ooo`000000
oooo00<000000P3oool01@000000oooo0?ooo`000000oooo008000006P3oool01`000000oooo0?oo
o`3oool000000?ooo`000000103oool010000000oooo0?ooo`3oool400000080oooo00@000000?oo
o`3oool000000P3oool00`000000oooo0?ooo`0H0?ooo`070000003oool0oooo0?ooo`000000oooo
000000040?ooo`050000003oool0oooo0000003oool00`0000020?ooo`040000003oool0oooo0000
00<0oooo00<000000?ooo`3oool02`3oool000`0oooo0`0000000`3oool000000?ooo`020?ooo`80
00001@3oool00`000000oooo000000070?ooo`040000003oool0oooo000001T0oooo0`0000000`3o
ool000000?ooo`020?ooo`8000001`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
01L0oooo0`0000000`3oool000000?ooo`020?ooo`8000001@3oool00`000000oooo0000000T0?oo
o`<0000000<0oooo0000003oool00P3oool2000000D0oooo00<000000?ooo`0000002P3oool00`00
0000oooo0?ooo`0<0?ooo`00303oool2000000d0oooo00<000000?ooo`3oool0903oool2000003@0
oooo0P00000e0?ooo`800000303oool00`000000oooo0?ooo`0I0?ooo`00303oool3000000`0oooo
00<000000?ooo`3oool0903oool3000003<0oooo0`00000d0?ooo`<000002`3oool00`000000oooo
0?ooo`0I0?ooo`003@3oool00`000000oooo0?ooo`080?ooo`80000000<0oooo0000003oool09P3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203o
ool4000002P0oooo00<000000?ooo`3oool01`3oool2000000030?ooo`000000oooo01X0oooo000H
0?ooo`800000=03oool3000003<0oooo00@000000?ooo`0000000000<P3oool3000001`0oooo000B
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0000000c
0?ooo`030000003oool0000003@0oooo00<000000?ooo`3oool06`3oool00080oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool03@3oool2000000@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00d0oooo0P0000050?ooo`800000603oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00d0oooo0P0000040?ooo`8000006@3oool00`000000oooo0?ooo`010?oo
o`000`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000000@0oooo0P00000H0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`020?ooo`80
0000603oool00`000000oooo0?ooo`020?ooo`00103oool2000000`0oooo0`0000030?ooo`800000
5`3oool00`000000oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo00H0oooo0P00000=0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
1P3oool2000000d0oooo00<000000?ooo`3oool01@3oool2000001@0oooo00<000000?ooo`3oool0
0`3oool000H0oooo00<000000?ooo`3oool04P3oool200000180oooo0P00000;0?ooo`030000003o
ool0oooo00T0oooo0`00000J0?ooo`8000002`3oool00`000000oooo0?ooo`090?ooo`<000006P3o
ool2000000/0oooo00<000000?ooo`3oool02@3oool3000000H0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo02@0oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`030000003oool0oooo
00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo
00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000007`3oool300000140
oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool3000000X0
oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo
00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo
0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00T0oooo
00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`00
00060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<00000
0?ooo`3oool0303oool000h0oooo0P00000=0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool06P3oool2000000d0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0J0?oo
o`8000003@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P00000>0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool00100oooo00<000000?ooo`3o
ool00P0000020?ooo`030000003oool0oooo00<0oooo00@000000?ooo`00000000000P3oool40000
01l0oooo00<000000?ooo`3oool00P0000030?ooo`030000003oool0oooo0080oooo00@000000?oo
o`00000000000P3oool4000001l0oooo00<000000?ooo`3oool00P0000030?ooo`030000003oool0
oooo0080oooo00@000000?ooo`00000000000P3oool4000001l0oooo00<000000?ooo`3oool00P00
00030?ooo`030000003oool0oooo00<0oooo0`0000020?ooo`80000000<0oooo0000003oool03`3o
ool00140oooo1`0000060?ooo`L000008P3oool7000000H0oooo1`00000R0?ooo`<0000000@0oooo
00000000000000001P3oool700000280oooo0`000000103oool000000000000000060?ooo`P00000
4@3oool00100oooo00<000000?ooo`0000000`3oool2000000H0oooo00@000000?ooo`3oool00000
0`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`000000
0`3oool2000000H0oooo00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool2000000H0oooo00D000000?ooo`3o
ool0oooo000000020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`00
0000oooo000000030?ooo`8000001P3oool01@000000oooo0?ooo`3oool0000000<0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo00D000000?oo
o`3oool0oooo000000090?ooo`8000001`3oool300000180oooo0P0000080?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool01P3oool2000000L0oooo0`00000B0?ooo`800000203oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`<000004P3oool2
000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00D0oooo0`0000050?ooo`003@3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`00
0000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool02@3oool40000
00/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo
00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`00
0000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000003oool00000
00D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`050?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo
00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo
00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00
000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<00000
0?ooo`3oool08@3oool200000100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0:
0?ooo`800000403oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00T0oooo0P00000@
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool02@3oool2000000T0oooo00070?oo
o`030000003oool0oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00060?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0000000>0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0000000=0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool04P3oool01`000000oooo0?ooo`3oool00000
0?ooo`0000003P3oool00`000000oooo0?ooo`040?ooo`00103oool2000001D0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`800000203oool2000001T0oooo0`00000>0?oo
o`800000203oool2000001X0oooo0`00000=0?ooo`800000203oool2000001D0oooo00<000000?oo
o`3oool00P3oool3000000h0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3o
ool04P3oool2000000030?ooo`000000oooo0080oooo0`00000@0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool04P3oool4000001H0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool05@3oool00`000000oooo0?ooo`020?ooo`00
0P3oool00`000000oooo0?ooo`0C0?ooo`8000006@3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01<0oooo0`00000H0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
4`3oool010000000oooo00000000000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool04`3oool3000001P0oooo00<000000?ooo`3oool00@3oool001T0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`0W0?ooo`030000003oool0000000T0oooo00<000000?ooo`3oool0
9`3oool00`000000oooo000000090?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`0@0?ooo`00903oool3000002L0oooo00<000000?ooo`3oool02@3o
ool3000002P0oooo0P0000090?ooo`<000009`3oool2000000X0oooo0`00000@0?ooo`00603oool2
000000X0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000003oool0
000002L0oooo0P00000:0?ooo`030000003oool000000100oooo000H0?ooo`8000002P3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0:P3oool20000
00L0oooo00<000000?ooo`3oool0403oool001/0oooo0P00002P0?ooo`030000003oool0oooo01T0
oooo000K0?ooo`030000003oool0oooo0;/0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo000>0?ooo`040000003oool0000000000080oooo0`0000000`3oool00000
000000020?ooo`030000003oool000000080oooo00D000000?ooo`00000000000?ooo`02000001`0
oooo00@000000?ooo`00000000000P3oool3000000D0oooo00<000000?ooo`0000000P3oool01@00
0000oooo000000000000oooo00800000703oool010000000oooo0000000000020?ooo`<0000000<0
oooo0000000000000P3oool00`000000oooo000000020?ooo`040000003oool00000000001l0oooo
00@000000?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo00<000000?ooo`00
00000P3oool01@000000oooo000000000000oooo008000003`3oool000h0oooo00@000000?ooo`3o
ool000000P3oool010000000oooo0?ooo`3oool3000000040?ooo`000000000000000080oooo00H0
00000?ooo`000000oooo0000003oool2000001/0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00<000000P3oool010000000oooo0000003oool4
000001/0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool200000080oooo
0`0000020?ooo`070000003oool000000?ooo`000000oooo0000000L0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool00P3oool2000000040?ooo`000000000000000080oooo00@0
00000?ooo`000000oooo0`00000?0?ooo`003P3oool01`000000oooo0?ooo`3oool000000?ooo`00
00000`3oool010000000oooo0?ooo`3oool300000080oooo00H000000?ooo`00000000000?ooo`00
000M0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0oooo00<0
00000P3oool01P000000oooo000000000000oooo000001d0oooo00L000000?ooo`3oool0oooo0000
003oool0000000<0oooo0P0000020?ooo`<000000P3oool010000000oooo0000000000020?ooo`03
0000003oool0oooo01X0oooo00L000000?ooo`3oool0oooo0000003oool0000000<0oooo00@00000
0?ooo`000000oooo0`0000020?ooo`050000003oool000000000003oool00P00000?0?ooo`00303o
ool4000000030?ooo`000000oooo0080oooo0P0000020?ooo`040000003oool0oooo000000L0oooo
00@000000?ooo`3oool000006P3oool4000000030?ooo`000000oooo0080oooo0P0000001P3oool0
0000000000000000oooo000000L0oooo00@000000?ooo`3oool000006P3oool4000000030?ooo`00
0000oooo0080oooo0P0000020?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`000000
0P00000I0?ooo`@0000000<0oooo0000003oool00P3oool200000080oooo00@000000?ooo`3oool0
00001`3oool010000000oooo0?ooo`00000?0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<0
0000303oool00`000000oooo0?ooo`0T0?ooo`<00000303oool00`000000oooo0?ooo`0I0?ooo`00
3@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
203oool4000002L0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02H0oooo
00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo01X0oooo000H0?ooo`<00000<`3o
ool010000000oooo00000000000b0?ooo`@00000<P3oool2000000030?ooo`000000oooo01X0oooo
000B0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`000000;@3oool00`000000oooo0?oo
o`030?ooo`030000003oool0000002d0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0703oool00080oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo0P00000H
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`02
0?ooo`8000006@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool00P3oool2000001T0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0000000N0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool02`3oool00`000000oooo000000040?ooo`800000603oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`000000103oool2000001P0oooo00<000000?oo
o`3oool00P3oool000@0oooo0P00000<0?ooo`<00000703oool00`000000oooo0?ooo`060?ooo`80
0000303oool3000001`0oooo00<000000?ooo`3oool01P3oool2000000`0oooo0`0000060?ooo`80
0000503oool00`000000oooo0?ooo`060?ooo`800000303oool3000000H0oooo0P00000D0?ooo`03
0000003oool0oooo00<0oooo00060?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000
003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4@3oool2000000/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo00@0oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3o
ool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00L0oooo00080?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool30000
01l0oooo0`00000A0?ooo`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000
oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0
oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo
00030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo1000
000;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`090?ooo`@000002`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0303oool000h0oooo0P000009
0?ooo`050000003oool0oooo0?ooo`0000002@3oool00`000000oooo0?ooo`0J0?ooo`8000002@3o
ool01@000000oooo0?ooo`3oool0000000T0oooo00<000000?ooo`3oool06P3oool2000000T0oooo
00D000000?ooo`3oool0oooo000000090?ooo`030000003oool0oooo01X0oooo0P0000090?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0=0?ooo`00
403oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool00`3oool010000000oooo
0000000000020?ooo`@000007`3oool00`000000oooo0?ooo`02000000<0oooo00<000000?ooo`3o
ool00P3oool010000000oooo0000000000020?ooo`@000007`3oool01@000000oooo0?ooo`3oool0
000000<0oooo00<000000?ooo`3oool00P3oool010000000oooo0000000000020?ooo`@000007`3o
ool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00`3oool300000080oooo
0P0000000`3oool000000?ooo`0?0?ooo`004@3oool7000000H0oooo1`00000R0?ooo`L000001P3o
ool700000280oooo1`0000060?ooo`L000008P3oool7000000H0oooo2000000A0?ooo`00403oool3
00000080oooo0`0000060?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000P3oool3000000H0oooo00<00000
0?ooo`3oool00P0000020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0503oool0
0`000000oooo000000020?ooo`<000001P3oool00`000000oooo0?ooo`0200000080oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P000006
0?ooo`030000003oool0oooo008000000`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00D0oooo000>0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00H0oooo0P0000070?ooo`<000004P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`060?ooo`8000001`3oool300000180oooo0P0000080?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool01P3oool2000000L0oooo0`00000B0?ooo`800000203oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool30000
00D0oooo000=0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000
oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool02@3oool4000000/0
oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0
oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000001@3oool000`0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo
000:0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007P3oool00`000000oooo0?oo
o`0C0?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007`3oool00`000000oooo0?oo
o`090?ooo`002@3oool200000200oooo0P00000B0?ooo`800000803oool300000140oooo0P00000P
0?ooo`<000004@3oool200000240oooo0P0000090?ooo`00203oool00`000000oooo0?ooo`0Q0?oo
o`800000403oool00`000000oooo0?ooo`0Q0?ooo`800000403oool00`000000oooo0?ooo`0Q0?oo
o`800000403oool00`000000oooo0?ooo`0Q0?ooo`8000002@3oool000L0oooo00<000000?ooo`3o
ool05@3oool2000000d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0E0?ooo`80
00003@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo0P00000<0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool2000000`0oooo00<000000?ooo`3o
ool01P3oool000H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00@0oooo00040?ooo`8000006P3oool00`000000oooo0?ooo`0=0?ooo`800000203o
ool2000001X0oooo00<000000?ooo`3oool03@3oool2000000P0oooo0P00000E0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0303oool2000000P0oooo0P00000E0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`000`3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00@0oooo0`00000@0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool04P3oool4000000<0oooo0`00000@0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000oooo00<0oooo0`00000?
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000
oooo00<0oooo0`00000?0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo
01<0oooo0`00000H0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool01000
0000oooo00000000000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool4
000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo
0000003oool05P3oool00`000000oooo0?ooo`010?ooo`00603oool00`000000oooo000000090?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`0000002@3oool00`000000oooo0?ooo`0X0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0100oooo000H0?ooo`030000003oool0oooo00T0oooo0`00000X0?ooo`80
00002@3oool3000002L0oooo0P00000:0?ooo`<000009`3oool2000000X0oooo0`00000@0?ooo`00
903oool00`000000oooo0000000c0?ooo`030000003oool0000002L0oooo0P00000:0?ooo`030000
003oool0000002L0oooo0P00000:0?ooo`030000003oool000000100oooo000T0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool0:P3oool2000000L0oooo00<000000?ooo`3oool0:P3o
ool2000000L0oooo00<000000?ooo`3oool0403oool008L0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool000h0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`0000000000
0080oooo00<000000?ooo`0000000P3oool010000000oooo0000000000020?ooo`030000003oool0
oooo01X0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo00<0
00000?ooo`0000000P3oool01@000000oooo000000000000oooo00800000703oool010000000oooo
0000000000020?ooo`<000001@3oool00`000000oooo000000020?ooo`050000003oool000000000
003oool00P00000L0?ooo`040000003oool0000000000080oooo0`0000000`3oool0000000000002
0?ooo`030000003oool000000080oooo00@000000?ooo`00000000004P3oool000h0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<000000P3o
ool01`000000oooo0000003oool000000?ooo`000000703oool010000000oooo0?ooo`0000020?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000020?ooo`070000003oool00000
0?ooo`000000oooo0000000L0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0
oooo0P0000020?ooo`<000000P3oool01`000000oooo0000003oool000000?ooo`000000703oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo0P000000103oool000000000
000000020?ooo`040000003oool000000?ooo`<000003`3oool000h0oooo00L000000?ooo`3oool0
oooo0000003oool0000000D0oooo00<000000?ooo`0000000P0000020?ooo`040000003oool00000
00000080oooo00<000000?ooo`3oool06P3oool01`000000oooo0?ooo`3oool000000?ooo`000000
103oool00`000000oooo0?ooo`0300000080oooo00@000000?ooo`00000000000`3oool00`000000
oooo0?ooo`0I0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`8000000P3oool3
00000080oooo00@000000?ooo`00000000000P3oool00`000000oooo0?ooo`0J0?ooo`070000003o
ool0oooo0?ooo`000000oooo000000030?ooo`040000003oool0oooo0?ooo`<000000P3oool01@00
0000oooo000000000000oooo008000003`3oool000`0oooo100000000`3oool000000?ooo`020?oo
o`8000000P3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo01/0oooo10000000
0`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`0000070?ooo`040000003o
ool0oooo000001X0oooo100000000`3oool000000?ooo`020?ooo`8000001@3oool00`000000oooo
0?ooo`050?ooo`040000003oool0oooo000001/0oooo0`0000000`3oool000000?ooo`020?ooo`80
00000P3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo0100oooo000<0?ooo`<0
0000<`3oool3000000`0oooo00<000000?ooo`3oool0903oool3000000`0oooo00<000000?ooo`3o
ool09@3oool3000000/0oooo00<000000?ooo`3oool06@3oool000d0oooo00<000000?ooo`3oool0
203oool4000002L0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02H0oooo
00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02L0oooo00<000000?ooo`3oool0
1`3oool2000000030?ooo`000000oooo01X0oooo000H0?ooo`040000003oool0000000000380oooo
1000000b0?ooo`80000000<0oooo0000003oool0<@3oool2000000030?ooo`000000oooo01X0oooo
000B0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`000000;@3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0]0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06`3oool00080oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`030?ooo`800000603oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00P3oool2000001T0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0080oooo0P00000I
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`0000007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`000000103oool2000001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0000000@0oooo0P00000H0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool02`3oool00`000000oooo000000040?ooo`800000603oool00`000000oooo0?ooo`020?oo
o`00103oool2000000`0oooo0`00000L0?ooo`030000003oool0oooo00H0oooo0P00000<0?ooo`<0
00001P3oool2000001@0oooo00<000000?ooo`3oool01P3oool2000000`0oooo0`0000060?ooo`80
0000503oool00`000000oooo0?ooo`060?ooo`800000303oool3000000<0oooo0P00000G0?ooo`03
0000003oool0oooo00<0oooo00060?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool04@3oool2000000/0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0A0?ooo`8000002`3oool00`000000oooo0?ooo`0B0?ooo`800000
4`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000007`3oool30000
0140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool30000
00X0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0
oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0
oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00T0
oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo
0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool02@3oool3000000`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00T0oooo0`00000<0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<0
00000?ooo`3oool0303oool000h0oooo0P0000090?ooo`030000003oool0oooo008000002@3oool0
0`000000oooo0?ooo`0J0?ooo`8000002@3oool00`000000oooo0?ooo`02000000T0oooo00<00000
0?ooo`3oool06P3oool2000000T0oooo00<000000?ooo`3oool00P0000090?ooo`030000003oool0
oooo01X0oooo0P0000090?ooo`060000003oool0oooo0000003oool00000203oool00`000000oooo
0?ooo`0=0?ooo`00403oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool0
0`3oool00`000000oooo000000030?ooo`040000003oool00000000001l0oooo00D000000?ooo`3o
ool0oooo000000030?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000000P3oool0
10000000oooo00000000000O0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo
0?ooo`020?ooo`040000003oool0oooo000000<0oooo0`00000O0?ooo`050000003oool0oooo0?oo
o`0000000`3oool00`000000oooo0?ooo`030?ooo`030000003oool0000000<0oooo00<000000?oo
o`000000403oool00140oooo1`0000060?ooo`L000008P3oool7000000H0oooo1`00000R0?ooo`03
0000003oool0000000@000001P3oool700000280oooo00<000000?ooo`0000000P0000000`3oool0
00000?ooo`050?ooo`P000004@3oool00100oooo0`0000020?ooo`<000001P3oool00`000000oooo
0?ooo`0200000080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`<00000
0P3oool3000000H0oooo00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0503oool300000080oooo0`0000060?ooo`030000003oool0oooo00800000
0P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo0`0000020?ooo`<00000
1P3oool00`000000oooo0?ooo`02000000<0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`050?ooo`003P3oool2000000P0oooo00<000000?ooo`3oool02`3oool2000000L0oooo0`00
000B0?ooo`800000203oool00`000000oooo0?ooo`0;0?ooo`8000001`3oool300000180oooo0P00
00080?ooo`030000003oool0oooo00/0oooo0P0000070?ooo`<000004P3oool2000000P0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo0`00
00050?ooo`003@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool000000140oooo00<000000?ooo`3oool02@3oool5000000X0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1@00000:
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`09
0?ooo`D000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000D0oooo000<0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?oo
o`002`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00T0oooo00090?ooo`800000
803oool200000180oooo0P00000P0?ooo`<000004@3oool200000200oooo0`00000A0?ooo`800000
8@3oool2000000T0oooo00080?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02P3o
ool200000100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0:0?ooo`800000403o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00T0oooo0P00000@0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool02@3oool2000000T0oooo00070?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`001P3oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0000000h0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0000000h0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0000000d0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0000000h0oooo00<000000?ooo`3oool0103oool000@0
oooo0P00000I0?ooo`<000003P3oool2000000P0oooo0P00000E0?ooo`040000003oool0oooo0?oo
o`<000003P3oool2000000P0oooo0P00000E0?ooo`030000003oool0oooo0080oooo0`00000=0?oo
o`800000203oool2000001D0oooo00<000000?ooo`3oool00P3oool3000000h0oooo00<000000?oo
o`3oool00`3oool000<0oooo00<000000?ooo`3oool04P3oool4000001H0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool05@3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0180oooo0P0000000`3oool000000?ooo`0E0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000030?ooo`000000oooo01D0
oooo00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool04`3oool010000000oooo
00000000000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool4000001L0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo0000003o
ool05P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0P0000000`3oool0
00000?ooo`0F0?ooo`030000003oool0oooo0040oooo000H0?ooo`030000003oool0000000T0oooo
00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool0403oool001T0oooo0P0000090?ooo`<000009`3oool2000000X0oooo0`00
000W0?ooo`8000002P3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo0000000W
0?ooo`8000002P3oool00`000000oooo0000000W0?ooo`8000002P3oool00`000000oooo0000000W
0?ooo`8000002P3oool00`000000oooo0000000@0?ooo`00903oool00`000000oooo0?ooo`0Z0?oo
o`8000001`3oool00`000000oooo0?ooo`0Z0?ooo`8000001`3oool00`000000oooo0?ooo`0W0?oo
o`8000002P3oool00`000000oooo0?ooo`0@0?ooo`00D@3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo03<0oooo0P00000J0?ooo`00_@3oool00`000000oooo0?ooo`0I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool005@0oooo0`000008
0?ooo`<000002`3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo000000L0oooo00@0
00000?ooo`3oool00000EP3oool005@0oooo0`0000080?ooo`<00000203oool5000000D0oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo
0`00000;0?ooo`030000003oool0oooo00@0oooo100000070?ooo`@00000EP3oool007@0oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C22) show
80.562500 14.625000 moveto
(N121) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C23) show
80.562500 14.625000 moveto
(N122) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C24) show
80.562500 14.625000 moveto
(N123) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C25) show
80.562500 14.625000 moveto
(N124) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N125) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N126) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N127) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N128) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N129) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N130) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N131) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .005 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.5 .48364 .07455 12.6804 167.32 arc
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(+) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N132) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N133) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N134) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N135) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56777 .49668 L
.56317 .49373 L
.55934 .4909 L
.55653 .48788 L
.5548 .4844 L
.55402 .48024 L
.55389 .47533 L
.55401 .46975 L
.55392 .46372 L
.55322 .45762 L
.55159 .45188 L
.54886 .44696 L
.54504 .44324 L
.54027 .441 L
.53482 .44031 L
.52903 .44108 L
.52324 .44299 L
.51771 .44562 L
.51264 .44843 L
.50807 .4509 L
.50392 .45259 L
.5 .45318 L
.49608 .45259 L
.49193 .4509 L
.48736 .44843 L
.48229 .44562 L
.47676 .44299 L
.47097 .44108 L
.46518 .44031 L
.45973 .441 L
.45496 .44324 L
.45114 .44696 L
.44841 .45188 L
.44678 .45762 L
.44608 .46372 L
.44599 .46975 L
.44611 .47533 L
.44598 .48024 L
.4452 .4844 L
.44347 .48788 L
.44066 .4909 L
.43683 .49373 L
.43223 .49668 L
.42727 .5 L
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N136) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N137) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N138) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N139) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N140) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000h0oooo00@000000?ooo`00
000000000P3oool300000080oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`040000003o
ool00000000001l0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`00000000000080
oooo00<000000?ooo`0000000P3oool01@000000oooo000000000000oooo00800000703oool01000
0000oooo0000000000020?ooo`<0000000<0oooo0000000000000P3oool00`000000oooo00000002
0?ooo`050000003oool000000000003oool00P00000L0?ooo`040000003oool0000000000080oooo
0`0000050?ooo`030000003oool000000080oooo00<000000?ooo`3oool00P3oool2000000l0oooo
000>0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0300000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0J0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030000
0080oooo00D000000?ooo`3oool000000?ooo`02000001`0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<000000P3oool010000000oooo0?oo
o`0000020?ooo`8000006`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?oo
o`8000000P3oool300000080oooo00P000000?ooo`00000000000?ooo`000000oooo000000h0oooo
000>0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0oooo00<0
00000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo01X0oooo00L000000?oo
o`3oool0oooo0000003oool0000000D0oooo00<000000?ooo`0000000P0000020?ooo`050000003o
ool0oooo0000003oool00P00000L0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?oo
o`030000003oool0oooo00<000000P3oool020000000oooo0?ooo`000000oooo0000003oool00000
6`3oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool200000080oooo0`0000020?oo
o`050000003oool000000000003oool00P00000?0?ooo`003@3oool3000000030?ooo`000000oooo
0080oooo0P0000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000000P00000J0?oo
o`<0000000<0oooo0000003oool00P3oool200000080oooo00@000000?ooo`3oool000001`3oool0
10000000oooo0?ooo`00000K0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`00000K0?ooo`<0000000<0oooo0000003o
ool00P3oool2000000D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`00
303oool2000003@0oooo0P00000d0?ooo`800000=@3oool2000002P0oooo000<0?ooo`<00000<`3o
ool3000003<0oooo0`00000d0?ooo`<000009`3oool000d0oooo00<000000?ooo`3oool0203oool4
000002L0oooo00<000000?ooo`3oool0203oool4000002L0oooo00<000000?ooo`3oool0203oool4
000002P0oooo00<000000?ooo`3oool01`3oool4000001/0oooo000H0?ooo`040000003oool00000
00000380oooo00@000000?ooo`0000000000<P3oool010000000oooo00000000000b0?ooo`040000
003oool00000000001/0oooo00020?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0000000H0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`030?ooo`030000003oool0000001P0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`000000
603oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0000000H0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool00`3oool2000001L0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo0P00000G0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`8000005`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00`3o
ool2000001L0oooo00<000000?ooo`3oool00P3oool000@0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0000000N0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0000000N0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0000000N0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0000000N0?ooo`030000003oool0oooo0080oooo00050?ooo`8000002`3oool30000
01`0oooo00<000000?ooo`3oool01`3oool2000000/0oooo0`00000L0?ooo`030000003oool0oooo
00L0oooo0P00000;0?ooo`<00000703oool00`000000oooo0?ooo`070?ooo`8000002`3oool30000
01`0oooo00<000000?ooo`3oool00`3oool000L0oooo00<000000?ooo`3oool09@3oool2000000`0
oooo00<000000?ooo`3oool09@3oool2000000`0oooo00<000000?ooo`3oool09@3oool2000000`0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`040?ooo`00203oool00`000000oooo
0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0S0?ooo`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?oo
o`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?oo
o`0>0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`002@3oool2000001l0oooo0`00000C
0?ooo`030000003oool0oooo01d0oooo0`00000C0?ooo`030000003oool0oooo01d0oooo0`00000C
0?ooo`030000003oool0oooo01d0oooo0`00000:0?ooo`000P3oool2000000L0oooo00<000000?oo
o`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?oo
o`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool3000000L0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`00
0`3oool00`000000oooo0?ooo`070?ooo`8000002P3oool4000000/0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool4000000/0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool4000000/0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool4000000/0oooo00<000000?ooo`3oool0
303oool000l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool03@3oool00100oooo00<000000?ooo`3oool00P000002
0?ooo`030000003oool0oooo00@0oooo0`0000020?ooo`@000007`3oool00`000000oooo0?ooo`02
00000080oooo00<000000?ooo`3oool0103oool300000080oooo1000000O0?ooo`030000003oool0
oooo008000000P3oool00`000000oooo0?ooo`040?ooo`<000000P3oool4000001l0oooo00<00000
0?ooo`3oool00P0000020?ooo`030000003oool0oooo00@0oooo0`0000020?ooo`80000000<0oooo
0000003oool03`3oool00100oooo200000060?ooo`L000002P3oool00`000000oooo0?ooo`0D0?oo
o`T000001@3oool7000000X0oooo00<000000?ooo`3oool0503oool4000000030?ooo`0000000000
008000001@3oool7000000X0oooo00<000000?ooo`3oool0503oool4000000030?ooo`0000000000
008000001@3oool8000000T0oooo00<000000?ooo`3oool01@3oool00100oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo0?ooo`070?ooo`<0
0000503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`070?ooo`<00000503oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`<0
0000503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`060?ooo`<000001@3oool000h0oooo0P0000080?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool01P3oool2000000L0oooo00<000000?ooo`000000
4P3oool2000000P0oooo00<000000?ooo`3oool00P3oool2000000L0oooo0P0000070?ooo`030000
003oool000000180oooo0P0000080?ooo`030000003oool0oooo0080oooo0P0000070?ooo`800000
1`3oool00`000000oooo0000000B0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`800000
203oool00`000000oooo0?ooo`050?ooo`030000003oool0000000D0oooo000=0?ooo`030000003o
ool0oooo00P0oooo1P00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`080?ooo`H000002P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0203oool6000000X0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00P0oooo1P00000:0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool01@3oool000`0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0;0?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool20000
01h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool20000
01l0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000D0?ooo`030000003oool0oooo00T0
oooo0P00000B0?ooo`800000503oool00`000000oooo0?ooo`090?ooo`<000004@3oool200000200
oooo0`00000A0?ooo`8000008@3oool2000000T0oooo00080?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool02@3oool200000100oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`090?ooo`800000403oool00`000000oooo0?ooo`0E0?ooo`8000002P3oool200000100oooo
00<000000?ooo`3oool05@3oool2000000X0oooo0P0000090?ooo`001`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0000000d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0000000d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01P3oool000H0oooo00<00000
0?ooo`3oool05P3oool3000000d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0F
0?ooo`<000003@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`00103oool2000002X0oooo0P0000080?oo
o`800000:P3oool2000000P0oooo0P00000J0?ooo`<000003@3oool2000000P0oooo0P00000J0?oo
o`<000003P3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0000001H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0000001H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0000001H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0000001H0oooo00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool0
4`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`@00000
5`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo1000000G0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool0
0@3oool001P0oooo100000080?ooo`030000003oool0oooo02L0oooo100000080?ooo`030000003o
ool0oooo02L0oooo100000080?ooo`030000003oool0oooo02L0oooo100000080?ooo`030000003o
ool0oooo0100oooo000H0?ooo`050000003oool000000?ooo`0000001`3oool3000002L0oooo00D0
00000?ooo`000000oooo000000070?ooo`<000009`3oool01@000000oooo0000003oool0000000L0
oooo0`00000W0?ooo`050000003oool000000?ooo`0000001`3oool300000100oooo000T0?ooo`03
0000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`03
0000003oool000000100oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`003P3oool010000000oooo00000000
00020?ooo`<000000P3oool01P000000oooo0?ooo`000000oooo00000080oooo00D000000?ooo`00
000000000?ooo`02000001`0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`000000
00000080oooo00<000000?ooo`0000000P3oool010000000oooo00000000000O0?ooo`040000003o
ool0000000000080oooo0`0000000`3oool00000000000020?ooo`030000003oool000000080oooo
00D000000?ooo`00000000000?ooo`02000001`0oooo00@000000?ooo`00000000000P3oool30000
00D0oooo00<000000?ooo`0000000P3oool01@000000oooo000000000000oooo008000003`3oool0
00h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00<000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo01X0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<00000
0P3oool01@000000oooo0?ooo`000000oooo00800000703oool010000000oooo0?ooo`0000020?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000020?ooo`040000003oool0oooo
00000080oooo0P00000K0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo
0P0000020?ooo`<000000P3oool01@000000oooo0?ooo`000000oooo00<000003P3oool000h0oooo
00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`3oool00`0000020?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool06P3oool01`000000oooo0?ooo`3o
ool000000?ooo`0000001@3oool00`000000oooo0000000200000080oooo00D000000?ooo`3oool0
00000?ooo`02000001`0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<00000
0?ooo`3oool00`0000020?ooo`060000003oool0oooo0000003oool000007@3oool01`000000oooo
0?ooo`3oool000000?ooo`0000000`3oool200000080oooo0`0000020?ooo`060000003oool0oooo
0000003oool00000403oool000d0oooo0`0000000`3oool000000?ooo`020?ooo`8000001@3oool0
0`000000oooo0?ooo`050?ooo`040000003oool0oooo000001/0oooo0`0000000`3oool000000?oo
o`020?ooo`8000000P3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo01`0oooo
0`0000000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`0000070?ooo`04
0000003oool0oooo000001/0oooo0`0000000`3oool000000?ooo`020?ooo`8000001@3oool00`00
0000oooo0?ooo`050?ooo`040000003oool0oooo000000l0oooo000<0?ooo`800000=03oool20000
03@0oooo0P00000e0?ooo`800000:03oool000`0oooo0`00000c0?ooo`<00000<`3oool3000003@0
oooo0`00000W0?ooo`003@3oool00`000000oooo0?ooo`090?ooo`030000003oool0000002L0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0000000W0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`000000:03oool00`000000oooo0?ooo`080?ooo`030000003oool0000001/0oooo
000H0?ooo`@00000<P3oool400000380oooo1000000b0?ooo`@000006`3oool00180oooo00<00000
0?ooo`3oool00`3oool4000002`0oooo00<000000?ooo`3oool00`3oool400000380oooo1000000b
0?ooo`@000006`3oool00080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`020?oo
o`050000003oool000000?ooo`0000005P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool00P3oool01@000000oooo0000003oool0000001H0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`800000103oool01@000000oooo0000
003oool0000001H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`800000
103oool01@000000oooo0000003oool0000001H0oooo00<000000?ooo`3oool00@3oool000<0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0000000N0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0000000N0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0080oooo
00040?ooo`800000303oool3000001`0oooo00<000000?ooo`3oool01P3oool2000000`0oooo0`00
000L0?ooo`030000003oool0oooo00H0oooo0P00000=0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool01P3oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`030?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V
0?ooo`8000002`3oool00`000000oooo0?ooo`090?ooo`<000006P3oool2000000/0oooo00<00000
0?ooo`3oool02@3oool3000001/0oooo00<000000?ooo`3oool0103oool000L0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3o
ool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3o
ool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool000P0oooo0`00000O0?ooo`<00000
4@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<00000
2P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3o
ool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703o
ool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool02@3o
ool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03@3oool3
000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`080?ooo`H000002P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0203oool6000000X0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo1P00000:0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`080?ooo`L000002@3oool00`00
0000oooo0?ooo`0<0?ooo`003P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P0000080?ooo`030000003oool0oooo0080
oooo0P0000080?ooo`030000003oool0oooo01X0oooo0P0000080?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0J0?ooo`800000203oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool00100oooo
00<000000?ooo`3oool00P0000020?ooo`800000103oool400000080oooo1000000O0?ooo`030000
003oool0oooo008000000P3oool2000000D0oooo0`0000020?ooo`@000007`3oool00`000000oooo
0?ooo`0200000080oooo0P0000050?ooo`<000000P3oool4000001l0oooo00<000000?ooo`3oool0
0P0000030?ooo`030000003oool0oooo00<0oooo0`0000020?ooo`80000000<0oooo0000003oool0
3`3oool00100oooo200000060?ooo`L000008@3oool8000000H0oooo1`00000Q0?ooo`@0000000@0
oooo00000000000000001P3oool700000240oooo100000000`3oool0000000000002000000D0oooo
2000000A0?ooo`00403oool00`000000oooo0?ooo`040?ooo`800000103oool2000000H0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo0P000008
0?ooo`030000003oool0oooo0080oooo0P0000070?ooo`8000001`3oool300000180oooo0P000008
0?ooo`030000003oool0oooo0080oooo0P0000070?ooo`8000001`3oool300000180oooo0P000008
0?ooo`030000003oool0oooo0080oooo0P0000080?ooo`030000003oool0oooo00D0oooo0`000005
0?ooo`003@3oool00`000000oooo0?ooo`080?ooo`H000002P3oool00`000000oooo0?ooo`040?oo
o`030000003oool000000140oooo00<000000?ooo`3oool0203oool6000000X0oooo00<000000?oo
o`3oool0103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00P0oooo1P00000:0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`080?oo
o`H000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000D0oooo000<0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`00
2P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool0
4`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo00<000000?ooo`3oool0
2@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?ooo`800000803o
ool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0:0?ooo`800000403oool00`000000oooo0?ooo`0Q0?ooo`800000403oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00T0oooo0P00000@0?ooo`030000003oool0
oooo0240oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0E0?ooo`8000003@3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo0P00000<0?ooo`030000003oool0oooo
00H0oooo00060?ooo`030000003oool0oooo01D0oooo00<000000?ooo`0000003P3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`0000003@3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`040?ooo`00103oool2000001T0oooo0`00000>0?ooo`800000203oool2000001X0oooo
00<000000?ooo`3oool03@3oool2000000P0oooo0P00000J0?ooo`<000003@3oool2000000P0oooo
0P00000K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00`3oool000<0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0000000F0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo000000030?ooo`<00000403oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo01<0oooo00<000000?ooo`0000005P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo01<0oooo00<000000?ooo`000000103oool3000000l0oooo00<0
00000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool04`3oool4000001L0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`@000005`3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo01<0oooo1000000G0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00@3oool001P0oooo100000080?oo
o`030000003oool0oooo02L0oooo100000080?ooo`030000003oool0oooo02L0oooo100000080?oo
o`030000003oool0oooo02L0oooo100000080?ooo`030000003oool0oooo0100oooo000H0?ooo`05
0000003oool000000?ooo`0000001`3oool3000002L0oooo00D000000?ooo`000000oooo00000007
0?ooo`<000009`3oool01@000000oooo0000003oool0000000L0oooo0`00000W0?ooo`050000003o
ool000000?ooo`0000001`3oool300000100oooo000T0?ooo`030000003oool0000003<0oooo00<0
00000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000003oool000000100oooo000T
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo0100oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`003P3oool010000000oooo0000000000020?ooo`<0000000<0oooo0000
000000000P3oool00`000000oooo000000020?ooo`050000003oool000000000003oool00P00000L
0?ooo`040000003oool0000000000080oooo0`0000000`3oool00000000000020?ooo`030000003o
ool000000080oooo00D000000?ooo`00000000000?ooo`02000001`0oooo00@000000?ooo`000000
00000P3oool3000000D0oooo00<000000?ooo`0000000P3oool010000000oooo0000000000020?oo
o`030000003oool0oooo01X0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`000000
00000080oooo00<000000?ooo`0000000P3oool01@000000oooo000000000000oooo008000003`3o
ool000h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`80000000@0
oooo00000000000000000P3oool010000000oooo0?ooo`0000020?ooo`8000006`3oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`<0000000@0oooo00000000000000000P3o
ool020000000oooo0?ooo`000000oooo0000003oool000006`3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000020?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool06P3oool010000000oooo0?ooo`0000020?ooo`040000
003oool0oooo0?ooo`8000000P3oool300000080oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`0=0?ooo`003P3oool01`000000oooo0?ooo`3oool000000?ooo`0000000`3oool0
10000000oooo0?ooo`3oool300000080oooo00@000000?ooo`3oool0oooo0P0000000`3oool00000
0?ooo`0J0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`040000003oool0oooo
0?ooo`<000000P3oool01@000000oooo0?ooo`000000oooo00800000703oool01`000000oooo0?oo
o`3oool000000?ooo`000000103oool00`000000oooo0?ooo`0300000080oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`0J0?ooo`070000003oool0oooo0?ooo`000000oooo0000
00030?ooo`8000000P3oool300000080oooo00@000000?ooo`3oool000000`3oool00`000000oooo
0?ooo`0<0?ooo`003@3oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`040000003o
ool0oooo000000L0oooo00@000000?ooo`3oool000006`3oool3000000030?ooo`000000oooo0080
oooo0P0000020?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool000006`3oool3
000000030?ooo`000000oooo0080oooo0P0000001P3oool00000000000000000oooo000000L0oooo
00<000000?ooo`3oool0703oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`040000
003oool0oooo000000L0oooo00@000000?ooo`3oool000003`3oool000`0oooo0P00000d0?ooo`80
00003@3oool00`000000oooo0?ooo`0T0?ooo`800000=@3oool2000000d0oooo00<000000?ooo`3o
ool0603oool000`0oooo0`00000;0?ooo`8000009P3oool3000000/0oooo0`00000U0?ooo`<00000
2`3oool2000002L0oooo0`00000;0?ooo`<000006@3oool000d0oooo00<000000?ooo`3oool01`3o
ool2000002X0oooo00<000000?ooo`3oool01`3oool200000080oooo00<000000?ooo`3oool09@3o
ool00`000000oooo0?ooo`070?ooo`800000:`3oool00`000000oooo0?ooo`060?ooo`8000000`3o
ool00`000000oooo0?ooo`0H0?ooo`005`3oool2000003@0oooo0P00000d0?ooo`800000=03oool2
000001h0oooo000G0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo01d0oooo00020?ooo`030000003oool0oooo00d0
oooo0P00000O0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool2000001l0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`8000007`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00d0oooo0P00000O0?ooo`030000003oool0oooo0040
oooo00030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`020?ooo`00103oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`060?ooo`8000003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00H0oooo0P00
000=0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01P3oool2000000d0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`09
0?ooo`<000006P3oool2000000/0oooo00<000000?ooo`3oool02@3oool3000001X0oooo0P00000;
0?ooo`030000003oool0oooo00T0oooo0`00000J0?ooo`8000002`3oool00`000000oooo0?ooo`09
0?ooo`<000006`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`80
00008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`80
00008`3oool00`000000oooo0?ooo`070?ooo`00203oool3000001l0oooo0`00000A0?ooo`<00000
7`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000:0?ooo`000P3o
ool2000000L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003o
ool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3o
ool2000000H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool3
000000L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00T0oooo1000000;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`0000000`3oool000000?ooo`090?oo
o`030000003oool0oooo00`0oooo000>0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool06P3oool2000000P0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P0000080?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0J0?ooo`80
0000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool03@3oool00100oooo00<000000?ooo`3oool00P0000090?ooo`<000000P3oool4000001l0oooo
00<000000?ooo`3oool00P0000090?ooo`<000000P3oool4000001l0oooo00<000000?ooo`3oool0
0P0000090?ooo`<000000P3oool4000001l0oooo00<000000?ooo`3oool00P0000090?ooo`<00000
0P3oool2000000030?ooo`000000oooo00l0oooo000A0?ooo`L000001P3oool700000280oooo1`00
00060?ooo`L000008P3oool3000000040?ooo`0000000000000000H0oooo1`00000R0?ooo`<00000
00@0oooo00000000000000001P3oool800000140oooo000@0?ooo`030000003oool0000000<0oooo
0P0000090?ooo`050000003oool0oooo0?ooo`0000002@3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0000000<0oooo0P0000090?ooo`050000003oool0oooo0?ooo`0000002@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0`0000090?ooo`040000003oool0oooo
000000T0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`<000002@3oool0
1@000000oooo0?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool000h0oooo0P000008
0?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180oooo0P000009
0?ooo`050000003oool0oooo0?ooo`000000203oool2000000L0oooo0`00000B0?ooo`8000002@3o
ool010000000oooo0?ooo`3oool2000000L0oooo0P0000070?ooo`<000004P3oool2000000T0oooo
00@000000?ooo`3oool0oooo0P0000080?ooo`030000003oool0oooo00D0oooo0`0000050?ooo`00
3@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000
003oool000000140oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?ooo`3oool0
103oool00`000000oooo0000000A0?ooo`030000003oool0oooo00T0oooo1000000;0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`090?ooo`@00000
2`3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000D0oooo000<0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`002P3oool2
000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2
000001h0oooo00<000000?ooo`3oool04`3oool2000001l0oooo00<000000?ooo`3oool02@3oool0
00T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?ooo`800000803oool30000
0140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0:0?ooo`800000403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00X0
oooo0P00000@0?ooo`030000003oool0oooo0240oooo0P00000@0?ooo`030000003oool0oooo0240
oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`800000303oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo01H0oooo0P00000<0?ooo`030000003oool0oooo00H0oooo
00060?ooo`030000003oool0oooo01D0oooo00<000000?ooo`0000003P3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0180oooo00H000000?ooo`3oool000000?ooo`00000>0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000D0?ooo`800000
0`3oool3000000h0oooo0P0000080?ooo`800000503oool300000080oooo0`00000>0?ooo`800000
203oool2000001@0oooo0P0000050?ooo`030000003oool0oooo00`0oooo0P0000080?ooo`800000
5@3oool3000000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`030?ooo`000`3o
ool00`000000oooo0?ooo`0A0?ooo`8000006@3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0140oooo0P0000020?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0A0?ooo`8000001`3oool3000000l0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0A0?ooo`8000000`3oool010000000oooo0?ooo`3oool3000000l0oooo
00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool04P3oool2000001X0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?ooo`8000006P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0180oooo0P00000J0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool04P3oool2000001X0oooo00<000000?ooo`3oool00@3oool001L0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0@0?ooo`00903oool3000003<0oooo0`00000c0?oo
o`<00000<`3oool300000100oooo000T0?ooo`030000003oool0000003<0oooo00<000000?ooo`00
0000<`3oool00`000000oooo0000000c0?ooo`030000003oool000000100oooo000T0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000
003oool0oooo0100oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool000h0oooo00@000000?ooo`00000000000P3oool300000080oooo00H00000
0?ooo`3oool000000?ooo`0000020?ooo`050000003oool000000000003oool00P00000L0?ooo`04
0000003oool0000000000080oooo0`0000000`3oool00000000000020?ooo`030000003oool00000
0080oooo00D000000?ooo`00000000000?ooo`02000001`0oooo00@000000?ooo`00000000000P3o
ool3000000030?ooo`00000000000080oooo00<000000?ooo`0000000P3oool010000000oooo0000
0000000O0?ooo`040000003oool0000000000080oooo0`0000050?ooo`030000003oool000000080
oooo00D000000?ooo`00000000000?ooo`02000000l0oooo000>0?ooo`040000003oool0oooo0000
0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0300000080oooo00@000000?oo
o`3oool000000P3oool2000001/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00<000000P3oool01@000000oooo0?ooo`000000oooo00<00000
6`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool06P3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo00D0
00000?ooo`3oool000000?ooo`02000000l0oooo000>0?ooo`070000003oool0oooo0?ooo`000000
oooo000000040?ooo`030000003oool0oooo00<000000P3oool010000000oooo0?ooo`3oool20000
01d0oooo00L000000?ooo`3oool0oooo0000003oool0000000D0oooo00<000000?ooo`0000000P00
00020?ooo`040000003oool0oooo0?ooo`8000007@3oool01`000000oooo0?ooo`3oool000000?oo
o`000000103oool00`000000oooo0?ooo`0300000080oooo00L000000?ooo`3oool0oooo0000003o
ool0000001`0oooo00L000000?ooo`3oool0oooo0000003oool0000000<0oooo0P0000020?ooo`<0
00000P3oool010000000oooo0?ooo`3oool3000000l0oooo000<0?ooo`@0000000<0oooo0000003o
ool00P3oool2000000D0oooo00<000000?ooo`3oool01@3oool010000000oooo0?ooo`00000J0?oo
o`@0000000<0oooo0000003oool00P3oool200000080oooo00@000000?ooo`00000000001`3oool0
10000000oooo0?ooo`00000J0?ooo`@0000000<0oooo0000003oool00P3oool200000080oooo00@0
00000?ooo`3oool000001`3oool00`000000oooo00000002000001T0oooo100000000`3oool00000
0?ooo`020?ooo`8000001@3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo000000l0
oooo000<0?ooo`<000002`3oool2000002H0oooo0`00000;0?ooo`<000009@3oool3000000/0oooo
0P00000V0?ooo`<00000303oool3000001T0oooo000=0?ooo`030000003oool0oooo00L0oooo0P00
000Z0?ooo`030000003oool0oooo00L0oooo0P0000020?ooo`030000003oool0oooo02D0oooo00<0
00000?ooo`3oool01`3oool2000002X0oooo00<000000?ooo`3oool01`3oool2000000<0oooo00<0
00000?ooo`3oool0603oool001L0oooo0P00000d0?ooo`800000=03oool2000003@0oooo0P00000N
0?ooo`004P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01d0oooo00020?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`010?ooo`00
0`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001h0oooo00<000000?ooo`3oool0
0P3oool000@0oooo0P00000<0?ooo`<00000703oool00`000000oooo0?ooo`060?ooo`800000303o
ool3000001`0oooo00<000000?ooo`3oool01P3oool2000000`0oooo0`00000L0?ooo`030000003o
ool0oooo00H0oooo0P00000<0?ooo`<00000703oool00`000000oooo0?ooo`030?ooo`001P3oool0
0`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool0
0`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo00@0oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000P0
oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0
oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0
oooo00080?ooo`<000007`3oool300000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo
0`00000A0?ooo`<000007`3oool3000000X0oooo00020?ooo`8000001`3oool00`000000oooo0?oo
o`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P00
00060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo00T0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=
0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00/0oooo00030?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3oool4000000/0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo1000000;0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?oo
o`@000002`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool02@3oool3000000030?ooo`000000oooo00T0oooo00<000000?ooo`3oool0303oool000h0
oooo0P0000080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0J0?ooo`800000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool06P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P0000080?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0=0?ooo`00403oool00`000000oooo
0?ooo`02000000T0oooo0`0000020?ooo`@000007`3oool00`000000oooo0?ooo`02000000T0oooo
0`0000020?ooo`@000007`3oool01@000000oooo0?ooo`3oool0000000T0oooo0`0000020?ooo`@0
00007`3oool01@000000oooo0?ooo`3oool0000000T0oooo0`0000020?ooo`80000000<0oooo0000
003oool03`3oool00140oooo1`0000060?ooo`L000008P3oool7000000H0oooo1`00000R0?ooo`L0
00001P3oool700000280oooo1`0000070?ooo`L000004@3oool00100oooo0`0000020?ooo`<00000
2@3oool200000080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool000000080oooo0`0000090?ooo`8000000P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`0000000P3oool4000000P0oooo0P0000020?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`<00000
203oool2000000<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`003P3o
ool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`8000001`3o
ool300000180oooo0P0000090?ooo`040000003oool0oooo0?ooo`8000001`3oool2000000L0oooo
0`00000B0?ooo`8000002@3oool010000000oooo0?ooo`3oool2000000L0oooo0P0000070?ooo`<0
00004P3oool2000000T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo0`0000050?ooo`003@3oool00`000000oooo0?ooo`090?ooo`@00000
2`3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<000000?ooo`3oool0
2@3oool4000000/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`030000
003oool0oooo00T0oooo1000000;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`000000
4@3oool00`000000oooo0?ooo`090?ooo`@000002`3oool00`000000oooo0?ooo`040?ooo`030000
003oool0000000D0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`050?ooo`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2
000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3oool04`3oool2
000001l0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool20000
0200oooo0`00000A0?ooo`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0:0?ooo`800000403oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo00X0oooo0P00000@0?ooo`030000003oool0oooo0240
oooo0P00000@0?ooo`030000003oool0oooo0240oooo0P0000090?ooo`001`3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0F0?ooo`800000303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo
0P00000<0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`0000003P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0180oooo00H0
00000?ooo`3oool000000?ooo`00000>0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0103oool000@0oooo0P00000D0?ooo`8000000`3oool3000000h0oooo0P0000080?ooo`800000
503oool300000080oooo0`00000>0?ooo`800000203oool2000001@0oooo0P0000050?ooo`030000
003oool0oooo00`0oooo0P0000080?ooo`8000005@3oool3000000<0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0A0?ooo`8000006@3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0140oooo0P0000020?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`8000001`3o
ool3000000l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`8000000`3o
ool010000000oooo0?ooo`3oool3000000l0oooo00<000000?ooo`3oool00P3oool00080oooo00<0
00000?ooo`3oool04P3oool2000001X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0B0?ooo`8000006P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo0P00
000J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool2000001X0oooo00<0
00000?ooo`3oool00@3oool001L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0@
0?ooo`00903oool3000003<0oooo0`00000c0?ooo`<00000<`3oool300000100oooo000T0?ooo`03
0000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`03
0000003oool000000100oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000d0oooo00@000000?oo
o`00000000000P3oool3000000040?ooo`00000000000?ooo`80000000@0oooo0000003oool00000
0P3oool010000000oooo0000000000020?ooo`030000003oool0oooo01P0oooo00@000000?ooo`00
000000000P3oool3000000040?ooo`00000000000?ooo`80000000@0oooo0000003oool000000P3o
ool01@000000oooo000000000000oooo008000006P3oool010000000oooo0000000000020?ooo`<0
000000<0oooo000000000000103oool00`000000oooo000000020?ooo`050000003oool000000000
003oool00P00000J0?ooo`040000003oool0000000000080oooo0`000000103oool000000000003o
ool2000000040?ooo`000000oooo00000080oooo00@000000?ooo`00000000004@3oool000d0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0000
003oool00`0000020?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0603oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool00000
0?ooo`0300000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0H0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool01@000000oooo000000000000
oooo00<000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo01P0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00D00000
0P3oool01@000000oooo0?ooo`000000oooo008000003P3oool000d0oooo00L000000?ooo`3oool0
oooo0000003oool0000000D0oooo00<000000?ooo`3oool0100000020?ooo`070000003oool0oooo
0?ooo`000000oooo0000000J0?ooo`070000003oool0oooo0?ooo`000000oooo000000050?ooo`04
0000003oool000000?ooo`<000000P3oool01@000000oooo0?ooo`3oool000000080oooo00<00000
0?ooo`3oool05`3oool01`000000oooo0?ooo`3oool000000?ooo`0000001@3oool3000000040?oo
o`000000000000000080oooo00L000000?ooo`3oool0oooo0000003oool0000001X0oooo00L00000
0?ooo`3oool0oooo0000003oool0000000D0oooo0P0000020?ooo`<000000P3oool010000000oooo
0?ooo`3oool3000000h0oooo000<0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo
00H000000?ooo`3oool000000?ooo`0000070?ooo`030000003oool0oooo01X0oooo0`0000000`3o
ool000000?ooo`020?ooo`8000000P3oool01P000000oooo0?ooo`000000oooo000000L0oooo00@0
00000?ooo`3oool000006@3oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool01@3oool010000000oooo0?ooo`00000I0?ooo`<0
000000<0oooo0000003oool00P3oool200000080oooo00H000000?ooo`3oool000000?ooo`000007
0?ooo`030000003oool0oooo00l0oooo000<0?ooo`<00000303oool00`000000oooo0?ooo`0T0?oo
o`<00000303oool00`000000oooo0?ooo`0T0?ooo`<00000303oool00`000000oooo0?ooo`0U0?oo
o`<000002`3oool00`000000oooo0?ooo`0I0?ooo`003@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0000002L0oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02H0
oooo00<000000?ooo`3oool0203oool2000000030?ooo`000000oooo02L0oooo00<000000?ooo`3o
ool01`3oool2000000030?ooo`000000oooo01X0oooo000I0?ooo`030000003oool000000380oooo
1000000b0?ooo`80000000<0oooo0000003oool0<@3oool2000000030?ooo`000000oooo01X0oooo
000H0?ooo`<00000=03oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo03@0oooo00<0
00000?ooo`3oool06`3oool00080oooo00<000000?ooo`3oool03@3oool2000000@0oooo0P00000I
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool2000000@0oooo0P00000I
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool2000000@0oooo0P00000I
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool2000001l0oooo00<00000
0?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`020?oo
o`800000603oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool00P3oool2000001P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo0080oooo0P00000H0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`020?ooo`800000603oool00`000000oooo0?ooo`020?oo
o`00103oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`060?ooo`80
00003@3oool00`000000oooo0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`060?ooo`80
00003@3oool00`000000oooo0?ooo`050?ooo`800000503oool00`000000oooo0?ooo`060?ooo`80
00003@3oool00`000000oooo0?ooo`020?ooo`8000005`3oool00`000000oooo0?ooo`030?ooo`00
1P3oool00`000000oooo0?ooo`090?ooo`<000006P3oool2000000/0oooo00<000000?ooo`3oool0
2@3oool3000000H0oooo00<000000?ooo`3oool04@3oool2000000/0oooo00<000000?ooo`3oool0
2@3oool3000000H0oooo00<000000?ooo`3oool04@3oool2000000/0oooo00<000000?ooo`3oool0
2@3oool3000000H0oooo0P00000C0?ooo`030000003oool0oooo00@0oooo00070?ooo`030000003o
ool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`00203oool2
000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2
000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool0
00P0oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000A0?ooo`<000007`3oool30000
0140oooo0`00000O0?ooo`<000002P3oool00080oooo0P0000070?ooo`030000003oool0oooo01`0
oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0
oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo
0`0000060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool000<0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`<00000303oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool3000000`0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo0`00
000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`090?ooo`<00000303oool00`000000oooo0?ooo`0<0?ooo`003P3oool2000000T0oooo00<00000
0?ooo`3oool00P0000090?ooo`030000003oool0oooo01X0oooo0P0000090?ooo`030000003oool0
oooo008000002@3oool00`000000oooo0?ooo`0J0?ooo`8000002@3oool00`000000oooo0?ooo`02
000000T0oooo00<000000?ooo`3oool06P3oool2000000T0oooo00H000000?ooo`3oool000000?oo
o`0000080?ooo`030000003oool0oooo00d0oooo000@0?ooo`050000003oool0oooo0?ooo`000000
0P3oool00`000000oooo0?ooo`030?ooo`030000003oool0000000<0oooo00@000000?ooo`000000
00007`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool00000000001l0oooo00D000000?ooo`3oool0oooo
000000030?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000000`3oool3000001l0
oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`0000000`3oool00`000000oooo0000000@0?ooo`004@3oool7000000H0oooo1`00000R0?ooo`L0
00001P3oool700000280oooo00<000000?ooo`000000100000060?ooo`L000008P3oool00`000000
oooo00000002000000030?ooo`000000oooo00D0oooo2000000A0?ooo`00403oool300000080oooo
0`0000060?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01@0oooo0`0000020?ooo`<000001P3oool00`000000oooo0?ooo`0200000080oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`<000000P3oool3000000H0oooo
00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
503oool300000080oooo0`0000060?ooo`030000003oool0oooo008000000`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00D0oooo000>0?ooo`800000203oool00`000000oooo0?oo
o`0;0?ooo`8000001`3oool300000180oooo0P0000080?ooo`030000003oool0oooo00/0oooo0P00
00070?ooo`<000004P3oool2000000P0oooo00<000000?ooo`3oool02`3oool2000000L0oooo0`00
000B0?ooo`800000203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0oooo00T0oooo1000000;
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`09
0?ooo`D000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo00<00000
0?ooo`3oool02@3oool5000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A
0?ooo`030000003oool0oooo00T0oooo1@00000:0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00D0oooo000;0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?ooo`800000
803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3oool08@3o
ool200000100oooo00<000000?ooo`3oool08@3oool200000100oooo00<000000?ooo`3oool08@3o
ool200000100oooo00<000000?ooo`3oool08@3oool2000000T0oooo00070?ooo`030000003oool0
oooo01D0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05@3oool2
000000d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`800000303oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo0P00000<0?ooo`030000003oool0
oooo00H0oooo00060?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`040?ooo`00103oool2000001D0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0=0?ooo`800000203oool2000001D0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0=0?ooo`800000203oool2000001D0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0<0?ooo`800000203oool2000001H0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`0000000`3oool300000100oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0B0?ooo`80000000<0oooo0000003oool00P3oool300000100oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0B0?ooo`80000000<0oooo0000003oool00`3oool3000000l0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`8000000P3oool010000000oooo
0?ooo`3oool3000000l0oooo00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool0
503oool00`000000oooo0000000G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
4`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`800000
00<0oooo0000003oool05P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo
0P0000020?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool00@3oool001P0oooo0`00
00090?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo0100oooo000H0?ooo`8000002P3oool3000002L0oooo0P00000:
0?ooo`<00000:03oool2000000T0oooo0`00000c0?ooo`<00000403oool001P0oooo0P00000:0?oo
o`030000003oool0000002L0oooo0P00000:0?ooo`030000003oool0000002P0oooo0P0000090?oo
o`030000003oool0000002P0oooo0P0000090?ooo`030000003oool000000100oooo000T0?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0:03oool2000000T0oooo00<000000?ooo`3o
ool0403oool00540oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0d0?ooo`030000
003oool0oooo01P0oooo001A0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0=03o
ool00`000000oooo0?ooo`0H0?ooo`00_P3oool00`000000oooo0?ooo`0H0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0oooo0`00000;0?oo
o`030000003oool0oooo00@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00
001F0?ooo`00E03oool3000000P0oooo0`0000080?ooo`D000001@3oool010000000oooo0?ooo`00
00070?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3000000/0oooo00<0
00000?ooo`3oool0103oool4000000L0oooo1000001F0?ooo`00M03oool010000000oooo0?ooo`00
00070?ooo`040000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N141) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N142) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56777 .50332 L
.56317 .50627 L
.55934 .5091 L
.55653 .51212 L
.5548 .5156 L
.55402 .51976 L
.55389 .52467 L
.55401 .53025 L
.55392 .53628 L
.55322 .54238 L
.55159 .54812 L
.54886 .55304 L
.54504 .55676 L
.54027 .559 L
.53482 .55969 L
.52903 .55892 L
.52324 .55701 L
.51771 .55438 L
.51264 .55157 L
.50807 .5491 L
.50392 .54741 L
.5 .54682 L
.49608 .54741 L
.49193 .5491 L
.48736 .55157 L
.48229 .55438 L
.47676 .55701 L
.47097 .55892 L
.46518 .55969 L
.45973 .559 L
.45496 .55676 L
.45114 .55304 L
.44841 .54812 L
.44678 .54238 L
.44608 .53628 L
.44599 .53025 L
.44611 .52467 L
.44598 .51976 L
.4452 .5156 L
.44347 .51212 L
.44066 .5091 L
.43683 .50627 L
.43223 .50332 L
.42727 .5 L
s
gsave
.5 .60115 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N143) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
gsave
.5 .41022 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56777 .50332 L
.56317 .50627 L
.55934 .5091 L
.55653 .51212 L
.5548 .5156 L
.55402 .51976 L
.55389 .52467 L
.55401 .53025 L
.55392 .53628 L
.55322 .54238 L
.55159 .54812 L
.54886 .55304 L
.54504 .55676 L
.54027 .559 L
.53482 .55969 L
.52903 .55892 L
.52324 .55701 L
.51771 .55438 L
.51264 .55157 L
.50807 .5491 L
.50392 .54741 L
.5 .54682 L
.49608 .54741 L
.49193 .5491 L
.48736 .55157 L
.48229 .55438 L
.47676 .55701 L
.47097 .55892 L
.46518 .55969 L
.45973 .559 L
.45496 .55676 L
.45114 .55304 L
.44841 .54812 L
.44678 .54238 L
.44608 .53628 L
.44599 .53025 L
.44611 .52467 L
.44598 .51976 L
.4452 .5156 L
.44347 .51212 L
.44066 .5091 L
.43683 .50627 L
.43223 .50332 L
.42727 .5 L
s
gsave
.5 .60115 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N144) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.47273 .44182 m
.52727 .42364 L
.52727 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.52727 .55818 m
.47273 .57636 L
.47273 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N145) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.42727 .5 m
.42273 .50435 L
.41818 .50803 L
.41364 .5105 L
.40909 .51136 L
.40455 .5105 L
.4 .50803 L
.39545 .50435 L
.39091 .5 L
.38636 .49565 L
.38182 .49196 L
.37727 .4895 L
.37273 .48863 L
.36818 .4895 L
.36364 .49196 L
.35909 .49565 L
.35455 .5 L
.35 .50435 L
.34545 .50803 L
.34091 .5105 L
.33636 .51136 L
.33182 .5105 L
.32727 .50803 L
.32273 .50435 L
.31818 .5 L
s
gsave
.37273 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.57273 .5 m
.57727 .49565 L
.58182 .49197 L
.58636 .4895 L
.59091 .48864 L
.59545 .4895 L
.6 .49197 L
.60455 .49565 L
.60909 .5 L
.61364 .50435 L
.61818 .50804 L
.62273 .5105 L
.62727 .51137 L
.63182 .5105 L
.63636 .50804 L
.64091 .50435 L
.64545 .5 L
.65 .49565 L
.65455 .49197 L
.65909 .4895 L
.66364 .48864 L
.66818 .4895 L
.67273 .49197 L
.67727 .49565 L
.68182 .5 L
s
gsave
.62727 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.5 .51636 .07455 192.68 347.32 arc
s
.52727 .44182 m
.47273 .42364 L
.47273 .46 L
F
gsave
.5 .39885 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.57273 .5 m
.56784 .50334 L
.56333 .50632 L
.55959 .5092 L
.55686 .51228 L
.5552 .51583 L
.55444 .52003 L
.55426 .52494 L
.55422 .53043 L
.55386 .53621 L
.55275 .54186 L
.55059 .54688 L
.54728 .5508 L
.54293 .55333 L
.53786 .55449 L
.53255 .55472 L
.5274 .5547 L
.52261 .55504 L
.51808 .55585 L
.51362 .55681 L
.50912 .55758 L
.50457 .55804 L
.5 .55818 L
.49543 .55804 L
.49088 .55758 L
.48638 .55681 L
.48192 .55585 L
.47739 .55504 L
.4726 .5547 L
.46745 .55472 L
.46214 .55449 L
.45707 .55333 L
.45272 .5508 L
.44941 .54688 L
.44725 .54186 L
.44614 .53621 L
.44578 .53043 L
.44574 .52494 L
.44556 .52003 L
.4448 .51583 L
.44314 .51228 L
.44041 .5092 L
.43667 .50632 L
.43216 .50334 L
.42727 .5 L
s
.47273 .55818 m
.52727 .57636 L
.52727 .54 L
F
gsave
.5 .60115 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.68182 .5 Mdot
.42727 .5 Mdot
.57273 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N146) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000>0?ooo`040000003oool0
000000000080oooo0`0000000`3oool00000000000020?ooo`030000003oool000000080oooo00<0
00000?ooo`3oool00P3oool2000001/0oooo00@000000?ooo`00000000000P3oool300000080oooo
00P000000?ooo`00000000000?ooo`000000oooo00000080oooo00<000000?ooo`3oool00P3oool2
000007X0oooo000>0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool2
000000040?ooo`000000000000000080oooo00@000000?ooo`00000000000P3oool2000001X0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool000000?oo
o`@000000P3oool01@000000oooo000000000000oooo00<00000N@3oool000h0oooo00L000000?oo
o`3oool0oooo0000003oool0000000<0oooo00@000000?ooo`000000oooo0`0000020?ooo`060000
003oool000000000003oool00000703oool01`000000oooo0?ooo`3oool000000?ooo`000000103o
ool01@000000oooo000000000000oooo00<000000P3oool01P000000oooo000000000000oooo0000
07/0oooo000<0?ooo`@0000000<0oooo0000003oool00P3oool200000080oooo00@000000?ooo`3o
ool000002P3oool00`000000oooo0?ooo`0H0?ooo`<0000000<0oooo0000003oool00P3oool20000
00D0oooo00<000000?ooo`0000002P3oool00`000000oooo0?ooo`1h0?ooo`00303oool3000003<0
oooo0`00002D0?ooo`003@3oool00`000000oooo0?ooo`080?ooo`@000009`3oool00`000000oooo
0?ooo`080?ooo`@00000Q`3oool001P0oooo00@000000?ooo`0000000000<P3oool010000000oooo
0000000000270?ooo`00603oool00`000000oooo0000000c0?ooo`030000003oool0000008P0oooo
00020?ooo`030000003oool0oooo00d0oooo0P0000050?ooo`800000603oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00d0oooo0P0000050?ooo`800000603oool00`000000oooo0?oo
o`1]0?ooo`000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool0KP3oool000@0oooo0P00000=0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool01P3oool2000000d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`1_0?oo
o`001P3oool00`000000oooo0?ooo`090?ooo`<000006P3oool2000000/0oooo00<000000?ooo`3o
ool02@3oool3000001X0oooo0P00001b0?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`1b0?ooo`00203o
ool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0L`3o
ool000P0oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00001f0?ooo`000P3oool20000
00L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo
01`0oooo0P00001g0?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0=0?ooo`<000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo07L0
oooo00030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3oool4000000/0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo
1000000;0?ooo`030000003oool0oooo07P0oooo000>0?ooo`8000002@3oool01@000000oooo0?oo
o`3oool0000000T0oooo00<000000?ooo`3oool06P3oool2000000T0oooo00D000000?ooo`3oool0
oooo000000090?ooo`030000003oool0oooo07T0oooo000@0?ooo`030000003oool0oooo00800000
0P3oool00`000000oooo0?ooo`030?ooo`040000003oool0000000000080oooo1000000O0?ooo`03
0000003oool0oooo008000000`3oool00`000000oooo0?ooo`020?ooo`040000003oool000000000
0080oooo1000001l0?ooo`004@3oool7000000H0oooo1`00000R0?ooo`L000001P3oool7000007h0
oooo000@0?ooo`<000000P3oool4000000@0oooo0P0000020?ooo`80000000<0oooo000000000000
2@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool000000080oooo0P0000000`3oool00000
0?ooo`040?ooo`030000003oool0oooo0080000000<0oooo0000000000002@3oool00`000000oooo
0?ooo`1a0?ooo`003P3oool2000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`060?ooo`8000001`3oool300000180oooo0P0000080?ooo`030000003oool0oooo0080oooo0P00
00070?ooo`8000001`3oool300000740oooo000=0?ooo`030000003oool0oooo00P0oooo1P00000:
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`08
0?ooo`H000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool000000740oooo000<0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1a0?oo
o`002P3oool2000001h0oooo00<000000?ooo`3oool04`3oool2000001h0oooo00<000000?ooo`3o
ool0MP3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00001e0?ooo`00203o
ool00`000000oooo0?ooo`0Q0?ooo`800000403oool00`000000oooo0?ooo`0Q0?ooo`800000M@3o
ool000L0oooo00<000000?ooo`3oool05@3oool2000000d0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0E0?ooo`8000003@3oool00`000000oooo0?ooo`1b0?ooo`001P3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0L@3oool000@0
oooo0P00000J0?ooo`030000003oool0oooo00d0oooo0P0000080?ooo`8000006P3oool00`000000
oooo0?ooo`0=0?ooo`800000L@3oool000<0oooo00<000000?ooo`3oool04`3oool00`000000oooo
000000030?ooo`<00000403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`0000000`3oool300000100oooo00<000000?ooo`3oool0KP3oool00080oooo00<0
00000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0C0?ooo`@000005`3oool00`000000oooo0?ooo`1]0?ooo`00603oool4000000P0oooo00<00000
0?ooo`3oool09`3oool4000000P0oooo00<000000?ooo`3oool0O03oool001P0oooo00D000000?oo
o`000000oooo000000070?ooo`<000009`3oool01@000000oooo0000003oool0000000L0oooo0`00
001l0?ooo`00903oool00`000000oooo0000000c0?ooo`030000003oool0000007`0oooo000T0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0O03oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`003P3oool010000000oooo0000
000000020?ooo`<0000000<0oooo0000000000000P3oool00`000000oooo000000020?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool06P3oool010000000oooo0000000000020?ooo`<0
000000<0oooo0000000000000P3oool00`000000oooo000000020?ooo`030000003oool0oooo0080
oooo0P00000L0?ooo`040000003oool0000000000080oooo0`0000050?ooo`030000003oool00000
0080oooo00<000000?ooo`3oool00P3oool2000001`0oooo00@000000?ooo`00000000000P3oool3
000000030?ooo`00000000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0C
0?ooo`003P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo0P000000
103oool000000000000000020?ooo`040000003oool0000000000080oooo00<000000?ooo`3oool0
6P3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`<0000000@0oooo0000
0000000000000P3oool010000000oooo0000000000020?ooo`030000003oool0oooo01X0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<00000
0P3oool010000000oooo0000000000020?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3o
ool000000P3oool010000000oooo0?ooo`3oool200000080oooo0`0000020?ooo`050000003oool0
00000000003oool00P00000?0?ooo`003P3oool01`000000oooo0?ooo`3oool000000?ooo`000000
0`3oool010000000oooo0?ooo`3oool300000080oooo00@000000?ooo`00000000000P3oool00`00
0000oooo0?ooo`0J0?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`040000003o
ool0oooo0?ooo`<000000P3oool010000000oooo0000000000030?ooo`030000003oool0oooo01T0
oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`3oool00`000002
0?ooo`040000003oool0000000000080oooo00<000000?ooo`3oool06P3oool01`000000oooo0?oo
o`3oool000000?ooo`0000000`3oool200000080oooo0`0000020?ooo`050000003oool000000000
003oool00P00000?0?ooo`00303oool4000000030?ooo`000000oooo0080oooo0P0000020?ooo`04
0000003oool0oooo000002D0oooo100000000`3oool000000?ooo`020?ooo`8000000P3oool01000
0000oooo0?ooo`00000:0?ooo`030000003oool0oooo01P0oooo100000000`3oool000000?ooo`02
0?ooo`80000000H0oooo00000000000000000?ooo`00000:0?ooo`030000003oool0oooo01T0oooo
0`0000000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`00000J0?ooo`00
303oool3000003<0oooo0`00000c0?ooo`<00000=03oool3000002L0oooo000=0?ooo`030000003o
ool0oooo00P0oooo1000000W0?ooo`030000003oool0oooo00P0oooo1000000W0?ooo`030000003o
ool0oooo03@0oooo00<000000?ooo`3oool09P3oool001P0oooo00@000000?ooo`0000000000<P3o
ool010000000oooo00000000000b0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0
7@3oool001P0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`800000<`3o
ool3000001d0oooo00020?ooo`030000003oool0oooo00d0oooo0P0000050?ooo`800000603oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0oooo0P0000050?ooo`800000603oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0oooo0P0000040?ooo`80000000<0oooo
0000000000005P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0oooo0P000003
0?ooo`030000003oool000000080oooo0P00000E0?ooo`030000003oool0oooo0040oooo00030?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020000
01D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`050000003oool0oooo
0?ooo`000000103oool2000001@0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000=0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool01P3oool2000000d0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`050?oo
o`800000503oool00`000000oooo0?ooo`060?ooo`8000003@3oool00`000000oooo0?ooo`060?oo
o`8000004`3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`090?ooo`<0
00006P3oool2000000/0oooo00<000000?ooo`3oool02@3oool3000001X0oooo0P00000;0?ooo`03
0000003oool0oooo00T0oooo0`00000J0?ooo`8000002`3oool00`000000oooo0?ooo`090?ooo`<0
00006`3oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3o
ool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3o
ool00`000000oooo0?ooo`070?ooo`00203oool3000001l0oooo0`00000A0?ooo`<000007`3oool3
00000140oooo0`00000O0?ooo`<000004@3oool3000001l0oooo0`00000:0?ooo`000P3oool20000
00L0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo
01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool20000
00H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`000P3oool3000000L0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`<000001P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool03@3oool3000000H0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0;0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00T0oooo
0`00000<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`090?ooo`<00000303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool02P3oool2000000`0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00X0oooo0P00000<0?ooo`030000003oool0oooo00`0oooo
000>0?ooo`8000002@3oool00`000000oooo0?ooo`02000000T0oooo00<000000?ooo`3oool06P3o
ool2000000T0oooo00<000000?ooo`3oool00P0000090?ooo`030000003oool0oooo01X0oooo0P00
00090?ooo`030000003oool0oooo008000002@3oool00`000000oooo0?ooo`0J0?ooo`8000002@3o
ool01P000000oooo0?ooo`000000oooo000000P0oooo00<000000?ooo`3oool03@3oool00100oooo
00D000000?ooo`3oool0oooo000000020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`00
00000`3oool010000000oooo00000000000O0?ooo`050000003oool0oooo0?ooo`0000000`3oool0
0`000000oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo00@000000?ooo`0000000000
7`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool00P3oool010000000
oooo0?ooo`0000030?ooo`<000007`3oool01@000000oooo0?ooo`3oool0000000<0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo000000030?ooo`030000003oool000000100oooo000A0?oo
o`L000001P3oool700000280oooo1`0000060?ooo`L000008P3oool00`000000oooo000000040000
00H0oooo1`00000R0?ooo`030000003oool000000080000000<0oooo0000003oool01@3oool80000
0140oooo000@0?ooo`<000000P3oool4000000@0oooo0P0000020?ooo`80000000<0oooo00000000
00002@3oool00`000000oooo0?ooo`0D0?ooo`<000000P3oool4000000D0oooo00<000000?ooo`3o
ool00P0000000`3oool00000000000090?ooo`030000003oool0oooo01@0oooo0`0000020?ooo`@0
00001@3oool00`000000oooo0?ooo`02000000030?ooo`000000000000T0oooo00<000000?ooo`3o
ool0503oool300000080oooo0P0000000`3oool000000?ooo`040?ooo`030000003oool0oooo0080
000000@0oooo0000003oool00000203oool00`000000oooo0?ooo`050?ooo`003P3oool2000000P0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`8000001`3oool300000180
oooo0P0000080?ooo`030000003oool0oooo0080oooo0P0000070?ooo`8000001`3oool300000180
oooo0P0000080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool2000000L0
oooo0`00000B0?ooo`800000203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool01@3oool3000000D0oooo000=0?ooo`030000003oool0oooo00P0oooo
1P00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004@3oool00`000000oooo
0?ooo`080?ooo`H000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo
00<000000?ooo`3oool0203oool6000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo
0000000A0?ooo`030000003oool0oooo00P0oooo1`0000090?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`0000001@3oool000`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00D0oooo000;0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool02@3oool000T0oooo0P00000P0?ooo`8000004P3oool200000200oooo0`00000A0?oo
o`800000803oool300000140oooo0P00000Q0?ooo`8000002@3oool000P0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0:0?ooo`800000403oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo00X0oooo0P00000@0?ooo`030000003oool0oooo0240oooo0P00000@0?ooo`03
0000003oool0oooo0240oooo0P0000090?ooo`001`3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`800000
303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo0P00000<0?ooo`030000
003oool0oooo00H0oooo00060?ooo`030000003oool0oooo01D0oooo00<000000?ooo`0000003P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`0000003P3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`040?ooo`00103oool2000001T0oooo0`00000>0?ooo`800000203oool2
000001T0oooo0`00000>0?ooo`800000203oool2000001/0oooo00<000000?ooo`3oool0303oool2
000000P0oooo0P00000K0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00`3oool0
00<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0000000F0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0000000F0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool04`3oool2000000D0oooo0`00000?0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool04`3oool2000000D0oooo0`00000?0?ooo`030000003oool0oooo
0080oooo00020?ooo`030000003oool0oooo01<0oooo1000000G0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool04`3oool4000001L0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool00@3oool001P0oooo
100000080?ooo`030000003oool0oooo02L0oooo100000080?ooo`030000003oool0oooo02T0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0403oool001P0oooo00D000000?ooo`000000oooo000000070?ooo`<00000
9`3oool01@000000oooo0000003oool0000000L0oooo0`00000X0?ooo`<00000203oool3000002P0
oooo0`0000080?ooo`<00000403oool002@0oooo00<000000?ooo`000000<`3oool00`000000oooo
0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000403oool002@0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0403oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool005@0oooo0`0000080?ooo`<000002`3oool00`000000oooo
0?ooo`040?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool00000EP3oool005@0
oooo0`0000080?ooo`<00000203oool5000000D0oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`030000003oool0oooo
00@0oooo100000070?ooo`@00000EP3oool007@0oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0?ooo`00001F0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(self\  = \ 
      CalcFeynAmp[\n\ \ CreateFeynAmp[ins], \n\ \ 
        Select[counter, \ DiagramType[#]\  == \ 2\ &]]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 29 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 29 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"10 Generic, 117 Classes \
amplitudes"\),
        SequenceForm[ 
        "> Top. ", 2, ": ", "10 Generic, 117 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"12 Generic, 146 Classes amplitudes"\),
        
        SequenceForm[ "in total: ", "12 Generic, 146 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]13
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 13, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]0
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 0, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell["running FORM... ", "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm
    \`\((Amp({ME, ME} \[Rule] {MHp, MHp}))\) (
      \(\((\((\((\(4\ Alfa2\)\/3 - 8\ Alfa\ dZAA1\ \[Pi])\)\ S - 
                    8\ Alfa2\ \(A0(MW2)\) - 
                    4\ Alfa2\ \((A0(MHp2) + A0(MW2))\) - 
                    4\ Alfa2\ MW2\ \(B0(S, MW2, MW2)\) + 
                    \((4\ Alfa2\ MW2 + 10\ Alfa2\ S)\)\ \(B0(S, MW2, MW2)\) + 
                    16\ Alfa2\ \(B00(S, MW2, MW2)\) + 
                    8\ Alfa2\ \((B00(S, MHp2, MHp2) + B00(S, MW2, MW2))\) + 
                    \((8\ Alfa2\ MHp2 - 4\ Alfa2\ \((T + U)\))\)\ 
                      \(B1(S, MW2, MW2)\))\)\ \(Den(S, 0)\)\^2 + 
              \((\(-\(\(6\ Alfa\ dZZA1\ \[Pi]\ SW2\ MZ2\)\/\(CW\ SW\)\)\) + 
                    \(2\ Alfa\ CW2\ dZZA1\ \[Pi]\ MZ2\)\/\(CW\ SW\) + 
                    \(2\ Alfa\ dZZA1\ \[Pi]\ MZ2\)\/\(CW\ SW\) + 
                    S\ \((\(2\ CW2\ Alfa2\)\/\(3\ SW2\) + 
                        \(2\ Alfa2\)\/\(3\ SW2\) - 2\ Alfa2 + 
                        \(6\ Alfa\ dZAZ1\ \[Pi]\ SW2\)\/\(CW\ SW\) + 
                        \(6\ Alfa\ dZZA1\ \[Pi]\ SW2\)\/\(CW\ SW\) - 
                        \(2\ Alfa\ CW2\ dZAZ1\ \[Pi]\)\/\(CW\ SW\) - 
                        \(2\ Alfa\ dZAZ1\ \[Pi]\)\/\(CW\ SW\) - 
                        \(2\ Alfa\ CW2\ dZZA1\ \[Pi]\)\/\(CW\ SW\) - 
                        \(2\ Alfa\ dZZA1\ \[Pi]\)\/\(CW\ SW\))\) + 
                    \((\(-\(\(4\ CW2\ Alfa2\)\/SW2\)\) - \(4\ Alfa2\)\/SW2 + 
                          12\ Alfa2)\)\ \(A0(MW2)\) + 
                    \((\(-\(\(3\ SW2\ Alfa2\)\/CW2\)\) + Alfa2\/CW2 - 
                          \(CW2\ Alfa2\)\/SW2 - Alfa2\/SW2 + 4\ Alfa2)\)\ 
                      \((A0(MHp2) + A0(MW2))\) + 
                    \((\(-6\)\ Alfa2\ MW2 + \(2\ Alfa2\ MW2\)\/SW2 + 
                          \(2\ Alfa2\ CW2\ MW2\)\/SW2 + 
                          S\ \((\(5\ CW2\ Alfa2\)\/SW2 + \(5\ Alfa2\)\/SW2 - 
                              15\ Alfa2)\))\)\ \(B0(S, MW2, MW2)\) + 
                    \((2\ Alfa2\ MW2 - \(6\ Alfa2\ SW2\ MW2\)\/CW2 + 
                          \(2\ Alfa2\ MW2\)\/CW2)\)\ \(B0(S, MW2, MW2)\) + 
                    \((\(8\ CW2\ Alfa2\)\/SW2 + \(8\ Alfa2\)\/SW2 - 24\ Alfa2)
                        \)\ \(B00(S, MW2, MW2)\) + 
                    \((\(6\ SW2\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/CW2 + 
                          \(2\ CW2\ Alfa2\)\/SW2 + \(2\ Alfa2\)\/SW2 - 
                          8\ Alfa2)\)\ 
                      \((B00(S, MHp2, MHp2) + B00(S, MW2, MW2))\) + 
                    \((\(-12\)\ Alfa2\ MHp2 + \(4\ Alfa2\ MHp2\)\/SW2 + 
                          \(4\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                          \((\(-\(\(2\ CW2\ Alfa2\)\/SW2\)\) - 
                                \(2\ Alfa2\)\/SW2 + 6\ Alfa2)\)\ \((T + U)\))
                        \)\ \(B1(S, MW2, MW2)\))\)\ \(Den(S, MZ2)\)\ 
                \(Den(S, 0)\) + 
              \((\(4\ Alfa\ \[Pi]\ SW2\ dMZsq1\)\/CW2 + 
                    \(2\ Alfa\ \[Pi]\ dMZsq1\)\/SW2 - 
                    4\ Alfa\ \[Pi]\ dMZsq1 - 
                    \(2\ Alfa\ \[Pi]\ dMZsq1\)\/CW2 + 
                    \(4\ Alfa\ dZZZ1\ MZ2\ \[Pi]\ SW2\)\/CW2 + 
                    S\ \((\(-\(\(2\ CW2\ Alfa2\)\/\(3\ SW2\)\)\) - 
                        Alfa2\/\(3\ SW2\) + \(CW2\ Alfa2\)\/\(3\ SW2\^2\) + 
                        \(2\ Alfa2\)\/3 - 
                        \(4\ Alfa\ dZZZ1\ \[Pi]\ SW2\)\/CW2 - 
                        \(2\ Alfa\ dZZZ1\ \[Pi]\)\/SW2 + 
                        4\ Alfa\ dZZZ1\ \[Pi] + 
                        \(2\ Alfa\ dZZZ1\ \[Pi]\)\/CW2)\) + 
                    \((\(4\ CW2\ Alfa2\)\/SW2 + \(2\ Alfa2\)\/SW2 - 
                          \(2\ CW2\ Alfa2\)\/SW2\^2 - 4\ Alfa2)\)\ 
                      \(A0(MW2)\) + 
                    \((\(-\(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                          \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                          \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - \(3\ Alfa2\)\/2 - 
                          \(3\ Alfa2\)\/\(4\ CW2\) + 
                          \(3\ Alfa2\)\/\(4\ SW2\) + 
                          \(Alfa2\ CW2\)\/\(2\ SW2\) - 
                          \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\)\ 
                      \((A0(MHp2) + A0(MW2))\) + 
                    \((Alfa2\/\(4\ CW2\ SW2\) + Alfa2\/\(8\ CW2\^2\ SW2\) - 
                          Alfa2\/\(4\ CW2\^2\) - Alfa2\/\(8\ CW2\ SW2\^2\))
                        \)\ \((A0(MA02) + A0(Mh02) + A0(MHH2) + A0(MZ2))\) + 
                    \((\(-\(\(Alfa2\ MW2\ SBA\^2\)\/CW2\^3\)\) + 
                          \(Alfa2\ MW2\ SBA\^2\)\/\(CW2\^2\ SW2\) + 
                          \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\^3\ SW2\) - 
                          \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\^2\ SW2\^2\))\)\ 
                      \(B0(S, Mh02, MZ2)\) + 
                    \((\(-\(\(Alfa2\ MW2\ CBA\^2\)\/CW2\^3\)\) + 
                          \(Alfa2\ MW2\ CBA\^2\)\/\(CW2\^2\ SW2\) + 
                          \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\^3\ SW2\) - 
                          \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\^2\ SW2\^2\))\)\ 
                      \(B0(S, MHH2, MZ2)\) + 
                    \((2\ Alfa2\ MW2 - \(Alfa2\ MW2\)\/SW2 - 
                          \(2\ Alfa2\ CW2\ MW2\)\/SW2 + 
                          \(Alfa2\ CW2\ MW2\)\/SW2\^2 + 
                          S\ \((\(-\(\(5\ CW2\ Alfa2\)\/SW2\)\) - 
                              \(5\ Alfa2\)\/\(2\ SW2\) + 
                              \(5\ CW2\ Alfa2\)\/\(2\ SW2\^2\) + 5\ Alfa2)\))
                        \)\ \(B0(S, MW2, MW2)\) + 
                    \((\(-\(\(2\ Alfa2\ MW2\ SW2\^2\)\/CW2\^2\)\) + 
                          \(2\ Alfa2\ MW2\ SW2\)\/CW2 + 
                          \(Alfa2\ MW2\ SW2\)\/CW2\^2 - \(Alfa2\ MW2\)\/CW2)
                        \)\ \(B0(S, MW2, MW2)\) + 
                    \((\(3\ Alfa2\)\/\(CW2\ SW2\) + 
                          \(3\ Alfa2\)\/\(2\ CW2\^2\ SW2\) - 
                          \(3\ Alfa2\)\/CW2\^2 - 
                          \(3\ Alfa2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                      \(B00(S, 0, 0)\) + 
                    \((\(-\(\(Alfa2\ SBA\^2\)\/\(CW2\ SW2\)\)\) - 
                          \(Alfa2\ SBA\^2\)\/\(2\ CW2\^2\ SW2\) + 
                          \(Alfa2\ SBA\^2\)\/CW2\^2 + 
                          \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                      \((B00(S, Mh02, MZ2) + B00(S, MHH2, MA02))\) + 
                    \((\(-\(\(Alfa2\ CBA\^2\)\/\(CW2\ SW2\)\)\) - 
                          \(Alfa2\ CBA\^2\)\/\(2\ CW2\^2\ SW2\) + 
                          \(Alfa2\ CBA\^2\)\/CW2\^2 + 
                          \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                      \((B00(S, Mh02, MA02) + B00(S, MHH2, MZ2))\) + 
                    \((\(-\(\(8\ CW2\ Alfa2\)\/SW2\)\) - \(4\ Alfa2\)\/SW2 + 
                          \(4\ CW2\ Alfa2\)\/SW2\^2 + 8\ Alfa2)\)\ 
                      \(B00(S, MW2, MW2)\) + 
                    \((\(Alfa2\ SW2\^2\)\/CW2\^2 - \(3\ Alfa2\ SW2\)\/CW2 - 
                          \(Alfa2\ SW2\)\/\(2\ CW2\^2\) + 3\ Alfa2 + 
                          \(3\ Alfa2\)\/\(2\ CW2\) - 
                          \(3\ Alfa2\)\/\(2\ SW2\) - \(Alfa2\ CW2\)\/SW2 + 
                          \(Alfa2\ CW2\)\/\(2\ SW2\^2\))\)\ 
                      \((B00(S, MHp2, MHp2) + B00(S, MW2, MW2))\) + 
                    \((\(-\(\(3\ Alfa2\ MHp2\)\/\(CW2\ SW2\)\)\) - 
                          \(3\ Alfa2\ MHp2\)\/\(2\ CW2\^2\ SW2\) + 
                          \(3\ Alfa2\ MHp2\)\/CW2\^2 + 
                          \(3\ Alfa2\ MHp2\)\/\(2\ CW2\ SW2\^2\) + 
                          \((\(3\ Alfa2\)\/\(2\ CW2\ SW2\) + 
                                \(3\ Alfa2\)\/\(4\ CW2\^2\ SW2\) - 
                                \(3\ Alfa2\)\/\(2\ CW2\^2\) - 
                                \(3\ Alfa2\)\/\(4\ CW2\ SW2\^2\))\)\ 
                            \((T + U)\))\)\ \(B1(S, 0, 0)\) + 
                    \((4\ Alfa2\ MHp2 - \(2\ Alfa2\ MHp2\)\/SW2 - 
                          \(4\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                          \(2\ Alfa2\ CW2\ MHp2\)\/SW2\^2 + 
                          \((\(2\ CW2\ Alfa2\)\/SW2 + Alfa2\/SW2 - 
                                \(CW2\ Alfa2\)\/SW2\^2 - 2\ Alfa2)\)\ 
                            \((T + U)\))\)\ \(B1(S, MW2, MW2)\) + 
                    \(2\ Alfa\ dZZZ1\ MZ2\ \[Pi]\)\/SW2 - 
                    4\ Alfa\ dZZZ1\ MZ2\ \[Pi] - 
                    \(2\ Alfa\ dZZZ1\ MZ2\ \[Pi]\)\/CW2)\)\ 
                \(Den(S, MZ2)\)\^2)\)\ \(Mat(F1)\) + 
        \((\((\((\(4\ Alfa2\)\/3 - 8\ Alfa\ dZAA1\ \[Pi])\)\ S - 
                    8\ Alfa2\ \(A0(MW2)\) - 
                    4\ Alfa2\ \((A0(MHp2) + A0(MW2))\) - 
                    4\ Alfa2\ MW2\ \(B0(S, MW2, MW2)\) + 
                    \((4\ Alfa2\ MW2 + 10\ Alfa2\ S)\)\ \(B0(S, MW2, MW2)\) + 
                    16\ Alfa2\ \(B00(S, MW2, MW2)\) + 
                    8\ Alfa2\ \((B00(S, MHp2, MHp2) + B00(S, MW2, MW2))\) + 
                    \((8\ Alfa2\ MHp2 - 4\ Alfa2\ \((T + U)\))\)\ 
                      \(B1(S, MW2, MW2)\))\)\ \(Den(S, 0)\)\^2 + 
              \((\(-\(\(4\ Alfa\ dZZA1\ \[Pi]\ SW\ MZ2\)\/CW\)\) - 
                    \(2\ Alfa\ dZZA1\ \[Pi]\ SW2\ MZ2\)\/\(CW\ SW\) + 
                    \(2\ Alfa\ CW2\ dZZA1\ \[Pi]\ MZ2\)\/\(CW\ SW\) + 
                    S\ \((\(2\ CW2\ Alfa2\)\/\(3\ SW2\) - 2\ Alfa2 + 
                        \(4\ Alfa\ dZAZ1\ \[Pi]\ SW\)\/CW + 
                        \(4\ Alfa\ dZZA1\ \[Pi]\ SW\)\/CW + 
                        \(2\ Alfa\ dZAZ1\ \[Pi]\ SW2\)\/\(CW\ SW\) + 
                        \(2\ Alfa\ dZZA1\ \[Pi]\ SW2\)\/\(CW\ SW\) - 
                        \(2\ Alfa\ CW2\ dZAZ1\ \[Pi]\)\/\(CW\ SW\) - 
                        \(2\ Alfa\ CW2\ dZZA1\ \[Pi]\)\/\(CW\ SW\))\) + 
                    \((12\ Alfa2 - \(4\ Alfa2\ CW2\)\/SW2)\)\ \(A0(MW2)\) + 
                    \((\(-\(\(3\ SW2\ Alfa2\)\/CW2\)\) - 
                          \(CW2\ Alfa2\)\/SW2 + 4\ Alfa2)\)\ 
                      \((A0(MHp2) + A0(MW2))\) + 
                    \((\(-6\)\ Alfa2\ MW2 + \(2\ Alfa2\ CW2\ MW2\)\/SW2 + 
                          S\ \((\(5\ Alfa2\ CW2\)\/SW2 - 15\ Alfa2)\))\)\ 
                      \(B0(S, MW2, MW2)\) + 
                    \((2\ Alfa2\ MW2 - \(6\ Alfa2\ MW2\ SW2\)\/CW2)\)\ 
                      \(B0(S, MW2, MW2)\) + 
                    \((\(8\ Alfa2\ CW2\)\/SW2 - 24\ Alfa2)\)\ 
                      \(B00(S, MW2, MW2)\) + 
                    \((\(6\ SW2\ Alfa2\)\/CW2 + \(2\ CW2\ Alfa2\)\/SW2 - 
                          8\ Alfa2)\)\ 
                      \((B00(S, MHp2, MHp2) + B00(S, MW2, MW2))\) + 
                    \((\(-12\)\ Alfa2\ MHp2 + \(4\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                          \((6\ Alfa2 - \(2\ Alfa2\ CW2\)\/SW2)\)\ 
                            \((T + U)\))\)\ \(B1(S, MW2, MW2)\))\)\ 
                \(Den(S, MZ2)\)\ \(Den(S, 0)\) + 
              \((\(4\ Alfa\ \[Pi]\ SW2\ dMZsq1\)\/CW2 - 
                    4\ Alfa\ \[Pi]\ dMZsq1 + 
                    \(4\ Alfa\ dZZZ1\ MZ2\ \[Pi]\ SW2\)\/CW2 + 
                    S\ \((\(-\(\(2\ CW2\ Alfa2\)\/\(3\ SW2\)\)\) + 
                        \(2\ Alfa2\)\/3 - 
                        \(4\ Alfa\ dZZZ1\ \[Pi]\ SW2\)\/CW2 + 
                        4\ Alfa\ dZZZ1\ \[Pi])\) + 
                    \((\(4\ Alfa2\ CW2\)\/SW2 - 4\ Alfa2)\)\ \(A0(MW2)\) + 
                    \((\(-\(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                          \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - \(3\ Alfa2\)\/2 + 
                          \(Alfa2\ CW2\)\/\(2\ SW2\))\)\ 
                      \((A0(MHp2) + A0(MW2))\) + 
                    \((Alfa2\/\(4\ CW2\ SW2\) - Alfa2\/\(4\ CW2\^2\))\)\ 
                      \((A0(MA02) + A0(Mh02) + A0(MHH2) + A0(MZ2))\) + 
                    \((\(Alfa2\ MW2\ SBA\^2\)\/\(CW2\^2\ SW2\) - 
                          \(Alfa2\ MW2\ SBA\^2\)\/CW2\^3)\)\ 
                      \(B0(S, Mh02, MZ2)\) + 
                    \((\(Alfa2\ CBA\^2\ MW2\)\/\(CW2\^2\ SW2\) - 
                          \(Alfa2\ CBA\^2\ MW2\)\/CW2\^3)\)\ 
                      \(B0(S, MHH2, MZ2)\) + 
                    \((2\ Alfa2\ MW2 - \(2\ Alfa2\ CW2\ MW2\)\/SW2 + 
                          S\ \((5\ Alfa2 - \(5\ Alfa2\ CW2\)\/SW2)\))\)\ 
                      \(B0(S, MW2, MW2)\) + 
                    \((\(2\ Alfa2\ MW2\ SW2\)\/CW2 - 
                          \(2\ Alfa2\ MW2\ SW2\^2\)\/CW2\^2)\)\ 
                      \(B0(S, MW2, MW2)\) + 
                    \((\(3\ Alfa2\)\/\(CW2\ SW2\) - \(3\ Alfa2\)\/CW2\^2)\)\ 
                      \(B00(S, 0, 0)\) + 
                    \((\(Alfa2\ SBA\^2\)\/CW2\^2 - 
                          \(Alfa2\ SBA\^2\)\/\(CW2\ SW2\))\)\ 
                      \((B00(S, Mh02, MZ2) + B00(S, MHH2, MA02))\) + 
                    \((\(Alfa2\ CBA\^2\)\/CW2\^2 - 
                          \(Alfa2\ CBA\^2\)\/\(CW2\ SW2\))\)\ 
                      \((B00(S, Mh02, MA02) + B00(S, MHH2, MZ2))\) + 
                    \((8\ Alfa2 - \(8\ Alfa2\ CW2\)\/SW2)\)\ 
                      \(B00(S, MW2, MW2)\) + 
                    \((\(Alfa2\ SW2\^2\)\/CW2\^2 - \(3\ Alfa2\ SW2\)\/CW2 + 
                          3\ Alfa2 - \(Alfa2\ CW2\)\/SW2)\)\ 
                      \((B00(S, MHp2, MHp2) + B00(S, MW2, MW2))\) + 
                    \((\(-\(\(3\ Alfa2\ MHp2\)\/\(CW2\ SW2\)\)\) + 
                          \(3\ Alfa2\ MHp2\)\/CW2\^2 + 
                          \((\(3\ Alfa2\)\/\(2\ CW2\ SW2\) - 
                                \(3\ Alfa2\)\/\(2\ CW2\^2\))\)\ \((T + U)\))
                        \)\ \(B1(S, 0, 0)\) + 
                    \((4\ Alfa2\ MHp2 - \(4\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                          \((\(2\ Alfa2\ CW2\)\/SW2 - 2\ Alfa2)\)\ 
                            \((T + U)\))\)\ \(B1(S, MW2, MW2)\) - 
                    4\ Alfa\ dZZZ1\ MZ2\ \[Pi])\)\ \(Den(S, MZ2)\)\^2)\)\ 
          \(Mat(F2)\), 
      \((\(Mat(F2)\)\ 
              \((\((8\ Alfa2\ \(A0(MLE2(Gen1))\) + 
                        8\/3\ Alfa2\ \(A0(MQD2(Gen1))\) + 
                        32\/3\ Alfa2\ \(A0(MQU2(Gen1))\) - 
                        16\ Alfa2\ \(B00(S, MLE2(Gen1), MLE2(Gen1))\) - 
                        16\/3\ Alfa2\ \(B00(S, MQD2(Gen1), MQD2(Gen1))\) - 
                        64\/3\ Alfa2\ \(B00(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \((16\ Alfa2\ MHp2 - 8\ Alfa2\ \((T + U)\))\)\ 
                          \(B1(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(16\ Alfa2\ MHp2\)\/3 - 8\/3\ Alfa2\ \((T + U)\))
                            \)\ \(B1(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(64\ Alfa2\ MHp2\)\/3 - 32\/3\ Alfa2\ \((T + U)\))
                            \)\ \(B1(S, MQU2(Gen1), MQU2(Gen1))\))\)\ 
                    \(Den(S, 0)\)\^2 + 
                  \((\((\(12\ SW2\ Alfa2\)\/CW2 - \(3\ Alfa2\)\/CW2 + 
                              Alfa2\/SW2 - 4\ Alfa2)\)\ \(A0(MLE2(Gen1))\) + 
                        \((\(4\ SW2\ Alfa2\)\/CW2 - \(3\ Alfa2\)\/CW2 + 
                              Alfa2\/SW2 - \(4\ Alfa2\)\/3)\)\ 
                          \(A0(MQD2(Gen1))\) + 
                        \((\(16\ SW2\ Alfa2\)\/CW2 - \(6\ Alfa2\)\/CW2 + 
                              \(2\ Alfa2\)\/SW2 - \(16\ Alfa2\)\/3)\)\ 
                          \(A0(MQU2(Gen1))\) + 
                        \((\(-\(\(24\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(6\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/SW2 + 
                              8\ Alfa2)\)\ 
                          \(B00(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(-\(\(8\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(6\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/SW2 + 
                              \(8\ Alfa2\)\/3)\)\ 
                          \(B00(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(-\(\(32\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(12\ Alfa2\)\/CW2 - \(4\ Alfa2\)\/SW2 + 
                              \(32\ Alfa2\)\/3)\)\ 
                          \(B00(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \((\(-8\)\ Alfa2\ MHp2 + 
                              \(24\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(6\ Alfa2\ MHp2\)\/CW2 + 
                              \(2\ Alfa2\ MHp2\)\/SW2 + 
                              \((\(-\(\(12\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(3\ Alfa2\)\/CW2 - Alfa2\/SW2 + 4\ Alfa2)
                                  \)\ \((T + U)\))\)\ 
                          \(B1(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(-\(\(8\ Alfa2\ MHp2\)\/3\)\) + 
                              \(8\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(6\ Alfa2\ MHp2\)\/CW2 + 
                              \(2\ Alfa2\ MHp2\)\/SW2 + 
                              \((\(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(3\ Alfa2\)\/CW2 - Alfa2\/SW2 + 
                                    \(4\ Alfa2\)\/3)\)\ \((T + U)\))\)\ 
                          \(B1(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(-\(\(32\ Alfa2\ MHp2\)\/3\)\) + 
                              \(32\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(12\ Alfa2\ MHp2\)\/CW2 + 
                              \(4\ Alfa2\ MHp2\)\/SW2 + 
                              \((\(-\(\(16\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(6\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/SW2 + 
                                    \(16\ Alfa2\)\/3)\)\ \((T + U)\))\)\ 
                          \(B1(S, MQU2(Gen1), MQU2(Gen1))\))\)\ 
                    \(Den(S, MZ2)\)\ \(Den(S, 0)\) + 
                  \(Den(S, MZ2)\)\^2\ 
                    \((\((\(4\ Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(4\ Alfa2\ SW2\)\/CW2 - 
                              \(2\ Alfa2\ SW2\)\/CW2\^2 + \(2\ Alfa2\)\/CW2 + 
                              Alfa2\/\(2\ CW2\^2\) - Alfa2\/\(2\ CW2\ SW2\))
                            \)\ \(A0(MLE2(Gen1))\) + 
                        \((\(4\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(4\ Alfa2\ SW2\)\/\(3\ CW2\) - 
                              \(2\ Alfa2\ SW2\)\/CW2\^2 + \(2\ Alfa2\)\/CW2 + 
                              \(3\ Alfa2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\)\/\(2\ CW2\ SW2\))\)\ 
                          \(A0(MQD2(Gen1))\) + 
                        \((\(16\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(16\ Alfa2\ SW2\)\/\(3\ CW2\) - 
                              \(4\ Alfa2\ SW2\)\/CW2\^2 + \(4\ Alfa2\)\/CW2 + 
                              \(3\ Alfa2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\)\/\(2\ CW2\ SW2\))\)\ 
                          \(A0(MQU2(Gen1))\) + 
                        \((Alfa2\/\(2\ CW2\ SW2\) - Alfa2\/\(2\ CW2\^2\))\)\ 
                          \(A0(MSf2(1, 1, Gen1))\) + 
                        \((\(-\(\(8\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(8\ Alfa2\ SW2\)\/CW2 + 
                              \(4\ Alfa2\ SW2\)\/CW2\^2 - \(4\ Alfa2\)\/CW2 - 
                              Alfa2\/CW2\^2 + Alfa2\/\(CW2\ SW2\))\)\ 
                          \(B00(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(-\(\(8\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\)\)\) + 
                              \(8\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                              \(4\ Alfa2\ SW2\)\/CW2\^2 - \(4\ Alfa2\)\/CW2 - 
                              \(3\ Alfa2\)\/CW2\^2 + 
                              \(3\ Alfa2\)\/\(CW2\ SW2\))\)\ 
                          \(B00(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(-\(\(32\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\)\)\) + 
                              \(32\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                              \(8\ Alfa2\ SW2\)\/CW2\^2 - \(8\ Alfa2\)\/CW2 - 
                              \(3\ Alfa2\)\/CW2\^2 + 
                              \(3\ Alfa2\)\/\(CW2\ SW2\))\)\ 
                          \(B00(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \((Alfa2\/CW2\^2 - Alfa2\/\(CW2\ SW2\))\)\ 
                          \(B00(S, MSf2(1, 1, Gen1), MSf2(1, 1, Gen1))\) + 
                        \((\(8\ Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                              \(8\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                              \(4\ Alfa2\ MHp2\ SW2\)\/CW2\^2 + 
                              \(4\ Alfa2\ MHp2\)\/CW2 + 
                              \(Alfa2\ MHp2\)\/CW2\^2 + 
                              \((\(-\(\(4\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                                    \(4\ Alfa2\ SW2\)\/CW2 + 
                                    \(2\ Alfa2\ SW2\)\/CW2\^2 - 
                                    \(2\ Alfa2\)\/CW2 - 
                                    Alfa2\/\(2\ CW2\^2\) + 
                                    Alfa2\/\(2\ CW2\ SW2\))\)\ \((T + U)\) - 
                              \(Alfa2\ MHp2\)\/\(CW2\ SW2\))\)\ 
                          \(B1(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(8\ Alfa2\ MHp2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(8\ Alfa2\ MHp2\ SW2\)\/\(3\ CW2\) - 
                              \(4\ Alfa2\ MHp2\ SW2\)\/CW2\^2 + 
                              \(4\ Alfa2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ MHp2\)\/CW2\^2 + 
                              \((\(-\(\(4\ Alfa2\ SW2\^2\)\/\(3\ 
                                        CW2\^2\)\)\) + 
                                    \(4\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                                    \(2\ Alfa2\ SW2\)\/CW2\^2 - 
                                    \(2\ Alfa2\)\/CW2 - 
                                    \(3\ Alfa2\)\/\(2\ CW2\^2\) + 
                                    \(3\ Alfa2\)\/\(2\ CW2\ SW2\))\)\ 
                                \((T + U)\) - 
                              \(3\ Alfa2\ MHp2\)\/\(CW2\ SW2\))\)\ 
                          \(B1(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(32\ Alfa2\ MHp2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(32\ Alfa2\ MHp2\ SW2\)\/\(3\ CW2\) - 
                              \(8\ Alfa2\ MHp2\ SW2\)\/CW2\^2 + 
                              \(8\ Alfa2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ MHp2\)\/CW2\^2 + 
                              \((\(-\(\(16\ Alfa2\ SW2\^2\)\/\(3\ 
                                        CW2\^2\)\)\) + 
                                    \(16\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                                    \(4\ Alfa2\ SW2\)\/CW2\^2 - 
                                    \(4\ Alfa2\)\/CW2 - 
                                    \(3\ Alfa2\)\/\(2\ CW2\^2\) + 
                                    \(3\ Alfa2\)\/\(2\ CW2\ SW2\))\)\ 
                                \((T + U)\) - 
                              \(3\ Alfa2\ MHp2\)\/\(CW2\ SW2\))\)\ 
                          \(B1(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \(B0(S, MLE2(Gen1), MLE2(Gen1))\)\ 
                          \((\(Alfa2\ \(MLE2(Gen1)\)\)\/\(2\ CW2\^2\) - 
                              \(Alfa2\ \(MLE2(Gen1)\)\)\/\(2\ CW2\ SW2\))\) + 
                        \(B0(S, MQD2(Gen1), MQD2(Gen1))\)\ 
                          \((\(3\ Alfa2\ \(MQD2(Gen1)\)\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ \(MQD2(Gen1)\)\)\/\(2\ CW2\ SW2\))
                            \) + \(B0(S, MQU2(Gen1), MQU2(Gen1))\)\ 
                          \((\(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ SW2\))
                            \))\))\) + 
            \(Mat(F1)\)\ 
              \((\((8\ Alfa2\ \(A0(MLE2(Gen1))\) + 
                        8\/3\ Alfa2\ \(A0(MQD2(Gen1))\) + 
                        32\/3\ Alfa2\ \(A0(MQU2(Gen1))\) - 
                        16\ Alfa2\ \(B00(S, MLE2(Gen1), MLE2(Gen1))\) - 
                        16\/3\ Alfa2\ \(B00(S, MQD2(Gen1), MQD2(Gen1))\) - 
                        64\/3\ Alfa2\ \(B00(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \((16\ Alfa2\ MHp2 - 8\ Alfa2\ \((T + U)\))\)\ 
                          \(B1(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(16\ Alfa2\ MHp2\)\/3 - 8\/3\ Alfa2\ \((T + U)\))
                            \)\ \(B1(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(64\ Alfa2\ MHp2\)\/3 - 32\/3\ Alfa2\ \((T + U)\))
                            \)\ \(B1(S, MQU2(Gen1), MQU2(Gen1))\))\)\ 
                    \(Den(S, 0)\)\^2 + 
                  \((\((\(12\ SW2\ Alfa2\)\/CW2 - \(7\ Alfa2\)\/CW2 + 
                              Alfa2\/\(CW2\ SW2\) + Alfa2\/SW2 - 4\ Alfa2)\)\ 
                          \(A0(MLE2(Gen1))\) + 
                        \((\(4\ SW2\ Alfa2\)\/CW2 - 
                              \(13\ Alfa2\)\/\(3\ CW2\) + 
                              Alfa2\/\(CW2\ SW2\) + Alfa2\/SW2 - 
                              \(4\ Alfa2\)\/3)\)\ \(A0(MQD2(Gen1))\) + 
                        \((\(16\ SW2\ Alfa2\)\/CW2 - 
                              \(34\ Alfa2\)\/\(3\ CW2\) + 
                              \(2\ Alfa2\)\/\(CW2\ SW2\) + 
                              \(2\ Alfa2\)\/SW2 - \(16\ Alfa2\)\/3)\)\ 
                          \(A0(MQU2(Gen1))\) + 
                        \((\(-\(\(24\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(14\ Alfa2\)\/CW2 - 
                              \(2\ Alfa2\)\/\(CW2\ SW2\) - 
                              \(2\ Alfa2\)\/SW2 + 8\ Alfa2)\)\ 
                          \(B00(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(-\(\(8\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(26\ Alfa2\)\/\(3\ CW2\) - 
                              \(2\ Alfa2\)\/\(CW2\ SW2\) - 
                              \(2\ Alfa2\)\/SW2 + \(8\ Alfa2\)\/3)\)\ 
                          \(B00(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(-\(\(32\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(68\ Alfa2\)\/\(3\ CW2\) - 
                              \(4\ Alfa2\)\/\(CW2\ SW2\) - 
                              \(4\ Alfa2\)\/SW2 + \(32\ Alfa2\)\/3)\)\ 
                          \(B00(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \((\(-8\)\ Alfa2\ MHp2 + 
                              \(24\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(14\ Alfa2\ MHp2\)\/CW2 + 
                              \(2\ Alfa2\ MHp2\)\/SW2 + 
                              \(2\ Alfa2\ MHp2\)\/\(CW2\ SW2\) + 
                              \((\(-\(\(12\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(7\ Alfa2\)\/CW2 - Alfa2\/\(CW2\ SW2\) - 
                                    Alfa2\/SW2 + 4\ Alfa2)\)\ \((T + U)\))\)\ 
                          \(B1(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(-\(\(8\ Alfa2\ MHp2\)\/3\)\) + 
                              \(8\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(26\ Alfa2\ MHp2\)\/\(3\ CW2\) + 
                              \(2\ Alfa2\ MHp2\)\/SW2 + 
                              \(2\ Alfa2\ MHp2\)\/\(CW2\ SW2\) + 
                              \((\(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(13\ Alfa2\)\/\(3\ CW2\) - 
                                    Alfa2\/\(CW2\ SW2\) - Alfa2\/SW2 + 
                                    \(4\ Alfa2\)\/3)\)\ \((T + U)\))\)\ 
                          \(B1(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(-\(\(32\ Alfa2\ MHp2\)\/3\)\) + 
                              \(32\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(68\ Alfa2\ MHp2\)\/\(3\ CW2\) + 
                              \(4\ Alfa2\ MHp2\)\/SW2 + 
                              \(4\ Alfa2\ MHp2\)\/\(CW2\ SW2\) + 
                              \((\(-\(\(16\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(34\ Alfa2\)\/\(3\ CW2\) - 
                                    \(2\ Alfa2\)\/\(CW2\ SW2\) - 
                                    \(2\ Alfa2\)\/SW2 + \(16\ Alfa2\)\/3)\)\ 
                                \((T + U)\))\)\ 
                          \(B1(S, MQU2(Gen1), MQU2(Gen1))\))\)\ 
                    \(Den(S, MZ2)\)\ \(Den(S, 0)\) + 
                  \(Den(S, MZ2)\)\^2\ 
                    \((\((\(4\ Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(4\ Alfa2\ SW2\)\/CW2 - 
                              \(4\ Alfa2\ SW2\)\/CW2\^2 + \(4\ Alfa2\)\/CW2 + 
                              \(3\ Alfa2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\)\/\(2\ CW2\ SW2\) - 
                              Alfa2\/\(4\ CW2\^2\ SW2\) + 
                              Alfa2\/\(4\ CW2\ SW2\^2\))\)\ 
                          \(A0(MLE2(Gen1))\) + 
                        \((\(4\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(4\ Alfa2\ SW2\)\/\(3\ CW2\) - 
                              \(8\ Alfa2\ SW2\)\/\(3\ CW2\^2\) + 
                              \(8\ Alfa2\)\/\(3\ CW2\) + 
                              \(5\ Alfa2\)\/\(2\ CW2\^2\) - 
                              \(5\ Alfa2\)\/\(2\ CW2\ SW2\) - 
                              \(3\ Alfa2\)\/\(4\ CW2\^2\ SW2\) + 
                              \(3\ Alfa2\)\/\(4\ CW2\ SW2\^2\))\)\ 
                          \(A0(MQD2(Gen1))\) + 
                        \((\(16\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(16\ Alfa2\ SW2\)\/\(3\ CW2\) - 
                              \(20\ Alfa2\ SW2\)\/\(3\ CW2\^2\) + 
                              \(20\ Alfa2\)\/\(3\ CW2\) + 
                              \(7\ Alfa2\)\/\(2\ CW2\^2\) - 
                              \(7\ Alfa2\)\/\(2\ CW2\ SW2\) - 
                              \(3\ Alfa2\)\/\(4\ CW2\^2\ SW2\) + 
                              \(3\ Alfa2\)\/\(4\ CW2\ SW2\^2\))\)\ 
                          \(A0(MQU2(Gen1))\) + 
                        \((Alfa2\/\(2\ CW2\ SW2\) + 
                              Alfa2\/\(4\ CW2\^2\ SW2\) - 
                              Alfa2\/\(2\ CW2\^2\) - 
                              Alfa2\/\(4\ CW2\ SW2\^2\))\)\ 
                          \(A0(MSf2(1, 1, Gen1))\) + 
                        \((\(-\(\(8\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(8\ Alfa2\ SW2\)\/CW2 + 
                              \(8\ Alfa2\ SW2\)\/CW2\^2 - \(8\ Alfa2\)\/CW2 - 
                              \(3\ Alfa2\)\/CW2\^2 + 
                              \(3\ Alfa2\)\/\(CW2\ SW2\) + 
                              Alfa2\/\(2\ CW2\^2\ SW2\) - 
                              Alfa2\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B00(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(-\(\(8\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\)\)\) + 
                              \(8\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                              \(16\ Alfa2\ SW2\)\/\(3\ CW2\^2\) - 
                              \(16\ Alfa2\)\/\(3\ CW2\) - 
                              \(5\ Alfa2\)\/CW2\^2 + 
                              \(5\ Alfa2\)\/\(CW2\ SW2\) + 
                              \(3\ Alfa2\)\/\(2\ CW2\^2\ SW2\) - 
                              \(3\ Alfa2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B00(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(-\(\(32\ Alfa2\ SW2\^2\)\/\(3\ CW2\^2\)\)\) + 
                              \(32\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                              \(40\ Alfa2\ SW2\)\/\(3\ CW2\^2\) - 
                              \(40\ Alfa2\)\/\(3\ CW2\) - 
                              \(7\ Alfa2\)\/CW2\^2 + 
                              \(7\ Alfa2\)\/\(CW2\ SW2\) + 
                              \(3\ Alfa2\)\/\(2\ CW2\^2\ SW2\) - 
                              \(3\ Alfa2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B00(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \((\(-\(Alfa2\/\(CW2\ SW2\)\)\) - 
                              Alfa2\/\(2\ CW2\^2\ SW2\) + Alfa2\/CW2\^2 + 
                              Alfa2\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B00(S, MSf2(1, 1, Gen1), MSf2(1, 1, Gen1))\) + 
                        \((\(8\ Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                              \(8\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                              \(8\ Alfa2\ MHp2\ SW2\)\/CW2\^2 + 
                              \(8\ Alfa2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ MHp2\)\/CW2\^2 + 
                              \((\(-\(\(4\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                                    \(4\ Alfa2\ SW2\)\/CW2 + 
                                    \(4\ Alfa2\ SW2\)\/CW2\^2 - 
                                    \(4\ Alfa2\)\/CW2 - 
                                    \(3\ Alfa2\)\/\(2\ CW2\^2\) + 
                                    \(3\ Alfa2\)\/\(2\ CW2\ SW2\) + 
                                    Alfa2\/\(4\ CW2\^2\ SW2\) - 
                                    Alfa2\/\(4\ CW2\ SW2\^2\))\)\ 
                                \((T + U)\) - 
                              \(3\ Alfa2\ MHp2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ MHp2\)\/\(2\ CW2\^2\ SW2\) + 
                              \(Alfa2\ MHp2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B1(S, MLE2(Gen1), MLE2(Gen1))\) + 
                        \((\(8\ Alfa2\ MHp2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(8\ Alfa2\ MHp2\ SW2\)\/\(3\ CW2\) - 
                              \(16\ Alfa2\ MHp2\ SW2\)\/\(3\ CW2\^2\) + 
                              \(16\ Alfa2\ MHp2\)\/\(3\ CW2\) + 
                              \(5\ Alfa2\ MHp2\)\/CW2\^2 + 
                              \((\(-\(\(4\ Alfa2\ SW2\^2\)\/\(3\ 
                                        CW2\^2\)\)\) + 
                                    \(4\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                                    \(8\ Alfa2\ SW2\)\/\(3\ CW2\^2\) - 
                                    \(8\ Alfa2\)\/\(3\ CW2\) - 
                                    \(5\ Alfa2\)\/\(2\ CW2\^2\) + 
                                    \(5\ Alfa2\)\/\(2\ CW2\ SW2\) + 
                                    \(3\ Alfa2\)\/\(4\ CW2\^2\ SW2\) - 
                                    \(3\ Alfa2\)\/\(4\ CW2\ SW2\^2\))\)\ 
                                \((T + U)\) - 
                              \(5\ Alfa2\ MHp2\)\/\(CW2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\)\/\(2\ CW2\^2\ SW2\) + 
                              \(3\ Alfa2\ MHp2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B1(S, MQD2(Gen1), MQD2(Gen1))\) + 
                        \((\(32\ Alfa2\ MHp2\ SW2\^2\)\/\(3\ CW2\^2\) - 
                              \(32\ Alfa2\ MHp2\ SW2\)\/\(3\ CW2\) - 
                              \(40\ Alfa2\ MHp2\ SW2\)\/\(3\ CW2\^2\) + 
                              \(40\ Alfa2\ MHp2\)\/\(3\ CW2\) + 
                              \(7\ Alfa2\ MHp2\)\/CW2\^2 + 
                              \((\(-\(\(16\ Alfa2\ SW2\^2\)\/\(3\ 
                                        CW2\^2\)\)\) + 
                                    \(16\ Alfa2\ SW2\)\/\(3\ CW2\) + 
                                    \(20\ Alfa2\ SW2\)\/\(3\ CW2\^2\) - 
                                    \(20\ Alfa2\)\/\(3\ CW2\) - 
                                    \(7\ Alfa2\)\/\(2\ CW2\^2\) + 
                                    \(7\ Alfa2\)\/\(2\ CW2\ SW2\) + 
                                    \(3\ Alfa2\)\/\(4\ CW2\^2\ SW2\) - 
                                    \(3\ Alfa2\)\/\(4\ CW2\ SW2\^2\))\)\ 
                                \((T + U)\) - 
                              \(7\ Alfa2\ MHp2\)\/\(CW2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\)\/\(2\ CW2\^2\ SW2\) + 
                              \(3\ Alfa2\ MHp2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(B1(S, MQU2(Gen1), MQU2(Gen1))\) + 
                        \(B0(S, MLE2(Gen1), MLE2(Gen1))\)\ 
                          \((\(-\(\(Alfa2\ \(MLE2(Gen1)\)\)\/\(2\ CW2\ SW2
                                      \)\)\) - 
                              \(Alfa2\ \(MLE2(Gen1)\)\)\/\(4\ CW2\^2\ SW2\) + 
                              \(Alfa2\ \(MLE2(Gen1)\)\)\/\(2\ CW2\^2\) + 
                              \(Alfa2\ \(MLE2(Gen1)\)\)\/\(4\ CW2\ SW2\^2\))
                            \) + \(B0(S, MQD2(Gen1), MQD2(Gen1))\)\ 
                          \((\(-\(\(3\ Alfa2\ \(MQD2(Gen1)\)\)\/\(2\ CW2\ SW2
                                      \)\)\) - 
                              \(3\ Alfa2\ \(MQD2(Gen1)\)\)\/\(4\ CW2\^2\ SW2
                                  \) + \(3\ Alfa2\ \(MQD2(Gen1)\)\)\/\(2\ 
                                  CW2\^2\) + 
                              \(3\ Alfa2\ \(MQD2(Gen1)\)\)\/\(4\ CW2\ 
                                  SW2\^2\))\) + 
                        \(B0(S, MQU2(Gen1), MQU2(Gen1))\)\ 
                          \((\(-\(\(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ SW2
                                      \)\)\) - 
                              \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(4\ CW2\^2\ SW2
                                  \) + \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ 
                                  CW2\^2\) + 
                              \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(4\ CW2\ 
                                  SW2\^2\))\))\))\))\)\ \(SumOver(Gen1, 3)\), 
      \(SumOver(Gen1, 3)\)\ \(SumOver(Sfe1, 2)\)\ 
        \((\(Mat(F2)\)\ 
              \((\((\(-4\)\ Alfa2\ \(A0(MSf2(Sfe1, 2, Gen1))\) - 
                        16\/3\ Alfa2\ \(A0(MSf2(Sfe1, 3, Gen1))\) - 
                        4\/3\ Alfa2\ \(A0(MSf2(Sfe1, 4, Gen1))\) + 
                        8\ Alfa2\ 
                          \(B00(S, MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1))
                            \) + 32\/3\ Alfa2\ 
                          \(B00(S, MSf2(Sfe1, 3, Gen1), MSf2(Sfe1, 3, Gen1))
                            \) + 8\/3\ Alfa2\ 
                          \(B00(S, MSf2(Sfe1, 4, Gen1), MSf2(Sfe1, 4, Gen1))
                            \))\)\ \(Den(S, 0)\)\^2 + 
                  \(Den(S, MZ2)\)\ 
                    \((\(B00(S, MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1))\)\ 
                          \((\(1\/CW2\((
                                6\ Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/SW2\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 2, Gen1))\)\ 
                          \((\(1\/SW2\((
                                Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/CW2\((
                                3\ Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\))\) + 
                        \(B00(S, MSf2(Sfe1, 3, Gen1), MSf2(Sfe1, 3, Gen1))\)\ 
                          \((\(1\/CW2\((
                                4\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/\(3\ SW2\)\((
                                4\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 3, Gen1))\)\ 
                          \((\(1\/\(3\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/CW2\((
                                2\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\))\) + 
                        \(B00(S, MSf2(Sfe1, 4, Gen1), MSf2(Sfe1, 4, Gen1))\)\ 
                          \((\(1\/CW2\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/\(3\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 4, Gen1))\)\ 
                          \((\(1\/\(3\ SW2\)\((
                                Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/CW2\((
                                Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\))\))
                      \)\ \(Den(S, 0)\) + 
                  \(Den(S, MZ2)\)\^2\ 
                    \((\(A0(MSf2(Sfe1, 2, Gen1))\)\ 
                          \((\(1\/\(2\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\)\ 
                                      \((1 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/\(2\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\)\ 
                                      \((1 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 3, Gen1))\)\ 
                          \((\(1\/\(6\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\)\ 
                                      \((3 - 4\ SW2)\)\^2 + 
                                    16\ SW2\^2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/\(6\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\)\ 
                                      \((3 - 4\ SW2)\)\^2 + 
                                    16\ SW2\^2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 4, Gen1))\)\ 
                          \((\(1\/\(6\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\)\ 
                                      \((3 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/\(6\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\)\ 
                                      \((3 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\))\))
                      \))\) + 
            \(Mat(F1)\)\ 
              \((\((\(-4\)\ Alfa2\ \(A0(MSf2(Sfe1, 2, Gen1))\) - 
                        16\/3\ Alfa2\ \(A0(MSf2(Sfe1, 3, Gen1))\) - 
                        4\/3\ Alfa2\ \(A0(MSf2(Sfe1, 4, Gen1))\) + 
                        8\ Alfa2\ 
                          \(B00(S, MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1))
                            \) + 32\/3\ Alfa2\ 
                          \(B00(S, MSf2(Sfe1, 3, Gen1), MSf2(Sfe1, 3, Gen1))
                            \) + 8\/3\ Alfa2\ 
                          \(B00(S, MSf2(Sfe1, 4, Gen1), MSf2(Sfe1, 4, Gen1))
                            \))\)\ \(Den(S, 0)\)\^2 + 
                  \(Den(S, MZ2)\)\ 
                    \((\(B00(S, MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1))\)\ 
                          \((\(1\/CW2\((
                                6\ Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/SW2\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 2, Gen1))\)\ 
                          \((\(-\(1\/CW2\((
                                  3\ Alfa2\ \((
                                    \((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                        \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                      2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                        \(USfC(Sfe1, 2, 2, Gen1)\))\))
                                  \)\)\) + 
                              \(1\/SW2\((
                                Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) + 
                              \(1\/\(CW2\ SW2\)\((
                                Alfa2\ \((
                                  \((2\ SW2 - 1)\)\ 
                                      \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\))\) + 
                        \(B00(S, MSf2(Sfe1, 3, Gen1), MSf2(Sfe1, 3, Gen1))\)\ 
                          \((\(1\/CW2\((
                                4\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/\(3\ SW2\)\((
                                4\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/\(3\ CW2\ SW2\)\((
                                4\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 3, Gen1))\)\ 
                          \((\(-\(1\/CW2\((
                                  2\ Alfa2\ \((
                                    \((4\ SW2 - 3)\)\ 
                                        \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                        \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                      4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                        \(USfC(Sfe1, 2, 3, Gen1)\))\))
                                  \)\)\) + 
                              \(1\/\(3\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) + 
                              \(1\/\(3\ CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((4\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                    4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\))\) + 
                        \(B00(S, MSf2(Sfe1, 4, Gen1), MSf2(Sfe1, 4, Gen1))\)\ 
                          \((\(1\/CW2\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/\(3\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/\(3\ CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 4, Gen1))\)\ 
                          \((\(-\(1\/CW2\((
                                  Alfa2\ \((
                                    \((2\ SW2 - 3)\)\ 
                                        \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                        \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                      2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                        \(USfC(Sfe1, 2, 4, Gen1)\))\))
                                  \)\)\) + 
                              \(1\/\(3\ SW2\)\((
                                Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) + 
                              \(1\/\(3\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \((2\ SW2 - 3)\)\ 
                                      \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                    2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\))\))
                      \)\ \(Den(S, 0)\) + 
                  \(Den(S, MZ2)\)\^2\ 
                    \((\(A0(MSf2(Sfe1, 2, Gen1))\)\ 
                          \((\(1\/\(2\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\)\ 
                                      \((1 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) + 
                              \(1\/\(4\ CW2\^2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\)\ 
                                      \((1 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/\(2\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\)\ 
                                      \((1 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\) - 
                              \(1\/\(4\ CW2\ SW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 2, Gen1)\)\ 
                                      \((1 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 2, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 3, Gen1))\)\ 
                          \((\(1\/\(6\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\)\ 
                                      \((3 - 4\ SW2)\)\^2 + 
                                    16\ SW2\^2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) + 
                              \(1\/\(12\ CW2\^2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\)\ 
                                      \((3 - 4\ SW2)\)\^2 + 
                                    16\ SW2\^2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/\(6\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\)\ 
                                      \((3 - 4\ SW2)\)\^2 + 
                                    16\ SW2\^2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\) - 
                              \(1\/\(12\ CW2\ SW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 3, Gen1)\)\ 
                                      \((3 - 4\ SW2)\)\^2 + 
                                    16\ SW2\^2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 3, Gen1)\))\))\)\))\) + 
                        \(A0(MSf2(Sfe1, 4, Gen1))\)\ 
                          \((\(1\/\(6\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\)\ 
                                      \((3 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) + 
                              \(1\/\(12\ CW2\^2\ SW2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\)\ 
                                      \((3 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/\(6\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\)\ 
                                      \((3 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\) - 
                              \(1\/\(12\ CW2\ SW2\^2\)\((
                                Alfa2\ \((
                                  \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 1, 4, Gen1)\)\ 
                                      \((3 - 2\ SW2)\)\^2 + 
                                    4\ SW2\^2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                      \(USfC(Sfe1, 2, 4, Gen1)\))\))\)\))\))
                      \))\))\), 
      \(SumOver(Cha1, 2)\)\ 
        \((\(Mat(F2)\)\ 
              \((\((8\ Alfa2\ \(A0(MCha2(Cha1))\) - 
                        16\ Alfa2\ \(B00(S, MCha2(Cha1), MCha2(Cha1))\) + 
                        \((16\ Alfa2\ MHp2 - 8\ Alfa2\ \((T + U)\))\)\ 
                          \(B1(S, MCha2(Cha1), MCha2(Cha1))\))\)\ 
                    \(Den(S, 0)\)\^2 + 
                  \(Den(S, MZ2)\)\ 
                    \((\(A0(MCha2(Cha1))\)\ 
                          \((\(6\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(2\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/SW2 + 
                              \(6\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(2\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/SW2)\) + 
                        \(B00(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                          \((\(-\(\(12\ Alfa2\ \((
                                      SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/CW2\)\) + 
                              \(4\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/SW2 - 
                              \(12\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/CW2 + 
                              \(4\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/SW2)\) + 
                        \(B1(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                          \((\(12\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(4\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/SW2 + 
                              \(12\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(4\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/SW2 + 
                              \((T + U)\)\ 
                                \((\(-
                                      \(\(6\ Alfa2\ \((
                                        SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/CW2\)\) + 
                                    \(2\ Alfa2\ \((
                                        SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/SW2 - 
                                    \(6\ Alfa2\ \((
                                        SW2 - 
                                        \(VCha(Cha1, 1)\)\ 
                                        \(VChaC(Cha1, 1)\) - 
                                        1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha1, 2)\))\)\)\/CW2 + 
                                    \(2\ Alfa2\ \((
                                        SW2 - 
                                        \(VCha(Cha1, 1)\)\ 
                                        \(VChaC(Cha1, 1)\) - 
                                        1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha1, 2)\))\)\)\/SW2)\))\))
                      \)\ \(Den(S, 0)\))\) + 
            \(Mat(F1)\)\ 
              \((\((8\ Alfa2\ \(A0(MCha2(Cha1))\) - 
                        16\ Alfa2\ \(B00(S, MCha2(Cha1), MCha2(Cha1))\) + 
                        \((16\ Alfa2\ MHp2 - 8\ Alfa2\ \((T + U)\))\)\ 
                          \(B1(S, MCha2(Cha1), MCha2(Cha1))\))\)\ 
                    \(Den(S, 0)\)\^2 + 
                  \(Den(S, MZ2)\)\ 
                    \((\(A0(MCha2(Cha1))\)\ 
                          \((\(6\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(2\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/SW2 - 
                              \(2\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\) + 
                              \(6\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(2\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/SW2 - 
                              \(2\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\))
                            \) + \(B00(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                          \((\(-\(\(12\ Alfa2\ \((
                                      SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/CW2\)\) + 
                              \(4\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/SW2 + 
                              \(4\ Alfa2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\) - 
                              \(12\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/CW2 + 
                              \(4\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/SW2 + 
                              \(4\ Alfa2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\))
                            \) + \(B1(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                          \((\(12\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(4\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/SW2 - 
                              \(4\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(UCha(Cha1, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha1, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\) + 
                              \(12\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/CW2 - 
                              \(4\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/SW2 - 
                              \(4\ Alfa2\ MHp2\ \((
                                  SW2 - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\) + 
                              \((T + U)\)\ 
                                \((\(-
                                      \(\(6\ Alfa2\ \((
                                        SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/CW2\)\) + 
                                    \(2\ Alfa2\ \((
                                        SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/SW2 + 
                                    \(2\ Alfa2\ \((
                                        SW2 - 
                                        \(UCha(Cha1, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2
                                        \) - 
                                    \(6\ Alfa2\ \((
                                        SW2 - 
                                        \(VCha(Cha1, 1)\)\ 
                                        \(VChaC(Cha1, 1)\) - 
                                        1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha1, 2)\))\)\)\/CW2 + 
                                    \(2\ Alfa2\ \((
                                        SW2 - 
                                        \(VCha(Cha1, 1)\)\ 
                                        \(VChaC(Cha1, 1)\) - 
                                        1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha1, 2)\))\)\)\/SW2 + 
                                    \(2\ Alfa2\ \((
                                        SW2 - 
                                        \(VCha(Cha1, 1)\)\ 
                                        \(VChaC(Cha1, 1)\) - 
                                        1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha1, 2)\))\)\)\/\(CW2\ SW2\))
                                  \))\))\)\ \(Den(S, 0)\))\))\), 
      \(SumOver(Neu1, 4)\)\ \(SumOver(Neu2, 4)\)\ 
        \((\(Mat(F2)\)\ 
              \((\(A0(MNeu2(Neu2))\)\ 
                    \((\(1\/\(2\ CW2\^2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/\(2\ CW2\ SW2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))\)\))
                      \) + \(B00(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                    \((\(1\/\(CW2\ SW2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/CW2\^2\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))\)\))
                      \) + \(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                    \((\(-\(1\/\(4\ CW2\ SW2\)\((
                            Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                              \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\^2)\)\)\) + 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                  \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\^2
                              \)\/\(4\ CW2\^2\) - 
                        \(1\/\(2\ CW2\ SW2\)\((
                          Alfa2\ \(MNeu2(Neu1)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                          \)\) + 
                        \(1\/\(2\ CW2\^2\)\((
                          Alfa2\ \(MNeu2(Neu1)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                          \)\) - 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\^2
                              \)\/\(4\ CW2\ SW2\) + 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\^2
                              \)\/\(4\ CW2\^2\))\) + 
                  \(B1(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                    \((\(1\/CW2\^2\((
                          Alfa2\ MHp2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          Alfa2\ MHp2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) + 
                        \((T + U)\)\ 
                          \((\(1\/\(2\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                  \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                      \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                    \))\)\) - 
                              \(1\/\(2\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                  \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                      \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                    \))\)\))\))\))\)\ \(Den(S, MZ2)\)\^2 + 
            \(Mat(F1)\)\ 
              \((\(A0(MNeu2(Neu2))\)\ 
                    \((\(-\(1\/\(2\ CW2\ SW2\)\((
                            Alfa2\ \((
                              \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                              \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                            \)\)\) - 
                        \(1\/\(4\ CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) + 
                        \(1\/\(2\ CW2\^2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) + 
                        \(1\/\(4\ CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))\)\))
                      \) + \(B00(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                    \((\(1\/\(CW2\ SW2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) + 
                        \(1\/\(2\ CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/CW2\^2\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/\(2\ CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))\)\))
                      \) + \(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                    \((\(-\(1\/\(4\ CW2\ SW2\)\((
                            Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                              \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\^2)\)\)\) - 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                  \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\^2
                              \)\/\(8\ CW2\^2\ SW2\) + 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                  \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\^2
                              \)\/\(4\ CW2\^2\) + 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                  \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\^2
                              \)\/\(8\ CW2\ SW2\^2\) - 
                        \(1\/\(2\ CW2\ SW2\)\((
                          Alfa2\ \(MNeu2(Neu1)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                          \)\) - 
                        \(1\/\(4\ CW2\^2\ SW2\)\((
                          Alfa2\ \(MNeu2(Neu1)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                          \)\) + 
                        \(1\/\(2\ CW2\^2\)\((
                          Alfa2\ \(MNeu2(Neu1)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                          \)\) + 
                        \(1\/\(4\ CW2\ SW2\^2\)\((
                          Alfa2\ \(MNeu2(Neu1)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                            \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                          \)\) - 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\^2
                              \)\/\(4\ CW2\ SW2\) - 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\^2
                              \)\/\(8\ CW2\^2\ SW2\) + 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\^2
                              \)\/\(4\ CW2\^2\) + 
                        \(Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                  \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\^2
                              \)\/\(8\ CW2\ SW2\^2\))\) + 
                  \(B1(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                    \((\(1\/CW2\^2\((
                          Alfa2\ MHp2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          Alfa2\ MHp2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) - 
                        \(1\/\(2\ CW2\^2\ SW2\)\((
                          Alfa2\ MHp2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) + 
                        \(1\/\(2\ CW2\ SW2\^2\)\((
                          Alfa2\ MHp2\ \((
                            \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                              \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                            \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                          \)\) + 
                        \((T + U)\)\ 
                          \((\(1\/\(2\ CW2\ SW2\)\((
                                Alfa2\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                  \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                      \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                    \))\)\) + 
                              \(1\/\(4\ CW2\^2\ SW2\)\((
                                Alfa2\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                  \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                      \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                    \))\)\) - 
                              \(1\/\(2\ CW2\^2\)\((
                                Alfa2\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                  \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                      \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                    \))\)\) - 
                              \(1\/\(4\ CW2\ SW2\^2\)\((
                                Alfa2\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                  \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                      \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                    \))\)\))\))\))\)\ \(Den(S, MZ2)\)\^2)\), 
      \(SumOver(Gen1, 3)\)\ \(SumOver(Sfe1, 2)\)\ \(SumOver(Sfe2, 2)\)\ 
        \((\(Mat(F2)\)\ 
              \((\(B00(S, MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1))\)\ 
                    \((\(1\/CW2\^2\((
                          Alfa2\ \((
                            \((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 2, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 2, Gen1)\))\)\ 
                            \((\((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 2, Gen1)\))\))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 2, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 2, Gen1)\))\)\ 
                            \((\((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 2, Gen1)\))\))\)\))\) + 
                  \(B00(S, MSf2(Sfe1, 3, Gen1), MSf2(Sfe2, 3, Gen1))\)\ 
                    \((\(1\/\(3\ CW2\^2\)\((
                          Alfa2\ \((
                            \((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 3, Gen1)\) + 
                              4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 3, Gen1)\))\)\ 
                            \((\((4\ SW2 - 3)\)\ \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 3, Gen1)\) + 
                                4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 3, Gen1)\))\))\)\) - 
                        \(1\/\(3\ CW2\ SW2\)\((
                          Alfa2\ \((
                            \((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 3, Gen1)\) + 
                              4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 3, Gen1)\))\)\ 
                            \((\((4\ SW2 - 3)\)\ \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 3, Gen1)\) + 
                                4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 3, Gen1)\))\))\)\))\) + 
                  \(B00(S, MSf2(Sfe1, 4, Gen1), MSf2(Sfe2, 4, Gen1))\)\ 
                    \((\(1\/\(3\ CW2\^2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 3)\)\ \(USf(Sfe2, 1, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 4, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 4, Gen1)\))\)\ 
                            \((\((2\ SW2 - 3)\)\ \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 4, Gen1)\))\))\)\) - 
                        \(1\/\(3\ CW2\ SW2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 3)\)\ \(USf(Sfe2, 1, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 4, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 4, Gen1)\))\)\ 
                            \((\((2\ SW2 - 3)\)\ \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 4, Gen1)\))\))\)\))\))\)\ 
              \(Den(S, MZ2)\)\^2 + 
            \(Mat(F1)\)\ 
              \((\(B00(S, MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1))\)\ 
                    \((\(-\(1\/\(CW2\ SW2\)\((
                            Alfa2\ \((
                              \((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe1, 2, 2, Gen1)\))\)\ 
                              \((\((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                    \(USfC(Sfe2, 1, 2, Gen1)\) + 
                                  2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                    \(USfC(Sfe2, 2, 2, Gen1)\))\))\)\)\) - 
                        \(1\/\(2\ CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 2, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 2, Gen1)\))\)\ 
                            \((\((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 2, Gen1)\))\))\)\) + 
                        \(1\/CW2\^2\((
                          Alfa2\ \((
                            \((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 2, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 2, Gen1)\))\)\ 
                            \((\((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 2, Gen1)\))\))\)\) + 
                        \(1\/\(2\ CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 2, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 2, Gen1)\))\)\ 
                            \((\((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 2, Gen1)\))\))\)\))\) + 
                  \(B00(S, MSf2(Sfe1, 3, Gen1), MSf2(Sfe2, 3, Gen1))\)\ 
                    \((\(-\(1\/\(3\ CW2\ SW2\)\((
                            Alfa2\ \((
                              \((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                  \(USfC(Sfe1, 1, 3, Gen1)\) + 
                                4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                  \(USfC(Sfe1, 2, 3, Gen1)\))\)\ 
                              \((\((4\ SW2 - 3)\)\ \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                    \(USfC(Sfe2, 1, 3, Gen1)\) + 
                                  4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                    \(USfC(Sfe2, 2, 3, Gen1)\))\))\)\)\) - 
                        \(1\/\(6\ CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            \((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 3, Gen1)\) + 
                              4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 3, Gen1)\))\)\ 
                            \((\((4\ SW2 - 3)\)\ \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 3, Gen1)\) + 
                                4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 3, Gen1)\))\))\)\) + 
                        \(1\/\(3\ CW2\^2\)\((
                          Alfa2\ \((
                            \((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 3, Gen1)\) + 
                              4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 3, Gen1)\))\)\ 
                            \((\((4\ SW2 - 3)\)\ \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 3, Gen1)\) + 
                                4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 3, Gen1)\))\))\)\) + 
                        \(1\/\(6\ CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            \((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 3, Gen1)\) + 
                              4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 3, Gen1)\))\)\ 
                            \((\((4\ SW2 - 3)\)\ \(USf(Sfe1, 1, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 3, Gen1)\) + 
                                4\ SW2\ \(USf(Sfe1, 2, 3, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 3, Gen1)\))\))\)\))\) + 
                  \(B00(S, MSf2(Sfe1, 4, Gen1), MSf2(Sfe2, 4, Gen1))\)\ 
                    \((\(-\(1\/\(3\ CW2\ SW2\)\((
                            Alfa2\ \((
                              \((2\ SW2 - 3)\)\ \(USf(Sfe2, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe1, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe2, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe1, 2, 4, Gen1)\))\)\ 
                              \((\((2\ SW2 - 3)\)\ \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                    \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                  2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                    \(USfC(Sfe2, 2, 4, Gen1)\))\))\)\)\) - 
                        \(1\/\(6\ CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 3)\)\ \(USf(Sfe2, 1, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 4, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 4, Gen1)\))\)\ 
                            \((\((2\ SW2 - 3)\)\ \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 4, Gen1)\))\))\)\) + 
                        \(1\/\(3\ CW2\^2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 3)\)\ \(USf(Sfe2, 1, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 4, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 4, Gen1)\))\)\ 
                            \((\((2\ SW2 - 3)\)\ \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 4, Gen1)\))\))\)\) + 
                        \(1\/\(6\ CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            \((2\ SW2 - 3)\)\ \(USf(Sfe2, 1, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 1, 4, Gen1)\) + 
                              2\ SW2\ \(USf(Sfe2, 2, 4, Gen1)\)\ 
                                \(USfC(Sfe1, 2, 4, Gen1)\))\)\ 
                            \((\((2\ SW2 - 3)\)\ \(USf(Sfe1, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe1, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 4, Gen1)\))\))\)\))\))\)\ 
              \(Den(S, MZ2)\)\^2)\), 
      \(SumOver(Cha1, 2)\)\ \(SumOver(Cha2, 2)\)\ 
        \((\(Mat(F2)\)\ 
              \((\(A0(MCha2(Cha2))\)\ 
                    \((\(-\(1\/\(CW2\ SW2\)\((
                            2\ Alfa2\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                  1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                                \))\)\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\))\) + 
                  \(B00(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                    \((\(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\))\) + 
                  \(B0(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                    \((\(-\(1\/\(CW2\ SW2\)\((
                            2\ Alfa2\ \(MCha2(Cha1)\)\ 
                              \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                  1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                                \))\)\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\))\) + 
                  \(B1(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                    \((\(1\/CW2\^2\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \((T + U)\)\ 
                          \((\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                      1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha2, 2)\))\))\)\) - 
                              \(1\/CW2\^2\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                      1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha2, 2)\))\))\)\) + 
                              \(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha2, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\))\)\) - 
                              \(1\/CW2\^2\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha2, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\))\)\))\))\))\)\ 
              \(Den(S, MZ2)\)\^2 + 
            \(Mat(F1)\)\ 
              \((\(A0(MCha2(Cha2))\)\ 
                    \((\(-\(1\/\(CW2\ SW2\)\((
                            2\ Alfa2\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                  1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                                \))\)\)\) - 
                        \(1\/\(CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\^2\ SW2\)\((
                          Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\^2\)\((
                          Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\))\) + 
                  \(B00(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                    \((\(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\^2\ SW2\)\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\^2\)\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\^2\ SW2\)\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          4\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\^2\)\((
                          2\ Alfa2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\))\) + 
                  \(B0(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                    \((\(-\(1\/\(CW2\ SW2\)\((
                            2\ Alfa2\ \(MCha2(Cha1)\)\ 
                              \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                  1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                                \))\)\)\) - 
                        \(1\/\(CW2\^2\ SW2\)\((
                          Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\^2\)\((
                          Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\^2\ SW2\)\((
                          Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\^2\)\((
                          Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\^2\ SW2\)\((
                          Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \))\)\) - 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\^2\)\((
                          Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          2\ Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\^2\ SW2\)\((
                          Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          2\ Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\^2\)\((
                          Alfa2\ \(MCha2(Cha1)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))
                              \)\ \((
                              SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\))\) + 
                  \(B1(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                    \((\(1\/CW2\^2\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\^2\ SW2\)\((
                          2\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\^2\)\((
                          2\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                              1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/CW2\^2\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\ SW2\)\((
                          4\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) - 
                        \(1\/\(CW2\^2\ SW2\)\((
                          2\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \(1\/\(CW2\ SW2\^2\)\((
                          2\ Alfa2\ MHp2\ \((
                            SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                              \))\)\) + 
                        \((T + U)\)\ 
                          \((\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                      1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha2, 2)\))\))\)\) + 
                              \(1\/\(CW2\^2\ SW2\)\((
                                Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                      1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha2, 2)\))\))\)\) - 
                              \(1\/CW2\^2\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                      1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha2, 2)\))\))\)\) - 
                              \(1\/\(CW2\ SW2\^2\)\((
                                Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                      1\/2\ \(UCha(Cha1, 2)\)\ 
                                        \(UChaC(Cha2, 2)\))\))\)\) + 
                              \(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha2, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\))\)\) + 
                              \(1\/\(CW2\^2\ SW2\)\((
                                Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha2, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\))\)\) - 
                              \(1\/CW2\^2\((
                                2\ Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha2, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\))\)\) - 
                              \(1\/\(CW2\ SW2\^2\)\((
                                Alfa2\ \((
                                  SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                                    1\/2\ \(VCha(Cha2, 2)\)\ 
                                      \(VChaC(Cha1, 2)\))\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\))\)\))\))\))\)\ 
              \(Den(S, MZ2)\)\^2)\)\))\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(tops\  = \ 
      CreateTopologies[1, \ 2\  -> \ 2, \n\ \ 
        ExcludeTopologies\  -> 
          \ {Tadpoles, \ WFCorrections, \ SelfEnergies, \ AllBoxes}]; \)\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ins\  = \ 
      DiagramSelect[inss, \n\ \ 
        FreeQ[#, \ Field[5]\  -> \ S]\ 
           (*\  && \ FreeQ[#, \ Field[6 | 7 | 8]\  -> \ V[1]]\ *) \ &]; \)\)],
   "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"6 Generic, 58 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "6 Generic, 58 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 3, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 4, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]5
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 5, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]6
          \[InvisibleSpace]": "\[InvisibleSpace]"7 Generic, 25 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 6, ": ", "7 Generic, 25 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]7
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 10 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 7, ": ", "1 Generic, 10 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]8
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 10 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 8, ": ", "1 Generic, 10 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]9
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 13 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 9, ": ", "2 Generic, 13 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]10
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 10, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]11
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 11, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]12
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 12, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]13
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 13, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]14
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 14, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]15
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 15, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]16
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 16, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]17
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 17, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]18
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 18, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]19
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 19, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]20
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 20, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]21
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 21, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]22
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 22, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]23
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 23, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]24
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 24, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"17 Generic, 116 Classes insertions"\),
        
        SequenceForm[ "in total: ", "17 Generic, 116 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DoPaint[ins, \ "\<vert\>"]; \)\)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"58 diagrams"\),
        SequenceForm[ "> Top. ", 1, ": ", "58 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"25 diagrams"\),
        SequenceForm[ "> Top. ", 2, ": ", "25 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"10 diagrams"\),
        SequenceForm[ "> Top. ", 3, ": ", "10 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"10 diagrams"\),
        SequenceForm[ "> Top. ", 4, ": ", "10 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]5
          \[InvisibleSpace]": "\[InvisibleSpace]"13 diagrams"\),
        SequenceForm[ "> Top. ", 5, ": ", "13 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.68932 .55463 m
.63184 .5534 L
.64407 .51916 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.63795 .46372 m
.69544 .4625 L
.68321 .42825 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N1) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
.79091 .52727 m
.77273 .47273 L
.80909 .47273 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N2) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N3) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.68932 .55463 m
.63184 .5534 L
.64407 .51916 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.63795 .46372 m
.69544 .4625 L
.68321 .42825 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N4) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
.79091 .52727 m
.77273 .47273 L
.80909 .47273 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
66.312500 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N5) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N6) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N7) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N8) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N9) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N10) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N11) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N12) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .52727 m
.77273 .47273 L
.80909 .47273 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N13) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.68932 .55463 m
.63184 .5534 L
.64407 .51916 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.63795 .46372 m
.69544 .4625 L
.68321 .42825 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N14) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .52727 m
.77273 .47273 L
.80909 .47273 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N15) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N16) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N17) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C11) show
80.562500 14.625000 moveto
(N18) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C12) show
80.562500 14.625000 moveto
(N19) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C13) show
80.562500 14.625000 moveto
(N20) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool000h0oooo00@000000?ooo`3o
ool000000P3oool300000080oooo00@000000?ooo`00000000000P3oool00`000000oooo00000002
0?ooo`030000003oool0oooo01d0oooo00@000000?ooo`3oool000000P3oool300000080oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo000000020?ooo`040000003oool00000000001`0
oooo00@000000?ooo`3oool000000P3oool300000080oooo00@000000?ooo`00000000000P3oool0
0`000000oooo000000020?ooo`040000003oool00000000001`0oooo00@000000?ooo`3oool00000
0P3oool300000080oooo00@000000?ooo`00000000000P3oool010000000oooo0000003oool20000
00030?ooo`000000000000l0oooo000>0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool00P3oool01P000000oooo0000003oool000000?ooo`<000000P3oool010000000oooo0?oo
o`00000L0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000
oooo0?ooo`0000020?ooo`<000000P3oool010000000oooo00000000000L0?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`0000020?ooo`<00000
0P3oool00`000000oooo0?ooo`02000001/0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo0`0000020?ooo`050000003oool00000
0?ooo`0000003P3oool000h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0000000000080oooo0`0000020?ooo`040000003oool0oooo000001`0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo0000
0080oooo0`0000020?ooo`040000003oool00000000001`0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`020?ooo`060000003oool0oooo0?ooo`000000oooo0`0000020?ooo`05
0000003oool000000?ooo`0000006`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool000000P3oool3000000<0oooo0`00000?0?ooo`003@3oool3
000000D0oooo0P0000050?ooo`040000003oool0oooo000000H0oooo0`00000J0?ooo`<000001@3o
ool2000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0I0?ooo`<000001@3o
ool2000000D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`0I0?ooo`<00000
1@3oool2000000D0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00000?0?oo
o`00303oool2000003@0oooo0P00000d0?ooo`800000=@3oool2000002P0oooo000<0?ooo`<00000
<`3oool3000003<0oooo0`00000d0?ooo`<000009`3oool000d0oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool09P3o
ool00=T0oooo00020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00@3oool000<0
oooo00<000000?ooo`3oool03`3oool2000001/0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0?0?ooo`8000006`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00l0
oooo0P00000K0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool2000001/0
oooo00<000000?ooo`3oool00P3oool000@0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo0080oooo00050?ooo`8000003`3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00L0oooo0P00000?0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool01`3oool2000000l0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`070?ooo`8000003`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00<0oooo0007
0?ooo`030000003oool0oooo00/0oooo0`00000E0?ooo`@00000303oool00`000000oooo0?ooo`0;
0?ooo`<000005@3oool4000000`0oooo00<000000?ooo`3oool02`3oool3000001D0oooo1000000<
0?ooo`030000003oool0oooo00/0oooo0`00000E0?ooo`@000001P3oool000P0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3o
ool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3o
ool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool000T0oooo0P00000Q0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0O0?ooo`030000
003oool0oooo00P0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0K0?ooo`<000003@3o
ool2000000H0oooo00<000000?ooo`3oool06`3oool3000000d0oooo0P0000060?ooo`030000003o
ool0oooo01/0oooo0`00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0K0?ooo`<000002`3o
ool00080oooo0`0000070?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool03`3oool3
000000H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0?0?ooo`<000001P3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0oooo0`0000060?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool03@3oool000<0oooo00<000000?ooo`3oool01`3oool20000
01D0oooo00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00H0oooo0P00000E0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`060?ooo`8000005@3oool00`000000oooo000000030?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool01P3oool2000001D0oooo00<000000?ooo`0000000`3oool00`00
0000oooo0?ooo`0:0?ooo`003`3oool00`000000oooo0?ooo`0>0?ooo`@000001P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool4000000D0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0
oooo100000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0?0?ooo`@000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080
oooo000@0?ooo`040000003oool0oooo0?ooo`8000000P3oool3000000<0oooo00<000000?ooo`00
00000P00000?0?ooo`<000004@3oool010000000oooo0?ooo`3oool200000080oooo0`0000030?oo
o`050000003oool000000000003oool00P0000050?ooo`030000003oool0oooo00D0oooo0`00000A
0?ooo`040000003oool0oooo0?ooo`8000000P3oool3000000<0oooo00D000000?ooo`0000000000
0?ooo`02000000D0oooo00<000000?ooo`3oool01@3oool300000140oooo00@000000?ooo`3oool0
oooo0P0000030?ooo`8000000`3oool01@000000oooo000000000000oooo008000001@3oool00`00
0000oooo0?ooo`050?ooo`<000000P3oool00140oooo0`0000000`3oool000000000000200000003
0?ooo`000000000000<0000000D0oooo0000003oool000000?ooo`02000000@0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0000000B0?ooo`<0000000<0oooo0000000000000P0000000`3o
ool000000000000300000080oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`050?oo
o`030000003oool000000180oooo200000000`3oool000000000000300000080oooo00<000000?oo
o`0000001P3oool00`000000oooo0?ooo`050?ooo`030000003oool000000180oooo200000000`3o
ool000000000000300000080oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`050?oo
o`030000003oool000000080oooo000@0?ooo`030000003oool0oooo00d0oooo0P0000020?ooo`80
0000103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool03@3oool4000000D0oooo0P0000070?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool03@3oool4000000D0oooo0P0000070?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool03@3oool4000000D0oooo0P0000070?ooo`030000003oool0oooo0080oooo000>0?ooo`800000
4`3oool3000000@0oooo00<000000?ooo`3oool05`3oool2000001<0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0G0?ooo`8000004`3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo01L0oooo0P00000C0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
2P3oool000d0oooo00<000000?ooo`3oool04`3oool2000000030?ooo`000000oooo0080oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<0
00000?ooo`3oool02P3oool000`0oooo00<000000?ooo`3oool0603oool010000000oooo00000000
000G0?ooo`030000003oool0oooo0180oooo0P0000040?ooo`040000003oool00000000001L0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`020?ooo`040000003oool00000000001L0
oooo00<000000?ooo`3oool04P3oool2000000@0oooo00@000000?ooo`0000000000303oool000X0
oooo0P00000F0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04`3oool2000001D0
oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`0C0?ooo`8000005P3oool2000000H0
oooo00<000000?ooo`3oool04`3oool2000001D0oooo00<000000?ooo`0000001P3oool2000000/0
oooo00090?ooo`8000005`3oool3000000H0oooo0P00000B0?ooo`8000005P3oool2000000P0oooo
0P00000B0?ooo`8000005`3oool2000000L0oooo0P00000B0?ooo`8000005P3oool2000000T0oooo
00<000000?ooo`3oool0203oool000P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0000
00080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`02000000H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo008000001P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool00P0000060?ooo`030000003oool0oooo00L0oooo00070?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01/0oooo0P0000070?ooo`8000003@3oool00`000000oooo0?ooo`0K0?ooo`8000001`3oool2
000000d0oooo00<000000?ooo`3oool06`3oool2000000L0oooo0P0000070?ooo`001P3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`030?ooo`800000203oool2000000/0oooo00<000000?ooo`3oool05P3oool20000
00@0oooo0P0000080?ooo`8000002`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
00<0oooo0P0000080?ooo`8000001P3oool000@0oooo0P00000J0?ooo`030000003oool0oooo00d0
oooo0P0000080?ooo`8000006@3oool2000000l0oooo0P0000080?ooo`8000006@3oool00`000000
oooo0000000>0?ooo`800000203oool2000001T0oooo0`00000?0?ooo`030000003oool0oooo00<0
oooo00030?ooo`030000003oool0oooo01X0oooo0`00000?0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool06@3oool200000080oooo0P00000=0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool06@3oool200000080oooo0P00000=0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool06@3oool010000000oooo0000003oool2000000d0oooo00<000000?ooo`3o
ool00P3oool00080oooo00<000000?ooo`3oool06`3oool00`000000oooo0000000@0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool06P3oool010000000oooo0?ooo`3oool2000000h0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0080
00003P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3o
ool0oooo0P00000>0?ooo`030000003oool0oooo0040oooo000P0?ooo`030000003oool0oooo03H0
oooo0P00000d0?ooo`800000=03oool200000180oooo003I0?ooo`009`3oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0=0?ooo`009`3oool3000003<0oooo0`00000c0?ooo`<00000<`3oool3000000d0oooo000W0?oo
o`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?oo
o`030000003oool0000000d0oooo000W0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`3oool000000P3oool30000
00070?ooo`00000000000?ooo`000000oooo000000020?ooo`040000003oool0000000000200oooo
00@000000?ooo`3oool000000P3oool3000000@0oooo00<000000?ooo`0000000P3oool00`000000
oooo0?ooo`0Q0?ooo`040000003oool0oooo00000080oooo0`0000001`3oool000000000003oool0
00000?ooo`0000000P3oool010000000oooo00000000000P0?ooo`040000003oool0oooo00000080
oooo0`0000000`3oool00000000000020?ooo`030000003oool000000080oooo00@000000?ooo`00
00000000403oool00100oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool6
00000080oooo00@000000?ooo`3oool00000803oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`040000003oool0000000000200
oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool2000000040?ooo`000000
000000000080oooo00<000000?ooo`3oool00P00000O0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00P3oool2000000040?ooo`000000000000000080oooo00<000000?ooo`00
00000P00000?0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`050000003oool0oooo0?oo
o`0000000P3oool300000080oooo00@000000?ooo`3oool00000803oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`040000003o
ool0000000000200oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool20000
00040?ooo`000000000000000080oooo00<000000?ooo`0000008@3oool010000000oooo0?ooo`00
00020?ooo`080000003oool0oooo0?ooo`000000oooo0000003oool300000080oooo00<000000?oo
o`0000004@3oool000l0oooo0`0000050?ooo`8000000P3oool00`000000oooo000000070?ooo`03
0000003oool0oooo01d0oooo0`0000050?ooo`80000000<0oooo0000000000000P00000W0?ooo`<0
00001@3oool200000080oooo00<000000?ooo`0000001`3oool00`000000oooo0?ooo`0M0?ooo`<0
00001@3oool200000080oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`0>0?oo
o`00303oool2000003@0oooo0P00000d0?ooo`800000=@3oool2000002P0oooo000<0?ooo`<00000
<`3oool3000003<0oooo0`00000d0?ooo`<000009`3oool000d0oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool09P3o
ool00=T0oooo000E0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`1E0?ooo`000P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`800000703oool00`000000
oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000001/0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000001/0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000001/0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool00P3oool000@0oooo0P00000?0?ooo`<000006@3oool00`000000oooo0?oo
o`060?ooo`8000003`3oool3000001T0oooo00<000000?ooo`3oool01P3oool2000000l0oooo0`00
000I0?ooo`030000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool00`3oool000H0oooo00<000000?ooo`3oool0903oool4000000/0oooo00<00000
0?ooo`3oool0903oool4000000/0oooo00<000000?ooo`3oool0903oool4000000/0oooo00<00000
0?ooo`3oool0303oool3000001D0oooo100000060?ooo`001`3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`80
00008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`80
00008`3oool00`000000oooo0?ooo`070?ooo`00203oool300000240oooo00<000000?ooo`3oool0
3`3oool300000240oooo00<000000?ooo`3oool03`3oool300000240oooo00<000000?ooo`3oool0
3`3oool300000240oooo00<000000?ooo`3oool0203oool00080oooo0P0000070?ooo`030000003o
ool0oooo01/0oooo0`00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0K0?ooo`<000003@3o
ool2000000H0oooo00<000000?ooo`3oool06`3oool3000000d0oooo0P0000060?ooo`030000003o
ool0oooo01/0oooo0`00000;0?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0?0?ooo`<000001P3oool00`000000oooo0?ooo`0D0?ooo`050000003oool0
oooo0?ooo`0000004@3oool3000000H0oooo00<000000?ooo`3oool0503oool01@000000oooo0?oo
o`3oool000000140oooo0`0000060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
3@3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0000000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01<0oooo0P0000000`3oool000000?ooo`020?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo0000003oool00P3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0503o
ool00`000000oooo000000030?ooo`030000003oool0oooo00X0oooo000>0?ooo`8000004@3oool4
000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`800000403oool5
000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`800000403oool4
000000H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`8000004@3oool0
0`000000oooo000000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool0
0100oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<000000`3oool01`000000oooo0000003o
ool000000?ooo`0000001@3oool00`000000oooo0?ooo`050?ooo`<000004@3oool010000000oooo
0?ooo`3oool200000080oooo0`0000030?ooo`040000003oool00000000000P0oooo00<000000?oo
o`3oool01@3oool300000140oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<000000`3oool0
10000000oooo0000000000080?ooo`030000003oool0oooo00D0oooo0`00000A0?ooo`040000003o
ool0oooo0?ooo`8000000`3oool2000000<0oooo00<000000?ooo`0000000P00000?0?ooo`<00000
0P3oool00140oooo0`0000000`3oool0000000000002000000030?ooo`000000000000<000000P3o
ool3000000H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0000000B0?ooo`<0000000<0
oooo0000000000000P0000000`3oool000000000000300000080oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool000000180oooo200000000`3oool000000000
0003000000030?ooo`000000oooo00P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0000
000B0?ooo`P0000000<0oooo0000000000000`0000020?ooo`80000000<0oooo000000000000103o
ool00`000000oooo0?ooo`050?ooo`030000003oool000000080oooo000@0?ooo`030000003oool0
oooo00d0oooo0`0000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo0P0000070?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool03@3oool2000000P0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo0P0000020?ooo`800000103oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo000>0?ooo`8000004@3oool30000
00H0oooo00<000000?ooo`3oool05`3oool200000100oooo100000060?ooo`030000003oool0oooo
01L0oooo0P00000@0?ooo`80000000<0oooo0000003oool01@3oool00`000000oooo0?ooo`0G0?oo
o`8000004@3oool00`000000oooo00000002000000@0oooo00<000000?ooo`3oool02P3oool000d0
oooo00<000000?ooo`3oool04`3oool2000000030?ooo`000000oooo0080oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0@0?ooo`D0000000<0oooo0000003oool00P3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo0P0000000`3oool000000?ooo`020?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`03
0000003oool0oooo00X0oooo000<0?ooo`030000003oool0oooo01H0oooo00@000000?ooo`000000
oooo0P00000G0?ooo`030000003oool0oooo01@0oooo0P000000103oool000000000003oool20000
01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`02000000030?ooo`0000000000
01L0oooo00<000000?ooo`3oool0603oool010000000oooo00000000000<0?ooo`002P3oool20000
01H0oooo0P0000000`3oool000000?ooo`030?ooo`030000003oool0oooo01<0oooo0P00000F0?oo
o`8000000P3oool01@000000oooo0?ooo`3oool0000001D0oooo0P00000F0?ooo`8000000P3oool0
1@000000oooo0?ooo`3oool0000001D0oooo0P00000F0?ooo`030000003oool0oooo00D0oooo0P00
000;0?ooo`002@3oool2000001L0oooo100000050?ooo`8000004P3oool2000001L0oooo0P000002
0?ooo`030000003oool0oooo0080oooo0P00000B0?ooo`8000005`3oool200000080oooo00<00000
0?ooo`3oool00P3oool200000180oooo0P00000G0?ooo`<000001`3oool00`000000oooo0?ooo`08
0?ooo`00203oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0000000P0oooo00<000000?oo
o`3oool01`3oool000L0oooo00<000000?ooo`3oool0603oool2000000X0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0I0?ooo`8000002@3oool2000000d0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`080?ooo`8000003@3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00T0oooo0P0000070?ooo`001P3oool00`000000oooo0?ooo`0I0?ooo`800000
2`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01X0oooo0P00000:0?ooo`800000
2`3oool00`000000oooo0?ooo`0J0?ooo`8000002P3oool2000000/0oooo00<000000?ooo`3oool0
9P3oool2000000H0oooo00040?ooo`8000006@3oool2000000040?ooo`000000oooo0?ooo`800000
2@3oool2000000P0oooo0P00000J0?ooo`80000000D0oooo0000003oool0oooo000000090?ooo`80
0000203oool2000001X0oooo0P0000000`3oool000000000000;0?ooo`800000203oool2000001X0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo
0?ooo`0I0?ooo`<000000`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool06P3oool2000000@0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo01X0oooo0P0000040?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0J0?ooo`<000003`3oool00`000000oooo
0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0000000@0?ooo`030000003oool0oooo0040oooo000P
0?ooo`800000=03oool2000003X0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0D
0?ooo`00803oool2000003@0oooo0P00000d0?ooo`800000B`3oool00280oooo0P0000030?ooo`03
0000003oool0oooo02l0oooo00D000000?ooo`3oool0oooo0000000^0?ooo`8000001@3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo000R0?ooo`030000003oool0oooo0080
oooo0`00000_0?ooo`040000003oool0oooo0?ooo`<00000;`3oool010000000oooo0?ooo`3oool3
000003<0oooo0`00000=0?ooo`009`3oool00`000000oooo0000000_0?ooo`070000003oool0oooo
0?ooo`000000oooo0000000_0?ooo`070000003oool0oooo0?ooo`000000oooo0000000c0?ooo`03
0000003oool0000000d0oooo000W0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
;`3oool01@000000oooo0?ooo`3oool0000003D0oooo00<000000?ooo`3oool03@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`004@3oool010000000oooo0?ooo`0000020?oo
o`<0000000P0oooo000000000000oooo0000003oool000000?ooo`8000008P3oool010000000oooo
0?ooo`0000020?ooo`<0000000L0oooo000000000000oooo0000003oool000000080oooo00@00000
0?ooo`0000000000803oool010000000oooo0?ooo`0000020?ooo`<00000103oool00`000000oooo
000000020?ooo`040000003oool0oooo00000200oooo00@000000?ooo`3oool000000P3oool30000
00030?ooo`00000000000080oooo00<000000?ooo`0000000P3oool010000000oooo00000000000@
0?ooo`004@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`0000000P0000020?ooo`8000008@3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`050000003oool000000?ooo`000000
7`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80000000@0oooo0000
0000000000000P3oool010000000oooo0?ooo`00000P0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00P3oool2000000040?ooo`000000000000000080oooo00@000000?ooo`3o
ool00000403oool00140oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?oo
o`@0000000@0oooo0000003oool000008@3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`040000003oool0000000000200oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool2000000040?ooo`0000000000
00000080oooo00@000000?ooo`3oool00000803oool010000000oooo0?ooo`0000020?ooo`050000
003oool0oooo0?ooo`0000000`3oool300000080oooo00D000000?ooo`3oool0oooo0000000?0?oo
o`00403oool3000000D0oooo0P0000020?ooo`030000003oool0000000@0oooo00<000000?ooo`3o
ool07`3oool3000000D0oooo0P0000020?ooo`030000003oool0000000L0oooo00<000000?ooo`3o
ool07@3oool3000000D0oooo0P0000040?ooo`030000003oool0oooo02D0oooo0`0000050?ooo`80
00000P3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo00h0oooo000<0?ooo`80
0000=03oool2000003@0oooo0P00000e0?ooo`800000:03oool000`0oooo0`00000c0?ooo`<00000
<`3oool3000003@0oooo0`00000W0?ooo`003@3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0V0?ooo`00f@3oool0
01D0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo
03<0oooo00<000000?ooo`3oool07`3oool00080oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`0000006`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`0000006`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`0000006`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`0000006`3oool00`000000oooo0?ooo`020?ooo`00103oool2000000l0oooo0`00
000I0?ooo`030000003oool0oooo00H0oooo0P00000?0?ooo`<000006@3oool00`000000oooo0?oo
o`060?ooo`8000003`3oool3000001T0oooo00<000000?ooo`3oool01P3oool2000000l0oooo0`00
000I0?ooo`030000003oool0oooo00<0oooo00060?ooo`030000003oool0oooo02@0oooo1000000;
0?ooo`030000003oool0oooo02@0oooo1000000;0?ooo`030000003oool0oooo02@0oooo1000000;
0?ooo`030000003oool0oooo02@0oooo100000060?ooo`001`3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`80
00008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`80
00008`3oool00`000000oooo0?ooo`070?ooo`00203oool300000240oooo00<000000?ooo`3oool0
3`3oool300000240oooo00<000000?ooo`3oool03`3oool300000240oooo00<000000?ooo`3oool0
3`3oool300000240oooo00<000000?ooo`3oool0203oool00080oooo0P0000070?ooo`030000003o
ool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3o
ool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003o
ool0oooo01`0oooo0P00000;0?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0000000?0?ooo`<000001P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
000000l0oooo0`0000060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`0000003`3oool3
000000H0oooo00<000000?ooo`3oool0603oool00`000000oooo0000000=0?ooo`000`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0503oool00`00
0000oooo000000030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`00
0000oooo0?ooo`0:0?ooo`003P3oool200000100oooo100000060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool03`3oool200000140oooo0`0000060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool03`3oool200000100oooo100000060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool03`3oool200000100oooo0P0000000`3oool000000?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00100oooo00@000000?ooo`3oool0
oooo0P0000020?ooo`<000000`3oool00`000000oooo00000002000000l0oooo0`00000A0?ooo`04
0000003oool0oooo0?ooo`8000000P3oool3000000<0oooo00<000000?ooo`0000000P00000?0?oo
o`<000004@3oool010000000oooo0?ooo`3oool200000080oooo0`0000030?ooo`030000003oool0
0000008000003`3oool300000140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`8000000`3o
ool01@000000oooo0000003oool0000000l0oooo0`0000020?ooo`004@3oool3000000030?ooo`00
000000000080000000<0oooo0000000000000P0000020?ooo`040000003oool000000?ooo`800000
103oool00`000000oooo0?ooo`050?ooo`030000003oool000000180oooo0`0000000`3oool00000
00000002000000030?ooo`0000000000008000000`3oool2000000030?ooo`000000000000@0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0000000B0?ooo`P0000000<0oooo000000000000
0P0000020?ooo`040000003oool000000?ooo`800000103oool00`000000oooo0?ooo`050?ooo`03
0000003oool000000180oooo200000000`3oool0000000000003000000030?ooo`00000000000080
oooo0P0000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000000P3oool00100oooo
00<000000?ooo`000000103oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`00
00000`3oool2000000@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0000000@0oooo00D000000?ooo`3oool0oooo000000020?ooo`040000003oool00000
00000080oooo0P0000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo000000040?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0000
00030?ooo`800000103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`000000103oool01@000000oooo0?ooo`3oool0000000@0oooo0P0000020?ooo`800000
103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo000>0?ooo`8000004@3o
ool00`000000oooo00000002000000@0oooo00<000000?ooo`3oool05`3oool200000140oooo00<0
00000?ooo`0000000P0000040?ooo`030000003oool0oooo01L0oooo0P00000A0?ooo`030000003o
ool0000000800000103oool00`000000oooo0?ooo`0G0?ooo`8000004@3oool00`000000oooo0000
0002000000@0oooo00<000000?ooo`3oool02P3oool000d0oooo00<000000?ooo`3oool04`3oool2
000000030?ooo`000000oooo0080oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool02P3oool000`0oooo00<000000?oo
o`3oool0603oool010000000oooo00000000000G0?ooo`030000003oool0oooo01P0oooo00@00000
0?ooo`00000000005`3oool00`000000oooo0?ooo`0H0?ooo`040000003oool00000000001L0oooo
00<000000?ooo`3oool0603oool010000000oooo00000000000<0?ooo`002P3oool2000001H0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0C0?ooo`8000005P3oool4000000@0oooo
00<000000?ooo`3oool04`3oool2000001H0oooo100000040?ooo`030000003oool0oooo01<0oooo
0P00000F0?ooo`@00000103oool2000000/0oooo00090?ooo`8000005`3oool3000000H0oooo0P00
000B0?ooo`8000005`3oool010000000oooo0000000000050?ooo`8000004P3oool2000001L0oooo
00@000000?ooo`00000000001@3oool200000180oooo0P00000G0?ooo`040000003oool000000000
00H0oooo00<000000?ooo`3oool0203oool000P0oooo00<000000?ooo`3oool05`3oool00`000000
oooo000000080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool00`000000
oooo000000080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool00`000000
oooo000000080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool00`000000
oooo000000080?ooo`030000003oool0oooo00L0oooo00070?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01T0oooo
0P0000090?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool06@3oool2000000T0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0I0?ooo`8000002@3oool00`000000oooo
0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<0
00000?ooo`3oool09P3oool2000000H0oooo00040?ooo`8000006P3oool00`000000oooo0?ooo`0=
0?ooo`800000203oool2000001X0oooo1000000<0?ooo`800000203oool2000001X0oooo1000000<
0?ooo`800000203oool2000001X0oooo1000000=0?ooo`030000003oool0oooo00<0oooo00030?oo
o`030000003oool0oooo01X0oooo0`00000?0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool06P3oool010000000oooo00000000000>0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool06P3oool010000000oooo00000000000>0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool06P3oool010000000oooo00000000000>0?ooo`030000003oool0oooo0080oooo
00020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo0?oo
o`010?ooo`00803oool00`000000oooo0?ooo`0d0?ooo`800000=03oool2000003@0oooo0P00000D
0?ooo`00f@3oool002L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo03<0oooo00<000000?ooo`3oool03@3oool002L0oooo0`00000c0?ooo`<00000
<`3oool3000003<0oooo0`00000=0?ooo`009`3oool00`000000oooo0000000c0?ooo`030000003o
ool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000=0?ooo`009`3oool0
0`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0=0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`004@3oool010000000oooo0?ooo`0000020?ooo`<0000000P0oooo000000000000oooo0000
003oool000000?ooo`8000008`3oool010000000oooo0?ooo`0000020?ooo`<0000000P0oooo0000
00000000oooo0000003oool000000?ooo`8000008`3oool010000000oooo0?ooo`0000020?ooo`<0
0000103oool00`000000oooo0000000V0?ooo`040000003oool0oooo00000080oooo0`0000020?oo
o`070000003oool0oooo0000003oool000000?ooo`0200000180oooo000A0?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool00P3oool500000080oooo0P00000R0?ooo`040000003o
ool0oooo00000080oooo00@000000?ooo`3oool0oooo1P000000103oool0000000000000000R0?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002
00000080oooo00<000000?ooo`3oool08@3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00`0000000`3oool000000000000B0?ooo`004@3oool0
10000000oooo0?ooo`0000020?ooo`050000003oool0oooo0?ooo`0000000P3oool3000000030?oo
o`000000oooo02<0oooo00@000000?ooo`3oool000000P3oool01@000000oooo0?ooo`3oool00000
0080oooo0`0000000`3oool000000?ooo`0S0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool08@3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`00
00000`3oool000000000000B0?ooo`00303oool200000080oooo0`0000050?ooo`8000000P3oool0
0`000000oooo000000040?ooo`030000003oool0oooo01`0oooo0P0000020?ooo`<000001@3oool2
00000080oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`0L0?ooo`8000000P3oool3
000000D0oooo0P0000000`3oool0000000000002000000<0oooo0`00000M0?ooo`8000000P3oool3
000000D0oooo0P0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool0
00`0oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000X0?ooo`003@3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0W0?ooo`00f@3oool00;L0oooo00<000000?ooo`3oool07`3oool00080oooo00<00000
0?ooo`3oool0403oool2000001`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@
0?ooo`800000703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo0P00000L
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`0000006`3oool00`000000oooo0?ooo`020?ooo`00103oool200000100oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`060?ooo`800000403oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool01P3oool2000000l0oooo0`00000I0?ooo`030000003oool0oooo00<0oooo00060?ooo`030000
003oool0oooo00`0oooo0`00000G0?ooo`8000002`3oool00`000000oooo0?ooo`0<0?ooo`<00000
5`3oool2000000/0oooo00<000000?ooo`3oool0303oool3000001L0oooo0P00000;0?ooo`030000
003oool0oooo02H0oooo0P0000060?ooo`001`3oool00`000000oooo0?ooo`0S0?ooo`8000003P3o
ool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo0?ooo`0S0?ooo`8000003P3o
ool00`000000oooo0?ooo`0S0?ooo`800000203oool000P0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0
oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000008@3oool00`00
0000oooo0?ooo`0?0?ooo`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<000008@3oool00`00
0000oooo0?ooo`0?0?ooo`<000008@3oool00`000000oooo0?ooo`080?ooo`000P3oool2000000L0
oooo00<000000?ooo`3oool06`3oool3000000d0oooo0P0000060?ooo`030000003oool0oooo01/0
oooo0`00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0K0?ooo`<000003@3oool2000000H0
oooo00<000000?ooo`3oool0703oool2000000/0oooo00020?ooo`<000001`3oool00`000000oooo
0?ooo`0G0?ooo`<0000000<0oooo0000003oool03@3oool3000000H0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`02000000030?ooo`000000oooo00d0oooo0`0000060?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool00P0000000`3oool000000?ooo`0=0?ooo`<00000
1P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0000000d0oooo00030?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0503oool2000000@0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0140oooo00<000000?ooo`0000000`00
00030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0@0?ooo`040000003oool000000?ooo`<000000`3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`030000
003oool0oooo00X0oooo000>0?ooo`8000004@3oool4000000D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0?0?ooo`8000004@3oool4000000D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0?0?ooo`8000004@3oool4000000D0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0?0?ooo`8000004@3oool00`000000oooo000000060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool00P3oool00100oooo00@000000?ooo`3oool0oooo0P00
00030?ooo`030000003oool0oooo0080oooo1@0000000`3oool000000?ooo`040?ooo`030000003o
ool0oooo00D0oooo0`00000A0?ooo`040000003oool0oooo0?ooo`8000000`3oool00`000000oooo
0?ooo`030?ooo`8000000P3oool2000000D0oooo00<000000?ooo`3oool01@3oool300000140oooo
00@000000?ooo`3oool0oooo0P0000030?ooo`030000003oool0oooo00<0oooo0P0000000`3oool0
0000000000060?ooo`030000003oool0oooo00D0oooo0`00000A0?ooo`040000003oool0oooo0?oo
o`8000000`3oool2000000<0oooo00<000000?ooo`0000000P00000?0?ooo`<000000P3oool00140
oooo200000000`3oool0000000000002000000<0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`050?ooo`030000003oool000000180oooo200000000`3oool0000000000003000000<0
oooo0P0000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000004P3oool00`000000
oooo00000005000000030?ooo`000000000000<000000P3oool2000000L0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0000000B0?ooo`030000003oool0000000X000000`3oool200000003
0?ooo`000000000000@0oooo00<000000?ooo`3oool01@3oool00`000000oooo000000020?ooo`00
403oool3000000<0oooo0P0000030?ooo`80000000@0oooo00000000000000002@3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00<000000?ooo`0000000`3oool2000000<0
oooo0P0000020?ooo`@000001P3oool2000000L0oooo00<000000?ooo`3oool04@3oool00`000000
oooo000000030?ooo`8000000`3oool200000080oooo0`0000080?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool04@3oool00`000000oooo000000040?ooo`040000003oool0oooo0?oo
o`80000000<0oooo0000003oool00P0000020?ooo`800000103oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0080oooo000>0?ooo`8000004@3oool2000000L0oooo00<000000?ooo`3o
ool05`3oool200000140oooo0`0000000`3oool000000?ooo`030?ooo`030000003oool0oooo01L0
oooo0P00000@0?ooo`L000000`3oool00`000000oooo0?ooo`0G0?ooo`8000004@3oool00`000000
oooo00000002000000@0oooo00<000000?ooo`3oool02P3oool000d0oooo00<000000?ooo`3oool0
4`3oool3000000@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool00000008000000`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo
0`0000030?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0503oool00`000000oooo
000000030?ooo`030000003oool0oooo00X0oooo000<0?ooo`030000003oool0oooo01L0oooo0`00
00000`3oool000000?ooo`0F0?ooo`030000003oool0oooo01@0oooo0`0000001@3oool000000000
003oool0000001L0oooo00<000000?ooo`3oool04`3oool00`000000oooo00000004000000030?oo
o`000000oooo01H0oooo00<000000?ooo`3oool0603oool010000000oooo00000000000<0?ooo`00
2P3oool2000001H0oooo0P0000020?ooo`030000003oool0oooo008000005@3oool2000001P0oooo
0P0000030?ooo`8000005@3oool2000001L0oooo0P000000103oool000000?ooo`3oool2000001D0
oooo0P00000F0?ooo`030000003oool0oooo00D0oooo0P00000;0?ooo`002@3oool2000001L0oooo
0P0000020?ooo`030000003oool0oooo0080oooo0P00000B0?ooo`800000603oool2000000H0oooo
0P00000B0?ooo`8000006@3oool2000000D0oooo0P00000B0?ooo`8000005`3oool3000000L0oooo
00<000000?ooo`3oool0203oool000P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool2000000T0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0H0?ooo`800000203oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`000000203oool00`000000oooo0?oo
o`070?ooo`001`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool05P3oool010000000oooo00000000000:0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool06@3oool2000000T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool000H0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0603oool3000000030?ooo`000000000000P0oooo0P00000;0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`0000000P00000:0?ooo`8000002`3oool00`000000oooo0?ooo`0V
0?ooo`8000001P3oool000@0oooo0P00000J0?ooo`80000000<0oooo0000003oool02`3oool20000
00P0oooo0P00000L0?ooo`80000000<0oooo0000000000002@3oool2000000P0oooo0P00000K0?oo
o`<00000303oool2000000P0oooo0P00000J0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool00`3oool000<0oooo00<000000?ooo`3oool06P3oool2000000030?ooo`000000oooo00d0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0K0?ooo`8000000P3oool2000000/0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0L0?ooo`8000003P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo01X0oooo0`00000?0?ooo`030000003oool0oooo0080
oooo00020?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo01/0oooo0P00000A0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0703oool200000100oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool000000100oooo00<000000?ooo`3oool00@3oool005H0oooo0P00
000d0?ooo`800000=03oool00`000000oooo0?ooo`0D0?ooo`00F@3oool200000340oooo0P00001;
0?ooo`009`3oool00`000000oooo0?ooo`0_0?ooo`8000000P3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool03@3oool002L0oooo0`00000_0?ooo`80
00000P3oool3000003<0oooo0`00000c0?ooo`<000003@3oool002L0oooo00<000000?ooo`000000
<`3oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000
3@3oool002L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool03@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00140oooo00@000000?ooo`3oool000000P3oool300000080oooo00D0
00000?ooo`000000oooo000000020?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool0
00000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2000002<0oooo00@0
00000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2
000002<0oooo00@000000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`0000005@3o
ool00140oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool00000008000000P3oool00`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool0
8@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`00
00000P0000020?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`3oool200000080oooo0`0000000`3oool000000000000B0?ooo`004@3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P000002
0?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`030?ooo`@000000`3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool0
8@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool30000
00030?ooo`00000000000180oooo000<0?ooo`8000000P3oool3000000D0oooo0P0000040?ooo`03
0000003oool0oooo0240oooo0P0000020?ooo`<000001@3oool200000080oooo00<000000?ooo`00
0000103oool00`000000oooo0?ooo`0L0?ooo`8000000P3oool3000000D0oooo0P0000020?ooo`03
0000003oool0000000@0oooo00<000000?ooo`3oool07@3oool2000000040?ooo`00000000000000
00D0oooo0P0000050?ooo`030000003oool0oooo01D0oooo000<0?ooo`<00000<`3oool3000003<0
oooo0`00000d0?ooo`<000009`3oool000d0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool09P3oool00=T0oooo000E
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`1E
0?ooo`000P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`800000703oool00`000000oooo0?ooo`010?oo
o`000`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000001/0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000001/0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000001/0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool00P3oool000@0oooo0P00000?0?ooo`<000006@3oool00`000000oooo0?ooo`060?ooo`800000
3`3oool3000001T0oooo00<000000?ooo`3oool01P3oool2000000l0oooo0`00000I0?ooo`030000
003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
0`3oool000H0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3o
ool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool0303o
ool3000001P0oooo00<000000?ooo`3oool0103oool000L0oooo00<000000?ooo`3oool08`3oool2
000000h0oooo00<000000?ooo`3oool08`3oool2000000h0oooo00<000000?ooo`3oool08`3oool2
000000h0oooo00<000000?ooo`3oool08`3oool2000000P0oooo00080?ooo`8000008`3oool00`00
0000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`00
0000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`00203oool300000240
oooo00<000000?ooo`3oool03`3oool300000240oooo00<000000?ooo`3oool03`3oool300000240
oooo00<000000?ooo`3oool03`3oool300000240oooo00<000000?ooo`3oool0203oool00080oooo
0P0000070?ooo`030000003oool0oooo01/0oooo0`00000=0?ooo`8000001P3oool00`000000oooo
0?ooo`0K0?ooo`<000003@3oool2000000H0oooo00<000000?ooo`3oool06`3oool3000000d0oooo
0P0000060?ooo`030000003oool0oooo01/0oooo0`00000;0?ooo`000P3oool3000000L0oooo00<0
00000?ooo`3oool05`3oool3000000030?ooo`000000oooo00d0oooo0`0000060?ooo`030000003o
ool0oooo01P0oooo0P0000000`3oool000000?ooo`0=0?ooo`<000001P3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo0080000000<0oooo0000003oool03@3oool3000000H0oooo00<0
00000?ooo`3oool0603oool2000000030?ooo`000000oooo00/0oooo00030?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool04`3oool3000000@0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo01@0oooo0`0000030?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0@0?ooo`040000003oool0
00000?ooo`<000000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0503oool3000000<0oooo00<000000?ooo`3oool02P3oool000h0oooo0P00000A
0?ooo`@000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000A
0?ooo`@000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000A
0?ooo`@000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000A
0?ooo`@000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo000@0?oo
o`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`020?ooo`D0000000<0oooo
0000003oool0103oool00`000000oooo0?ooo`050?ooo`<000004@3oool01@000000oooo0?ooo`3o
ool0000000@0oooo00<000000?ooo`3oool00`3oool4000000030?ooo`000000oooo00@0oooo00<0
00000?ooo`3oool01@3oool300000140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`030000
003oool0oooo00<0oooo0P0000000`3oool00000000000060?ooo`030000003oool0oooo00D0oooo
0`00000A0?ooo`040000003oool0oooo0?ooo`8000000`3oool00`000000oooo0?ooo`030?ooo`@0
000000<0oooo0000003oool0103oool00`000000oooo0?ooo`050?ooo`<000000P3oool00140oooo
00<000000?ooo`0000002P0000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0000000B0?ooo`030000003oool0000000/000000P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000004P3oool00`000000oooo
0000000;00000080oooo0P0000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`000000
4P3oool00`000000oooo00000003000000030?ooo`000000000000D000000`3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000000P3oool00100oooo0`00
00030?ooo`8000000`3oool2000000040?ooo`0000000000000000T0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0000000<0oooo0P0000030?ooo`800000
0P3oool2000000T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`040000
003oool0000000000080oooo0P0000030?ooo`8000000P3oool3000000P0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0A0?ooo`040000003oool0000000000080oooo0P0000030?oo
o`8000000P3oool2000000T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?oo
o`003P3oool200000140oooo0`0000050?ooo`8000006@3oool200000140oooo0`0000060?ooo`03
0000003oool0oooo01L0oooo0P00000@0?ooo`@000001P3oool00`000000oooo0?ooo`0G0?ooo`80
00004@3oool3000000D0oooo0P00000<0?ooo`003@3oool00`000000oooo0?ooo`0B0?ooo`@00000
103oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo0`0000030?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool0503oool3000000<0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0B0?ooo`D000000`3oool00`000000oooo0?ooo`0:0?ooo`00303o
ool00`000000oooo0?ooo`0G0?ooo`<0000000<0oooo0000003oool05P3oool00`000000oooo0?oo
o`0G0?ooo`<0000000<0oooo0000003oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0000000@0000000<0oooo0000003oool05P3oool00`000000oooo0?ooo`0G0?ooo`<0000000<0
oooo0000003oool02`3oool000/0oooo00<000000?ooo`3oool0503oool200000080oooo00<00000
0?ooo`3oool00P00000F0?ooo`030000003oool0oooo01@0oooo0P0000020?ooo`030000003oool0
oooo008000005P3oool00`000000oooo0?ooo`0E0?ooo`@000000P3oool2000001H0oooo00<00000
0?ooo`3oool0503oool200000080oooo00<000000?ooo`3oool00`00000;0?ooo`002@3oool20000
01L0oooo0P0000020?ooo`040000003oool0oooo0?ooo`<000004P3oool2000001L0oooo0P000002
0?ooo`040000003oool0oooo0?ooo`<000004P3oool2000001T0oooo0P0000040?ooo`<000004P3o
ool2000001L0oooo0P0000020?ooo`070000003oool0oooo0?ooo`000000oooo0000000:0?ooo`00
203oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0H0?ooo`800000203oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`001`3o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01T0oooo0P0000090?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`060?ooo`001P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00T0oooo0P00000;0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`0000000P00000:0?ooo`8000002`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo00T0oooo0P0000060?ooo`00103oool2000001X0oooo0P0000000`3o
ool000000?ooo`0;0?ooo`800000203oool2000001X0oooo0P0000000`3oool000000?ooo`0;0?oo
o`800000203oool2000001/0oooo0`00000<0?ooo`800000203oool2000001X0oooo0P0000000`3o
ool000000?ooo`0<0?ooo`030000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo01X0
oooo0P0000000`3oool000000?ooo`0=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool06P3oool2000000030?ooo`000000oooo00d0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0L0?ooo`8000003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01X0
oooo0P0000000`3oool000000?ooo`0=0?ooo`030000003oool0oooo0080oooo00020?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01`0oooo0P00000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0040oooo002<0?ooo`800000B`3o
ool008`0oooo0P00001;0?ooo`00f@3oool002L0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool03@3oool002L0oooo
0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000=0?ooo`009`3oool00`000000oooo0000
000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000
000=0?ooo`009`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0=0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`030000003oool0oooo
00@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F0?ooo`00E03oool3
000000P0oooo0`0000080?ooo`D000001@3oool010000000oooo0?ooo`0000070?ooo`040000003o
ool0oooo000005H0oooo001D0?ooo`<00000203oool3000000/0oooo00<000000?ooo`3oool0103o
ool4000000L0oooo1000001F0?ooo`00M03oool010000000oooo0?ooo`0000070?ooo`040000003o
ool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C14) show
80.562500 14.625000 moveto
(N21) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C15) show
80.562500 14.625000 moveto
(N22) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C16) show
80.562500 14.625000 moveto
(N23) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C17) show
80.562500 14.625000 moveto
(N24) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.68932 .55463 m
.63184 .5534 L
.64407 .51916 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.63795 .46372 m
.69544 .4625 L
.68321 .42825 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C18) show
80.562500 14.625000 moveto
(N25) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .52727 m
.77273 .47273 L
.80909 .47273 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C19) show
80.562500 14.625000 moveto
(N26) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.68932 .55463 m
.63184 .5534 L
.64407 .51916 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.63795 .46372 m
.69544 .4625 L
.68321 .42825 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(u) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C20) show
80.562500 14.625000 moveto
(N27) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .52727 m
.77273 .47273 L
.80909 .47273 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(u) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C21) show
80.562500 14.625000 moveto
(N28) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N29) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N30) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N31) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49456 L
.5424 .48966 L
.54587 .48575 L
.54981 .48317 L
.55428 .48205 L
.55925 .48234 L
.56463 .48379 L
.57027 .48598 L
.576 .4884 L
.58162 .49051 L
.58695 .49181 L
.59185 .49191 L
.59624 .49058 L
.60011 .4878 L
.60352 .48373 L
.6066 .47872 L
.60951 .47324 L
.61245 .46785 L
.6156 .46307 L
.61913 .45933 L
.62315 .45695 L
.62769 .45604 L
.63272 .45652 L
.63815 .45809 L
.64382 .46035 L
.64955 .46276 L
.65513 .46479 L
.66041 .46594 L
.66524 .46584 L
.66955 .4643 L
.67335 .46132 L
.6767 .45708 L
.67975 .45198 L
.68265 .44648 L
.68561 .44115 L
.68881 .43649 L
.6924 .43293 L
.69649 .43076 L
.7011 .43006 L
.7062 .43071 L
.71167 .43241 L
.71736 .43472 L
.72308 .43711 L
.72864 .43904 L
.73386 .44003 L
.73861 .43973 L
.74285 .43798 L
.74658 .4348 L
.74988 .43042 L
Mistroke
.75289 .42523 L
.75579 .41972 L
.75877 .41445 L
.76202 .40992 L
.76568 .40655 L
.76984 .40459 L
.77453 .40409 L
.77969 .40492 L
.7852 .40674 L
.79091 .40909 L
Mfstroke
gsave
.68733 .4934 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N32) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49456 L
.5424 .48966 L
.54587 .48575 L
.54981 .48317 L
.55428 .48205 L
.55925 .48234 L
.56463 .48379 L
.57027 .48598 L
.576 .4884 L
.58162 .49051 L
.58695 .49181 L
.59185 .49191 L
.59624 .49058 L
.60011 .4878 L
.60352 .48373 L
.6066 .47872 L
.60951 .47324 L
.61245 .46785 L
.6156 .46307 L
.61913 .45933 L
.62315 .45695 L
.62769 .45604 L
.63272 .45652 L
.63815 .45809 L
.64382 .46035 L
.64955 .46276 L
.65513 .46479 L
.66041 .46594 L
.66524 .46584 L
.66955 .4643 L
.67335 .46132 L
.6767 .45708 L
.67975 .45198 L
.68265 .44648 L
.68561 .44115 L
.68881 .43649 L
.6924 .43293 L
.69649 .43076 L
.7011 .43006 L
.7062 .43071 L
.71167 .43241 L
.71736 .43472 L
.72308 .43711 L
.72864 .43904 L
.73386 .44003 L
.73861 .43973 L
.74285 .43798 L
.74658 .4348 L
.74988 .43042 L
Mistroke
.75289 .42523 L
.75579 .41972 L
.75877 .41445 L
.76202 .40992 L
.76568 .40655 L
.76984 .40459 L
.77453 .40409 L
.77969 .40492 L
.7852 .40674 L
.79091 .40909 L
Mfstroke
gsave
.68733 .4934 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N33) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N34) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N35) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N36) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N37) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N38) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N39) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
gsave
.63994 .58431 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N40) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`3o
ool000000P3oool300000080oooo00H000000?ooo`000000oooo0000003oool2000002<0oooo00@0
00000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2
000000030?ooo`00000000000200oooo00@000000?ooo`3oool000000P3oool3000000080?ooo`00
000000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200oooo00@000000?oo
o`3oool000000P3oool3000000D0oooo00<000000?ooo`000000103oool200000100oooo000@0?oo
o`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002
00000080oooo00@000000?ooo`3oool00000803oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`040000003oool0000000000200
oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool00000
008000000P3oool00`000000oooo0?ooo`02000001l0oooo00@000000?ooo`3oool000000P3oool0
10000000oooo0?ooo`3oool200000080oooo0`0000001`3oool000000000003oool000000?ooo`00
00003`3oool00100oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool00000008000000P3oool010000000oooo0?ooo`00000P0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool00`3oool400000080oooo00@000000?ooo`0000000000803o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`000000
0P0000020?ooo`050000003oool000000?ooo`0000007`3oool010000000oooo0?ooo`0000020?oo
o`040000003oool0oooo0?ooo`8000000P3oool3000000040?ooo`00000000000?ooo`800000403o
ool000l0oooo0`0000050?ooo`800000103oool00`000000oooo0?ooo`020?ooo`030000003oool0
0000008000007P3oool3000000D0oooo0P0000020?ooo`030000003oool0000000@0oooo00@00000
0?ooo`3oool000007`3oool3000000D0oooo0P0000020?ooo`030000003oool0000000@0oooo00@0
00000?ooo`3oool000007`3oool3000000D0oooo0P0000050?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool03P3oool000`0oooo0P00000d0?ooo`800000=03oool2000003D0oooo0P00
000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03oool3000002L0oooo000=0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0d0?oo
o`030000003oool0oooo02H0oooo003I0?ooo`000P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000
003oool0oooo0040oooo00030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo0P00000K0?ooo`030000003o
ool0oooo0080oooo00040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`0000006`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`0000006`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`0000006`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`020?ooo`001@3oool2000000h0oooo0`00000I0?ooo`030000003oool0oooo
00L0oooo0P00000>0?ooo`<000006@3oool00`000000oooo0?ooo`070?ooo`8000003P3oool30000
01T0oooo00<000000?ooo`3oool01`3oool2000000l0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`030?ooo`001`3oool00`000000oooo0?ooo`0S0?ooo`@00000303oool00`000000
oooo0?ooo`0S0?ooo`@00000303oool00`000000oooo0?ooo`0S0?ooo`@00000303oool00`000000
oooo0?ooo`0;0?ooo`<000005@3oool4000000H0oooo00080?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool01P3oool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00
000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00
000S0?ooo`030000003oool0oooo00L0oooo00090?ooo`8000008@3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`08
0?ooo`000P3oool2000000L0oooo00<000000?ooo`3oool06`3oool3000000d0oooo0P0000060?oo
o`030000003oool0oooo01/0oooo0`00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0K0?oo
o`<000003@3oool2000000H0oooo00<000000?ooo`3oool06`3oool3000000/0oooo00020?ooo`<0
00001`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0oooo0`0000060?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool03`3oool3000000H0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0?0?ooo`<000001P3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00d0oooo00030?ooo`030000003oool0oooo00L0oooo0P00000E0?ooo`030000
003oool0000000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`060?ooo`800000
5@3oool00`000000oooo000000030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
1P3oool2000001D0oooo00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00H0oooo0P00000E0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool0
2P3oool000l0oooo00<000000?ooo`3oool03`3oool3000000H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo100000060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`@000001P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3o
ool00`000000oooo000000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3o
ool00100oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<000000`3oool00`000000oooo0000
0002000000l0oooo0`00000A0?ooo`040000003oool0oooo0?ooo`8000000P3oool3000000<0oooo
00<000000?ooo`0000000P00000?0?ooo`<000004@3oool010000000oooo0?ooo`3oool200000080
oooo0`0000030?ooo`050000003oool000000?ooo`0000003`3oool300000140oooo00@000000?oo
o`3oool0oooo0P0000030?ooo`8000000`3oool00`000000oooo00000003000000H0oooo00<00000
0?ooo`3oool01@3oool300000080oooo000A0?ooo`<0000000<0oooo0000000000000P0000000`3o
ool000000000000300000080oooo0P0000000`3oool00000000000040?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`0000004P3oool3000000030?ooo`00000000000080000000<0oooo0000
000000000`0000001@3oool000000?ooo`000000oooo00800000103oool00`000000oooo0?ooo`05
0?ooo`030000003oool000000180oooo200000000`3oool000000000000600000080oooo0P000004
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000004P3oool8000000030?ooo`000000
000000@0000000@0oooo0000003oool000001P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool000000080oooo000@0?ooo`030000003oool0oooo00`0oooo100000000`3oool0000000000004
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0<
0?ooo`@0000000<0oooo000000000000103oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool0303oool4000000030?ooo`000000000000@0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo10000006
0?ooo`8000001`3oool00`000000oooo0?ooo`020?ooo`003P3oool200000180oooo1@0000030?oo
o`030000003oool0oooo01L0oooo0P00000B0?ooo`D000000`3oool00`000000oooo0?ooo`0G0?oo
o`8000004P3oool5000000<0oooo00<000000?ooo`3oool05`3oool200000180oooo0P0000000`3o
ool00000000000030?ooo`030000003oool0oooo00X0oooo000=0?ooo`030000003oool0oooo01<0
oooo0P0000000`3oool000000?ooo`020?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool04`3oool4000000<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`@0
00000`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo1@0000020?ooo`03
0000003oool0oooo00X0oooo000<0?ooo`030000003oool0oooo01L0oooo1@00000G0?ooo`030000
003oool0oooo01P0oooo1000000G0?ooo`030000003oool0oooo01P0oooo1000000G0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool00P3oool4000000`0oooo000:0?ooo`8000005P3o
ool4000000@0oooo00<000000?ooo`3oool04`3oool2000001H0oooo100000040?ooo`030000003o
ool0oooo01<0oooo0P00000F0?ooo`@00000103oool00`000000oooo0?ooo`0C0?ooo`8000005P3o
ool2000000H0oooo0P00000;0?ooo`002@3oool2000001L0oooo00@000000?ooo`00000000001@3o
ool200000180oooo0P00000G0?ooo`040000003oool00000000000D0oooo0P00000B0?ooo`800000
5`3oool010000000oooo0000000000050?ooo`8000004P3oool2000001L0oooo0P0000080?ooo`03
0000003oool0oooo00P0oooo00080?ooo`030000003oool0oooo01L0oooo00<000000?ooo`000000
203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`000000
203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`000000
203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
0P0000060?ooo`030000003oool0oooo00L0oooo00070?ooo`030000003oool0oooo01T0oooo0P00
00090?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool06@3oool2000000T0oooo0P00
000=0?ooo`030000003oool0oooo01T0oooo0P0000090?ooo`8000003@3oool00`000000oooo0?oo
o`0K0?ooo`8000001`3oool2000000L0oooo00060?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V
0?ooo`8000002`3oool00`000000oooo0?ooo`0L0?ooo`800000203oool2000000H0oooo00040?oo
o`8000006P3oool00`000000oooo0000000=0?ooo`800000203oool2000001X0oooo00<000000?oo
o`0000003@3oool2000000P0oooo0P00000J0?ooo`030000003oool0000000d0oooo0P0000080?oo
o`8000006P3oool2000000l0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3o
ool06@3oool4000000l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0I0?ooo`@0
00003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo1000000?0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo0080oooo00020?ooo`030000003oool0oooo01X0oooo1000000@0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool06P3oool400000100oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0J0?ooo`@00000403oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`010?ooo`007`3o
ool01@000000oooo0000003oool000000340oooo00D000000?ooo`000000oooo0000000a0?ooo`05
0000003oool000000?ooo`000000<P3oool3000001@0oooo003I0?ooo`009`3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0=0?ooo`009`3oool3000003<0oooo0`00000c0?ooo`<00000<`3oool3000000d0oooo000W
0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c
0?ooo`030000003oool0000000d0oooo000W0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`3oool000000P3oool3
000000080?ooo`00000000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200
oooo00@000000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000
003oool2000002<0oooo00@000000?ooo`3oool000000P3oool3000000@0oooo00@000000?ooo`00
0000oooo0P0000000`3oool000000000000P0?ooo`040000003oool0oooo00000080oooo0`000000
0`3oool00000000000020?ooo`040000003oool000000?ooo`80000000<0oooo000000000000403o
ool00100oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`D000000P3o
ool010000000oooo0?ooo`00000P0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3o
ool0oooo1P0000020?ooo`040000003oool0000000000200oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool00000008000000P3oool00`000000oooo0?oo
o`02000001l0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool200000080
oooo0`0000020?ooo`030000003oool00000008000003`3oool00100oooo00@000000?ooo`3oool0
00000P3oool01@000000oooo0?ooo`3oool000000080oooo0`0000020?ooo`040000003oool0oooo
00000200oooo00@000000?ooo`3oool000000P3oool01@000000oooo0?ooo`3oool000000080oooo
0`0000020?ooo`040000003oool0000000000200oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool00000008000000P3oool00`000000oooo0000000Q0?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<000000P3o
ool00`000000oooo0000000A0?ooo`003`3oool3000000D0oooo0P0000020?ooo`030000003oool0
000000@0oooo00@000000?ooo`3oool000007`3oool3000000D0oooo0P0000020?ooo`030000003o
ool0000000@0oooo00<000000?ooo`3oool0803oool3000000D0oooo0P0000000`3oool000000000
0002000000@0oooo00@000000?ooo`3oool000007`3oool3000000D0oooo0P0000020?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool00000403oool000`0oooo0P00000d0?ooo`80
0000=03oool2000003D0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000
=03oool3000002L0oooo000=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo02H0oooo003I0?ooo`00f@3oool00080
oooo00<000000?ooo`3oool0403oool2000001`0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0@0?ooo`800000703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100
oooo0P00000L0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool2000001`0
oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo0080oooo00040?ooo`800000403oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool01P3oool200000100oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`060?ooo`800000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00<0oooo0006
0?ooo`030000003oool0oooo00`0oooo0`00000E0?ooo`@000002`3oool00`000000oooo0?ooo`0<
0?ooo`<000005@3oool4000000/0oooo00<000000?ooo`3oool0303oool3000001D0oooo1000000;
0?ooo`030000003oool0oooo00`0oooo0`00000E0?ooo`@000001P3oool000L0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3o
ool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3o
ool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool000P0oooo0`00000Q0?ooo`030000
003oool0oooo00l0oooo0`00000Q0?ooo`030000003oool0oooo00l0oooo0`00000Q0?ooo`030000
003oool0oooo00l0oooo0`00000Q0?ooo`030000003oool0oooo00P0oooo00020?ooo`8000001`3o
ool00`000000oooo0?ooo`0K0?ooo`<000003@3oool2000000H0oooo00<000000?ooo`3oool06`3o
ool3000000d0oooo0P0000060?ooo`030000003oool0oooo01/0oooo0`00000=0?ooo`8000001P3o
ool00`000000oooo0?ooo`0K0?ooo`<000002`3oool00080oooo0`0000070?ooo`030000003oool0
oooo01H0oooo1000000@0?ooo`<000001P3oool00`000000oooo0?ooo`0F0?ooo`@00000403oool3
000000H0oooo00<000000?ooo`3oool05P3oool400000100oooo0`0000060?ooo`030000003oool0
oooo01H0oooo1@00000=0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0140oooo0P0000001@3oool000000000003oool000000080oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo01<0oooo0`0000001@3oool000000?ooo`3o
ool0000000l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0C0?ooo`<0000000D0
oooo000000000000oooo0000000?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
4`3oool3000000050?ooo`00000000000?ooo`000000303oool000h0oooo0P00000@0?ooo`D00000
1@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000A0?ooo`<00000
1P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000@0?ooo`@00000
1P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000@0?ooo`@00000
1P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo000@0?ooo`040000003o
ool0oooo0?ooo`8000000P3oool300000080oooo1`0000060?ooo`030000003oool0oooo00D0oooo
0`00000A0?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo1P00000?0?ooo`<0
00004@3oool010000000oooo0?ooo`3oool200000080oooo0`0000020?ooo`H000003`3oool30000
0140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`8000000P3oool6000000l0oooo0`000002
0?ooo`00403oool4000000030?ooo`00000000000080000000<0oooo0000000000000`0000020?oo
o`030000003oool0000000H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0000000A0?oo
o`@0000000<0oooo0000000000000P0000000`3oool000000000000300000080oooo0P0000000`3o
ool00000000000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000004@3oool90000
00030?ooo`000000000000<0000000D0oooo0000003oool000000?ooo`02000000@0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0000000A0?ooo`T0000000<0oooo0000000000000`000000
0`3oool00000000000020?ooo`800000103oool00`000000oooo0?ooo`050?ooo`030000003oool0
00000080oooo000@0?ooo`030000003oool0oooo00d0oooo0P0000070?ooo`8000001`3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo0P0000020?ooo`800000103oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool01000
0000oooo0?ooo`3oool2000000@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00d0oooo0P0000020?ooo`800000103oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0080oooo000>0?ooo`8000004@3oool00`000000oooo000000060?oo
o`030000003oool0oooo01L0oooo0P00000A0?ooo`030000003oool0000000800000103oool00`00
0000oooo0?ooo`0G0?ooo`8000004@3oool00`000000oooo00000002000000@0oooo00<000000?oo
o`3oool05`3oool200000140oooo00<000000?ooo`0000000P0000040?ooo`030000003oool0oooo
00X0oooo000=0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`0:0?ooo`00303oool00`000000oooo0?ooo`0H0?ooo`040000003oool00000000001L0
oooo00<000000?ooo`3oool0603oool010000000oooo00000000000G0?ooo`030000003oool0oooo
01P0oooo00@000000?ooo`00000000005`3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0
0000000000`0oooo000:0?ooo`8000005`3oool2000000D0oooo00<000000?ooo`3oool04`3oool2
000001L0oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`0C0?ooo`8000005`3oool0
0`000000oooo000000040?ooo`030000003oool0oooo01<0oooo0P00000G0?ooo`030000003oool0
000000@0oooo0P00000;0?ooo`002@3oool2000001T0oooo00<000000?ooo`3oool0103oool20000
0180oooo0P00000G0?ooo`@000001@3oool200000180oooo0P00000G0?ooo`@000001@3oool20000
0180oooo0P00000G0?ooo`@000001P3oool00`000000oooo0?ooo`080?ooo`00203oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0G0?ooo`@000001`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0
oooo100000070?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool4000000L0
oooo00<000000?ooo`3oool01`3oool000L0oooo00<000000?ooo`3oool06@3oool3000000P0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0H0?ooo`050000003oool000000?ooo`00
00001`3oool2000000d0oooo00<000000?ooo`3oool0603oool01@000000oooo0000003oool00000
00L0oooo0P00000=0?ooo`030000003oool0oooo01P0oooo00D000000?ooo`000000oooo00000007
0?ooo`8000001`3oool000H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?oo
o`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3oool2000000H0oooo00040?ooo`80
00006@3oool3000000h0oooo0P0000080?ooo`8000006P3oool4000000`0oooo0P0000080?ooo`80
00006P3oool4000000`0oooo0P0000080?ooo`8000006P3oool4000000d0oooo00<000000?ooo`3o
ool00`3oool000<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0000000@0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000>0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000>0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000>
0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo01X0oooo00@000000?oo
o`3oool0oooo0P00000>0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0000000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0000000@0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0000000@0?ooo`030000003oool0oooo0040oooo000S0?ooo`800000<P3oool20000
03@0oooo0P00000d0?ooo`800000503oool002<0oooo0P00002d0?ooo`009`3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0=0?ooo`009`3oool3000003<0oooo0`00000c0?ooo`<00000<`3oool3000000d0oooo000W
0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c
0?ooo`030000003oool0000000d0oooo000W0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`3oool000000P3oool3
00000080oooo00H000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200oooo
00@000000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003o
ool2000000030?ooo`00000000000200oooo00@000000?ooo`3oool000000P3oool3000000080?oo
o`00000000000?ooo`000000oooo0000003oool200000080oooo00<000000?ooo`3oool07P3oool0
10000000oooo0?ooo`0000020?ooo`<000001@3oool010000000oooo0000003oool2000000030?oo
o`00000000000100oooo000@0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool00P00000O0?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000200000080
oooo00D000000?ooo`000000oooo0000000O0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00@000000?ooo`3oool00000803o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080
oooo00@000000?ooo`3oool00000403oool00100oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool00000008000000`3oool2000000030?ooo`000000oooo
01h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?ooo`@000000P3oool0
10000000oooo00000000000P0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0000000200000080oooo00@000000?ooo`3oool00000803oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo00D00000
0?ooo`3oool0oooo0000000?0?ooo`003`3oool3000000D0oooo0P0000040?ooo`030000003oool0
oooo0080oooo00@000000?ooo`3oool000007`3oool3000000D0oooo0P0000020?ooo`030000003o
ool0000000@0oooo00@000000?ooo`3oool000007`3oool3000000D0oooo0P0000020?ooo`030000
003oool0000000@0oooo00<000000?ooo`3oool0803oool3000000D0oooo0P0000050?ooo`030000
003oool0oooo0080oooo00@000000?ooo`3oool00000403oool000`0oooo0P00000d0?ooo`800000
=03oool2000003D0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03o
ool3000002L0oooo000=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0d0?ooo`030000003oool0oooo02H0oooo003I0?ooo`005@3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0E@3oool00080oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool0403oool2000001`0oooo00<000000?ooo`3oool00@3oool000<0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0000000K0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0000000K0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0000000K0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0080oooo0004
0?ooo`8000003`3oool3000001T0oooo00<000000?ooo`3oool01P3oool2000000l0oooo0`00000I
0?ooo`030000003oool0oooo00H0oooo0P00000?0?ooo`<000006@3oool00`000000oooo0?ooo`06
0?ooo`800000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00<0oooo00060?oo
o`030000003oool0oooo02@0oooo1000000;0?ooo`030000003oool0oooo02@0oooo1000000;0?oo
o`030000003oool0oooo02@0oooo1000000;0?ooo`030000003oool0oooo00`0oooo0`00000E0?oo
o`@000001P3oool000L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`060?ooo`00
203oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0
3P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0
1`3oool000P0oooo0`00000Q0?ooo`030000003oool0oooo00l0oooo0`00000Q0?ooo`030000003o
ool0oooo00l0oooo0`00000Q0?ooo`030000003oool0oooo00l0oooo0`00000Q0?ooo`030000003o
ool0oooo00P0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0K0?ooo`<000003@3oool2
000000H0oooo00<000000?ooo`3oool06`3oool3000000d0oooo0P0000060?ooo`030000003oool0
oooo01/0oooo0`00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0K0?ooo`<000002`3oool0
0080oooo0`0000070?ooo`030000003oool0oooo01H0oooo100000000`3oool000000?ooo`0=0?oo
o`<000001P3oool00`000000oooo0?ooo`0F0?ooo`@0000000<0oooo0000003oool03@3oool30000
00H0oooo00<000000?ooo`3oool05P3oool4000000030?ooo`000000oooo00d0oooo0`0000060?oo
o`030000003oool0oooo01H0oooo1P00000<0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo01<0oooo0`0000001@3oool000000000003oool0000000l0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0C0?ooo`<0000000D0oooo000000000000oooo0000000?
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04`3oool3000000050?ooo`000000
00000?ooo`0000003`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01@0oooo0P00
00001@3oool000000000003oool0000000`0oooo000>0?ooo`800000403oool4000000H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`800000403oool4000000H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`800000403oool4000000H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`8000004@3oool4000000D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`00403oool010000000oooo0?ooo`3o
ool200000080oooo0`0000020?ooo`H000003`3oool300000140oooo00@000000?ooo`3oool0oooo
0P0000020?ooo`<000000P3oool6000000l0oooo0`00000A0?ooo`040000003oool0oooo0?ooo`80
00000P3oool300000080oooo100000000`3oool000000?ooo`0>0?ooo`<000004@3oool010000000
oooo0?ooo`3oool2000000<0oooo0P0000020?ooo`L000001P3oool00`000000oooo0?ooo`050?oo
o`<000000P3oool00100oooo100000000`3oool0000000000002000000030?ooo`00000000000080
00000`3oool2000000030?ooo`000000000000@0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0000000A0?ooo`@0000000<0oooo0000000000000P0000000`3oool000000000000200000080
oooo00@000000?ooo`000000oooo0P0000040?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`0000004@3oool9000000030?ooo`0000000000008000000P3oool200000080oooo0P0000040?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`0000004@3oool9000000030?ooo`0000000000
00<000000P3oool00`000000oooo000000060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`0000000P3oool00100oooo00<000000?ooo`000000103oool01@000000oooo0?ooo`3oool00000
0080oooo00@000000?ooo`00000000000P3oool2000000@0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0000000@0oooo00D000000?ooo`3oool0oooo0000
00020?ooo`030000003oool0000000<0oooo0P0000040?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo000000040?ooo`050000003oool0oooo0?ooo`000000
0P3oool010000000oooo0000000000020?ooo`800000103oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`000000103oool01@000000oooo0?ooo`3oool00000
00@0oooo0P0000070?ooo`8000001`3oool00`000000oooo0?ooo`020?ooo`003P3oool200000140
oooo00<000000?ooo`0000000P0000040?ooo`030000003oool0oooo01L0oooo0P00000A0?ooo`03
0000003oool0000000800000103oool00`000000oooo0?ooo`0G0?ooo`8000004@3oool00`000000
oooo00000002000000@0oooo00<000000?ooo`3oool05`3oool200000140oooo00<000000?ooo`00
00001P3oool00`000000oooo0?ooo`0:0?ooo`003@3oool00`000000oooo0?ooo`0C0?ooo`800000
00<0oooo0000003oool00P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0:0?ooo`00303oool00`000000oooo0?oo
o`0H0?ooo`040000003oool00000000001L0oooo00<000000?ooo`3oool0603oool010000000oooo
00000000000G0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`00000000005`3oool00`00
0000oooo0?ooo`0H0?ooo`040000003oool00000000000`0oooo000:0?ooo`8000005`3oool00`00
0000oooo000000040?ooo`030000003oool0oooo01<0oooo0P00000G0?ooo`030000003oool00000
00@0oooo00<000000?ooo`3oool04`3oool2000001L0oooo00<000000?ooo`000000103oool00`00
0000oooo0?ooo`0C0?ooo`8000005`3oool2000000D0oooo0P00000;0?ooo`002@3oool2000001L0
oooo100000050?ooo`8000004P3oool2000001L0oooo100000050?ooo`8000004P3oool2000001L0
oooo100000050?ooo`8000004P3oool2000001T0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`080?ooo`00203oool00`000000oooo0?ooo`0G0?ooo`@000001`3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo01L0oooo100000070?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool05`3oool4000000L0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool000L0oooo00<0
00000?ooo`3oool0603oool01@000000oooo0000003oool0000000L0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0H0?ooo`050000003oool000000?ooo`0000001`3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo01P0oooo00D000000?ooo`000000oooo000000070?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool06@3oool3000000P0oooo00<000000?oo
o`3oool01P3oool000H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0303oool2000000H0oooo00040?ooo`800000
6P3oool4000000`0oooo0P0000080?ooo`8000006P3oool4000000`0oooo0P0000080?ooo`800000
6P3oool4000000`0oooo0P0000080?ooo`8000006P3oool2000000l0oooo00<000000?ooo`3oool0
0`3oool000<0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000>0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000>0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000>0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool200000100oooo00<000000?oo
o`3oool00P3oool00080oooo00<000000?ooo`3oool06`3oool00`000000oooo0000000@0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool00`000000oooo0000000@0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool00`000000oooo0000000@0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`02000000h0
oooo00<000000?ooo`3oool00@3oool00240oooo0P00000d0?ooo`800000=03oool2000003H0oooo
0P00000B0?ooo`00a@3oool200000180oooo000W0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo000W0?oo
o`<00000<`3oool3000003<0oooo0`00000c0?ooo`<000003@3oool002L0oooo00<000000?ooo`00
0000<`3oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`00
00003@3oool002L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool03@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool000h0oooo00@000000?ooo`3oool000000P3oool300000080oooo
00@000000?ooo`00000000000P3oool010000000oooo0000003oool2000000030?ooo`0000000000
01`0oooo00@000000?ooo`3oool000000P3oool300000080oooo00@000000?ooo`00000000000P3o
ool010000000oooo0000003oool2000000030?ooo`000000000001`0oooo00@000000?ooo`3oool0
00000P3oool3000000040?ooo`00000000000?ooo`8000000P3oool010000000oooo0000003oool2
000001l0oooo00@000000?ooo`3oool000000P3oool3000000030?ooo`00000000000080oooo00L0
00000?ooo`3oool000000?ooo`000000oooo0080000000<0oooo0000000000003`3oool000h0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool000000000
0080oooo0`0000020?ooo`030000003oool0oooo008000006`3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P000000103oool0000000000000
00020?ooo`030000003oool00000008000006`3oool010000000oooo0?ooo`0000020?ooo`030000
003oool0oooo0080oooo00H000000?ooo`000000oooo0000003oool300000080oooo00@000000?oo
o`3oool00000703oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00@0
00000?ooo`3oool000000P3oool300000080oooo00@000000?ooo`00000000003`3oool000h0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`040000003oool000000000
0080oooo0`0000030?ooo`8000007@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo0080oooo00H000000?ooo`000000oooo0000003oool3000000<0oooo0P00000M0?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool00`3oool300000080oooo0`0000030?ooo`03
0000003oool0000001`0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?oo
o`030000003oool000000080oooo0`0000030?ooo`<000003`3oool000`0oooo100000050?ooo`80
00001@3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000001X0oooo10000005
0?ooo`8000001@3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000001X0oooo
100000050?ooo`8000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080
oooo00<000000?ooo`0000000P00000I0?ooo`@000001@3oool200000080oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000000l0oooo000<0?ooo`<0
0000<`3oool3000003<0oooo0`00000c0?ooo`<00000:03oool000d0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
9`3oool00=T0oooo003I0?ooo`000P3oool00`000000oooo0?ooo`0@0?ooo`800000703oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo0P00000L0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0403oool2000001`0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0@0?ooo`800000703oool00`000000oooo0?ooo`010?ooo`000`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool00P3oool000@0oooo
0P00000@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool01P3oool200000100oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`060?ooo`800000403oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool00`3oool000H0oooo00<000000?ooo`3oool0303oool3000001L0oooo0P00
000;0?ooo`030000003oool0oooo00`0oooo0`00000G0?ooo`8000002`3oool00`000000oooo0?oo
o`0<0?ooo`<000005`3oool2000000/0oooo00<000000?ooo`3oool0303oool3000001L0oooo0P00
00060?ooo`001`3oool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo0?ooo`0S
0?ooo`8000003P3oool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo0?ooo`0S
0?ooo`800000203oool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?oo
o`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?oo
o`030000003oool0oooo00L0oooo00080?ooo`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<0
00008@3oool00`000000oooo0?ooo`0?0?ooo`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<0
00008@3oool00`000000oooo0?ooo`080?ooo`000P3oool2000000L0oooo00<000000?ooo`3oool0
703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`800000
1P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3oool0
703oool2000000/0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0000000l0oooo0`0000060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`0000003`3o
ool3000000H0oooo00<000000?ooo`3oool0603oool00`000000oooo0000000?0?ooo`<000001P3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0000000d0oooo00030?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool04`3oool2000000030?ooo`000000oooo0080oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`0000000`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool04`3oool2000000030?ooo`000000oooo0080oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo01<0oooo0P0000000`3oool000000?ooo`02
0?ooo`030000003oool0oooo00X0oooo000>0?ooo`800000403oool5000000D0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`8000004@3oool4000000D0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`800000403oool5000000D0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`800000403oool2000000030?ooo`000000oooo
00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`00403oool010000000
oooo0?ooo`3oool2000000<0oooo00<000000?ooo`3oool00P3oool00`000000oooo000000030000
00h0oooo0`00000A0?ooo`040000003oool0oooo0?ooo`8000000`3oool00`000000oooo0?ooo`02
0?ooo`070000003oool000000?ooo`000000oooo0000000=0?ooo`<000004@3oool010000000oooo
0?ooo`3oool2000000<0oooo00<000000?ooo`3oool00P3oool01P000000oooo000000000000oooo
000000h0oooo0`00000A0?ooo`040000003oool0oooo0?ooo`8000000`3oool2000000<0oooo00H0
00000?ooo`00000000000?ooo`00000>0?ooo`<000000P3oool00140oooo200000000`3oool00000
00000002000000<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool000000180oooo200000000`3oool0000000000002000000<0oooo0`0000060?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`0000004P3oool00`000000oooo00000005000000030?oo
o`0000000000008000000`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`0000004P3oool00`000000oooo0000000:00000080oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`050?ooo`030000003oool000000080oooo000@0?ooo`<000000`3o
ool2000000<0oooo0P0000000`3oool000000?ooo`02000000L0oooo0P0000070?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`0000000`3oool2000000<0oooo0P0000000`3oool000000?oo
o`03000000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0000000<0oooo0P0000030?ooo`80000000@0oooo0000003oool00000203oool2000000L0oooo
00<000000?ooo`3oool04@3oool00`000000oooo000000040?ooo`040000003oool0oooo0?ooo`80
000000<0oooo0000003oool00`0000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool00P3oool000h0oooo0P00000@0?ooo`@000001P3oool00`000000oooo0?ooo`0G0?ooo`800000
4@3oool3000000H0oooo00<000000?ooo`3oool05`3oool200000100oooo100000060?ooo`030000
003oool0oooo01L0oooo0P00000A0?ooo`<000001P3oool00`000000oooo0?ooo`0:0?ooo`003@3o
ool00`000000oooo0?ooo`0B0?ooo`<0000000<0oooo0000003oool00P3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo0100oooo1@0000000`3oool000000?ooo`020?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool04`3oool2000000030?ooo`000000oooo0080oooo00<0
00000?ooo`3oool02P3oool000`0oooo00<000000?ooo`3oool0503oool01P000000oooo0000003o
ool000000?ooo`8000005`3oool00`000000oooo0?ooo`0G0?ooo`80000000<0oooo000000000000
5`3oool00`000000oooo0?ooo`0D0?ooo`<0000000<0oooo0000003oool00P00000G0?ooo`030000
003oool0oooo01@0oooo00H000000?ooo`000000oooo0000003oool2000000`0oooo000:0?ooo`80
00005`3oool00`000000oooo000000040?ooo`030000003oool0oooo01<0oooo0P00000F0?ooo`80
00000P3oool01@000000oooo0?ooo`3oool0000001D0oooo0P00000F0?ooo`80000000<0oooo0000
003oool00`3oool00`000000oooo0?ooo`0C0?ooo`8000005P3oool2000000030?ooo`000000oooo
00<0oooo0P00000;0?ooo`002@3oool2000001P0oooo00<000000?ooo`0000001@3oool200000180
oooo0P00000G0?ooo`<0000000<0oooo0000003oool00`3oool200000180oooo0P00000G0?ooo`80
000000<0oooo0000003oool0103oool200000180oooo0P00000G0?ooo`80000000<0oooo0000003o
ool01@3oool00`000000oooo0?ooo`080?ooo`00203oool00`000000oooo0?ooo`0G0?ooo`<00000
203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`070?ooo`001`3oool00`000000oooo0?ooo`0H0?ooo`8000002P3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01T0oooo0P0000090?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0603oool2000000X0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00H0oooo00060?ooo`030000003oool0
oooo01T0oooo0P00000;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool06@3oool3
000000X0oooo0P00000;0?ooo`030000003oool0oooo01T0oooo0P00000;0?ooo`8000002`3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00T0oooo0P0000060?ooo`00103oool20000
01/0oooo0P00000=0?ooo`800000203oool2000001T0oooo0P0000000`3oool000000?ooo`020?oo
o`030000003oool0oooo00L0oooo0P0000080?ooo`8000006P3oool2000000030?ooo`000000oooo
008000002@3oool2000000P0oooo0P00000J0?ooo`@000003@3oool00`000000oooo0?ooo`030?oo
o`000`3oool00`000000oooo0?ooo`0K0?ooo`8000003`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01T0oooo100000030?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0J0?ooo`80000000@0oooo0000003oool00000303oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo01X0oooo100000000`3oool000000000000;0?oo
o`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo01/0oooo0`00000@0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0L0?ooo`05
0000003oool0oooo0?ooo`0000003@3oool00`000000oooo0?ooo`010?ooo`00803oool2000003@0
oooo0P00000d0?ooo`800000B`3oool00200oooo0P00000d0?ooo`800000=03oool2000003@0oooo
0P00000E0?ooo`009`3oool00`000000oooo0?ooo`0^0?ooo`8000000`3oool00`000000oooo0?oo
o`0_0?ooo`050000003oool0oooo0?ooo`000000;P3oool2000000D0oooo00<000000?ooo`3oool0
3@3oool002L0oooo0`00000^0?ooo`030000003oool0oooo0080oooo0`00000_0?ooo`040000003o
ool0oooo0?ooo`<00000;`3oool010000000oooo0?ooo`3oool3000000d0oooo000W0?ooo`030000
003oool0000003<0oooo00<000000?ooo`000000;`3oool01`000000oooo0?ooo`3oool000000?oo
o`000000;`3oool01`000000oooo0?ooo`3oool000000?ooo`0000003@3oool002L0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo02l0oooo00D00000
0?ooo`3oool0oooo0000000?0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`003P3oool010000000oooo0?ooo`0000020?ooo`<000000P3oool00`000000oooo0?oo
o`030?ooo`040000003oool000000?ooo`8000000P3oool00`000000oooo0?ooo`0J0?ooo`040000
003oool0oooo00000080oooo0`0000020?ooo`040000003oool0000000000080oooo00@000000?oo
o`000000oooo0P0000000`3oool000000000000L0?ooo`040000003oool0oooo00000080oooo0`00
00020?ooo`040000003oool0000000000080oooo00@000000?ooo`000000oooo0P0000000`3oool0
00000000000L0?ooo`040000003oool0oooo00000080oooo0`0000020?ooo`030000003oool0oooo
00<0oooo00@000000?ooo`000000oooo0P00000B0?ooo`003P3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`00000000000P3oool300000080oooo00@0
00000?ooo`3oool00000703oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P000000103oool000000000000000020?ooo`040000003oool0oooo
000001`0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool000000080000000@0oooo00000000000000000P3oool010000000oooo0?ooo`00000L0?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`000002
0?ooo`<000000P3oool010000000oooo00000000000?0?ooo`003P3oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo0080oooo00@000000?ooo`00000000000P3oool3000000<0oooo
00<000000?ooo`000000703oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080
oooo00<000000?ooo`0000000`3oool3000000<0oooo00@000000?ooo`3oool000006`3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000`3oool3
000000<0oooo00<000000?ooo`000000703oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool000000P3oool3000000<0oooo0`00000?0?ooo`00303o
ool4000000D0oooo0P0000080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06`3o
ool4000000D0oooo0P0000050?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0
00006P3oool4000000D0oooo0P0000050?ooo`040000003oool0oooo000000@0oooo00@000000?oo
o`3oool000006`3oool3000000D0oooo0P0000040?ooo`<0000000<0oooo0000003oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000=03oool30000
02L0oooo000=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0d0?ooo`030000003oool0oooo02H0oooo003I0?ooo`00f@3oool00080oooo00<00000
0?ooo`3oool0403oool2000001`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@
0?ooo`800000703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo0P00000L
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool2000001`0oooo00<00000
0?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo0080oooo00040?ooo`800000403oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool01P3oool200000100oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`060?ooo`80
0000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00<0oooo00060?ooo`030000
003oool0oooo00`0oooo0`00000G0?ooo`8000002`3oool00`000000oooo0?ooo`0<0?ooo`<00000
5`3oool2000000/0oooo00<000000?ooo`3oool0303oool3000001L0oooo0P00000;0?ooo`030000
003oool0oooo00`0oooo0`00000H0?ooo`030000003oool0oooo00@0oooo00070?ooo`030000003o
ool0oooo02<0oooo0P00000>0?ooo`030000003oool0oooo02<0oooo0P00000>0?ooo`030000003o
ool0oooo02<0oooo0P00000>0?ooo`030000003oool0oooo02<0oooo0P0000080?ooo`00203oool2
000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3oool2
000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool01`3oool0
00P0oooo0`00000Q0?ooo`030000003oool0oooo00l0oooo0`00000Q0?ooo`030000003oool0oooo
00l0oooo0`00000Q0?ooo`030000003oool0oooo00l0oooo0`00000Q0?ooo`030000003oool0oooo
00P0oooo00020?ooo`8000001`3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0
oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0
oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000002`3oool00080oooo
0`0000070?ooo`030000003oool0oooo01P0oooo00<000000?ooo`0000003`3oool3000000H0oooo
00<000000?ooo`3oool0603oool00`000000oooo0000000?0?ooo`<000001P3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0000000l0oooo0`0000060?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`0000003@3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0503oool00`000000oooo0000
00030?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool02P3oool000h0oooo0P00000A
0?ooo`@000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000A
0?ooo`<000001P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000@
0?ooo`@000001P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000@
0?ooo`80000000<0oooo0000003oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo0080oooo000@0?ooo`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`04
0?ooo`80000000<0oooo0000000000001@3oool00`000000oooo0?ooo`050?ooo`<000004@3oool0
1@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0103oool3000000L0oooo00<0
00000?ooo`3oool01@3oool300000140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`030000
003oool0oooo00@0oooo0`0000070?ooo`030000003oool0oooo00D0oooo0`00000A0?ooo`040000
003oool0oooo0?ooo`8000000`3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000L0
oooo00<000000?ooo`3oool01@3oool300000080oooo000A0?ooo`030000003oool0000000/00000
0P3oool00`000000oooo000000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`000000
4P3oool00`000000oooo0000000;00000080oooo0P0000000`3oool00000000000040?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`0000004P3oool00`000000oooo0000000;000000050?oo
o`000000oooo0000003oool00P0000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`00
00004P3oool00`000000oooo00000003000000030?ooo`000000000000D0000000<0oooo00000000
00000P3oool2000000@0oooo00<000000?ooo`3oool01@3oool00`000000oooo000000020?ooo`00
403oool3000000<0oooo0P0000030?ooo`80000000D0oooo0000003oool000000?ooo`02000000H0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0000000<0
oooo0P0000030?ooo`80000000<0oooo0000003oool00P0000020?ooo`800000103oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo0140oooo00@000000?ooo`00000000000P3oool20000
00<0oooo0P000000103oool000000?ooo`0000030?ooo`800000103oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0140oooo00@000000?ooo`00000000000P3oool2000000<0oooo0P00
00000`3oool000000?ooo`0200000080oooo0P0000040?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool00P3oool000h0oooo0P00000A0?ooo`030000003oool0000000D0oooo0P00000I
0?ooo`8000004@3oool00`000000oooo00000002000000@0oooo00<000000?ooo`3oool05`3oool2
00000140oooo00<000000?ooo`0000000P0000040?ooo`030000003oool0oooo01L0oooo0P00000A
0?ooo`030000003oool0000000800000103oool00`000000oooo0?ooo`0:0?ooo`003@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool02P3oool000`0
oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`00000G0?ooo`030000003oool0oooo
01P0oooo00@000000?ooo`3oool000005`3oool00`000000oooo0?ooo`0H0?ooo`040000003oool0
oooo000001L0oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`00000<0?ooo`002`3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00D0oooo0P00000F0?ooo`030000003o
ool0oooo01@0oooo100000030?ooo`8000005P3oool00`000000oooo0?ooo`0D0?ooo`@000000`3o
ool2000001H0oooo00<000000?ooo`3oool0503oool4000000<0oooo0`00000;0?ooo`002@3oool2
000001H0oooo0`0000070?ooo`8000004P3oool2000001L0oooo00@000000?ooo`00000000001@3o
ool200000180oooo0P00000G0?ooo`040000003oool00000000000D0oooo0P00000B0?ooo`800000
5`3oool010000000oooo0000000000060?ooo`030000003oool0oooo00P0oooo00080?ooo`030000
003oool0oooo01H0oooo00@000000?ooo`000000oooo0P0000060?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool05`3oool00`000000oooo000000080?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool05`3oool00`000000oooo000000080?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool05`3oool00`000000oooo000000080?ooo`030000003oool0oooo00L0
oooo00070?ooo`030000003oool0oooo01L0oooo00@000000?ooo`3oool0oooo0P0000070?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool06@3oool2000000T0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0I0?ooo`8000002@3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo01T0oooo0P0000090?ooo`030000003oool0oooo00H0oooo00060?ooo`030000
003oool0oooo01H0oooo0P0000040?ooo`800000203oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000
003oool0oooo02H0oooo0P0000060?ooo`00103oool2000001T0oooo00<000000?ooo`0000003P3o
ool2000000P0oooo0P00000J0?ooo`@00000303oool2000000P0oooo0P00000J0?ooo`@00000303o
ool2000000P0oooo0P00000J0?ooo`@000003@3oool00`000000oooo0?ooo`030?ooo`000`3oool0
0`000000oooo0?ooo`0I0?ooo`8000004@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo01X0oooo00@000000?ooo`00000000003P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo01X0oooo00@000000?ooo`00000000003P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo01X0oooo00@000000?ooo`00000000003P3oool00`000000oooo0?ooo`020?ooo`00
0P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo008000003P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`000000403oool00`000000oooo
0?ooo`010?ooo`008`3oool200000380oooo0P00000d0?ooo`800000=03oool2000001@0oooo000S
0?ooo`800000]03oool00=T0oooo000W0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo000W0?ooo`<00000
<`3oool3000003<0oooo0`00000c0?ooo`<000003@3oool002L0oooo00<000000?ooo`000000<`3o
ool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`0000003@3o
ool002L0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo03<0oooo00<000000?ooo`3oool03@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool005@0oooo0`0000080?ooo`<000002`3oool00`000000oooo0?ooo`040?oo
o`040000003oool0oooo000000L0oooo00@000000?ooo`3oool00000EP3oool005@0oooo0`000008
0?ooo`<00000203oool5000000D0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?oo
o`00001F0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`030000003oool0oooo00@0oooo1000
00070?ooo`@00000EP3oool007@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?oo
o`00001F0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo0000\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
gsave
.63994 .58431 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N41) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N42) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N43) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N44) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
gsave
.74795 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N45) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
gsave
.74795 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N46) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
gsave
.74795 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N47) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
gsave
.74795 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N48) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N49) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N50) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N51) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.79526 .41364 L
.79894 .41818 L
.80141 .42273 L
.80227 .42727 L
.80141 .43182 L
.79894 .43636 L
.79526 .44091 L
.79091 .44545 L
.78656 .45 L
.78287 .45455 L
.78041 .45909 L
.77954 .46364 L
.78041 .46818 L
.78287 .47273 L
.78656 .47727 L
.7909 .48182 L
.79525 .48636 L
.79894 .49091 L
.8014 .49545 L
.80227 .5 L
.8014 .50455 L
.79894 .50909 L
.79525 .51364 L
.7909 .51818 L
.78656 .52273 L
.78287 .52727 L
.78041 .53182 L
.77954 .53636 L
.78041 .54091 L
.78287 .54545 L
.78656 .55 L
.79091 .55455 L
.79526 .55909 L
.79894 .56364 L
.80141 .56818 L
.80227 .57273 L
.80141 .57727 L
.79894 .58182 L
.79526 .58636 L
.79091 .59091 L
s
.79091 .47273 m
.77273 .52727 L
.80909 .52727 L
F
gsave
.74795 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.53636 .5 L
s
gsave
.64376 .57361 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .40909 m
.53636 .5 L
s
gsave
.68351 .4827 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N52) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N53) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N54) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N55) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N56) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N57) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.79091 .59091 m
.79091 .40909 L
s
gsave
.75931 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.54207 .49764 L
.54759 .49582 L
.55274 .495 L
.55743 .4955 L
.56159 .49746 L
.56526 .50083 L
.5685 .50536 L
.57148 .51063 L
.57438 .51613 L
.57739 .52133 L
.58069 .52571 L
.58442 .52889 L
.58866 .53064 L
.59342 .53094 L
.59864 .52995 L
.60419 .52802 L
.60991 .52563 L
.6156 .52332 L
.62107 .52162 L
.62617 .52097 L
.63078 .52167 L
.63487 .52384 L
.63847 .52739 L
.64167 .53205 L
.64462 .53739 L
.64753 .54289 L
.65057 .54799 L
.65392 .55222 L
.65772 .55521 L
.66203 .55675 L
.66686 .55685 L
.67214 .5557 L
.67773 .55367 L
.68345 .55126 L
.68912 .549 L
.69455 .54743 L
.69959 .54695 L
.70413 .54786 L
.70814 .55024 L
.71167 .55397 L
.71483 .55876 L
.71777 .56415 L
.72068 .56962 L
.72375 .57463 L
.72716 .57871 L
.73103 .58149 L
.73542 .58282 L
.74032 .58272 L
.74565 .58142 L
Mistroke
.75127 .57931 L
.757 .57688 L
.76265 .57469 L
.76803 .57325 L
.77299 .57296 L
.77746 .57408 L
.7814 .57666 L
.78487 .58057 L
.78799 .58547 L
.79091 .59091 L
Mfstroke
.63795 .53628 m
.68321 .57175 L
.69544 .5375 L
F
gsave
.63994 .58431 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.53636 .5 m
.53929 .49458 L
.54242 .48971 L
.5459 .48582 L
.54985 .48326 L
.55431 .48215 L
.55928 .48244 L
.56465 .48386 L
.57028 .486 L
.57599 .48835 L
.58157 .49037 L
.58686 .49155 L
.59171 .49153 L
.59606 .49009 L
.59991 .48725 L
.60334 .48321 L
.60648 .47839 L
.60953 .47332 L
.61271 .4686 L
.61622 .46481 L
.62021 .46235 L
.6247 .46131 L
.6296 .46139 L
.63464 .46189 L
.63955 .46203 L
.64418 .46137 L
.64858 .46006 L
.65288 .45848 L
.65717 .45688 L
.66148 .45533 L
.6658 .45378 L
.67012 .45226 L
.67445 .45077 L
.67878 .44927 L
.68301 .4475 L
.68701 .44508 L
.69073 .44186 L
.69431 .43828 L
.69805 .43512 L
.70218 .43307 L
.70683 .43246 L
.71194 .43316 L
.71739 .4348 L
.72297 .43679 L
.72845 .43853 L
.73366 .43948 L
.73844 .43925 L
.74272 .4376 L
.74649 .43454 L
.74983 .43028 L
Mistroke
.75287 .42518 L
.7558 .41975 L
.7588 .41452 L
.76205 .41002 L
.76571 .40665 L
.76987 .40468 L
.77455 .40416 L
.7797 .40496 L
.78521 .40676 L
.79091 .40909 L
Mfstroke
.68932 .44537 m
.64407 .48084 L
.63184 .4466 L
F
gsave
.68733 .4934 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.79091 .40909 Mdot
.53636 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N58) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N59) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
gsave
.31649 .5173 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
gsave
.35624 .42639 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N60) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`3o
ool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2000002<0oooo
00@000000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003o
ool2000000030?ooo`00000000000200oooo00@000000?ooo`00000000000P3oool300000080oooo
00H000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200oooo00@000000?oo
o`00000000000P3oool3000000030?ooo`00000000000080oooo00@000000?ooo`000000oooo0P00
00000`3oool000000000000@0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo0080oooo1@0000020?ooo`80000000<0oooo0000003oool07`3oool010000000oooo0?oo
o`0000020?ooo`040000003oool0oooo0?ooo`H000000P3oool4000001T0oooo0P0000050?ooo`04
0000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo000000020000
0080oooo0P0000000`3oool000000000000I0?ooo`800000103oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000000`3oool0000000000002
000000030?ooo`000000oooo00h0oooo000@0?ooo`040000003oool0oooo00000080oooo00D00000
0?ooo`3oool0oooo000000020?ooo`<0000000<0oooo0000003oool00P3oool00`000000oooo0?oo
o`0N0?ooo`040000003oool0oooo00000080oooo00D000000?ooo`3oool0oooo000000020?ooo`<0
000000@0oooo0000003oool0oooo0P00000I0?ooo`<00000103oool01`000000oooo0?ooo`3oool0
00000?ooo`000000103oool00`000000oooo00000002000000070?ooo`000000oooo0?ooo`000000
oooo0000000I0?ooo`<000000`3oool01`000000oooo0?ooo`3oool000000?ooo`0000001@3oool0
0`000000oooo00000002000000040?ooo`000000oooo0?ooo`800000403oool000l0oooo0`000005
0?ooo`8000000P3oool00`000000oooo000000040?ooo`030000003oool00000008000007P3oool3
000000D0oooo0P0000020?ooo`030000003oool0000000@0oooo00@000000?ooo`3oool000006P3o
ool00`000000oooo0?ooo`020?ooo`<0000000<0oooo0000003oool00P3oool2000000@0oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`00000K0?ooo`040000003oool0oooo0?ooo`<0
000000<0oooo0000003oool00P3oool200000080oooo00@000000?ooo`3oool00000103oool01000
0000oooo0?ooo`00000@0?ooo`00303oool2000003@0oooo0P00000l0?ooo`030000003oool0oooo
03<0oooo00<000000?ooo`3oool0803oool000`0oooo0`00000c0?ooo`<00000>`3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo0200oooo000=0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool0=`3oool010000000oooo00000000000a0?ooo`050000003oool000000?oo
o`0000008P3oool007h0oooo0`00000b0?ooo`80000000<0oooo0000003oool08@3oool00080oooo
00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?ooo`8000007P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00d0oooo0P00000O0?ooo`030000003oool0oooo0040oooo
00030?ooo`030000003oool0oooo00l0oooo0P00000K0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03`3oool2000001/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0<0?ooo`8000007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo0P00
000O0?ooo`030000003oool0oooo0080oooo00040?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0oooo0P00000O
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02@3oool200000200oooo00<00000
0?ooo`3oool00P3oool000D0oooo0P00000?0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool01`3oool2000000l0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`070?oo
o`8000001`3oool00`000000oooo00000002000001h0oooo00<000000?ooo`3oool01`3oool20000
00L0oooo00@000000?ooo`00000000007`3oool00`000000oooo0?ooo`030?ooo`001`3oool00`00
0000oooo0?ooo`0;0?ooo`<000005@3oool4000000`0oooo00<000000?ooo`3oool02`3oool30000
01D0oooo1000000:0?ooo`<000001`3oool2000000030?ooo`000000oooo01/0oooo0P00000:0?oo
o`<000001`3oool3000000030?ooo`000000000001/0oooo00<000000?ooo`3oool0103oool000P0
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?ooo`800000703oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo0P0000000`3oool000000000000I0?ooo`03
0000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`800000
8`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0080oooo0`00000L0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool00`3oool200000080oooo0P00000H0?ooo`030000
003oool0oooo00L0oooo00090?ooo`8000008@3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool04@3oool010000000oooo0?ooo`3oool2000001X0oooo
0`00000C0?ooo`040000003oool0oooo0?ooo`8000006P3oool3000000X0oooo00020?ooo`800000
1`3oool00`000000oooo0?ooo`0J0?ooo`@000003@3oool2000000H0oooo00<000000?ooo`3oool0
6P3oool4000001@0oooo0`00000M0?ooo`800000503oool3000000030?ooo`000000000001/0oooo
00<000000?ooo`3oool02@3oool00080oooo0`0000070?ooo`030000003oool0oooo01H0oooo1000
000@0?ooo`<000001P3oool00`000000oooo0?ooo`0F0?ooo`@000006P3oool3000001T0oooo00<0
00000?ooo`3oool05`3oool3000000030?ooo`000000000001H0oooo00<000000?ooo`3oool02`3o
ool000<0oooo00<000000?ooo`3oool01`3oool2000001@0oooo0`0000001@3oool000000?ooo`3o
ool0000000l0oooo00<000000?ooo`3oool01P3oool2000001@0oooo0`0000001@3oool000000?oo
o`3oool0000000/0oooo0P0000000`3oool00000000000060?ooo`030000003oool0oooo0080oooo
0P00000F0?ooo`030000003oool0oooo00/0oooo0P0000090?ooo`030000003oool0oooo0080oooo
0`00000E0?ooo`030000003oool0oooo00`0oooo000?0?ooo`030000003oool0oooo00h0oooo1000
00060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0>0?ooo`@000001P3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo0?ooo`D00000
1P3oool00`000000oooo0?ooo`020?ooo`D000004P3oool00`000000oooo0?ooo`0<0?ooo`<00000
203oool00`000000oooo0?ooo`030?ooo`@000004P3oool00`000000oooo0?ooo`0=0?ooo`00403o
ool010000000oooo0?ooo`3oool200000080oooo0`0000020?ooo`H000003`3oool300000140oooo
00@000000?ooo`3oool0oooo0P0000020?ooo`<000000P3oool4000000030?ooo`000000oooo00h0
oooo0`0000020?ooo`800000203oool00`000000oooo0?ooo`070?ooo`<000003@3oool200000100
oooo0`0000070?ooo`030000003oool0oooo00L0oooo0`00000>0?ooo`030000003oool0oooo00h0
oooo000@0?ooo`@0000000<0oooo0000000000000P0000000`3oool0000000000003000000050?oo
o`000000oooo0000003oool00P0000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`00
00004@3oool4000000030?ooo`00000000000080000000<0oooo0000000000000`0000000`3oool0
0000000000020?ooo`800000103oool00`000000oooo0?ooo`050?ooo`030000003oool000000080
oooo0`0000070?ooo`030000003oool0oooo00X0oooo1`000000103oool000000000003oool20000
00X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00X0oooo1`000000103oool000000000003oool3000000T0oooo00<000000?ooo`3oool01@3oool0
0100oooo00<000000?ooo`3oool0303oool4000000030?ooo`000000000000@0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo100000000`3o
ool00000000000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool20000
00P0oooo00<000000?ooo`3oool01`3oool3000000d0oooo00<000000?ooo`3oool01`3oool30000
00D0oooo0P0000080?ooo`030000003oool0oooo00L0oooo0`00000>0?ooo`030000003oool0oooo
00H0oooo0`0000050?ooo`003P3oool200000180oooo1@0000030?ooo`030000003oool0oooo01L0
oooo0P00000B0?ooo`D000000`3oool00`000000oooo0?ooo`0:0?ooo`800000203oool00`000000
oooo0?ooo`040?ooo`<000004@3oool2000000L0oooo00<000000?ooo`0000001@3oool2000000P0
oooo00<000000?ooo`3oool01@3oool200000180oooo00<000000?ooo`3oool01@3oool00`000000
oooo000000050?ooo`003@3oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo0000003oool0
0P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo100000030?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool00P3oool4000001@0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`080?ooo`8000001@3oool00`000000oooo0?ooo`020?ooo`<00000
5@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo000<0?ooo`030000003o
ool0oooo01L0oooo1@00000G0?ooo`030000003oool0oooo01P0oooo1000000F0?ooo`030000003o
ool00000008000006@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`0000000P00000I0?ooo`030000003oool0oooo00/0oooo000:0?ooo`8000005`3oool0
0`000000oooo000000040?ooo`030000003oool0oooo01<0oooo0P00000G0?ooo`030000003oool0
000000@0oooo00<000000?ooo`3oool04`3oool3000001d0oooo00<000000?ooo`3oool04`3oool3
000001h0oooo00<000000?ooo`3oool02@3oool000T0oooo0P00000G0?ooo`@000001@3oool20000
0180oooo0P00000G0?ooo`@000001@3oool2000001<0oooo00<000000?ooo`3oool07P3oool30000
0180oooo00<000000?ooo`3oool07`3oool2000000T0oooo00080?ooo`030000003oool0oooo01L0
oooo100000070?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool4000000L0
oooo00<000000?ooo`3oool03P3oool200000180oooo00<000000?ooo`3oool03@3oool200000100
oooo0P00000B0?ooo`030000003oool0oooo00d0oooo0P0000090?ooo`001`3oool00`000000oooo
0?ooo`0H0?ooo`050000003oool000000?ooo`0000001`3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo01P0oooo00D000000?ooo`000000oooo000000070?ooo`800000303oool20000
01@0oooo0P00000@0?ooo`030000003oool0oooo00/0oooo0P00000D0?ooo`800000403oool00`00
0000oooo0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool04`3oool200000003
0?ooo`000000000000h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0C0?ooo`80
000000<0oooo0000000000003`3oool00`000000oooo0?ooo`040?ooo`00103oool2000001X0oooo
00<000000?ooo`0000003@3oool2000000P0oooo0P00000J0?ooo`030000003oool0000000d0oooo
0P0000080?ooo`8000005P3oool00`000000oooo0?ooo`02000000l0oooo0P0000080?ooo`800000
5P3oool00`000000oooo0?ooo`0200000100oooo00<000000?ooo`3oool00`3oool000<0oooo00<0
00000?ooo`3oool06@3oool4000000l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0I0?ooo`@000003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo0P00
000A0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool200000140oooo00<0
00000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool06P3oool400000100oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0J0?ooo`@00000403oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^
0?ooo`030000003oool0oooo0040oooo000O0?ooo`050000003oool000000?ooo`000000<@3oool0
1@000000oooo0000003oool0000003H0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0@0?ooo`00T03oool3000003<0oooo0`00000@0?ooo`009`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo0300oooo00<000000?ooo`000000<`3oool00`000000oooo0000000@0?oo
o`009`3oool3000003<0oooo0`00000`0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0403oool002L0oooo00<000000?ooo`000000<`3oool00`000000oooo0000001i0?ooo`009`3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo07T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`3oool000000P3oool300000080oooo
00H000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200oooo00@000000?oo
o`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2000002<0
oooo00@000000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000
003oool2000000030?ooo`00000000000200oooo00@000000?ooo`3oool000000P3oool3000000D0
oooo00@000000?ooo`000000oooo0P0000000`3oool000000000000@0?ooo`00403oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?oo
o`80000000<0oooo0000003oool07`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`0000000P0000020?ooo`@00000803oool010000000oooo0?ooo`00
00020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`80000000<0
oooo0000000000007`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80
00000P3oool300000080oooo1@00000?0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`0000000P0000000`3oool000000?ooo`020?ooo`03
0000003oool0oooo01h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?oo
o`@0000000@0oooo0000003oool0oooo0P00000P0?ooo`040000003oool0oooo00000080oooo00<0
00000?ooo`3oool00P3oool00`000000oooo00000002000000050?ooo`000000oooo0?ooo`000000
8@3oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool30000
00050?ooo`000000oooo0?ooo`0000004@3oool000l0oooo0`0000050?ooo`800000103oool00`00
0000oooo0?ooo`020?ooo`040000003oool0oooo000001l0oooo0`0000050?ooo`8000000P3oool0
0`000000oooo000000040?ooo`030000003oool0oooo0200oooo0`0000050?ooo`8000000P3oool0
0`000000oooo000000040?ooo`040000003oool0oooo000001l0oooo0`0000050?ooo`8000001@3o
ool00`000000oooo0?ooo`020?ooo`040000003oool0oooo00000100oooo000<0?ooo`800000=03o
ool2000003@0oooo0P00000e0?ooo`800000:03oool000`0oooo0`00000c0?ooo`<00000<`3oool3
000003@0oooo0`00000W0?ooo`003@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo
03<0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0V0?ooo`00f@3oool001D0oooo
00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo05D0oooo
00020?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0100oooo0P00000L0?ooo`030000003oool0oooo0040oooo0003
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`0000006`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`0000006`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`0000006`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`02
0?ooo`00103oool2000000l0oooo0`00000I0?ooo`030000003oool0oooo00H0oooo0P00000?0?oo
o`<000006@3oool00`000000oooo0?ooo`060?ooo`8000003`3oool3000001T0oooo00<000000?oo
o`3oool01P3oool200000100oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`030?oo
o`001P3oool00`000000oooo0?ooo`0T0?ooo`@000002`3oool00`000000oooo0?ooo`0T0?ooo`@0
00002`3oool00`000000oooo0?ooo`0T0?ooo`@000002`3oool00`000000oooo0?ooo`0<0?ooo`<0
00005@3oool4000000H0oooo00070?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
1P3oool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003o
ool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003o
ool0oooo00L0oooo00080?ooo`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<000008@3oool0
0`000000oooo0?ooo`0?0?ooo`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<000008@3oool0
0`000000oooo0?ooo`080?ooo`000P3oool2000000L0oooo00<000000?ooo`3oool06`3oool30000
00d0oooo0P0000060?ooo`030000003oool0oooo01/0oooo0`00000=0?ooo`8000001P3oool00`00
0000oooo0?ooo`0K0?ooo`<000003@3oool2000000H0oooo00<000000?ooo`3oool06`3oool30000
00/0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0F0?ooo`@00000403oool3000000H0
oooo00<000000?ooo`3oool05P3oool400000100oooo0`0000060?ooo`030000003oool0oooo01H0
oooo1000000@0?ooo`<000001P3oool00`000000oooo0?ooo`0F0?ooo`D000003@3oool000<0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0C0?ooo`<0000000D0oooo0000003oool0
oooo0000000?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04`3oool300000005
0?ooo`000000oooo0?ooo`0000003`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
01<0oooo0`0000001@3oool000000000003oool0000000l0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0C0?ooo`<0000000D0oooo000000000000oooo0000000<0?ooo`003P3oool2
00000100oooo100000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool2
00000100oooo100000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool2
00000100oooo100000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool2
00000140oooo0`0000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool0
0100oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<000000P3oool6000000l0oooo0`00000A
0?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo1P00000?0?ooo`<000004@3o
ool010000000oooo0?ooo`3oool200000080oooo0`0000020?ooo`@0000000<0oooo0000003oool0
3P3oool300000140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`8000000P3oool6000000l0
oooo0`0000020?ooo`00403oool4000000030?ooo`00000000000080000000<0oooo000000000000
0`0000020?ooo`80000000<0oooo000000000000103oool00`000000oooo0?ooo`050?ooo`030000
003oool000000140oooo100000000`3oool0000000000002000000030?ooo`000000000000<00000
00D0oooo0000003oool000000?ooo`02000000@0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0000000A0?ooo`T0000000<0oooo0000000000001P0000020?ooo`800000103oool00`000000
oooo0?ooo`050?ooo`030000003oool000000140oooo2@0000000`3oool000000000000400000004
0?ooo`00000000000?ooo`800000103oool00`000000oooo0?ooo`050?ooo`030000003oool00000
0080oooo000@0?ooo`030000003oool0oooo00`0oooo100000000`3oool00000000000040?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0<0?ooo`@0
000000<0oooo000000000000103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0303oool4000000030?ooo`000000000000@0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo100000000`3oool0
0000000000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool000h0oooo
0P00000@0?ooo`030000003oool0000000@000000`3oool00`000000oooo0?ooo`0G0?ooo`800000
403oool00`000000oooo00000004000000<0oooo00<000000?ooo`3oool05`3oool200000100oooo
00<000000?ooo`000000100000030?ooo`030000003oool0oooo01L0oooo0P00000@0?ooo`030000
003oool0000000@000000`3oool00`000000oooo0?ooo`0:0?ooo`003@3oool00`000000oooo0?oo
o`0C0?ooo`80000000<0oooo0000003oool00P3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo01<0oooo100000030?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3o
ool4000000<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`D000000P3o
ool00`000000oooo0?ooo`0:0?ooo`00303oool00`000000oooo0?ooo`0G0?ooo`D000005`3oool0
0`000000oooo0?ooo`0H0?ooo`@000005`3oool00`000000oooo0?ooo`0H0?ooo`@000005`3oool0
0`000000oooo0?ooo`0H0?ooo`@00000303oool000X0oooo0P00000G0?ooo`030000003oool00000
00@0oooo00<000000?ooo`3oool04`3oool2000001L0oooo00<000000?ooo`000000103oool00`00
0000oooo0?ooo`0C0?ooo`8000005`3oool00`000000oooo000000040?ooo`030000003oool0oooo
01<0oooo0P00000G0?ooo`030000003oool0000000@0oooo0P00000;0?ooo`002@3oool2000001L0
oooo100000050?ooo`8000004P3oool2000001L0oooo100000050?ooo`8000004P3oool2000001L0
oooo100000050?ooo`8000004P3oool2000001L0oooo100000060?ooo`030000003oool0oooo00P0
oooo00080?ooo`030000003oool0oooo01L0oooo100000070?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool05`3oool4000000L0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0G0?ooo`@000001`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo
100000070?ooo`030000003oool0oooo00L0oooo00070?ooo`030000003oool0oooo01P0oooo00D0
00000?ooo`000000oooo000000070?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
603oool01@000000oooo0000003oool0000000L0oooo0P00000=0?ooo`030000003oool0oooo01P0
oooo00D000000?ooo`000000oooo000000070?ooo`8000003@3oool00`000000oooo0?ooo`0H0?oo
o`050000003oool000000?ooo`0000001`3oool2000000L0oooo00060?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`00
0000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000001P3oool000@0
oooo0P00000J0?ooo`030000003oool0000000d0oooo0P0000080?ooo`8000006P3oool00`000000
oooo0000000=0?ooo`800000203oool2000001X0oooo00<000000?ooo`0000003@3oool2000000P0
oooo0P00000J0?ooo`030000003oool0000000h0oooo00<000000?ooo`3oool00`3oool000<0oooo
00<000000?ooo`3oool06@3oool4000000l0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0I0?ooo`@000003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo
1000000?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool4000000l0oooo
00<000000?ooo`3oool00P3oool00080oooo00<000000?ooo`3oool06P3oool400000100oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0J0?ooo`@00000403oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo01X0oooo1000000@0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool06P3oool400000100oooo00<000000?ooo`3oool00@3oool001l0oooo00D00000
0?ooo`000000oooo0000000a0?ooo`050000003oool000000?ooo`000000<@3oool01@000000oooo
0000003oool000000340oooo00D000000?ooo`000000oooo0000000C0?ooo`00f@3oool002L0oooo
00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool03@3oool002L0oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00
000=0?ooo`009`3oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<00000
0?ooo`000000<`3oool00`000000oooo0000000=0?ooo`009`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0=0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000@0?ooo`040000003oool0oooo
00000080oooo0`0000001`3oool000000000003oool000000?ooo`000000103oool200000200oooo
00@000000?ooo`3oool000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003o
ool2000000030?ooo`00000000000200oooo00@000000?ooo`3oool000000P3oool3000000@0oooo
00@000000?ooo`000000oooo0P0000020?ooo`030000003oool0oooo01h0oooo00@000000?ooo`3o
ool000000P3oool3000000030?ooo`00000000000080oooo00@000000?ooo`000000oooo0P000000
0`3oool000000000000@0?ooo`00403oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo0080oooo1@0000000`3oool00000000000020?ooo`8000007`3oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo0?ooo`H000000P3oool3000000030?ooo`000000oooo01h0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool000000080
00000P3oool2000000030?ooo`000000oooo01l0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`3oool200000080oooo0`0000020?ooo`80000000<0oooo0000003oool03`3oool0
0100oooo00@000000?ooo`3oool000000P3oool01@000000oooo0?ooo`3oool000000080oooo0`00
00001`3oool000000000003oool000000?ooo`0000007`3oool010000000oooo0?ooo`0000020?oo
o`050000003oool0oooo0?ooo`0000000P3oool3000000040?ooo`000000oooo0?ooo`800000803o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`000000
0P0000000`3oool000000?ooo`020?ooo`030000003oool0oooo01h0oooo00@000000?ooo`3oool0
00000P3oool010000000oooo0?ooo`3oool200000080oooo0`0000000`3oool000000?ooo`030?oo
o`030000003oool0oooo00d0oooo000?0?ooo`<000001@3oool200000080oooo00<000000?ooo`00
00001`3oool00`000000oooo0?ooo`0M0?ooo`<000001@3oool200000080oooo00<000000?ooo`00
0000103oool010000000oooo0?ooo`00000O0?ooo`<000001@3oool2000000030?ooo`0000000000
00800000103oool00`000000oooo0?ooo`0P0?ooo`<000001@3oool200000080oooo00@000000?oo
o`3oool00000103oool010000000oooo0?ooo`00000@0?ooo`00303oool2000003@0oooo0P00000d
0?ooo`800000=@3oool2000002P0oooo000<0?ooo`<00000<`3oool3000003<0oooo0`00000d0?oo
o`<000009`3oool000d0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool09P3oool00=T0oooo003I0?ooo`000P3oool0
0`000000oooo0?ooo`0@0?ooo`800000703oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0100oooo0P00000L0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool2
000001`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`800000703oool0
0`000000oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000@0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool01P3oool200000100oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`060?ooo`800000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00H0
oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool00`3oool000H0oooo
00<000000?ooo`3oool0303oool3000001D0oooo1000000;0?ooo`030000003oool0oooo00`0oooo
0`00000E0?ooo`@000002`3oool00`000000oooo0?ooo`0<0?ooo`<000005@3oool4000000/0oooo
00<000000?ooo`3oool0303oool3000001D0oooo100000060?ooo`001`3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00H0oooo00080?ooo`8000008`3oool00`000000oooo0?ooo`0>
0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>
0?ooo`8000008`3oool00`000000oooo0?ooo`070?ooo`00203oool300000240oooo00<000000?oo
o`3oool03`3oool300000240oooo00<000000?ooo`3oool03`3oool300000240oooo00<000000?oo
o`3oool03`3oool300000240oooo00<000000?ooo`3oool0203oool00080oooo0P0000070?ooo`03
0000003oool0oooo01`0oooo0P00000=0?ooo`8000001P3oool00`000000oooo0?ooo`0L0?ooo`80
00003@3oool2000000H0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`03
0000003oool0oooo01`0oooo0P00000;0?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool0
603oool01@000000oooo0000003oool0000000d0oooo0`0000060?ooo`030000003oool0oooo01P0
oooo00D000000?ooo`000000oooo0000000=0?ooo`<000001P3oool00`000000oooo0?ooo`0H0?oo
o`050000003oool000000?ooo`0000003@3oool3000000H0oooo00<000000?ooo`3oool0603oool0
1@000000oooo0000003oool0000000/0oooo00030?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0503oool00`000000oooo000000030?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P00000>
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0503oool00`000000oooo00000003
0?ooo`8000003P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`0000000`3oool2000000/0oooo000>0?ooo`8000004@3oool00`000000oooo000000060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool200000140oooo00<000000?oo
o`0000001P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo0P00000A0?oo
o`030000003oool0000000H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0?0?oo
o`8000004@3oool00`000000oooo000000060?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool00P3oool00100oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<000000`3oool00`00
0000oooo00000003000000H0oooo0P0000060?ooo`<000004@3oool010000000oooo0?ooo`3oool2
00000080oooo0`0000030?ooo`030000003oool0000000<000001P3oool2000000H0oooo0`00000A
0?ooo`040000003oool0oooo0?ooo`8000000P3oool3000000<0oooo00<000000?ooo`0000000`00
00060?ooo`8000001P3oool300000140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`800000
0`3oool00`000000oooo00000003000000H0oooo0P0000060?ooo`<000000P3oool00140oooo0`00
00000`3oool0000000000002000000030?ooo`0000000000008000000P3oool4000000H0oooo0P00
00060?ooo`030000003oool000000180oooo0`0000000`3oool0000000000002000000030?ooo`00
00000000008000000P3oool4000000H0oooo0P0000060?ooo`030000003oool000000180oooo2000
00000`3oool000000000000200000080oooo100000060?ooo`8000001P3oool00`000000oooo0000
000B0?ooo`P0000000<0oooo0000000000000`0000000`3oool0000000000002000000H0oooo0P00
00060?ooo`030000003oool000000080oooo000@0?ooo`030000003oool0000000@0oooo00D00000
0?ooo`3oool0oooo000000020?ooo`070000003oool000000?ooo`000000oooo000000040?ooo`80
00001`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0000000@0oooo00D000000?ooo`3o
ool0oooo000000020?ooo`070000003oool000000?ooo`000000oooo000000040?ooo`<000001P3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0000000@0oooo00D000000?ooo`3oool0oooo
000000020?ooo`070000003oool000000?ooo`000000oooo000000040?ooo`<000001P3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0000000@0oooo00D000000?ooo`3oool0oooo00000004
0?ooo`050000003oool000000?ooo`000000103oool3000000H0oooo00<000000?ooo`3oool00P3o
ool000h0oooo0P00000A0?ooo`030000003oool0000000H0oooo00<000000?ooo`3oool05`3oool2
00000140oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`0G0?ooo`8000004@3oool0
0`000000oooo000000060?ooo`030000003oool0oooo01L0oooo0P00000A0?ooo`030000003oool0
000000H0oooo00<000000?ooo`3oool02P3oool000d0oooo00<000000?ooo`3oool0503oool00`00
0000oooo000000030?ooo`8000005`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool00000
00<0oooo0P00000G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool20000
01L0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`8000002`3oool000`0
oooo00<000000?ooo`3oool04P3oool2000000@0oooo00D000000?ooo`000000oooo0000000F0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0000003oool0
000001H0oooo00<000000?ooo`3oool04P3oool2000000@0oooo00D000000?ooo`000000oooo0000
000F0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00`3oool01@000000oooo0000
003oool0000000/0oooo000:0?ooo`8000005@3oool00`000000oooo000000060?ooo`030000003o
ool0oooo01<0oooo0P00000F0?ooo`8000001P3oool2000001@0oooo0P00000E0?ooo`030000003o
ool0000000H0oooo0P00000D0?ooo`8000005@3oool3000000H0oooo0P00000;0?ooo`002@3oool2
000001H0oooo0P0000080?ooo`8000004P3oool2000001L0oooo0P0000070?ooo`8000004P3oool2
000001H0oooo0P0000080?ooo`8000004P3oool2000001H0oooo00<000000?ooo`000000203oool0
0`000000oooo0?ooo`080?ooo`00203oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
008000001P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool00P0000060?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`02000000H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0F0?oo
o`040000003oool0oooo0?ooo`8000001P3oool00`000000oooo0?ooo`070?ooo`001`3oool00`00
0000oooo0?ooo`0K0?ooo`8000001`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
01/0oooo0P0000070?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool06`3oool20000
00L0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0K0?ooo`8000001`3oool00`00
0000oooo0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00<0
oooo0P0000080?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05P3oool2000000@0
oooo0P0000080?ooo`8000002`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00<0
oooo0P0000080?ooo`8000002`3oool00`000000oooo0?ooo`0F0?ooo`800000103oool2000000P0
oooo0P0000060?ooo`00103oool2000001T0oooo0P00000?0?ooo`800000203oool2000001T0oooo
00<000000?ooo`0000003P3oool2000000P0oooo0P00000I0?ooo`<000003P3oool2000000P0oooo
0P00000I0?ooo`030000003oool0000000l0oooo00<000000?ooo`3oool00`3oool000<0oooo00<0
00000?ooo`3oool06@3oool200000140oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0I0?ooo`8000004@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`000000403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo0P00
000A0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo01X0oooo00@00000
0?ooo`3oool0oooo0P00000>0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`02000000h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0J0?ooo`040000003oool0oooo0?ooo`8000003P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00P00000>0?ooo`030000003oool0oooo0040oooo
000S0?ooo`800000=03oool2000003@0oooo0P00000d0?ooo`8000004P3oool002<0oooo0P00000d
0?ooo`800000=03oool2000003@0oooo0P00000B0?ooo`009`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0=0?oo
o`009`3oool3000003<0oooo0`00000c0?ooo`<00000<`3oool3000000d0oooo000W0?ooo`030000
003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000
003oool0000000d0oooo000W0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00d0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo000@0?ooo`040000003oool0oooo00000080oooo0`00
00020?ooo`050000003oool000000?ooo`000000103oool200000200oooo00@000000?ooo`3oool0
00000P3oool3000000070?ooo`00000000000?ooo`000000oooo000000040?ooo`800000803oool0
10000000oooo0?ooo`0000020?ooo`<0000000L0oooo000000000000oooo0000003oool0000002H0
oooo00@000000?ooo`3oool000000P3oool3000000D0oooo00<000000?ooo`000000103oool20000
0100oooo000@0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`00
0000oooo00000002000000030?ooo`00000000000080oooo0P00000O0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002000000040?ooo`000000
00000?ooo`<000007`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo
00<000000?ooo`0000000P0000000`3oool00000000000020?ooo`030000003oool0oooo01h0oooo
00@000000?ooo`3oool000000P3oool010000000oooo0?ooo`3oool200000080oooo0`000000103o
ool000000000003oool200000100oooo000@0?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo00000002000000050?ooo`00000000000?ooo`0000008@3o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00<0oooo100000001@3oool00000
0000003oool000000240oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool000000080000000<0oooo0000000000000P3oool00`000000oooo0?ooo`0N0?oo
o`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<0000000@0
oooo000000000000oooo0P00000@0?ooo`00303oool2000000040?ooo`0000000000000000D0oooo
0P0000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool06P3oool2000000040?oo
o`0000000000000000D0oooo0P0000020?ooo`030000003oool0000000L0oooo00<000000?ooo`3o
ool06P3oool2000000040?ooo`0000000000000000D0oooo0P0000020?ooo`030000003oool00000
00H0oooo0`00000K0?ooo`80000000@0oooo00000000000000001@3oool2000000D0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0>0?ooo`00303oool3000003<0oooo0`00000c0?oo
o`<00000<`3oool3000002P0oooo000=0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo02L0oooo003I0?ooo`005@3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo08/0oooo00020?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0100oooo0P00000L0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool2
000001`0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0000000K0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0000000K0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo0080oooo00040?ooo`8000003`3oool3000001T0
oooo00<000000?ooo`3oool01P3oool2000000l0oooo0`00000I0?ooo`030000003oool0oooo00H0
oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool01P3oool200000100
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo
0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo
0?ooo`0<0?ooo`<000005`3oool2000000/0oooo00<000000?ooo`3oool0303oool3000001L0oooo
0P0000060?ooo`001`3oool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo0?oo
o`0S0?ooo`8000003P3oool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo0?oo
o`0S0?ooo`800000203oool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S
0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S
0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000008@3oool00`000000oooo0?ooo`0?0?oo
o`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<000008@3oool00`000000oooo0?ooo`0?0?oo
o`<000008@3oool00`000000oooo0?ooo`080?ooo`000P3oool2000000L0oooo00<000000?ooo`3o
ool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?ooo`80
00001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?ooo`3o
ool0703oool2000000/0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0H0?ooo`050000
003oool000000?ooo`0000003@3oool3000000H0oooo00<000000?ooo`3oool0603oool01@000000
oooo0000003oool0000000d0oooo0`0000060?ooo`030000003oool0oooo01P0oooo00D000000?oo
o`000000oooo0000000=0?ooo`<000001P3oool00`000000oooo0?ooo`0H0?ooo`050000003oool0
00000?ooo`0000002`3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0B
0?ooo`050000003oool000000?ooo`0000000`3oool2000000h0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P00000>0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool04P3oool01@000000oooo0000003oool0000000<0oooo
0P00000>0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0503oool00`000000oooo
000000030?ooo`8000002`3oool000h0oooo0P00000A0?ooo`<000001P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00l0oooo0P00000A0?ooo`<000001P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00l0oooo0P00000A0?ooo`<000001P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00l0oooo0P00000A0?ooo`<000001P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo0080oooo000@0?ooo`040000003oool0oooo0?ooo`8000000`3o
ool00`000000oooo0?ooo`020?ooo`060000003oool000000000003oool000001P3oool00`000000
oooo0?ooo`050?ooo`<000004@3oool010000000oooo0?ooo`3oool2000000<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo000000020?ooo`030000003oool0oooo00@0oooo0P0000060?oo
o`<000004@3oool010000000oooo0?ooo`3oool2000000<0oooo00<000000?ooo`3oool00P3oool0
1P000000oooo000000000000oooo000000H0oooo0P0000060?ooo`<000004@3oool010000000oooo
0?ooo`3oool2000000<0oooo0P0000030?ooo`030000003oool000000080oooo00<000000?ooo`3o
ool0103oool2000000H0oooo0`0000020?ooo`004@3oool8000000030?ooo`000000000000800000
103oool3000000H0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000B0?ooo`P00000
00<0oooo0000000000000P0000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
103oool00`000000oooo0000000B0?ooo`030000003oool0000000D0000000<0oooo000000000000
0P0000040?ooo`<000001P3oool00`000000oooo0?ooo`040?ooo`030000003oool000000180oooo
00<000000?ooo`0000002P0000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
103oool00`000000oooo000000020?ooo`00403oool3000000<0oooo0P0000030?ooo`80000000@0
oooo0000003oool000002@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0140oooo
00<000000?ooo`0000000`3oool2000000<0oooo0P0000001@3oool000000?ooo`000000oooo00<0
00001@3oool2000000H0oooo00<000000?ooo`3oool04@3oool00`000000oooo000000030?ooo`80
00000`3oool2000000040?ooo`000000oooo000000T0oooo0P0000060?ooo`030000003oool0oooo
0140oooo00<000000?ooo`000000103oool010000000oooo0?ooo`3oool2000000050?ooo`000000
oooo0000003oool00`0000050?ooo`8000001P3oool00`000000oooo0?ooo`020?ooo`003P3oool2
00000140oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`0G0?ooo`8000004@3oool0
0`000000oooo000000060?ooo`030000003oool0oooo01L0oooo0P00000A0?ooo`030000003oool0
000000H0oooo00<000000?ooo`3oool05`3oool200000140oooo00<000000?ooo`0000001P3oool0
0`000000oooo0?ooo`0:0?ooo`003@3oool00`000000oooo0?ooo`0C0?ooo`80000000<0oooo0000
003oool00P3oool2000001L0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?oo
o`8000005`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000000<0oooo0P00000G0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`0000000`3oool2000000/0oooo000<0?ooo`03
0000003oool0oooo01P0oooo00D000000?ooo`000000oooo0000000F0?ooo`030000003oool0oooo
01P0oooo00D000000?ooo`000000oooo0000000F0?ooo`030000003oool0oooo01P0oooo00D00000
0?ooo`000000oooo0000000F0?ooo`030000003oool0oooo01P0oooo00D000000?ooo`000000oooo
0000000;0?ooo`002P3oool2000001H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`0C0?ooo`8000005P3oool00`000000oooo0?ooo`050?ooo`800000503oool2000001H0oooo00<0
00000?ooo`3oool01@3oool2000001@0oooo0P00000F0?ooo`030000003oool0oooo00D0oooo0P00
000;0?ooo`002@3oool2000001L0oooo0`0000060?ooo`8000004P3oool2000001L0oooo0`000006
0?ooo`8000004P3oool2000001L0oooo0`0000060?ooo`8000004P3oool2000001L0oooo0`000007
0?ooo`030000003oool0oooo00P0oooo00080?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`000000203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`000000203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`000000203oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`000000203oool00`000000oooo0?ooo`070?ooo`001`3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo0P00000;0?ooo`030000
003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo0P0000060?ooo`00103o
ool2000001X0oooo00<000000?ooo`3oool03@3oool2000000P0oooo0P00000J0?ooo`030000003o
ool0oooo00d0oooo0P0000080?ooo`8000006P3oool00`000000oooo0?ooo`0=0?ooo`800000203o
ool2000001X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`030?ooo`000`3oool0
0`000000oooo0?ooo`0J0?ooo`<000003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo01X0oooo0`00000?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool3
000000l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0J0?ooo`<000003`3oool0
0`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool00000
0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool00000
0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool00000
0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool00000
0100oooo00<000000?ooo`3oool00@3oool00200oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0503oool00=T0oooo
000W0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo00d0oooo000W0?ooo`<00000<`3oool3000003<0oooo0`00000c
0?ooo`<000003@3oool002L0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?oo
o`030000003oool0000003<0oooo00<000000?ooo`0000003@3oool002L0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool03@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo
00@000000?ooo`3oool000000P3oool3000000070?ooo`00000000000?ooo`000000oooo00000005
0?ooo`030000003oool0oooo01h0oooo00@000000?ooo`3oool000000P3oool3000000070?ooo`00
000000000?ooo`000000oooo000000040?ooo`800000803oool010000000oooo0?ooo`0000020?oo
o`<00000103oool00`000000oooo000000040?ooo`800000803oool010000000oooo0?ooo`000002
0?ooo`<0000000<0oooo0000000000000P3oool00`000000oooo0000000F0?ooo`00403oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo1@0000000`3oool0000000000002
0?ooo`030000003oool0oooo01h0oooo00@000000?ooo`3oool000000P3oool010000000oooo0?oo
o`3oool6000000030?ooo`00000000000080oooo00<000000?ooo`3oool07P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000000`3oool0
0000000000020?ooo`030000003oool0oooo01h0oooo00@000000?ooo`3oool000000P3oool01000
0000oooo0?ooo`3oool200000080oooo0`000000103oool000000000003oool200000100oooo000@
0?ooo`040000003oool0oooo00000080oooo00D000000?ooo`3oool0oooo000000020?ooo`<00000
00<0oooo0000000000000P3oool00`000000oooo0?ooo`0N0?ooo`040000003oool0oooo00000080
oooo00D000000?ooo`3oool0oooo000000020?ooo`<0000000<0oooo0000000000000`3oool00`00
0000oooo0?ooo`0M0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo00000002000000030?ooo`00000000000080oooo00<000000?ooo`3oool07P3oool0
10000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`8000000P3oool3000000040?oo
o`00000000000?ooo`800000403oool000`0oooo0P000000103oool000000000000000050?ooo`80
00000P3oool00`000000oooo0000000T0?ooo`80000000@0oooo00000000000000001@3oool20000
0080oooo00<000000?ooo`0000001`3oool00`000000oooo0?ooo`0J0?ooo`80000000@0oooo0000
0000000000001@3oool2000000030?ooo`0000000000008000001`3oool00`000000oooo0?ooo`0K
0?ooo`D000001@3oool200000080oooo00@000000?ooo`3oool00000603oool000`0oooo0`00000c
0?ooo`<00000<`3oool3000003@0oooo0`00000W0?ooo`003@3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo03<0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0V0?oo
o`00f@3oool00=T0oooo00020?ooo`030000003oool0oooo0100oooo0P00000L0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0403oool2000001`0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0@0?ooo`800000703oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0100oooo0P00000L0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`020?ooo`00103oool20000
0100oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`060?ooo`800000403oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool01P3oool200000100oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`0<0?ooo`<000005`3oool2000000/0
oooo00<000000?ooo`3oool0303oool3000001L0oooo0P00000;0?ooo`030000003oool0oooo00`0
oooo0`00000G0?ooo`8000002`3oool00`000000oooo0?ooo`0<0?ooo`<00000603oool00`000000
oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo
0?ooo`0S0?ooo`8000003P3oool00`000000oooo0?ooo`0S0?ooo`8000003P3oool00`000000oooo
0?ooo`0S0?ooo`800000203oool000P0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00
000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00
000S0?ooo`030000003oool0oooo00L0oooo00080?ooo`<000008@3oool00`000000oooo0?ooo`0?
0?ooo`<000008@3oool00`000000oooo0?ooo`0?0?ooo`<000008@3oool00`000000oooo0?ooo`0?
0?ooo`<000008@3oool00`000000oooo0?ooo`080?ooo`000P3oool2000000L0oooo00<000000?oo
o`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0oooo01`0oooo0P00000=0?oo
o`8000001P3oool00`000000oooo0?ooo`0L0?ooo`8000003@3oool2000000H0oooo00<000000?oo
o`3oool0703oool2000000/0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0000000l0oooo0`0000060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`00
00003`3oool3000000H0oooo00<000000?ooo`3oool0603oool00`000000oooo0000000?0?ooo`<0
00001P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0000000d0oooo00030?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0000000<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0503oool00`000000oooo000000030?ooo`030000
003oool0oooo00X0oooo000>0?ooo`8000004@3oool00`000000oooo000000060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool03`3oool200000140oooo0`0000060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool03`3oool200000100oooo100000060?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool03`3oool200000100oooo0P0000000`3oool000000?oo
o`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00100oooo00D00000
0?ooo`3oool0oooo000000040?ooo`030000003oool0oooo00@0oooo100000060?ooo`030000003o
ool0oooo00D0oooo0`00000A0?ooo`050000003oool0oooo0?ooo`000000103oool00`000000oooo
0?ooo`040?ooo`<000001`3oool00`000000oooo0?ooo`050?ooo`<000004@3oool010000000oooo
0?ooo`3oool2000000<0oooo00<000000?ooo`3oool0103oool3000000L0oooo00<000000?ooo`3o
ool01@3oool300000140oooo00@000000?ooo`3oool0oooo0P0000030?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`0000001`3oool00`000000oooo0?ooo`050?ooo`<000000P3oool00140
oooo00<000000?ooo`0000002`0000020?ooo`030000003oool0000000H0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0000000B0?ooo`030000003oool0000000/000000P3oool200000003
0?ooo`000000000000@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0000000B0?ooo`03
0000003oool0000000/0000000D0oooo0000003oool000000?ooo`02000000@0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0000000B0?ooo`030000003oool0000000<0000000<0oooo0000
000000001@0000000`3oool00000000000020?ooo`800000103oool00`000000oooo0?ooo`050?oo
o`030000003oool000000080oooo000@0?ooo`<000000`3oool2000000<0oooo0P0000000`3oool0
000000000003000000L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0000000<0oooo0P0000030?ooo`80000000@0oooo000000000000oooo0P0000000`3o
ool00000000000040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool01000
0000oooo0000000000020?ooo`8000000`3oool200000080oooo100000000`3oool0000000000004
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool010000000oooo00000000
00020?ooo`8000000`3oool200000080oooo00D000000?ooo`00000000000?ooo`02000000@0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`003P3oool2000000l0oooo0P00
00000`3oool00000000000050?ooo`8000006@3oool2000000l0oooo0P0000000`3oool000000000
0002000000@0oooo00<000000?ooo`3oool05`3oool2000000l0oooo0P0000000`3oool000000000
0002000000@0oooo00<000000?ooo`3oool05`3oool2000000l0oooo0P0000000`3oool000000000
0002000000@0oooo00<000000?ooo`3oool02P3oool000d0oooo00<000000?ooo`3oool04`3oool4
000000<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`@000000`3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo100000030?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool04`3oool4000000<0oooo00<000000?ooo`3oool02P3oool0
00`0oooo00<000000?ooo`3oool05`3oool5000001L0oooo00<000000?ooo`3oool05`3oool50000
01L0oooo00<000000?ooo`3oool05`3oool5000001L0oooo00<000000?ooo`3oool0603oool40000
00`0oooo000;0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool00`3oool200000003
0?ooo`000000oooo01D0oooo00<000000?ooo`3oool0503oool4000000050?ooo`00000000000?oo
o`0000005P3oool00`000000oooo0?ooo`0D0?ooo`@0000000D0oooo000000000000oooo0000000F
0?ooo`030000003oool0oooo01@0oooo10000000103oool000000000003oool2000000/0oooo0009
0?ooo`8000005P3oool3000000L0oooo0P00000B0?ooo`8000005`3oool010000000oooo00000000
00050?ooo`8000004P3oool2000001L0oooo00@000000?ooo`00000000001@3oool200000180oooo
0P00000G0?ooo`040000003oool00000000000H0oooo00<000000?ooo`3oool0203oool000P0oooo
00<000000?ooo`3oool05P3oool010000000oooo0000003oool2000000H0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0000000P0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0000000P0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0000000P0oooo00<000000?ooo`3o
ool01`3oool000L0oooo00<000000?ooo`3oool05`3oool010000000oooo0?ooo`3oool2000000L0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0I0?ooo`8000002@3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo01T0oooo0P0000090?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool06@3oool2000000T0oooo00<000000?ooo`3oool01P3oool000H0oooo
00<000000?ooo`3oool0703oool2000000P0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0V0?ooo`8000002`3oool00`000000oooo0?ooo`0V0?ooo`8000002`3oool00`000000oooo
0?ooo`0V0?ooo`8000001P3oool000@0oooo0P00000J0?ooo`8000003P3oool2000000P0oooo0P00
000J0?ooo`030000003oool0000000d0oooo0P0000080?ooo`8000006P3oool00`000000oooo0000
000=0?ooo`800000203oool2000001X0oooo00<000000?ooo`0000003P3oool00`000000oooo0?oo
o`030?ooo`000`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0I0?ooo`@000003`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo01T0oooo1000000?0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool06@3oool4000000l0oooo00<000000?ooo`3oool00P3oool00080oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool06P3oool400000100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0J0?oo
o`@00000403oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01X0oooo1000000@0?oo
o`030000003oool0oooo0040oooo000P0?ooo`<00000<P3oool01@000000oooo0000003oool00000
0340oooo00D000000?ooo`000000oooo0000000a0?ooo`050000003oool000000?ooo`0000004`3o
ool00=T0oooo003I0?ooo`009`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0=0?ooo`009`3oool3000003<0oooo
0`00000c0?ooo`<00000<`3oool3000000d0oooo000W0?ooo`030000003oool0000003<0oooo00<0
00000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000003oool0000000d0oooo000W
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo00d0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo001D0?ooo`<00000203oool3000000/0oooo00<000000?ooo`3oool0103oool01000
0000oooo0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3
000000P0oooo1@0000050?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool00000
EP3oool005@0oooo0`0000080?ooo`<000002`3oool00`000000oooo0?ooo`040?ooo`@000001`3o
ool4000005H0oooo001d0?ooo`040000003oool0oooo000000L0oooo00@000000?ooo`3oool00000
EP3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N61) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
gsave
.31649 .5173 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
gsave
.35624 .42639 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N62) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N63) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
gsave
.17749 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N64) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N65) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
gsave
.17749 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N66) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.46071 .50544 L
.4576 .51034 L
.45413 .51425 L
.45019 .51683 L
.44572 .51795 L
.44075 .51766 L
.43537 .51621 L
.42973 .51402 L
.424 .5116 L
.41838 .50949 L
.41305 .50819 L
.40815 .50809 L
.40376 .50942 L
.39989 .5122 L
.39648 .51627 L
.3934 .52128 L
.39049 .52676 L
.38755 .53215 L
.3844 .53693 L
.38087 .54067 L
.37685 .54305 L
.37231 .54396 L
.36728 .54348 L
.36185 .54191 L
.35618 .53965 L
.35045 .53724 L
.34487 .53521 L
.33959 .53406 L
.33476 .53416 L
.33045 .5357 L
.32665 .53868 L
.3233 .54292 L
.32025 .54802 L
.31735 .55352 L
.31439 .55885 L
.31119 .56351 L
.3076 .56707 L
.30351 .56924 L
.2989 .56994 L
.2938 .56929 L
.28833 .56759 L
.28264 .56528 L
.27692 .56289 L
.27136 .56096 L
.26614 .55997 L
.26139 .56027 L
.25715 .56202 L
.25342 .5652 L
.25012 .56958 L
Mistroke
.24711 .57477 L
.24421 .58028 L
.24123 .58555 L
.23798 .59008 L
.23432 .59345 L
.23016 .59541 L
.22547 .59591 L
.22031 .59508 L
.2148 .59326 L
.20909 .59091 L
Mfstroke
gsave
.31267 .5066 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.45793 .50236 L
.45241 .50418 L
.44726 .505 L
.44257 .5045 L
.43841 .50254 L
.43474 .49917 L
.4315 .49464 L
.42852 .48937 L
.42562 .48387 L
.42261 .47867 L
.41931 .47429 L
.41558 .47111 L
.41134 .46936 L
.40658 .46906 L
.40136 .47005 L
.39581 .47198 L
.39009 .47437 L
.3844 .47668 L
.37893 .47838 L
.37383 .47903 L
.36922 .47833 L
.36513 .47616 L
.36153 .47261 L
.35833 .46795 L
.35538 .46261 L
.35247 .45711 L
.34943 .45201 L
.34608 .44778 L
.34228 .44479 L
.33797 .44325 L
.33314 .44315 L
.32786 .4443 L
.32227 .44633 L
.31655 .44874 L
.31088 .451 L
.30545 .45257 L
.30041 .45305 L
.29587 .45214 L
.29186 .44976 L
.28833 .44603 L
.28517 .44124 L
.28223 .43585 L
.27932 .43038 L
.27625 .42537 L
.27284 .42129 L
.26897 .41851 L
.26458 .41718 L
.25968 .41728 L
.25435 .41858 L
Mistroke
.24873 .42069 L
.243 .42312 L
.23735 .42531 L
.23197 .42675 L
.22701 .42704 L
.22254 .42592 L
.2186 .42334 L
.21513 .41943 L
.21201 .41453 L
.20909 .40909 L
Mfstroke
gsave
.36006 .41569 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N67) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.46364 .5 m
.46071 .50544 L
.4576 .51034 L
.45413 .51425 L
.45019 .51683 L
.44572 .51795 L
.44075 .51766 L
.43537 .51621 L
.42973 .51402 L
.424 .5116 L
.41838 .50949 L
.41305 .50819 L
.40815 .50809 L
.40376 .50942 L
.39989 .5122 L
.39648 .51627 L
.3934 .52128 L
.39049 .52676 L
.38755 .53215 L
.3844 .53693 L
.38087 .54067 L
.37685 .54305 L
.37231 .54396 L
.36728 .54348 L
.36185 .54191 L
.35618 .53965 L
.35045 .53724 L
.34487 .53521 L
.33959 .53406 L
.33476 .53416 L
.33045 .5357 L
.32665 .53868 L
.3233 .54292 L
.32025 .54802 L
.31735 .55352 L
.31439 .55885 L
.31119 .56351 L
.3076 .56707 L
.30351 .56924 L
.2989 .56994 L
.2938 .56929 L
.28833 .56759 L
.28264 .56528 L
.27692 .56289 L
.27136 .56096 L
.26614 .55997 L
.26139 .56027 L
.25715 .56202 L
.25342 .5652 L
.25012 .56958 L
Mistroke
.24711 .57477 L
.24421 .58028 L
.24123 .58555 L
.23798 .59008 L
.23432 .59345 L
.23016 .59541 L
.22547 .59591 L
.22031 .59508 L
.2148 .59326 L
.20909 .59091 L
Mfstroke
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.45793 .50236 L
.45241 .50418 L
.44726 .505 L
.44257 .5045 L
.43841 .50254 L
.43474 .49917 L
.4315 .49464 L
.42852 .48937 L
.42562 .48387 L
.42261 .47867 L
.41931 .47429 L
.41558 .47111 L
.41134 .46936 L
.40658 .46906 L
.40136 .47005 L
.39581 .47198 L
.39009 .47437 L
.3844 .47668 L
.37893 .47838 L
.37383 .47903 L
.36922 .47833 L
.36513 .47616 L
.36153 .47261 L
.35833 .46795 L
.35538 .46261 L
.35247 .45711 L
.34943 .45201 L
.34608 .44778 L
.34228 .44479 L
.33797 .44325 L
.33314 .44315 L
.32786 .4443 L
.32227 .44633 L
.31655 .44874 L
.31088 .451 L
.30545 .45257 L
.30041 .45305 L
.29587 .45214 L
.29186 .44976 L
.28833 .44603 L
.28517 .44124 L
.28223 .43585 L
.27932 .43038 L
.27625 .42537 L
.27284 .42129 L
.26897 .41851 L
.26458 .41718 L
.25968 .41728 L
.25435 .41858 L
Mistroke
.24873 .42069 L
.243 .42312 L
.23735 .42531 L
.23197 .42675 L
.22701 .42704 L
.22254 .42592 L
.2186 .42334 L
.21513 .41943 L
.21201 .41453 L
.20909 .40909 L
Mfstroke
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N68) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.46071 .50544 L
.4576 .51034 L
.45413 .51425 L
.45019 .51683 L
.44572 .51795 L
.44075 .51766 L
.43537 .51621 L
.42973 .51402 L
.424 .5116 L
.41838 .50949 L
.41305 .50819 L
.40815 .50809 L
.40376 .50942 L
.39989 .5122 L
.39648 .51627 L
.3934 .52128 L
.39049 .52676 L
.38755 .53215 L
.3844 .53693 L
.38087 .54067 L
.37685 .54305 L
.37231 .54396 L
.36728 .54348 L
.36185 .54191 L
.35618 .53965 L
.35045 .53724 L
.34487 .53521 L
.33959 .53406 L
.33476 .53416 L
.33045 .5357 L
.32665 .53868 L
.3233 .54292 L
.32025 .54802 L
.31735 .55352 L
.31439 .55885 L
.31119 .56351 L
.3076 .56707 L
.30351 .56924 L
.2989 .56994 L
.2938 .56929 L
.28833 .56759 L
.28264 .56528 L
.27692 .56289 L
.27136 .56096 L
.26614 .55997 L
.26139 .56027 L
.25715 .56202 L
.25342 .5652 L
.25012 .56958 L
Mistroke
.24711 .57477 L
.24421 .58028 L
.24123 .58555 L
.23798 .59008 L
.23432 .59345 L
.23016 .59541 L
.22547 .59591 L
.22031 .59508 L
.2148 .59326 L
.20909 .59091 L
Mfstroke
gsave
.31267 .5066 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.45793 .50236 L
.45241 .50418 L
.44726 .505 L
.44257 .5045 L
.43841 .50254 L
.43474 .49917 L
.4315 .49464 L
.42852 .48937 L
.42562 .48387 L
.42261 .47867 L
.41931 .47429 L
.41558 .47111 L
.41134 .46936 L
.40658 .46906 L
.40136 .47005 L
.39581 .47198 L
.39009 .47437 L
.3844 .47668 L
.37893 .47838 L
.37383 .47903 L
.36922 .47833 L
.36513 .47616 L
.36153 .47261 L
.35833 .46795 L
.35538 .46261 L
.35247 .45711 L
.34943 .45201 L
.34608 .44778 L
.34228 .44479 L
.33797 .44325 L
.33314 .44315 L
.32786 .4443 L
.32227 .44633 L
.31655 .44874 L
.31088 .451 L
.30545 .45257 L
.30041 .45305 L
.29587 .45214 L
.29186 .44976 L
.28833 .44603 L
.28517 .44124 L
.28223 .43585 L
.27932 .43038 L
.27625 .42537 L
.27284 .42129 L
.26897 .41851 L
.26458 .41718 L
.25968 .41728 L
.25435 .41858 L
Mistroke
.24873 .42069 L
.243 .42312 L
.23735 .42531 L
.23197 .42675 L
.22701 .42704 L
.22254 .42592 L
.2186 .42334 L
.21513 .41943 L
.21201 .41453 L
.20909 .40909 L
Mfstroke
gsave
.36006 .41569 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N69) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.46364 .5 L
s
gsave
.57273 .5316 -66.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.46364 .5 m
.46071 .50544 L
.4576 .51034 L
.45413 .51425 L
.45019 .51683 L
.44572 .51795 L
.44075 .51766 L
.43537 .51621 L
.42973 .51402 L
.424 .5116 L
.41838 .50949 L
.41305 .50819 L
.40815 .50809 L
.40376 .50942 L
.39989 .5122 L
.39648 .51627 L
.3934 .52128 L
.39049 .52676 L
.38755 .53215 L
.3844 .53693 L
.38087 .54067 L
.37685 .54305 L
.37231 .54396 L
.36728 .54348 L
.36185 .54191 L
.35618 .53965 L
.35045 .53724 L
.34487 .53521 L
.33959 .53406 L
.33476 .53416 L
.33045 .5357 L
.32665 .53868 L
.3233 .54292 L
.32025 .54802 L
.31735 .55352 L
.31439 .55885 L
.31119 .56351 L
.3076 .56707 L
.30351 .56924 L
.2989 .56994 L
.2938 .56929 L
.28833 .56759 L
.28264 .56528 L
.27692 .56289 L
.27136 .56096 L
.26614 .55997 L
.26139 .56027 L
.25715 .56202 L
.25342 .5652 L
.25012 .56958 L
Mistroke
.24711 .57477 L
.24421 .58028 L
.24123 .58555 L
.23798 .59008 L
.23432 .59345 L
.23016 .59541 L
.22547 .59591 L
.22031 .59508 L
.2148 .59326 L
.20909 .59091 L
Mfstroke
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.45793 .50236 L
.45241 .50418 L
.44726 .505 L
.44257 .5045 L
.43841 .50254 L
.43474 .49917 L
.4315 .49464 L
.42852 .48937 L
.42562 .48387 L
.42261 .47867 L
.41931 .47429 L
.41558 .47111 L
.41134 .46936 L
.40658 .46906 L
.40136 .47005 L
.39581 .47198 L
.39009 .47437 L
.3844 .47668 L
.37893 .47838 L
.37383 .47903 L
.36922 .47833 L
.36513 .47616 L
.36153 .47261 L
.35833 .46795 L
.35538 .46261 L
.35247 .45711 L
.34943 .45201 L
.34608 .44778 L
.34228 .44479 L
.33797 .44325 L
.33314 .44315 L
.32786 .4443 L
.32227 .44633 L
.31655 .44874 L
.31088 .451 L
.30545 .45257 L
.30041 .45305 L
.29587 .45214 L
.29186 .44976 L
.28833 .44603 L
.28517 .44124 L
.28223 .43585 L
.27932 .43038 L
.27625 .42537 L
.27284 .42129 L
.26897 .41851 L
.26458 .41718 L
.25968 .41728 L
.25435 .41858 L
Mistroke
.24873 .42069 L
.243 .42312 L
.23735 .42531 L
.23197 .42675 L
.22701 .42704 L
.22254 .42592 L
.2186 .42334 L
.21513 .41943 L
.21201 .41453 L
.20909 .40909 L
Mfstroke
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N70) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N71) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N72) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
gsave
.31649 .5173 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
gsave
.35624 .42639 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N73) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
gsave
.17749 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N74) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N75) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
gsave
.17749 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N76) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.21344 .41364 L
.21713 .41818 L
.21959 .42273 L
.22045 .42727 L
.21959 .43182 L
.21712 .43636 L
.21344 .44091 L
.20909 .44545 L
.20474 .45 L
.20105 .45455 L
.19859 .45909 L
.19772 .46364 L
.19859 .46818 L
.20105 .47273 L
.20474 .47727 L
.20909 .48182 L
.21344 .48636 L
.21712 .49091 L
.21959 .49545 L
.22045 .5 L
.21959 .50455 L
.21712 .50909 L
.21344 .51364 L
.20909 .51818 L
.20474 .52273 L
.20105 .52727 L
.19859 .53182 L
.19772 .53636 L
.19859 .54091 L
.20105 .54545 L
.20474 .55 L
.20909 .55455 L
.21344 .55909 L
.21712 .56364 L
.21959 .56818 L
.22045 .57273 L
.21959 .57727 L
.21713 .58182 L
.21344 .58636 L
.20909 .59091 L
s
gsave
.16613 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N77) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.21344 .41364 L
.21713 .41818 L
.21959 .42273 L
.22045 .42727 L
.21959 .43182 L
.21712 .43636 L
.21344 .44091 L
.20909 .44545 L
.20474 .45 L
.20105 .45455 L
.19859 .45909 L
.19772 .46364 L
.19859 .46818 L
.20105 .47273 L
.20474 .47727 L
.20909 .48182 L
.21344 .48636 L
.21712 .49091 L
.21959 .49545 L
.22045 .5 L
.21959 .50455 L
.21712 .50909 L
.21344 .51364 L
.20909 .51818 L
.20474 .52273 L
.20105 .52727 L
.19859 .53182 L
.19772 .53636 L
.19859 .54091 L
.20105 .54545 L
.20474 .55 L
.20909 .55455 L
.21344 .55909 L
.21712 .56364 L
.21959 .56818 L
.22045 .57273 L
.21959 .57727 L
.21713 .58182 L
.21344 .58636 L
.20909 .59091 L
s
gsave
.16613 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N78) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.21344 .41364 L
.21713 .41818 L
.21959 .42273 L
.22045 .42727 L
.21959 .43182 L
.21712 .43636 L
.21344 .44091 L
.20909 .44545 L
.20474 .45 L
.20105 .45455 L
.19859 .45909 L
.19772 .46364 L
.19859 .46818 L
.20105 .47273 L
.20474 .47727 L
.20909 .48182 L
.21344 .48636 L
.21712 .49091 L
.21959 .49545 L
.22045 .5 L
.21959 .50455 L
.21712 .50909 L
.21344 .51364 L
.20909 .51818 L
.20474 .52273 L
.20105 .52727 L
.19859 .53182 L
.19772 .53636 L
.19859 .54091 L
.20105 .54545 L
.20474 .55 L
.20909 .55455 L
.21344 .55909 L
.21712 .56364 L
.21959 .56818 L
.22045 .57273 L
.21959 .57727 L
.21713 .58182 L
.21344 .58636 L
.20909 .59091 L
s
gsave
.16613 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N79) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.21344 .41364 L
.21713 .41818 L
.21959 .42273 L
.22045 .42727 L
.21959 .43182 L
.21712 .43636 L
.21344 .44091 L
.20909 .44545 L
.20474 .45 L
.20105 .45455 L
.19859 .45909 L
.19772 .46364 L
.19859 .46818 L
.20105 .47273 L
.20474 .47727 L
.20909 .48182 L
.21344 .48636 L
.21712 .49091 L
.21959 .49545 L
.22045 .5 L
.21959 .50455 L
.21712 .50909 L
.21344 .51364 L
.20909 .51818 L
.20474 .52273 L
.20105 .52727 L
.19859 .53182 L
.19772 .53636 L
.19859 .54091 L
.20105 .54545 L
.20474 .55 L
.20909 .55455 L
.21344 .55909 L
.21712 .56364 L
.21959 .56818 L
.22045 .57273 L
.21959 .57727 L
.21713 .58182 L
.21344 .58636 L
.20909 .59091 L
s
gsave
.16613 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N80) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00
000000000P3oool300000080oooo00D000000?ooo`000000oooo0000000V0?ooo`040000003oool0
000000000080oooo0`0000001`3oool000000000003oool000000?ooo`000000103oool200000200
oooo00@000000?ooo`00000000000P3oool3000000070?ooo`00000000000?ooo`000000oooo0000
00040?ooo`800000803oool010000000oooo0000000000020?ooo`<000001@3oool010000000oooo
0000003oool2000000030?ooo`00000000000100oooo00090?ooo`8000001@3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`04
0000003oool0oooo000001T0oooo0P0000050?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00@000000?ooo`00000000006@3o
ool2000000D0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000
003oool00000008000000P3oool00`000000oooo0?ooo`02000001T0oooo0P0000040?ooo`040000
003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000020?ooo`<0000000L0oooo0000
00000000oooo0000003oool0000000l0oooo00090?ooo`<00000103oool01`000000oooo0?ooo`3o
ool000000?ooo`000000103oool00`000000oooo0000000200000080oooo00@000000?ooo`3oool0
00006@3oool3000000@0oooo00L000000?ooo`3oool0oooo0000003oool0000000D0oooo10000002
0?ooo`040000003oool00000000001T0oooo0`0000040?ooo`070000003oool0oooo0?ooo`000000
oooo000000040?ooo`030000003oool00000008000000P3oool01@000000oooo0000003oool00000
01T0oooo0`0000030?ooo`070000003oool0oooo0?ooo`000000oooo000000030?ooo`8000000P3o
ool3000000040?ooo`00000000000?ooo`800000403oool000X0oooo00<000000?ooo`3oool00P3o
ool3000000030?ooo`000000oooo0080oooo0P0000040?ooo`040000003oool0oooo0?ooo`H00000
6@3oool00`000000oooo0?ooo`020?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo
00<000000?ooo`0000000`3oool3000000030?ooo`000000oooo01T0oooo00<000000?ooo`3oool0
0P3oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`030000003oool0000000<0oooo
0`0000000`3oool000000?ooo`0J0?ooo`040000003oool0oooo0?ooo`<0000000<0oooo0000003o
ool00P3oool2000000D0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00000@0?oo
o`00f@3oool00=T0oooo003I0?ooo`004@3oool2000003@0oooo0P00000d0?ooo`800000=03oool2
000002@0oooo00020?ooo`030000003oool0oooo00`0oooo0`00000O0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0303oool3000001l0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0<0?ooo`<000007`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00`0oooo0`00000O0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`020?ooo`00103oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0:`3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo0080oooo00050?ooo`800000:P3oool00`000000oooo0?oo
o`070?ooo`800000:P3oool00`000000oooo0?ooo`070?ooo`800000:P3oool00`000000oooo0?oo
o`070?ooo`800000:P3oool00`000000oooo0?ooo`030?ooo`001@3oool3000000H0oooo0P00000O
0?ooo`8000002P3oool3000000H0oooo0P00000O0?ooo`8000002P3oool3000000L0oooo0P00000N
0?ooo`8000002P3oool3000000L0oooo0P00000O0?ooo`030000003oool0oooo00@0oooo00080?oo
o`030000003oool0oooo00<0oooo0`00000M0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool00`3oool3000001d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?oo
o`<00000703oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo0`00000L0?oo
o`030000003oool0oooo00H0oooo00090?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`070?ooo`002P3oool00`000000oooo0?ooo`0M0?ooo`<00000
4`3oool00`000000oooo0?ooo`0M0?ooo`<000004`3oool00`000000oooo0?ooo`0M0?ooo`<00000
4`3oool00`000000oooo0?ooo`0M0?ooo`<000002P3oool000X0oooo0`00000M0?ooo`800000503o
ool3000001d0oooo0P00000D0?ooo`<000007@3oool2000001@0oooo0`00000N0?ooo`030000003o
ool0oooo00T0oooo000;0?ooo`030000003oool00000008000006@3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool00000008000006@3oool00`000000oooo0?ooo`0E0?ooo`D000006@3oool0
0`000000oooo0?ooo`0E0?ooo`D000006@3oool00`000000oooo0?ooo`0;0?ooo`0000@000000?oo
o`3oool000001P3oool2000000@0oooo0P00000F0?ooo`030000003oool0oooo00/0oooo0P000008
0?ooo`800000103oool2000001H0oooo00<000000?ooo`3oool02`3oool2000000030?ooo`000000
oooo00D0oooo0P0000040?ooo`8000005P3oool00`000000oooo0?ooo`0;0?ooo`8000002@3oool0
0`000000oooo0?ooo`020?ooo`8000005P3oool00`000000oooo0?ooo`0<0?ooo`000P0000020?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool500000180oooo00<000000?oo
o`3oool0303oool5000000D0oooo00<000000?ooo`3oool00`3oool500000180oooo00<000000?oo
o`3oool0303oool3000000030?ooo`000000oooo00@0oooo00<000000?ooo`3oool00`3oool50000
0180oooo00<000000?ooo`3oool0303oool5000000D0oooo0P0000040?ooo`D000004P3oool00`00
0000oooo0?ooo`0=0?ooo`0000D000000?ooo`000000oooo000000060?ooo`030000003oool0oooo
00L0oooo0`000000103oool000000000000000020?ooo`8000000P3oool500000100oooo00@00000
0?ooo`3oool000001P3oool00`000000oooo0?ooo`070?ooo`<0000000@0oooo0000000000000000
0P3oool200000080oooo1@00000@0?ooo`80000000<0oooo0000003oool01@3oool00`000000oooo
0?ooo`070?ooo`<0000000@0oooo00000000000000000P3oool2000000<0oooo1000000@0?ooo`04
0000003oool0oooo000000H0oooo00<000000?ooo`3oool01`3oool3000000040?ooo`0000000000
00000080oooo0P0000030?ooo`80000000<0oooo0000003oool03`3oool000<0oooo0`0000050?oo
o`030000003oool0oooo00X0oooo0P0000001@3oool00000000000000000oooo00@0000000<0oooo
0000003oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool02P3oool2000000030?ooo`00000000000080000000H0oooo00000000000000000?oo
o`00000:0?ooo`030000003oool0oooo00L0oooo0`0000050?ooo`8000002`3oool2000000030?oo
o`00000000000080000000H0oooo00000000000000000?ooo`00000:0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0103oool2000000/0oooo0P0000000`3oool00000000000020000
00050?ooo`00000000000000003oool00P0000090?ooo`030000003oool0oooo00D0oooo000:0?oo
o`8000002@3oool3000000d0oooo00<000000?ooo`3oool01`3oool3000000L0oooo0`0000040?oo
o`8000002@3oool3000000d0oooo00<000000?ooo`3oool01`3oool3000000h0oooo0P0000090?oo
o`<000003@3oool00`000000oooo0?ooo`070?ooo`<000001`3oool3000000@0oooo0P0000090?oo
o`<000003P3oool00`000000oooo0?ooo`060?ooo`<000001@3oool000X0oooo0P0000060?ooo`<0
00004@3oool2000000L0oooo00<000000?ooo`0000003P3oool2000000H0oooo0`00000A0?ooo`80
00001`3oool00`000000oooo0000000>0?ooo`8000001P3oool300000140oooo0P0000070?ooo`03
0000003oool0000000h0oooo0P0000060?ooo`<000004P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0000000D0oooo000;0?ooo`030000003oool0oooo0080oooo0`00000E0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`<00000
5@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
0P3oool4000001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`800000
0`3oool4000001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`050?ooo`002`3o
ool00`000000oooo00000002000001T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0000
0002000001T0oooo00<000000?ooo`3oool05@3oool5000001T0oooo00<000000?ooo`3oool05@3o
ool5000001T0oooo00<000000?ooo`3oool02`3oool000X0oooo0`00000M0?ooo`030000003oool0
oooo01<0oooo0`00000M0?ooo`030000003oool0oooo01<0oooo0`00000M0?ooo`030000003oool0
oooo01<0oooo0`00000N0?ooo`030000003oool0oooo00T0oooo000:0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool02`3oool2000001<0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0;0?ooo`<000004P3oool00`000000oooo0?ooo`0N0?ooo`<000004P3oool00`00
0000oooo0?ooo`0O0?ooo`8000002@3oool000P0oooo0P00000D0?ooo`030000003oool0oooo00/0
oooo0P00000@0?ooo`800000503oool00`000000oooo0?ooo`0;0?ooo`800000403oool2000001<0
oooo0P00000=0?ooo`800000403oool2000001<0oooo0P00000=0?ooo`8000002@3oool000H0oooo
0P00000D0?ooo`030000003oool0000000l0oooo00<000000?ooo`3oool02`3oool2000001@0oooo
00<000000?ooo`0000003`3oool00`000000oooo0?ooo`0;0?ooo`8000005P3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00/0oooo0P00000F0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool0503oool3000000l0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0D0?ooo`<000003`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`040?ooo`00103oool2000002X0oooo0P0000080?ooo`800000:P3oool2000000P0oooo0P00000G
0?ooo`<00000403oool2000000P0oooo0P00000G0?ooo`<000004@3oool00`000000oooo0?ooo`03
0?ooo`000`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0080oooo00020?ooo`03
0000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo02h0oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
403oool002@0oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool0
0`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool0
0`000000oooo0000000@0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo
03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`000000
00000P3oool3000000070?ooo`00000000000?ooo`000000oooo000000040?ooo`800000803oool0
10000000oooo0000000000020?ooo`<000000P3oool01@000000oooo0000003oool0000002H0oooo
00@000000?ooo`00000000000P3oool3000000070?ooo`00000000000?ooo`000000oooo00000004
0?ooo`800000803oool010000000oooo0000000000020?ooo`<0000000<0oooo0000000000000P3o
ool00`000000oooo000000040?ooo`800000403oool000T0oooo0P0000050?ooo`040000003oool0
oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000200000080oooo00@0
00000?ooo`3oool000006@3oool2000000D0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`020?ooo`030000003oool00000008000000P3oool010000000oooo00000000000I0?oo
o`8000001@3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`0000000P0000020?ooo`030000003oool0oooo008000006@3oool2000000@0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<000000P3o
ool00`000000oooo00000002000000l0oooo00090?ooo`<00000103oool01`000000oooo0?ooo`3o
ool000000?ooo`000000103oool00`000000oooo0000000200000080oooo00@000000?ooo`3oool0
00006@3oool3000000@0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<00000
0?ooo`0000000P0000020?ooo`040000003oool00000000001T0oooo0`0000040?ooo`070000003o
ool0oooo0?ooo`000000oooo000000050?ooo`@000000P3oool00`000000oooo0000000K0?ooo`<0
00000`3oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool00`000000oooo0?ooo`03
00000080oooo00<000000?ooo`0000004@3oool000X0oooo00<000000?ooo`3oool00P3oool30000
00030?ooo`000000oooo0080oooo0P0000020?ooo`030000003oool0000000<0oooo0`0000000`3o
ool000000?ooo`0I0?ooo`030000003oool0oooo0080oooo0`0000000`3oool000000?ooo`020?oo
o`800000103oool010000000oooo0?ooo`3oool3000001`0oooo00<000000?ooo`3oool00P3oool3
000000030?ooo`000000oooo0080oooo0P0000020?ooo`030000003oool0000000<0oooo0`000000
0`3oool000000?ooo`0J0?ooo`040000003oool0oooo0?ooo`<0000000<0oooo0000003oool00P3o
ool200000080oooo00@000000?ooo`3oool000000`3oool3000000030?ooo`000000oooo00l0oooo
003I0?ooo`00503oool00`000000oooo0?ooo`320?ooo`00403oool01@000000oooo0000003oool0
000006X0oooo0P00001H0?ooo`004@3oool2000000030?ooo`000000oooo0300oooo0P00000e0?oo
o`040000003oool0000000000380oooo0P00000T0?ooo`004P3oool300000340oooo0`00000d0?oo
o`030000003oool0oooo03<0oooo0`00000S0?ooo`000P3oool00`000000oooo0?ooo`0<0?ooo`80
0000803oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo0`00000P0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00X0oooo0P00000P0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool02P3oool200000200oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0:0?ooo`800000803oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00/0oooo0P00000O0?ooo`030000003oool0oooo0080oooo00040?ooo`800000203o
ool010000000oooo00000000000O0?ooo`030000003oool0oooo00H0oooo0P00000:0?ooo`800000
7`3oool00`000000oooo0?ooo`060?ooo`8000002P3oool2000001l0oooo00<000000?ooo`3oool0
1P3oool2000000/0oooo0P00000N0?ooo`030000003oool0oooo00<0oooo00050?ooo`800000203o
ool3000000030?ooo`000000000001X0oooo0P00000:0?ooo`8000001P3oool2000000@0oooo0P00
000J0?ooo`8000002P3oool2000000P0oooo0`00000M0?ooo`8000002P3oool2000000L0oooo0P00
00040?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0103oool000L0oooo00<00000
0?ooo`3oool01P3oool2000000030?ooo`000000000001T0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`030?ooo`<000000`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0103oool3000000<0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`060?ooo`00203oool2000000D0oooo0P0000020?ooo`800000603oool00`00
0000oooo0?ooo`0>0?ooo`800000103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
00h0oooo0P0000040?ooo`<00000703oool00`000000oooo0?ooo`0>0?ooo`8000001@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool01`3oool000X0
oooo00@000000?ooo`3oool0oooo0P00000J0?ooo`<000004`3oool010000000oooo0?ooo`3oool2
000001X0oooo0`00000C0?ooo`040000003oool0oooo0?ooo`8000006P3oool3000001<0oooo00@0
00000?ooo`3oool0oooo0P00000J0?ooo`<000002P3oool000X0oooo0`0000000`3oool000000000
000J0?ooo`8000005@3oool2000000030?ooo`000000000001X0oooo0P00000E0?ooo`80000000<0
oooo0000000000006P3oool2000001D0oooo1@00000K0?ooo`030000003oool0oooo00T0oooo000=
0?ooo`<0000000<0oooo0000000000005P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00@0oooo00@000000?ooo`3oool000000P3oool2000001H0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`030?ooo`040000003oool0oooo000001X0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo00000080oooo0P00000F0?ooo`03
0000003oool0oooo00/0oooo00000`000000oooo0?ooo`080?ooo`030000003oool0oooo0080oooo
0`00000E0?ooo`030000003oool0oooo00/0oooo0P0000000`3oool00000000000060?ooo`070000
003oool0oooo0?ooo`000000oooo0000000F0?ooo`030000003oool0oooo00/0oooo0P000000103o
ool000000?ooo`0000050?ooo`070000003oool0oooo0?ooo`000000oooo0000000F0?ooo`030000
003oool0oooo00/0oooo0P0000000`3oool00000000000060?ooo`040000003oool0oooo0?ooo`<0
00005P3oool00`000000oooo0?ooo`0<0?ooo`000`0000080?ooo`030000003oool0oooo00<0oooo
1000000B0?ooo`030000003oool0oooo00`0oooo1@0000060?ooo`030000003oool0oooo0080oooo
0`0000000`3oool000000?ooo`0A0?ooo`030000003oool0oooo00`0oooo100000000`3oool00000
0?ooo`040?ooo`030000003oool0oooo0080oooo1@00000B0?ooo`030000003oool0oooo00`0oooo
1@0000060?ooo`030000003oool0oooo0080oooo1@00000B0?ooo`030000003oool0oooo00d0oooo
0004000000L0oooo00<000000?ooo`3oool01`3oool3000000030?ooo`000000000000<0oooo0P00
00030?ooo`@00000403oool010000000oooo0000000000060?ooo`030000003oool0oooo00L0oooo
0`0000000`3oool00000000000030?ooo`8000000`3oool400000100oooo00@000000?ooo`000000
00001P3oool00`000000oooo0?ooo`070?ooo`<0000000<0oooo0000000000000`3oool2000000<0
oooo1000000@0?ooo`040000003oool00000000000H0oooo00<000000?ooo`3oool01`3oool30000
00040?ooo`000000000000000080oooo0P0000030?ooo`80000000<0oooo0000003oool03`3oool0
0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0:0?ooo`80000000<0oooo0000
000000001P0000000`3oool000000?ooo`0B0?ooo`8000001`3oool00`000000oooo0?ooo`090?oo
o`<0000000<0oooo0000000000001P0000000`3oool000000?ooo`0B0?ooo`8000001`3oool00`00
0000oooo0?ooo`090?ooo`<0000000<0oooo0000000000002000000C0?ooo`8000001`3oool00`00
0000oooo0?ooo`090?ooo`<0000000<0oooo0000000000002@00000A0?ooo`0000<0oooo00000000
0000203oool00`000000oooo0?ooo`070?ooo`<000003@3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00D0oooo0P0000080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo0P0000080?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00D0oooo0P0000080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00000`3oool00000
000000080?ooo`030000003oool0oooo00D0oooo0P00000A0?ooo`8000001`3oool3000000D0oooo
0P0000000`3oool00000000000050?ooo`030000003oool0oooo00@0oooo00@000000?ooo`000000
0000403oool2000000L0oooo0`0000050?ooo`800000203oool00`000000oooo0?ooo`040?ooo`@0
0000403oool2000000L0oooo0`0000050?ooo`80000000<0oooo0000000000001@3oool00`000000
oooo0?ooo`040?ooo`@000004@3oool00`000000oooo0?ooo`050?ooo`<000001@3oool000@0oooo
0P0000050?ooo`030000003oool0oooo0080oooo0`00000E0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`000000203oool2000000D0oooo00@000000?ooo`3oool0oooo1000000E0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`0000003`3oool010000000oooo0?ooo`3oool50000
01@0oooo00<000000?ooo`3oool0103oool00`000000oooo000000080?ooo`8000001@3oool00`00
0000oooo0?ooo`020?ooo`@00000503oool00`000000oooo0?ooo`040?ooo`030000003oool00000
00D0oooo00040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000000P00000I0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool2000000D0oooo0P0000000`3oool0
00000?ooo`0I0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool200000003
0?ooo`000000oooo01T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`080?ooo`80
00001@3oool00`000000oooo00000002000001T0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`050?ooo`002P3oool3000001d0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0L0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0D0?ooo`8000007P3oool00`000000oooo0?ooo`090?ooo`002P3oool00`000000oooo0?oo
o`0N0?ooo`8000004`3oool00`000000oooo0?ooo`0N0?ooo`<000004P3oool00`000000oooo0?oo
o`0N0?ooo`<000004P3oool00`000000oooo0?ooo`0O0?ooo`8000002@3oool000P0oooo0P00000R
0?ooo`800000403oool2000001<0oooo00<000000?ooo`3oool0303oool200000100oooo0P00000R
0?ooo`800000403oool200000280oooo0P0000090?ooo`001P3oool2000001D0oooo0P00000?0?oo
o`030000003oool0oooo00/0oooo0P00000F0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool02`3oool2000001D0oooo0P00000?0?ooo`030000003oool0oooo00/0oooo0P00000E0?oo
o`8000003`3oool00`000000oooo0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool000000100oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool000@0oooo0P00000H0?ooo`030000
003oool0oooo00l0oooo0P0000080?ooo`8000005`3oool300000100oooo0P0000080?ooo`800000
603oool00`000000oooo0?ooo`0?0?ooo`800000203oool2000001P0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0G0?ooo`<000004P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0G0?ooo`<000004P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo01L0oooo0`00000B0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool002@0
oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000oooo
0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo
0000000@0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00@000000?ooo`00000000000P3oool3
000000080?ooo`00000000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200
oooo00@000000?ooo`00000000000P3oool3000000@0oooo00<000000?ooo`000000103oool20000
0200oooo00@000000?ooo`00000000000P3oool300000080oooo00D000000?ooo`000000oooo0000
00050?ooo`030000003oool0oooo01h0oooo00@000000?ooo`00000000000P3oool3000000030?oo
o`00000000000080oooo00<000000?ooo`000000103oool200000100oooo00090?ooo`8000001@3o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`000000
0P0000001@3oool00000000000000000oooo00800000603oool2000000D0oooo00@000000?ooo`3o
ool000000P3oool010000000oooo0?ooo`3oool2000000040?ooo`000000000000000080oooo00D0
00000?ooo`000000oooo0000000H0?ooo`8000001@3oool010000000oooo0?ooo`0000020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`0000000P0000020?ooo`040000003oool0oooo0000
01X0oooo0P0000040?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0300000080oooo00@000000?ooo`3oool00000403oool000T0oooo0`000004
0?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`030000003oool0000000800000
00L0oooo0000003oool0oooo0000003oool0000001P0oooo0`0000040?ooo`070000003oool0oooo
0?ooo`000000oooo000000030?ooo`80000000@0oooo00000000000000000P3oool010000000oooo
00000000000I0?ooo`<00000103oool01`000000oooo0?ooo`3oool000000?ooo`000000103oool0
0`000000oooo0000000200000080oooo00@000000?ooo`3oool000006P3oool3000000<0oooo00L0
00000?ooo`3oool0oooo0000003oool0000000D0oooo00<000000?ooo`0000000P0000020?ooo`05
0000003oool0oooo0?ooo`0000003`3oool000X0oooo00<000000?ooo`3oool00P3oool300000003
0?ooo`000000oooo0080oooo0P0000020?ooo`030000003oool0000000@0oooo00@000000?ooo`3o
ool000006P3oool00`000000oooo0?ooo`020?ooo`<0000000<0oooo0000003oool00P3oool20000
00@0oooo00@000000?ooo`3oool0oooo0`0000000`3oool000000?ooo`0I0?ooo`030000003oool0
oooo0080oooo0`0000000`3oool000000?ooo`020?ooo`800000103oool010000000oooo0?ooo`3o
ool3000001d0oooo00@000000?ooo`3oool0oooo0`0000000`3oool000000?ooo`020?ooo`800000
0P3oool010000000oooo0?ooo`0000030?ooo`<0000000<0oooo0000003oool03`3oool00;H0oooo
00<000000?ooo`3oool0803oool00800oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0P0?ooo`00O@3oool010000000oooo00000000000b0?ooo`040000003oool0000000000280oooo
000A0?ooo`800000=03oool00`000000oooo0000000d0?ooo`<00000<`3oool300000280oooo000B
0?ooo`030000003oool0oooo0340oooo1000000e0?ooo`800000=03oool200000280oooo00020?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00/0oooo1000000O0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool03@3oool2000001l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?oo
o`8000007`3oool00`000000oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0;0?ooo`<0
00007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00X0oooo00D000000?ooo`00
0000oooo0000000M0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool20000
01l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0;0?ooo`8000007`3oool00`00
0000oooo0?ooo`020?ooo`00103oool2000002/0oooo00<000000?ooo`3oool01P3oool2000002/0
oooo00<000000?ooo`3oool01P3oool2000000P0oooo00<000000?ooo`0000000P00000N0?ooo`03
0000003oool0oooo00H0oooo0P0000080?ooo`030000003oool00000008000007P3oool00`000000
oooo0?ooo`030?ooo`001@3oool2000000L0oooo0P00000O0?ooo`8000002P3oool2000000L0oooo
00<000000?ooo`0000007P3oool2000000X0oooo0P0000080?ooo`80000000<0oooo0000003oool0
6`3oool2000000X0oooo0P0000080?ooo`80000000<0oooo0000003oool0703oool00`000000oooo
0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`030?ooo`@000007@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00H0oooo0P00000L0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool01P3oool2000001`0oooo00<000000?ooo`3oool01P3oool000P0oooo0P000005
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool03P3oool2000000<0oooo1000000L
0?ooo`030000003oool0oooo00h0oooo0P0000050?ooo`800000703oool00`000000oooo0?ooo`0>
0?ooo`8000001@3oool2000001`0oooo00<000000?ooo`3oool01`3oool000X0oooo00@000000?oo
o`3oool0oooo0`00000I0?ooo`<000004`3oool020000000oooo0?ooo`000000oooo0000003oool0
0000603oool3000001<0oooo00@000000?ooo`3oool0oooo0P00000J0?ooo`<000004`3oool01000
0000oooo0?ooo`3oool2000001X0oooo0`00000:0?ooo`002`3oool010000000oooo0?ooo`00000K
0?ooo`8000005@3oool010000000oooo0?ooo`00000K0?ooo`8000005@3oool2000000030?ooo`00
0000000001X0oooo0P00000E0?ooo`80000000<0oooo0000000000006`3oool00`000000oooo0?oo
o`090?ooo`002P3oool6000001T0oooo00<000000?ooo`3oool0503oool6000001T0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`03000001T0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`03000001T0oooo00<000000?ooo`3oool02`3oool000030000003oool0oooo
00P0oooo00@000000?ooo`3oool0oooo1000000E0?ooo`030000003oool0oooo00/0oooo0P000000
0`3oool00000000000060?ooo`040000003oool0oooo0?ooo`@000005@3oool00`000000oooo0?oo
o`0;0?ooo`80000000<0oooo0000000000001P3oool00`000000oooo0?ooo`020?ooo`8000005P3o
ool00`000000oooo0?ooo`0;0?ooo`80000000<0oooo0000000000001P3oool00`000000oooo0?oo
o`020?ooo`8000005P3oool00`000000oooo0?ooo`0<0?ooo`000P0000000`3oool0000000000006
0?ooo`030000003oool0oooo00D0oooo00@000000?ooo`0000000000403oool00`000000oooo0?oo
o`0<0?ooo`D000001P3oool00`000000oooo0?ooo`020?ooo`@0000000<0oooo000000000000403o
ool00`000000oooo0?ooo`0<0?ooo`D000001P3oool00`000000oooo0?ooo`020?ooo`D000004P3o
ool00`000000oooo0?ooo`0<0?ooo`D000001P3oool00`000000oooo0?ooo`020?ooo`D000004P3o
ool00`000000oooo0?ooo`0=0?ooo`0000<000000?ooo`3oool00`0000050?ooo`030000003oool0
oooo00D0oooo1@00000=0?ooo`800000403oool2000000P0oooo00<000000?ooo`3oool01@3oool5
000000d0oooo0P00000@0?ooo`800000203oool00`000000oooo0?ooo`070?ooo`<0000000<0oooo
0000000000000`3oool2000000<0oooo1000000@0?ooo`800000203oool00`000000oooo0?ooo`07
0?ooo`<000000P3oool200000080oooo0P0000030?ooo`80000000<0oooo0000003oool03`3oool0
00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`d0000000<0oooo0000
000000004P3oool3000000L0oooo00<000000?ooo`3oool01`3oool:000000040?ooo`0000000000
0?ooo`8000004P3oool3000000L0oooo00<000000?ooo`3oool02P3oool2000000030?ooo`000000
000000P000004P3oool3000000L0oooo00<000000?ooo`3oool02P3oool2000000030?ooo`000000
000000T000004@3oool000/0oooo00<000000?ooo`3oool01`3oool4000000`0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo1000000<0?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool2000000P0oooo00<000000?oo
o`3oool01`3oool3000000@0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool0
000000T0oooo00<000000?ooo`3oool01@3oool2000000P0oooo00<000000?ooo`3oool01`3oool3
000000@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool01@3oool000/0oooo00<000000?ooo`3oool00`3oool40000
0140oooo0P0000070?ooo`<000003`3oool00`000000oooo0?ooo`030?ooo`D00000403oool20000
00L0oooo0`0000050?ooo`800000203oool00`000000oooo0?ooo`040?ooo`<000004@3oool20000
00L0oooo0`0000050?ooo`800000203oool00`000000oooo0?ooo`040?ooo`<000004P3oool00`00
0000oooo0?ooo`050?ooo`<000001@3oool000/0oooo00<000000?ooo`0000001000000F0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`0000003`3oool00`000000oooo00000005000001D0
oooo00<000000?ooo`3oool0103oool00`000000oooo0000000?0?ooo`030000003oool0oooo0080
oooo1000000D0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000003`3oool00`000000
oooo0?ooo`020?ooo`@00000503oool00`000000oooo0?ooo`040?ooo`030000003oool0000000D0
oooo000;0?ooo`040000003oool000000?ooo`800000603oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00l0oooo00@000000?ooo`000000oooo0P00000H0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool03`3oool00`000000oooo00000002000001T0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool00000008000006@3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo000:0?ooo`@00000703oool00`000000
oooo0?ooo`0C0?ooo`@00000703oool00`000000oooo0?ooo`0C0?ooo`<000007@3oool00`000000
oooo0?ooo`0C0?ooo`<000007P3oool00`000000oooo0?ooo`090?ooo`002P3oool00`000000oooo
0?ooo`0N0?ooo`8000004`3oool00`000000oooo0?ooo`0N0?ooo`<000004P3oool00`000000oooo
0?ooo`0N0?ooo`<000004P3oool00`000000oooo0?ooo`0O0?ooo`8000002@3oool000P0oooo0P00
000A0?ooo`030000003oool0oooo00h0oooo0P00000@0?ooo`8000004@3oool00`000000oooo0?oo
o`0>0?ooo`800000403oool2000001<0oooo00<000000?ooo`3oool0303oool200000100oooo0P00
000R0?ooo`8000002@3oool000H0oooo0P00000C0?ooo`<00000403oool00`000000oooo0?ooo`0;
0?ooo`8000004`3oool300000100oooo00<000000?ooo`3oool02`3oool2000001H0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?ooo`8000005@3oool2000000l0oooo00<00000
0?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool04P3oool010000000oooo0000003oool2
000000h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0B0?ooo`040000003oool0
00000?ooo`8000003P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`000000403oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`00103oool2000001D0oooo00@00000
0?ooo`3oool0oooo0P00000?0?ooo`800000203oool2000001D0oooo00@000000?ooo`3oool0oooo
0P00000?0?ooo`800000203oool2000001L0oooo0`00000@0?ooo`800000203oool2000001P0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?oo
o`0I0?ooo`8000004@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo0P00
000A0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo01L0oooo0`00000B0?ooo`030000003oool0oooo0080oooo0002
0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0403oool002@0oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00
903oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000
<`3oool00`000000oooo0000000@0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo00090?ooo`80
00001@3oool010000000oooo0000000000020?ooo`<0000000P0oooo000000000000oooo0000003o
ool000000?ooo`80000000<0oooo0000000000006@3oool2000000D0oooo00@000000?ooo`000000
00000P3oool3000000@0oooo00@000000?ooo`000000oooo0P0000000`3oool000000000000I0?oo
o`8000001@3oool010000000oooo0000000000020?ooo`<000000P3oool01P000000oooo0000003o
ool000000?ooo`8000007@3oool2000000@0oooo00@000000?ooo`00000000000P3oool300000003
0?ooo`00000000000080oooo00@000000?ooo`000000oooo0P0000000`3oool000000000000@0?oo
o`002@3oool3000000@0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool000000080000000D0oooo00000000000000000?ooo`02000001P0oooo0`000004
0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P0000001@3oool00000
000000000000oooo00H00000603oool3000000@0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool000000080000000H0oooo00000000000000000?ooo`00
000J0?ooo`<000000`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00`0000000`3oool000000000000300000100oooo000:0?ooo`030000003o
ool0oooo00<0oooo00L000000?ooo`3oool0oooo0000003oool0000000@0oooo00<000000?ooo`00
00000P0000001@3oool000000?ooo`3oool0000001/0oooo00<000000?ooo`3oool00`3oool01`00
0000oooo0?ooo`3oool000000?ooo`0000000`3oool2000000060?ooo`00000000000000003oool0
00000P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00<0oooo00L000000?ooo`3o
ool0oooo0000003oool0000000@0oooo00<000000?ooo`0000000P0000000`3oool000000?ooo`02
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool00P3oool01`000000oooo0?ooo`3o
ool000000?ooo`0000001@3oool00`000000oooo00000002000000040?ooo`000000oooo0?ooo`80
0000403oool000l0oooo0`0000000`3oool000000?ooo`020?ooo`8000000P3oool00`000000oooo
000000040?ooo`040000003oool0oooo000001l0oooo0`0000000`3oool000000?ooo`020?ooo`80
0000103oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000001l0oooo0`0000000`3o
ool000000?ooo`020?ooo`800000103oool00`000000oooo0?ooo`020?ooo`030000003oool00000
008000007P3oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`040000003oool0oooo
000000@0oooo00@000000?ooo`3oool00000403oool00=T0oooo000C0?ooo`800000a03oool00140
oooo00@000000?ooo`0000000000<P3oool2000003@0oooo0P00000d0?ooo`030000003oool00000
02<0oooo000A0?ooo`030000003oool0oooo03<0oooo0`00000d0?ooo`030000003oool0oooo0340
oooo1000000S0?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`<00000803oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00/0oooo1000000O0?ooo`030000003oool0oooo0040oooo
00030?ooo`030000003oool0oooo00X0oooo0P00000P0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool02`3oool2000001l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0;0?ooo`<000007P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00X0oooo00D0
00000?ooo`000000oooo0000000M0?ooo`030000003oool0oooo0080oooo00040?ooo`8000002P3o
ool2000001l0oooo00<000000?ooo`3oool01P3oool2000000/0oooo0P00000N0?ooo`030000003o
ool0oooo00H0oooo0P00000[0?ooo`030000003oool0oooo00H0oooo0P00000[0?ooo`030000003o
ool0oooo00<0oooo00050?ooo`8000001`3oool010000000oooo00000000000M0?ooo`8000002P3o
ool2000000H0oooo0P0000050?ooo`030000003oool0oooo01P0oooo0P00000:0?ooo`800000203o
ool2000001h0oooo0P00000:0?ooo`800000203oool00`000000oooo0000000N0?ooo`030000003o
ool0oooo00@0oooo00070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00<0oooo0`0000040?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`@00000703oool0
0`000000oooo0?ooo`060?ooo`00203oool2000000<0oooo0`00000M0?ooo`030000003oool0oooo
00h0oooo0P0000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`0>0?ooo`8000001P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00h0oooo0P0000040?ooo`@000006`3oool00`000000oooo0?ooo`070?ooo`002P3oool010000000
oooo0?ooo`3oool2000001X0oooo0`00000C0?ooo`040000003oool0oooo0?ooo`8000006P3oool3
000001<0oooo00<000000?ooo`3oool00P3oool3000001P0oooo0`00000C0?ooo`090000003oool0
oooo0?ooo`000000oooo0000003oool0000001L0oooo0`00000:0?ooo`002`3oool00`000000oooo
0?ooo`02000001X0oooo0P00000E0?ooo`030000003oool0oooo008000006P3oool2000001D0oooo
00<000000?ooo`3oool0703oool2000001D0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`090?ooo`001@3oool00`000000oooo0?ooo`030?ooo`80000000<0oooo0000003oool06@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo0P000000103oool000000?oo
o`3oool2000001H0oooo00<000000?ooo`3oool0503oool6000001T0oooo00<000000?ooo`3oool0
503oool6000001T0oooo00<000000?ooo`3oool02`3oool000060?ooo`000000oooo0000003oool0
00001@3oool01`000000oooo0?ooo`3oool000000?ooo`0000005P3oool00`000000oooo0?ooo`0<
0?ooo`040000003oool00000000000H0oooo00L000000?ooo`3oool0oooo0000003oool0000001H0
oooo00<000000?ooo`3oool05P3oool010000000oooo0?ooo`3oool5000001@0oooo00<000000?oo
o`3oool03P3oool2000000H0oooo00@000000?ooo`3oool0oooo1@00000D0?ooo`030000003oool0
oooo00`0oooo00001P000000oooo000000000000oooo000000D0oooo00<000000?ooo`3oool00P3o
ool3000000030?ooo`000000oooo0140oooo00<000000?ooo`3oool0303oool5000000H0oooo00<0
00000?ooo`3oool00P3oool3000000030?ooo`000000oooo0140oooo00<000000?ooo`3oool0303o
ool2000000030?ooo`000000000000H0oooo00<000000?ooo`3oool01@3oool010000000oooo0000
0000000@0?ooo`030000003oool0oooo00`0oooo1@0000060?ooo`030000003oool0oooo0080oooo
100000000`3oool000000000000@0?ooo`030000003oool0oooo00d0oooo0002000000030?ooo`00
0000000000H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0000000=0?ooo`8000003`3o
ool5000000H0oooo00<000000?ooo`3oool01`3oool3000000d0oooo0P00000?0?ooo`H000001@3o
ool00`000000oooo0?ooo`050?ooo`D000003@3oool2000000l0oooo0`0000080?ooo`030000003o
ool0oooo00D0oooo1@00000>0?ooo`030000003oool0oooo00h0oooo000010000000oooo00000000
00070?ooo`030000003oool0oooo00X0oooo2P0000000`3oool000000000000B0?ooo`<000001`3o
ool00`000000oooo0?ooo`0:0?ooo`L0000000@0oooo000000000000oooo0P00000B0?ooo`040000
003oool0oooo000000H0oooo00<000000?ooo`3oool0203oool9000000040?ooo`00000000000?oo
o`8000004P3oool3000000L0oooo00<000000?ooo`3oool0203oool9000000040?ooo`0000000000
0?ooo`<000004@3oool000030?ooo`000000000000P0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0000000=0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool20000
00P0oooo00<000000?ooo`3oool0203oool3000000`0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo1000000;0?ooo`8000002@3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo1000000;0?ooo`030000003oool00000
00P0oooo00<000000?ooo`3oool01@3oool000030?ooo`000000000000P0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0000000A0?ooo`8000001`3oool3000000D0oooo0P0000000`3oool0
0000000000050?ooo`030000003oool0oooo00@0oooo00@000000?ooo`0000000000403oool20000
00L0oooo0`00000?0?ooo`030000003oool0oooo00<0oooo1@00000@0?ooo`8000001`3oool30000
00l0oooo00<000000?ooo`3oool00`3oool500000140oooo00<000000?ooo`3oool01@3oool30000
00D0oooo000;0?ooo`040000003oool0oooo0?ooo`@000005@3oool00`000000oooo0?ooo`040?oo
o`030000003oool0000000P0oooo0P0000050?ooo`040000003oool0oooo0?ooo`@000005@3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0000000l0oooo00<000000?ooo`3oool01000000F
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000003`3oool00`000000oooo0?ooo`02
000000040?ooo`0000000000000001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0000
00050?ooo`002`3oool2000000030?ooo`000000oooo01T0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`080?ooo`8000001@3oool2000000030?ooo`000000oooo01T0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`040000003oool000000?ooo`800000603o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo00@000000?ooo`000000oooo
0P00000H0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool000/0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool04`3oool4000001`0oooo00<000000?ooo`3oool04`3oool4000001d0oooo00<0
00000?ooo`3oool02@3oool000X0oooo00<000000?ooo`3oool07P3oool2000001<0oooo00<00000
0?ooo`3oool07P3oool300000180oooo00<000000?ooo`3oool07P3oool300000180oooo00<00000
0?ooo`3oool07`3oool2000000T0oooo00080?ooo`8000004@3oool00`000000oooo0?ooo`0>0?oo
o`800000403oool200000140oooo00<000000?ooo`3oool03P3oool200000100oooo0P00000B0?oo
o`030000003oool0oooo00d0oooo0P00000@0?ooo`8000004P3oool00`000000oooo0?ooo`0=0?oo
o`8000002@3oool000L0oooo00<000000?ooo`3oool04@3oool300000100oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0A0?ooo`<00000403oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0180oooo0P00000@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool04P3oool200000100oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?ooo`3oool0
4P3oool010000000oooo0000003oool2000000h0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0B0?ooo`040000003oool000000?ooo`8000003P3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo01<0oooo0P0000000`3oool000000000000>0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool04`3oool2000000030?ooo`000000000000l0oooo00<000000?oo
o`3oool0103oool000@0oooo0P00000E0?ooo`040000003oool0oooo0?ooo`8000003`3oool20000
00P0oooo0P00000E0?ooo`040000003oool0oooo0?ooo`8000003`3oool2000000P0oooo0P00000F
0?ooo`030000003oool0oooo008000003`3oool2000000P0oooo0P00000F0?ooo`030000003oool0
oooo00800000403oool00`000000oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0I0?oo
o`8000004@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo0P00000A0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool200000140oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0I0?ooo`8000004@3oool00`000000oooo0?ooo`020?oo
o`000P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0;P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0040oooo000T0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000
003oool0oooo0100oooo000T0?ooo`<00000<`3oool3000003<0oooo0`00000c0?ooo`<00000403o
ool002@0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`030000003oool0
000003<0oooo00<000000?ooo`000000403oool002@0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`002@3o
ool2000000D0oooo00@000000?ooo`00000000000P3oool3000000080?ooo`00000000000?ooo`00
0000oooo0000003oool200000080oooo00<000000?ooo`3oool05`3oool2000000D0oooo00@00000
0?ooo`00000000000P3oool3000000@0oooo00@000000?ooo`000000oooo0P0000000`3oool00000
0000000I0?ooo`8000001@3oool010000000oooo0000000000020?ooo`<000000P3oool01P000000
oooo0000003oool000000?ooo`80000000<0oooo0000000000006P3oool2000000@0oooo00@00000
0?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo00@000000?ooo`000000oooo
0P00000C0?ooo`002@3oool3000000@0oooo00@000000?ooo`3oool000000P3oool00`000000oooo
0?ooo`020?ooo`030000003oool000000080000000H0oooo00000000000000000?ooo`00000I0?oo
o`<00000103oool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80000000D0
oooo00000000000000000?ooo`03000000030?ooo`000000oooo01P0oooo0`0000040?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000200000006
0?ooo`00000000000000003oool000006P3oool3000000<0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0000000<0oooo0000000000000`00
000@0?ooo`002P3oool00`000000oooo0?ooo`030?ooo`070000003oool0oooo0?ooo`000000oooo
000000040?ooo`030000003oool000000080000000<0oooo0000003oool00P3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo00<0oooo00L000000?ooo`3oool0oooo0000003oool00000
00<0oooo0P0000001P3oool00000000000000000oooo000000@0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`030?ooo`070000003oool0oooo0?ooo`000000oooo000000040?ooo`03
0000003oool000000080000000<0oooo0000003oool00P3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo0080oooo00L000000?ooo`3oool0oooo0000003oool0000000D0oooo00<00000
0?ooo`0000000P000000103oool000000?ooo`3oool200000100oooo000?0?ooo`<0000000<0oooo
0000000000000P3oool200000080oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`0P
0?ooo`<0000000<0oooo0000003oool00P3oool2000000@0oooo00<000000?ooo`3oool00P3oool0
10000000oooo0?ooo`00000O0?ooo`<0000000<0oooo0000003oool00P3oool2000000@0oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`00000O0?ooo`<0000000<0oooo0000003oool0
0P3oool200000080oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0A0?ooo`00
503oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo08`0oooo000A0?ooo`040000003o
ool0000000000340oooo00D000000?ooo`000000oooo0000000d0?ooo`800000F03oool00180oooo
0`00000b0?ooo`80000000<0oooo0000003oool0<@3oool010000000oooo00000000000b0?ooo`80
0000903oool001<0oooo0P00000c0?ooo`<00000<P3oool00`000000oooo0?ooo`0c0?ooo`<00000
8`3oool00080oooo00<000000?ooo`3oool03@3oool2000001l0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0<0?ooo`800000803oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00/0oooo0`00000P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0
oooo00/0oooo0P00000O0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02P3oool2
00000200oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0:0?ooo`800000803oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo0P00000O0?ooo`030000003oool0
oooo0080oooo00040?ooo`800000203oool00`000000oooo00000002000001h0oooo00<000000?oo
o`3oool01P3oool2000000P0oooo00@000000?ooo`00000000007`3oool00`000000oooo0?ooo`06
0?ooo`8000002P3oool2000001l0oooo00<000000?ooo`3oool01P3oool2000000/0oooo0P00000N
0?ooo`030000003oool0oooo00<0oooo00050?ooo`800000203oool3000001d0oooo0P00000:0?oo
o`800000203oool3000000030?ooo`000000000001X0oooo0P00000:0?ooo`800000203oool30000
01d0oooo0P00000:0?ooo`8000001`3oool2000000@0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`060?ooo`800000703oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo0P0000000`3oool000000000000I0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0103oool3000000<0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`060?ooo`00203oool2000000D0oooo0P00000L0?ooo`030000
003oool0oooo00h0oooo0P0000050?ooo`8000000P3oool2000001P0oooo00<000000?ooo`3oool0
3P3oool2000000@0oooo0`00000L0?ooo`030000003oool0oooo00h0oooo0P0000050?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`070?ooo`002P3o
ool010000000oooo0?ooo`3oool2000001X0oooo0`00000C0?ooo`040000003oool0oooo0?ooo`80
00006P3oool3000001<0oooo00@000000?ooo`3oool0oooo0P00000J0?ooo`<000004`3oool01000
0000oooo0?ooo`3oool2000001X0oooo0`00000:0?ooo`002`3oool2000000030?ooo`0000000000
01X0oooo0P00000E0?ooo`80000000<0oooo0000000000006P3oool2000001D0oooo00<000000?oo
o`3oool00P00000J0?ooo`8000005@3oool00`000000oooo0?ooo`02000001/0oooo00<000000?oo
o`3oool02@3oool000X0oooo00<000000?ooo`3oool00`00000I0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool00`0000000`3oool000000000000F0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool00`3oool2000000030?ooo`000000oooo01T0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`040?ooo`@000000P3oool2000001H0oooo00<000000?ooo`3o
ool02`3oool000<0oooo0P0000060?ooo`030000003oool0oooo0080oooo0P00000F0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool00P3oool3000001D0oooo00<000000?ooo`3oool0
303oool01@000000oooo0000003oool0000000D0oooo00L000000?ooo`3oool0oooo0000003oool0
000001H0oooo00<000000?ooo`3oool0303oool010000000oooo0000000000060?ooo`040000003o
ool0oooo0?ooo`<000005P3oool00`000000oooo0?ooo`0<0?ooo`0000<000000?ooo`0000000P00
00060?ooo`030000003oool0oooo0080oooo1@00000B0?ooo`030000003oool0oooo00`0oooo0`00
00080?ooo`030000003oool0oooo00<0oooo1000000B0?ooo`030000003oool0oooo00`0oooo1000
00000`3oool000000?ooo`040?ooo`030000003oool0oooo0080oooo1@00000B0?ooo`030000003o
ool0oooo00`0oooo1@0000060?ooo`030000003oool0oooo0080oooo1@00000B0?ooo`030000003o
ool0oooo00d0oooo0003000000P0oooo00<000000?ooo`3oool01`3oool3000000d0oooo0P00000?
0?ooo`@000001`3oool00`000000oooo0?ooo`070?ooo`<000003@3oool2000000l0oooo1@000006
0?ooo`030000003oool0oooo00L0oooo0P00000>0?ooo`8000003`3oool5000000H0oooo00<00000
0?ooo`3oool01`3oool2000000l0oooo00<000000?ooo`3oool03P3oool000@000001`3oool00`00
0000oooo0?ooo`0:0?ooo`X0000000<0oooo0000000000004P3oool2000000P0oooo00<000000?oo
o`3oool02P3oool7000000040?ooo`00000000000?ooo`8000004P3oool3000000L0oooo00<00000
0?ooo`3oool02@3oool8000000040?ooo`00000000000?ooo`8000004P3oool3000000L0oooo00<0
00000?ooo`3oool02@3oool8000000040?ooo`00000000000?ooo`<000004@3oool000030?ooo`00
0000000000P0oooo00<000000?ooo`3oool01`3oool4000000/0oooo0P0000090?ooo`030000003o
ool0oooo00D0oooo0P0000080?ooo`030000003oool0oooo00L0oooo1000000;0?ooo`8000002@3o
ool00`000000oooo0?ooo`050?ooo`800000203oool00`000000oooo0?ooo`080?ooo`<00000303o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo0P0000080?ooo`030000003o
ool0oooo00P0oooo0`00000=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool000030?ooo`000000000000P0oooo00<000000?ooo`3oool0103oool300000140oooo0P000007
0?ooo`<000001@3oool2000000P0oooo00<000000?ooo`3oool01@3oool200000140oooo0P000007
0?ooo`<000001@3oool2000000P0oooo00<000000?ooo`3oool0103oool400000100oooo0P000007
0?ooo`<000001@3oool2000000P0oooo00<000000?ooo`3oool0103oool400000140oooo00<00000
0?ooo`3oool01@3oool3000000D0oooo000;0?ooo`030000003oool0oooo0080oooo0`00000E0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`000000203oool2000000D0oooo00<000000?oo
o`3oool00P3oool3000001D0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000?0?oo
o`040000003oool0oooo0?ooo`D00000503oool00`000000oooo0?ooo`040?ooo`030000003oool0
000000P0oooo0P0000050?ooo`030000003oool0oooo0080oooo1000000D0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`0000001@3oool000/0oooo00<000000?ooo`0000000P00000I0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`040?oo
o`030000003oool00000008000006@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00l0oooo00@000000?ooo`3oool000006P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00P0oooo0P0000050?ooo`030000003oool0oooo008000006@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00D0oooo000;0?ooo`8000007@3oool00`000000oooo0?ooo`0D0?oo
o`8000007@3oool00`000000oooo0?ooo`0D0?ooo`8000007@3oool00`000000oooo0?ooo`0=0?oo
o`8000001@3oool3000001d0oooo00<000000?ooo`3oool02@3oool000X0oooo00<000000?ooo`3o
ool07P3oool2000001<0oooo00<000000?ooo`3oool07P3oool300000180oooo00<000000?ooo`3o
ool07P3oool300000180oooo00<000000?ooo`3oool07`3oool2000000T0oooo00080?ooo`800000
4@3oool00`000000oooo0?ooo`0>0?ooo`800000403oool200000140oooo00<000000?ooo`3oool0
3P3oool200000100oooo0P00000B0?ooo`030000003oool0oooo00d0oooo0P00000@0?ooo`800000
4P3oool00`000000oooo0?ooo`0=0?ooo`8000002@3oool000L0oooo00<000000?ooo`3oool04@3o
ool300000100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0A0?ooo`<00000403o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0180oooo0P00000@0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool04P3oool200000100oooo00<000000?ooo`3oool01P3o
ool000H0oooo0P00000C0?ooo`040000003oool000000?ooo`8000003P3oool00`000000oooo0?oo
o`0:0?ooo`8000004`3oool010000000oooo0000003oool2000000h0oooo00<000000?ooo`3oool0
2P3oool2000001@0oooo0P0000000`3oool000000000000>0?ooo`030000003oool0oooo00X0oooo
0P00000D0?ooo`80000000<0oooo0000000000003`3oool00`000000oooo0?ooo`040?ooo`00103o
ool2000001D0oooo00@000000?ooo`3oool0oooo0P00000?0?ooo`800000203oool2000001D0oooo
00@000000?ooo`3oool0oooo0P00000?0?ooo`800000203oool2000001H0oooo00<000000?ooo`3o
ool00P00000?0?ooo`800000203oool2000001H0oooo00<000000?ooo`3oool00P00000@0?ooo`03
0000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo01T0oooo0P00000A0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool200000140oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0I0?ooo`8000004@3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo01T0oooo0P00000A0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003o
ool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo02h0oooo00<000000?ooo`3oool00@3oool002@0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403oool0
02@0oooo0`00000c0?ooo`<00000<`3oool3000003<0oooo0`00000@0?ooo`00903oool00`000000
oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000
oooo0000000@0?ooo`00903oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0
oooo0`00000;0?ooo`030000003oool0oooo00@0oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`0000080?ooo`D000001@3oool01000
0000oooo0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3
000000/0oooo00<000000?ooo`3oool0103oool4000000L0oooo1000001F0?ooo`00M03oool01000
0000oooo0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .40909 m
.21344 .41364 L
.21713 .41818 L
.21959 .42273 L
.22045 .42727 L
.21959 .43182 L
.21712 .43636 L
.21344 .44091 L
.20909 .44545 L
.20474 .45 L
.20105 .45455 L
.19859 .45909 L
.19772 .46364 L
.19859 .46818 L
.20105 .47273 L
.20474 .47727 L
.20909 .48182 L
.21344 .48636 L
.21712 .49091 L
.21959 .49545 L
.22045 .5 L
.21959 .50455 L
.21712 .50909 L
.21344 .51364 L
.20909 .51818 L
.20474 .52273 L
.20105 .52727 L
.19859 .53182 L
.19772 .53636 L
.19859 .54091 L
.20105 .54545 L
.20474 .55 L
.20909 .55455 L
.21344 .55909 L
.21712 .56364 L
.21959 .56818 L
.22045 .57273 L
.21959 .57727 L
.21713 .58182 L
.21344 .58636 L
.20909 .59091 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.46364 .5 L
s
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .40909 m
.46364 .5 L
s
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N81) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.46364 .5 m
.46071 .50544 L
.4576 .51034 L
.45413 .51425 L
.45019 .51683 L
.44572 .51795 L
.44075 .51766 L
.43537 .51621 L
.42973 .51402 L
.424 .5116 L
.41838 .50949 L
.41305 .50819 L
.40815 .50809 L
.40376 .50942 L
.39989 .5122 L
.39648 .51627 L
.3934 .52128 L
.39049 .52676 L
.38755 .53215 L
.3844 .53693 L
.38087 .54067 L
.37685 .54305 L
.37231 .54396 L
.36728 .54348 L
.36185 .54191 L
.35618 .53965 L
.35045 .53724 L
.34487 .53521 L
.33959 .53406 L
.33476 .53416 L
.33045 .5357 L
.32665 .53868 L
.3233 .54292 L
.32025 .54802 L
.31735 .55352 L
.31439 .55885 L
.31119 .56351 L
.3076 .56707 L
.30351 .56924 L
.2989 .56994 L
.2938 .56929 L
.28833 .56759 L
.28264 .56528 L
.27692 .56289 L
.27136 .56096 L
.26614 .55997 L
.26139 .56027 L
.25715 .56202 L
.25342 .5652 L
.25012 .56958 L
Mistroke
.24711 .57477 L
.24421 .58028 L
.24123 .58555 L
.23798 .59008 L
.23432 .59345 L
.23016 .59541 L
.22547 .59591 L
.22031 .59508 L
.2148 .59326 L
.20909 .59091 L
Mfstroke
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.45793 .50236 L
.45241 .50418 L
.44726 .505 L
.44257 .5045 L
.43841 .50254 L
.43474 .49917 L
.4315 .49464 L
.42852 .48937 L
.42562 .48387 L
.42261 .47867 L
.41931 .47429 L
.41558 .47111 L
.41134 .46936 L
.40658 .46906 L
.40136 .47005 L
.39581 .47198 L
.39009 .47437 L
.3844 .47668 L
.37893 .47838 L
.37383 .47903 L
.36922 .47833 L
.36513 .47616 L
.36153 .47261 L
.35833 .46795 L
.35538 .46261 L
.35247 .45711 L
.34943 .45201 L
.34608 .44778 L
.34228 .44479 L
.33797 .44325 L
.33314 .44315 L
.32786 .4443 L
.32227 .44633 L
.31655 .44874 L
.31088 .451 L
.30545 .45257 L
.30041 .45305 L
.29587 .45214 L
.29186 .44976 L
.28833 .44603 L
.28517 .44124 L
.28223 .43585 L
.27932 .43038 L
.27625 .42537 L
.27284 .42129 L
.26897 .41851 L
.26458 .41718 L
.25968 .41728 L
.25435 .41858 L
Mistroke
.24873 .42069 L
.243 .42312 L
.23735 .42531 L
.23197 .42675 L
.22701 .42704 L
.22254 .42592 L
.2186 .42334 L
.21513 .41943 L
.21201 .41453 L
.20909 .40909 L
Mfstroke
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N82) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.20909 .59091 L
s
.10632 .67655 m
.13658 .62766 L
.15986 .6556 L
F
gsave
.10966 .63246 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.20909 .40909 L
s
.14822 .35837 m
.11796 .30948 L
.09468 .33742 L
F
gsave
.14489 .31428 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.46364 .5 m
.46818 .49635 L
.47273 .49308 L
.47727 .49055 L
.48182 .48903 L
.48636 .48866 L
.49091 .4895 L
.49546 .49146 L
.5 .49432 L
.50455 .49779 L
.50909 .50149 L
.51364 .50503 L
.51818 .50804 L
.52273 .5102 L
.52727 .51127 L
.53182 .51115 L
.53636 .50985 L
.54091 .5075 L
.54545 .50435 L
.55 .50075 L
.55455 .49707 L
.55909 .49369 L
.56364 .49099 L
.56818 .48925 L
.57273 .48864 L
.57727 .48925 L
.58182 .49099 L
.58636 .49369 L
.59091 .49707 L
.59545 .50075 L
.6 .50435 L
.60455 .5075 L
.60909 .50985 L
.61364 .51115 L
.61818 .51127 L
.62273 .5102 L
.62727 .50804 L
.63182 .50503 L
.63636 .50149 L
.64091 .49779 L
.64545 .49432 L
.65 .49146 L
.65454 .4895 L
.65909 .48866 L
.66364 .48903 L
.66818 .49055 L
.67273 .49308 L
.67727 .49635 L
.68182 .5 L
s
gsave
.57273 .54296 -65 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.20909 .59091 m
.20909 .40909 L
s
.20909 .52727 m
.19091 .47273 L
.22727 .47273 L
F
gsave
.16613 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.46364 .5 m
.46071 .50544 L
.4576 .51034 L
.45413 .51425 L
.45019 .51683 L
.44572 .51795 L
.44075 .51766 L
.43537 .51621 L
.42973 .51402 L
.424 .5116 L
.41838 .50949 L
.41305 .50819 L
.40815 .50809 L
.40376 .50942 L
.39989 .5122 L
.39648 .51627 L
.3934 .52128 L
.39049 .52676 L
.38755 .53215 L
.3844 .53693 L
.38087 .54067 L
.37685 .54305 L
.37231 .54396 L
.36728 .54348 L
.36185 .54191 L
.35618 .53965 L
.35045 .53724 L
.34487 .53521 L
.33959 .53406 L
.33476 .53416 L
.33045 .5357 L
.32665 .53868 L
.3233 .54292 L
.32025 .54802 L
.31735 .55352 L
.31439 .55885 L
.31119 .56351 L
.3076 .56707 L
.30351 .56924 L
.2989 .56994 L
.2938 .56929 L
.28833 .56759 L
.28264 .56528 L
.27692 .56289 L
.27136 .56096 L
.26614 .55997 L
.26139 .56027 L
.25715 .56202 L
.25342 .5652 L
.25012 .56958 L
Mistroke
.24711 .57477 L
.24421 .58028 L
.24123 .58555 L
.23798 .59008 L
.23432 .59345 L
.23016 .59541 L
.22547 .59591 L
.22031 .59508 L
.2148 .59326 L
.20909 .59091 L
Mfstroke
.31068 .55463 m
.35593 .51916 L
.36816 .5534 L
F
gsave
.31267 .5066 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.46364 .5 m
.45793 .50236 L
.45241 .50418 L
.44726 .505 L
.44257 .5045 L
.43841 .50254 L
.43474 .49917 L
.4315 .49464 L
.42852 .48937 L
.42562 .48387 L
.42261 .47867 L
.41931 .47429 L
.41558 .47111 L
.41134 .46936 L
.40658 .46906 L
.40136 .47005 L
.39581 .47198 L
.39009 .47437 L
.3844 .47668 L
.37893 .47838 L
.37383 .47903 L
.36922 .47833 L
.36513 .47616 L
.36153 .47261 L
.35833 .46795 L
.35538 .46261 L
.35247 .45711 L
.34943 .45201 L
.34608 .44778 L
.34228 .44479 L
.33797 .44325 L
.33314 .44315 L
.32786 .4443 L
.32227 .44633 L
.31655 .44874 L
.31088 .451 L
.30545 .45257 L
.30041 .45305 L
.29587 .45214 L
.29186 .44976 L
.28833 .44603 L
.28517 .44124 L
.28223 .43585 L
.27932 .43038 L
.27625 .42537 L
.27284 .42129 L
.26897 .41851 L
.26458 .41718 L
.25968 .41728 L
.25435 .41858 L
Mistroke
.24873 .42069 L
.243 .42312 L
.23735 .42531 L
.23197 .42675 L
.22701 .42704 L
.22254 .42592 L
.2186 .42334 L
.21513 .41943 L
.21201 .41453 L
.20909 .40909 L
Mfstroke
.36205 .46372 m
.31679 .42825 L
.30456 .4625 L
F
gsave
.36006 .41569 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.20909 .59091 Mdot
.20909 .40909 Mdot
.68182 .5 Mdot
.46364 .5 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N83) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
.65412 .5537 m
.67804 .60598 L
.70463 .58117 L
F
gsave
.64871 .59422 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66968 .45617 L
.67517 .45757 L
.67997 .45818 L
.68409 .45755 L
.68771 .45547 L
.69108 .452 L
.6945 .44748 L
.69821 .44244 L
.70237 .43756 L
.70699 .43353 L
.71199 .43095 L
.71714 .43021 L
.7222 .43145 L
.72692 .4345 L
.73113 .43896 L
.7348 .44422 L
.738 .4496 L
.74096 .45444 L
.74398 .45821 L
.74738 .46063 L
.75142 .46166 L
.75625 .46153 L
.76185 .46066 L
.76803 .45964 L
.77441 .45904 L
.78054 .45937 L
.78592 .461 L
.79013 .46406 L
.79289 .46848 L
.79415 .47396 L
.79406 .48009 L
.79302 .48642 L
.79157 .4925 L
.79032 .49804 L
.78986 .50285 L
.7906 .50695 L
.79279 .51051 L
.79634 .51378 L
.80096 .51706 L
.80611 .52063 L
.8111 .52465 L
.81526 .52916 L
.81799 .53408 L
.81887 .53921 L
.81778 .5443 L
.81486 .5491 L
.81052 .55344 L
.80537 .55725 L
.80009 .56061 L
Mistroke
.79534 .5637 L
.79165 .56683 L
.78932 .57029 L
.78841 .57436 L
.78868 .57919 L
.78971 .58477 L
.79091 .59091 L
Mfstroke
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N84) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
.65412 .5537 m
.67804 .60598 L
.70463 .58117 L
F
gsave
.64871 .59422 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66968 .45617 L
.67517 .45757 L
.67997 .45818 L
.68409 .45755 L
.68771 .45547 L
.69108 .452 L
.6945 .44748 L
.69821 .44244 L
.70237 .43756 L
.70699 .43353 L
.71199 .43095 L
.71714 .43021 L
.7222 .43145 L
.72692 .4345 L
.73113 .43896 L
.7348 .44422 L
.738 .4496 L
.74096 .45444 L
.74398 .45821 L
.74738 .46063 L
.75142 .46166 L
.75625 .46153 L
.76185 .46066 L
.76803 .45964 L
.77441 .45904 L
.78054 .45937 L
.78592 .461 L
.79013 .46406 L
.79289 .46848 L
.79415 .47396 L
.79406 .48009 L
.79302 .48642 L
.79157 .4925 L
.79032 .49804 L
.78986 .50285 L
.7906 .50695 L
.79279 .51051 L
.79634 .51378 L
.80096 .51706 L
.80611 .52063 L
.8111 .52465 L
.81526 .52916 L
.81799 .53408 L
.81887 .53921 L
.81778 .5443 L
.81486 .5491 L
.81052 .55344 L
.80537 .55725 L
.80009 .56061 L
Mistroke
.79534 .5637 L
.79165 .56683 L
.78932 .57029 L
.78841 .57436 L
.78868 .57919 L
.78971 .58477 L
.79091 .59091 L
Mfstroke
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N85) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
gsave
.65702 .58646 -71 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66967 .45614 L
.67515 .45752 L
.67994 .45809 L
.68405 .45743 L
.68767 .45534 L
.69105 .45189 L
.69448 .4474 L
.69821 .44244 L
.70238 .43767 L
.70701 .43379 L
.71199 .43137 L
.71711 .43079 L
.72213 .43213 L
.72682 .43517 L
.73104 .43943 L
.7348 .44422 L
.73824 .4488 L
.74165 .45247 L
.74536 .45482 L
.74958 .45588 L
.75431 .45614 L
.75922 .45647 L
.76387 .45757 L
.76803 .45964 L
.77175 .46235 L
.77524 .46537 L
.7786 .46852 L
.78182 .47182 L
.78488 .47526 L
.7878 .47882 L
.79057 .48251 L
.79302 .48642 L
.79479 .4907 L
.79558 .49542 L
.79556 .50034 L
.7955 .50508 L
.79626 .50937 L
.79836 .51322 L
.80178 .51688 L
.80611 .52063 L
.81063 .5247 L
.81459 .52921 L
.81731 .5341 L
.81829 .53919 L
.81736 .54427 L
.8146 .54907 L
.81041 .55342 L
.80537 .55725 L
.80016 .56063 L
Mistroke
.79545 .56374 L
.79177 .56688 L
.78943 .57034 L
.78849 .5744 L
.78874 .57922 L
.78973 .58478 L
.79091 .59091 L
Mfstroke
.76321 .45188 m
.81372 .47935 L
.78713 .50416 L
F
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N86) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
gsave
.65702 .58646 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66967 .45614 L
.67515 .45752 L
.67994 .45809 L
.68405 .45743 L
.68767 .45534 L
.69105 .45189 L
.69448 .4474 L
.69821 .44244 L
.70238 .43767 L
.70701 .43379 L
.71199 .43137 L
.71711 .43079 L
.72213 .43213 L
.72682 .43517 L
.73104 .43943 L
.7348 .44422 L
.73824 .4488 L
.74165 .45247 L
.74536 .45482 L
.74958 .45588 L
.75431 .45614 L
.75922 .45647 L
.76387 .45757 L
.76803 .45964 L
.77175 .46235 L
.77524 .46537 L
.7786 .46852 L
.78182 .47182 L
.78488 .47526 L
.7878 .47882 L
.79057 .48251 L
.79302 .48642 L
.79479 .4907 L
.79558 .49542 L
.79556 .50034 L
.7955 .50508 L
.79626 .50937 L
.79836 .51322 L
.80178 .51688 L
.80611 .52063 L
.81063 .5247 L
.81459 .52921 L
.81731 .5341 L
.81829 .53919 L
.81736 .54427 L
.8146 .54907 L
.81041 .55342 L
.80537 .55725 L
.80016 .56063 L
Mistroke
.79545 .56374 L
.79177 .56688 L
.78943 .57034 L
.78849 .5744 L
.78874 .57922 L
.78973 .58478 L
.79091 .59091 L
Mfstroke
.76321 .45188 m
.81372 .47935 L
.78713 .50416 L
F
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N87) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
gsave
.65702 .58646 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66967 .45614 L
.67515 .45752 L
.67994 .45809 L
.68405 .45743 L
.68767 .45534 L
.69105 .45189 L
.69448 .4474 L
.69821 .44244 L
.70238 .43767 L
.70701 .43379 L
.71199 .43137 L
.71711 .43079 L
.72213 .43213 L
.72682 .43517 L
.73104 .43943 L
.7348 .44422 L
.73824 .4488 L
.74165 .45247 L
.74536 .45482 L
.74958 .45588 L
.75431 .45614 L
.75922 .45647 L
.76387 .45757 L
.76803 .45964 L
.77175 .46235 L
.77524 .46537 L
.7786 .46852 L
.78182 .47182 L
.78488 .47526 L
.7878 .47882 L
.79057 .48251 L
.79302 .48642 L
.79479 .4907 L
.79558 .49542 L
.79556 .50034 L
.7955 .50508 L
.79626 .50937 L
.79836 .51322 L
.80178 .51688 L
.80611 .52063 L
.81063 .5247 L
.81459 .52921 L
.81731 .5341 L
.81829 .53919 L
.81736 .54427 L
.8146 .54907 L
.81041 .55342 L
.80537 .55725 L
.80016 .56063 L
Mistroke
.79545 .56374 L
.79177 .56688 L
.78943 .57034 L
.78849 .5744 L
.78874 .57922 L
.78973 .58478 L
.79091 .59091 L
Mfstroke
.76321 .45188 m
.81372 .47935 L
.78713 .50416 L
F
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N88) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
.65412 .5537 m
.67804 .60598 L
.70463 .58117 L
F
gsave
.64871 .59422 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66968 .45617 L
.67517 .45757 L
.67997 .45818 L
.68409 .45755 L
.68771 .45547 L
.69108 .452 L
.6945 .44748 L
.69821 .44244 L
.70237 .43756 L
.70699 .43353 L
.71199 .43095 L
.71714 .43021 L
.7222 .43145 L
.72692 .4345 L
.73113 .43896 L
.7348 .44422 L
.738 .4496 L
.74096 .45444 L
.74398 .45821 L
.74738 .46063 L
.75142 .46166 L
.75625 .46153 L
.76185 .46066 L
.76803 .45964 L
.77441 .45904 L
.78054 .45937 L
.78592 .461 L
.79013 .46406 L
.79289 .46848 L
.79415 .47396 L
.79406 .48009 L
.79302 .48642 L
.79157 .4925 L
.79032 .49804 L
.78986 .50285 L
.7906 .50695 L
.79279 .51051 L
.79634 .51378 L
.80096 .51706 L
.80611 .52063 L
.8111 .52465 L
.81526 .52916 L
.81799 .53408 L
.81887 .53921 L
.81778 .5443 L
.81486 .5491 L
.81052 .55344 L
.80537 .55725 L
.80009 .56061 L
Mistroke
.79534 .5637 L
.79165 .56683 L
.78932 .57029 L
.78841 .57436 L
.78868 .57919 L
.78971 .58477 L
.79091 .59091 L
Mfstroke
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N89) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
.65412 .5537 m
.67804 .60598 L
.70463 .58117 L
F
gsave
.64871 .59422 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66968 .45617 L
.67517 .45757 L
.67997 .45818 L
.68409 .45755 L
.68771 .45547 L
.69108 .452 L
.6945 .44748 L
.69821 .44244 L
.70237 .43756 L
.70699 .43353 L
.71199 .43095 L
.71714 .43021 L
.7222 .43145 L
.72692 .4345 L
.73113 .43896 L
.7348 .44422 L
.738 .4496 L
.74096 .45444 L
.74398 .45821 L
.74738 .46063 L
.75142 .46166 L
.75625 .46153 L
.76185 .46066 L
.76803 .45964 L
.77441 .45904 L
.78054 .45937 L
.78592 .461 L
.79013 .46406 L
.79289 .46848 L
.79415 .47396 L
.79406 .48009 L
.79302 .48642 L
.79157 .4925 L
.79032 .49804 L
.78986 .50285 L
.7906 .50695 L
.79279 .51051 L
.79634 .51378 L
.80096 .51706 L
.80611 .52063 L
.8111 .52465 L
.81526 .52916 L
.81799 .53408 L
.81887 .53921 L
.81778 .5443 L
.81486 .5491 L
.81052 .55344 L
.80537 .55725 L
.80009 .56061 L
Mistroke
.79534 .5637 L
.79165 .56683 L
.78932 .57029 L
.78841 .57436 L
.78868 .57919 L
.78971 .58477 L
.79091 .59091 L
Mfstroke
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N90) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
gsave
.65702 .58646 -71 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66967 .45614 L
.67515 .45752 L
.67994 .45809 L
.68405 .45743 L
.68767 .45534 L
.69105 .45189 L
.69448 .4474 L
.69821 .44244 L
.70238 .43767 L
.70701 .43379 L
.71199 .43137 L
.71711 .43079 L
.72213 .43213 L
.72682 .43517 L
.73104 .43943 L
.7348 .44422 L
.73824 .4488 L
.74165 .45247 L
.74536 .45482 L
.74958 .45588 L
.75431 .45614 L
.75922 .45647 L
.76387 .45757 L
.76803 .45964 L
.77175 .46235 L
.77524 .46537 L
.7786 .46852 L
.78182 .47182 L
.78488 .47526 L
.7878 .47882 L
.79057 .48251 L
.79302 .48642 L
.79479 .4907 L
.79558 .49542 L
.79556 .50034 L
.7955 .50508 L
.79626 .50937 L
.79836 .51322 L
.80178 .51688 L
.80611 .52063 L
.81063 .5247 L
.81459 .52921 L
.81731 .5341 L
.81829 .53919 L
.81736 .54427 L
.8146 .54907 L
.81041 .55342 L
.80537 .55725 L
.80016 .56063 L
Mistroke
.79545 .56374 L
.79177 .56688 L
.78943 .57034 L
.78849 .5744 L
.78874 .57922 L
.78973 .58478 L
.79091 .59091 L
Mfstroke
.76321 .45188 m
.81372 .47935 L
.78713 .50416 L
F
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N91) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
gsave
.65702 .58646 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66967 .45614 L
.67515 .45752 L
.67994 .45809 L
.68405 .45743 L
.68767 .45534 L
.69105 .45189 L
.69448 .4474 L
.69821 .44244 L
.70238 .43767 L
.70701 .43379 L
.71199 .43137 L
.71711 .43079 L
.72213 .43213 L
.72682 .43517 L
.73104 .43943 L
.7348 .44422 L
.73824 .4488 L
.74165 .45247 L
.74536 .45482 L
.74958 .45588 L
.75431 .45614 L
.75922 .45647 L
.76387 .45757 L
.76803 .45964 L
.77175 .46235 L
.77524 .46537 L
.7786 .46852 L
.78182 .47182 L
.78488 .47526 L
.7878 .47882 L
.79057 .48251 L
.79302 .48642 L
.79479 .4907 L
.79558 .49542 L
.79556 .50034 L
.7955 .50508 L
.79626 .50937 L
.79836 .51322 L
.80178 .51688 L
.80611 .52063 L
.81063 .5247 L
.81459 .52921 L
.81731 .5341 L
.81829 .53919 L
.81736 .54427 L
.8146 .54907 L
.81041 .55342 L
.80537 .55725 L
.80016 .56063 L
Mistroke
.79545 .56374 L
.79177 .56688 L
.78943 .57034 L
.78849 .5744 L
.78874 .57922 L
.78973 .58478 L
.79091 .59091 L
Mfstroke
.76321 .45188 m
.81372 .47935 L
.78713 .50416 L
F
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N92) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .59091 L
s
.85178 .64163 m
.88204 .69052 L
.90532 .66258 L
F
gsave
.85511 .68572 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.66364 .45455 L
s
.83222 .34918 m
.7956 .39351 L
.77633 .36267 L
F
gsave
.81894 .39589 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.65963 .45924 L
.65555 .46337 L
.65133 .46646 L
.64693 .46816 L
.64233 .46833 L
.63753 .46702 L
.63258 .46449 L
.62752 .46118 L
.62242 .45761 L
.61737 .45436 L
.61243 .45195 L
.60765 .45079 L
.60307 .45113 L
.5987 .453 L
.5945 .45622 L
.59043 .46045 L
.58642 .46517 L
.58241 .46982 L
.57832 .47386 L
.57408 .47681 L
.56966 .47835 L
.56504 .47835 L
.56022 .47688 L
.55525 .47425 L
.55018 .47088 L
.54509 .46732 L
.54005 .46413 L
.53512 .46184 L
.53037 .46085 L
.52581 .46136 L
.52145 .46339 L
.51727 .46675 L
.51321 .47105 L
.50921 .47579 L
.50519 .48041 L
.50109 .48435 L
.49683 .48715 L
.49239 .48852 L
.48774 .48835 L
.48291 .48674 L
.47792 .484 L
.47285 .48058 L
.46776 .47703 L
.46272 .47393 L
.45781 .47176 L
.45308 .47093 L
.44855 .47161 L
.44421 .47381 L
.44005 .47729 L
Mistroke
.436 .48167 L
.432 .48643 L
.42798 .49099 L
.42386 .49483 L
.41958 .49748 L
.41512 .49868 L
.41045 .49834 L
.40559 .49659 L
.40059 .49375 L
.39551 .49029 L
.39043 .48676 L
.3854 .48374 L
.38051 .48171 L
.3758 .48104 L
.37129 .48189 L
.36697 .48424 L
.36282 .48785 L
.35879 .4923 L
.35479 .49706 L
.35076 .50157 L
.34663 .5053 L
.34233 .5078 L
.33784 .50883 L
.33315 .50832 L
.32828 .50643 L
.32327 .5035 L
.31818 .5 L
Mfstroke
gsave
.48172 .43491 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.74261 .50841 .0956 59.6553 214.295 arc
s
gsave
.65702 .58646 -72 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .45455 m
.66967 .45614 L
.67515 .45752 L
.67994 .45809 L
.68405 .45743 L
.68767 .45534 L
.69105 .45189 L
.69448 .4474 L
.69821 .44244 L
.70238 .43767 L
.70701 .43379 L
.71199 .43137 L
.71711 .43079 L
.72213 .43213 L
.72682 .43517 L
.73104 .43943 L
.7348 .44422 L
.73824 .4488 L
.74165 .45247 L
.74536 .45482 L
.74958 .45588 L
.75431 .45614 L
.75922 .45647 L
.76387 .45757 L
.76803 .45964 L
.77175 .46235 L
.77524 .46537 L
.7786 .46852 L
.78182 .47182 L
.78488 .47526 L
.7878 .47882 L
.79057 .48251 L
.79302 .48642 L
.79479 .4907 L
.79558 .49542 L
.79556 .50034 L
.7955 .50508 L
.79626 .50937 L
.79836 .51322 L
.80178 .51688 L
.80611 .52063 L
.81063 .5247 L
.81459 .52921 L
.81731 .5341 L
.81829 .53919 L
.81736 .54427 L
.8146 .54907 L
.81041 .55342 L
.80537 .55725 L
.80016 .56063 L
Mistroke
.79545 .56374 L
.79177 .56688 L
.78943 .57034 L
.78849 .5744 L
.78874 .57922 L
.78973 .58478 L
.79091 .59091 L
Mfstroke
.76321 .45188 m
.81372 .47935 L
.78713 .50416 L
F
gsave
.80583 .45124 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .59091 Mdot
.66364 .45455 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N93) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
.80043 .50824 m
.7765 .56053 L
.74992 .53571 L
F
gsave
.80583 .54876 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66484 .53931 L
.66586 .53374 L
.66613 .52891 L
.66522 .52484 L
.6629 .52138 L
.65921 .51825 L
.65446 .51515 L
.64918 .5118 L
.64402 .50799 L
.63969 .50365 L
.63677 .49884 L
.63568 .49375 L
.63656 .48862 L
.63928 .4837 L
.64344 .47919 L
.64844 .47517 L
.65358 .47161 L
.6582 .46832 L
.66176 .46505 L
.66394 .4615 L
.66469 .45739 L
.66422 .45258 L
.66298 .44705 L
.66152 .44096 L
.66048 .43464 L
.6604 .4285 L
.66165 .42302 L
.66442 .41861 L
.66863 .41555 L
.67401 .41392 L
.68014 .41358 L
.68652 .41418 L
.69269 .41521 L
.6983 .41607 L
.70313 .41621 L
.70717 .41518 L
.71057 .41276 L
.71359 .40898 L
.71655 .40415 L
.71975 .39877 L
.72341 .39351 L
.72763 .38905 L
.73235 .38599 L
.7374 .38476 L
.74256 .3855 L
.74755 .38808 L
.75218 .3921 L
.75634 .39698 L
.76005 .40202 L
Mistroke
.76346 .40655 L
.76684 .41001 L
.77045 .41209 L
.77458 .41272 L
.77937 .41212 L
.78486 .41071 L
.79091 .40909 L
Mfstroke
gsave
.64871 .40578 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N94) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
.80043 .50824 m
.7765 .56053 L
.74992 .53571 L
F
gsave
.80583 .54876 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66484 .53931 L
.66586 .53374 L
.66613 .52891 L
.66522 .52484 L
.6629 .52138 L
.65921 .51825 L
.65446 .51515 L
.64918 .5118 L
.64402 .50799 L
.63969 .50365 L
.63677 .49884 L
.63568 .49375 L
.63656 .48862 L
.63928 .4837 L
.64344 .47919 L
.64844 .47517 L
.65358 .47161 L
.6582 .46832 L
.66176 .46505 L
.66394 .4615 L
.66469 .45739 L
.66422 .45258 L
.66298 .44705 L
.66152 .44096 L
.66048 .43464 L
.6604 .4285 L
.66165 .42302 L
.66442 .41861 L
.66863 .41555 L
.67401 .41392 L
.68014 .41358 L
.68652 .41418 L
.69269 .41521 L
.6983 .41607 L
.70313 .41621 L
.70717 .41518 L
.71057 .41276 L
.71359 .40898 L
.71655 .40415 L
.71975 .39877 L
.72341 .39351 L
.72763 .38905 L
.73235 .38599 L
.7374 .38476 L
.74256 .3855 L
.74755 .38808 L
.75218 .3921 L
.75634 .39698 L
.76005 .40202 L
Mistroke
.76346 .40655 L
.76684 .41001 L
.77045 .41209 L
.77458 .41272 L
.77937 .41212 L
.78486 .41071 L
.79091 .40909 L
Mfstroke
gsave
.64871 .40578 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N95) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
gsave
.79753 .54101 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66482 .53933 L
.66581 .53377 L
.66605 .52895 L
.66511 .52489 L
.66278 .52142 L
.6591 .51829 L
.65439 .51517 L
.64918 .5118 L
.64413 .50797 L
.63994 .50362 L
.63719 .49881 L
.63625 .49374 L
.63724 .48864 L
.63995 .48375 L
.64391 .47925 L
.64844 .47517 L
.65276 .47142 L
.65619 .46777 L
.65829 .46391 L
.65905 .45962 L
.65898 .45489 L
.65897 .44997 L
.65975 .44525 L
.66152 .44096 L
.66398 .43706 L
.66675 .43337 L
.66966 .4298 L
.67273 .42636 L
.67595 .42307 L
.67931 .41991 L
.68279 .4169 L
.68652 .41418 L
.69067 .41212 L
.69533 .41101 L
.70024 .41069 L
.70496 .41042 L
.70919 .40937 L
.71289 .40701 L
.71631 .40334 L
.71975 .39877 L
.7235 .39397 L
.72772 .38971 L
.73241 .38667 L
.73743 .38533 L
.74256 .38592 L
.74754 .38834 L
.75217 .39222 L
.75634 .39698 L
.76006 .40195 L
Mistroke
.76349 .40643 L
.76687 .40989 L
.77049 .41198 L
.77461 .41264 L
.7794 .41206 L
.78488 .41069 L
.79091 .40909 L
Mfstroke
.69134 .40643 m
.64083 .4339 L
.66741 .45871 L
F
gsave
.64871 .40578 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N96) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
gsave
.79753 .54101 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66482 .53933 L
.66581 .53377 L
.66605 .52895 L
.66511 .52489 L
.66278 .52142 L
.6591 .51829 L
.65439 .51517 L
.64918 .5118 L
.64413 .50797 L
.63994 .50362 L
.63719 .49881 L
.63625 .49374 L
.63724 .48864 L
.63995 .48375 L
.64391 .47925 L
.64844 .47517 L
.65276 .47142 L
.65619 .46777 L
.65829 .46391 L
.65905 .45962 L
.65898 .45489 L
.65897 .44997 L
.65975 .44525 L
.66152 .44096 L
.66398 .43706 L
.66675 .43337 L
.66966 .4298 L
.67273 .42636 L
.67595 .42307 L
.67931 .41991 L
.68279 .4169 L
.68652 .41418 L
.69067 .41212 L
.69533 .41101 L
.70024 .41069 L
.70496 .41042 L
.70919 .40937 L
.71289 .40701 L
.71631 .40334 L
.71975 .39877 L
.7235 .39397 L
.72772 .38971 L
.73241 .38667 L
.73743 .38533 L
.74256 .38592 L
.74754 .38834 L
.75217 .39222 L
.75634 .39698 L
.76006 .40195 L
Mistroke
.76349 .40643 L
.76687 .40989 L
.77049 .41198 L
.77461 .41264 L
.7794 .41206 L
.78488 .41069 L
.79091 .40909 L
Mfstroke
.69134 .40643 m
.64083 .4339 L
.66741 .45871 L
F
gsave
.64871 .40578 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N97) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
gsave
.79753 .54101 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66482 .53933 L
.66581 .53377 L
.66605 .52895 L
.66511 .52489 L
.66278 .52142 L
.6591 .51829 L
.65439 .51517 L
.64918 .5118 L
.64413 .50797 L
.63994 .50362 L
.63719 .49881 L
.63625 .49374 L
.63724 .48864 L
.63995 .48375 L
.64391 .47925 L
.64844 .47517 L
.65276 .47142 L
.65619 .46777 L
.65829 .46391 L
.65905 .45962 L
.65898 .45489 L
.65897 .44997 L
.65975 .44525 L
.66152 .44096 L
.66398 .43706 L
.66675 .43337 L
.66966 .4298 L
.67273 .42636 L
.67595 .42307 L
.67931 .41991 L
.68279 .4169 L
.68652 .41418 L
.69067 .41212 L
.69533 .41101 L
.70024 .41069 L
.70496 .41042 L
.70919 .40937 L
.71289 .40701 L
.71631 .40334 L
.71975 .39877 L
.7235 .39397 L
.72772 .38971 L
.73241 .38667 L
.73743 .38533 L
.74256 .38592 L
.74754 .38834 L
.75217 .39222 L
.75634 .39698 L
.76006 .40195 L
Mistroke
.76349 .40643 L
.76687 .40989 L
.77049 .41198 L
.77461 .41264 L
.7794 .41206 L
.78488 .41069 L
.79091 .40909 L
Mfstroke
.69134 .40643 m
.64083 .4339 L
.66741 .45871 L
F
gsave
.64871 .40578 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N98) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
.80043 .50824 m
.7765 .56053 L
.74992 .53571 L
F
gsave
.80583 .54876 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66484 .53931 L
.66586 .53374 L
.66613 .52891 L
.66522 .52484 L
.6629 .52138 L
.65921 .51825 L
.65446 .51515 L
.64918 .5118 L
.64402 .50799 L
.63969 .50365 L
.63677 .49884 L
.63568 .49375 L
.63656 .48862 L
.63928 .4837 L
.64344 .47919 L
.64844 .47517 L
.65358 .47161 L
.6582 .46832 L
.66176 .46505 L
.66394 .4615 L
.66469 .45739 L
.66422 .45258 L
.66298 .44705 L
.66152 .44096 L
.66048 .43464 L
.6604 .4285 L
.66165 .42302 L
.66442 .41861 L
.66863 .41555 L
.67401 .41392 L
.68014 .41358 L
.68652 .41418 L
.69269 .41521 L
.6983 .41607 L
.70313 .41621 L
.70717 .41518 L
.71057 .41276 L
.71359 .40898 L
.71655 .40415 L
.71975 .39877 L
.72341 .39351 L
.72763 .38905 L
.73235 .38599 L
.7374 .38476 L
.74256 .3855 L
.74755 .38808 L
.75218 .3921 L
.75634 .39698 L
.76005 .40202 L
Mistroke
.76346 .40655 L
.76684 .41001 L
.77045 .41209 L
.77458 .41272 L
.77937 .41212 L
.78486 .41071 L
.79091 .40909 L
Mfstroke
gsave
.64871 .40578 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N99) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
.80043 .50824 m
.7765 .56053 L
.74992 .53571 L
F
gsave
.80583 .54876 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66484 .53931 L
.66586 .53374 L
.66613 .52891 L
.66522 .52484 L
.6629 .52138 L
.65921 .51825 L
.65446 .51515 L
.64918 .5118 L
.64402 .50799 L
.63969 .50365 L
.63677 .49884 L
.63568 .49375 L
.63656 .48862 L
.63928 .4837 L
.64344 .47919 L
.64844 .47517 L
.65358 .47161 L
.6582 .46832 L
.66176 .46505 L
.66394 .4615 L
.66469 .45739 L
.66422 .45258 L
.66298 .44705 L
.66152 .44096 L
.66048 .43464 L
.6604 .4285 L
.66165 .42302 L
.66442 .41861 L
.66863 .41555 L
.67401 .41392 L
.68014 .41358 L
.68652 .41418 L
.69269 .41521 L
.6983 .41607 L
.70313 .41621 L
.70717 .41518 L
.71057 .41276 L
.71359 .40898 L
.71655 .40415 L
.71975 .39877 L
.72341 .39351 L
.72763 .38905 L
.73235 .38599 L
.7374 .38476 L
.74256 .3855 L
.74755 .38808 L
.75218 .3921 L
.75634 .39698 L
.76005 .40202 L
Mistroke
.76346 .40655 L
.76684 .41001 L
.77045 .41209 L
.77458 .41272 L
.77937 .41212 L
.78486 .41071 L
.79091 .40909 L
Mfstroke
gsave
.64871 .40578 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N100) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00100oooo00<000000?ooo`3o
ool00`3oool3000000@0oooo00@000000?ooo`000000oooo0P00000S0?ooo`030000003oool0oooo
00<0oooo0`000000203oool000000000003oool000000?ooo`000000oooo0P0000000`3oool00000
0000000P0?ooo`030000003oool0oooo00<0oooo0`000000203oool000000000003oool000000?oo
o`000000oooo0P0000000`3oool000000000000N0?ooo`030000003oool0oooo00<0oooo0`000005
0?ooo`030000003oool000000080oooo00D000000?ooo`00000000000?ooo`02000000l0oooo000@
0?ooo`040000003oool0000000000080oooo00@000000?ooo`3oool0oooo0P000000103oool00000
0000000000020?ooo`80000000<0oooo0000003oool07`3oool010000000oooo0000000000020?oo
o`030000003oool0oooo0080oooo1@0000020?ooo`@00000803oool010000000oooo000000000002
0?ooo`040000003oool0oooo0?ooo`H000000P3oool2000000030?ooo`000000000001d0oooo00@0
00000?ooo`00000000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<00000
0P3oool010000000oooo0000003oool2000000030?ooo`000000oooo00d0oooo000@0?ooo`040000
003oool0000000000080oooo00@000000?ooo`3oool0oooo0P0000002P3oool00000000000000000
oooo0000003oool000000?ooo`00000P0?ooo`040000003oool0000000000080oooo00D000000?oo
o`3oool0oooo000000020?ooo`<0000000<0oooo0000003oool00`00000P0?ooo`040000003oool0
000000000080oooo00D000000?ooo`3oool0oooo000000020?ooo`<0000000<0oooo0000003oool0
0P0000000`3oool000000?ooo`0L0?ooo`040000003oool0000000000080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0300000080oooo00D000000?ooo`00000000000?ooo`020000
00l0oooo000?0?ooo`<000001@3oool2000000@0oooo00<000000?ooo`3oool00P3oool00`000000
oooo00000002000001h0oooo0`0000050?ooo`8000000P3oool00`000000oooo000000040?ooo`04
0000003oool0oooo000001l0oooo0`0000050?ooo`8000000P3oool00`000000oooo000000040?oo
o`040000003oool0oooo000001d0oooo0`0000050?ooo`80000000H0oooo00000000000000000?oo
o`0000070?ooo`040000003oool0oooo000000l0oooo000<0?ooo`800000=03oool2000003@0oooo
0P00000e0?ooo`800000:03oool000`0oooo0`00000c0?ooo`<00000<`3oool3000000h0oooo00<0
00000?ooo`3oool08`3oool3000002L0oooo000=0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`000000903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0000002@0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00`0oooo0P00000H
0?ooo`006`3oool400000380oooo1000000c0?ooo`030000003oool0000003D0oooo00<000000?oo
o`3oool05P3oool00080oooo00<000000?ooo`3oool05P3oool4000001@0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0F0?ooo`@00000503oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo01P0oooo0`00000C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0040oooo00030?ooo`030000
003oool0oooo01<0oooo00L000000?ooo`000000oooo0000003oool000000180oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0C0?ooo`070000003oool000000?ooo`000000oooo0000
000B0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo01L0oooo0`00000B0?ooo`030000003oool0oooo0080oooo0004
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo0180oooo0P00000G0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool04P3oool2000001L0oooo00<000000?ooo`3oool00P3oool000D0oooo0P00000A0?oo
o`030000003oool0000001H0oooo00<000000?ooo`3oool01`3oool200000140oooo00<000000?oo
o`0000005P3oool00`000000oooo0?ooo`070?ooo`8000004`3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo00L0oooo0P00000C0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool00`3oool000L0oooo00<000000?ooo`3oool03`3oool300000140oooo1000000<0?ooo`03
0000003oool0oooo00l0oooo0`00000A0?ooo`@00000303oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo1000000<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0403oool4000000H0oooo00080?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3P3oool300000180oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0>0?ooo`<00000
4P3oool00`000000oooo0?ooo`060?ooo`00203oool2000002<0oooo00<000000?ooo`3oool03P3o
ool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3o
ool2000002<0oooo00<000000?ooo`3oool01`3oool000T0oooo0P00000K0?ooo`@000000P3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01T0oooo100000020?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool06P3oool300000080oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0J0?ooo`<000000P3oool00`000000oooo0?ooo`080?ooo`000P3oool20000
00L0oooo00<000000?ooo`3oool05P3oool300000080oooo0`00000=0?ooo`8000001P3oool00`00
0000oooo0?ooo`0F0?ooo`<000000P3oool3000000d0oooo0P0000060?ooo`030000003oool0oooo
01H0oooo0`0000020?ooo`<000003@3oool2000000H0oooo00<000000?ooo`3oool05P3oool30000
0080oooo0`00000;0?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool04`3oool3000000H0
oooo00<000000?ooo`3oool02`3oool3000000H0oooo00<000000?ooo`3oool04`3oool3000000H0
oooo00<000000?ooo`3oool02`3oool3000000H0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00/0oooo0`0000060?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`090?ooo`000`3oool00`000000oooo
0?ooo`070?ooo`800000503oool2000001D0oooo00<000000?ooo`3oool01P3oool2000001@0oooo
0P00000E0?ooo`030000003oool0oooo00H0oooo0P00000D0?ooo`8000005@3oool00`000000oooo
0?ooo`060?ooo`800000503oool200000180oooo000?0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`8000001P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool2000000H0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`00403oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0103oool300000140oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00@0oooo0`00000A
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`04
0?ooo`<000004@3oool00`000000oooo0?ooo`0?0?ooo`8000001`3oool00`000000oooo0?ooo`04
0?ooo`<000000P3oool00100oooo1P000000103oool000000000000000080?ooo`8000001`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool000000140oooo1P000000103oool0000000000000
00080?ooo`8000001`3oool00`000000oooo0?ooo`040?ooo`030000003oool000000140oooo1P00
0000103oool000000000000000080?ooo`8000001`3oool00`000000oooo0?ooo`040?ooo`030000
003oool000000140oooo1P000000103oool000000000000000080?ooo`8000001`3oool00`000000
oooo0?ooo`040?ooo`030000003oool000000080oooo000@0?ooo`030000003oool000000080oooo
0`0000000`3oool000000?ooo`0300000080oooo0P0000000`3oool00000000000050?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool00`000000oooo000000020?ooo`<00000
00<0oooo0000000000000`0000020?ooo`80000000<0oooo0000000000001@3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0140oooo00<000000?ooo`0000000P3oool3000000030?oo
o`000000000000<000000P3oool2000000030?ooo`000000000000D0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool000000080oooo0`0000000`3oool00000
0000000300000080oooo0P0000000`3oool00000000000050?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool00P3oool000h0oooo0P00000:0?ooo`030000003oool0oooo00<0000000<0
oooo0000000000000P0000030?ooo`8000006P3oool2000000X0oooo0P0000000`3oool000000000
0006000000@0oooo00<000000?ooo`3oool0603oool2000000X0oooo0P0000000`3oool000000000
0006000000<0oooo0`00000I0?ooo`8000002P3oool2000000030?ooo`000000000000H000000`3o
ool3000000`0oooo000=0?ooo`030000003oool0oooo0180oooo1P00000K0?ooo`030000003oool0
oooo0180oooo1P00000K0?ooo`030000003oool0oooo0180oooo1P00000K0?ooo`030000003oool0
oooo0180oooo1P00000?0?ooo`00303oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0F0?ooo`<000003`3oool000X0
oooo0P00000L0?ooo`030000003oool0oooo01D0oooo0P00000L0?ooo`030000003oool0oooo01D0
oooo0P00000L0?ooo`030000003oool0oooo01D0oooo0P00000L0?ooo`030000003oool0oooo00`0
oooo00090?ooo`8000007P3oool00`000000oooo0?ooo`0C0?ooo`8000007`3oool00`000000oooo
0?ooo`0B0?ooo`8000007`3oool00`000000oooo0?ooo`0B0?ooo`8000007`3oool00`000000oooo
0?ooo`0:0?ooo`00203oool00`000000oooo0?ooo`0O0?ooo`@00000403oool00`000000oooo0?oo
o`0P0?ooo`@000003`3oool00`000000oooo0?ooo`0P0?ooo`8000004@3oool00`000000oooo0?oo
o`0P0?ooo`8000002P3oool000L0oooo00<000000?ooo`3oool08P3oool4000000d0oooo00<00000
0?ooo`3oool08P3oool2000000030?ooo`000000oooo00`0oooo00<000000?ooo`3oool08P3oool2
000000l0oooo00<000000?ooo`3oool08P3oool2000000T0oooo00060?ooo`030000003oool0oooo
02@0oooo0`00000<0?ooo`030000003oool0oooo02@0oooo0P00000=0?ooo`030000003oool0oooo
02@0oooo0`00000<0?ooo`030000003oool0oooo02@0oooo0`0000070?ooo`00103oool2000002L0
oooo00<000000?ooo`0000000`0000070?ooo`800000:03oool5000000L0oooo0P00000W0?ooo`04
0000003oool00000000000T0oooo0P00000W0?ooo`030000003oool00000008000001@3oool000<0
oooo00<000000?ooo`3oool0:`3oool2000000H0oooo00<000000?ooo`3oool0:`3oool2000000H0
oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00020?ooo`030000003oool0oooo02`0oooo0`0000040?ooo`030000003oool0oooo02`0oooo
0`0000040?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo0040oooo003I0?ooo`00903oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0@
0?ooo`00903oool3000003<0oooo0`00000c0?ooo`<00000<`3oool300000100oooo000T0?ooo`03
0000003oool0000003<0oooo00<000000?ooo`000000<`3oool00`000000oooo0000000c0?ooo`03
0000003oool000000100oooo000T0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0100oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo000>0?ooo`040000003oool0000000000080oooo
0`0000020?ooo`040000003oool0000000000080oooo00@000000?ooo`000000oooo0P0000000`3o
ool000000000000N0?ooo`030000003oool0oooo00<0oooo0`0000020?ooo`060000003oool00000
0?ooo`000000oooo0P00000S0?ooo`030000003oool0oooo00<0oooo0`000000203oool000000000
003oool000000?ooo`000000oooo0P0000000`3oool000000000000P0?ooo`030000003oool0oooo
00<0oooo0`0000000`3oool00000000000020?ooo`040000003oool000000?ooo`80000000<0oooo
000000000000403oool000h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`02
0?ooo`060000003oool000000?ooo`000000oooo0`0000020?ooo`80000000<0oooo0000003oool0
7@3oool010000000oooo0000000000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`00
00000P0000020?ooo`@00000803oool010000000oooo0000000000020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`0000000P0000020?ooo`80000000<0oooo0000000000007`3oool01000
0000oooo0000000000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`000002
0?ooo`D000003`3oool000h0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`02
0?ooo`040000003oool0000000000080oooo0`0000001P3oool000000?ooo`000000oooo000001h0
oooo00@000000?ooo`00000000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool00000
0080000000<0oooo0000003oool00`00000P0?ooo`040000003oool0000000000080oooo00<00000
0?ooo`3oool00`3oool4000000030?ooo`000000oooo008000008@3oool010000000oooo00000000
00020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000000`3oool000000?oo
o`0200000140oooo000=0?ooo`<0000000<0oooo0000003oool00P3oool2000000D0oooo00@00000
0?ooo`3oool00000103oool010000000oooo0?ooo`00000M0?ooo`<000001@3oool2000000@0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0P0?ooo`<000001@3oool200000080oooo
00<000000?ooo`000000103oool010000000oooo0?ooo`00000O0?ooo`<000001@3oool200000080
oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00000@0?ooo`00303oool20000
03@0oooo0P00000d0?ooo`800000=@3oool2000002P0oooo000<0?ooo`<00000<`3oool3000000h0
oooo00<000000?ooo`3oool08P3oool3000003@0oooo0`00000W0?ooo`003@3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`0=0?ooo`8000009@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000001P0oooo
001B0?ooo`030000003oool0000003D0oooo00<000000?ooo`3oool0<03oool4000001P0oooo000H
0?ooo`800000>@3oool3000003@0oooo00<000000?ooo`3oool0<03oool4000001P0oooo00020?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool04P3oool010000000oooo0000003oool2
000000@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`3oool3000001<0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0D0?ooo`070000003oool000000?ooo`000000
oooo0000000C0?ooo`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool04@3oool4000000P0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0080oooo00040?ooo`8000004P3oool300000180oooo1000
00090?ooo`8000004P3oool00`000000oooo0000000F0?ooo`030000003oool0oooo00H0oooo0P00
000B0?ooo`030000003oool0000001H0oooo00<000000?ooo`3oool01P3oool200000180oooo00<0
00000?ooo`0000005P3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`0T
0?ooo`<0000000<0oooo0000003oool02@3oool00`000000oooo0?ooo`0@0?ooo`<000004@3oool4
000000/0oooo00<000000?ooo`3oool0403oool300000140oooo1000000;0?ooo`030000003oool0
oooo0100oooo0`00000A0?ooo`@000001P3oool000L0oooo00<000000?ooo`3oool08P3oool20000
00l0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
00H0oooo00080?ooo`800000803oool200000180oooo0P00000S0?ooo`030000003oool0oooo00h0
oooo0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00L0
oooo00080?ooo`<000007@3oool3000001<0oooo0`00000K0?ooo`<000000`3oool00`000000oooo
0?ooo`0?0?ooo`<00000703oool300000080oooo00<000000?ooo`3oool03`3oool3000001`0oooo
0`0000020?ooo`030000003oool0oooo00P0oooo00020?ooo`8000001`3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo00l0oooo0P0000060?ooo`030000003oool0oooo01D0oooo1000
00020?ooo`<000003@3oool2000000H0oooo00<000000?ooo`3oool05P3oool300000080oooo0`00
000=0?ooo`8000001P3oool00`000000oooo0?ooo`0G0?ooo`8000000P3oool3000000/0oooo0002
0?ooo`<000001`3oool00`000000oooo0?ooo`0F0?ooo`<000004@3oool3000000H0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00/0oooo0`000006
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0;
0?ooo`<000001P3oool00`000000oooo0?ooo`0C0?ooo`<000001P3oool00`000000oooo0?ooo`09
0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0100oooo0P0000001@3o
ool00000000000000000oooo00<00000403oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo01<0oooo0P00000E0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04`3oool2
00000180oooo000>0?ooo`800000203oool2000000030?ooo`000000000000<0000000<0oooo0000
000000000P0000040?ooo`8000006@3oool2000001<0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo0P00000C0?ooo`8000001P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo0P00000C0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`00403oool00`000000
oooo0?ooo`03000000030?ooo`000000oooo00<000000P3oool2000000<0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`040?ooo`<000004@3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0103oool300000140oooo00<000000?ooo`3o
ool03`3oool2000000L0oooo00<000000?ooo`3oool0103oool300000080oooo000@0?ooo`@00000
00<0oooo0000000000002`3oool00`000000oooo0?ooo`060?ooo`040000003oool0oooo000001L0
oooo1P0000000`3oool00000000000090?ooo`8000001`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool000000140oooo0P000000103oool000000000000000020?ooo`800000203oool20000
00L0oooo00<000000?ooo`3oool0103oool00`000000oooo0000000A0?ooo`H000000P3oool20000
00P0oooo0P0000070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000000P3oool00100
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`060?ooo`80000000@0oooo00000000
000000005@3oool00`000000oooo000000020?ooo`<0000000<0oooo0000000000000`0000020?oo
o`80000000<0oooo0000000000001@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0140oooo00<000000?ooo`0000000P3oool3000000030?ooo`000000000000<000000P3oool20000
00030?ooo`000000000000D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool000000080oooo0`0000000`3oool000000000000300000080oooo0P0000000`3o
ool00000000000050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool000h0
oooo0P00000C0?ooo`030000003oool0oooo00H0oooo00D000000?ooo`000000oooo0000000C0?oo
o`8000002P3oool2000000030?ooo`000000000000H000000`3oool3000001T0oooo0P00000:0?oo
o`80000000<0oooo0000000000001P0000030?ooo`<000006@3oool2000000X0oooo0P0000000`3o
ool0000000000006000000@0oooo00<000000?ooo`3oool02`3oool000d0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`040?ooo`8000000P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0180oooo1P00000K0?ooo`030000003oool0oooo0180oooo1P00000K0?ooo`03
0000003oool0oooo0180oooo1P00000?0?ooo`00303oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0000001X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0000000J0?ooo`030000
003oool0oooo01H0oooo0`00000?0?ooo`002P3oool2000001X0oooo0P0000000`3oool000000000
000E0?ooo`800000703oool00`000000oooo0?ooo`0E0?ooo`800000703oool00`000000oooo0?oo
o`0E0?ooo`800000703oool00`000000oooo0?ooo`0<0?ooo`002@3oool200000200oooo0P00000B
0?ooo`8000007`3oool00`000000oooo0?ooo`0B0?ooo`8000007`3oool00`000000oooo0?ooo`0B
0?ooo`8000007`3oool00`000000oooo0?ooo`0:0?ooo`00203oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0803oool200000140oooo00<000000?oo
o`3oool0803oool200000140oooo00<000000?ooo`3oool0803oool3000000T0oooo00070?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0R0?ooo`80
00003`3oool00`000000oooo0?ooo`0R0?ooo`8000003`3oool00`000000oooo0?ooo`0R0?ooo`<0
0000203oool000H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo02@0oooo0`00000<0?ooo`030000003oool0oooo02@0oooo0`00000<0?ooo`030000
003oool0oooo02@0oooo0P0000080?ooo`00103oool2000001@0oooo0P00000D0?ooo`800000203o
ool2000002L0oooo00@000000?ooo`00000000002@3oool2000002L0oooo00@000000?ooo`000000
00002@3oool2000002L0oooo00<000000?ooo`0000000P0000050?ooo`000`3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0000001D0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0Z0?ooo`<00000103oool00080oooo00<000000?ooo`3oool05@3oool200000080oooo0P00
000C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0[0?ooo`80000000<0oooo0000003oool00P3oool001/0oooo00<000000?ooo`3oool00P00002i
0?ooo`007P3oool2000003X0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo0100oooo000W0?ooo`030000003oool0oooo0300oooo0`00000c0?ooo`<0
0000<`3oool300000100oooo000W0?ooo`<00000<03oool00`000000oooo0000000c0?ooo`030000
003oool0000003<0oooo00<000000?ooo`000000403oool002L0oooo00<000000?ooo`000000<03o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0403o
ool002L0oooo00<000000?ooo`3oool0[`3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00403oool010000000oooo0000000000020?ooo`<0000000P0oooo000000000000oooo
0000003oool000000?ooo`80000000<0oooo000000000000803oool010000000oooo000000000002
0?ooo`<00000103oool010000000oooo0000003oool2000000030?ooo`00000000000200oooo00@0
00000?ooo`00000000000P3oool3000000080?ooo`00000000000?ooo`000000oooo0000003oool2
00000080oooo00<000000?ooo`3oool07P3oool010000000oooo0000000000020?ooo`<0000000<0
oooo0000000000000P3oool010000000oooo0000003oool2000000030?ooo`00000000000100oooo
000@0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo1P0000000`3oool0
0000000000020?ooo`8000007`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`0000000P0000020?ooo`<0000000<0oooo0000003oool07P3oool01000
0000oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80000000@0oooo0000000000000000
0P3oool2000000030?ooo`000000oooo01l0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`020?ooo`80000000@0oooo00000000000000000P3oool2000000030?ooo`000000oooo
00l0oooo000@0?ooo`040000003oool0oooo00000080oooo00D000000?ooo`3oool0oooo00000002
0?ooo`<0000000L0oooo000000000000oooo0000003oool0000001l0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`020?ooo`030000003oool000000080000000<0oooo0000003o
ool00`00000P0?ooo`040000003oool0oooo00000080oooo00@000000?ooo`3oool0oooo0P000000
2P3oool00000000000000000oooo0000003oool000000?ooo`00000P0?ooo`040000003oool0oooo
00000080oooo00P000000?ooo`3oool0oooo0000003oool000000?ooo`<0000000@0oooo0000003o
ool000000P3oool00`000000oooo0?ooo`0=0?ooo`003`3oool3000000030?ooo`000000oooo0080
oooo0P0000020?ooo`030000003oool0000000@0oooo00@000000?ooo`3oool000007`3oool30000
00030?ooo`000000oooo0080oooo0P0000000`3oool0000000000002000000@0oooo00@000000?oo
o`3oool000007`3oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`030000003oool0
000000@0oooo00<000000?ooo`3oool0803oool3000000030?ooo`000000oooo0080oooo0P000002
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool00000403oool000`0oooo0P00
000d0?ooo`800000=03oool2000003D0oooo0P00000X0?ooo`00303oool3000003<0oooo0`00000c
0?ooo`<00000=03oool3000002L0oooo000=0?ooo`030000003oool0oooo03<0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo02H0oooo003I0?ooo`00
603oool2000001<0oooo00<000000?ooo`3oool07P3oool2000003@0oooo0P00000d0?ooo`800000
7@3oool00080oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`040000003o
ool0oooo0?ooo`800000103oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo
0P0000030?ooo`800000103oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo
00@000000?ooo`000000oooo0P0000040?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool04P3oool010000000oooo0000003oool2000000<0oooo00030?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0403oool01@000000oooo0000003oool0000000P0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0000000P0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`@00000203oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0140oooo1@0000050?ooo`00103oool200000180oooo0`00000B0?ooo`<0
00002P3oool200000180oooo0`00000C0?ooo`8000002P3oool200000180oooo0`00000B0?ooo`@0
00002@3oool200000180oooo0`00000B0?ooo`@000001P3oool000H0oooo00<000000?ooo`3oool0
9@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02@0oooo0`00000<0?ooo`030000
003oool0oooo02@0oooo0`0000000`3oool000000?ooo`090?ooo`030000003oool0oooo02@0oooo
0`0000000`3oool000000?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0R0?ooo`8000003`3o
ool00`000000oooo0?ooo`0R0?ooo`8000003`3oool00`000000oooo0?ooo`0R0?ooo`8000003`3o
ool00`000000oooo0?ooo`0R0?ooo`8000002@3oool000P0oooo0P00000P0?ooo`8000004P3oool2
00000240oooo00<000000?ooo`3oool0403oool200000240oooo00<000000?ooo`3oool0403oool2
00000240oooo00<000000?ooo`3oool02@3oool000P0oooo0`00000N0?ooo`8000004`3oool30000
01l0oooo00<000000?ooo`3oool04@3oool3000001l0oooo00<000000?ooo`3oool04@3oool30000
01l0oooo00<000000?ooo`3oool02P3oool00080oooo0P0000070?ooo`030000003oool0oooo01T0
oooo0P00000@0?ooo`8000001P3oool00`000000oooo0?ooo`0I0?ooo`800000403oool2000000H0
oooo00<000000?ooo`3oool06@3oool200000100oooo0P0000060?ooo`030000003oool0oooo01T0
oooo0P00000>0?ooo`000P3oool3000000L0oooo00<000000?ooo`3oool05P3oool300000140oooo
0`0000060?ooo`030000003oool0oooo01H0oooo0`00000A0?ooo`<000001P3oool00`000000oooo
0?ooo`0F0?ooo`<000004@3oool3000000H0oooo00<000000?ooo`3oool05`3oool2000000l0oooo
00030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0403oool2000000050?ooo`00
000000000000003oool00P00000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
403oool2000000030?ooo`000000000000@000004@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo0100oooo0P0000020?ooo`D000004@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo0100oooo0P0000020?ooo`D000003P3oool000h0oooo0P0000080?ooo`80000000D0
oooo00000000000000000?ooo`06000000<0oooo0`00000I0?ooo`800000203oool2000000050?oo
o`00000000000000003oool01`0000020?ooo`<000006@3oool2000000P0oooo0P0000001@3oool0
0000000000000000oooo00L000000P3oool3000001T0oooo0P0000080?ooo`80000000D0oooo0000
0000000000000?ooo`0700000080oooo0`00000<0?ooo`00403oool00`000000oooo0?ooo`030000
00030?ooo`000000oooo00<000000P3oool2000000<0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00<0000000<0oooo0000003oool00`0000020?oo
o`8000000`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool00`0000000`3oool000000?ooo`0300000080oooo0P0000030?ooo`030000003oool0oooo
00@0oooo0P00000J0?ooo`030000003oool0oooo00<0000000<0oooo0000000000000`0000020?oo
o`8000000`3oool00`000000oooo0?ooo`040?ooo`8000002`3oool00100oooo100000000`3oool0
00000000000;0?ooo`030000003oool0oooo00D0oooo00D000000?ooo`3oool0oooo0000000G0?oo
o`@0000000<0oooo000000000000303oool00`000000oooo0?ooo`040?ooo`050000003oool0oooo
0?ooo`0000005`3oool4000000040?ooo`0000000000000000/0oooo00<000000?ooo`3oool01@3o
ool010000000oooo0?ooo`00000G0?ooo`@0000000@0oooo00000000000000002`3oool00`000000
oooo0?ooo`050?ooo`040000003oool0oooo000000P0oooo000@0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool01P3oool2000000040?ooo`0000000000000001D0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`050?ooo`80000000@0oooo00000000000000005@3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00D0oooo0P000000103oool0000000000000
000E0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01@3oool2000000040?ooo`00
00000000000000H0oooo000>0?ooo`800000703oool01@000000oooo0000003oool0000001<0oooo
0P00000L0?ooo`050000003oool000000?ooo`0000004`3oool2000001`0oooo00D000000?ooo`00
0000oooo0000000C0?ooo`800000703oool01@000000oooo0000003oool0000000H0oooo000=0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0103oool3000000030?ooo`000000oooo
01<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`050?ooo`80000000<0oooo0000
003oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00D0oooo0P0000000`3o
ool000000?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool01@3oool20000
00030?ooo`000000oooo00L0oooo000<0?ooo`030000003oool0oooo01D0oooo0P0000040?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool05@3oool2000000@0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00<0oooo0P00000;0?ooo`00
2P3oool2000001P0oooo00D000000?ooo`00000000000?ooo`02000001D0oooo0P00000H0?ooo`05
0000003oool000000000003oool00P00000E0?ooo`8000006P3oool2000000030?ooo`0000000000
01D0oooo0P00000J0?ooo`80000000@0oooo00000000000000002`3oool000T0oooo0P00000P0?oo
o`8000004P3oool200000200oooo0P00000B0?ooo`800000803oool200000180oooo0P00000Q0?oo
o`030000003oool0oooo00P0oooo00080?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool01`3oool000L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`060?ooo`001P3o
ool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3o
ool2000000/0oooo00<000000?ooo`3oool09P3oool2000000/0oooo00<000000?ooo`3oool09P3o
ool2000000H0oooo00040?ooo`8000005P3oool00`000000oooo0?ooo`0A0?ooo`800000203oool2
000001H0oooo00<000000?ooo`3oool04@3oool2000000P0oooo0P00000E0?ooo`030000003oool0
oooo0180oooo0P0000080?ooo`800000503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo00<0oooo00030?ooo`030000003oool0oooo01H0oooo0`00000C0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool05P3oool3000001<0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0E0?ooo`8000005@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
01@0oooo0`00000E0?ooo`030000003oool0oooo0080oooo00020?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`000000503oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`000000503oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01H0
oooo0P0000000`3oool000000000000C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool05@3oool00`000000oooo000000020?ooo`8000004P3oool00`000000oooo0?ooo`010?ooo`00
703oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0
0P00000`0?ooo`030000003oool0oooo0080oooo0P00000F0?ooo`00RP3oool2000003D0oooo0P00
000F0?ooo`009`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0=0?ooo`009`3oool3000003<0oooo0`00000c0?oo
o`<00000<`3oool3000000d0oooo000W0?ooo`030000003oool0000003<0oooo00<000000?ooo`00
0000<`3oool00`000000oooo0000000c0?ooo`030000003oool0000000d0oooo000W0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000
003oool0oooo00d0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
000@0?ooo`040000003oool0000000000080oooo0`000000203oool000000000003oool000000?oo
o`000000oooo0P0000000`3oool000000000000P0?ooo`040000003oool0000000000080oooo0`00
0000203oool000000000003oool000000?ooo`000000oooo0P0000000`3oool000000000000P0?oo
o`040000003oool0000000000080oooo0`0000040?ooo`040000003oool000000?ooo`8000008`3o
ool010000000oooo0000000000020?ooo`<0000000<0oooo0000000000000P3oool010000000oooo
0000003oool2000000030?ooo`00000000000100oooo000@0?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool00P3oool00`000000oooo00000002000000030?ooo`00000000000080
oooo0P00000O0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`00
0000oooo00000002000000040?ooo`00000000000?ooo`<000007`3oool010000000oooo0?ooo`00
00020?ooo`040000003oool0oooo0?ooo`80000000D0oooo00000000000000000?ooo`0200000080
oooo00<000000?ooo`3oool07P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
0080oooo0P0000001@3oool00000000000000000oooo0080000000<0oooo000000000000403oool0
0100oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`030?ooo`@0000000D0oooo
000000000000oooo0000000Q0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
0P3oool00`000000oooo00000002000000050?ooo`00000000000?ooo`0000008@3oool010000000
oooo0?ooo`0000020?ooo`040000003oool0oooo0?ooo`80000000D0oooo00000000000000000?oo
o`0200000080oooo00<000000?ooo`3oool07P3oool010000000oooo0?ooo`0000020?ooo`050000
003oool0oooo0?ooo`0000000`3oool3000000040?ooo`00000000000?ooo`800000403oool000`0
oooo0P0000001P3oool00000000000000000oooo000000<0oooo0P0000020?ooo`030000003oool0
000000@0oooo00@000000?ooo`3oool00000703oool2000000060?ooo`00000000000000003oool0
00000`3oool200000080oooo00<000000?ooo`000000103oool010000000oooo0?ooo`00000L0?oo
o`80000000H0oooo00000000000000000?ooo`0000030?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`030000003oool00000008000006`3oool2000000060?ooo`00000000000000003oool0
00000`3oool200000080oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`00000@
0?ooo`00303oool3000003<0oooo0`00000c0?ooo`<00000<`3oool3000002P0oooo000=0?ooo`03
0000003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo02L0oooo000H0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01`0oooo000I0?ooo`030000003o
ool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo01/0oooo00020?ooo`030000003oool0oooo0180oooo00<000000?ooo`0000004`3oool2
000000@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
000001@0oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0180oooo00<000000?ooo`000000503oool00`000000oooo000000020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool04P3oool00`000000oooo0000000D0?ooo`030000003oool0
00000080oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3oool04P3oool30000
01<0oooo00D000000?ooo`000000oooo000000060?ooo`030000003oool0oooo0180oooo0`00000B
0?ooo`@0000000<0oooo0000003oool01@3oool00`000000oooo0?ooo`0B0?ooo`<000004P3oool4
000000030?ooo`000000oooo00D0oooo00<000000?ooo`3oool04P3oool300000180oooo1P000004
0?ooo`00103oool2000002P0oooo0P00000:0?ooo`8000009`3oool4000000T0oooo0P00000W0?oo
o`@000002@3oool2000002L0oooo100000060?ooo`001P3oool00`000000oooo0?ooo`0T0?ooo`<0
0000303oool00`000000oooo0?ooo`0T0?ooo`<0000000<0oooo0000003oool02@3oool00`000000
oooo0?ooo`0T0?ooo`<0000000<0oooo0000003oool02@3oool00`000000oooo0?ooo`0T0?ooo`<0
000000<0oooo0000003oool0103oool000L0oooo00<000000?ooo`3oool08P3oool2000000l0oooo
00<000000?ooo`3oool08P3oool2000000l0oooo00<000000?ooo`3oool08P3oool2000000l0oooo
00<000000?ooo`3oool08P3oool2000000T0oooo00080?ooo`8000007`3oool300000180oooo0P00
000P0?ooo`8000004P3oool200000200oooo0P00000B0?ooo`800000803oool2000000/0oooo0008
0?ooo`<000007P3oool2000001<0oooo0`00000O0?ooo`030000003oool0oooo0140oooo0`00000O
0?ooo`030000003oool0oooo0140oooo0`00000O0?ooo`030000003oool0oooo00X0oooo00020?oo
o`8000001`3oool00`000000oooo0?ooo`0I0?ooo`800000403oool2000000H0oooo00<000000?oo
o`3oool06@3oool200000100oooo0P0000060?ooo`030000003oool0oooo01T0oooo0P00000@0?oo
o`8000001P3oool00`000000oooo0?ooo`0I0?ooo`8000003P3oool00080oooo0`0000070?ooo`03
0000003oool0oooo01H0oooo0`00000A0?ooo`<000001P3oool00`000000oooo0?ooo`0F0?ooo`<0
00004@3oool3000000H0oooo00<000000?ooo`3oool05P3oool300000140oooo0`0000060?ooo`03
0000003oool0oooo01L0oooo0P00000?0?ooo`000`3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo0100oooo0P0000020?ooo`80000000<0oooo0000003oool04@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0100oooo0P0000020?ooo`D000004@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0100oooo0P0000020?ooo`D000004@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0100oooo0P0000020?ooo`D000003P3oool000h0oooo0P00
00080?ooo`030000003oool0oooo00<0000000<0oooo0000000000001@0000020?ooo`<000006@3o
ool2000000P0oooo00<000000?ooo`3oool00`0000000`3oool000000000000500000080oooo0`00
000I0?ooo`800000203oool00`000000oooo0?ooo`03000000030?ooo`000000000000D000000P3o
ool3000001T0oooo0P0000080?ooo`030000003oool0oooo00<0000000<0oooo0000000000001@00
00020?ooo`<00000303oool00100oooo00<000000?ooo`3oool00`0000000`3oool0000000000003
00000080oooo0P0000030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`03000000030?ooo`000000000000<000000P3oool2000000<0oooo00<00000
0?ooo`3oool0103oool2000001X0oooo00<000000?ooo`3oool00`0000000`3oool0000000000003
00000080oooo0P0000030?ooo`030000003oool0oooo00@0oooo0P00000J0?ooo`030000003oool0
oooo00<0000000<0oooo0000000000000`0000020?ooo`8000000`3oool00`000000oooo0?ooo`04
0?ooo`8000002`3oool00140oooo0`000000103oool0000000000000000:0?ooo`030000003oool0
oooo00D0oooo00D000000?ooo`3oool0oooo0000000H0?ooo`<0000000@0oooo0000000000000000
2`3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo000001P0oooo0`000000103oool0
000000000000000;0?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3oool00000603oool3
000000040?ooo`0000000000000000/0oooo00<000000?ooo`3oool01@3oool010000000oooo0?oo
o`0000080?ooo`00403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00D0oooo0P00
00020?ooo`<000005@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool00`00000E0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`03000001D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`050?ooo`80000000@0oooo00000000000000001P3oool000h0oooo0P00000L0?ooo`050000003o
ool000000?ooo`0000004`3oool2000001`0oooo00D000000?ooo`000000oooo0000000C0?ooo`80
0000703oool01@000000oooo0000003oool0000001<0oooo0P00000L0?ooo`050000003oool00000
0?ooo`0000001P3oool000d0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`040?oo
o`<0000000<0oooo0000003oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
00D0oooo0P0000000`3oool000000?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool01@3oool2000000030?ooo`000000oooo01<0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`050?ooo`80000000<0oooo0000003oool01`3oool000`0oooo00<000000?ooo`3o
ool05@3oool2000000@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo00<0oooo0P00000;0?ooo`002P3oool2000001P0oooo00<000000?ooo`000000
0`3oool00`000000oooo0?ooo`0C0?ooo`8000006P3oool01@000000oooo0?ooo`3oool0000001D0
oooo0P00000J0?ooo`050000003oool0oooo0?ooo`0000005@3oool2000001X0oooo00@000000?oo
o`3oool0oooo0P00000;0?ooo`002@3oool2000001`0oooo00@000000?ooo`000000oooo0P00000B
0?ooo`800000703oool010000000oooo0000003oool200000180oooo0P00000L0?ooo`040000003o
ool000000?ooo`8000004P3oool2000001`0oooo00<000000?ooo`0000000P3oool00`000000oooo
0?ooo`080?ooo`00203oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool08P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0oooo0007
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool01P3oool000H0oooo00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo0P00000;0?oo
o`030000003oool0oooo02H0oooo0P00000;0?ooo`030000003oool0oooo02H0oooo0P0000060?oo
o`00103oool2000001H0oooo00<000000?ooo`3oool04@3oool2000000P0oooo0P00000E0?ooo`03
0000003oool0oooo0180oooo0P0000080?ooo`800000503oool00`000000oooo0?ooo`0C0?ooo`80
0000203oool2000001@0oooo0P00000E0?ooo`030000003oool0oooo00<0oooo00030?ooo`030000
003oool0oooo01H0oooo0`00000C0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
5@3oool2000001D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`<00000
5@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01@0oooo00<000000?ooo`000000
5@3oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0000001@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0F0?ooo`80000000<0
oooo0000000000004`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01D0oooo00@0
00000?ooo`000000oooo0P00000C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
5@3oool200000080oooo0P00000C0?ooo`030000003oool0oooo0040oooo000L0?ooo`030000003o
ool0oooo0380oooo00<000000?ooo`3oool00P00000`0?ooo`040000003oool0oooo0?ooo`800000
<@3oool00`000000oooo0?ooo`02000001L0oooo001D0?ooo`800000=03oool2000003@0oooo0P00
000G0?ooo`009`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0=0?ooo`009`3oool3000003<0oooo0`00000c0?oo
o`<00000<`3oool3000000d0oooo000W0?ooo`030000003oool0000003<0oooo00<000000?ooo`00
0000<`3oool00`000000oooo0000000c0?ooo`030000003oool0000000d0oooo000W0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000
003oool0oooo00d0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
00090?ooo`8000001@3oool010000000oooo0000000000020?ooo`<0000000P0oooo000000000000
oooo0000003oool000000?ooo`8000000P3oool00`000000oooo0?ooo`0G0?ooo`8000001@3oool0
10000000oooo0000000000020?ooo`<000000P3oool01P000000oooo0000003oool000000?ooo`80
000000<0oooo0000000000006@3oool2000000D0oooo00@000000?ooo`00000000000P3oool30000
00080?ooo`00000000000?ooo`000000oooo0000003oool2000000030?ooo`00000000000200oooo
00@000000?ooo`00000000000P3oool300000080oooo00L000000?ooo`3oool000000?ooo`000000
oooo008000004`3oool000T0oooo0`0000040?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`3oool00P3oool5000000030?ooo`00000000000080oooo00<000000?ooo`3oool05`3oool3
000000@0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool000000080000000<0oooo0000000000000P3oool00`000000oooo0?ooo`0G0?ooo`<00000103o
ool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`000000
0P0000000`3oool00000000000020?ooo`030000003oool0oooo01h0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0000000@0oooo00000000
0000oooo0P00000@0?ooo`002P3oool00`000000oooo0?ooo`030?ooo`070000003oool0oooo0?oo
o`000000oooo000000030?ooo`030000003oool0oooo00<0000000<0oooo0000000000000P3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00<0oooo00L000000?ooo`3oool0oooo0000
003oool0000000@0oooo00<000000?ooo`0000000P0000000`3oool00000000000030?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool00`3oool01`000000oooo0?ooo`3oool000000?oo
o`0000001@3oool4000000030?ooo`00000000000080oooo00<000000?ooo`3oool07P3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`000000
103oool000000000003oool200000100oooo000?0?ooo`<0000000<0oooo0000003oool00P3oool2
00000080oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`0P0?ooo`<0000000<0oooo
0000003oool00P3oool2000000@0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00
000O0?ooo`<0000000<0oooo0000003oool00P3oool200000080oooo00<000000?ooo`000000103o
ool010000000oooo0?ooo`00000M0?ooo`D0000000<0oooo0000003oool00P3oool2000000D0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0A0?ooo`00[`3oool3000002L0oooo000C
0?ooo`800000V`3oool00`000000oooo0?ooo`0V0?ooo`004@3oool010000000oooo00000000000b
0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000>P3oool00`000000oooo0?ooo`0L
0?ooo`004@3oool00`000000oooo0?ooo`0b0?ooo`@00000<P3oool4000003/0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`070?ooo`000P3oool00`000000oooo0?ooo`0;0?ooo`<0
0000803oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo1000000O0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool4000001l0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0000001@0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0/0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool02P3oool01@000000oooo0000003oool0000001d0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0:0?ooo`050000003oool000000?oo
o`0000007@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo0`00000A0?oo
o`030000003oool0000000<0oooo00<000000?ooo`3oool00P3oool000@0oooo0P00000:0?ooo`80
00007`3oool00`000000oooo0?ooo`060?ooo`800000:`3oool00`000000oooo0?ooo`060?ooo`80
0000:`3oool00`000000oooo0?ooo`060?ooo`8000009`3oool5000000D0oooo00050?ooo`800000
1`3oool010000000oooo00000000000M0?ooo`8000002P3oool2000000L0oooo00<000000?ooo`00
00007P3oool2000000X0oooo0P0000080?ooo`030000003oool0000001d0oooo0P00000;0?ooo`03
0000003oool0oooo02D0oooo00<000000?ooo`3oool01P3oool000L0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
0`3oool4000001d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`@00000
703oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0280oooo0P0000090?ooo`00203o
ool2000000<0oooo0`00000M0?ooo`030000003oool0oooo00h0oooo0P0000030?ooo`@00000703o
ool00`000000oooo0?ooo`0>0?ooo`800000103oool4000001/0oooo00<000000?ooo`3oool03P3o
ool200000200oooo0P00000;0?ooo`002P3oool00`000000oooo0?ooo`0M0?ooo`<000004`3oool0
20000000oooo0?ooo`000000oooo0000003oool00000603oool3000001<0oooo00T000000?ooo`3o
ool0oooo0000003oool000000?ooo`0000005`3oool300000140oooo0`00000O0?ooo`030000003o
ool0oooo00X0oooo000;0?ooo`8000007@3oool2000001D0oooo00<000000?ooo`3oool0703oool2
000001D0oooo00<000000?ooo`3oool0703oool2000000d0oooo0P0000060?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool0303oool000X0oooo0P000000103oool0000000000000000I
0?ooo`030000003oool0oooo01@0oooo1P00000I0?ooo`030000003oool0oooo01@0oooo1P00000I
0?ooo`030000003oool0oooo00d0oooo0`0000060?ooo`030000003oool0oooo01L0oooo0P00000?
0?ooo`002`3oool00`000000oooo0?ooo`020?ooo`8000005P3oool00`000000oooo0?ooo`0>0?oo
o`8000001P3oool010000000oooo0?ooo`3oool4000001D0oooo00<000000?ooo`3oool03P3oool2
000000H0oooo00@000000?ooo`3oool0oooo1@00000D0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0@0?ooo`8000000P3oool5000000h0oooo0000
1@000000oooo0000003oool0000000D0oooo00<000000?ooo`3oool00`3oool500000180oooo00<0
00000?ooo`3oool0303oool5000000H0oooo00<000000?ooo`3oool00P3oool4000000030?ooo`00
000000000100oooo00<000000?ooo`3oool0303oool5000000H0oooo00<000000?ooo`3oool00P3o
ool4000000030?ooo`00000000000100oooo00<000000?ooo`3oool06P3oool2000000`0oooo0P00
00000`3oool000000000000500000080oooo1000000;0?ooo`001@0000050?ooo`8000002@3oool3
00000080oooo00D000000?ooo`3oool0oooo000000040?ooo`040000003oool00000000000l0oooo
0`0000080?ooo`030000003oool0oooo00D0oooo1@0000020?ooo`030000003oool0oooo0080oooo
00H000000?ooo`3oool0oooo0000003oool2000000l0oooo0`0000080?ooo`030000003oool0oooo
00D0oooo1@0000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool0oooo1000000O
0?ooo`030000003oool0oooo00<0000000<0oooo0000000000000`000000103oool0000000000000
000;0?ooo`030000003oool0oooo00T0oooo0005000000H0oooo00<000000?ooo`3oool02P3oool4
000000030?ooo`00000000000080000000@0oooo00000000000000004P3oool3000000L0oooo00<0
00000?ooo`3oool02@3oool5000000030?ooo`00000000000080000000@0oooo0000000000000000
4P3oool3000000L0oooo00<000000?ooo`3oool02@3oool5000000030?ooo`000000000000800000
00@0oooo00000000000000008P3oool3000000040?ooo`0000000000000000/0oooo00<000000?oo
o`3oool0103oool200000080oooo00<000000?ooo`3oool01P3oool000060?ooo`000000oooo0000
003oool000001@3oool00`000000oooo0?ooo`070?ooo`@000000`3oool010000000oooo0?ooo`3o
ool200000080oooo0P0000090?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3o
ool4000000<0oooo0P0000020?ooo`8000000P3oool2000000T0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`070?ooo`@000000`3oool2000000<0oooo0P0000000`3oool000000000
00090?ooo`030000003oool0oooo01@0oooo00<000000?ooo`000000403oool00`000000oooo0?oo
o`050?ooo`80000000@0oooo00000000000000001P3oool000X0oooo00<000000?ooo`3oool01@3o
ool300000140oooo0P0000070?ooo`<000003`3oool00`000000oooo0?ooo`030?ooo`D00000403o
ool2000000L0oooo0`00000?0?ooo`030000003oool0oooo00<0oooo1@00000@0?ooo`8000001`3o
ool300000180oooo0P00000L0?ooo`050000003oool000000?ooo`0000001P3oool000X0oooo0P00
00040?ooo`<000005@3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000l0oooo00<0
00000?ooo`0000001@00000E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000003`3o
ool00`000000oooo0?ooo`06000001@0oooo00<000000?ooo`3oool0103oool00`000000oooo0000
000A0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool01@3oool2000000030?ooo`00
0000oooo00L0oooo000;0?ooo`030000003oool00000008000006@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00l0oooo00@000000?ooo`000000oooo0P00000H0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool03`3oool010000000oooo0000003oool2000001P0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0oooo
0P0000040?ooo`8000002`3oool000/0oooo0P00000M0?ooo`030000003oool0oooo01@0oooo0`00
000L0?ooo`030000003oool0oooo01@0oooo0`00000L0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo000000030?ooo`8000002`3oool000X0oooo00<00000
0?ooo`3oool07P3oool2000001<0oooo00<000000?ooo`3oool07P3oool300000180oooo00<00000
0?ooo`3oool07P3oool300000140oooo0P00000L0?ooo`060000003oool000000000003oool00000
2P3oool000P0oooo0P00000R0?ooo`800000403oool2000001<0oooo00<000000?ooo`3oool0303o
ool200000100oooo0P00000R0?ooo`800000403oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo00L0oooo00070?ooo`030000003oool0oooo01<0oooo0P00000?0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool04`3oool2000000l0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00H0oooo00060?ooo`8000005P3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00X0oooo0P00000D0?ooo`030000003oool00000
0100oooo00<000000?ooo`3oool02P3oool2000001H0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo0P0000060?ooo`00103oool2000001P0
oooo00<000000?ooo`3oool03`3oool2000000P0oooo0P00000G0?ooo`<00000403oool2000000P0
oooo0P00000H0?ooo`030000003oool0oooo00l0oooo0P0000080?ooo`800000:`3oool00`000000
oooo0?ooo`030?ooo`000`3oool00`000000oooo0?ooo`0G0?ooo`<000004P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0G0?ooo`<000004P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?oo
o`0^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0G0?ooo`<00000503oool00`000000oooo0?ooo`010?ooo`00903oool00`000000oooo0?ooo`0c
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0000000H
0?ooo`00903oool3000003<0oooo0`00000c0?ooo`<00000:`3oool00`000000oooo0?ooo`0H0?oo
o`00903oool00`000000oooo0000000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`00
0000AP3oool002@0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000
003oool0oooo03H0oooo00<000000?ooo`3oool03@3oool00<T0oooo0`00000=0?ooo`00b@3oool0
0`000000oooo0000000=0?ooo`00b@3oool00`000000oooo0?ooo`0=0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0oooo0`00000;0?ooo`03
0000003oool0oooo00@0oooo00@000000?ooo`3oool000001`3oool010000000oooo0?ooo`00001F
0?ooo`00E03oool3000000P0oooo0`0000080?ooo`D000001@3oool010000000oooo0?ooo`000007
0?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3000000/0oooo00<00000
0?ooo`3oool0103oool4000000L0oooo1000001F0?ooo`00M03oool010000000oooo0?ooo`000007
0?ooo`040000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}, {{0, 54.0625}, {286.75, 232.688}}
   -> {-1.00011, -1.10185, 0.406941, 0.406941}, {{54.0625, 108.125}, {286.75, 
  232.688}} -> {-23.0004, -1.10185, 0.406941, 0.406941}, {{108.125, 162.188},
   {286.75, 232.688}} -> {-45.0006, -1.10185, 0.406941, 0.406941}, {{162.188, 
  216.313}, {286.75, 232.688}} -> {-67.0136, -1.10185, 0.406941, 0.406941}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
gsave
.79753 .54101 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66482 .53933 L
.66581 .53377 L
.66605 .52895 L
.66511 .52489 L
.66278 .52142 L
.6591 .51829 L
.65439 .51517 L
.64918 .5118 L
.64413 .50797 L
.63994 .50362 L
.63719 .49881 L
.63625 .49374 L
.63724 .48864 L
.63995 .48375 L
.64391 .47925 L
.64844 .47517 L
.65276 .47142 L
.65619 .46777 L
.65829 .46391 L
.65905 .45962 L
.65898 .45489 L
.65897 .44997 L
.65975 .44525 L
.66152 .44096 L
.66398 .43706 L
.66675 .43337 L
.66966 .4298 L
.67273 .42636 L
.67595 .42307 L
.67931 .41991 L
.68279 .4169 L
.68652 .41418 L
.69067 .41212 L
.69533 .41101 L
.70024 .41069 L
.70496 .41042 L
.70919 .40937 L
.71289 .40701 L
.71631 .40334 L
.71975 .39877 L
.7235 .39397 L
.72772 .38971 L
.73241 .38667 L
.73743 .38533 L
.74256 .38592 L
.74754 .38834 L
.75217 .39222 L
.75634 .39698 L
.76006 .40195 L
Mistroke
.76349 .40643 L
.76687 .40989 L
.77049 .41198 L
.77461 .41264 L
.7794 .41206 L
.78488 .41069 L
.79091 .40909 L
Mfstroke
.69134 .40643 m
.64083 .4339 L
.66741 .45871 L
F
gsave
.64871 .40578 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N101) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
gsave
.79753 .54101 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66482 .53933 L
.66581 .53377 L
.66605 .52895 L
.66511 .52489 L
.66278 .52142 L
.6591 .51829 L
.65439 .51517 L
.64918 .5118 L
.64413 .50797 L
.63994 .50362 L
.63719 .49881 L
.63625 .49374 L
.63724 .48864 L
.63995 .48375 L
.64391 .47925 L
.64844 .47517 L
.65276 .47142 L
.65619 .46777 L
.65829 .46391 L
.65905 .45962 L
.65898 .45489 L
.65897 .44997 L
.65975 .44525 L
.66152 .44096 L
.66398 .43706 L
.66675 .43337 L
.66966 .4298 L
.67273 .42636 L
.67595 .42307 L
.67931 .41991 L
.68279 .4169 L
.68652 .41418 L
.69067 .41212 L
.69533 .41101 L
.70024 .41069 L
.70496 .41042 L
.70919 .40937 L
.71289 .40701 L
.71631 .40334 L
.71975 .39877 L
.7235 .39397 L
.72772 .38971 L
.73241 .38667 L
.73743 .38533 L
.74256 .38592 L
.74754 .38834 L
.75217 .39222 L
.75634 .39698 L
.76006 .40195 L
Mistroke
.76349 .40643 L
.76687 .40989 L
.77049 .41198 L
.77461 .41264 L
.7794 .41206 L
.78488 .41069 L
.79091 .40909 L
Mfstroke
.69134 .40643 m
.64083 .4339 L
.66741 .45871 L
F
gsave
.64871 .40578 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N102) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.66364 .54545 L
s
.78596 .62191 m
.82258 .66624 L
.84185 .6354 L
F
gsave
.79924 .66862 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .40909 L
s
.89368 .32345 m
.86342 .37234 L
.84014 .3444 L
F
gsave
.89034 .36754 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.65855 .54895 L
.65354 .55189 L
.64867 .55378 L
.64398 .55429 L
.63949 .55326 L
.63519 .55075 L
.63106 .54703 L
.62703 .54252 L
.62303 .53775 L
.61899 .5333 L
.61485 .5297 L
.61053 .52735 L
.60602 .52649 L
.60131 .52716 L
.59641 .52919 L
.59139 .53222 L
.5863 .53574 L
.58122 .5392 L
.57623 .54205 L
.57137 .5438 L
.5667 .54414 L
.56224 .54294 L
.55796 .54028 L
.55384 .53645 L
.54982 .53188 L
.54582 .52712 L
.54177 .52274 L
.53761 .51926 L
.53327 .51707 L
.52874 .51638 L
.524 .51722 L
.51909 .51938 L
.51406 .52249 L
.50897 .52603 L
.5039 .52945 L
.49891 .5322 L
.49408 .5338 L
.48943 .53398 L
.48498 .53261 L
.48073 .5298 L
.47662 .52586 L
.47261 .52125 L
.46861 .51651 L
.46455 .5122 L
.46037 .50885 L
.45601 .50681 L
.45145 .5063 L
.4467 .5073 L
.44177 .50959 L
Mistroke
.43673 .51277 L
.43164 .51633 L
.42657 .5197 L
.4216 .52234 L
.41678 .5238 L
.41216 .5238 L
.40774 .52227 L
.4035 .51932 L
.39941 .51528 L
.39539 .51062 L
.39139 .5059 L
.38732 .50168 L
.38312 .49845 L
.37874 .49659 L
.37416 .49625 L
.36939 .4974 L
.36445 .49981 L
.3594 .50306 L
.3543 .50663 L
.34924 .50995 L
.34429 .51247 L
.33949 .51378 L
.33489 .51362 L
.33049 .51191 L
.32627 .50883 L
.32219 .50469 L
.31818 .5 L
Mfstroke
gsave
.5001 .48037 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.71193 .46295 .0956 325.705 480.345 arc
s
gsave
.79753 .54101 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.66364 .54545 m
.66482 .53933 L
.66581 .53377 L
.66605 .52895 L
.66511 .52489 L
.66278 .52142 L
.6591 .51829 L
.65439 .51517 L
.64918 .5118 L
.64413 .50797 L
.63994 .50362 L
.63719 .49881 L
.63625 .49374 L
.63724 .48864 L
.63995 .48375 L
.64391 .47925 L
.64844 .47517 L
.65276 .47142 L
.65619 .46777 L
.65829 .46391 L
.65905 .45962 L
.65898 .45489 L
.65897 .44997 L
.65975 .44525 L
.66152 .44096 L
.66398 .43706 L
.66675 .43337 L
.66966 .4298 L
.67273 .42636 L
.67595 .42307 L
.67931 .41991 L
.68279 .4169 L
.68652 .41418 L
.69067 .41212 L
.69533 .41101 L
.70024 .41069 L
.70496 .41042 L
.70919 .40937 L
.71289 .40701 L
.71631 .40334 L
.71975 .39877 L
.7235 .39397 L
.72772 .38971 L
.73241 .38667 L
.73743 .38533 L
.74256 .38592 L
.74754 .38834 L
.75217 .39222 L
.75634 .39698 L
.76006 .40195 L
Mistroke
.76349 .40643 L
.76687 .40989 L
.77049 .41198 L
.77461 .41264 L
.7794 .41206 L
.78488 .41069 L
.79091 .40909 L
Mfstroke
.69134 .40643 m
.64083 .4339 L
.66741 .45871 L
F
gsave
.64871 .40578 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.79091 .40909 Mdot
.66364 .54545 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N103) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N104) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N105) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N106) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N107) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N108) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N109) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(G) show
67.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N110) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C8) show
77.812500 14.625000 moveto
(N111) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.500000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C9) show
77.812500 14.625000 moveto
(N112) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(u) show
63.375000 10.312500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 9.625000 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C10) show
80.562500 14.625000 moveto
(N113) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
newpath
.66364 .52864 .13045 192.68 347.32 arc
s
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
newpath
.66364 .47136 .13045 12.6804 167.32 arc
s
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -65.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(d) show
63.375000 9.125000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 15.937500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
66.312500 8.437500 moveto
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -78 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C11) show
80.562500 14.625000 moveto
(N114) show
92.375000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.79091 .5 m
.78588 .49659 L
.7813 .49334 L
.7776 .49005 L
.77508 .48649 L
.77385 .48248 L
.7738 .4779 L
.77465 .47271 L
.77599 .467 L
.77733 .46098 L
.77817 .45493 L
.77809 .44917 L
.7768 .44404 L
.77416 .43981 L
.77022 .43665 L
.76518 .43459 L
.75938 .43351 L
.75322 .43317 L
.74712 .43318 L
.74143 .43312 L
.73644 .43257 L
.73226 .43116 L
.72889 .42868 L
.7262 .42504 L
.72394 .42037 L
.72183 .41497 L
.71957 .40925 L
.71691 .40372 L
.71366 .3989 L
.70977 .39525 L
.70527 .39307 L
.70027 .3925 L
.69497 .39349 L
.68957 .39576 L
.68427 .39889 L
.67921 .40234 L
.67445 .40556 L
.66998 .40803 L
.66573 .40937 L
.66155 .40937 L
.65729 .40803 L
.65282 .40556 L
.64806 .40234 L
.643 .39889 L
.6377 .39576 L
.6323 .39349 L
.627 .3925 L
.62201 .39307 L
.6175 .39525 L
.61361 .3989 L
Mistroke
.61037 .40372 L
.6077 .40925 L
.60544 .41497 L
.60333 .42037 L
.60107 .42504 L
.59838 .42868 L
.59501 .43116 L
.59084 .43257 L
.58584 .43312 L
.58016 .43318 L
.57405 .43317 L
.56789 .43351 L
.56209 .43459 L
.55705 .43665 L
.55311 .43981 L
.55047 .44404 L
.54918 .44917 L
.5491 .45493 L
.54994 .46098 L
.55128 .467 L
.55262 .47271 L
.55347 .4779 L
.55342 .48248 L
.55219 .48649 L
.54967 .49005 L
.54598 .49334 L
.54139 .49659 L
.53636 .5 L
Mfstroke
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .5 m
.78589 .50341 L
.78131 .50666 L
.77762 .50996 L
.77511 .51352 L
.77388 .51753 L
.77383 .52212 L
.77467 .5273 L
.776 .533 L
.77732 .53901 L
.77814 .54506 L
.77805 .5508 L
.77674 .55591 L
.77409 .56013 L
.77014 .56329 L
.76511 .56535 L
.75934 .56644 L
.75321 .56683 L
.74716 .56688 L
.74154 .56702 L
.73661 .56767 L
.7325 .56918 L
.72917 .57176 L
.72649 .57544 L
.72418 .58006 L
.72196 .58527 L
.7195 .59059 L
.71655 .59544 L
.71295 .59926 L
.7087 .60165 L
.7039 .60249 L
.69881 .60208 L
.69372 .60111 L
.68882 .60038 L
.68414 .60029 L
.67958 .60071 L
.67505 .60124 L
.67049 .60162 L
.66592 .6018 L
.66135 .6018 L
.65678 .60162 L
.65223 .60124 L
.64769 .60071 L
.64313 .60029 L
.63845 .60038 L
.63355 .60111 L
.62846 .60208 L
.62337 .60249 L
.61858 .60165 L
.61432 .59926 L
Mistroke
.61073 .59544 L
.60778 .59059 L
.60532 .58527 L
.60309 .58006 L
.60079 .57544 L
.5981 .57176 L
.59477 .56918 L
.59066 .56767 L
.58573 .56702 L
.58011 .56688 L
.57406 .56683 L
.56794 .56644 L
.56216 .56535 L
.55713 .56329 L
.55319 .56013 L
.55053 .55591 L
.54922 .5508 L
.54913 .54506 L
.54995 .53901 L
.55127 .533 L
.5526 .5273 L
.55344 .52212 L
.55339 .51753 L
.55216 .51352 L
.54965 .50996 L
.54596 .50666 L
.54138 .50341 L
.53636 .5 L
Mfstroke
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N115) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .5 L
s
.16087 .6311 m
.19113 .58221 L
.21441 .61014 L
F
gsave
.16421 .587 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .5 L
s
.20277 .40382 m
.17251 .35494 L
.14923 .38287 L
F
gsave
.19943 .35973 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.79091 .5 L
s
.85679 .5915 m
.87391 .64639 L
.90342 .62515 L
F
gsave
.84304 .62731 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.79091 .5 L
s
.88866 .36423 m
.87155 .41912 L
.84204 .39787 L
F
gsave
.90241 .40004 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.53636 .5 m
.53182 .50365 L
.52727 .50692 L
.52273 .50945 L
.51818 .51097 L
.51364 .51134 L
.50909 .5105 L
.50454 .50854 L
.5 .50568 L
.49545 .50221 L
.49091 .49851 L
.48636 .49497 L
.48182 .49196 L
.47727 .4898 L
.47273 .48873 L
.46818 .48885 L
.46364 .49015 L
.45909 .4925 L
.45455 .49565 L
.45 .49925 L
.44545 .50293 L
.44091 .50631 L
.43636 .50901 L
.43182 .51075 L
.42727 .51136 L
.42273 .51075 L
.41818 .50901 L
.41364 .50631 L
.40909 .50293 L
.40455 .49925 L
.4 .49565 L
.39545 .4925 L
.39091 .49015 L
.38636 .48885 L
.38182 .48873 L
.37727 .4898 L
.37273 .49196 L
.36818 .49497 L
.36364 .49851 L
.35909 .50221 L
.35455 .50568 L
.35 .50854 L
.34546 .5105 L
.34091 .51134 L
.33636 .51097 L
.33182 .50945 L
.32727 .50692 L
.32273 .50365 L
.31818 .5 L
s
gsave
.42727 .45704 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .5 m
.78588 .49659 L
.7813 .49334 L
.7776 .49005 L
.77508 .48649 L
.77385 .48248 L
.7738 .4779 L
.77465 .47271 L
.77599 .467 L
.77733 .46098 L
.77817 .45493 L
.77809 .44917 L
.7768 .44404 L
.77416 .43981 L
.77022 .43665 L
.76518 .43459 L
.75938 .43351 L
.75322 .43317 L
.74712 .43318 L
.74143 .43312 L
.73644 .43257 L
.73226 .43116 L
.72889 .42868 L
.7262 .42504 L
.72394 .42037 L
.72183 .41497 L
.71957 .40925 L
.71691 .40372 L
.71366 .3989 L
.70977 .39525 L
.70527 .39307 L
.70027 .3925 L
.69497 .39349 L
.68957 .39576 L
.68427 .39889 L
.67921 .40234 L
.67445 .40556 L
.66998 .40803 L
.66573 .40937 L
.66155 .40937 L
.65729 .40803 L
.65282 .40556 L
.64806 .40234 L
.643 .39889 L
.6377 .39576 L
.6323 .39349 L
.627 .3925 L
.62201 .39307 L
.6175 .39525 L
.61361 .3989 L
Mistroke
.61037 .40372 L
.6077 .40925 L
.60544 .41497 L
.60333 .42037 L
.60107 .42504 L
.59838 .42868 L
.59501 .43116 L
.59084 .43257 L
.58584 .43312 L
.58016 .43318 L
.57405 .43317 L
.56789 .43351 L
.56209 .43459 L
.55705 .43665 L
.55311 .43981 L
.55047 .44404 L
.54918 .44917 L
.5491 .45493 L
.54994 .46098 L
.55128 .467 L
.55262 .47271 L
.55347 .4779 L
.55342 .48248 L
.55219 .48649 L
.54967 .49005 L
.54598 .49334 L
.54139 .49659 L
.53636 .5 L
Mfstroke
.63636 .39818 m
.69091 .38 L
.69091 .41636 L
F
gsave
.66364 .35522 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.79091 .5 m
.78589 .50341 L
.78131 .50666 L
.77762 .50996 L
.77511 .51352 L
.77388 .51753 L
.77383 .52212 L
.77467 .5273 L
.776 .533 L
.77732 .53901 L
.77814 .54506 L
.77805 .5508 L
.77674 .55591 L
.77409 .56013 L
.77014 .56329 L
.76511 .56535 L
.75934 .56644 L
.75321 .56683 L
.74716 .56688 L
.74154 .56702 L
.73661 .56767 L
.7325 .56918 L
.72917 .57176 L
.72649 .57544 L
.72418 .58006 L
.72196 .58527 L
.7195 .59059 L
.71655 .59544 L
.71295 .59926 L
.7087 .60165 L
.7039 .60249 L
.69881 .60208 L
.69372 .60111 L
.68882 .60038 L
.68414 .60029 L
.67958 .60071 L
.67505 .60124 L
.67049 .60162 L
.66592 .6018 L
.66135 .6018 L
.65678 .60162 L
.65223 .60124 L
.64769 .60071 L
.64313 .60029 L
.63845 .60038 L
.63355 .60111 L
.62846 .60208 L
.62337 .60249 L
.61858 .60165 L
.61432 .59926 L
Mistroke
.61073 .59544 L
.60778 .59059 L
.60532 .58527 L
.60309 .58006 L
.60079 .57544 L
.5981 .57176 L
.59477 .56918 L
.59066 .56767 L
.58573 .56702 L
.58011 .56688 L
.57406 .56683 L
.56794 .56644 L
.56216 .56535 L
.55713 .56329 L
.55319 .56013 L
.55053 .55591 L
.54922 .5508 L
.54913 .54506 L
.54995 .53901 L
.55127 .533 L
.5526 .5273 L
.55344 .52212 L
.55339 .51753 L
.55216 .51352 L
.54965 .50996 L
.54596 .50666 L
.54138 .50341 L
.53636 .5 L
Mfstroke
.69091 .60182 m
.63636 .62 L
.63636 .58364 L
F
gsave
.66364 .64478 -66 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .5 Mdot
.53636 .5 Mdot
.79091 .5 Mdot
gsave
.5 .02273 -76.5 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T5) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N116) show
89.625000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{217.313, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003I000182000`400?l00000o`00003of@3oool0
0=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
00d0oooo00@000000?ooo`00000000000P3oool300000080oooo00P000000?ooo`00000000000?oo
o`000000oooo00000080oooo00D000000?ooo`3oool000000?ooo`02000001X0oooo00@000000?oo
o`00000000000P3oool300000080oooo00P000000?ooo`3oool000000?ooo`000000oooo00000080
oooo00@000000?ooo`3oool000007P3oool010000000oooo0000000000020?ooo`<000000P3oool0
1P000000oooo0?ooo`000000oooo00000080oooo00D000000?ooo`3oool000000?ooo`02000001`0
oooo00@000000?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo00<000000?oo
o`0000000P3oool01@000000oooo0?ooo`000000oooo008000003`3oool000d0oooo00<000000?oo
o`3oool00P0000000`3oool000000?ooo`030?ooo`040000003oool000000?ooo`@000000P3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool00P00
00000`3oool000000?ooo`030?ooo`050000003oool0oooo0000003oool00`0000020?ooo`050000
003oool0oooo0000003oool00P00000K0?ooo`030000003oool0oooo0080000000<0oooo0000003o
ool00`3oool00`000000oooo0?ooo`0300000080oooo00@000000?ooo`3oool000000P3oool20000
01/0oooo00<000000?ooo`3oool00P0000000`3oool000000?ooo`030?ooo`030000003oool0oooo
00<000000P3oool01@000000oooo0?ooo`000000oooo00<000003P3oool000d0oooo00<000000?oo
o`0000000`3oool00`000000oooo0?ooo`020?ooo`050000003oool000000000003oool00`000002
0?ooo`040000003oool0000000000080oooo00<000000?ooo`3oool0603oool00`000000oooo0000
00030?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool000000?ooo`0300000080oooo
00D000000?ooo`00000000000?ooo`02000001/0oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00<000000P3oool01P000000oooo0?ooo`000000oooo
000001d0oooo00<000000?ooo`0000000`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
0000008000000P3oool01P000000oooo0?ooo`000000oooo00000100oooo000<0?ooo`<0000000<0
oooo0000003oool00P3oool2000000D0oooo00<000000?ooo`0000000P3oool200000080oooo00D0
00000?ooo`3oool0oooo0000000I0?ooo`<0000000<0oooo0000003oool00P3oool2000000L0oooo
00<000000?ooo`3oool00P0000020?ooo`030000003oool0oooo01`0oooo0`0000000`3oool00000
0?ooo`020?ooo`8000001@3oool00`000000oooo0?ooo`030?ooo`030000003oool000000080oooo
00<000000?ooo`3oool06@3oool3000000030?ooo`000000oooo0080oooo0P0000020?ooo`040000
003oool0oooo000000D0oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0=0?ooo`00
303oool200000180oooo0P0000020?ooo`030000003oool0oooo01/0oooo0P00000B0?ooo`800000
0P3oool00`000000oooo0?ooo`0K0?ooo`8000004`3oool4000001h0oooo0P00000B0?ooo`@00000
4P3oool000`0oooo0`00000A0?ooo`030000003oool0oooo01l0oooo0`00000B0?ooo`030000003o
ool0oooo01h0oooo0`00000B0?ooo`@000007P3oool300000140oooo1000000B0?ooo`003@3oool0
0`000000oooo0?ooo`0c0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool04@3oool0
1@000000oooo0000003oool0000001h0oooo00<000000?ooo`3oool0403oool01@000000oooo0000
003oool000000140oooo000Q0?ooo`800000]P3oool00240oooo0P0000000`3oool000000000000a
0?ooo`800000:03oool00`000000oooo0?ooo`1E0?ooo`000P3oool00`000000oooo0?ooo`0@0?oo
o`8000003@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0403oool2000000X0oooo0P00000@0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0403oool2000001`0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0;0?ooo`800000303oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`0000006`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`020?ooo`00
103oool200000100oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`060?ooo`800000
403oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
1P3oool2000000l0oooo0`00000I0?ooo`030000003oool0oooo00H0oooo0P00000@0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool00`3oool000H0oooo00<000000?ooo`3oool0303o
ool3000001P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`<00000603o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0<0?ooo`<00000603oool00`000000oooo0?ooo`040?ooo`001`3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00@0oooo00080?ooo`8000009@3oool00`00
0000oooo0?ooo`0<0?ooo`8000009@3oool00`000000oooo0?ooo`0<0?ooo`8000009@3oool00`00
0000oooo0?ooo`0<0?ooo`8000009P3oool00`000000oooo0?ooo`040?ooo`00203oool3000001L0
oooo0`0000080?ooo`<000003P3oool3000001L0oooo0`0000080?ooo`<000003P3oool3000001H0
oooo1P0000060?ooo`<000003P3oool3000001H0oooo1P0000060?ooo`<000001`3oool00080oooo
0P0000070?ooo`030000003oool0oooo0180oooo0`000000103oool000000000000000060?ooo`03
0000003oool0oooo00T0oooo0P0000060?ooo`030000003oool0oooo01<0oooo0P0000001@3oool0
00000000003oool0000000D0oooo00<000000?ooo`3oool02@3oool2000000H0oooo00<000000?oo
o`3oool04P3oool2000000040?ooo`00000000000?ooo`8000001P3oool00`000000oooo0?ooo`08
0?ooo`8000001P3oool00`000000oooo0?ooo`0B0?ooo`80000000@0oooo000000000000oooo0P00
00060?ooo`030000003oool0oooo00H0oooo00020?ooo`<000001`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00D0oooo0P0000030?ooo`030000003oool0oooo00X0oooo0`000006
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00T0oooo0`0000060?ooo`030000003oool0oooo00l0oooo0`000006
0?ooo`<000000`3oool00`000000oooo0?ooo`090?ooo`<000001P3oool00`000000oooo0?ooo`0?
0?ooo`<000001P3oool3000000<0oooo00<000000?ooo`3oool01`3oool000<0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo00@000000?oo
o`3oool000003@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0203oool01@000000oooo0?ooo`3oool0000000`0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0=0?ooo`8000002P3oool200000080oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00d0oooo0P00000;0?ooo`040000003o
ool0oooo000000T0oooo000>0?ooo`8000003@3oool00`000000oooo0?ooo`090?ooo`040000003o
ool0oooo000001L0oooo0P00000=0?ooo`030000003oool0oooo00T0oooo00@000000?ooo`3oool0
00005`3oool2000000d0oooo0P00000;0?ooo`030000003oool0000001L0oooo0P00000=0?ooo`80
00002`3oool00`000000oooo0000000:0?ooo`00403oool010000000oooo0?ooo`3oool200000080
oooo0`0000000`3oool000000?ooo`0<0?ooo`8000006P3oool010000000oooo0?ooo`3oool20000
0080oooo0`0000020?ooo`030000003oool0oooo00X0oooo0P00000J0?ooo`040000003oool0oooo
0?ooo`8000000P3oool300000080oooo0P00000:0?ooo`<000006P3oool010000000oooo0?ooo`3o
ool2000000<0oooo0P0000020?ooo`8000002`3oool2000000/0oooo000A0?ooo`<0000000<0oooo
0000000000000P000000103oool0000000000000000=0?ooo`8000001P3oool00`000000oooo0?oo
o`0B0?ooo`<0000000<0oooo0000000000000P0000000`3oool0000000000002000000`0oooo0P00
00060?ooo`030000003oool0oooo0140oooo2@0000000`3oool0000000000002000000`0oooo0P00
00060?ooo`030000003oool0oooo0140oooo2@0000000`3oool0000000000002000000`0oooo0P00
00070?ooo`030000003oool0oooo0040oooo000@0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02P3oool2000000H0oooo0`00000A0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02P3oool2000000H0oooo0`00000A0?ooo`030000003oool0oooo00X0oooo0P00000:
0?ooo`<000001P3oool300000140oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`09
0?ooo`8000001`3oool300000040oooo000>0?ooo`8000003@3oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`0000003`3oool2000000d0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`030?ooo`030000003oool0000000l0oooo0P00000=0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`0000001@3oool00`000000oooo0000000?0?oo
o`8000003@3oool2000000/0oooo00<000000?ooo`0000001P3oool00`000000oooo000000010?oo
o`003@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo00@000000?ooo`3o
ool000001@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0203oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0=0?ooo`<00000203oool300000080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo0`0000080?ooo`<000000P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0040oooo000<0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0103oool2000000D0oooo00<000000?ooo`3oool01P3oool000X0
oooo0P00000D0?ooo`030000003oool0000000@000006@3oool2000001@0oooo00<000000?ooo`00
00001000000I0?ooo`800000503oool8000001P0oooo0P00000E0?ooo`L000003`3oool000T0oooo
0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000S0?ooo`030000003oool0oooo00h0oooo
0P00000S0?ooo`030000003oool0oooo00h0oooo0P00000T0?ooo`030000003oool0oooo00D0oooo
00080?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo02@0oooo0P0000060?ooo`001`3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo00@0oooo00060?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0103oool000@0oooo0P00000[0?ooo`030000003oool0oooo00H0oooo0P00000[0?ooo`03
0000003oool0oooo00H0oooo0P00000[0?ooo`030000003oool0oooo00H0oooo0P00000[0?ooo`03
0000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0
0P3oool00080oooo00<000000?ooo`3oool07`3oool00`000000oooo0000000<0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool07`3oool00`000000oooo0000000<0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo0040oooo000Q0?ooo`80000000<0oooo0000003oool00`00000^0?ooo`80000000<0oooo
0000003oool00`00000_0?ooo`040000003oool000000?ooo`<00000;`3oool010000000oooo0000
003oool3000000h0oooo000Q0?ooo`80000000H0oooo0000003oool000000?ooo`00000^0?ooo`80
000000H0oooo0000003oool000000?ooo`00000^0?ooo`@0000000@0oooo0000003oool00000;P3o
ool4000000040?ooo`000000oooo000000h0oooo000Q0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0;`3oool01@000000oooo0?ooo`3oool000000300oooo100000000`3oool00000
0?ooo`0_0?ooo`@0000000<0oooo0000003oool03`3oool00240oooo0P00001Z0?ooo`050000003o
ool000000?ooo`000000<@3oool01@000000oooo0000003oool000000140oooo000Q0?ooo`800000
]P3oool002@0oooo00<000000?ooo`3oool0<03oool200000800oooo000T0?ooo`030000003oool0
oooo0300oooo0P0000000`3oool000000?ooo`1m0?ooo`00903oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo07`0oooo001J0?ooo`030000003oool0oooo07`0oooo003I0?ooo`00f@3o
ool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00
f@3oool000h0oooo00@000000?ooo`00000000000P3oool3000000030?ooo`00000000000080oooo
00<000000?ooo`0000000P3oool01@000000oooo000000000000oooo00800000703oool010000000
oooo0000000000020?ooo`<000001@3oool00`000000oooo000000020?ooo`050000003oool0oooo
0000003oool00P00000L0?ooo`040000003oool0000000000080oooo0`0000000`3oool000000000
00020?ooo`030000003oool000000080oooo00@000000?ooo`3oool000000P3oool00`000000oooo
0?ooo`0J0?ooo`040000003oool0000000000080oooo0`0000000`3oool00000000000020?ooo`03
0000003oool000000080oooo00D000000?ooo`3oool000000?ooo`02000000l0oooo000>0?ooo`03
0000003oool0oooo0080000000<0oooo0000003oool00P3oool3000000040?ooo`00000000000000
0080oooo00H000000?ooo`000000oooo0000003oool2000001/0oooo00<000000?ooo`3oool00P00
00000`3oool000000?ooo`030?ooo`030000003oool0oooo00<000000P3oool020000000oooo0?oo
o`000000oooo0000003oool000006`3oool00`000000oooo0?ooo`02000000030?ooo`000000oooo
0080oooo0P0000020?ooo`<000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool00P0000000`3oool000000?ooo`030?ooo`80000000@0oooo
00000000000000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00d0oooo
000>0?ooo`030000003oool0000000<0oooo00D000000?ooo`3oool0oooo000000030?ooo`<00000
0P3oool020000000oooo000000000000oooo0000003oool000006`3oool00`000000oooo00000003
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000020?ooo`050000003oool0
oooo0000003oool00P00000L0?ooo`030000003oool0000000<0oooo00@000000?ooo`3oool0oooo
0P0000020?ooo`<000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`0000000`3oool020000000oooo0?ooo`3oool000000?ooo`000000oooo0`00
00020?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3oool0303oool000d0oooo0`00
00000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo00000080oooo00<000000?ooo`3oool06@3oool3000000030?ooo`000000oooo0080
oooo0P0000001P3oool00000000000000000oooo000000@0oooo100000020?ooo`030000003oool0
oooo01T0oooo0`0000000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`00
00050?ooo`030000003oool0000001h0oooo0`0000000`3oool000000?ooo`020?ooo`8000000P3o
ool010000000oooo0?ooo`0000040?ooo`80000000D0oooo0000003oool0oooo0000000?0?ooo`00
303oool2000001<0oooo0`00000N0?ooo`8000004`3oool010000000oooo00000000000M0?ooo`80
0000503oool00`000000oooo0000000N0?ooo`8000004P3oool400000180oooo000<0?ooo`<00000
4P3oool00`000000oooo0000000N0?ooo`<000004P3oool00`000000oooo0000000N0?ooo`<00000
4P3oool4000001h0oooo0`00000B0?ooo`030000003oool0oooo0180oooo000=0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0B0?ooo`8000007`3o
ool00`000000oooo0?ooo`0A0?ooo`8000008@3oool00`000000oooo0?ooo`0V0?ooo`00S@3oool2
000003@0oooo0P00000D0?ooo`00``3oool2000001@0oooo00020?ooo`030000003oool0oooo0100
oooo0P00000L0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool2000001`0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`800000703oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo0100oooo0P00000=0?ooo`8000003@3oool00`000000
oooo0?ooo`010?ooo`000`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`020?ooo`00103oool200000100oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`060?ooo`800000403oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00H0oooo0P00000@0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool01P3oool200000100oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`030?ooo`001P3oool00`000000oooo0?ooo`0<0?ooo`<00000603oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00`0oooo0`00000H0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0303oool3000001P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<
0?ooo`<00000603oool00`000000oooo0?ooo`040?ooo`001`3oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo00@0oooo00080?ooo`8000009@3oool00`000000oooo0?ooo`0<0?ooo`80
00009@3oool00`000000oooo0?ooo`0<0?ooo`8000009@3oool00`000000oooo0?ooo`0<0?ooo`80
00009P3oool00`000000oooo0?ooo`040?ooo`00203oool3000001L0oooo0`0000080?ooo`<00000
3P3oool3000001L0oooo0`0000080?ooo`<000003P3oool3000001L0oooo0`0000080?ooo`<00000
3P3oool3000001L0oooo0`0000080?ooo`<000001`3oool00080oooo0P0000070?ooo`030000003o
ool0oooo0180oooo0`000000103oool000000000000000070?ooo`8000002@3oool2000000H0oooo
00<000000?ooo`3oool04`3oool2000000050?ooo`00000000000?ooo`0000001P3oool2000000T0
oooo0P0000060?ooo`030000003oool0oooo01<0oooo0P0000020?ooo`030000003oool0000000H0
oooo0P0000090?ooo`8000001P3oool00`000000oooo0?ooo`0C0?ooo`8000000P3oool00`000000
oooo000000060?ooo`8000001`3oool00080oooo0`0000070?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool01@3oool2000000@0oooo00<000000?ooo`3oool02@3oool3000000H0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool02@3oool3000000H0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool3000000H0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool01P3oool000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00P0oooo00@000000?ooo`3oool000003@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0203oool01@000000oooo
0?ooo`3oool0000000`0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00P0oooo00D000000?ooo`3oool0oooo0000000<0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo`050000003oool0oooo
0?ooo`0000002@3oool000h0oooo0P00000=0?ooo`030000003oool0oooo00T0oooo00@000000?oo
o`3oool000005`3oool2000000d0oooo00<000000?ooo`3oool02@3oool010000000oooo0?ooo`00
000G0?ooo`8000003@3oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo000001L0oooo
0P00000=0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`0000002P3oool00100oooo00@0
00000?ooo`3oool0oooo0P0000020?ooo`<0000000<0oooo0000003oool0303oool2000001X0oooo
00@000000?ooo`3oool0oooo0P0000020?ooo`<000000P3oool00`000000oooo0?ooo`0:0?ooo`80
00006P3oool010000000oooo0?ooo`3oool200000080oooo0`0000020?ooo`030000003oool0oooo
00X0oooo0P00000J0?ooo`040000003oool0oooo0?ooo`8000000`3oool200000080oooo00<00000
0?ooo`3oool02P3oool2000000/0oooo000A0?ooo`<0000000<0oooo0000000000000P000000103o
ool0000000000000000=0?ooo`8000001P3oool00`000000oooo0?ooo`0B0?ooo`<0000000<0oooo
0000000000000P0000000`3oool0000000000002000000`0oooo0P0000060?ooo`030000003oool0
oooo0180oooo200000000`3oool0000000000002000000`0oooo0P0000060?ooo`030000003oool0
oooo0180oooo200000000`3oool0000000000002000000`0oooo0P0000070?ooo`030000003oool0
oooo0040oooo000@0?ooo`030000003oool0000000@0oooo00L000000?ooo`3oool0oooo0000003o
ool0000000`0oooo0P0000060?ooo`<000004@3oool00`000000oooo000000040?ooo`070000003o
ool0oooo0?ooo`000000oooo0000000<0?ooo`8000001P3oool300000140oooo00<000000?ooo`00
0000103oool01`000000oooo0?ooo`3oool000000?ooo`000000303oool2000000H0oooo0`00000A
0?ooo`030000003oool0000000@0oooo00L000000?ooo`3oool0oooo0000003oool0000000`0oooo
0P0000070?ooo`<000000@3oool000h0oooo0P00000=0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0000000?0?ooo`8000003@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`0000003`3oool2000000h0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`030?ooo`030000003oool0000000l0oooo0P00
000>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0103oool00`000000oooo0000
00010?ooo`003@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo00@00000
0?ooo`3oool000001@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0203oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo00D000000?ooo`3oool0
oooo000000040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`080?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`010?oo
o`00303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`060?ooo`002P3oool2000001@0oooo00<000000?ooo`000000
1000000I0?ooo`800000503oool00`000000oooo00000004000001T0oooo0P00000D0?ooo`030000
003oool0000000@000006@3oool2000001@0oooo00<000000?ooo`0000001000000@0?ooo`002@3o
ool2000002<0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool03P3o
ool2000002<0oooo00<000000?ooo`3oool03P3oool2000002@0oooo00<000000?ooo`3oool01@3o
ool000P0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo02@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0903oool2000000H0oooo00070?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool0103oool000H0oooo00<000000?ooo`3oool09`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`0W0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool09`3oool00`000000
oooo0?ooo`040?ooo`00103oool2000002/0oooo00<000000?ooo`3oool01P3oool2000002/0oooo
00<000000?ooo`3oool01P3oool2000002/0oooo00<000000?ooo`3oool01P3oool2000002/0oooo
00<000000?ooo`3oool00`3oool000<0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`020?ooo`000P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0000000`0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0000000`0oooo00<00000
0?ooo`3oool00@3oool00240oooo00<000000?ooo`3oool00P3oool3000002h0oooo10000000103o
ool0000000000000000_0?ooo`040000003oool000000?ooo`<00000;P3oool2000000030?ooo`00
0000oooo00<000003P3oool00240oooo0`0000020?ooo`030000003oool0000002h0oooo00P00000
0?ooo`00000000000?ooo`000000oooo000002l0oooo00L000000?ooo`000000oooo0000003oool0
000002h0oooo10000000103oool000000?ooo`00000>0?ooo`008@3oool00`000000oooo00000002
0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0^
0?ooo`<000000P3oool00`000000oooo0?ooo`0_0?ooo`050000003oool0oooo0?ooo`000000403o
ool00240oooo00<000000?ooo`3oool0=03oool2000003<0oooo0P00001:0?ooo`00S@3oool20000
03@0oooo0P00000D0?ooo`00``3oool2000000030?ooo`000000oooo0140oooo00360?ooo`030000
003oool0oooo0100oooo00360?ooo`030000003oool0oooo0100oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool0
0=T0oooo003I0?ooo`003P3oool010000000oooo0000000000020?ooo`<0000000<0oooo00000000
00000P3oool00`000000oooo000000020?ooo`050000003oool000000000003oool00P00000L0?oo
o`040000003oool0000000000080oooo0`0000000`3oool00000000000020?ooo`030000003oool0
00000080oooo00D000000?ooo`00000000000?ooo`02000001`0oooo00@000000?ooo`0000000000
0P3oool3000000D0oooo00<000000?ooo`0000000P3oool010000000oooo00000000000O0?ooo`04
0000003oool0000000000080oooo0`0000000`3oool00000000000020?ooo`030000003oool00000
0080oooo00D000000?ooo`00000000000?ooo`02000000l0oooo000>0?ooo`030000003oool0oooo
0080000000<0oooo0000003oool00`3oool00`000000oooo0?ooo`0300000080oooo00H000000?oo
o`000000oooo0000003oool2000001/0oooo00<000000?ooo`3oool00P0000000`3oool000000?oo
o`030?ooo`030000003oool0oooo00<000000P3oool00`000000oooo00000005000001/0oooo00<0
00000?ooo`3oool00P0000000`3oool000000?ooo`020?ooo`8000000P3oool300000080oooo00<0
00000?ooo`0000000P0000000`3oool000000?ooo`0K0?ooo`030000003oool0oooo0080000000<0
oooo0000003oool00`3oool2000000040?ooo`000000000000000080oooo00<000000?ooo`000000
1000000?0?ooo`003P3oool00`000000oooo000000030?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`0000000P0000020?ooo`060000003oool000000000003oool000007@3oool00`000000
oooo000000030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`0000020?ooo`06
0000003oool000000000003oool000007@3oool00`000000oooo000000030?ooo`040000003oool0
oooo0?ooo`8000000P3oool300000080oooo00@000000?ooo`00000000000P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0000000<0oooo00D000000?ooo`3oool0oooo000000030?ooo`<0
00000P3oool01@000000oooo000000000000oooo008000003`3oool000`0oooo100000000`3oool0
00000?ooo`020?ooo`8000000P3oool010000000oooo0?ooo`0000040?ooo`@000000P3oool00`00
0000oooo0?ooo`0H0?ooo`@0000000<0oooo0000003oool00P3oool200000080oooo00@000000?oo
o`3oool000000`3oool200000080oooo00@000000?ooo`3oool000006P3oool4000000030?ooo`00
0000oooo0080oooo0P0000050?ooo`030000003oool0oooo0080oooo0P0000000`3oool000000?oo
o`03000001T0oooo100000000`3oool000000?ooo`020?ooo`8000000P3oool010000000oooo0?oo
o`0000040?ooo`80000000D0oooo0000003oool0oooo0000000?0?ooo`00303oool300000180oooo
00@000000?ooo`00000000007@3oool300000140oooo0`00000O0?ooo`<000004P3oool2000001l0
oooo0`00000B0?ooo`800000503oool000d0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0000000O0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0B0?ooo`008P3oool2000003<0oooo0P00000d0?ooo`800000BP3oool001D0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`090?ooo`800000:03oool00`000000oooo0?ooo`09
0?ooo`800000:03oool00`000000oooo0?ooo`090?ooo`800000503oool00080oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0;0?ooo`8000003@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool02`3oool2000000d0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00P0oooo0P00000@0?oo
o`030000003oool0oooo0040oooo00030?ooo`030000003oool0oooo00h0oooo00<000000?ooo`00
00006`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`00
00003@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0000000=0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0000000d0oooo0P00000<0?ooo`03
0000003oool0oooo0080oooo00040?ooo`8000003`3oool3000001T0oooo00<000000?ooo`3oool0
1P3oool2000000l0oooo0`00000I0?ooo`030000003oool0oooo00H0oooo0P00000?0?ooo`<00000
6@3oool00`000000oooo0?ooo`060?ooo`8000003`3oool3000000`0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`030?ooo`001P3oool00`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo02L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo00@0oooo00070?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0103oool0
00P0oooo0P00000U0?ooo`030000003oool0oooo00`0oooo0P00000U0?ooo`030000003oool0oooo
00`0oooo0P00000U0?ooo`030000003oool0oooo00`0oooo0P00000V0?ooo`030000003oool0oooo
00@0oooo00080?ooo`<00000603oool00`000000oooo0?ooo`070?ooo`<000003P3oool3000001P0
oooo00<000000?ooo`3oool01`3oool3000000h0oooo0`00000I0?ooo`030000003oool0oooo00H0
oooo0`00000>0?ooo`<000006@3oool00`000000oooo0?ooo`060?ooo`<000001`3oool00080oooo
0P0000070?ooo`030000003oool0oooo0180oooo0`000000103oool000000000000000070?ooo`80
00002@3oool2000000H0oooo00<000000?ooo`3oool04`3oool2000000050?ooo`00000000000?oo
o`0000001P3oool2000000T0oooo0P0000060?ooo`030000003oool0oooo01<0oooo0P0000020?oo
o`030000003oool0000000H0oooo0P0000090?ooo`8000001P3oool00`000000oooo0?ooo`0C0?oo
o`8000000P3oool00`000000oooo000000060?ooo`8000001`3oool00080oooo0`0000070?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool01@3oool2000000@0oooo00<000000?ooo`3o
ool02@3oool3000000H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool02@3oool3000000H0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0203oool3000000H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool000<0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo00@000000?ooo`3oool0
00003@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool0203oool010000000oooo0?ooo`00000=0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`080?ooo`050000003oool0oooo0?ooo`000000303oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0203oool0
1@000000oooo0?ooo`3oool0000000T0oooo000>0?ooo`8000006@3oool00`000000oooo0000000H
0?ooo`8000006@3oool010000000oooo0?ooo`00000G0?ooo`8000006@3oool010000000oooo0?oo
o`00000G0?ooo`8000003@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0000000X0oooo
000@0?ooo`040000003oool0oooo0?ooo`8000000`3oool01@000000oooo0?ooo`3oool0000000`0
oooo0P00000J0?ooo`040000003oool0oooo0?ooo`8000000`3oool01@000000oooo0?ooo`3oool0
000000`0oooo0P00000J0?ooo`040000003oool0oooo0?ooo`8000000`3oool01@000000oooo0?oo
o`3oool0000000`0oooo0P00000J0?ooo`040000003oool0oooo0?ooo`8000000`3oool200000080
oooo00<000000?ooo`3oool02P3oool2000000/0oooo000A0?ooo`P0000000@0oooo000000000000
00003@3oool2000000H0oooo00<000000?ooo`3oool04P3oool8000000030?ooo`00000000000080
0000303oool2000000H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0000000500000003
0?ooo`000000000000800000303oool2000000H0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0000000:000000`0oooo0P0000070?ooo`030000003oool0oooo0040oooo000@0?ooo`<00000
0`3oool2000000<0oooo0`00000<0?ooo`8000001P3oool300000140oooo00<000000?ooo`000000
0`3oool2000000<0oooo0`00000<0?ooo`8000001P3oool300000140oooo00<000000?ooo`000000
0`3oool2000000<0oooo0`00000<0?ooo`8000001P3oool300000140oooo00<000000?ooo`000000
103oool010000000oooo0?ooo`3oool3000000`0oooo0P0000070?ooo`<000000@3oool000h0oooo
0P00000=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0000000?0?ooo`8000003@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`0000003`3oool2000000h0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0000000l0oooo0P00000>0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`0000001P3oool00`000000oooo000000010?ooo`003@3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0203oool01@000000oooo
0?ooo`3oool0000000@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00P0oooo00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo`050000003oool0oooo
0?ooo`0000001@3oool00`000000oooo0?ooo`010?ooo`00303oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?oo
o`002P3oool2000001@0oooo00<000000?ooo`0000000P3oool2000001T0oooo0P00000D0?ooo`03
0000003oool000000080oooo0P00000I0?ooo`800000503oool00`000000oooo000000020?ooo`80
00006@3oool2000001@0oooo00<000000?ooo`0000000P3oool200000100oooo00090?ooo`800000
5`3oool3000000T0oooo00<000000?ooo`3oool03P3oool2000001L0oooo0`0000090?ooo`030000
003oool0oooo00h0oooo0P00000G0?ooo`<000002@3oool00`000000oooo0?ooo`0>0?ooo`800000
5`3oool3000000X0oooo00<000000?ooo`3oool01@3oool000P0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903o
ool2000000H0oooo00070?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool09P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0103oool0
00H0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
02L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`040?ooo`00103oool2000002/0
oooo00<000000?ooo`3oool01P3oool2000002/0oooo00<000000?ooo`3oool01P3oool2000002/0
oooo00<000000?ooo`3oool01P3oool2000002/0oooo00<000000?ooo`3oool00`3oool000<0oooo
00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`020?ooo`000P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0000000`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0000000`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0000000`0oooo00<000000?ooo`3oool00@3oool00240oooo10000000
103oool0000000000000000]0?ooo`8000000P3oool00`000000oooo00000002000002h0oooo0P00
00000`3oool000000?ooo`03000002h0oooo0P0000000`3oool000000?ooo`03000000h0oooo000Q
0?ooo`080000003oool000000000003oool000000?ooo`00000]0?ooo`<0000000H0oooo0000003o
ool000000?ooo`00000^0?ooo`80000000H0oooo0000003oool000000?ooo`00000^0?ooo`800000
00H0oooo0000003oool000000?ooo`00000>0?ooo`008@3oool00`000000oooo000000020?ooo`03
0000003oool0oooo02h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0^0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0;`3oool01@000000oooo0?ooo`3oool00000
0100oooo000R0?ooo`800000]@3oool005L0oooo0P00000d0?ooo`800000BP3oool005L0oooo0P00
00000`3oool000000000000a0?ooo`80000000<0oooo000000000000<@3oool2000001@0oooo001J
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0<03oool2000000030?ooo`000000
00000140oooo00360?ooo`030000003oool0oooo0100oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo
003I0?ooo`003P3oool00`000000oooo0?ooo`030?ooo`<0000000<0oooo0000000000000P3oool0
0`000000oooo000000020?ooo`040000003oool0000000000080oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`030?ooo`<0000000<0oooo0000000000000P3oool00`000000oooo0000
00020?ooo`050000003oool000000000003oool00P00000K0?ooo`030000003oool0oooo00<0oooo
0`0000020?ooo`080000003oool000000000003oool000000?ooo`0000020?ooo`050000003oool0
00000000003oool00P00000K0?ooo`040000003oool0000000000080oooo0`0000020?ooo`060000
003oool0oooo0000003oool000000P3oool010000000oooo00000000000B0?ooo`003P3oool01000
0000oooo0000000000020?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo00L0
00000?ooo`000000oooo0000003oool0000001`0oooo00@000000?ooo`00000000000P3oool00`00
0000oooo0?ooo`020?ooo`80000000@0oooo00000000000000000P3oool01`000000oooo0000003o
ool000000?ooo`0000006`3oool010000000oooo0000000000020?ooo`030000003oool0oooo0080
oooo00@000000?ooo`000000oooo100000020?ooo`070000003oool000000?ooo`000000oooo0000
000K0?ooo`030000003oool0oooo0080000000<0oooo0000003oool00`3oool00`000000oooo0?oo
o`0300000080oooo00@000000?ooo`000000oooo0`00000?0?ooo`003P3oool010000000oooo0000
000000020?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo00@000000?ooo`00
000000000P3oool00`000000oooo0?ooo`0J0?ooo`040000003oool0000000000080oooo00P00000
0?ooo`3oool0oooo0000003oool000000?ooo`<000000P3oool010000000oooo0000000000030?oo
o`030000003oool0oooo01P0oooo00@000000?ooo`00000000000P3oool00`000000oooo0?ooo`02
0?ooo`050000003oool000000000003oool00`0000020?ooo`040000003oool0000000000080oooo
00<000000?ooo`3oool06@3oool00`000000oooo000000030?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00`0000020?ooo`050000003oool000000000003oool00P00000?0?ooo`00
303oool4000000D0oooo0P0000020?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3o
ool06`3oool4000000D0oooo0P0000020?ooo`040000003oool0oooo000000L0oooo00@000000?oo
o`3oool000006P3oool3000000D0oooo0P0000050?ooo`030000003oool0000000L0oooo00@00000
0?ooo`3oool000006P3oool3000000030?ooo`000000oooo0080oooo0P0000050?ooo`030000003o
ool0oooo0080oooo00@000000?ooo`3oool000004P3oool000`0oooo0`00000c0?ooo`<00000<`3o
ool3000003@0oooo0`00000A0?ooo`<000004`3oool000d0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0000000T0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`000000903oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0000002D0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0000000C0?ooo`006`3oool400000380oooo1000000b0?ooo`@00000>03oool00`00
0000oooo0?ooo`0C0?ooo`006`3oool400000380oooo1000000b0?ooo`@00000;03oool00`000000
oooo0?ooo`0O0?ooo`000P3oool00`000000oooo0?ooo`0F0?ooo`050000003oool000000?ooo`00
00004`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01H0oooo00D000000?ooo`00
0000oooo0000000C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05P3oool01@00
0000oooo0000003oool0000001<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00@3oool000<0oooo00<000000?oo
o`3oool04`3oool2000001L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?oo
o`8000005`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo0P00000G0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0000000K0?oo
o`030000003oool0oooo0080oooo00040?ooo`800000503oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo00H0oooo0P00000D0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool01P3oool2000001@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`060?ooo`80
00003`3oool3000001T0oooo00<000000?ooo`3oool00`3oool000H0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0B0?ooo`8000002`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo0180oooo0P00000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
4P3oool2000000/0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`040?ooo`001`3o
ool00`000000oooo0?ooo`0?0?ooo`<000004@3oool2000000h0oooo00<000000?ooo`3oool03`3o
ool300000140oooo0P00000>0?ooo`030000003oool0oooo00l0oooo0`00000A0?ooo`8000003P3o
ool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00@0oooo00080?ooo`8000008`3oool0
0`000000oooo0?ooo`0>0?ooo`8000008`3oool00`000000oooo0?ooo`0>0?ooo`8000008`3oool0
0`000000oooo0?ooo`0>0?ooo`8000009P3oool00`000000oooo0?ooo`040?ooo`00203oool30000
01`0oooo0P0000030?ooo`030000003oool0oooo00l0oooo0`00000L0?ooo`8000000`3oool00`00
0000oooo0?ooo`0?0?ooo`<00000703oool2000000<0oooo00<000000?ooo`3oool03`3oool30000
02@0oooo00<000000?ooo`3oool01@3oool00080oooo0P0000070?ooo`030000003oool0oooo01L0
oooo0P0000000`3oool0000000000002000000d0oooo0P0000060?ooo`030000003oool0oooo01L0
oooo0P0000000`3oool0000000000002000000d0oooo0P0000060?ooo`030000003oool0oooo01L0
oooo0`000000103oool0000000000000000=0?ooo`8000001P3oool00`000000oooo0?ooo`0C0?oo
o`D0000000<0oooo0000003oool0103oool3000000L0oooo00020?ooo`<000001`3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0303oool3000000H0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00`0
oooo0`0000060?ooo`030000003oool0oooo01D0oooo0P0000040?ooo`030000003oool0oooo00`0
oooo0`0000060?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00030?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool04P3oool3000000H0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0180oooo0`0000060?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0B0?ooo`<000001P3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`080?ooo`050000003oool0oooo0?ooo`0000002@3oool000h0oooo0P00000C0?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`800000
4`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
3`3oool2000001<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00l0oooo0P00000J0?ooo`030000003oool0000000X0oooo000@0?ooo`030000003o
ool0oooo00l0oooo0P0000070?ooo`030000003oool0oooo00@0oooo0`00000A0?ooo`030000003o
ool0oooo00l0oooo0P0000070?ooo`030000003oool0oooo00@0oooo0`00000A0?ooo`030000003o
ool0oooo00l0oooo0P0000070?ooo`030000003oool0oooo00@0oooo0`00000A0?ooo`040000003o
ool0oooo0?ooo`8000000`3oool01@000000oooo0?ooo`3oool0000000`0oooo0P00000;0?ooo`00
4@3oool00`000000oooo0000000200000080oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000004P3oool00`000000oooo
0000000200000080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`0000004P3oool00`000000oooo0000000200000080oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`0000004P3oool00`000000oooo00000003000000030?ooo`000000000000@00000
303oool2000000L0oooo00<000000?ooo`3oool00@3oool00100oooo0`0000020?ooo`D0000000@0
oooo0000000000000000103oool2000000H0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0A0?ooo`<000000P3oool5000000040?ooo`0000000000000000@0oooo0`0000050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool4000000030?ooo`000000000000<0
000000@0oooo0000000000000000103oool3000000D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0A0?ooo`040000003oool0000000000080oooo0P0000030?ooo`<00000303oool2
000000L0oooo0`0000010?ooo`003P3oool2000000T0oooo0`0000000`3oool00000000000030000
00030?ooo`000000000000D0oooo00<000000?ooo`3oool05`3oool2000000X0oooo0P0000000`3o
ool000000000000300000080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0G0?oo
o`8000002P3oool2000000030?ooo`000000000000@0000000<0oooo0000003oool0103oool00`00
0000oooo0?ooo`0G0?ooo`8000003P3oool00`000000oooo0?ooo`090?ooo`030000003oool00000
00H0oooo00<000000?ooo`0000000@3oool000d0oooo00<000000?ooo`3oool04P3oool8000001T0
oooo00<000000?ooo`3oool04P3oool6000000030?ooo`000000oooo01P0oooo00<000000?ooo`3o
ool04P3oool6000000030?ooo`000000oooo01P0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`080?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`010?oo
o`00303oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`070?ooo`002`3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool000000080oooo0P0000070?ooo`030000003oool0oooo00H0oooo00090?ooo`8000007@3o
ool2000001D0oooo0P00000M0?ooo`030000003oool0000001@0oooo0P00000M0?ooo`030000003o
ool0000001@0oooo0P00000G0?ooo`<000002P3oool00`000000oooo0?ooo`050?ooo`00203oool0
0`000000oooo0?ooo`0O0?ooo`8000004P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo02@0oooo0P0000060?ooo`001`3oool00`000000oooo0?ooo`0Q0?ooo`<000003`3oool00`00
0000oooo0?ooo`0Q0?ooo`<000003`3oool00`000000oooo0?ooo`0Q0?ooo`@000003P3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo00@0oooo00060?ooo`030000003oool0oooo02@0
oooo0P00000=0?ooo`030000003oool0oooo02@0oooo0`00000<0?ooo`030000003oool0oooo02@0
oooo0`00000<0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0103oool000@0oooo
0P00000W0?ooo`D00000203oool2000002L0oooo00@000000?ooo`000000oooo0P0000070?ooo`80
00009`3oool3000000030?ooo`000000000000L0oooo0P00000[0?ooo`030000003oool0oooo00<0
oooo00030?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool00P0000070?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool00`0000060?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`0000000P0000060?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0
0P3oool00080oooo00<000000?ooo`3oool0:`3oool4000000@0oooo00<000000?ooo`3oool0;03o
ool3000000@0oooo00<000000?ooo`3oool0;03oool3000000@0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0040oooo00330?ooo`030000003oool0
oooo0080oooo0`00000>0?ooo`00``3oool300000080oooo00<000000?ooo`0000003P3oool002@0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0300
oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0>0?ooo`00903oool3000003<0oooo
0`00000c0?ooo`<00000<03oool00`000000oooo0?ooo`0C0?ooo`00903oool00`000000oooo0000
000c0?ooo`030000003oool0000003<0oooo00<000000?ooo`000000AP3oool002@0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo04H0oooo003I0?oo
o`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I
0?ooo`00f@3oool00=T0oooo003I0?ooo`00f@3oool00=T0oooo003I0?ooo`00E03oool3000000P0
oooo0`00000;0?ooo`030000003oool0oooo00@0oooo00@000000?ooo`3oool000001`3oool01000
0000oooo0?ooo`00001F0?ooo`00E03oool3000000P0oooo0`0000080?ooo`D000001@3oool01000
0000oooo0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo001D0?ooo`<00000203oool3
000000/0oooo00<000000?ooo`3oool0103oool4000000L0oooo1000001F0?ooo`00M03oool01000
0000oooo0?ooo`0000070?ooo`040000003oool0oooo000005H0oooo003I0?ooo`00f@3oool00=T0
oooo003I0?ooo`00f@3oool00001\
\>"],
  ImageRangeCache->{{{0, 216.313}, {287, 0}} -> {-0.000440006, -0.0790874, 
  0.406824, 0.406824}, {{0, 54.0625}, {70.4375, 16.375}} -> {-1.00011,
   -89.1283, 0.406941, 0.406941}, {{54.0625, 108.125}, {70.4375, 16.375}} ->
   {-23.0004, -89.1283, 0.406941, 0.406941}, {{108.125, 162.188}, {70.4375, 
  16.375}} -> {-45.0006, -89.1283, 0.406941, 0.406941}, {{162.188, 216.313}, {
  70.4375, 16.375}} -> {-67.0136, -89.1283, 0.406941, 0.406941}, {{0, 
  54.0625}, {124.563, 70.4375}} -> {-1.00011, -67.1153, 0.406941, 0.406941},
   {{54.0625, 108.125}, {124.563, 70.4375}} -> {-23.0004, -67.1153, 0.406941, 
  0.406941}, {{108.125, 162.188}, {124.563, 70.4375}} -> {-45.0006, -67.1153, 
  0.406941, 0.406941}, {{162.188, 216.313}, {124.563, 70.4375}} -> {-66.9246,
   -67.0262, 0.406471, 0.406471}, {{0, 54.0625}, {178.625, 124.563}} ->
   {-1.00011, -45.1023, 0.406941, 0.406941}, {{54.0625, 108.125}, {178.625, 
  124.563}} -> {-23.0004, -45.1023, 0.406941, 0.406941}, {{108.125, 162.188},
   {178.625, 124.563}} -> {-45.0006, -45.1023, 0.406941, 0.406941}, {{162.188,
   216.313}, {178.625, 124.563}} -> {-67.0136, -45.1023, 0.406941, 0.406941},
   {{0, 54.0625}, {232.688, 178.625}} -> {-1.00011, -23.1021, 0.406941, 
  0.406941}, {{54.0625, 108.125}, {232.688, 178.625}} -> {-23.0004, -23.1021, 
  0.406941, 0.406941}, {{108.125, 162.188}, {232.688, 178.625}} -> {-45.0006,
   -23.1021, 0.406941, 0.406941}, {{162.188, 216.313}, {232.688, 178.625}} ->
   {-67.0136, -23.1021, 0.406941, 0.406941}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(vert\  = \ 
      CalcFeynAmp[\n\ \ CreateFeynAmp[ins], \n\ \ 
        Select[counter, \ DiagramType[#]\  == \ 1\ &]]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"6 Generic, 58 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "6 Generic, 58 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"7 Generic, 25 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "7 Generic, 25 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 10 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 3, ": ", "1 Generic, 10 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"1 Generic, 10 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 4, ": ", "1 Generic, 10 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]5
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 13 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 5, ": ", "2 Generic, 13 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"17 Generic, 116 Classes amplitudes"\),
        
        SequenceForm[ "in total: ", "17 Generic, 116 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]19
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 19, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]0
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 0, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell["running FORM... ", "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    FormBox[
      RowBox[{\((Amp({ME, ME} \[Rule] {MHp, MHp}))\), "(", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\(Den(S, 0)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                      \(-\(\(Alfa2\ \(B0(MHp2, Mh02, MW2)\)\ CBA\^2\)\/\(2\ 
                                SW2\)\)\), "+", 
                        \(\(Alfa2\ \(B1(MHp2, Mh02, MW2)\)\ CBA\^2\)\/\(2\ SW2
                              \)\), "-", 
                        \(\(Alfa2\ 
                              \(C0i(cc00, MHp2, S, MHp2, Mh02, MW2, MW2)\)\ 
                              CBA\^2\)\/SW2\), "-", \(2\ Alfa2\), "-", 
                        \(\(2\ Alfa2\ SW2\)\/CW2\), "-", 
                        \(\(6\ Alfa\ dZZA1\ \[Pi]\ SW2\)\/\(CW\ SW\)\), "-", 
                        \(4\ Alfa2\ \(B0(MHp2, 0, MHp2)\)\), "-", 
                        \(\(Alfa2\ \(B0(MHp2, MA02, MW2)\)\)\/\(2\ SW2\)\), 
                        "-", \(\(Alfa2\ SBA\^2\ \(B0(MHp2, MHH2, MW2)\)\)\/\(2
                              \ SW2\)\), "+", 
                        \(\((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                              \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\)\ 
                          \(B0(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(4\ SW2\ Alfa2\)\/CW2 - \(4\ Alfa2\)\/CW2 + 
                              Alfa2\/\(CW2\ SW2\) + 4\ Alfa2)\)\ 
                          \(B0(S, ME2, ME2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\)\/SW2 + 
                              \(Alfa2\ SBA\^2\)\/SW2 + \(3\ Alfa2\)\/SW2)\)\ 
                          \(B0(S, MW2, MW2)\)\), "+", 
                        \(\(Alfa2\ \(B1(MHp2, MA02, MW2)\)\)\/\(2\ SW2\)\), 
                        "+", \(\(Alfa2\ SBA\^2\ \(B1(MHp2, MHH2, MW2)\)\)\/\(2
                              \ SW2\)\), "+", 
                        \(4\ Alfa2\ \(B1(MHp2, MHp2, 0)\)\), "+", 
                        \(\((\(SW2\ Alfa2\)\/CW2 + \(CW2\ Alfa2\)\/SW2 - 
                              2\ Alfa2)\)\ \(B1(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(4\ Alfa2\ SW2\ MZ2\)\/CW2 - 
                              \(4\ Alfa2\ MZ2\)\/CW2 + 
                              \(Alfa2\ MZ2\)\/\(CW2\ SW2\) + 
                              S\ \((\(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                                  \(4\ Alfa2\)\/CW2 - Alfa2\/\(CW2\ SW2\))\))
                            \)\ \(C0i(cc0, ME2, S, ME2, MZ2, ME2, ME2)\)\), 
                        "+", \(\((8\ Alfa2\ MHp2 - 4\ Alfa2\ S)\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MA02\)\/SW2 + \(Alfa2\ S\)\/\(4\ SW2\))
                            \)\ \(C0i(cc0, MHp2, S, MHp2, MA02, MW2, MW2)\)\),
                         "+", \(\((
                            \(Alfa2\ Mh02\ CBA\^2\)\/SW2 + 
                              \(Alfa2\ S\ CBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\ MW2\ S2B\ SAB\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(2\ CW2\^2\ SW2
                                  \))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(2\ CW2\^2\ SW2
                                  \))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ MHH2\ SBA\^2\)\/SW2 + 
                              \(Alfa2\ S\ SBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-4\)\ Alfa2\ MHp2 + 
                              \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(2\ Alfa2\ CW2\ MHp2\)\/SW2 - Alfa2\ MZ2 + 
                              \(Alfa2\ MZ2\ SW2\)\/\(2\ CW2\) + 
                              S\ \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                  \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "-",
                         \(8\ Alfa2\ 
                          \(C0i(cc00, ME2, S, ME2, 0, ME2, ME2)\)\), "+", 
                        \(\(4\ Alfa2\ 
                              \(C0i(cc00, ME2, S, ME2, 0, MW2, MW2)\)\)\/SW2
                            \), "+", 
                        \(\((\(-\(\(8\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(8\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/\(CW2\ SW2\))
                            \)\ \(C0i(cc00, ME2, S, ME2, MZ2, ME2, ME2)\)\), 
                        "+", \(8\ Alfa2\ 
                          \(C0i(cc00, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "-", 
                        \(\(Alfa2\ 
                              \(C0i(cc00, MHp2, S, MHp2, MA02, MW2, MW2)
                                \)\)\/SW2\), "-", 
                        \(\(Alfa2\ SBA\^2\ 
                              \(C0i(cc00, MHp2, S, MHp2, MHH2, MW2, MW2)
                                \)\)\/SW2\), "+", 
                        \(\((\(2\ SW2\ Alfa2\)\/CW2 + 
                              \(2\ CW2\ Alfa2\)\/SW2 - 4\ Alfa2)\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((
                            12\ Alfa2\ MHp2 - 
                              4\ Alfa2\ \((\(3\ S\)\/2 - MHp2)\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S)\)\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ \((MHp2 + S)\)\)\/\(2\ SW2\))
                            \)\ \(C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2)\)\),
                         "+", \(\((
                            \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S)\)\ SBA\^2\)\/\(2\ SW2\))
                            \)\ \(C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2)\)\),
                         "+", \(\((
                            \(-6\)\ Alfa2\ MHp2 + 
                              \(3\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ CW2\ MHp2\)\/SW2 - Alfa2\ MZ2 + 
                              \(Alfa2\ MZ2\ SW2\)\/\(2\ CW2\) + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                    \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((4\ Alfa2\ MHp2 + 2\ Alfa2\ \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MHp2\)\/\(2\ SW2\)\)\) - 
                              \(Alfa2\ \((T + U)\)\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(-
                                \(\(Alfa2\ \((T + U)\)\ CBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(-
                                \(\(Alfa2\ \((T + U)\)\ SBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(-2\)\ Alfa2\ MHp2 + 
                              \(Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(Alfa2\ CW2\ MHp2\)\/SW2 + 
                              \((\(SW2\ Alfa2\)\/\(2\ CW2\) + 
                                    \(CW2\ Alfa2\)\/\(2\ SW2\) - Alfa2)\)\ 
                                \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((16\ Alfa2\ MHp2 - 4\ Alfa2\ S)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ S\)\/\(2\ SW2\) - 
                              \(2\ Alfa2\ MHp2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ CBA\^2\ S\)\/\(2\ SW2\) - 
                              \(2\ Alfa2\ CBA\^2\ MHp2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ S\ SBA\^2\)\/\(2\ SW2\) - 
                              \(2\ Alfa2\ MHp2\ SBA\^2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(-8\)\ Alfa2\ MHp2 + 
                              \(4\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(4\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                              S\ \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                  \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "-", \(4\ Alfa2\ S\ \((
                          C0i(cc0, ME2, S, ME2, 0, ME2, ME2) + 
                            C0i(cc1, ME2, S, ME2, 0, ME2, ME2) + 
                            C0i(cc2, ME2, S, ME2, 0, ME2, ME2))\)\), "-", 
                        \(\(2\ Alfa2\ S\ \((
                              C0i(cc1, ME2, S, ME2, 0, MW2, MW2) + 
                                C0i(cc2, ME2, S, ME2, 0, MW2, MW2))\)\)\/SW2
                            \), "+", 
                        \(S\ \((
                          \(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                            \(4\ Alfa2\)\/CW2 - Alfa2\/\(CW2\ SW2\))\)\ 
                          \((C0i(cc1, ME2, S, ME2, MZ2, ME2, ME2) + 
                              C0i(cc2, ME2, S, ME2, MZ2, ME2, ME2))\)\), "+", 
                        \(\((20\ Alfa2\ MHp2 - 2\ Alfa2\ \((4\ S + T + U)\))
                            \)\ \(C0i(cc2, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ MHp2\)\/\(2\ SW2\)\)\) - 
                              \(Alfa2\ \((S - T - U)\)\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "-", 
                        \(1\/SW2\((
                          Alfa2\ MW2\ \((
                            C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2))\))\)\),
                         "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(2\ CW2\^2\ 
                                      SW2\)\)\) - 
                              \(2\ Alfa2\ C2B\ MW2\ SBA\ SAB\)\/\(CW2\ SW2
                                  \) - \(2\ Alfa2\ MW2\ SBA\^2\)\/SW2)\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ \((S - T - U)\)\ CBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MW2\ CBA\^2\)\/SW2\)\) + 
                              \(3\ Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(2\ 
                                  CW2\^2\ SW2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(2\ CW2\^2\ 
                                      SW2\)\)\) + 
                              \(2\ Alfa2\ C2B\ CBA\ MW2\ CAB\)\/\(CW2\ SW2
                                  \) - \(2\ Alfa2\ CBA\^2\ MW2\)\/SW2)\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ \((S - T - U)\)\ SBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(2\ 
                                      CW2\^2\ SW2\)\)\) + 
                              \(3\ Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ SBA\^2\)\/SW2)\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-10\)\ Alfa2\ MHp2 + 
                              \(5\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(5\ Alfa2\ CW2\ MHp2\)\/SW2 - Alfa2\ MZ2 + 
                              \(Alfa2\ MZ2\ SW2\)\/\(2\ CW2\) + 
                              1\/4\ \((
                                  \(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) - 
                                    \(2\ CW2\ Alfa2\)\/SW2 + 4\ Alfa2)\)\ 
                                \((4\ S + T + U)\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((12\ Alfa2\ MHp2 - 4\ Alfa2\ \((MHp2 + S\/2)\))
                            \)\ \(C0i(cc22, MHp2, S, MHp2, 0, MHp2, MHp2)\)\),
                         "+", \(\((
                            \(Alfa2\ \((MHp2 + S\/2)\)\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ CBA\^2\ \((MHp2 + S\/2)\)\)\/\(2\ SW2
                                  \) - \(3\ Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\))
                            \)\ \(C0i(cc22, MHp2, S, MHp2, Mh02, MW2, MW2)
                            \)\), "+", 
                        \(\((\(Alfa2\ \((MHp2 + S\/2)\)\ SBA\^2\)\/\(2\ SW2
                                  \) - \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))
                            \)\ \(C0i(cc22, MHp2, S, MHp2, MHH2, MW2, MW2)
                            \)\), "+", 
                        \(\((\(-6\)\ Alfa2\ MHp2 + 
                              \(3\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                    \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", 
                        RowBox[{"4", " ", "Alfa", " ", "\[Pi]", " ", 
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{
                                \(dZfL1(2, 1, 1)\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\^"*"\),
                                    Superscript[ "*"],
                                    Editable->False]}],
                                SequenceForm[ 
                                  dZfL1[ 2, 1, 1], 
                                  Superscript[ "*"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}], "+", 
                        \(4\ Alfa\ \[Pi]\ \(dZfL1(2, 1, 1)\)\), "+", 
                        \(\(2\ Alfa2\)\/CW2\), "+", 
                        \(\(2\ Alfa\ CW2\ dZZA1\ \[Pi]\)\/\(CW\ SW\)\), "+", 
                        \(\(2\ Alfa\ dZZA1\ \[Pi]\)\/\(CW\ SW\)\), "-", 
                        \(Alfa2\/SW2\), "-", \(Alfa2\/\(2\ CW2\ SW2\)\), "+", 
                        \(8\ Alfa\ dZAA1\ \[Pi]\), "+", 
                        \(16\ Alfa\ dZe1\ \[Pi]\), "+", 
                        \(8\ Alfa\ dZGp1\ \[Pi]\)}], ")"}]}], "+", 
                  RowBox[{\(Den(S, MZ2)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                      \(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\), "+", 
                        \(\(2\ Alfa\ \[Pi]\ \(dZfL1(2, 1, 1)\)\ SW2\)\/CW2\), 
                        "-", \(\(2\ Alfa\ dSW1\ \[Pi]\ SW2\)\/\(CW2\^2\ SW
                              \)\), "-", 
                        \(\(2\ Alfa\ dZAZ1\ \[Pi]\ SW2\)\/\(CW\ SW\)\), "+", 
                        \(\(3\ Alfa2\ SW2\)\/\(2\ CW2\^2\)\), "+", 
                        \(\(8\ Alfa\ dZe1\ \[Pi]\ SW2\)\/CW2\), "+", 
                        \(\(4\ Alfa\ dZGp1\ \[Pi]\ SW2\)\/CW2\), "+", 
                        \(\(4\ Alfa\ dZZZ1\ \[Pi]\ SW2\)\/CW2\), "+", "Alfa2",
                         "+", \(\(\(-
                                \(\(4\ Alfa\ dZAZ1\ \[Pi]\ SW2\^2\)\/\(CW\ SW
                                      \)\)\) + 
                              \(4\ Alfa\ CW2\ dZAZ1\ \[Pi]\ SW2\)\/\(CW\ SW
                                  \) + \(2\ Alfa\ dZAZ1\ \[Pi]\ SW2\)\/\(CW\ 
                                  SW\) - 
                              \(2\ Alfa\ CW2\ dZAZ1\ \[Pi]\)\/\(CW\ SW
                                  \)\)\/\(SW2 - CW2\)\), "+", 
                        \(\((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + Alfa2\/CW2 - 
                              Alfa2\/SW2 + 2\ Alfa2)\)\ 
                          \(B0(MHp2, 0, MHp2)\)\), "+", 
                        \(\((Alfa2\/\(4\ CW2\ SW2\) - Alfa2\/\(2\ CW2\))\)\ 
                          \(B0(MHp2, MA02, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\) - 
                              \(Alfa2\ CBA\^2\)\/\(2\ CW2\))\)\ 
                          \(B0(MHp2, Mh02, MW2)\)\), "+", 
                        \(\((\(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\) - 
                              \(Alfa2\ SBA\^2\)\/\(2\ CW2\))\)\ 
                          \(B0(MHp2, MHH2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                              \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - 
                              \(3\ Alfa2\)\/2 - \(3\ Alfa2\)\/\(4\ CW2\) + 
                              \(3\ Alfa2\)\/\(4\ SW2\) + 
                              \(Alfa2\ CW2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\)\ 
                          \(B0(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((Alfa2\/\(2\ CW2\ SW2\) - Alfa2\/\(2\ SW2\^2\))
                            \)\ \(B0(S, 0, 0)\)\), "+", 
                        \(\((\(2\ Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(3\ Alfa2\ SW2\)\/CW2\^2 - 2\ Alfa2 + 
                              \(2\ Alfa2\)\/CW2 + 
                              \(3\ Alfa2\)\/\(2\ CW2\^2\) + Alfa2\/SW2 - 
                              \(3\ Alfa2\)\/\(2\ CW2\ SW2\) - 
                              Alfa2\/\(4\ CW2\^2\ SW2\) + 
                              Alfa2\/\(4\ CW2\ SW2\^2\))\)\ 
                          \(B0(S, ME2, ME2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ CBA\^2\)\/SW2\)\) + 
                              \(Alfa2\ CBA\^2\)\/\(2\ SW2\^2\) - 
                              \(Alfa2\ SBA\^2\)\/SW2 - \(2\ Alfa2\)\/SW2 + 
                              \(Alfa2\ SBA\^2\)\/\(2\ SW2\^2\) + 
                              Alfa2\/\(2\ SW2\^2\) + \(Alfa2\ CW2\)\/SW2\^2)
                            \)\ \(B0(S, MW2, MW2)\)\), "+", 
                        \(\((Alfa2\/\(2\ CW2\) - Alfa2\/\(4\ CW2\ SW2\))\)\ 
                          \(B1(MHp2, MA02, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\)\/\(2\ CW2\) - 
                              \(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\))\)\ 
                          \(B1(MHp2, Mh02, MW2)\)\), "+", 
                        \(\((\(Alfa2\ SBA\^2\)\/\(2\ CW2\) - 
                              \(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\))\)\ 
                          \(B1(MHp2, MHH2, MW2)\)\), "+", 
                        \(\((\(2\ SW2\ Alfa2\)\/CW2 - Alfa2\/CW2 + 
                              Alfa2\/SW2 - 2\ Alfa2)\)\ 
                          \(B1(MHp2, MHp2, 0)\)\), "+", 
                        \(\((\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - 
                              \(Alfa2\ SW2\)\/\(4\ CW2\^2\) + 
                              \(3\ Alfa2\)\/2 + \(3\ Alfa2\)\/\(4\ CW2\) - 
                              \(3\ Alfa2\)\/\(4\ SW2\) - 
                              \(Alfa2\ CW2\)\/\(2\ SW2\) + 
                              \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\)\ 
                          \(B1(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(Alfa2\ MW2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\)\/\(2\ SW2\^2\) + 
                              S\ \((Alfa2\/\(2\ SW2\^2\) - 
                                  Alfa2\/\(2\ CW2\ SW2\))\))\)\ 
                          \(C0i(cc0, ME2, S, ME2, MW2, 0, 0)\)\), "+", 
                        \(\((\(2\ Alfa2\ MZ2\ SW2\^2\)\/CW2\^2 - 
                              \(2\ Alfa2\ MZ2\ SW2\)\/CW2 - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/CW2\^2 + 
                              \(3\ Alfa2\ MZ2\)\/CW2 + 
                              \(3\ Alfa2\ MZ2\)\/\(2\ CW2\^2\) + 
                              S\ \((\(-\(\(2\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                                  \(2\ Alfa2\ SW2\)\/CW2 + 
                                  \(3\ Alfa2\ SW2\)\/CW2\^2 - 
                                  \(3\ Alfa2\)\/CW2 - 
                                  \(3\ Alfa2\)\/\(2\ CW2\^2\) + 
                                  \(3\ Alfa2\)\/\(2\ CW2\ SW2\) + 
                                  Alfa2\/\(4\ CW2\^2\ SW2\) - 
                                  Alfa2\/\(4\ CW2\ SW2\^2\))\) - 
                              \(3\ Alfa2\ MZ2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MZ2\)\/\(4\ CW2\^2\ SW2\) + 
                              \(Alfa2\ MZ2\)\/\(4\ CW2\ SW2\^2\))\)\ 
                          \(C0i(cc0, ME2, S, ME2, MZ2, ME2, ME2)\)\), "+", 
                        \(\((\(-4\)\ Alfa2\ MHp2 + 
                              \(4\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(2\ Alfa2\ MHp2\)\/CW2 + 
                              \(2\ Alfa2\ MHp2\)\/SW2 + 
                              S\ \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + 
                                  Alfa2\/CW2 - Alfa2\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MA02\)\/SW2\)\) + 
                              \(Alfa2\ MA02\)\/\(2\ SW2\^2\) + 
                              S\ \((Alfa2\/\(8\ SW2\^2\) - Alfa2\/\(4\ SW2\))
                                \))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ MW2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\)\/\(4\ SW2\) - 
                              \(Alfa2\ MW2\)\/\(8\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\)\/\(8\ SW2\^2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ Mh02\ CBA\^2\)\/SW2\)\) + 
                              \(Alfa2\ Mh02\ CBA\^2\)\/\(2\ SW2\^2\) + 
                              S\ \((\(Alfa2\ CBA\^2\)\/\(8\ SW2\^2\) - 
                                  \(Alfa2\ CBA\^2\)\/\(4\ SW2\))\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ SW2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(8\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(8\ SW2\^2\) - 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ CW2\ SW2
                                  \) + \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2
                                  \ SW2\^2\) - 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ CW2\^3\) + 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(8\ CW2\^2\ 
                                  SW2\^2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MHH2\ SBA\^2\)\/SW2\)\) + 
                              \(Alfa2\ MHH2\ SBA\^2\)\/\(2\ SW2\^2\) + 
                              S\ \((\(Alfa2\ SBA\^2\)\/\(8\ SW2\^2\) - 
                                  \(Alfa2\ SBA\^2\)\/\(4\ SW2\))\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-
                                \(\(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(8\ CW2\^3\ SW2
                                  \) - \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) - 
                              \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ CW2\ SW2
                                  \) + \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(4\ CW2
                                  \ SW2\^2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ SW2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(8\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(8\ SW2\^2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\^2\ SW2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^3\) + 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(8\ CW2\^2\ 
                                  SW2\^2\))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHp2, Mh02, MZ2) + 
                              C0i(cc0, MHp2, S, MHp2, MHp2, MZ2, Mh02))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\^2\ SW2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) + 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^3\) - 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(8\ 
                                  CW2\^3\ SW2\) + 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(8\ CW2\^2\ 
                                  SW2\^2\))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHp2, MHH2, MZ2) + 
                              C0i(cc0, MHp2, S, MHp2, MHp2, MZ2, MHH2))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ CBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\^2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) + 
                              S\ \((\(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                  \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc0, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\^2\) + 
                              \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) + 
                              S\ \((\(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                  \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc0, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 + 
                              \(Alfa2\ MZ2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                              \(3\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                              \(Alfa2\ MHp2\ SW2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/\(4\ CW2\) - 
                              \(Alfa2\ MZ2\ SW2\)\/\(8\ CW2\^2\) + 
                              3\ Alfa2\ MHp2 + 
                              \(3\ Alfa2\ MHp2\)\/\(2\ CW2\) + 
                              \(3\ Alfa2\ MZ2\)\/4 + 
                              \(3\ Alfa2\ MZ2\)\/\(8\ CW2\) + 
                              S\ \((\(-
                                    \(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                                  \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                                  \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - 
                                  \(3\ Alfa2\)\/2 - 
                                  \(3\ Alfa2\)\/\(4\ CW2\) + 
                                  \(3\ Alfa2\)\/\(4\ SW2\) + 
                                  \(Alfa2\ CW2\)\/\(2\ SW2\) - 
                                  \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\) - 
                              \(3\ Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CW2\ MHp2\)\/SW2 - 
                              \(3\ Alfa2\ MZ2\)\/\(8\ SW2\) - 
                              \(Alfa2\ CW2\ MZ2\)\/\(4\ SW2\) + 
                              \(Alfa2\ CW2\ MHp2\)\/\(2\ SW2\^2\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(8\ SW2\^2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((\(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(2\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/SW2 + 
                              4\ Alfa2)\)\ 
                          \(C0i(cc00, ME2, S, ME2, 0, ME2, ME2)\)\), "+", 
                        \(\((\(2\ Alfa2\ CW2\)\/SW2\^2 - \(2\ Alfa2\)\/SW2)
                            \)\ \(C0i(cc00, ME2, S, ME2, 0, MW2, MW2)\)\), 
                        "+", \(\((Alfa2\/SW2\^2 - Alfa2\/\(CW2\ SW2\))\)\ 
                          \(C0i(cc00, ME2, S, ME2, MW2, 0, 0)\)\), "+", 
                        \(\((\(-\(\(4\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(4\ Alfa2\ SW2\)\/CW2 + 
                              \(6\ Alfa2\ SW2\)\/CW2\^2 - \(6\ Alfa2\)\/CW2 - 
                              \(3\ Alfa2\)\/CW2\^2 + 
                              \(3\ Alfa2\)\/\(CW2\ SW2\) + 
                              Alfa2\/\(2\ CW2\^2\ SW2\) - 
                              Alfa2\/\(2\ CW2\ SW2\^2\))\)\ 
                          \(C0i(cc00, ME2, S, ME2, MZ2, ME2, ME2)\)\), "+", 
                        \(\((\(4\ SW2\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/CW2 + 
                              \(2\ Alfa2\)\/SW2 - 4\ Alfa2)\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((Alfa2\/SW2 - Alfa2\/\(2\ SW2\^2\))\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ CBA\^2\)\/SW2 - 
                              \(Alfa2\ CBA\^2\)\/\(2\ SW2\^2\))\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ SBA\^2\)\/SW2 - 
                              \(Alfa2\ SBA\^2\)\/\(2\ SW2\^2\))\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ CBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \((C0i(cc00, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc00, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ SBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\^2\))\)\ 
                          \((C0i(cc00, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc00, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(3\ Alfa2\ SW2\)\/CW2 - 
                              \(Alfa2\ SW2\)\/\(2\ CW2\^2\) + 3\ Alfa2 + 
                              \(3\ Alfa2\)\/\(2\ CW2\) - 
                              \(3\ Alfa2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CW2\)\/SW2 + 
                              \(Alfa2\ CW2\)\/\(2\ SW2\^2\))\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((
                            \(-6\)\ Alfa2\ MHp2 + 
                              \(6\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(3\ Alfa2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ MHp2\)\/SW2 + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + 
                                    Alfa2\/CW2 - Alfa2\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MHp2\)\/\(2\ SW2\)\)\) + 
                              \(Alfa2\ MHp2\)\/\(4\ SW2\^2\) + 
                              2\/3\ \((MHp2 + S)\)\ 
                                \((\(3\ Alfa2\)\/\(4\ SW2\) - 
                                    \(3\ Alfa2\)\/\(8\ SW2\^2\))\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\)\)\) + 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(4\ SW2\^2\) + 
                              2\/3\ \((MHp2 + S)\)\ 
                                \((\(3\ Alfa2\ CBA\^2\)\/\(4\ SW2\) - 
                                    \(3\ Alfa2\ CBA\^2\)\/\(8\ SW2\^2\))\))
                            \)\ \(C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2)\)\),
                         "+", \(\((
                            \(-\(\(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\)\)\) + 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(4\ SW2\^2\) + 
                              2\/3\ \((MHp2 + S)\)\ 
                                \((\(3\ Alfa2\ SBA\^2\)\/\(4\ SW2\) - 
                                    \(3\ Alfa2\ SBA\^2\)\/\(8\ SW2\^2\))\))
                            \)\ \(C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2)\)\),
                         "+", \(\((
                            \(3\ Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\ CBA\^2\)\/\(4\ CW2\ 
                                  SW2\^2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc1, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(4\ CW2\ 
                                  SW2\^2\) + 
                              \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc1, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) + 
                              \(Alfa2\ MZ2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                              \(9\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) - 
                              \(3\ Alfa2\ MHp2\ SW2\)\/\(4\ CW2\^2\) - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/\(4\ CW2\) - 
                              \(Alfa2\ MZ2\ SW2\)\/\(8\ CW2\^2\) + 
                              \(9\ Alfa2\ MHp2\)\/2 + 
                              \(9\ Alfa2\ MHp2\)\/\(4\ CW2\) + 
                              \(3\ Alfa2\ MZ2\)\/4 + 
                              \(3\ Alfa2\ MZ2\)\/\(8\ CW2\) + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(-
                                      \(\(Alfa2\ SW2\^2\)\/\(2\ 
                                        CW2\^2\)\)\) + 
                                    \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                                    \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - 
                                    \(3\ Alfa2\)\/2 - 
                                    \(3\ Alfa2\)\/\(4\ CW2\) + 
                                    \(3\ Alfa2\)\/\(4\ SW2\) + 
                                    \(Alfa2\ CW2\)\/\(2\ SW2\) - 
                                    \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\) - 
                              \(9\ Alfa2\ MHp2\)\/\(4\ SW2\) - 
                              \(3\ Alfa2\ CW2\ MHp2\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MZ2\)\/\(8\ SW2\) - 
                              \(Alfa2\ CW2\ MZ2\)\/\(4\ SW2\) + 
                              \(3\ Alfa2\ CW2\ MHp2\)\/\(4\ SW2\^2\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(8\ SW2\^2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((\(-2\)\ Alfa2\ MHp2 + 
                              \(2\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(Alfa2\ MHp2\)\/CW2 + \(Alfa2\ MHp2\)\/SW2 + 
                              \((\(SW2\ Alfa2\)\/CW2 - Alfa2\/\(2\ CW2\) + 
                                    Alfa2\/\(2\ SW2\) - Alfa2)\)\ \((T + U)\))
                            \)\ \(C0i(cc11, MHp2, S, MHp2, 0, MHp2, MHp2)\)\),
                         "+", \(\((
                            \(Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\)\/\(4\ SW2\^2\) + 
                              \((Alfa2\/\(4\ SW2\) - Alfa2\/\(8\ SW2\^2\))\)\ 
                                \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(4\ SW2\^2\) + 
                              \((\(Alfa2\ CBA\^2\)\/\(4\ SW2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(8\ SW2\^2\))\)\ 
                                \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(4\ SW2\^2\) + 
                              \((\(Alfa2\ SBA\^2\)\/\(4\ SW2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(8\ SW2\^2\))\)\ 
                                \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) + 
                              \((\(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(8\ CW2\ SW2\^2\))\)\ 
                                \((T + U)\))\)\ 
                          \((C0i(cc11, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc11, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) + 
                              \((\(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(8\ CW2\ SW2\^2\))\)\ 
                                \((T + U)\))\)\ 
                          \((C0i(cc11, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc11, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) - 
                              \(Alfa2\ MHp2\ SW2\)\/\(4\ CW2\^2\) + 
                              \(3\ Alfa2\ MHp2\)\/2 + 
                              \(3\ Alfa2\ MHp2\)\/\(4\ CW2\) + 
                              \((\(Alfa2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                                    \(3\ Alfa2\ SW2\)\/\(4\ CW2\) - 
                                    \(Alfa2\ SW2\)\/\(8\ CW2\^2\) + 
                                    \(3\ Alfa2\)\/4 + 
                                    \(3\ Alfa2\)\/\(8\ CW2\) - 
                                    \(3\ Alfa2\)\/\(8\ SW2\) - 
                                    \(Alfa2\ CW2\)\/\(4\ SW2\) + 
                                    \(Alfa2\ CW2\)\/\(8\ SW2\^2\))\)\ 
                                \((T + U)\) - 
                              \(3\ Alfa2\ MHp2\)\/\(4\ SW2\) - 
                              \(Alfa2\ CW2\ MHp2\)\/\(2\ SW2\) + 
                              \(Alfa2\ CW2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((
                            \(-8\)\ Alfa2\ MHp2 + 
                              \(8\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(4\ Alfa2\ MHp2\)\/CW2 + 
                              \(4\ Alfa2\ MHp2\)\/SW2 + 
                              S\ \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + 
                                  Alfa2\/CW2 - Alfa2\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(2\ Alfa2\ MHp2\)\/SW2 - 
                              \(Alfa2\ MHp2\)\/SW2\^2 + 
                              S\ \((Alfa2\/\(4\ SW2\^2\) - Alfa2\/\(2\ SW2\))
                                \))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(2\ Alfa2\ MHp2\ CBA\^2\)\/SW2 - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/SW2\^2 + 
                              S\ \((\(Alfa2\ CBA\^2\)\/\(4\ SW2\^2\) - 
                                  \(Alfa2\ CBA\^2\)\/\(2\ SW2\))\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(2\ Alfa2\ MHp2\ SBA\^2\)\/SW2 - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/SW2\^2 + 
                              S\ \((\(Alfa2\ SBA\^2\)\/\(4\ SW2\^2\) - 
                                  \(Alfa2\ SBA\^2\)\/\(2\ SW2\))\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(2\ Alfa2\ MHp2\ CBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(CW2\ SW2\^2\) + 
                              S\ \((\(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                  \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc12, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc12, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(2\ Alfa2\ MHp2\ SBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(CW2\ SW2\^2\) + 
                              S\ \((\(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                  \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc12, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc12, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(2\ Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                              \(6\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                              \(Alfa2\ MHp2\ SW2\)\/CW2\^2 + 6\ Alfa2\ MHp2 + 
                              \(3\ Alfa2\ MHp2\)\/CW2 + 
                              S\ \((\(-
                                    \(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                                  \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                                  \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - 
                                  \(3\ Alfa2\)\/2 - 
                                  \(3\ Alfa2\)\/\(4\ CW2\) + 
                                  \(3\ Alfa2\)\/\(4\ SW2\) + 
                                  \(Alfa2\ CW2\)\/\(2\ SW2\) - 
                                  \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\) - 
                              \(3\ Alfa2\ MHp2\)\/SW2 - 
                              \(2\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                              \(Alfa2\ CW2\ MHp2\)\/SW2\^2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(S\ \((
                          \(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + Alfa2\/CW2 - 
                            Alfa2\/SW2 + 2\ Alfa2)\)\ 
                          \((C0i(cc0, ME2, S, ME2, 0, ME2, ME2) + 
                              C0i(cc1, ME2, S, ME2, 0, ME2, ME2) + 
                              C0i(cc2, ME2, S, ME2, 0, ME2, ME2))\)\), "+", 
                        \(S\ \((Alfa2\/SW2 - \(Alfa2\ CW2\)\/SW2\^2)\)\ 
                          \((C0i(cc1, ME2, S, ME2, 0, MW2, MW2) + 
                              C0i(cc2, ME2, S, ME2, 0, MW2, MW2))\)\), "+", 
                        \(S\ \((Alfa2\/\(2\ SW2\^2\) - Alfa2\/\(2\ CW2\ SW2\))
                          \)\ \((
                            C0i(cc1, ME2, S, ME2, MW2, 0, 0) + 
                              C0i(cc2, ME2, S, ME2, MW2, 0, 0))\)\), "+", 
                        \(S\ \((
                          \(-\(\(2\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                            \(2\ Alfa2\ SW2\)\/CW2 + 
                            \(3\ Alfa2\ SW2\)\/CW2\^2 - \(3\ Alfa2\)\/CW2 - 
                            \(3\ Alfa2\)\/\(2\ CW2\^2\) + 
                            \(3\ Alfa2\)\/\(2\ CW2\ SW2\) + 
                            Alfa2\/\(4\ CW2\^2\ SW2\) - 
                            Alfa2\/\(4\ CW2\ SW2\^2\))\)\ 
                          \((C0i(cc1, ME2, S, ME2, MZ2, ME2, ME2) + 
                              C0i(cc2, ME2, S, ME2, MZ2, ME2, ME2))\)\), "+", 
                        \(\((\(-10\)\ Alfa2\ MHp2 + 
                              \(10\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(5\ Alfa2\ MHp2\)\/CW2 + 
                              \(5\ Alfa2\ MHp2\)\/SW2 + 
                              1\/4\ \((
                                  \(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                                    \(2\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/SW2 + 
                                    4\ Alfa2)\)\ \((4\ S + T + U)\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\)\/\(4\ SW2\^2\) + 
                              \((Alfa2\/\(4\ SW2\) - Alfa2\/\(8\ SW2\^2\))\)\ 
                                \((S - T - U)\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(3\ Alfa2\ MW2\)\/\(4\ CW2\)\)\) + 
                              \(Alfa2\ MW2\)\/\(4\ SW2\) + 
                              \(3\ Alfa2\ MW2\)\/\(8\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\)\/\(8\ SW2\^2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(8\ CW2\^2\ 
                                  SW2\^2\) - 
                              \(Alfa2\ C2B\ MW2\ SBA\ SAB\)\/CW2\^2 + 
                              \(Alfa2\ C2B\ MW2\ SBA\ SAB\)\/\(CW2\ SW2\) + 
                              \(Alfa2\ C2B\ MW2\ SBA\ SAB\)\/\(2\ CW2\^2\ SW2
                                  \) - \(Alfa2\ C2B\ MW2\ SBA\ SAB\)\/\(2\ CW2
                                  \ SW2\^2\) - \(Alfa2\ MW2\ SBA\^2\)\/CW2 + 
                              \(Alfa2\ MW2\ SBA\^2\)\/SW2 + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(2\ SW2\^2\))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(4\ SW2\^2\) + 
                              \((\(Alfa2\ CBA\^2\)\/\(4\ SW2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(8\ SW2\^2\))\)\ 
                                \((S - T - U)\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(3\ Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\)\)\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ SW2\) + 
                              \(3\ Alfa2\ MW2\ CBA\^2\)\/\(8\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(8\ SW2\^2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/CW2\^2 - 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ 
                                  CW2\^2\ SW2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\ 
                                  SW2\^2\) - 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ CW2\^3\) + 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(8\ CW2\^2\ 
                                  SW2\^2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(8\ CW2\^2\ 
                                  SW2\^2\) + 
                              \(Alfa2\ C2B\ CBA\ MW2\ CAB\)\/CW2\^2 - 
                              \(Alfa2\ C2B\ CBA\ MW2\ CAB\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ C2B\ CBA\ MW2\ CAB\)\/\(2\ CW2\^2\ SW2
                                  \) + \(Alfa2\ C2B\ CBA\ MW2\ CAB\)\/\(2\ CW2
                                  \ SW2\^2\) - \(Alfa2\ CBA\^2\ MW2\)\/CW2 + 
                              \(Alfa2\ CBA\^2\ MW2\)\/SW2 + 
                              \(Alfa2\ CBA\^2\ MW2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ MW2\)\/\(2\ SW2\^2\))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(4\ SW2\^2\) + 
                              \((\(Alfa2\ SBA\^2\)\/\(4\ SW2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(8\ SW2\^2\))\)\ 
                                \((S - T - U)\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-
                                \(\(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(8\ CW2\^2\ 
                                  SW2\^2\) + 
                              \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/CW2\^2 - 
                              \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ 
                                  CW2\^2\ SW2\) + 
                              \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(4\ CW2\ 
                                  SW2\^2\) - 
                              \(3\ Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ SW2\) + 
                              \(3\ Alfa2\ MW2\ SBA\^2\)\/\(8\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(8\ SW2\^2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2))\)\), 
                        "+", \(\((
                            \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\^2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\^2\ SW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) + 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^3\) - 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(8\ 
                                  CW2\^3\ SW2\) + 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(8\ CW2\^2\ 
                                  SW2\^2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHp2, Mh02, MZ2) + 
                              C0i(cc1, MHp2, S, MHp2, MHp2, MZ2, Mh02) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, Mh02, MZ2) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, MZ2, Mh02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\^2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\^2\ SW2\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^3\) + 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(8\ 
                                  CW2\^3\ SW2\) - 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(8\ CW2\^2\ 
                                  SW2\^2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHp2, MHH2, MZ2) + 
                              C0i(cc1, MHp2, S, MHp2, MHp2, MZ2, MHH2) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, MHH2, MZ2) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, MZ2, MHH2))\)\), 
                        "+", \(\((
                            \(5\ Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(5\ Alfa2\ MHp2\ CBA\^2\)\/\(4\ CW2\ 
                                  SW2\^2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) + 
                              1\/4\ \((
                                  \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\^2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(CW2\ SW2\))\)\ 
                                \((4\ S + T + U)\))\)\ 
                          \((C0i(cc2, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc2, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(5\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(5\ Alfa2\ MHp2\ SBA\^2\)\/\(4\ CW2\ 
                                  SW2\^2\) + 
                              \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(8\ 
                                  CW2\^2\ SW2\^2\) + 
                              1\/4\ \((
                                  \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\^2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(CW2\ SW2\))\)\ 
                                \((4\ S + T + U)\))\)\ 
                          \((C0i(cc2, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc2, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(5\ Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) + 
                              \(Alfa2\ MZ2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                              \(15\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) - 
                              \(5\ Alfa2\ MHp2\ SW2\)\/\(4\ CW2\^2\) - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/\(4\ CW2\) - 
                              \(Alfa2\ MZ2\ SW2\)\/\(8\ CW2\^2\) + 
                              \(15\ Alfa2\ MHp2\)\/2 + 
                              \(15\ Alfa2\ MHp2\)\/\(4\ CW2\) + 
                              \(3\ Alfa2\ MZ2\)\/4 + 
                              \(3\ Alfa2\ MZ2\)\/\(8\ CW2\) + 
                              1\/4\ \((
                                  \(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                                    \(3\ Alfa2\ SW2\)\/CW2 + 
                                    \(Alfa2\ SW2\)\/\(2\ CW2\^2\) - 
                                    3\ Alfa2 - \(3\ Alfa2\)\/\(2\ CW2\) + 
                                    \(3\ Alfa2\)\/\(2\ SW2\) + 
                                    \(Alfa2\ CW2\)\/SW2 - 
                                    \(Alfa2\ CW2\)\/\(2\ SW2\^2\))\)\ 
                                \((4\ S + T + U)\) - 
                              \(15\ Alfa2\ MHp2\)\/\(4\ SW2\) - 
                              \(5\ Alfa2\ CW2\ MHp2\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MZ2\)\/\(8\ SW2\) - 
                              \(Alfa2\ CW2\ MZ2\)\/\(4\ SW2\) + 
                              \(5\ Alfa2\ CW2\ MHp2\)\/\(4\ SW2\^2\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(8\ SW2\^2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((\(-6\)\ Alfa2\ MHp2 + 
                              \(6\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                              \(3\ Alfa2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ MHp2\)\/SW2 + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + 
                                    Alfa2\/CW2 - Alfa2\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(3\ Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\)\/\(4\ SW2\^2\) + 
                              \((MHp2 + S\/2)\)\ 
                                \((Alfa2\/\(4\ SW2\^2\) - Alfa2\/\(2\ SW2\))
                                  \))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(3\ Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\ CBA\^2\)\/\(4\ SW2\^2\) + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(Alfa2\ CBA\^2\)\/\(4\ SW2\^2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(2\ SW2\))\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(4\ SW2\^2\) + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(Alfa2\ SBA\^2\)\/\(4\ SW2\^2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(2\ SW2\))\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(3\ Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\ CBA\^2\)\/\(4\ CW2\ 
                                  SW2\^2\) + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(Alfa2\ CBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                    \(Alfa2\ CBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc22, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc22, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(4\ CW2\ 
                                  SW2\^2\) + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(Alfa2\ SBA\^2\)\/\(4\ CW2\ SW2\^2\) - 
                                    \(Alfa2\ SBA\^2\)\/\(2\ CW2\ SW2\))\))\)\ 
                          \((C0i(cc22, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc22, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) - 
                              \(9\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) - 
                              \(3\ Alfa2\ MHp2\ SW2\)\/\(4\ CW2\^2\) + 
                              \(9\ Alfa2\ MHp2\)\/2 + 
                              \(9\ Alfa2\ MHp2\)\/\(4\ CW2\) + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(-
                                      \(\(Alfa2\ SW2\^2\)\/\(2\ 
                                        CW2\^2\)\)\) + 
                                    \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                                    \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - 
                                    \(3\ Alfa2\)\/2 - 
                                    \(3\ Alfa2\)\/\(4\ CW2\) + 
                                    \(3\ Alfa2\)\/\(4\ SW2\) + 
                                    \(Alfa2\ CW2\)\/\(2\ SW2\) - 
                                    \(Alfa2\ CW2\)\/\(4\ SW2\^2\))\) - 
                              \(9\ Alfa2\ MHp2\)\/\(4\ SW2\) - 
                              \(3\ Alfa2\ CW2\ MHp2\)\/\(2\ SW2\) + 
                              \(3\ Alfa2\ CW2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", 
                        RowBox[{
                        \((\(2\ \[Pi]\ SW2\ Alfa\)\/CW2 + 
                              \(\[Pi]\ Alfa\)\/SW2 - \(\[Pi]\ Alfa\)\/CW2 - 
                              2\ \[Pi]\ Alfa)\), " ", 
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{
                                \(dZfL1(2, 1, 1)\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\^"*"\),
                                    Superscript[ "*"],
                                    Editable->False]}],
                                SequenceForm[ 
                                  dZfL1[ 2, 1, 1], 
                                  Superscript[ "*"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}], "-", 
                        \(2\ Alfa\ \[Pi]\ \(dZfL1(2, 1, 1)\)\), "-", 
                        \(\(Alfa\ \[Pi]\ \(dZfL1(2, 1, 1)\)\)\/CW2\), "-", 
                        \(Alfa2\/CW2\), "+", 
                        \(\(2\ Alfa\ CW2\ dZAZ1\ \[Pi]\)\/\(CW\ SW\)\), "+", 
                        \(\(\(-\(\(4\ Alfa\ \[Pi]\ SW2\ dSW1\)\/CW2\)\) - 
                              \(2\ Alfa\ \[Pi]\ dSW1\)\/SW2 + 
                              4\ Alfa\ \[Pi]\ dSW1 + 
                              \(2\ Alfa\ \[Pi]\ dSW1\)\/CW2\)\/\(CW2\ SW\ SW2
                                 - CW2\^2\ SW\)\), "-", 
                        \(\(3\ Alfa2\)\/\(4\ CW2\^2\)\), "-", 
                        \(8\ Alfa\ dZe1\ \[Pi]\), "-", 
                        \(\(4\ Alfa\ dZe1\ \[Pi]\)\/CW2\), "-", 
                        \(4\ Alfa\ dZGp1\ \[Pi]\), "-", 
                        \(\(2\ Alfa\ dZGp1\ \[Pi]\)\/CW2\), "-", 
                        \(4\ Alfa\ dZZZ1\ \[Pi]\), "-", 
                        \(\(2\ Alfa\ dZZZ1\ \[Pi]\)\/CW2\), "+", 
                        \(\(Alfa\ \[Pi]\ \(dZfL1(2, 1, 1)\)\)\/SW2\), "+", 
                        \(Alfa2\/\(2\ CW2\ SW2\)\), "+", 
                        \(\(2\ Alfa\ dSW1\ \[Pi]\)\/\(SW\ SW2\)\), "+", 
                        \(Alfa2\/\(8\ CW2\^2\ SW2\)\), "+", 
                        \(\(4\ Alfa\ dZe1\ \[Pi]\)\/SW2\), "+", 
                        \(\(2\ Alfa\ dZGp1\ \[Pi]\)\/SW2\), "+", 
                        \(\(2\ Alfa\ dZZZ1\ \[Pi]\)\/SW2\), "+", 
                        \(Alfa2\/\(4\ SW2\^2\)\), "-", 
                        \(\(Alfa2\ CW2\)\/\(2\ SW2\^2\)\), "-", 
                        \(Alfa2\/\(8\ CW2\ SW2\^2\)\)}], ")"}]}]}], ")"}], 
              " ", \(Mat(F1)\)}], "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\(Den(S, 0)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                      \(-\(\(Alfa2\ \(B0(MHp2, Mh02, MW2)\)\ CBA\^2\)\/\(2\ 
                                SW2\)\)\), "+", 
                        \(\(Alfa2\ \(B1(MHp2, Mh02, MW2)\)\ CBA\^2\)\/\(2\ SW2
                              \)\), "-", 
                        \(\(Alfa2\ 
                              \(C0i(cc00, MHp2, S, MHp2, Mh02, MW2, MW2)\)\ 
                              CBA\^2\)\/SW2\), "-", \(2\ Alfa2\), "-", 
                        \(\(4\ Alfa\ dZZA1\ \[Pi]\ SW\)\/CW\), "-", 
                        \(\(2\ Alfa2\ SW2\)\/CW2\), "-", 
                        \(\(2\ Alfa\ dZZA1\ \[Pi]\ SW2\)\/\(CW\ SW\)\), "-", 
                        \(4\ Alfa2\ \(B0(MHp2, 0, MHp2)\)\), "-", 
                        \(\(Alfa2\ \(B0(MHp2, MA02, MW2)\)\)\/\(2\ SW2\)\), 
                        "-", \(\(Alfa2\ SBA\^2\ \(B0(MHp2, MHH2, MW2)\)\)\/\(2
                              \ SW2\)\), "+", 
                        \(\((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                              \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\)\ 
                          \(B0(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(4\ SW2\ Alfa2\)\/CW2 + 4\ Alfa2)\)\ 
                          \(B0(S, ME2, ME2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\)\/SW2 + 
                              \(Alfa2\ SBA\^2\)\/SW2 + Alfa2\/SW2)\)\ 
                          \(B0(S, MW2, MW2)\)\), "+", 
                        \(\(Alfa2\ \(B1(MHp2, MA02, MW2)\)\)\/\(2\ SW2\)\), 
                        "+", \(\(Alfa2\ SBA\^2\ \(B1(MHp2, MHH2, MW2)\)\)\/\(2
                              \ SW2\)\), "+", 
                        \(4\ Alfa2\ \(B1(MHp2, MHp2, 0)\)\), "+", 
                        \(\((\(SW2\ Alfa2\)\/CW2 + \(CW2\ Alfa2\)\/SW2 - 
                              2\ Alfa2)\)\ \(B1(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(4\ Alfa2\ MZ2\ SW2\)\/CW2 - 
                              \(4\ Alfa2\ S\ SW2\)\/CW2)\)\ 
                          \(C0i(cc0, ME2, S, ME2, MZ2, ME2, ME2)\)\), "+", 
                        \(\((8\ Alfa2\ MHp2 - 4\ Alfa2\ S)\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MA02\)\/SW2 + \(Alfa2\ S\)\/\(4\ SW2\))
                            \)\ \(C0i(cc0, MHp2, S, MHp2, MA02, MW2, MW2)\)\),
                         "+", \(\((
                            \(Alfa2\ Mh02\ CBA\^2\)\/SW2 + 
                              \(Alfa2\ S\ CBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\ MW2\ S2B\ SAB\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(2\ CW2\^2\ SW2
                                  \))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(2\ CW2\^2\ SW2
                                  \))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ MHH2\ SBA\^2\)\/SW2 + 
                              \(Alfa2\ S\ SBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-4\)\ Alfa2\ MHp2 + 
                              \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(2\ Alfa2\ CW2\ MHp2\)\/SW2 - Alfa2\ MZ2 + 
                              \(Alfa2\ MZ2\ SW2\)\/\(2\ CW2\) + 
                              S\ \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                  \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "-",
                         \(8\ Alfa2\ 
                          \(C0i(cc00, ME2, S, ME2, 0, ME2, ME2)\)\), "-", 
                        \(\(8\ Alfa2\ SW2\ 
                              \(C0i(cc00, ME2, S, ME2, MZ2, ME2, ME2)\)\)\/CW2
                            \), "+", 
                        \(8\ Alfa2\ 
                          \(C0i(cc00, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "-", 
                        \(\(Alfa2\ 
                              \(C0i(cc00, MHp2, S, MHp2, MA02, MW2, MW2)
                                \)\)\/SW2\), "-", 
                        \(\(Alfa2\ SBA\^2\ 
                              \(C0i(cc00, MHp2, S, MHp2, MHH2, MW2, MW2)
                                \)\)\/SW2\), "+", 
                        \(\((\(2\ SW2\ Alfa2\)\/CW2 + 
                              \(2\ CW2\ Alfa2\)\/SW2 - 4\ Alfa2)\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((
                            12\ Alfa2\ MHp2 - 
                              4\ Alfa2\ \((\(3\ S\)\/2 - MHp2)\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S)\)\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ \((MHp2 + S)\)\)\/\(2\ SW2\))
                            \)\ \(C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2)\)\),
                         "+", \(\((
                            \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S)\)\ SBA\^2\)\/\(2\ SW2\))
                            \)\ \(C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2)\)\),
                         "+", \(\((
                            \(-6\)\ Alfa2\ MHp2 + 
                              \(3\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ CW2\ MHp2\)\/SW2 - Alfa2\ MZ2 + 
                              \(Alfa2\ MZ2\ SW2\)\/\(2\ CW2\) + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                    \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((4\ Alfa2\ MHp2 + 2\ Alfa2\ \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MHp2\)\/\(2\ SW2\)\)\) - 
                              \(Alfa2\ \((T + U)\)\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(-
                                \(\(Alfa2\ \((T + U)\)\ CBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(-
                                \(\(Alfa2\ \((T + U)\)\ SBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(-2\)\ Alfa2\ MHp2 + 
                              \(Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(Alfa2\ CW2\ MHp2\)\/SW2 + 
                              \((\(SW2\ Alfa2\)\/\(2\ CW2\) + 
                                    \(CW2\ Alfa2\)\/\(2\ SW2\) - Alfa2)\)\ 
                                \((T + U)\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((16\ Alfa2\ MHp2 - 4\ Alfa2\ S)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ S\)\/\(2\ SW2\) - 
                              \(2\ Alfa2\ MHp2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ CBA\^2\ S\)\/\(2\ SW2\) - 
                              \(2\ Alfa2\ CBA\^2\ MHp2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ S\ SBA\^2\)\/\(2\ SW2\) - 
                              \(2\ Alfa2\ MHp2\ SBA\^2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(-8\)\ Alfa2\ MHp2 + 
                              \(4\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(4\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                              S\ \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                  \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "-", \(4\ Alfa2\ S\ \((
                          C0i(cc0, ME2, S, ME2, 0, ME2, ME2) + 
                            C0i(cc1, ME2, S, ME2, 0, ME2, ME2) + 
                            C0i(cc2, ME2, S, ME2, 0, ME2, ME2))\)\), "-", 
                        \(\(4\ Alfa2\ S\ SW2\ \((
                              C0i(cc1, ME2, S, ME2, MZ2, ME2, ME2) + 
                                C0i(cc2, ME2, S, ME2, MZ2, ME2, ME2))\)\)\/CW2
                            \), "+", 
                        \(\((20\ Alfa2\ MHp2 - 2\ Alfa2\ \((4\ S + T + U)\))
                            \)\ \(C0i(cc2, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ MHp2\)\/\(2\ SW2\)\)\) - 
                              \(Alfa2\ \((S - T - U)\)\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "-", 
                        \(1\/SW2\((
                          Alfa2\ MW2\ \((
                            C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2))\))\)\),
                         "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(2\ CW2\^2\ 
                                      SW2\)\)\) - 
                              \(2\ Alfa2\ C2B\ MW2\ SBA\ SAB\)\/\(CW2\ SW2
                                  \) - \(2\ Alfa2\ MW2\ SBA\^2\)\/SW2)\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ \((S - T - U)\)\ CBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MW2\ CBA\^2\)\/SW2\)\) + 
                              \(3\ Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(2\ 
                                  CW2\^2\ SW2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(2\ CW2\^2\ 
                                      SW2\)\)\) + 
                              \(2\ Alfa2\ C2B\ CBA\ MW2\ CAB\)\/\(CW2\ SW2
                                  \) - \(2\ Alfa2\ CBA\^2\ MW2\)\/SW2)\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ \((S - T - U)\)\ SBA\^2\)\/\(4\ SW2
                                      \)\)\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(2\ 
                                      CW2\^2\ SW2\)\)\) + 
                              \(3\ Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ SBA\^2\)\/SW2)\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-10\)\ Alfa2\ MHp2 + 
                              \(5\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(5\ Alfa2\ CW2\ MHp2\)\/SW2 - Alfa2\ MZ2 + 
                              \(Alfa2\ MZ2\ SW2\)\/\(2\ CW2\) + 
                              1\/4\ \((
                                  \(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) - 
                                    \(2\ CW2\ Alfa2\)\/SW2 + 4\ Alfa2)\)\ 
                                \((4\ S + T + U)\) + 
                              \(Alfa2\ CW2\ MZ2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((12\ Alfa2\ MHp2 - 4\ Alfa2\ \((MHp2 + S\/2)\))
                            \)\ \(C0i(cc22, MHp2, S, MHp2, 0, MHp2, MHp2)\)\),
                         "+", \(\((
                            \(Alfa2\ \((MHp2 + S\/2)\)\)\/\(2\ SW2\) - 
                              \(3\ Alfa2\ MHp2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ CBA\^2\ \((MHp2 + S\/2)\)\)\/\(2\ SW2
                                  \) - \(3\ Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\))
                            \)\ \(C0i(cc22, MHp2, S, MHp2, Mh02, MW2, MW2)
                            \)\), "+", 
                        \(\((\(Alfa2\ \((MHp2 + S\/2)\)\ SBA\^2\)\/\(2\ SW2
                                  \) - \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))
                            \)\ \(C0i(cc22, MHp2, S, MHp2, MHH2, MW2, MW2)
                            \)\), "+", 
                        \(\((\(-6\)\ Alfa2\ MHp2 + 
                              \(3\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \(3\ Alfa2\ CW2\ MHp2\)\/SW2 + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) - 
                                    \(CW2\ Alfa2\)\/SW2 + 2\ Alfa2)\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", 
                        RowBox[{"4", " ", "Alfa", " ", "\[Pi]", " ", 
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{
                                \(dZfR1(2, 1, 1)\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\^"*"\),
                                    Superscript[ "*"],
                                    Editable->False]}],
                                SequenceForm[ 
                                  dZfR1[ 2, 1, 1], 
                                  Superscript[ "*"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}], "+", 
                        \(4\ Alfa\ \[Pi]\ \(dZfR1(2, 1, 1)\)\), "+", 
                        \(\(2\ Alfa\ CW2\ dZZA1\ \[Pi]\)\/\(CW\ SW\)\), "+", 
                        \(8\ Alfa\ dZAA1\ \[Pi]\), "+", 
                        \(16\ Alfa\ dZe1\ \[Pi]\), "+", 
                        \(8\ Alfa\ dZGp1\ \[Pi]\)}], ")"}]}], "+", 
                  RowBox[{\(Den(S, MZ2)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                      \(-\(\(Alfa2\ \(B0(MHp2, Mh02, MW2)\)\ CBA\^2\)\/\(2\ 
                                CW2\)\)\), "+", 
                        \(\(Alfa2\ \(B1(MHp2, Mh02, MW2)\)\ CBA\^2\)\/\(2\ CW2
                              \)\), "+", 
                        \(\(Alfa2\ 
                              \(C0i(cc00, MHp2, S, MHp2, Mh02, MW2, MW2)\)\ 
                              CBA\^2\)\/SW2\), "+", 
                        \(1\/\(CW2\ SW2\)\((
                          Alfa2\ \((
                            C0i(cc00, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc00, MHp2, S, MHp2, MW2, Mh02, MA02))\)\ 
                            CBA\^2)\)\), "-", \(\(Alfa2\ SW2\^2\)\/CW2\^2\), 
                        "+", "Alfa2", "-", 
                        \(\(4\ Alfa\ dSW1\ \[Pi]\ SW2\)\/\(CW2\^2\ SW\)\), 
                        "-", \(\(2\ Alfa\ dZAZ1\ \[Pi]\ SW2\)\/\(CW\ SW\)\), 
                        "+", \(\(8\ Alfa\ dZe1\ \[Pi]\ SW2\)\/CW2\), "+", 
                        \(\(4\ Alfa\ dZGp1\ \[Pi]\ SW2\)\/CW2\), "+", 
                        \(\(4\ Alfa\ dZZZ1\ \[Pi]\ SW2\)\/CW2\), "+", 
                        \(\(\(4\ Alfa\ CW2\ dZAZ1\ \[Pi]\ SW\)\/CW - 
                              \(4\ Alfa\ dZAZ1\ \[Pi]\ SW\ SW2\)\/CW\)\/\(SW2
                               - CW2\)\), "+", 
                        \(\((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\)\ 
                          \(B0(MHp2, 0, MHp2)\)\), "-", 
                        \(\(Alfa2\ \(B0(MHp2, MA02, MW2)\)\)\/\(2\ CW2\)\), 
                        "-", \(\(Alfa2\ SBA\^2\ \(B0(MHp2, MHH2, MW2)\)\)\/\(2
                              \ CW2\)\), "+", 
                        \(\((\(-\(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - 
                              \(3\ Alfa2\)\/2 + \(Alfa2\ CW2\)\/\(2\ SW2\))
                            \)\ \(B0(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(2\ Alfa2\ SW2\^2\)\/CW2\^2 - 2\ Alfa2)\)\ 
                          \(B0(S, ME2, ME2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ CBA\^2\)\/SW2\)\) - 
                              \(Alfa2\ SBA\^2\)\/SW2 - Alfa2\/SW2)\)\ 
                          \(B0(S, MW2, MW2)\)\), "+", 
                        \(\(Alfa2\ \(B1(MHp2, MA02, MW2)\)\)\/\(2\ CW2\)\), 
                        "+", 
                        \(\(Alfa2\ SBA\^2\ \(B1(MHp2, MHH2, MW2)\)\)\/\(2\ CW2
                              \)\), "+", 
                        \(\((\(2\ Alfa2\ SW2\)\/CW2 - 2\ Alfa2)\)\ 
                          \(B1(MHp2, MHp2, 0)\)\), "+", 
                        \(\((\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ SW2\)\/\(2\ CW2\) + 
                              \(3\ Alfa2\)\/2 - \(Alfa2\ CW2\)\/\(2\ SW2\))
                            \)\ \(B1(MHp2, MHp2, MZ2)\)\), "+", 
                        \(\((\(2\ Alfa2\ MZ2\ SW2\^2\)\/CW2\^2 - 
                              \(2\ Alfa2\ MZ2\ SW2\)\/CW2 + 
                              S\ \((\(2\ Alfa2\ SW2\)\/CW2 - 
                                  \(2\ Alfa2\ SW2\^2\)\/CW2\^2)\))\)\ 
                          \(C0i(cc0, ME2, S, ME2, MZ2, ME2, ME2)\)\), "+", 
                        \(\((\(-4\)\ Alfa2\ MHp2 + 
                              \(4\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              S\ \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MW2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MA02\)\/SW2\)\) - 
                              \(Alfa2\ S\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ Mh02\ CBA\^2\)\/SW2\)\) - 
                              \(Alfa2\ S\ CBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ SW2\) - 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ 
                                  CW2\^3\) + 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(-
                                \(\(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ CW2
                                  \ SW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MHH2\ SBA\^2\)\/SW2\)\) - 
                              \(Alfa2\ S\ SBA\^2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^3\) + 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHp2, Mh02, MZ2) + 
                              C0i(cc0, MHp2, S, MHp2, MHp2, MZ2, Mh02))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^3\) - 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHp2, MHH2, MZ2) + 
                              C0i(cc0, MHp2, S, MHp2, MHp2, MZ2, MHH2))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ CBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ S\ CBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc0, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ S\ SBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc0, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 + 
                              \(Alfa2\ MZ2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                              \(3\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/\(4\ CW2\) + 
                              3\ Alfa2\ MHp2 + \(3\ Alfa2\ MZ2\)\/4 + 
                              S\ \((\(-
                                    \(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                                  \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - 
                                  \(3\ Alfa2\)\/2 + 
                                  \(Alfa2\ CW2\)\/\(2\ SW2\))\) - 
                              \(Alfa2\ CW2\ MHp2\)\/SW2 - 
                              \(Alfa2\ CW2\ MZ2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc0, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((4\ Alfa2 - \(4\ Alfa2\ SW2\)\/CW2)\)\ 
                          \(C0i(cc00, ME2, S, ME2, 0, ME2, ME2)\)\), "+", 
                        \(\((\(4\ Alfa2\ SW2\)\/CW2 - 
                              \(4\ Alfa2\ SW2\^2\)\/CW2\^2)\)\ 
                          \(C0i(cc00, ME2, S, ME2, MZ2, ME2, ME2)\)\), "+", 
                        \(\((\(4\ Alfa2\ SW2\)\/CW2 - 4\ Alfa2)\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\(Alfa2\ 
                              \(C0i(cc00, MHp2, S, MHp2, MA02, MW2, MW2)
                                \)\)\/SW2\), "+", 
                        \(\(Alfa2\ SBA\^2\ 
                              \(C0i(cc00, MHp2, S, MHp2, MHH2, MW2, MW2)
                                \)\)\/SW2\), "+", 
                        \(1\/\(CW2\ SW2\)\((
                          Alfa2\ SBA\^2\ \((
                            C0i(cc00, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc00, MHp2, S, MHp2, MW2, MHH2, MA02))\))
                          \)\), "+", 
                        \(\((\(Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(3\ Alfa2\ SW2\)\/CW2 + 3\ Alfa2 - 
                              \(Alfa2\ CW2\)\/SW2)\)\ 
                          \(C0i(cc00, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((
                            \(-6\)\ Alfa2\ MHp2 + 
                              \(6\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ \((MHp2 + S)\)\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ CBA\^2\ \((MHp2 + S)\)\)\/\(2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ \((MHp2 + S)\)\ SBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(3\ Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ \((\(3\ S\)\/2 - MHp2)\)\ CBA\^2\)\/\(2
                                  \ CW2\ SW2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc1, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ \((\(3\ S\)\/2 - MHp2)\)\ SBA\^2\)\/\(2
                                  \ CW2\ SW2\) + 
                              \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc1, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) + 
                              \(Alfa2\ MZ2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                              \(9\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/\(4\ CW2\) + 
                              \(9\ Alfa2\ MHp2\)\/2 + \(3\ Alfa2\ MZ2\)\/4 + 
                              \((\(3\ S\)\/2 - MHp2)\)\ 
                                \((\(-
                                      \(\(Alfa2\ SW2\^2\)\/\(2\ 
                                        CW2\^2\)\)\) + 
                                    \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - 
                                    \(3\ Alfa2\)\/2 + 
                                    \(Alfa2\ CW2\)\/\(2\ SW2\))\) - 
                              \(3\ Alfa2\ CW2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CW2\ MZ2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc1, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((\(-2\)\ Alfa2\ MHp2 + 
                              \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \((\(Alfa2\ SW2\)\/CW2 - Alfa2)\)\ \((T + U)\))
                            \)\ \(C0i(cc11, MHp2, S, MHp2, 0, MHp2, MHp2)\)\),
                         "+", \(\((
                            \(Alfa2\ MHp2\)\/\(2\ SW2\) + 
                              \(Alfa2\ \((T + U)\)\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ \((T + U)\)\ CBA\^2\)\/\(4\ SW2\) + 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ \((T + U)\)\ SBA\^2\)\/\(4\ SW2\) + 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(Alfa2\ \((T + U)\)\ CBA\^2\)\/\(4\ CW2\ SW2
                                  \) + \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2
                                  \))\)\ 
                          \((C0i(cc11, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc11, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ \((T + U)\)\ SBA\^2\)\/\(4\ CW2\ SW2\) + 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\))\)\ 
                          \((C0i(cc11, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc11, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) - 
                              \(3\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) + 
                              \(3\ Alfa2\ MHp2\)\/2 + 
                              \((\(Alfa2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                                    \(3\ Alfa2\ SW2\)\/\(4\ CW2\) + 
                                    \(3\ Alfa2\)\/4 - 
                                    \(Alfa2\ CW2\)\/\(4\ SW2\))\)\ 
                                \((T + U)\) - 
                              \(Alfa2\ CW2\ MHp2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc11, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(\((
                            \(-8\)\ Alfa2\ MHp2 + 
                              \(8\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              S\ \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(2\ Alfa2\ MHp2\)\/SW2 - 
                              \(Alfa2\ S\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(2\ Alfa2\ CBA\^2\ MHp2\)\/SW2 - 
                              \(Alfa2\ CBA\^2\ S\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(2\ Alfa2\ MHp2\ SBA\^2\)\/SW2 - 
                              \(Alfa2\ S\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(2\ Alfa2\ CBA\^2\ MHp2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ S\)\/\(2\ CW2\ SW2\))\)\ 
                          \((C0i(cc12, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc12, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(2\ Alfa2\ MHp2\ SBA\^2\)\/\(CW2\ SW2\) - 
                              \(Alfa2\ S\ SBA\^2\)\/\(2\ CW2\ SW2\))\)\ 
                          \((C0i(cc12, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc12, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(2\ Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                              \(6\ Alfa2\ MHp2\ SW2\)\/CW2 + 6\ Alfa2\ MHp2 + 
                              S\ \((\(-
                                    \(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                                  \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - 
                                  \(3\ Alfa2\)\/2 + 
                                  \(Alfa2\ CW2\)\/\(2\ SW2\))\) - 
                              \(2\ Alfa2\ CW2\ MHp2\)\/SW2)\)\ 
                          \(C0i(cc12, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", \(S\ \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\)\ 
                          \((C0i(cc0, ME2, S, ME2, 0, ME2, ME2) + 
                              C0i(cc1, ME2, S, ME2, 0, ME2, ME2) + 
                              C0i(cc2, ME2, S, ME2, 0, ME2, ME2))\)\), "+", 
                        \(S\ \((
                          \(2\ Alfa2\ SW2\)\/CW2 - 
                            \(2\ Alfa2\ SW2\^2\)\/CW2\^2)\)\ 
                          \((C0i(cc1, ME2, S, ME2, MZ2, ME2, ME2) + 
                              C0i(cc2, ME2, S, ME2, MZ2, ME2, ME2))\)\), "+", 
                        \(\((\(-10\)\ Alfa2\ MHp2 + 
                              \(10\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              1\/4\ \((4\ Alfa2 - \(4\ Alfa2\ SW2\)\/CW2)\)\ 
                                \((4\ S + T + U)\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(Alfa2\ MHp2\)\/\(2\ SW2\) + 
                              \(Alfa2\ \((S - T - U)\)\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+", 
                        \(\((\(Alfa2\ MW2\)\/\(4\ SW2\) - 
                              \(3\ Alfa2\ MW2\)\/\(4\ CW2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MA02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MA02, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ C2B\^2\ MW2\ SAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \) - \(Alfa2\ C2B\ MW2\ SBA\ SAB\)\/CW2\^2
                                   + \(Alfa2\ C2B\ MW2\ SBA\ SAB\)\/\(CW2\ SW2
                                  \) - \(Alfa2\ MW2\ SBA\^2\)\/CW2 + 
                              \(Alfa2\ MW2\ SBA\^2\)\/SW2)\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, Mh02, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MHp2, MHp2))\)\), 
                        "+", 
                        \(\((\(Alfa2\ \((S - T - U)\)\ CBA\^2\)\/\(4\ SW2\) + 
                              \(Alfa2\ MHp2\ CBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+", 
                        \(\((\(-\(\(3\ Alfa2\ MW2\ CBA\^2\)\/\(4\ CW2\)\)\) + 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(4\ SW2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/CW2\^2 - 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(2\ CW2\ SW2
                                  \) - \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ 
                                  CW2\^3\) + 
                              \(Alfa2\ MW2\ S2B\^2\ SAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, Mh02, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, Mh02, MW2, MW2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ C2B\^2\ MW2\ CAB\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ C2B\ CBA\ MW2\ CAB\)\/CW2\^2
                                   - \(Alfa2\ C2B\ CBA\ MW2\ CAB\)\/\(CW2\ SW2
                                  \) - \(Alfa2\ CBA\^2\ MW2\)\/CW2 + 
                              \(Alfa2\ CBA\^2\ MW2\)\/SW2)\)\ 
                          \((C0i(cc0, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc1, MHp2, S, MHp2, MHH2, MHp2, MHp2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MHp2, MHp2))\)\), 
                        "+", \(\((
                            \(Alfa2\ \((S - T - U)\)\ SBA\^2\)\/\(4\ SW2\) + 
                              \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+", 
                        \(\((\(-
                                \(\(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ 
                                      CW2\^3\)\)\) + 
                              \(Alfa2\ CAB\^2\ MW2\ S2B\^2\)\/\(4\ CW2\^2\ SW2
                                  \) + \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/CW2\^2
                                   - \(Alfa2\ CAB\ MW2\ SBA\ S2B\)\/\(2\ CW2\ 
                                  SW2\) - 
                              \(3\ Alfa2\ MW2\ SBA\^2\)\/\(4\ CW2\) + 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(4\ SW2\))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHH2, MW2, MW2) + 
                              C0i(cc2, MHp2, S, MHp2, MHH2, MW2, MW2))\)\), 
                        "+", \(\((
                            \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\^2\) - 
                              \(Alfa2\ MW2\ SBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^3\) - 
                              \(Alfa2\ C2B\ MW2\ SAB\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHp2, Mh02, MZ2) + 
                              C0i(cc1, MHp2, S, MHp2, MHp2, MZ2, Mh02) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, Mh02, MZ2) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, MZ2, Mh02))\)\), 
                        "+", \(\((
                            \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\^2\) - 
                              \(Alfa2\ MW2\ CBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^3\) + 
                              \(Alfa2\ C2B\ CAB\ MW2\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc1, MHp2, S, MHp2, MHp2, MHH2, MZ2) + 
                              C0i(cc1, MHp2, S, MHp2, MHp2, MZ2, MHH2) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, MHH2, MZ2) + 
                              C0i(cc2, MHp2, S, MHp2, MHp2, MZ2, MHH2))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ \((4\ S + T + U)\)\ CBA\^2\)\/\(4\ 
                                      CW2\ SW2\)\)\) + 
                              \(5\ Alfa2\ MHp2\ CBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ MW2\ S2B\ SAB\ CBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc2, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc2, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(-\(\(Alfa2\ \((4\ S + T + U)\)\ SBA\^2\)\/\(4\ 
                                      CW2\ SW2\)\)\) + 
                              \(5\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) + 
                              \(Alfa2\ CAB\ MW2\ S2B\ SBA\)\/\(4\ CW2\^2\ SW2
                                  \))\)\ 
                          \((C0i(cc2, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc2, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(5\ Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) + 
                              \(Alfa2\ MZ2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                              \(15\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) - 
                              \(3\ Alfa2\ MZ2\ SW2\)\/\(4\ CW2\) + 
                              \(15\ Alfa2\ MHp2\)\/2 + \(3\ Alfa2\ MZ2\)\/4 + 
                              1\/4\ \((
                                  \(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                                    \(3\ Alfa2\ SW2\)\/CW2 - 3\ Alfa2 + 
                                    \(Alfa2\ CW2\)\/SW2)\)\ 
                                \((4\ S + T + U)\) - 
                              \(5\ Alfa2\ CW2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CW2\ MZ2\)\/\(4\ SW2\))\)\ 
                          \(C0i(cc2, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), "+",
                         \(\((\(-6\)\ Alfa2\ MHp2 + 
                              \(6\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                              \((MHp2 + S\/2)\)\ 
                                \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, 0, MHp2, MHp2)\)\), "+", 
                        \(\((\(3\ Alfa2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S\/2)\)\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MA02, MW2, MW2)\)\), "+",
                         \(\((\(3\ Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ \((MHp2 + S\/2)\)\)\/\(2\ SW2
                                  \))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, Mh02, MW2, MW2)\)\), "+",
                         \(\((\(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S\/2)\)\ SBA\^2\)\/\(2\ SW2
                                  \))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MHH2, MW2, MW2)\)\), "+",
                         \(\((\(3\ Alfa2\ CBA\^2\ MHp2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ CBA\^2\ \((MHp2 + S\/2)\)\)\/\(2\ CW2\ 
                                  SW2\))\)\ 
                          \((C0i(cc22, MHp2, S, MHp2, MW2, MA02, Mh02) + 
                              C0i(cc22, MHp2, S, MHp2, MW2, Mh02, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SBA\^2\)\/\(2\ CW2\ SW2\) - 
                              \(Alfa2\ \((MHp2 + S\/2)\)\ SBA\^2\)\/\(2\ CW2\ 
                                  SW2\))\)\ 
                          \((C0i(cc22, MHp2, S, MHp2, MW2, MA02, MHH2) + 
                              C0i(cc22, MHp2, S, MHp2, MW2, MHH2, MA02))\)\), 
                        "+", \(\((
                            \(3\ Alfa2\ MHp2\ SW2\^2\)\/\(2\ CW2\^2\) - 
                              \(9\ Alfa2\ MHp2\ SW2\)\/\(2\ CW2\) + 
                              \(9\ Alfa2\ MHp2\)\/2 + 
                              \((MHp2 + S\/2)\)\ 
                                \((\(-
                                      \(\(Alfa2\ SW2\^2\)\/\(2\ 
                                        CW2\^2\)\)\) + 
                                    \(3\ Alfa2\ SW2\)\/\(2\ CW2\) - 
                                    \(3\ Alfa2\)\/2 + 
                                    \(Alfa2\ CW2\)\/\(2\ SW2\))\) - 
                              \(3\ Alfa2\ CW2\ MHp2\)\/\(2\ SW2\))\)\ 
                          \(C0i(cc22, MHp2, S, MHp2, MZ2, MHp2, MHp2)\)\), 
                        "+", 
                        RowBox[{
                        \((\(2\ Alfa\ \[Pi]\ SW2\)\/CW2 - 2\ Alfa\ \[Pi])\), 
                          " ", 
                          RowBox[{"(", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{
                                \(dZfR1(2, 1, 1)\), "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\^"*"\),
                                    Superscript[ "*"],
                                    Editable->False]}],
                                SequenceForm[ 
                                  dZfR1[ 2, 1, 1], 
                                  Superscript[ "*"]],
                                Editable->False],
                              "TraditionalForm"], ")"}]}], "+", 
                        \(\(2\ Alfa\ \[Pi]\ SW2\ \(dZfR1(2, 1, 1)\)\)\/CW2\), 
                        "-", \(2\ Alfa\ \[Pi]\ \(dZfR1(2, 1, 1)\)\), "+", 
                        \(\(4\ Alfa\ dSW1\ \[Pi]\)\/\(CW2\ SW\)\), "+", 
                        \(\(2\ Alfa\ CW2\ dZAZ1\ \[Pi]\)\/\(CW\ SW\)\), "+", 
                        \(\(4\ Alfa\ dSW1\ \[Pi] - 
                              \(4\ Alfa\ dSW1\ \[Pi]\ SW2\)\/CW2\)\/\(CW2\ SW
                                \ SW2 - CW2\^2\ SW\)\), "-", 
                        \(8\ Alfa\ dZe1\ \[Pi]\), "-", 
                        \(4\ Alfa\ dZGp1\ \[Pi]\), "-", 
                        \(4\ Alfa\ dZZZ1\ \[Pi]\)}], ")"}]}]}], ")"}], " ", 
              \(Mat(F2)\)}]}], ",", 
          \(\(SumOver(Neu1, 4)\)\ \(SumOver(Sfe1, 2)\)\ 
            \((\(Den(S, 0)\)\ \(Mat(F3)\)\ 
                  \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\)\((
                        Alfa2\ \((T - U)\)\ 
                          \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), 
                            MSf2(Sfe1, 2, 1), MSf2(Sfe1, 2, 1))\)\ 
                          \(MNeu(Neu1)\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\))\)\) + 
                      \(C0i(cc1, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe1, 2, 1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ \((S + 2\ T)\)\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\))\)\) - 
                            \(1\/\(CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\))\)\))\) + 
                      \(C0i(cc2, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe1, 2, 1))\)\ 
                        \((\(1\/\(CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ \((S + 2\ U)\)\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\))\)\))\))\) + 
                \(1\/\(CB2\ CW2\ MW2\ SW2\)\((
                  2\ Alfa2\ 
                    \(C0i(cc00, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                      MSf2(Sfe1, 2, 1))\)\ \(Den(S, 0)\)\ \(Mat(F2)\)\ 
                    \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                          \(ZNeu(Neu1, 1)\) + 
                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                    \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                          \(ZNeuC(Neu1, 1)\) + 
                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                      \))\)\) + 
                \(1\/\(CB2\ CW2\ MW2\ SW2\)\((
                  2\ Alfa2\ 
                    \(C0i(cc00, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                      MSf2(Sfe1, 2, 1))\)\ \(Den(S, 0)\)\ \(Mat(F1)\)\ 
                    \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                          \((SW\ \(ZNeu(Neu1, 1)\) + CW\ \(ZNeu(Neu1, 2)\))
                            \) - CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                          \(ZNeu(Neu1, 3)\))\)\ 
                    \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                          \(ZNeuC(Neu1, 1)\) + 
                        CW\ \((
                          CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                              \(ZNeuC(Neu1, 2)\) - 
                            ME\ \(USf(Sfe1, 2, 2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                      \))\)\) + 
                \(Den(S, 0)\)\ \(Mat(F4)\)\ 
                  \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\)\((
                        Alfa2\ \((T - U)\)\ 
                          \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), 
                            MSf2(Sfe1, 2, 1), MSf2(Sfe1, 2, 1))\)\ 
                          \(MNeu(Neu1)\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 3)\))\))\))\)\) + 
                      \(C0i(cc1, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe1, 2, 1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ \((S + 2\ T)\)\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\) - 
                            \(1\/\(CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\))\) + 
                      \(C0i(cc2, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe1, 2, 1))\)\ 
                        \((\(1\/\(CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\)\((
                              Alfa2\ \((S + 2\ U)\)\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\))\))\))
              \)\), ",", 
          \(\((\(Mat(F2)\)\ 
                  \((\(Den(S, 0)\)\ 
                        \((\(B0(S, MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2\ SW2
                                      \) + \(Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2\ 
                                      SW2\ TB2\))\) + 
                            \(B0(S, MQU2(Gen1), MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2\ SW2
                                      \) + \(2\ Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2
                                      \ SW2\ TB2\))\) + 
                            \(C0i(cc0, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(4\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ 
                                      SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\))
                                \) + \(C0i(cc0, MHp2, S, MHp2, MQU2(Gen1), 
                                MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2\ 
                                      TB2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(MW2\ SW2\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ 
                                      SW2\) + 
                                  \(4\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \ TB2\) + 
                                  \(4\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(MW2\ 
                                      SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2
                                        \ SW2\)\)\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2\ SW2\ 
                                        TB2\))\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \) + \(2\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2\ 
                                      TB2\) + 
                                  \(2\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(MW2\ 
                                      SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ 
                                        MW2\ SW2\)\)\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ MW2\ SW2
                                        \ TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \) + \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2\ 
                                      TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ MW2\ 
                                        SW2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ MW2\ SW2
                                        \ TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ 
                                      SW2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \ TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2\ SW2
                                        \) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2\ SW2\ 
                                        TB2\))\))\))\) + 
                      \(Den(S, MZ2)\)\ 
                        \((\(B0(S, MQU2(Gen1), MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2\ MW2
                                      \) - \(3\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ SW2
                                      \) + \(2\ Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2
                                      \ MW2\ TB2\))\) + 
                            \(B0(S, MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2\ MW2
                                      \) + \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ TB2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                      \ SW2\ TB2\))\) + 
                            \(C0i(cc0, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(4\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ 
                                      MW2\) - 
                                  \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2
                                      \ MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\))
                                \) + \(C0i(cc0, MHp2, S, MHp2, MQU2(Gen1), 
                                MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ SW2\ TB2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\) + 
                                  \(4\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ SW2\))
                                \) + \(C0i(cc2, MHp2, S, MHp2, MQD2(Gen1), 
                                MQU2(Gen1), MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ 
                                      MW2\) - 
                                  \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(2\ 
                                      CW2\ MW2\ SW2\) + 
                                  \(4\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ TB2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  \(4\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2
                                        \ MW2\)\)\) + 
                                      \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(4\ 
                                        CW2\ MW2\ SW2\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                        TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ 
                                      MW2\) - 
                                  \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(2\ 
                                      CW2\ MW2\ SW2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ TB2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2\ MW2
                                        \) - 
                                      \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(4\ 
                                        CW2\ MW2\ SW2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                        TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \) - \(3\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\^2\)\/\(2\ CW2\ MW2\ SW2
                                      \) + \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ CW2\ 
                                        MW2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                        \ TB2\) - 
                                      \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(4\ CW2\ 
                                        MW2\ SW2\ TB2\))\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \) - \(3\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\^2\)\/\(2\ CW2\ MW2\ SW2
                                      \) + \(2\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) + 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  \(2\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ TB2\) - 
                                  \(3\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ 
                                        CW2\ MW2\)\)\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                        \ TB2\) + 
                                      \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(4\ CW2\ 
                                        MW2\ SW2\ TB2\))\))\))\))\) + 
                \(Mat(F1)\)\ 
                  \((\(Den(S, 0)\)\ 
                        \((\(B0(S, MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2\ SW2
                                      \) + \(Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2\ 
                                      SW2\ TB2\))\) + 
                            \(B0(S, MQU2(Gen1), MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2\ SW2
                                      \) + \(2\ Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2
                                      \ SW2\ TB2\))\) + 
                            \(C0i(cc0, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(4\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ 
                                      SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\))
                                \) + \(C0i(cc0, MHp2, S, MHp2, MQU2(Gen1), 
                                MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2\ 
                                      TB2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(MW2\ SW2\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ 
                                      SW2\) + 
                                  \(4\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \ TB2\) + 
                                  \(4\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(MW2\ 
                                      SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2
                                        \ SW2\)\)\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2\ SW2\ 
                                        TB2\))\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \) + \(2\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2\ 
                                      TB2\) + 
                                  \(2\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(MW2\ 
                                      SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ 
                                        MW2\ SW2\)\)\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ MW2\ SW2
                                        \ TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \) + \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2\ 
                                      TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ MW2\ 
                                        SW2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ MW2\ SW2
                                        \ TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(MW2\ 
                                      SW2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(MW2\ SW2\ TB2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(MW2\ SW2
                                      \ TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(MW2\ SW2
                                        \) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(MW2\ SW2\ 
                                        TB2\))\))\))\) + 
                      \(Den(S, MZ2)\)\ 
                        \((\(B0(S, MQD2(Gen1), MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2\ MW2
                                      \) - \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2
                                      \ CW2\ MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ TB2
                                      \) - \(2\ Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2
                                      \ MW2\ SW2\ TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(4\ CW2\ MW2
                                      \ SW2\^2\ TB2\))\) + 
                            \(B0(S, MQU2(Gen1), MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2\ MW2
                                      \) - \(5\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ SW2
                                      \) + \(3\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(4\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ SW2\ 
                                      TB2\))\) + 
                            \(C0i(cc0, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(4\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ 
                                      MW2\) - 
                                  \(5\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2
                                      \ MW2\ SW2\) + 
                                  \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(2\ 
                                      CW2\ MW2\ SW2\^2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(7\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) - 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\ TB2
                                      \) + \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ 
                                      SW2\^2\))\) + 
                            \(C0i(cc0, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(4\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ SW2\ TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\^2\ TB2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(Alfa2\ TB2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(4\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(5\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(3\ Alfa2\ \(MQD2(Gen1)\)\ 
                                      \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2\ 
                                      SW2\^2\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \) - \(2\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\^2\)\/\(CW2\ MW2\ SW2
                                      \) + \(3\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\^2\)\/\(4\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ SW2
                                      \) + \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(5\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) - 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ SW2\ 
                                      TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ SW2\ TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(4\ CW2\ 
                                      MW2\ SW2\^2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ CW2\ 
                                        MW2\) - 
                                      \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(4\ CW2
                                        \ MW2\ SW2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                        \ TB2\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                        SW2\ TB2\) + 
                                      \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(8\ CW2\ 
                                        MW2\ SW2\^2\ TB2\))\))\) + 
                            \(C0i(cc1, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ 
                                      MW2\) - 
                                  \(5\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(2\ 
                                      CW2\ MW2\ SW2\) + 
                                  \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(4\ 
                                      CW2\ MW2\ SW2\^2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(7\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) - 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\ TB2
                                      \) + \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ TB2\) - 
                                  \(5\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(4\ CW2\ 
                                      MW2\ SW2\^2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2\ MW2
                                        \) - 
                                      \(5\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(4\ 
                                        CW2\ MW2\ SW2\) + 
                                      \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(8\ 
                                        CW2\ MW2\ SW2\^2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                        TB2\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                        \ SW2\ TB2\))\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQD2(Gen1), MQU2(Gen1), 
                                MQU2(Gen1))\)\ 
                              \((\(2\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ 
                                      MW2\) - 
                                  \(5\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(2\ 
                                      CW2\ MW2\ SW2\) + 
                                  \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(4\ 
                                      CW2\ MW2\ SW2\^2\) + 
                                  \(4\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(5\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(3\ Alfa2\ MHp2\ TB2\ \(MQD2(Gen1)\)\)\/\(2
                                      \ CW2\ MW2\ SW2\^2\) + 
                                  \(2\ Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(8\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(7\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) - 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\ TB2
                                      \) + \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ TB2\) - 
                                  \(5\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(2\ CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(4\ CW2\ 
                                      MW2\ SW2\^2\ TB2\) + 
                                  \(4\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ TB2\) - 
                                  \(2\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2
                                        \ MW2\)\)\) + 
                                      \(5\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(4\ 
                                        CW2\ MW2\ SW2\) - 
                                      \(3\ Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(8\ 
                                        CW2\ MW2\ SW2\^2\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                        TB2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                        \ SW2\ TB2\))\))\) + 
                            \(C0i(cc2, MHp2, S, MHp2, MQU2(Gen1), MQD2(Gen1), 
                                MQD2(Gen1))\)\ 
                              \((\(Alfa2\ TB2\ \(MQD2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \) - \(2\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\^2\)\/\(CW2\ MW2\ SW2
                                      \) + \(3\ Alfa2\ TB2\ 
                                      \(MQD2(Gen1)\)\^2\)\/\(4\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(2\ Alfa2\ MHp2\ TB2\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(Alfa2\ MHp2\ TB2\ \(MQD2(Gen1)\)\)\/\(CW2
                                      \ MW2\ SW2\) + 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(Alfa2\ TB2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ SW2
                                      \) + \(4\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\) - 
                                  \(5\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ SW2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(CW2\ MW2\ TB2\) - 
                                  \(Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ SW2\ 
                                      TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\ 
                                      \(MQD2(Gen1)\)\)\/\(2\ CW2\ MW2\ 
                                      SW2\^2\) + 
                                  \(Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2\ 
                                      TB2\) - 
                                  \(2\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(CW2\ MW2
                                      \ SW2\ TB2\) + 
                                  \(3\ Alfa2\ \(MQU2(Gen1)\)\^2\)\/\(4\ CW2\ 
                                      MW2\ SW2\^2\ TB2\) + 
                                  \(2\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ TB2\) - 
                                  \(4\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(CW2\ 
                                      MW2\ SW2\ TB2\) + 
                                  \(3\ Alfa2\ MHp2\ \(MQU2(Gen1)\)\)\/\(2\ CW2
                                      \ MW2\ SW2\^2\ TB2\) + 
                                  2\ MHp2\ \((
                                    \(-\(\(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(2\ 
                                        CW2\ MW2\)\)\) + 
                                      \(Alfa2\ TB2\ \(MQD2(Gen1)\)\)\/\(4\ CW2
                                        \ MW2\ SW2\) - 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(2\ CW2\ MW2
                                        \ TB2\) + 
                                      \(Alfa2\ \(MQU2(Gen1)\)\)\/\(CW2\ MW2\ 
                                        SW2\ TB2\) - 
                                      \(3\ Alfa2\ \(MQU2(Gen1)\)\)\/\(8\ CW2\ 
                                        MW2\ SW2\^2\ TB2\))\))\))\))\))\)\ 
            \(SumOver(Gen1, 3)\)\), ",", 
          RowBox[{\(SumOver(Gen1, 3)\), " ", \(SumOver(Sfe1, 2)\), " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(Mat(F2)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", \((
                            C0i(cc0, MHp2, S, MHp2, MSf2(Sfe1, 2, Gen1), 
                                MSf2(1, 1, Gen1), MSf2(1, 1, Gen1)) + 
                              C0i(cc1, MHp2, S, MHp2, MSf2(Sfe1, 2, Gen1), 
                                MSf2(1, 1, Gen1), MSf2(1, 1, Gen1)) + 
                              C0i(cc2, MHp2, S, MHp2, MSf2(Sfe1, 2, Gen1), 
                                MSf2(1, 1, Gen1), MSf2(1, 1, Gen1)))\), " ", 
                            \(Den(S, MZ2)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, Gen1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(MLE(Gen1)\), " ", 
                                  \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USfC(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(MLE(Gen1)\), " ", 
                                  \(USfC(Sfe1, 2, 2, Gen1)\)}]}], ")"}]}], 
                          ")"}]}], "-", 
                      RowBox[{\(1\/\(MW2\ SW2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", \((
                            C0i(cc0, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1)) + 
                              C0i(cc1, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1)) + 
                              C0i(cc2, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1)))\), 
                            " ", \(Den(S, 0)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, Gen1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(MLE(Gen1)\), " ", 
                                  \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USfC(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(MLE(Gen1)\), " ", 
                                  \(USfC(Sfe1, 2, 2, Gen1)\)}]}], ")"}]}], 
                          ")"}]}]}], ")"}]}], "+", 
                RowBox[{\(Mat(F1)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \((C0i(cc0, MHp2, S, MHp2, MSf2(Sfe1, 2, Gen1), 
                              MSf2(1, 1, Gen1), MSf2(1, 1, Gen1)) + 
                            C0i(cc1, MHp2, S, MHp2, MSf2(Sfe1, 2, Gen1), 
                              MSf2(1, 1, Gen1), MSf2(1, 1, Gen1)) + 
                            C0i(cc2, MHp2, S, MHp2, MSf2(Sfe1, 2, Gen1), 
                              MSf2(1, 1, Gen1), MSf2(1, 1, Gen1)))\), " ", 
                        \(Den(S, MZ2)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(2, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MLE(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], 
                                  " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USfC(Sfe1, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MLE(Gen1)\), " ", 
                                        \(USfC(Sfe1, 2, 2, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}], "-", 
                            RowBox[{\(1\/\(4\ CW2\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(2, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MLE(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], 
                                  " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USfC(Sfe1, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MLE(Gen1)\), " ", 
                                        \(USfC(Sfe1, 2, 2, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}]}], ")"}]}], "-", 
                      RowBox[{\(1\/\(MW2\ SW2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", \((
                            C0i(cc0, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1)) + 
                              C0i(cc1, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1)) + 
                              C0i(cc2, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe1, 2, Gen1)))\), 
                            " ", \(Den(S, 0)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, Gen1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(MLE(Gen1)\), " ", 
                                  \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USfC(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(MLE(Gen1)\), " ", 
                                  \(USfC(Sfe1, 2, 2, Gen1)\)}]}], ")"}]}], 
                          ")"}]}]}], ")"}]}]}], ")"}]}], ",", 
          RowBox[{
          \(SumOver(Gen1, 3)\), " ", \(SumOver(Sfe1, 2)\), " ", 
            \(SumOver(Sfe2, 2)\), " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(Mat(F2)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(Den(S, 0)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              RowBox[{\(1\/\(MW2\ SW2\ TB2\)\), 
                                RowBox[{"(", 
                                  RowBox[{
                                  "2", " ", "Alfa2", " ", \((
                                    C0i(cc0, MHp2, S, MHp2, 
                                        MSf2(Sfe1, 4, Gen1), 
                                        MSf2(Sfe2, 3, Gen1), 
                                        MSf2(Sfe2, 3, Gen1)) + 
                                      C0i(cc1, MHp2, S, MHp2, 
                                        MSf2(Sfe1, 4, Gen1), 
                                        MSf2(Sfe2, 3, Gen1), 
                                        MSf2(Sfe2, 3, Gen1)) + 
                                      C0i(cc2, MHp2, S, MHp2, 
                                        MSf2(Sfe1, 4, Gen1), 
                                        MSf2(Sfe2, 3, Gen1), 
                                        MSf2(Sfe2, 3, Gen1)))\), " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((TB\ \(MQD2(Gen1)\) - 
                                        MW2\ S2B)\) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 4, Gen1)\)}], 
                                        "-", 
                                        RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{\(Af(4, Gen1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe2, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 4, Gen1)\)}]}], 
                                      ")"}], " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((TB\ \(MQD2(Gen1)\) - 
                                        MW2\ S2B)\) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 3, Gen1)\)}], 
                                        "-", 
                                        RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 3, Gen1)\)}]}], 
                                      ")"}]}], ")"}]}]}], "-", 
                            RowBox[{\(1\/\(MW2\ SW2\ TB2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", \((
                                  C0i(cc0, MHp2, S, MHp2, 
                                      MSf2(Sfe1, 3, Gen1), 
                                      MSf2(Sfe2, 4, Gen1), 
                                      MSf2(Sfe2, 4, Gen1)) + 
                                    C0i(cc1, MHp2, S, MHp2, 
                                      MSf2(Sfe1, 3, Gen1), 
                                      MSf2(Sfe2, 4, Gen1), 
                                      MSf2(Sfe2, 4, Gen1)) + 
                                    C0i(cc2, MHp2, S, MHp2, 
                                      MSf2(Sfe1, 3, Gen1), 
                                      MSf2(Sfe2, 4, Gen1), 
                                      MSf2(Sfe2, 4, Gen1)))\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 3, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe2, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 3, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 4, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 4, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}]}], ")"}]}], "-", 
                      RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", \((
                            C0i(cc0, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1)) + 
                              C0i(cc1, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1)) + 
                              C0i(cc2, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                                MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1)))\), 
                            " ", \(Den(S, MZ2)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, Gen1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(MLE(Gen1)\), " ", 
                                  \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], " ", 
                            \((\((2\ SW2 - 1)\)\ \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                  \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                  \(USfC(Sfe1, 2, 2, Gen1)\))\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                  \(USfC(Sfe2, 1, 2, Gen1)\)\), "-", 
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(MLE(Gen1)\), " ", 
                                  \(USfC(Sfe2, 2, 2, Gen1)\)}]}], ")"}]}], 
                          ")"}]}]}], ")"}]}], "+", 
                RowBox[{\(Mat(F1)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \((C0i(cc0, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                              MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1)) + 
                            C0i(cc1, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                              MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1)) + 
                            C0i(cc2, MHp2, S, MHp2, MSf2(1, 1, Gen1), 
                              MSf2(Sfe1, 2, Gen1), MSf2(Sfe2, 2, Gen1)))\), 
                        " ", \(Den(S, MZ2)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(4\ CW2\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(2, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MLE(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], 
                                  " ", \((
                                    \((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                        \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                      2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                        \(USfC(Sfe1, 2, 2, Gen1)\))\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USfC(Sfe2, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MLE(Gen1)\), " ", 
                                        \(USfC(Sfe2, 2, 2, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}], "-", 
                            RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(2, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MLE(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 2, Gen1)\)}]}], ")"}], 
                                  " ", \((
                                    \((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe2, 1, 2, Gen1)\)\ 
                                        \(USfC(Sfe1, 1, 2, Gen1)\) + 
                                      2\ SW2\ \(USf(Sfe2, 2, 2, Gen1)\)\ 
                                        \(USfC(Sfe1, 2, 2, Gen1)\))\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - TB\ \(MLE2(Gen1)\))\)\ 
                                        \(USfC(Sfe2, 1, 2, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MLE(Gen1)\), " ", 
                                        \(USfC(Sfe2, 2, 2, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}]}], ")"}]}], "+", 
                      RowBox[{\(Den(S, 0)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              RowBox[{\(1\/\(MW2\ SW2\ TB2\)\), 
                                RowBox[{"(", 
                                  RowBox[{
                                  "2", " ", "Alfa2", " ", \((
                                    C0i(cc0, MHp2, S, MHp2, 
                                        MSf2(Sfe1, 4, Gen1), 
                                        MSf2(Sfe2, 3, Gen1), 
                                        MSf2(Sfe2, 3, Gen1)) + 
                                      C0i(cc1, MHp2, S, MHp2, 
                                        MSf2(Sfe1, 4, Gen1), 
                                        MSf2(Sfe2, 3, Gen1), 
                                        MSf2(Sfe2, 3, Gen1)) + 
                                      C0i(cc2, MHp2, S, MHp2, 
                                        MSf2(Sfe1, 4, Gen1), 
                                        MSf2(Sfe2, 3, Gen1), 
                                        MSf2(Sfe2, 3, Gen1)))\), " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((TB\ \(MQD2(Gen1)\) - 
                                        MW2\ S2B)\) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 4, Gen1)\)}], 
                                        "-", 
                                        RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe2, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 4, Gen1)\)}]}], 
                                      ")"}], " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((TB\ \(MQD2(Gen1)\) - 
                                        MW2\ S2B)\) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 3, Gen1)\)}], 
                                        "-", 
                                        RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 3, Gen1)\)}]}], 
                                      ")"}]}], ")"}]}]}], "-", 
                            RowBox[{\(1\/\(MW2\ SW2\ TB2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", \((
                                  C0i(cc0, MHp2, S, MHp2, 
                                      MSf2(Sfe1, 3, Gen1), 
                                      MSf2(Sfe2, 4, Gen1), 
                                      MSf2(Sfe2, 4, Gen1)) + 
                                    C0i(cc1, MHp2, S, MHp2, 
                                      MSf2(Sfe1, 3, Gen1), 
                                      MSf2(Sfe2, 4, Gen1), 
                                      MSf2(Sfe2, 4, Gen1)) + 
                                    C0i(cc2, MHp2, S, MHp2, 
                                      MSf2(Sfe1, 3, Gen1), 
                                      MSf2(Sfe2, 4, Gen1), 
                                      MSf2(Sfe2, 4, Gen1)))\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 3, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe2, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 3, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 4, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 4, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], 
              ")"}]}], ",", 
          \(\(Mat(F1)\)\ \(SumOver(Cha1, 2)\)\ 
            \((\(C0i(cc00, ME2, S, ME2, MCha2(Cha1), MSf2(1, 1, 1), 
                    MSf2(1, 1, 1))\)\ \(Den(S, MZ2)\)\ 
                  \((\(Alfa2\ \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\)\)\/\(CW2\ 
                          SW2\) - 
                      \(Alfa2\ \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\)\)\/SW2
                          \^2)\) + 
                \(Den(S, 0)\)\ 
                  \((\(2\ Alfa2\ \(B0(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                          \(VCha(Cha1, 1)\)\ \(VChaC(Cha1, 1)\)\)\/SW2 - 
                      \(1\/SW2\((
                        4\ Alfa2\ 
                          \(C0i(cc00, ME2, S, ME2, MSf2(1, 1, 1), 
                            MCha2(Cha1), MCha2(Cha1))\)\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha1, 1)\))\)\) + 
                      \(C0i(cc0, ME2, S, ME2, MSf2(1, 1, 1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(2\ Alfa2\ \(MSf2(1, 1, 1)\)\ \(VCha(Cha1, 1)\)\ 
                                \(VChaC(Cha1, 1)\)\)\/SW2 - 
                            \(2\ Alfa2\ \(MCha2(Cha1)\)\ \(VCha(Cha1, 1)\)\ 
                                \(VChaC(Cha1, 1)\)\)\/SW2)\))\))\)\), ",", 
          \(\(SumOver(Cha1, 2)\)\ \(SumOver(Neu1, 4)\)\ 
            \((\(Den(S, 0)\)\ \(Mat(F1)\)\ 
                  \((\(B0(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                        \((\(1\/SW2\((
                              2\ Alfa2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/SW2\((
                                4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/SW2\((
                              4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc2, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/SW2\((
                                4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ CB2\ MHp2\ \((
                                \(\(VChaC(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                  \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/SW2\((
                              4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(-\(1\/SW2\((
                                    Alfa2\ SB2\ \((
                                      \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                        \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\)\) - 
                                \(1\/SW2\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\) + 
                      \(C0i(cc1, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/SW2\((
                                4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/SW2\((
                              4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/SW2\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\) + 
                                \(1\/SW2\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\))\) + 
                \(Den(S, 0)\)\ \(Mat(F2)\)\ 
                  \((\(B0(S, MCha2(Cha1), MCha2(Cha1))\)\ 
                        \((\(1\/SW2\((
                              2\ Alfa2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/SW2\((
                                4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/SW2\((
                              4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc2, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/SW2\((
                                4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ CB2\ MHp2\ \((
                                \(\(VChaC(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                  \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              4\ Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/SW2\((
                              4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(-\(1\/SW2\((
                                    Alfa2\ SB2\ \((
                                      \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                        \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\)\) - 
                                \(1\/SW2\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\) + 
                      \(C0i(cc1, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/SW2\((
                                4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/SW2\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/SW2\((
                              4\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/SW2\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\) + 
                                \(1\/SW2\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\))\))
              \)\), ",", 
          RowBox[{
          \(SumOver(Neu1, 4)\), " ", \(SumOver(Sfe1, 2)\), " ", 
            \(SumOver(Sfe2, 2)\), " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(Mat(F3)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe2, 2, 1))\), " ", \(Den(S, MHH2)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "C2B", " ", "CAB", " ", 
                                  \(MNeu(Neu1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((CAB\ CB\ MW\ MZ\ \((1 - 2\ SW2)
                                        \) - 2\ CA\ CW\ ME2)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "-", 
                                        \(CA\ \(Af(2, 1)\)\)}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 2, 1)\)}], "+", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ SA\), "-", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "-", 
                                        \(2\ CA\ CW\ ME2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "+", 
                                        \(2\ CAB\ CB\ MW\ MZ\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                    ")"}], " ", 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                      CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)}], ")"}], "/", 
                              \((8\ CB\^3\ CW\ CW2\^2\ MW2\ SW2\^2)\)}], "-", 
                            
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "CBA", " ", \(MNeu(Neu1)\), " ",
                                   
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((CAB\ CB\ MW\ MZ\ \((1 - 2\ SW2)
                                        \) - 2\ CA\ CW\ ME2)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "-", 
                                        \(CA\ \(Af(2, 1)\)\)}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 2, 1)\)}], "+", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ SA\), "-", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "-", 
                                        \(2\ CA\ CW\ ME2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "+", 
                                        \(2\ CAB\ CB\ MW\ MZ\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                    ")"}], " ", 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                      CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)}], ")"}], "/", 
                              \((4\ CB\^3\ CW\ CW2\ MW2\ SW2\^2)\)}]}], 
                          ")"}]}], "+", 
                      RowBox[{
                      \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe2, 2, 1))\), " ", \(Den(S, Mh02)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              RowBox[{
                                RowBox[{"(", 
                                  RowBox[{
                                  "Alfa2", " ", "C2B", " ", "SAB", " ", 
                                    \(MNeu(Neu1)\), " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((2\ CW\ ME2\ SA + 
                                        CB\ MW\ MZ\ SAB\ \((2\ SW2 - 1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(SA\ \(Af(2, 1)\)\), "+", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 2, 2, 1)\)}]}], 
                                        ")"}], " ", \(USfC(Sfe2, 1, 2, 1)\)}],
                                         "+", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(CA\ MUE\), "+", 
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{\(Af(2, 1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "+", 
                                        \(2\ CW\ ME2\ SA\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "-", 
                                        \(2\ CB\ MW\ MZ\ SAB\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                      ")"}], " ", 
                                    \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                    \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)}], ")"}], "/", 
                                \((8\ CB\^3\ CW\ CW2\^2\ MW2\ SW2\^2)\)}]}], 
                            "-", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "SBA", " ", \(MNeu(Neu1)\), " ",
                                   
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((2\ CW\ ME2\ SA + 
                                        CB\ MW\ MZ\ SAB\ \((2\ SW2 - 1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(SA\ \(Af(2, 1)\)\), "+", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 2, 2, 1)\)}]}], 
                                        ")"}], " ", \(USfC(Sfe2, 1, 2, 1)\)}],
                                       "+", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(CA\ MUE\), "+", 
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "+", 
                                        \(2\ CW\ ME2\ SA\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "-", 
                                        \(2\ CB\ MW\ MZ\ SAB\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                    ")"}], " ", 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                      CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)}], ")"}], "/", 
                              \((4\ CB\^3\ CW\ CW2\ MW2\ SW2\^2)\)}]}], 
                          ")"}]}], "+", 
                      \(\(Den(S, MZ2)\)\ 
                        \((\((T - U)\)\ 
                              \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), 
                                MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                              \((\(1\/\(8\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    Alfa2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)\) - 
                                  \(1\/\(8\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)\))\) + 
                            \(C0i(cc2, ME2, S, ME2, MNeu2(Neu1), 
                                MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                              \((\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)\) - 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)\) + 
                                  \((S + 2\ U)\)\ 
                                    \((\((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)/
                                        \((8\ CB2\ CW2\ MW2\ SW2\^2)\) - 
                                        \((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\))\))
                                \) + \(C0i(cc1, ME2, S, ME2, MNeu2(Neu1), 
                                MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                              \((\(-\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                      Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)\)\) + 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)\) + 
                                  \((S + 2\ T)\)\ 
                                    \((\((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\) - 
                                        \((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                        CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\))\)/
                                        \((8\ CB2\ CW2\ MW2\ SW2\^2)\))\))\))
                          \)\)}], ")"}]}], "+", 
                \(\(C0i(cc00, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                    MSf2(Sfe2, 2, 1))\)\ \(Den(S, MZ2)\)\ \(Mat(F2)\)\ 
                  \((\(1\/\(2\ CB2\ CW2\^2\ MW2\ SW2\)\((
                        Alfa2\ \((
                          \((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, 1)\)\ 
                              \(USfC(Sfe2, 1, 2, 1)\) + 
                            2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                              \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 1)\) + 
                              CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\))\)\) - 
                      \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \((
                          \((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, 1)\)\ 
                              \(USfC(Sfe2, 1, 2, 1)\) + 
                            2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                              \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 1)\) + 
                              CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\))\)\))\)\), "+", 
                \(\(C0i(cc00, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                    MSf2(Sfe2, 2, 1))\)\ \(Den(S, MZ2)\)\ \(Mat(F1)\)\ 
                  \((\(1\/\(2\ CB2\ CW2\^2\ MW2\ SW2\)\((
                        Alfa2\ \((
                          \((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, 1)\)\ 
                              \(USfC(Sfe2, 1, 2, 1)\) + 
                            2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                              \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 2)\) - 
                                  ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 3)\))\))\))\)\) - 
                      \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \((
                          \((2\ SW2 - 1)\)\ \(USf(Sfe1, 1, 2, 1)\)\ 
                              \(USfC(Sfe2, 1, 2, 1)\) + 
                            2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                              \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 2)\) - 
                                  ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 3)\))\))\))\)\))\)\), "+", 
                RowBox[{\(Mat(F4)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe2, 2, 1))\), " ", \(Den(S, MHH2)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "C2B", " ", "CAB", " ", 
                                  \(MNeu(Neu1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((CAB\ CB\ MW\ MZ\ \((1 - 2\ SW2)
                                        \) - 2\ CA\ CW\ ME2)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "-", 
                                        \(CA\ \(Af(2, 1)\)\)}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 2, 1)\)}], "+", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ SA\), "-", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "-", 
                                        \(2\ CA\ CW\ ME2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "+", 
                                        \(2\ CAB\ CB\ MW\ MZ\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                    ")"}], " ", 
                                  \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\), " ", 
                                  \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)}], ")"}], 
                              "/", \((8\ CB\^3\ CW\ CW2\^2\ MW2\ SW2\^2)\)}], 
                            "-", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "CBA", " ", \(MNeu(Neu1)\), " ",
                                   
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((CAB\ CB\ MW\ MZ\ \((1 - 2\ SW2)
                                        \) - 2\ CA\ CW\ ME2)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}], "-", 
                                        \(CA\ \(Af(2, 1)\)\)}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 2, 1)\)}], "+", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ SA\), "-", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "-", 
                                        \(2\ CA\ CW\ ME2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "+", 
                                        \(2\ CAB\ CB\ MW\ MZ\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                    ")"}], " ", 
                                  \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\), " ", 
                                  \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)}], ")"}], 
                              "/", \((4\ CB\^3\ CW\ CW2\ MW2\ SW2\^2)\)}]}], 
                          ")"}]}], "+", 
                      RowBox[{
                      \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                          MSf2(Sfe2, 2, 1))\), " ", \(Den(S, Mh02)\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              RowBox[{
                                RowBox[{"(", 
                                  RowBox[{
                                  "Alfa2", " ", "C2B", " ", "SAB", " ", 
                                    \(MNeu(Neu1)\), " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((2\ CW\ ME2\ SA + 
                                        CB\ MW\ MZ\ SAB\ \((2\ SW2 - 1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(SA\ \(Af(2, 1)\)\), "+", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 2, 2, 1)\)}]}], 
                                        ")"}], " ", \(USfC(Sfe2, 1, 2, 1)\)}],
                                         "+", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(CA\ MUE\), "+", 
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{\(Af(2, 1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "+", 
                                        \(2\ CW\ ME2\ SA\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "-", 
                                        \(2\ CB\ MW\ MZ\ SAB\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                      ")"}], " ", 
                                    \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\), " ", 
                                    \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)}], ")"}], 
                                "/", \((8\ CB\^3\ CW\ CW2\^2\ MW2\ SW2\^2)
                                  \)}]}], "-", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "SBA", " ", \(MNeu(Neu1)\), " ",
                                   
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\((2\ CW\ ME2\ SA + 
                                        CB\ MW\ MZ\ SAB\ \((2\ SW2 - 1)\))\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "+", 
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(SA\ \(Af(2, 1)\)\), "+", 
                                        RowBox[{"CA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 2, 2, 1)\)}]}], 
                                        ")"}], " ", \(USfC(Sfe2, 1, 2, 1)\)}],
                                       "+", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"CW", " ", "ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(CA\ MUE\), "+", 
                                        RowBox[{"SA", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 2, 1)\)}], "+", 
                                        \(2\ CW\ ME2\ SA\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\), "-", 
                                        \(2\ CB\ MW\ MZ\ SAB\ SW2\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], 
                                    ")"}], " ", 
                                  \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\), " ", 
                                  \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)}], ")"}], 
                              "/", \((4\ CB\^3\ CW\ CW2\ MW2\ SW2\^2)\)}]}], 
                          ")"}]}], "+", 
                      \(\(Den(S, MZ2)\)\ 
                        \((\((T - U)\)\ 
                              \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), 
                                MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                              \((\(1\/\(8\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    Alfa2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\) - 
                                  \(1\/\(8\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\))\) + 
                            \(C0i(cc2, ME2, S, ME2, MNeu2(Neu1), 
                                MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                              \((\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\) - 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\) + 
                                  \((S + 2\ U)\)\ 
                                    \((\((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe2, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)/
                                        \((8\ CB2\ CW2\ MW2\ SW2\^2)\) - 
                                        \((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe2, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\))\))
                                \) + \(C0i(cc1, ME2, S, ME2, MNeu2(Neu1), 
                                MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                              \((\(-\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                      Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\)\) + 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)\) + 
                                  \((S + 2\ T)\)\ 
                                    \((\((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe2, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\) - 
                                        \((Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((\((2\ SW2 - 1)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(USfC(Sfe2, 1, 2, 1)\) + 
                                        2\ SW2\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe2, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe2, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\))\)/
                                        \((8\ CB2\ CW2\ MW2\ SW2\^2)\))\))\))
                          \)\)}], ")"}]}]}], ")"}]}], ",", 
          \(\(SumOver(Neu1, 4)\)\ \(SumOver(Neu2, 4)\)\ \(SumOver(Sfe1, 2)\)\ 
            \((\(Den(S, MZ2)\)\ \(Mat(F2)\)\ 
                  \((\(C0i(cc00, ME2, S, ME2, MSf2(Sfe1, 2, 1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((
                                2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 1)\) + 
                                  CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \((
                                2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 1)\) + 
                                  CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                        \((\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \((
                                2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 1)\) + 
                                  CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((
                                2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 1)\) + 
                                  CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, ME2, S, ME2, MSf2(Sfe1, 2, 1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\))\) + 
                \(Den(S, MZ2)\)\ \(Mat(F1)\)\ 
                  \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                        \((\(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(C0i(cc00, ME2, S, ME2, MSf2(Sfe1, 2, 1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(C0i(cc0, ME2, S, ME2, MSf2(Sfe1, 2, 1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                  \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \)\ \((
                                    CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\))\) + 
                \(Mat(F3)\)\ 
                  \((\(Den(S, Mh02)\)\ 
                        \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\((Alfa2\ C2B\ SAB\ \((
                                        CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((SA\ \(ZNeuC(Neu1, 3)\) + 
                                        CA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        SA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ SBA\ \((
                                        CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((SA\ \(ZNeuC(Neu1, 3)\) + 
                                        CA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        SA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\))\) + 
                            \(C0i(cc0, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\((Alfa2\ C2B\ SAB\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((SA\ \(ZNeu(Neu1, 3)\) + 
                                        CA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) - 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) - 
                                        \((CA\ CW\ \(ZNeu(Neu1, 2)\) - 
                                        CA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ SBA\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((SA\ \(ZNeu(Neu1, 3)\) + 
                                        CA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) - 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) - 
                                        \((CA\ CW\ \(ZNeu(Neu1, 2)\) - 
                                        CA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ C2B\ SAB\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((SA\ \(ZNeuC(Neu1, 3)\) + 
                                        CA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        SA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ SBA\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((SA\ \(ZNeuC(Neu1, 3)\) + 
                                        CA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        SA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\))\))\) + 
                      \(Den(S, MHH2)\)\ 
                        \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\((Alfa2\ C2B\ CAB\ \((
                                        CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((CA\ \(ZNeuC(Neu1, 3)\) - 
                                        SA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeuC(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeuC(Neu1, 1)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) - 
                                  \((Alfa2\ CBA\ \((
                                        CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((CA\ \(ZNeuC(Neu1, 3)\) - 
                                        SA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeuC(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeuC(Neu1, 1)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\))\) + 
                            \(C0i(cc0, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\(-\(\((
                                        Alfa2\ CBA\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((CA\ \(ZNeu(Neu1, 3)\) - 
                                        SA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) + 
                                        CA\ \((SW\ \(ZNeu(Neu1, 1)\) - 
                                        CW\ \(ZNeu(Neu1, 2)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\))\)/
                                      \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\)\)\) + 
                                  \((Alfa2\ C2B\ CAB\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((CA\ \(ZNeu(Neu1, 3)\) - 
                                        SA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeu(Neu1, 1)\) - 
                                        CW\ \(ZNeu(Neu1, 2)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) - 
                                  \((Alfa2\ CBA\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((CA\ \(ZNeuC(Neu1, 3)\) - 
                                        SA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeuC(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeuC(Neu1, 1)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ C2B\ CAB\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\((CA\ \(ZNeuC(Neu1, 3)\) - 
                                        SA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeuC(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeuC(Neu1, 1)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\))\))
                          \) + \(Den(S, MZ2)\)\ 
                        \((\(C0i(cc2, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\(-\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                      Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)\)\) + 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                      \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)\) + 
                                  \((S + 2\ U)\)\ 
                                    \((\((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\) - 
                                        \((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)/\((8\ CB2\ CW2\ MW2\ SW2\^2)\))
                                      \))\) + 
                            \(C0i(cc1, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\(-\(1\/\(2\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                      Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)\)\) + 
                                  \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                      \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)\) + 
                                  2\ MHp2\ \((
                                    \((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)/
                                        \((4\ CB2\ CW2\^2\ MW2\ SW2)\) - 
                                      \((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \))\)/\((4\ CB2\ CW2\ MW2\ SW2\^2)\))
                                    \) - \(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    3\ Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                      \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)\) + 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    3\ Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                      \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)\) + 
                                  1\/3\ \((6\ MHp2 + T - U)\)\ 
                                    \((\((3\ Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\) - 
                                        \((3\ Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                        \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)/\((8\ CB2\ CW2\ MW2\ SW2\^2)\))
                                      \))\))\))\) + 
                \(Mat(F4)\)\ 
                  \((\(Den(S, Mh02)\)\ 
                        \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\((Alfa2\ C2B\ SAB\ \((
                                        \((SA\ \(ZNeu(Neu1, 3)\) + 
                                        CA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) - 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) - 
                                        \((CA\ CW\ \(ZNeu(Neu1, 2)\) - 
                                        CA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ SBA\ \((
                                        \((SA\ \(ZNeu(Neu1, 3)\) + 
                                        CA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) - 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) - 
                                        \((CA\ CW\ \(ZNeu(Neu1, 2)\) - 
                                        CA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\))\) + 
                            \(C0i(cc0, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\((Alfa2\ C2B\ SAB\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((\((SA\ \(ZNeu(Neu1, 3)\) + 
                                        CA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) - 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) - 
                                        \((CA\ CW\ \(ZNeu(Neu1, 2)\) - 
                                        CA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ SBA\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((\((SA\ \(ZNeu(Neu1, 3)\) + 
                                        CA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) - 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) - 
                                        \((CA\ CW\ \(ZNeu(Neu1, 2)\) - 
                                        CA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ C2B\ SAB\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((\((SA\ \(ZNeuC(Neu1, 3)\) + 
                                        CA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        SA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ SBA\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((\((SA\ \(ZNeuC(Neu1, 3)\) + 
                                        CA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        SA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\))\))\) + 
                      \(Den(S, MHH2)\)\ 
                        \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\((Alfa2\ C2B\ CAB\ \((
                                        \((CA\ \(ZNeu(Neu1, 3)\) - 
                                        SA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeu(Neu1, 1)\) - 
                                        CW\ \(ZNeu(Neu1, 2)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) - 
                                  \((Alfa2\ CBA\ \((
                                        \((CA\ \(ZNeu(Neu1, 3)\) - 
                                        SA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeu(Neu1, 1)\) - 
                                        CW\ \(ZNeu(Neu1, 2)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\))\) + 
                            \(C0i(cc0, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\(-\(\((
                                        Alfa2\ CBA\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((\((CA\ \(ZNeu(Neu1, 3)\) - 
                                        SA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) + 
                                        CA\ \((SW\ \(ZNeu(Neu1, 1)\) - 
                                        CW\ \(ZNeu(Neu1, 2)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                      \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\)\)\) + 
                                  \((Alfa2\ C2B\ CAB\ \(MSf2(Sfe1, 2, 1)\)\ 
                                        \((\((CA\ \(ZNeu(Neu1, 3)\) - 
                                        SA\ \(ZNeu(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeu(Neu2, 1)\) - 
                                        CW\ \(ZNeu(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeu(Neu1, 1)\) - 
                                        CW\ \(ZNeu(Neu1, 2)\))\)\ 
                                        \(ZNeu(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeu(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeu(Neu1, 1)\))\)\ 
                                        \(ZNeu(Neu2, 4)\))\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\) - 
                                  \((Alfa2\ CBA\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((\((CA\ \(ZNeuC(Neu1, 3)\) - 
                                        SA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeuC(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeuC(Neu1, 1)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((4\ CB2\ CW\ CW2\ MW\ SW2\^2)\) + 
                                  \((Alfa2\ C2B\ CAB\ \(MNeu(Neu1)\)\ 
                                        \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((\((CA\ \(ZNeuC(Neu1, 3)\) - 
                                        SA\ \(ZNeuC(Neu1, 4)\))\)\ 
                                        \((SW\ \(ZNeuC(Neu2, 1)\) - 
                                        CW\ \(ZNeuC(Neu2, 2)\))\) + 
                                        CA\ \((
                                        SW\ \(ZNeuC(Neu1, 1)\) - 
                                        CW\ \(ZNeuC(Neu1, 2)\))\)\ 
                                        \(ZNeuC(Neu2, 3)\) + 
                                        \((CW\ SA\ \(ZNeuC(Neu1, 2)\) - 
                                        SA\ SW\ \(ZNeuC(Neu1, 1)\))\)\ 
                                        \(ZNeuC(Neu2, 4)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                    \((8\ CB2\ CW\ CW2\^2\ MW\ SW2\^2)\))\))
                          \) + \(Den(S, MZ2)\)\ 
                        \((\(C0i(cc1, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    3\ Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \)\ \((
                                        CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    3\ Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \)\ \((
                                        CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                                  \(1\/\(2\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \)\ \((
                                        CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                                  \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \)\ \((
                                        CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                                  1\/3\ \((6\ MHp2 + T - U)\)\ 
                                    \((\((3\ Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \)\ \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe1, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                        \((8\ CB2\ CW2\ MW2\ SW2\^2)\) - 
                                        \((3\ Alfa2\ \(MNeu(Neu1)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \)\ \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe1, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)/
                                        \((8\ CB2\ CW2\^2\ MW2\ SW2)\))\) + 
                                  2\ MHp2\ \((
                                    \((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                        \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)/
                                        \((4\ CB2\ CW2\ MW2\ SW2\^2)\) - 
                                      \((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                        \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)/\((4\ CB2\ CW2\^2\ MW2\ SW2)\))
                                    \))\) + 
                            \(C0i(cc2, ME2, S, ME2, MSf2(Sfe1, 2, 1), 
                                MNeu2(Neu1), MNeu2(Neu2))\)\ 
                              \((\(1\/\(4\ CB2\ CW2\^2\ MW2\ SW2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                      \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)\) - 
                                  \(1\/\(4\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                      \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)\) + 
                                  \((S + 2\ U)\)\ 
                                    \((\((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe1, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                        \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)/
                                        \((8\ CB2\ CW2\ MW2\ SW2\^2)\) - 
                                        \((Alfa2\ \(MNeu(Neu2)\)\ 
                                        \((2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                        \((CB\ MW\ SW\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((CB\ MW\ \(USf(Sfe1, 1, 2, 1)
                                        \)\ \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                        \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                        \))\)/\((8\ CB2\ CW2\^2\ MW2\ SW2)\))
                                      \))\))\))\))\)\), ",", 
          RowBox[{
          \(SumOver(Gen1, 3)\), " ", \(SumOver(Sfe1, 2)\), " ", 
            \(SumOver(Sfe2, 2)\), " ", \(SumOver(Sfe3, 2)\), " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(Den(S, MZ2)\), " ", \(Mat(F2)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", 
                        RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\ TB2\)\), 
                          RowBox[{"(", 
                            RowBox[{
                            "Alfa2", " ", \((
                              C0i(cc0, MHp2, S, MHp2, MSf2(Sfe1, 4, Gen1), 
                                  MSf2(Sfe2, 3, Gen1), MSf2(Sfe3, 3, Gen1)) + 
                                C0i(cc1, MHp2, S, MHp2, MSf2(Sfe1, 4, Gen1), 
                                  MSf2(Sfe2, 3, Gen1), MSf2(Sfe3, 3, Gen1)) + 
                                C0i(cc2, MHp2, S, MHp2, MSf2(Sfe1, 4, Gen1), 
                                  MSf2(Sfe2, 3, Gen1), MSf2(Sfe3, 3, Gen1)))
                              \), " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{
                                    RowBox[{"(", 
                                      RowBox[{
                                      \(\(-\((
                                        TB\ \((TB\ \(MQD2(Gen1)\) - MW2\ S2B)
                                        \) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe3, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe3, 2, 3, Gen1)\)}]}], ")"}], 
                                    " ", \(USfC(Sfe1, 1, 4, Gen1)\)}], "-", 
                                  RowBox[{\(MQD(Gen1)\), " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe3, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe3, 2, 3, Gen1)\)\)}], ")"}], 
                                    " ", \(USfC(Sfe1, 2, 4, Gen1)\)}]}], 
                                ")"}], " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{
                                    RowBox[{"(", 
                                      RowBox[{
                                      \(\(-\((
                                        TB\ \((TB\ \(MQD2(Gen1)\) - MW2\ S2B)
                                        \) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 4, Gen1)\)}]}], ")"}], 
                                    " ", \(USfC(Sfe2, 1, 3, Gen1)\)}], "-", 
                                  RowBox[{\(MQU(Gen1)\), " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 4, Gen1)\)\)}], ")"}], 
                                    " ", \(USfC(Sfe2, 2, 3, Gen1)\)}]}], 
                                ")"}], " ", 
                              \((\((4\ SW2 - 3)\)\ \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                    \(USfC(Sfe3, 1, 3, Gen1)\) + 
                                  4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                    \(USfC(Sfe3, 2, 3, Gen1)\))\)}], 
                            ")"}]}]}], "-", 
                      RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\ TB2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", \((
                            C0i(cc0, MHp2, S, MHp2, MSf2(Sfe1, 3, Gen1), 
                                MSf2(Sfe2, 4, Gen1), MSf2(Sfe3, 4, Gen1)) + 
                              C0i(cc1, MHp2, S, MHp2, MSf2(Sfe1, 3, Gen1), 
                                MSf2(Sfe2, 4, Gen1), MSf2(Sfe3, 4, Gen1)) + 
                              C0i(cc2, MHp2, S, MHp2, MSf2(Sfe1, 3, Gen1), 
                                MSf2(Sfe2, 4, Gen1), MSf2(Sfe3, 4, Gen1)))\), 
                            " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\(-\((
                                        TB\ \((TB\ \(MQD2(Gen1)\) - MW2\ S2B)
                                        \) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 4, Gen1)\)\), "-", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 4, Gen1)\)}]}], ")"}], 
                                  " ", \(USfC(Sfe1, 1, 3, Gen1)\)}], "-", 
                                RowBox[{\(MQU(Gen1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe2, 1, 4, Gen1)\)}], "+", 
                                      \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 4, Gen1)\)\)}], ")"}], 
                                  " ", \(USfC(Sfe1, 2, 3, Gen1)\)}]}], ")"}], 
                            " ", \((
                              \((2\ SW2 - 3)\)\ \(USf(Sfe3, 1, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                2\ SW2\ \(USf(Sfe3, 2, 4, Gen1)\)\ 
                                  \(USfC(Sfe2, 2, 4, Gen1)\))\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\(-\((
                                        TB\ \((TB\ \(MQD2(Gen1)\) - MW2\ S2B)
                                        \) + MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 3, Gen1)\)\), "-", 
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 3, Gen1)\)}]}], ")"}], 
                                  " ", \(USfC(Sfe3, 1, 4, Gen1)\)}], "-", 
                                RowBox[{\(MQD(Gen1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 3, Gen1)\)}], "+",
                                       \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 3, Gen1)\)\)}], ")"}], 
                                  " ", \(USfC(Sfe3, 2, 4, Gen1)\)}]}], 
                              ")"}]}], ")"}]}]}], ")"}]}], "+", 
                RowBox[{\(Den(S, MZ2)\), " ", \(Mat(F1)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                      \((C0i(cc0, MHp2, S, MHp2, MSf2(Sfe1, 4, Gen1), 
                              MSf2(Sfe2, 3, Gen1), MSf2(Sfe3, 3, Gen1)) + 
                            C0i(cc1, MHp2, S, MHp2, MSf2(Sfe1, 4, Gen1), 
                              MSf2(Sfe2, 3, Gen1), MSf2(Sfe3, 3, Gen1)) + 
                            C0i(cc2, MHp2, S, MHp2, MSf2(Sfe1, 4, Gen1), 
                              MSf2(Sfe2, 3, Gen1), MSf2(Sfe3, 3, Gen1)))\), 
                        " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(4\ CW2\ MW2\ SW2\^2\ TB2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe3, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe3, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 4, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{\(Af(4, Gen1)\), 
                                        "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe3, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe3, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 4, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 3, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 3, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  \((\((4\ SW2 - 3)\)\ 
                                        \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                        \(USfC(Sfe3, 1, 3, Gen1)\) + 
                                      4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                        \(USfC(Sfe3, 2, 3, Gen1)\))\)}], 
                                ")"}]}], "-", 
                            RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\ TB2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe3, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe3, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 4, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe3, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe3, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 4, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe2, 1, 3, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe2, 2, 3, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  \((\((4\ SW2 - 3)\)\ 
                                        \(USf(Sfe2, 1, 3, Gen1)\)\ 
                                        \(USfC(Sfe3, 1, 3, Gen1)\) + 
                                      4\ SW2\ \(USf(Sfe2, 2, 3, Gen1)\)\ 
                                        \(USfC(Sfe3, 2, 3, Gen1)\))\)}], 
                                ")"}]}]}], ")"}]}], "+", 
                      RowBox[{
                      \((C0i(cc0, MHp2, S, MHp2, MSf2(Sfe1, 3, Gen1), 
                              MSf2(Sfe2, 4, Gen1), MSf2(Sfe3, 4, Gen1)) + 
                            C0i(cc1, MHp2, S, MHp2, MSf2(Sfe1, 3, Gen1), 
                              MSf2(Sfe2, 4, Gen1), MSf2(Sfe3, 4, Gen1)) + 
                            C0i(cc2, MHp2, S, MHp2, MSf2(Sfe1, 3, Gen1), 
                              MSf2(Sfe2, 4, Gen1), MSf2(Sfe3, 4, Gen1)))\), 
                        " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(4\ CW2\ MW2\ SW2\^2\ TB2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 3, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe2, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 3, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  \((\((2\ SW2 - 3)\)\ 
                                        \(USf(Sfe3, 1, 4, Gen1)\)\ 
                                        \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                      2\ SW2\ \(USf(Sfe3, 2, 4, Gen1)\)\ 
                                        \(USfC(Sfe2, 2, 4, Gen1)\))\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe3, 1, 4, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe3, 2, 4, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}], "-", 
                            RowBox[{\(1\/\(2\ CW2\ MW2\ SW2\ TB2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe2, 1, 4, Gen1)\)\), "-", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(TB\ \(Af(4, Gen1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(MQD(Gen1)\), " ", 
                                        \(USf(Sfe2, 2, 4, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 1, 3, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQU(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(MUE\ TB\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(3, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 3, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe2, 1, 4, Gen1)\)}], "+", 
                                        \(\((TB2 + 1)\)\ \(MQD(Gen1)\)\ 
                                        \(USf(Sfe2, 2, 4, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 3, Gen1)\)}]}], 
                                    ")"}], " ", 
                                  \((\((2\ SW2 - 3)\)\ 
                                        \(USf(Sfe3, 1, 4, Gen1)\)\ 
                                        \(USfC(Sfe2, 1, 4, Gen1)\) + 
                                      2\ SW2\ \(USf(Sfe3, 2, 4, Gen1)\)\ 
                                        \(USfC(Sfe2, 2, 4, Gen1)\))\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        \(\(-\((TB\ \((
                                        TB\ \(MQD2(Gen1)\) - MW2\ S2B)\) + 
                                        MQU2(Gen1))\)\)\ 
                                        \(USf(Sfe1, 1, 3, Gen1)\)\), "-", 
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{\(Af(3, Gen1)\), "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(MQU(Gen1)\), " ", 
                                        \(USf(Sfe1, 2, 3, Gen1)\)}]}], ")"}], 
                                        " ", \(USfC(Sfe3, 1, 4, Gen1)\)}], 
                                      "-", 
                                      RowBox[{\(MQD(Gen1)\), " ", 
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(4, Gen1)\), "\[InvisibleSpace]", 
                                        
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 4, Gen1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USf(Sfe1, 1, 3, Gen1)\)}], "+",
                                         \(\((TB2 + 1)\)\ \(MQU(Gen1)\)\ 
                                        \(USf(Sfe1, 2, 3, Gen1)\)\)}], ")"}], 
                                        " ", \(USfC(Sfe3, 2, 4, Gen1)\)}]}], 
                                    ")"}]}], ")"}]}]}], ")"}]}]}], ")"}]}]}], 
              ")"}]}], ",", 
          \(\(SumOver(Cha1, 2)\)\ \(SumOver(Neu1, 4)\)\ \(SumOver(Neu2, 4)\)\ 
            \((\(Den(S, MZ2)\)\ \(Mat(F2)\)\ 
                  \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                        \((\(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, MHp2, S, MHp2, MCha2(Cha1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc1, MHp2, S, MHp2, MCha2(Cha1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                      \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                      \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) - 
                                \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                      \))\)\))\))\) + 
                      \(C0i(cc2, MHp2, S, MHp2, MCha2(Cha1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ MHp2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                      \))\)\) - 
                                \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                      \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                      \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\))\))\))\) + 
                \(Den(S, MZ2)\)\ \(Mat(F1)\)\ 
                  \((\(B0(S, MNeu2(Neu1), MNeu2(Neu2))\)\ 
                        \((\(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, MHp2, S, MHp2, MCha2(Cha1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc1, MHp2, S, MHp2, MCha2(Cha1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                      \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                      \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) - 
                                \(1\/\(4\ CW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ 
                                    \((\(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                      \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) - 
                                \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                      \))\)\) + 
                                \(1\/\(4\ CW2\ SW2\^2\)\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                      \))\)\))\))\) + 
                      \(C0i(cc2, MHp2, S, MHp2, MCha2(Cha1), MNeu2(Neu1), 
                          MNeu2(Neu2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                      \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                    \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MCha2(Cha1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ MHp2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ MHp2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(ZNeu(Neu1, 3)\)\ \(ZNeuC(Neu2, 3)\) - 
                                    \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(-\(1\/\(2\ CW2\ SW2\)\((
                                    Alfa2\ SB2\ \((
                                      \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                        \(ZNeu(Neu2, 3)\)\ 
                                        \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                        \)\ \((
                                        \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\)\) + 
                                \(1\/\(4\ CW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                      \(ZNeu(Neu2, 3)\)\ \(ZNeuC(Neu1, 3)\) - 
                                        \(ZNeu(Neu2, 4)\)\ \(ZNeuC(Neu1, 4)\))
                                      \)\ \((
                                      \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) + 
                                \(1\/\(2\ CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                      \))\)\) - 
                                \(1\/\(4\ CW2\ SW2\^2\)\((
                                  Alfa2\ CB2\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((\(ZNeu(Neu1, 3)\)\ 
                                        \(ZNeuC(Neu2, 3)\) - 
                                        \(ZNeu(Neu1, 4)\)\ \(ZNeuC(Neu2, 4)\))
                                      \))\)\))\))\))\))\)\), ",", 
          \(\(Den(S, MZ2)\)\ \(Mat(F1)\)\ \(SumOver(Cha1, 2)\)\ 
            \(SumOver(Cha2, 2)\)\ 
            \((\(C0i(cc00, ME2, S, ME2, MSf2(1, 1, 1), MCha2(Cha1), 
                    MCha2(Cha2))\)\ 
                  \((\(1\/SW2\^2\((
                        2\ Alfa2\ \(VCha(Cha1, 1)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                          \(VChaC(Cha2, 1)\))\)\) - 
                      \(1\/\(CW2\ SW2\)\((
                        2\ Alfa2\ \(VCha(Cha1, 1)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                          \(VChaC(Cha2, 1)\))\)\))\) + 
                \(B0(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                  \((\(1\/\(CW2\ SW2\)\((
                        Alfa2\ \(VCha(Cha1, 1)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                          \(VChaC(Cha2, 1)\))\)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ \(VCha(Cha1, 1)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                          \(VChaC(Cha2, 1)\))\)\))\) + 
                \(C0i(cc0, ME2, S, ME2, MSf2(1, 1, 1), MCha2(Cha1), 
                    MCha2(Cha2))\)\ 
                  \((\(-\(1\/\(CW2\ SW2\)\((
                          Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                            \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                                1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))
                              \)\ \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\))
                          \)\)\) + 
                      \(1\/SW2\^2\((
                        Alfa2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(UCha(Cha1, 1)\)\ \(UChaC(Cha2, 1)\) - 
                              1\/2\ \(UCha(Cha1, 2)\)\ \(UChaC(Cha2, 2)\))\)\ 
                          \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\))\)\) + 
                      \(1\/\(CW2\ SW2\)\((
                        Alfa2\ \(MSf2(1, 1, 1)\)\ \(VCha(Cha1, 1)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                          \(VChaC(Cha2, 1)\))\)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ \(MSf2(1, 1, 1)\)\ \(VCha(Cha1, 1)\)\ 
                          \((SW2\ \(Delta(Cha1, Cha2)\) - 
                              \(VCha(Cha2, 1)\)\ \(VChaC(Cha1, 1)\) - 
                              1\/2\ \(VCha(Cha2, 2)\)\ \(VChaC(Cha1, 2)\))\)\ 
                          \(VChaC(Cha2, 1)\))\)\))\))\)\), ",", 
          \(\(SumOver(Cha1, 2)\)\ \(SumOver(Cha2, 2)\)\ \(SumOver(Neu1, 4)\)\ 
            \((\(Den(S, MZ2)\)\ \(Mat(F2)\)\ 
                  \((\(B0(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                        \((\(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                  1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                                \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc2, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ CB2\ MHp2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                  1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                                \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ MHp2\ SB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                  \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(-\(1\/\(CW2\ SW2\)\((
                                    Alfa2\ SB2\ \((
                                      SW2\ \(Delta(Cha1, Cha2)\) - 
                                        \(UCha(Cha2, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                      \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                        \)\ \((
                                        \(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\)\) - 
                                \(1\/\(CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\)\ 
                                    \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\) + 
                      \(C0i(cc1, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/\(CW2\ SW2\)\((
                                  Alfa2\ SB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      \(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\) + 
                                \(1\/\(CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\)\ 
                                    \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\))\) + 
                \(Den(S, MZ2)\)\ \(Mat(F1)\)\ 
                  \((\(B0(S, MCha2(Cha1), MCha2(Cha2))\)\ 
                        \((\(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                  \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                  \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                  1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                                \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                  1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                                \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc2, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ CB2\ MHp2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                  1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                                \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              2\ Alfa2\ CB2\ MHp2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                  1\/2\ \(VCha(Cha1, 2)\)\ \(VChaC(Cha2, 2)\))
                                \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              4\ Alfa2\ MHp2\ SB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                  \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              2\ Alfa2\ MHp2\ SB2\ \((
                                SW2\ \(Delta(Cha1, Cha2)\) - 
                                  \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                  1\/2\ \(UCha(Cha2, 2)\)\ \(UChaC(Cha1, 2)\))
                                \)\ \((
                                  \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(-\(1\/\(CW2\ SW2\)\((
                                    Alfa2\ SB2\ \((
                                      SW2\ \(Delta(Cha1, Cha2)\) - 
                                        \(UCha(Cha2, 1)\)\ 
                                        \(UChaC(Cha1, 1)\) - 
                                        1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                      \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                        \)\ \((
                                        \(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\)\) + 
                                \(1\/\(2\ CW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      \(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\) - 
                                \(1\/\(CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\)\ 
                                    \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\) + 
                                \(1\/\(2\ CW2\ SW2\^2\)\((
                                  Alfa2\ CB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\)\ 
                                    \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\) + 
                      \(C0i(cc1, MHp2, S, MHp2, MNeu2(Neu1), MCha2(Cha1), 
                          MCha2(Cha2))\)\ 
                        \((\(-\(1\/\(CW2\ SW2\)\((
                                2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                  \(MNeu(Neu1)\)\ 
                                  \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                                \)\)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu2(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                    1\/2\ \(UCha(Cha2, 2)\)\ 
                                      \(UChaC(Cha1, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ SW2\)\((
                              2\ Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ 
                                \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(CW2\ SW2\^2\)\((
                              Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                                \((SW2\ \(Delta(Cha1, Cha2)\) - 
                                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                    1\/2\ \(VCha(Cha1, 2)\)\ 
                                      \(VChaC(Cha2, 2)\))\)\ 
                                \((\(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            2\ MHp2\ \((
                              \(1\/\(CW2\ SW2\)\((
                                  Alfa2\ SB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      \(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\) - 
                                \(1\/\(2\ CW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(UCha(Cha2, 1)\)\ \(UChaC(Cha1, 1)\) - 
                                      1\/2\ \(UCha(Cha2, 2)\)\ 
                                        \(UChaC(Cha1, 2)\))\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      \(\(UChaC(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\) + 
                                \(1\/\(CW2\ SW2\)\((
                                  Alfa2\ CB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\)\ 
                                    \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\) - 
                                \(1\/\(2\ CW2\ SW2\^2\)\((
                                  Alfa2\ CB2\ \((
                                    SW2\ \(Delta(Cha1, Cha2)\) - 
                                      \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\) - 
                                      1\/2\ \(VCha(Cha1, 2)\)\ 
                                        \(VChaC(Cha2, 2)\))\)\ 
                                    \((\(\(VCha(Cha2, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\))\))
              \)\)}], ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(tops\  = \ 
      CreateTopologies[1, \ 2\  -> \ 2, \n\ \ 
        ExcludeTopologies\  -> 
          \ {Tadpoles, \ WFCorrections, \ SelfEnergies, \ Triangles}]; \)\n
     (*\ ins\  = \ 
        DiagramSelect[inss, \ FreeQ[#, \ Field[6 | 7]\  -> \ V[1]]&]; \ *) 
      \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ins\  = \ inss; \)\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 7 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 7 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 3, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 4, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]5
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 5, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]6
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 6, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]7
          \[InvisibleSpace]": "\[InvisibleSpace]"3 Generic, 10 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 7, ": ", "3 Generic, 10 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]8
          \[InvisibleSpace]": "\[InvisibleSpace]"3 Generic, 6 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 8, ": ", "3 Generic, 6 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]9
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 2 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 9, ": ", "2 Generic, 2 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]10
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 10, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]11
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 11, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]12
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 12, ": ", "0 Generic, 0 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"10 Generic, 25 Classes insertions"\),
        SequenceForm[ "in total: ", "10 Generic, 25 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DoPaint[ins, \ "\<box\>"]; \)\)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"7 diagrams"\),
        SequenceForm[ "> Top. ", 1, ": ", "7 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"10 diagrams"\),
        SequenceForm[ "> Top. ", 2, ": ", "10 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"6 diagrams"\),
        SequenceForm[ "> Top. ", 3, ": ", "6 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"2 diagrams"\),
        SequenceForm[ "> Top. ", 4, ": ", "2 diagrams"],
        Editable->False], TraditionalForm]], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .59091 m
.68182 .5 L
s
.47354 .55207 m
.52205 .5212 L
.53087 .55648 L
F
gsave
.48291 .50459 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .5 L
s
.52646 .46116 m
.47795 .43029 L
.46913 .46557 L
F
gsave
.51709 .41369 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N1) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
gsave
.28658 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .59091 m
.68182 .5 L
s
.47354 .55207 m
.52205 .5212 L
.53087 .55648 L
F
gsave
.48291 .50459 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .5 L
s
.52646 .46116 m
.47795 .43029 L
.46913 .46557 L
F
gsave
.51709 .41369 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N2) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67831 .50483 L
.6747 .50923 L
.67088 .5128 L
.66678 .51525 L
.66237 .51643 L
.65763 .51633 L
.65261 .51509 L
.64737 .51298 L
.64201 .51039 L
.63664 .50775 L
.63137 .5055 L
.62629 .50404 L
.62148 .50367 L
.61699 .50456 L
.61283 .50674 L
.60895 .51008 L
.6053 .51432 L
.60178 .5191 L
.59829 .52397 L
.59471 .5285 L
.59094 .53228 L
.58691 .53501 L
.58257 .53648 L
.5779 .53665 L
.57293 .53564 L
.56774 .5337 L
.5624 .53118 L
.55702 .52851 L
.55171 .52614 L
.54658 .52447 L
.54171 .52384 L
.53715 .52444 L
.53292 .52634 L
.52898 .52945 L
.52529 .53352 L
.52175 .53821 L
.51826 .5431 L
.51471 .54774 L
.51099 .55172 L
.50702 .5547 L
.50275 .55646 L
.49815 .55692 L
.49325 .55616 L
.4881 .5544 L
.48278 .55197 L
.4774 .5493 L
.47207 .54682 L
.4669 .54497 L
.46196 .54408 L
Mistroke
.45733 .5444 L
.45302 .54601 L
.44902 .54886 L
.44528 .55275 L
.44172 .55734 L
.43823 .56222 L
.4347 .56696 L
.43103 .57112 L
.42712 .57435 L
.42292 .57639 L
.4184 .57713 L
.41356 .57663 L
.40846 .57506 L
.40317 .57275 L
.39779 .5701 L
.39244 .56754 L
.38722 .56551 L
.38223 .56439 L
.37752 .56443 L
.37314 .56575 L
.36908 .56834 L
.36529 .57202 L
.36169 .57649 L
.3582 .58135 L
.35469 .58615 L
.35106 .59048 L
.34721 .59394 L
.34308 .59626 L
.33863 .59729 L
.33386 .59706 L
.32881 .5957 L
.32355 .59353 L
.31818 .59091 L
Mfstroke
gsave
.48291 .50459 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .5 m
.67645 .50262 L
.67119 .50479 L
.66614 .50615 L
.66137 .50638 L
.65692 .50535 L
.65279 .50303 L
.64894 .49957 L
.64531 .49525 L
.6418 .49044 L
.63831 .48558 L
.63471 .48111 L
.63092 .47743 L
.62686 .47484 L
.62248 .47352 L
.61777 .47348 L
.61278 .47461 L
.60756 .47663 L
.60221 .47919 L
.59683 .48184 L
.59154 .48416 L
.58644 .48572 L
.5816 .48623 L
.57708 .48548 L
.57288 .48344 L
.56897 .48021 L
.5653 .47605 L
.56177 .47131 L
.55828 .46643 L
.55472 .46184 L
.55098 .45796 L
.54698 .4551 L
.54267 .45349 L
.53804 .45317 L
.5331 .45406 L
.52793 .45591 L
.5226 .45839 L
.51722 .46106 L
.5119 .46349 L
.50675 .46525 L
.50185 .46601 L
.49725 .46555 L
.49298 .46379 L
.48901 .46081 L
.48529 .45683 L
.48174 .45219 L
.47825 .4473 L
.47471 .44261 L
.47102 .43854 L
.46708 .43543 L
Mistroke
.46285 .43353 L
.45829 .43293 L
.45342 .43357 L
.44829 .43523 L
.44298 .4376 L
.4376 .44027 L
.43226 .44279 L
.42707 .44473 L
.4221 .44574 L
.41743 .44557 L
.41309 .4441 L
.40906 .44138 L
.40529 .43759 L
.40171 .43306 L
.39822 .42819 L
.3947 .42342 L
.39105 .41917 L
.38717 .41583 L
.38301 .41365 L
.37852 .41276 L
.37371 .41313 L
.36863 .41459 L
.36336 .41684 L
.35799 .41948 L
.35263 .42207 L
.34739 .42418 L
.34237 .42542 L
.33763 .42553 L
.33322 .42435 L
.32912 .42189 L
.3253 .41832 L
.32169 .41393 L
.31818 .40909 L
Mfstroke
gsave
.51709 .41369 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N3) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67831 .50483 L
.6747 .50923 L
.67088 .5128 L
.66678 .51525 L
.66237 .51643 L
.65763 .51633 L
.65261 .51509 L
.64737 .51298 L
.64201 .51039 L
.63664 .50775 L
.63137 .5055 L
.62629 .50404 L
.62148 .50367 L
.61699 .50456 L
.61283 .50674 L
.60895 .51008 L
.6053 .51432 L
.60178 .5191 L
.59829 .52397 L
.59471 .5285 L
.59094 .53228 L
.58691 .53501 L
.58257 .53648 L
.5779 .53665 L
.57293 .53564 L
.56774 .5337 L
.5624 .53118 L
.55702 .52851 L
.55171 .52614 L
.54658 .52447 L
.54171 .52384 L
.53715 .52444 L
.53292 .52634 L
.52898 .52945 L
.52529 .53352 L
.52175 .53821 L
.51826 .5431 L
.51471 .54774 L
.51099 .55172 L
.50702 .5547 L
.50275 .55646 L
.49815 .55692 L
.49325 .55616 L
.4881 .5544 L
.48278 .55197 L
.4774 .5493 L
.47207 .54682 L
.4669 .54497 L
.46196 .54408 L
Mistroke
.45733 .5444 L
.45302 .54601 L
.44902 .54886 L
.44528 .55275 L
.44172 .55734 L
.43823 .56222 L
.4347 .56696 L
.43103 .57112 L
.42712 .57435 L
.42292 .57639 L
.4184 .57713 L
.41356 .57663 L
.40846 .57506 L
.40317 .57275 L
.39779 .5701 L
.39244 .56754 L
.38722 .56551 L
.38223 .56439 L
.37752 .56443 L
.37314 .56575 L
.36908 .56834 L
.36529 .57202 L
.36169 .57649 L
.3582 .58135 L
.35469 .58615 L
.35106 .59048 L
.34721 .59394 L
.34308 .59626 L
.33863 .59729 L
.33386 .59706 L
.32881 .5957 L
.32355 .59353 L
.31818 .59091 L
Mfstroke
gsave
.48291 .50459 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .5 m
.67645 .50262 L
.67119 .50479 L
.66614 .50615 L
.66137 .50638 L
.65692 .50535 L
.65279 .50303 L
.64894 .49957 L
.64531 .49525 L
.6418 .49044 L
.63831 .48558 L
.63471 .48111 L
.63092 .47743 L
.62686 .47484 L
.62248 .47352 L
.61777 .47348 L
.61278 .47461 L
.60756 .47663 L
.60221 .47919 L
.59683 .48184 L
.59154 .48416 L
.58644 .48572 L
.5816 .48623 L
.57708 .48548 L
.57288 .48344 L
.56897 .48021 L
.5653 .47605 L
.56177 .47131 L
.55828 .46643 L
.55472 .46184 L
.55098 .45796 L
.54698 .4551 L
.54267 .45349 L
.53804 .45317 L
.5331 .45406 L
.52793 .45591 L
.5226 .45839 L
.51722 .46106 L
.5119 .46349 L
.50675 .46525 L
.50185 .46601 L
.49725 .46555 L
.49298 .46379 L
.48901 .46081 L
.48529 .45683 L
.48174 .45219 L
.47825 .4473 L
.47471 .44261 L
.47102 .43854 L
.46708 .43543 L
Mistroke
.46285 .43353 L
.45829 .43293 L
.45342 .43357 L
.44829 .43523 L
.44298 .4376 L
.4376 .44027 L
.43226 .44279 L
.42707 .44473 L
.4221 .44574 L
.41743 .44557 L
.41309 .4441 L
.40906 .44138 L
.40529 .43759 L
.40171 .43306 L
.39822 .42819 L
.3947 .42342 L
.39105 .41917 L
.38717 .41583 L
.38301 .41365 L
.37852 .41276 L
.37371 .41313 L
.36863 .41459 L
.36336 .41684 L
.35799 .41948 L
.35263 .42207 L
.34739 .42418 L
.34237 .42542 L
.33763 .42553 L
.33322 .42435 L
.32912 .42189 L
.3253 .41832 L
.32169 .41393 L
.31818 .40909 L
Mfstroke
gsave
.51709 .41369 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N4) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67831 .50483 L
.6747 .50923 L
.67088 .5128 L
.66678 .51525 L
.66237 .51643 L
.65763 .51633 L
.65261 .51509 L
.64737 .51298 L
.64201 .51039 L
.63664 .50775 L
.63137 .5055 L
.62629 .50404 L
.62148 .50367 L
.61699 .50456 L
.61283 .50674 L
.60895 .51008 L
.6053 .51432 L
.60178 .5191 L
.59829 .52397 L
.59471 .5285 L
.59094 .53228 L
.58691 .53501 L
.58257 .53648 L
.5779 .53665 L
.57293 .53564 L
.56774 .5337 L
.5624 .53118 L
.55702 .52851 L
.55171 .52614 L
.54658 .52447 L
.54171 .52384 L
.53715 .52444 L
.53292 .52634 L
.52898 .52945 L
.52529 .53352 L
.52175 .53821 L
.51826 .5431 L
.51471 .54774 L
.51099 .55172 L
.50702 .5547 L
.50275 .55646 L
.49815 .55692 L
.49325 .55616 L
.4881 .5544 L
.48278 .55197 L
.4774 .5493 L
.47207 .54682 L
.4669 .54497 L
.46196 .54408 L
Mistroke
.45733 .5444 L
.45302 .54601 L
.44902 .54886 L
.44528 .55275 L
.44172 .55734 L
.43823 .56222 L
.4347 .56696 L
.43103 .57112 L
.42712 .57435 L
.42292 .57639 L
.4184 .57713 L
.41356 .57663 L
.40846 .57506 L
.40317 .57275 L
.39779 .5701 L
.39244 .56754 L
.38722 .56551 L
.38223 .56439 L
.37752 .56443 L
.37314 .56575 L
.36908 .56834 L
.36529 .57202 L
.36169 .57649 L
.3582 .58135 L
.35469 .58615 L
.35106 .59048 L
.34721 .59394 L
.34308 .59626 L
.33863 .59729 L
.33386 .59706 L
.32881 .5957 L
.32355 .59353 L
.31818 .59091 L
Mfstroke
gsave
.48291 .50459 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67645 .50262 L
.67119 .50479 L
.66614 .50615 L
.66137 .50638 L
.65692 .50535 L
.65279 .50303 L
.64894 .49957 L
.64531 .49525 L
.6418 .49044 L
.63831 .48558 L
.63471 .48111 L
.63092 .47743 L
.62686 .47484 L
.62248 .47352 L
.61777 .47348 L
.61278 .47461 L
.60756 .47663 L
.60221 .47919 L
.59683 .48184 L
.59154 .48416 L
.58644 .48572 L
.5816 .48623 L
.57708 .48548 L
.57288 .48344 L
.56897 .48021 L
.5653 .47605 L
.56177 .47131 L
.55828 .46643 L
.55472 .46184 L
.55098 .45796 L
.54698 .4551 L
.54267 .45349 L
.53804 .45317 L
.5331 .45406 L
.52793 .45591 L
.5226 .45839 L
.51722 .46106 L
.5119 .46349 L
.50675 .46525 L
.50185 .46601 L
.49725 .46555 L
.49298 .46379 L
.48901 .46081 L
.48529 .45683 L
.48174 .45219 L
.47825 .4473 L
.47471 .44261 L
.47102 .43854 L
.46708 .43543 L
Mistroke
.46285 .43353 L
.45829 .43293 L
.45342 .43357 L
.44829 .43523 L
.44298 .4376 L
.4376 .44027 L
.43226 .44279 L
.42707 .44473 L
.4221 .44574 L
.41743 .44557 L
.41309 .4441 L
.40906 .44138 L
.40529 .43759 L
.40171 .43306 L
.39822 .42819 L
.3947 .42342 L
.39105 .41917 L
.38717 .41583 L
.38301 .41365 L
.37852 .41276 L
.37371 .41313 L
.36863 .41459 L
.36336 .41684 L
.35799 .41948 L
.35263 .42207 L
.34739 .42418 L
.34237 .42542 L
.33763 .42553 L
.33322 .42435 L
.32912 .42189 L
.3253 .41832 L
.32169 .41393 L
.31818 .40909 L
Mfstroke
gsave
.51709 .41369 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N5) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.75 0.5 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67831 .50483 L
.6747 .50923 L
.67088 .5128 L
.66678 .51525 L
.66237 .51643 L
.65763 .51633 L
.65261 .51509 L
.64737 .51298 L
.64201 .51039 L
.63664 .50775 L
.63137 .5055 L
.62629 .50404 L
.62148 .50367 L
.61699 .50456 L
.61283 .50674 L
.60895 .51008 L
.6053 .51432 L
.60178 .5191 L
.59829 .52397 L
.59471 .5285 L
.59094 .53228 L
.58691 .53501 L
.58257 .53648 L
.5779 .53665 L
.57293 .53564 L
.56774 .5337 L
.5624 .53118 L
.55702 .52851 L
.55171 .52614 L
.54658 .52447 L
.54171 .52384 L
.53715 .52444 L
.53292 .52634 L
.52898 .52945 L
.52529 .53352 L
.52175 .53821 L
.51826 .5431 L
.51471 .54774 L
.51099 .55172 L
.50702 .5547 L
.50275 .55646 L
.49815 .55692 L
.49325 .55616 L
.4881 .5544 L
.48278 .55197 L
.4774 .5493 L
.47207 .54682 L
.4669 .54497 L
.46196 .54408 L
Mistroke
.45733 .5444 L
.45302 .54601 L
.44902 .54886 L
.44528 .55275 L
.44172 .55734 L
.43823 .56222 L
.4347 .56696 L
.43103 .57112 L
.42712 .57435 L
.42292 .57639 L
.4184 .57713 L
.41356 .57663 L
.40846 .57506 L
.40317 .57275 L
.39779 .5701 L
.39244 .56754 L
.38722 .56551 L
.38223 .56439 L
.37752 .56443 L
.37314 .56575 L
.36908 .56834 L
.36529 .57202 L
.36169 .57649 L
.3582 .58135 L
.35469 .58615 L
.35106 .59048 L
.34721 .59394 L
.34308 .59626 L
.33863 .59729 L
.33386 .59706 L
.32881 .5957 L
.32355 .59353 L
.31818 .59091 L
Mfstroke
gsave
.48291 .50459 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67645 .50262 L
.67119 .50479 L
.66614 .50615 L
.66137 .50638 L
.65692 .50535 L
.65279 .50303 L
.64894 .49957 L
.64531 .49525 L
.6418 .49044 L
.63831 .48558 L
.63471 .48111 L
.63092 .47743 L
.62686 .47484 L
.62248 .47352 L
.61777 .47348 L
.61278 .47461 L
.60756 .47663 L
.60221 .47919 L
.59683 .48184 L
.59154 .48416 L
.58644 .48572 L
.5816 .48623 L
.57708 .48548 L
.57288 .48344 L
.56897 .48021 L
.5653 .47605 L
.56177 .47131 L
.55828 .46643 L
.55472 .46184 L
.55098 .45796 L
.54698 .4551 L
.54267 .45349 L
.53804 .45317 L
.5331 .45406 L
.52793 .45591 L
.5226 .45839 L
.51722 .46106 L
.5119 .46349 L
.50675 .46525 L
.50185 .46601 L
.49725 .46555 L
.49298 .46379 L
.48901 .46081 L
.48529 .45683 L
.48174 .45219 L
.47825 .4473 L
.47471 .44261 L
.47102 .43854 L
.46708 .43543 L
Mistroke
.46285 .43353 L
.45829 .43293 L
.45342 .43357 L
.44829 .43523 L
.44298 .4376 L
.4376 .44027 L
.43226 .44279 L
.42707 .44473 L
.4221 .44574 L
.41743 .44557 L
.41309 .4441 L
.40906 .44138 L
.40529 .43759 L
.40171 .43306 L
.39822 .42819 L
.3947 .42342 L
.39105 .41917 L
.38717 .41583 L
.38301 .41365 L
.37852 .41276 L
.37371 .41313 L
.36863 .41459 L
.36336 .41684 L
.35799 .41948 L
.35263 .42207 L
.34739 .42418 L
.34237 .42542 L
.33763 .42553 L
.33322 .42435 L
.32912 .42189 L
.3253 .41832 L
.32169 .41393 L
.31818 .40909 L
Mfstroke
gsave
.51709 .41369 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N6) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.75 0.75 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .5 L
s
.79723 .59618 m
.82749 .64506 L
.85077 .61713 L
F
gsave
.80057 .64027 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .5 L
s
.83913 .3689 m
.80887 .41779 L
.78559 .38986 L
F
gsave
.83579 .413 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .5 m
.67831 .50483 L
.6747 .50923 L
.67088 .5128 L
.66678 .51525 L
.66237 .51643 L
.65763 .51633 L
.65261 .51509 L
.64737 .51298 L
.64201 .51039 L
.63664 .50775 L
.63137 .5055 L
.62629 .50404 L
.62148 .50367 L
.61699 .50456 L
.61283 .50674 L
.60895 .51008 L
.6053 .51432 L
.60178 .5191 L
.59829 .52397 L
.59471 .5285 L
.59094 .53228 L
.58691 .53501 L
.58257 .53648 L
.5779 .53665 L
.57293 .53564 L
.56774 .5337 L
.5624 .53118 L
.55702 .52851 L
.55171 .52614 L
.54658 .52447 L
.54171 .52384 L
.53715 .52444 L
.53292 .52634 L
.52898 .52945 L
.52529 .53352 L
.52175 .53821 L
.51826 .5431 L
.51471 .54774 L
.51099 .55172 L
.50702 .5547 L
.50275 .55646 L
.49815 .55692 L
.49325 .55616 L
.4881 .5544 L
.48278 .55197 L
.4774 .5493 L
.47207 .54682 L
.4669 .54497 L
.46196 .54408 L
Mistroke
.45733 .5444 L
.45302 .54601 L
.44902 .54886 L
.44528 .55275 L
.44172 .55734 L
.43823 .56222 L
.4347 .56696 L
.43103 .57112 L
.42712 .57435 L
.42292 .57639 L
.4184 .57713 L
.41356 .57663 L
.40846 .57506 L
.40317 .57275 L
.39779 .5701 L
.39244 .56754 L
.38722 .56551 L
.38223 .56439 L
.37752 .56443 L
.37314 .56575 L
.36908 .56834 L
.36529 .57202 L
.36169 .57649 L
.3582 .58135 L
.35469 .58615 L
.35106 .59048 L
.34721 .59394 L
.34308 .59626 L
.33863 .59729 L
.33386 .59706 L
.32881 .5957 L
.32355 .59353 L
.31818 .59091 L
Mfstroke
.47354 .55207 m
.52205 .5212 L
.53087 .55648 L
F
gsave
.48291 .50459 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .5 m
.67645 .50262 L
.67119 .50479 L
.66614 .50615 L
.66137 .50638 L
.65692 .50535 L
.65279 .50303 L
.64894 .49957 L
.64531 .49525 L
.6418 .49044 L
.63831 .48558 L
.63471 .48111 L
.63092 .47743 L
.62686 .47484 L
.62248 .47352 L
.61777 .47348 L
.61278 .47461 L
.60756 .47663 L
.60221 .47919 L
.59683 .48184 L
.59154 .48416 L
.58644 .48572 L
.5816 .48623 L
.57708 .48548 L
.57288 .48344 L
.56897 .48021 L
.5653 .47605 L
.56177 .47131 L
.55828 .46643 L
.55472 .46184 L
.55098 .45796 L
.54698 .4551 L
.54267 .45349 L
.53804 .45317 L
.5331 .45406 L
.52793 .45591 L
.5226 .45839 L
.51722 .46106 L
.5119 .46349 L
.50675 .46525 L
.50185 .46601 L
.49725 .46555 L
.49298 .46379 L
.48901 .46081 L
.48529 .45683 L
.48174 .45219 L
.47825 .4473 L
.47471 .44261 L
.47102 .43854 L
.46708 .43543 L
Mistroke
.46285 .43353 L
.45829 .43293 L
.45342 .43357 L
.44829 .43523 L
.44298 .4376 L
.4376 .44027 L
.43226 .44279 L
.42707 .44473 L
.4221 .44574 L
.41743 .44557 L
.41309 .4441 L
.40906 .44138 L
.40529 .43759 L
.40171 .43306 L
.39822 .42819 L
.3947 .42342 L
.39105 .41917 L
.38717 .41583 L
.38301 .41365 L
.37852 .41276 L
.37371 .41313 L
.36863 .41459 L
.36336 .41684 L
.35799 .41948 L
.35263 .42207 L
.34739 .42418 L
.34237 .42542 L
.33763 .42553 L
.33322 .42435 L
.32912 .42189 L
.3253 .41832 L
.32169 .41393 L
.31818 .40909 L
Mfstroke
.52646 .46116 m
.47795 .43029 L
.46913 .46557 L
F
gsave
.51709 .41369 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .5 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T1) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N7) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.75 1 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.68182 .59091 L
s
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .40909 L
s
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.68182 .40909 L
s
gsave
.65022 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N8) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.5 0.25 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.68182 .59091 L
s
gsave
.5 .55931 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .40909 L
s
gsave
.5 .37749 -66.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -74 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N9) show
84.125000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.5 0.5 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
gsave
.28658 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .59091 m
.68182 .59091 L
s
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .40909 L
s
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -65.5 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.68182 .40909 L
s
.68182 .52727 m
.66364 .47273 L
.7 .47273 L
F
gsave
.63885 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N10) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.5 0.75 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
gsave
.5 .54795 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
gsave
.5 .36613 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N11) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.5 1 0.75 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
gsave
.5 .54795 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
gsave
.5 .36613 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N12) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.25 0.25 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
gsave
.5 .54795 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
gsave
.5 .36613 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N13) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0.25 0.5 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
gsave
.5 .54795 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
gsave
.5 .36613 -65 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N14) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0.25 0.75 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
gsave
.65022 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(h) show
66.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C5) show
77.812500 14.625000 moveto
(N15) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0.25 1 0.5 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
gsave
.65022 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C6) show
77.812500 14.625000 moveto
(N16) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 0.25 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.67727 .59484 L
.67273 .59829 L
.66818 .60082 L
.66363 .60213 L
.65909 .60205 L
.65454 .60059 L
.65 .59794 L
.64545 .59441 L
.64091 .59046 L
.63636 .58655 L
.63182 .58319 L
.62727 .58077 L
.62273 .57961 L
.61818 .57985 L
.61364 .58145 L
.60909 .58422 L
.60455 .58781 L
.6 .59179 L
.59545 .59565 L
.59091 .59893 L
.58636 .60121 L
.58182 .60222 L
.57727 .60183 L
.57273 .60009 L
.56818 .59721 L
.56364 .59355 L
.55909 .58956 L
.55455 .58573 L
.55 .58255 L
.54546 .58039 L
.54091 .57954 L
.53636 .58008 L
.53182 .58197 L
.52727 .58495 L
.52273 .58867 L
.51818 .59267 L
.51364 .59644 L
.50909 .59953 L
.50455 .60155 L
.5 .60225 L
.49545 .60155 L
.49091 .59953 L
.48636 .59644 L
.48182 .59267 L
.47727 .58867 L
.47273 .58495 L
.46818 .58197 L
.46364 .58008 L
.45909 .57954 L
Mistroke
.45454 .58039 L
.45 .58255 L
.44545 .58573 L
.44091 .58956 L
.43636 .59355 L
.43182 .59721 L
.42727 .60009 L
.42273 .60183 L
.41818 .60222 L
.41364 .60121 L
.40909 .59893 L
.40455 .59565 L
.4 .59179 L
.39545 .58781 L
.39091 .58422 L
.38636 .58145 L
.38182 .57985 L
.37727 .57961 L
.37273 .58077 L
.36818 .58319 L
.36364 .58655 L
.35909 .59046 L
.35455 .59441 L
.35 .59794 L
.34546 .60059 L
.34091 .60205 L
.33637 .60213 L
.33182 .60082 L
.32727 .59829 L
.32273 .59484 L
.31818 .59091 L
Mfstroke
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67727 .41302 L
.67273 .41647 L
.66818 .419 L
.66363 .42031 L
.65909 .42023 L
.65454 .41877 L
.65 .41612 L
.64545 .4126 L
.64091 .40864 L
.63636 .40473 L
.63182 .40137 L
.62727 .39896 L
.62273 .3978 L
.61818 .39803 L
.61364 .39963 L
.60909 .4024 L
.60455 .40599 L
.6 .40997 L
.59545 .41384 L
.59091 .41711 L
.58636 .4194 L
.58182 .42041 L
.57727 .42001 L
.57273 .41827 L
.56818 .41539 L
.56364 .41173 L
.55909 .40774 L
.55455 .40392 L
.55 .40073 L
.54546 .39858 L
.54091 .39772 L
.53636 .39827 L
.53182 .40015 L
.52727 .40314 L
.52273 .40686 L
.51818 .41085 L
.51364 .41463 L
.50909 .41771 L
.50455 .41973 L
.5 .42044 L
.49545 .41973 L
.49091 .41771 L
.48636 .41463 L
.48182 .41085 L
.47727 .40686 L
.47273 .40314 L
.46818 .40015 L
.46364 .39827 L
.45909 .39772 L
Mistroke
.45454 .39858 L
.45 .40073 L
.44545 .40392 L
.44091 .40774 L
.43636 .41173 L
.43182 .41539 L
.42727 .41827 L
.42273 .42001 L
.41818 .42041 L
.41364 .4194 L
.40909 .41711 L
.40455 .41384 L
.4 .40997 L
.39545 .40599 L
.39091 .4024 L
.38636 .39963 L
.38182 .39803 L
.37727 .3978 L
.37273 .39896 L
.36818 .40137 L
.36364 .40473 L
.35909 .40864 L
.35455 .4126 L
.35 .41612 L
.34546 .41877 L
.34091 .42023 L
.33637 .42031 L
.33182 .419 L
.32727 .41647 L
.32273 .41302 L
.31818 .40909 L
Mfstroke
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(W) show
69.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
gsave
.65022 .5 -72 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(A) show
66.937500 12.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T2) show
70.187500 14.625000 moveto
(C7) show
77.812500 14.625000 moveto
(N17) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 0 0.5 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.68182 .40909 L
s
gsave
.50106 .47464 -72 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .59091 L
s
gsave
.49894 .47464 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.187500 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(j) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N18) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 0 0.75 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .59091 m
.68182 .40909 L
s
.47561 .5122 m
.51626 .47154 L
.53252 .50407 L
F
gsave
.49598 .46447 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.31818 .40909 m
.68182 .59091 L
s
.52439 .5122 m
.48374 .47154 L
.46748 .50407 L
F
gsave
.50402 .46447 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -71 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N19) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 0 1 0.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67932 .41431 L
.67666 .41921 L
.6737 .4235 L
.67033 .42697 L
.66647 .42946 L
.6621 .43094 L
.65725 .43146 L
.652 .43117 L
.64646 .4303 L
.64078 .42915 L
.63511 .42804 L
.62963 .42728 L
.62445 .42715 L
.61971 .42787 L
.61545 .42957 L
.6117 .43229 L
.60843 .43596 L
.60555 .44041 L
.60294 .44541 L
.60045 .45065 L
.59793 .45583 L
.59522 .46061 L
.59218 .46474 L
.5887 .468 L
.58473 .47027 L
.58025 .47153 L
.5753 .47185 L
.56997 .47141 L
.56439 .47045 L
.5587 .46929 L
.55306 .46822 L
.54763 .46758 L
.54255 .46763 L
.53791 .46856 L
.53376 .47049 L
.53012 .47343 L
.52695 .47728 L
.52414 .48188 L
.52157 .48696 L
.51909 .49222 L
.51654 .49733 L
.51377 .502 L
.51063 .50595 L
.50705 .50899 L
.50296 .51104 L
.49838 .51208 L
.49334 .51221 L
.48794 .51163 L
.48232 .5106 L
Mistroke
.47663 .50943 L
.47102 .50844 L
.46566 .50793 L
.46067 .50815 L
.45613 .5093 L
.4521 .51146 L
.44857 .51461 L
.44549 .51865 L
.44274 .52338 L
.44021 .52853 L
.43773 .53378 L
.43514 .53882 L
.4323 .54335 L
.42907 .54711 L
.42538 .54994 L
.42118 .55176 L
.41649 .55259 L
.41136 .55254 L
.4059 .55184 L
.40025 .55075 L
.39456 .54959 L
.389 .54868 L
.38371 .54832 L
.37881 .54874 L
.37439 .55011 L
.37047 .55249 L
.36705 .55585 L
.36405 .56007 L
.36136 .56491 L
.35886 .57011 L
.35637 .57535 L
.35374 .5803 L
.35082 .58467 L
.34749 .58824 L
.34369 .59085 L
.33938 .59244 L
.33458 .59306 L
.32937 .59285 L
.32386 .59204 L
.31818 .59091 L
Mfstroke
gsave
.49598 .46447 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.67614 .59204 L
.67063 .59285 L
.66542 .59306 L
.66062 .59244 L
.65631 .59085 L
.65251 .58824 L
.64918 .58467 L
.64626 .5803 L
.64363 .57535 L
.64114 .57011 L
.63864 .56491 L
.63595 .56007 L
.63295 .55585 L
.62953 .55249 L
.62561 .55011 L
.62119 .54874 L
.61629 .54832 L
.611 .54868 L
.60544 .54959 L
.59975 .55075 L
.5941 .55184 L
.58864 .55254 L
.58351 .55259 L
.57882 .55176 L
.57462 .54994 L
.57093 .54711 L
.5677 .54335 L
.56486 .53882 L
.56227 .53378 L
.55979 .52853 L
.55726 .52338 L
.55451 .51865 L
.55143 .51461 L
.5479 .51146 L
.54387 .5093 L
.53933 .50815 L
.53434 .50793 L
.52898 .50844 L
.52337 .50943 L
.51768 .5106 L
.51206 .51163 L
.50666 .51221 L
.50162 .51208 L
.49704 .51104 L
.49295 .50899 L
.48937 .50595 L
.48623 .502 L
.48346 .49733 L
.48091 .49222 L
Mistroke
.47843 .48696 L
.47586 .48188 L
.47305 .47728 L
.46988 .47343 L
.46624 .47049 L
.46209 .46856 L
.45745 .46763 L
.45237 .46758 L
.44694 .46822 L
.4413 .46929 L
.43561 .47045 L
.43003 .47141 L
.4247 .47185 L
.41975 .47153 L
.41527 .47027 L
.4113 .468 L
.40782 .46474 L
.40478 .46061 L
.40207 .45583 L
.39955 .45065 L
.39706 .44541 L
.39445 .44041 L
.39157 .43596 L
.3883 .43229 L
.38455 .42957 L
.38029 .42787 L
.37555 .42715 L
.37037 .42728 L
.36489 .42804 L
.35922 .42915 L
.35354 .4303 L
.348 .43117 L
.34275 .43146 L
.3379 .43094 L
.33353 .42946 L
.32967 .42697 L
.3263 .4235 L
.32334 .41921 L
.32068 .41431 L
.31818 .40909 L
Mfstroke
gsave
.50402 .46447 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N20) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCacheValid->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.325 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0113636 0 0.0113636 [
[ 0 0 0 0 ]
[ 1 1.325 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.325 L
0 1.325 L
closepath
clip
newpath
% Start of sub-graphic
p
0 1 0.25 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67932 .41431 L
.67666 .41921 L
.6737 .4235 L
.67033 .42697 L
.66647 .42946 L
.6621 .43094 L
.65725 .43146 L
.652 .43117 L
.64646 .4303 L
.64078 .42915 L
.63511 .42804 L
.62963 .42728 L
.62445 .42715 L
.61971 .42787 L
.61545 .42957 L
.6117 .43229 L
.60843 .43596 L
.60555 .44041 L
.60294 .44541 L
.60045 .45065 L
.59793 .45583 L
.59522 .46061 L
.59218 .46474 L
.5887 .468 L
.58473 .47027 L
.58025 .47153 L
.5753 .47185 L
.56997 .47141 L
.56439 .47045 L
.5587 .46929 L
.55306 .46822 L
.54763 .46758 L
.54255 .46763 L
.53791 .46856 L
.53376 .47049 L
.53012 .47343 L
.52695 .47728 L
.52414 .48188 L
.52157 .48696 L
.51909 .49222 L
.51654 .49733 L
.51377 .502 L
.51063 .50595 L
.50705 .50899 L
.50296 .51104 L
.49838 .51208 L
.49334 .51221 L
.48794 .51163 L
.48232 .5106 L
Mistroke
.47663 .50943 L
.47102 .50844 L
.46566 .50793 L
.46067 .50815 L
.45613 .5093 L
.4521 .51146 L
.44857 .51461 L
.44549 .51865 L
.44274 .52338 L
.44021 .52853 L
.43773 .53378 L
.43514 .53882 L
.4323 .54335 L
.42907 .54711 L
.42538 .54994 L
.42118 .55176 L
.41649 .55259 L
.41136 .55254 L
.4059 .55184 L
.40025 .55075 L
.39456 .54959 L
.389 .54868 L
.38371 .54832 L
.37881 .54874 L
.37439 .55011 L
.37047 .55249 L
.36705 .55585 L
.36405 .56007 L
.36136 .56491 L
.35886 .57011 L
.35637 .57535 L
.35374 .5803 L
.35082 .58467 L
.34749 .58824 L
.34369 .59085 L
.33938 .59244 L
.33458 .59306 L
.32937 .59285 L
.32386 .59204 L
.31818 .59091 L
Mfstroke
gsave
.49598 .46447 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.68182 .59091 m
.67614 .59204 L
.67063 .59285 L
.66542 .59306 L
.66062 .59244 L
.65631 .59085 L
.65251 .58824 L
.64918 .58467 L
.64626 .5803 L
.64363 .57535 L
.64114 .57011 L
.63864 .56491 L
.63595 .56007 L
.63295 .55585 L
.62953 .55249 L
.62561 .55011 L
.62119 .54874 L
.61629 .54832 L
.611 .54868 L
.60544 .54959 L
.59975 .55075 L
.5941 .55184 L
.58864 .55254 L
.58351 .55259 L
.57882 .55176 L
.57462 .54994 L
.57093 .54711 L
.5677 .54335 L
.56486 .53882 L
.56227 .53378 L
.55979 .52853 L
.55726 .52338 L
.55451 .51865 L
.55143 .51461 L
.5479 .51146 L
.54387 .5093 L
.53933 .50815 L
.53434 .50793 L
.52898 .50844 L
.52337 .50943 L
.51768 .5106 L
.51206 .51163 L
.50666 .51221 L
.50162 .51208 L
.49704 .51104 L
.49295 .50899 L
.48937 .50595 L
.48623 .502 L
.48346 .49733 L
.48091 .49222 L
Mistroke
.47843 .48696 L
.47586 .48188 L
.47305 .47728 L
.46988 .47343 L
.46624 .47049 L
.46209 .46856 L
.45745 .46763 L
.45237 .46758 L
.44694 .46822 L
.4413 .46929 L
.43561 .47045 L
.43003 .47141 L
.4247 .47185 L
.41975 .47153 L
.41527 .47027 L
.4113 .468 L
.40782 .46474 L
.40478 .46061 L
.40207 .45583 L
.39955 .45065 L
.39706 .44541 L
.39445 .44041 L
.39157 .43596 L
.3883 .43229 L
.38455 .42957 L
.38029 .42787 L
.37555 .42715 L
.37037 .42728 L
.36489 .42804 L
.35922 .42915 L
.35354 .4303 L
.348 .43117 L
.34275 .43146 L
.3379 .43094 L
.33353 .42946 L
.32967 .42697 L
.3263 .4235 L
.32334 .41921 L
.32068 .41431 L
.31818 .40909 L
Mfstroke
gsave
.50402 .46447 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C2) show
77.812500 14.625000 moveto
(N21) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.25 1 0.5 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67932 .41431 L
.67666 .41921 L
.6737 .4235 L
.67033 .42697 L
.66647 .42946 L
.6621 .43094 L
.65725 .43146 L
.652 .43117 L
.64646 .4303 L
.64078 .42915 L
.63511 .42804 L
.62963 .42728 L
.62445 .42715 L
.61971 .42787 L
.61545 .42957 L
.6117 .43229 L
.60843 .43596 L
.60555 .44041 L
.60294 .44541 L
.60045 .45065 L
.59793 .45583 L
.59522 .46061 L
.59218 .46474 L
.5887 .468 L
.58473 .47027 L
.58025 .47153 L
.5753 .47185 L
.56997 .47141 L
.56439 .47045 L
.5587 .46929 L
.55306 .46822 L
.54763 .46758 L
.54255 .46763 L
.53791 .46856 L
.53376 .47049 L
.53012 .47343 L
.52695 .47728 L
.52414 .48188 L
.52157 .48696 L
.51909 .49222 L
.51654 .49733 L
.51377 .502 L
.51063 .50595 L
.50705 .50899 L
.50296 .51104 L
.49838 .51208 L
.49334 .51221 L
.48794 .51163 L
.48232 .5106 L
Mistroke
.47663 .50943 L
.47102 .50844 L
.46566 .50793 L
.46067 .50815 L
.45613 .5093 L
.4521 .51146 L
.44857 .51461 L
.44549 .51865 L
.44274 .52338 L
.44021 .52853 L
.43773 .53378 L
.43514 .53882 L
.4323 .54335 L
.42907 .54711 L
.42538 .54994 L
.42118 .55176 L
.41649 .55259 L
.41136 .55254 L
.4059 .55184 L
.40025 .55075 L
.39456 .54959 L
.389 .54868 L
.38371 .54832 L
.37881 .54874 L
.37439 .55011 L
.37047 .55249 L
.36705 .55585 L
.36405 .56007 L
.36136 .56491 L
.35886 .57011 L
.35637 .57535 L
.35374 .5803 L
.35082 .58467 L
.34749 .58824 L
.34369 .59085 L
.33938 .59244 L
.33458 .59306 L
.32937 .59285 L
.32386 .59204 L
.31818 .59091 L
Mfstroke
gsave
.49598 .46447 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .59091 m
.67614 .59204 L
.67063 .59285 L
.66542 .59306 L
.66062 .59244 L
.65631 .59085 L
.65251 .58824 L
.64918 .58467 L
.64626 .5803 L
.64363 .57535 L
.64114 .57011 L
.63864 .56491 L
.63595 .56007 L
.63295 .55585 L
.62953 .55249 L
.62561 .55011 L
.62119 .54874 L
.61629 .54832 L
.611 .54868 L
.60544 .54959 L
.59975 .55075 L
.5941 .55184 L
.58864 .55254 L
.58351 .55259 L
.57882 .55176 L
.57462 .54994 L
.57093 .54711 L
.5677 .54335 L
.56486 .53882 L
.56227 .53378 L
.55979 .52853 L
.55726 .52338 L
.55451 .51865 L
.55143 .51461 L
.5479 .51146 L
.54387 .5093 L
.53933 .50815 L
.53434 .50793 L
.52898 .50844 L
.52337 .50943 L
.51768 .5106 L
.51206 .51163 L
.50666 .51221 L
.50162 .51208 L
.49704 .51104 L
.49295 .50899 L
.48937 .50595 L
.48623 .502 L
.48346 .49733 L
.48091 .49222 L
Mistroke
.47843 .48696 L
.47586 .48188 L
.47305 .47728 L
.46988 .47343 L
.46624 .47049 L
.46209 .46856 L
.45745 .46763 L
.45237 .46758 L
.44694 .46822 L
.4413 .46929 L
.43561 .47045 L
.43003 .47141 L
.4247 .47185 L
.41975 .47153 L
.41527 .47027 L
.4113 .468 L
.40782 .46474 L
.40478 .46061 L
.40207 .45583 L
.39955 .45065 L
.39706 .44541 L
.39445 .44041 L
.39157 .43596 L
.3883 .43229 L
.38455 .42957 L
.38029 .42787 L
.37555 .42715 L
.37037 .42728 L
.36489 .42804 L
.35922 .42915 L
.35354 .4303 L
.348 .43117 L
.34275 .43146 L
.3379 .43094 L
.33353 .42946 L
.32967 .42697 L
.3263 .4235 L
.32334 .41921 L
.32068 .41431 L
.31818 .40909 L
Mfstroke
gsave
.50402 .46447 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(g) show
66.250000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C3) show
77.812500 14.625000 moveto
(N22) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.5 1 0.75 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.31818 .40909 L
s
.31818 .52727 m
.3 .47273 L
.33636 .47273 L
F
gsave
.27522 .5 -69 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .40909 m
.67932 .41431 L
.67666 .41921 L
.6737 .4235 L
.67033 .42697 L
.66647 .42946 L
.6621 .43094 L
.65725 .43146 L
.652 .43117 L
.64646 .4303 L
.64078 .42915 L
.63511 .42804 L
.62963 .42728 L
.62445 .42715 L
.61971 .42787 L
.61545 .42957 L
.6117 .43229 L
.60843 .43596 L
.60555 .44041 L
.60294 .44541 L
.60045 .45065 L
.59793 .45583 L
.59522 .46061 L
.59218 .46474 L
.5887 .468 L
.58473 .47027 L
.58025 .47153 L
.5753 .47185 L
.56997 .47141 L
.56439 .47045 L
.5587 .46929 L
.55306 .46822 L
.54763 .46758 L
.54255 .46763 L
.53791 .46856 L
.53376 .47049 L
.53012 .47343 L
.52695 .47728 L
.52414 .48188 L
.52157 .48696 L
.51909 .49222 L
.51654 .49733 L
.51377 .502 L
.51063 .50595 L
.50705 .50899 L
.50296 .51104 L
.49838 .51208 L
.49334 .51221 L
.48794 .51163 L
.48232 .5106 L
Mistroke
.47663 .50943 L
.47102 .50844 L
.46566 .50793 L
.46067 .50815 L
.45613 .5093 L
.4521 .51146 L
.44857 .51461 L
.44549 .51865 L
.44274 .52338 L
.44021 .52853 L
.43773 .53378 L
.43514 .53882 L
.4323 .54335 L
.42907 .54711 L
.42538 .54994 L
.42118 .55176 L
.41649 .55259 L
.41136 .55254 L
.4059 .55184 L
.40025 .55075 L
.39456 .54959 L
.389 .54868 L
.38371 .54832 L
.37881 .54874 L
.37439 .55011 L
.37047 .55249 L
.36705 .55585 L
.36405 .56007 L
.36136 .56491 L
.35886 .57011 L
.35637 .57535 L
.35374 .5803 L
.35082 .58467 L
.34749 .58824 L
.34369 .59085 L
.33938 .59244 L
.33458 .59306 L
.32937 .59285 L
.32386 .59204 L
.31818 .59091 L
Mfstroke
gsave
.49598 .46447 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.68182 .59091 m
.67614 .59204 L
.67063 .59285 L
.66542 .59306 L
.66062 .59244 L
.65631 .59085 L
.65251 .58824 L
.64918 .58467 L
.64626 .5803 L
.64363 .57535 L
.64114 .57011 L
.63864 .56491 L
.63595 .56007 L
.63295 .55585 L
.62953 .55249 L
.62561 .55011 L
.62119 .54874 L
.61629 .54832 L
.611 .54868 L
.60544 .54959 L
.59975 .55075 L
.5941 .55184 L
.58864 .55254 L
.58351 .55259 L
.57882 .55176 L
.57462 .54994 L
.57093 .54711 L
.5677 .54335 L
.56486 .53882 L
.56227 .53378 L
.55979 .52853 L
.55726 .52338 L
.55451 .51865 L
.55143 .51461 L
.5479 .51146 L
.54387 .5093 L
.53933 .50815 L
.53434 .50793 L
.52898 .50844 L
.52337 .50943 L
.51768 .5106 L
.51206 .51163 L
.50666 .51221 L
.50162 .51208 L
.49704 .51104 L
.49295 .50899 L
.48937 .50595 L
.48623 .502 L
.48346 .49733 L
.48091 .49222 L
Mistroke
.47843 .48696 L
.47586 .48188 L
.47305 .47728 L
.46988 .47343 L
.46624 .47049 L
.46209 .46856 L
.45745 .46763 L
.45237 .46758 L
.44694 .46822 L
.4413 .46929 L
.43561 .47045 L
.43003 .47141 L
.4247 .47185 L
.41975 .47153 L
.41527 .47027 L
.4113 .468 L
.40782 .46474 L
.40478 .46061 L
.40207 .45583 L
.39955 .45065 L
.39706 .44541 L
.39445 .44041 L
.39157 .43596 L
.3883 .43229 L
.38455 .42957 L
.38029 .42787 L
.37555 .42715 L
.37037 .42728 L
.36489 .42804 L
.35922 .42915 L
.35354 .4303 L
.348 .43117 L
.34275 .43146 L
.3379 .43094 L
.33353 .42946 L
.32967 .42697 L
.3263 .4235 L
.32334 .41921 L
.32068 .41431 L
.31818 .40909 L
Mfstroke
gsave
.50402 .46447 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Z) show
66.687500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.68182 .59091 m
.68182 .40909 L
s
.68182 .47273 m
.66364 .52727 L
.7 .52727 L
F
gsave
.63885 .5 -70 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T3) show
70.187500 14.625000 moveto
(C4) show
77.812500 14.625000 moveto
(N23) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.75 1 1 1.25 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.68182 .59091 L
s
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .59091 m
.68182 .40909 L
s
.47561 .5122 m
.51626 .47154 L
.53252 .50407 L
F
gsave
.49598 .46447 -71 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.31818 .40909 m
.68182 .59091 L
s
gsave
.49894 .47464 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .40909 m
.68182 .40909 L
s
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N24) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0.75 0.25 1 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0454545 0.0454545 0.0454545 0.0454545 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.005 w
.04545 .72727 m
.31818 .59091 L
s
.15742 .67129 m
.19808 .63063 L
.21434 .66316 L
F
gsave
.1778 .62357 -69 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.04545 .27273 m
.31818 .40909 L
s
.20621 .35311 m
.16556 .31245 L
.14929 .34497 L
F
gsave
.18584 .30538 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
[ .03 .03 ] 0 setdash
.95455 .72727 m
.68182 .59091 L
s
.79379 .64689 m
.83444 .68755 L
.85071 .65503 L
F
gsave
.81416 .69462 -70 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.95455 .27273 m
.68182 .40909 L
s
.84258 .32871 m
.80192 .36937 L
.78566 .33684 L
F
gsave
.8222 .37643 -61 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
67.312500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
.31818 .59091 m
.68182 .59091 L
s
.47273 .59091 m
.52727 .57273 L
.52727 .60909 L
F
gsave
.5 .54795 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
63.375000 10.187500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
66.312500 9.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(s) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.31818 .59091 m
.68182 .40909 L
s
gsave
.50106 .47464 -72 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
67.000000 10.000000 moveto
(0) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ .03 .03 ] 0 setdash
.31818 .40909 m
.68182 .59091 L
s
.52439 .5122 m
.48374 .47154 L
.46748 .50407 L
F
gsave
.50402 .46447 -61 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(n) show
63.125000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
65.750000 15.500000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
[ ] 0 setdash
.31818 .40909 m
.68182 .40909 L
s
.52727 .40909 m
.47273 .39091 L
.47273 .42727 L
F
gsave
.5 .36613 -66 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.687500 14.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 6.187500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(c) show
63.750000 10.687500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(~) show
67.000000 16.312500 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.375000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(i) show
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
.04 w
.31818 .59091 Mdot
.31818 .40909 Mdot
.68182 .59091 Mdot
.68182 .40909 Mdot
gsave
.5 .02273 -75 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(T4) show
70.187500 14.625000 moveto
(C1) show
77.812500 14.625000 moveto
(N25) show
86.875000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 4.937500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.39773 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.44886 1.295 -65 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(e) show
66.812500 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.5 1.295 -66 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
63.000000 14.625000 moveto
(\\256) show
68.750000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Math1
%%DocumentNeededResources: + font Helvetica
%%DocumentNeededFonts: Math1
%%DocumentNeededFont: + Helvetica
%%DocumentFonts: font Math1
%%DocumentFonts: + font Helvetica
grestore
gsave
.55114 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
gsave
.60227 1.295 -65.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(H) show
68.000000 14.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 7.437500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Helvetica
%%DocumentNeededFonts: Helvetica
%%DocumentFonts: font Helvetica
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCacheValid->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(box\  = \ 
      CalcFeynAmp[\n\ \ CreateFeynAmp[ins], \n\ \ 
        Select[counter, \ DiagramType[#]\  == \ 0\ &]]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 7 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 7 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"3 Generic, 10 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "3 Generic, 10 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]3
          \[InvisibleSpace]": "\[InvisibleSpace]"3 Generic, 6 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 3, ": ", "3 Generic, 6 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]4
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 2 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 4, ": ", "2 Generic, 2 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"10 Generic, 25 Classes amplitudes"\),
        SequenceForm[ "in total: ", "10 Generic, 25 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]5
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 5, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]5
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 5, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell["running FORM... ", "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    FormBox[
      RowBox[{\((Amp({ME, ME} \[Rule] {MHp, MHp}))\), "(", 
        RowBox[{
        \(\((\(Alfa2\ \(C0i(cc0, S, MHp2, MHp2, MW2, MW2, Mh02)\)\ 
                      CBA\^2\)\/\(2\ SW2\^2\) + 
                  \(Alfa2\ \(C0i(cc2, S, MHp2, MHp2, MW2, MW2, Mh02)\)\ 
                      CBA\^2\)\/\(8\ SW2\^2\) + 
                  \(Alfa2\ \(C0i(cc0, S, MHp2, MHp2, MW2, MW2, MA02)\)\)\/\(2
                      \ SW2\^2\) + 
                  \(Alfa2\ SBA\^2\ 
                      \(C0i(cc0, S, MHp2, MHp2, MW2, MW2, MHH2)\)\)\/\(2\ 
                      SW2\^2\) + 
                  \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + Alfa2\/CW2 - 
                        Alfa2\/SW2 + 2\ Alfa2)\)\ 
                    \((C0i(cc0, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc0, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(2\ SW2\ Alfa2\)\/CW2 - Alfa2\/CW2 + Alfa2\/SW2 - 
                        2\ Alfa2)\)\ 
                    \((C0i(cc0, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc0, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                        \(2\ Alfa2\ SW2\)\/CW2 + \(Alfa2\ SW2\)\/CW2\^2 - 
                        Alfa2 - \(2\ Alfa2\)\/CW2 - Alfa2\/\(4\ CW2\^2\) + 
                        Alfa2\/SW2 + Alfa2\/\(2\ CW2\ SW2\) - 
                        Alfa2\/\(4\ SW2\^2\))\)\ 
                    \(C0i(cc0, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((\(Alfa2\ SW2\^2\)\/CW2\^2 - \(2\ Alfa2\ SW2\)\/CW2 - 
                        \(Alfa2\ SW2\)\/CW2\^2 + Alfa2 + \(2\ Alfa2\)\/CW2 + 
                        Alfa2\/\(4\ CW2\^2\) - Alfa2\/SW2 - 
                        Alfa2\/\(2\ CW2\ SW2\) + Alfa2\/\(4\ SW2\^2\))\)\ 
                    \(C0i(cc0, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \(Alfa2\ \(C0i(cc2, S, MHp2, MHp2, MW2, MW2, MA02)\)\)\/\(8
                      \ SW2\^2\) + 
                  \(Alfa2\ SBA\^2\ 
                      \(C0i(cc2, S, MHp2, MHp2, MW2, MW2, MHH2)\)\)\/\(8\ 
                      SW2\^2\) + 
                  \((\(-\(\(SW2\ Alfa2\)\/\(2\ CW2\)\)\) + 
                        Alfa2\/\(4\ CW2\) - Alfa2\/\(4\ SW2\) + Alfa2\/2)\)\ 
                    \((C0i(cc2, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc2, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(SW2\ Alfa2\)\/\(2\ CW2\) - Alfa2\/\(4\ CW2\) + 
                        Alfa2\/\(4\ SW2\) - Alfa2\/2)\)\ 
                    \((C0i(cc2, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc2, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ SW2\^2\)\/\(4\ CW2\^2\)\)\) + 
                        \(Alfa2\ SW2\)\/\(2\ CW2\) + 
                        \(Alfa2\ SW2\)\/\(4\ CW2\^2\) - Alfa2\/4 - 
                        Alfa2\/\(2\ CW2\) - Alfa2\/\(16\ CW2\^2\) + 
                        Alfa2\/\(4\ SW2\) + Alfa2\/\(8\ CW2\ SW2\) - 
                        Alfa2\/\(16\ SW2\^2\))\)\ 
                    \(C0i(cc2, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((\(Alfa2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                        \(Alfa2\ SW2\)\/\(2\ CW2\) - 
                        \(Alfa2\ SW2\)\/\(4\ CW2\^2\) + Alfa2\/4 + 
                        Alfa2\/\(2\ CW2\) + Alfa2\/\(16\ CW2\^2\) - 
                        Alfa2\/\(4\ SW2\) - Alfa2\/\(8\ CW2\ SW2\) + 
                        Alfa2\/\(16\ SW2\^2\))\)\ 
                    \(C0i(cc2, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((\(Alfa2\ T\)\/\(2\ SW2\^2\) - 
                        \(Alfa2\ MHp2\)\/\(2\ SW2\^2\))\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, MA02)
                      \) + \((
                      \(Alfa2\ CBA\^2\ T\)\/\(2\ SW2\^2\) - 
                        \(Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\^2\))\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, Mh02)
                      \) + \((
                      \(Alfa2\ SBA\^2\ T\)\/\(2\ SW2\^2\) - 
                        \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\^2\))\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, MHH2)
                      \) + \((4\ Alfa2\ T - 4\ Alfa2\ MHp2)\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      2\ Alfa2\ MHp2 - \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \(Alfa2\ MHp2\)\/CW2 - \(Alfa2\ MHp2\)\/SW2 + 
                        \((\(2\ SW2\ Alfa2\)\/CW2 - Alfa2\/CW2 + Alfa2\/SW2 - 
                              2\ Alfa2)\)\ T)\)\ 
                    \((D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2\)\) + 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 + 
                        \(Alfa2\ MHp2\ SW2\)\/CW2\^2 - Alfa2\ MHp2 - 
                        \(2\ Alfa2\ MHp2\)\/CW2 - 
                        \(Alfa2\ MHp2\)\/\(4\ CW2\^2\) + 
                        \((\(Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(2\ Alfa2\ SW2\)\/CW2 - 
                              \(Alfa2\ SW2\)\/CW2\^2 + Alfa2 + 
                              \(2\ Alfa2\)\/CW2 + Alfa2\/\(4\ CW2\^2\) - 
                              Alfa2\/SW2 - Alfa2\/\(2\ CW2\ SW2\) + 
                              Alfa2\/\(4\ SW2\^2\))\)\ T + 
                        \(Alfa2\ MHp2\)\/SW2 + 
                        \(Alfa2\ MHp2\)\/\(2\ CW2\ SW2\) - 
                        \(Alfa2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((4\ Alfa2\ \((S + T)\) - 4\ Alfa2\ MHp2)\)\ 
                    \(D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      2\ Alfa2\ MHp2 - \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \(Alfa2\ MHp2\)\/CW2 - \(Alfa2\ MHp2\)\/SW2 + 
                        \((\(2\ SW2\ Alfa2\)\/CW2 - Alfa2\/CW2 + Alfa2\/SW2 - 
                              2\ Alfa2)\)\ \((S + T)\))\)\ 
                    \((D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2\)\) + 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 + 
                        \(Alfa2\ MHp2\ SW2\)\/CW2\^2 - Alfa2\ MHp2 - 
                        \(2\ Alfa2\ MHp2\)\/CW2 - 
                        \(Alfa2\ MHp2\)\/\(4\ CW2\^2\) + 
                        \((\(Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(2\ Alfa2\ SW2\)\/CW2 - 
                              \(Alfa2\ SW2\)\/CW2\^2 + Alfa2 + 
                              \(2\ Alfa2\)\/CW2 + Alfa2\/\(4\ CW2\^2\) - 
                              Alfa2\/SW2 - Alfa2\/\(2\ CW2\ SW2\) + 
                              Alfa2\/\(4\ SW2\^2\))\)\ \((S + T)\) + 
                        \(Alfa2\ MHp2\)\/SW2 + 
                        \(Alfa2\ MHp2\)\/\(2\ CW2\ SW2\) - 
                        \(Alfa2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                    \(D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((\(Alfa2\ MHp2\)\/\(2\ SW2\^2\) - 
                        \(Alfa2\ \((S + U)\)\)\/\(2\ SW2\^2\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, MA02)
                      \) + \((
                      \(Alfa2\ CBA\^2\ MHp2\)\/\(2\ SW2\^2\) - 
                        \(Alfa2\ CBA\^2\ \((S + U)\)\)\/\(2\ SW2\^2\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, Mh02)
                      \) + \((
                      \(Alfa2\ MHp2\ SBA\^2\)\/\(2\ SW2\^2\) - 
                        \(Alfa2\ SBA\^2\ \((S + U)\)\)\/\(2\ SW2\^2\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, MHH2)
                      \) + \((4\ Alfa2\ MHp2 - 4\ Alfa2\ \((S + U)\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-2\)\ Alfa2\ MHp2 + \(2\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                        \(Alfa2\ MHp2\)\/CW2 + \(Alfa2\ MHp2\)\/SW2 + 
                        \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + Alfa2\/CW2 - 
                              Alfa2\/SW2 + 2\ Alfa2)\)\ \((S + U)\))\)\ 
                    \((D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                        \(Alfa2\ MHp2\ SW2\)\/CW2\^2 + Alfa2\ MHp2 + 
                        \(2\ Alfa2\ MHp2\)\/CW2 + 
                        \(Alfa2\ MHp2\)\/\(4\ CW2\^2\) + 
                        \((\(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(2\ Alfa2\ SW2\)\/CW2 + 
                              \(Alfa2\ SW2\)\/CW2\^2 - Alfa2 - 
                              \(2\ Alfa2\)\/CW2 - Alfa2\/\(4\ CW2\^2\) + 
                              Alfa2\/SW2 + Alfa2\/\(2\ CW2\ SW2\) - 
                              Alfa2\/\(4\ SW2\^2\))\)\ \((S + U)\) - 
                        \(Alfa2\ MHp2\)\/SW2 - 
                        \(Alfa2\ MHp2\)\/\(2\ CW2\ SW2\) + 
                        \(Alfa2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((4\ Alfa2\ MHp2 - 4\ Alfa2\ U)\)\ 
                    \(D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-2\)\ Alfa2\ MHp2 + \(2\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                        \(Alfa2\ MHp2\)\/CW2 + \(Alfa2\ MHp2\)\/SW2 + 
                        \((\(-\(\(2\ SW2\ Alfa2\)\/CW2\)\) + Alfa2\/CW2 - 
                              Alfa2\/SW2 + 2\ Alfa2)\)\ U)\)\ 
                    \((D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                        \(Alfa2\ MHp2\ SW2\)\/CW2\^2 + Alfa2\ MHp2 + 
                        \(2\ Alfa2\ MHp2\)\/CW2 + 
                        \(Alfa2\ MHp2\)\/\(4\ CW2\^2\) + 
                        \((\(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(2\ Alfa2\ SW2\)\/CW2 + 
                              \(Alfa2\ SW2\)\/CW2\^2 - Alfa2 - 
                              \(2\ Alfa2\)\/CW2 - Alfa2\/\(4\ CW2\^2\) + 
                              Alfa2\/SW2 + Alfa2\/\(2\ CW2\ SW2\) - 
                              Alfa2\/\(4\ SW2\^2\))\)\ U - 
                        \(Alfa2\ MHp2\)\/SW2 - 
                        \(Alfa2\ MHp2\)\/\(2\ CW2\ SW2\) + 
                        \(Alfa2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((\(2\ Alfa2\ MHp2\)\/SW2\^2 - 
                        \(Alfa2\ \((S + T\/2 + U)\)\)\/SW2\^2)\)\ 
                    \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, MA02)
                      \) + \((
                      \(2\ Alfa2\ CBA\^2\ MHp2\)\/SW2\^2 - 
                        \(Alfa2\ CBA\^2\ \((S + T\/2 + U)\)\)\/SW2\^2)\)\ 
                    \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, Mh02)
                      \) + \((
                      \(2\ Alfa2\ MHp2\ SBA\^2\)\/SW2\^2 - 
                        \(Alfa2\ SBA\^2\ \((S + T\/2 + U)\)\)\/SW2\^2)\)\ 
                    \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 0, MW2, MW2, MHH2)
                      \) + \((16\ Alfa2\ MHp2 - 8\ Alfa2\ \((S + T\/2 + U)\))
                      \)\ \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 0, 
                      MHp2)\) + 
                  \((\(-8\)\ Alfa2\ MHp2 + \(8\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                        \(4\ Alfa2\ MHp2\)\/CW2 + \(4\ Alfa2\ MHp2\)\/SW2 + 
                        \((\(-\(\(4\ SW2\ Alfa2\)\/CW2\)\) + 
                              \(2\ Alfa2\)\/CW2 - \(2\ Alfa2\)\/SW2 + 
                              4\ Alfa2)\)\ \((S + T\/2 + U)\))\)\ 
                    \((D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(4\ Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(8\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                        \(4\ Alfa2\ MHp2\ SW2\)\/CW2\^2 + 4\ Alfa2\ MHp2 + 
                        \(8\ Alfa2\ MHp2\)\/CW2 + \(Alfa2\ MHp2\)\/CW2\^2 + 
                        \((\(-\(\(2\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(4\ Alfa2\ SW2\)\/CW2 + 
                              \(2\ Alfa2\ SW2\)\/CW2\^2 - 2\ Alfa2 - 
                              \(4\ Alfa2\)\/CW2 - Alfa2\/\(2\ CW2\^2\) + 
                              \(2\ Alfa2\)\/SW2 + Alfa2\/\(CW2\ SW2\) - 
                              Alfa2\/\(2\ SW2\^2\))\)\ \((S + T\/2 + U)\) - 
                        \(4\ Alfa2\ MHp2\)\/SW2 - 
                        \(2\ Alfa2\ MHp2\)\/\(CW2\ SW2\) + 
                        \(Alfa2\ MHp2\)\/SW2\^2)\)\ 
                    \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((4\ Alfa2\ MHp2 - 4\ Alfa2\ \((MHp2 + U)\))\)\ 
                    \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-2\)\ Alfa2\ MHp2 + \(2\ Alfa2\ SW2\ MHp2\)\/CW2 - 
                        \(Alfa2\ MHp2\)\/CW2 + \(Alfa2\ MHp2\)\/SW2 + 
                        2\ \((\(-\(\(SW2\ Alfa2\)\/CW2\)\) + 
                              Alfa2\/\(2\ CW2\) - Alfa2\/\(2\ SW2\) + Alfa2)
                            \)\ \((MHp2 + U)\))\)\ 
                    \((D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 - 
                        \(Alfa2\ MHp2\ SW2\)\/CW2\^2 + Alfa2\ MHp2 + 
                        \(2\ Alfa2\ MHp2\)\/CW2 + 
                        \(Alfa2\ MHp2\)\/\(4\ CW2\^2\) + 
                        2\ \((\(-\(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(Alfa2\ SW2\)\/CW2 + 
                              \(Alfa2\ SW2\)\/\(2\ CW2\^2\) - Alfa2\/2 - 
                              Alfa2\/CW2 - Alfa2\/\(8\ CW2\^2\) + 
                              Alfa2\/\(2\ SW2\) + Alfa2\/\(4\ CW2\ SW2\) - 
                              Alfa2\/\(8\ SW2\^2\))\)\ \((MHp2 + U)\) - 
                        \(Alfa2\ MHp2\)\/SW2 - 
                        \(Alfa2\ MHp2\)\/\(2\ CW2\ SW2\) + 
                        \(Alfa2\ MHp2\)\/\(4\ SW2\^2\))\)\ 
                    \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\))\)\ \(Mat(F1)\) + 
            \((\((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\)\ 
                    \((C0i(cc0, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc0, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(2\ Alfa2\ SW2\)\/CW2 - 2\ Alfa2)\)\ 
                    \((C0i(cc0, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc0, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                        \(2\ Alfa2\ SW2\)\/CW2 - Alfa2)\)\ 
                    \(C0i(cc0, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((\(Alfa2\ SW2\^2\)\/CW2\^2 - \(2\ Alfa2\ SW2\)\/CW2 + 
                        Alfa2)\)\ 
                    \(C0i(cc0, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((Alfa2\/2 - \(Alfa2\ SW2\)\/\(2\ CW2\))\)\ 
                    \((C0i(cc2, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc2, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(Alfa2\ SW2\)\/\(2\ CW2\) - Alfa2\/2)\)\ 
                    \((C0i(cc2, S, MHp2, MHp2, 0, MZ2, MHp2) + 
                        C0i(cc2, S, MHp2, MHp2, MZ2, 0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ SW2\^2\)\/\(4\ CW2\^2\)\)\) + 
                        \(Alfa2\ SW2\)\/\(2\ CW2\) - Alfa2\/4)\)\ 
                    \(C0i(cc2, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((\(Alfa2\ SW2\^2\)\/\(4\ CW2\^2\) - 
                        \(Alfa2\ SW2\)\/\(2\ CW2\) + Alfa2\/4)\)\ 
                    \(C0i(cc2, S, MHp2, MHp2, MZ2, MZ2, MHp2)\) + 
                  \((4\ Alfa2\ T - 4\ Alfa2\ MHp2)\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      2\ Alfa2\ MHp2 - \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \((\(2\ Alfa2\ SW2\)\/CW2 - 2\ Alfa2)\)\ T)\)\ 
                    \((D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(-\(\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2\)\) + 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 - Alfa2\ MHp2 + 
                        \((\(Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(2\ Alfa2\ SW2\)\/CW2 + Alfa2)\)\ T)\)\ 
                    \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((4\ Alfa2\ \((S + T)\) - 4\ Alfa2\ MHp2)\)\ 
                    \(D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      2\ Alfa2\ MHp2 - \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \((\(2\ Alfa2\ SW2\)\/CW2 - 2\ Alfa2)\)\ \((S + T)\))
                      \)\ \((D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 
                          MZ2, MHp2) + 
                        D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 0, 
                          MHp2))\) + 
                  \((\(-\(\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2\)\) + 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 - Alfa2\ MHp2 + 
                        \((\(Alfa2\ SW2\^2\)\/CW2\^2 - 
                              \(2\ Alfa2\ SW2\)\/CW2 + Alfa2)\)\ \((S + T)\))
                      \)\ \(D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 
                      MZ2, MHp2)\) + 
                  \((4\ Alfa2\ MHp2 - 4\ Alfa2\ \((S + U)\))\)\ 
                    \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-2\)\ Alfa2\ MHp2 + \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\)\ \((S + U)\))
                      \)\ \((D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 
                          MZ2, MHp2) + 
                        D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 0, 
                          MHp2))\) + 
                  \((\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 + Alfa2\ MHp2 + 
                        \((\(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(2\ Alfa2\ SW2\)\/CW2 - Alfa2)\)\ \((S + U)\))
                      \)\ \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 
                      MZ2, MHp2)\) + 
                  \((4\ Alfa2\ MHp2 - 4\ Alfa2\ U)\)\ 
                    \(D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-2\)\ Alfa2\ MHp2 + \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \((2\ Alfa2 - \(2\ Alfa2\ SW2\)\/CW2)\)\ U)\)\ 
                    \((D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 + Alfa2\ MHp2 + 
                        \((\(-\(\(Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(2\ Alfa2\ SW2\)\/CW2 - Alfa2)\)\ U)\)\ 
                    \(D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((16\ Alfa2\ MHp2 - 8\ Alfa2\ \((S + T\/2 + U)\))\)\ 
                    \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-8\)\ Alfa2\ MHp2 + \(8\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        \((4\ Alfa2 - \(4\ Alfa2\ SW2\)\/CW2)\)\ 
                          \((S + T\/2 + U)\))\)\ 
                    \((D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, 0, MZ2, MHp2)
                           + D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, 
                          0, MHp2))\) + 
                  \((\(4\ Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(8\ Alfa2\ MHp2\ SW2\)\/CW2 + 4\ Alfa2\ MHp2 + 
                        \((\(-\(\(2\ Alfa2\ SW2\^2\)\/CW2\^2\)\) + 
                              \(4\ Alfa2\ SW2\)\/CW2 - 2\ Alfa2)\)\ 
                          \((S + T\/2 + U)\))\)\ 
                    \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\) + 
                  \((4\ Alfa2\ MHp2 - 4\ Alfa2\ \((MHp2 + U)\))\)\ 
                    \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 0, MHp2)
                      \) + \((
                      \(-2\)\ Alfa2\ MHp2 + \(2\ Alfa2\ SW2\ MHp2\)\/CW2 + 
                        2\ \((Alfa2 - \(Alfa2\ SW2\)\/CW2)\)\ \((MHp2 + U)\))
                      \)\ \((D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, 0, 
                          MZ2, MHp2) + 
                        D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, 0, 
                          MHp2))\) + 
                  \((\(Alfa2\ MHp2\ SW2\^2\)\/CW2\^2 - 
                        \(2\ Alfa2\ MHp2\ SW2\)\/CW2 + Alfa2\ MHp2 + 
                        2\ \((\(-\(\(Alfa2\ SW2\^2\)\/\(2\ CW2\^2\)\)\) + 
                              \(Alfa2\ SW2\)\/CW2 - Alfa2\/2)\)\ 
                          \((MHp2 + U)\))\)\ 
                    \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, ME2, MZ2, MZ2, 
                      MHp2)\))\)\ \(Mat(F2)\)\), ",", 
          RowBox[{
          \(SumOver(Neu1, 4)\), " ", \(SumOver(Sfe1, 2)\), " ", 
            \(SumOver(Sfe2, 2)\), " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(Mat(F3)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(1\/\(4\ CB2\ CW2\ MW2\^2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", 
                            \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                              MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                              MSf2(Sfe2, 2, 1), MSf2(1, 1, 1))\), " ", 
                            \(MNeu(Neu1)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USfC(Sfe2, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], ")"}], 
                            " ", \((
                              2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 1)\) + 
                                CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 3)\))\), " ", 
                            \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                  \((SW\ \(ZNeu(Neu1, 1)\) + 
                                      CW\ \(ZNeu(Neu1, 2)\))\) - 
                                CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 3)\))\)}], ")"}]}], "-", 
                      \(1\/\(8\ CB2\ CW2\^2\ MW2\^2\ SW2\^2\)\((
                        Alfa2\ \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), 
                            MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                          \(MNeu(Neu1)\)\ 
                          \((C2B\ MW2\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(USfC(Sfe2, 1, 2, 1)\) - 
                              2\ \((C2B\ MW2\ SW2 + CW2\ ME2\ TB2)\)\ 
                                \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 1)\) + 
                              CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\))\)\)}], ")"}]}], "+", 
                RowBox[{\(1\/\(4\ CB2\ CW2\ MW2\^2\ SW2\^2\)\), 
                  RowBox[{"(", 
                    RowBox[{
                    "Alfa2", " ", 
                      \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, MNeu2(Neu1), 
                        MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1), MSf2(1, 1, 1))\), 
                      " ", \(Mat(F2)\), " ", 
                      RowBox[{"(", 
                        RowBox[{
                        \(\((MW2\ S2B - ME2\ TB)\)\ \(USf(Sfe1, 1, 2, 1)\)\), 
                          "-", 
                          RowBox[{"ME", " ", 
                            RowBox[{"(", 
                              RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{"MUE", "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                    SequenceForm[ MUE, 
                                      Superscript[ "*"]],
                                    Editable->False],
                                  "TraditionalForm"]}], ")"}], " ", 
                            \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                      RowBox[{"(", 
                        RowBox[{
                        \(\((MW2\ S2B - ME2\ TB)\)\ \(USfC(Sfe2, 1, 2, 1)\)\),
                           "-", 
                          RowBox[{"ME", " ", 
                            RowBox[{"(", 
                              RowBox[{"MUE", "+", 
                                RowBox[{"TB", " ", 
                                  RowBox[{"(", 
                                    FormBox[
                                      InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                      "TraditionalForm"], ")"}]}]}], ")"}], 
                            " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], ")"}], " ", 
                      \((2\ CB\ MW\ SW\ \(USf(Sfe2, 2, 2, 1)\)\ 
                            \(ZNeu(Neu1, 1)\) + 
                          CW\ ME\ \(USf(Sfe2, 1, 2, 1)\)\ \(ZNeu(Neu1, 3)\))
                        \), " ", 
                      \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                            \(ZNeuC(Neu1, 1)\) + 
                          CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                        \)}], ")"}]}], "+", 
                RowBox[{\(1\/\(4\ CB2\ CW2\ MW2\^2\ SW2\^2\)\), 
                  RowBox[{"(", 
                    RowBox[{
                    "Alfa2", " ", 
                      \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, MNeu2(Neu1), 
                        MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1), MSf2(1, 1, 1))\), 
                      " ", \(Mat(F1)\), " ", 
                      RowBox[{"(", 
                        RowBox[{
                        \(\((MW2\ S2B - ME2\ TB)\)\ \(USf(Sfe1, 1, 2, 1)\)\), 
                          "-", 
                          RowBox[{"ME", " ", 
                            RowBox[{"(", 
                              RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{"MUE", "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                    SequenceForm[ MUE, 
                                      Superscript[ "*"]],
                                    Editable->False],
                                  "TraditionalForm"]}], ")"}], " ", 
                            \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                      RowBox[{"(", 
                        RowBox[{
                        \(\((MW2\ S2B - ME2\ TB)\)\ \(USfC(Sfe2, 1, 2, 1)\)\),
                           "-", 
                          RowBox[{"ME", " ", 
                            RowBox[{"(", 
                              RowBox[{"MUE", "+", 
                                RowBox[{"TB", " ", 
                                  RowBox[{"(", 
                                    FormBox[
                                      InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                      "TraditionalForm"], ")"}]}]}], ")"}], 
                            " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], ")"}], " ", 
                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                            \((SW\ \(ZNeu(Neu1, 1)\) + CW\ \(ZNeu(Neu1, 2)\))
                              \) - 
                          CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ \(ZNeu(Neu1, 3)\))
                        \), " ", 
                      \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                            \(ZNeuC(Neu1, 1)\) + 
                          CW\ \((
                            CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 2)\) - 
                              ME\ \(USf(Sfe2, 2, 2, 1)\)\ \(ZNeuC(Neu1, 3)\))
                            \))\)}], ")"}]}], "+", 
                RowBox[{\(Mat(F4)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(1\/\(4\ CB2\ CW2\ MW2\^2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", 
                            \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                              MNeu2(Neu1), MSf2(Sfe1, 2, 1), 
                              MSf2(Sfe2, 2, 1), MSf2(1, 1, 1))\), " ", 
                            \(MNeu(Neu1)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USfC(Sfe2, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(USfC(Sfe2, 2, 2, 1)\)}]}], ")"}], 
                            " ", \((
                              2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 1)\) + 
                                CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 3)\))\), " ", 
                            \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 1)\) + 
                                CW\ \((
                                  CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 2)\) - 
                                    ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)}], ")"}]}], 
                      "-", \(1\/\(8\ CB2\ CW2\^2\ MW2\^2\ SW2\^2\)\((
                        Alfa2\ \(C0i(cc0, ME2, S, ME2, MNeu2(Neu1), 
                            MSf2(Sfe1, 2, 1), MSf2(Sfe2, 2, 1))\)\ 
                          \(MNeu(Neu1)\)\ 
                          \((C2B\ MW2\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(USfC(Sfe2, 1, 2, 1)\) - 
                              2\ \((C2B\ MW2\ SW2 + CW2\ ME2\ TB2)\)\ 
                                \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(USfC(Sfe2, 2, 2, 1)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe2, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 2)\) - 
                                  ME\ \(USf(Sfe2, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 3)\))\))\))\)\)}], 
                    ")"}]}]}], ")"}]}], ",", 
          RowBox[{"-", 
            RowBox[{\(1\/\(2\ MW2\ SW2\^2\)\), 
              RowBox[{"(", 
                RowBox[{
                "Alfa2", " ", 
                  \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, MCha2(Cha1), 
                    MSf2(1, 1, 1), MSf2(1, 1, 1), MSf2(Sfe1, 2, 1))\), " ", 
                  \(Mat(F1)\), " ", \(SumOver(Cha1, 2)\), " ", 
                  \(SumOver(Sfe1, 2)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                    \(\((MW2\ S2B - ME2\ TB)\)\ \(USf(Sfe1, 1, 2, 1)\)\), "-",
                       
                      RowBox[{"ME", " ", 
                        RowBox[{"(", 
                          RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                            FormBox[
                              InterpretationBox[
                                RowBox[{"MUE", "\[InvisibleSpace]", 
                                  InterpretationBox[\(\[Null]\^"*"\),
                                    Superscript[ "*"],
                                    Editable->False]}],
                                SequenceForm[ MUE, 
                                  Superscript[ "*"]],
                                Editable->False],
                              "TraditionalForm"]}], ")"}], " ", 
                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                  RowBox[{"(", 
                    RowBox[{
                    \(\((MW2\ S2B - ME2\ TB)\)\ \(USfC(Sfe1, 1, 2, 1)\)\), 
                      "-", 
                      RowBox[{"ME", " ", 
                        RowBox[{"(", 
                          RowBox[{"MUE", "+", 
                            RowBox[{"TB", " ", 
                              RowBox[{"(", 
                                FormBox[
                                  InterpretationBox[
                                    RowBox[{
                                    \(Af(2, 1)\), "\[InvisibleSpace]", 
                                      InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                    SequenceForm[ 
                                      Af[ 2, 1], 
                                      Superscript[ "*"]],
                                    Editable->False],
                                  "TraditionalForm"], ")"}]}]}], ")"}], " ", 
                        \(USfC(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                  \(VCha(Cha1, 1)\), " ", \(VChaC(Cha1, 1)\)}], ")"}]}]}], 
          ",", 
          RowBox[{
          \(SumOver(Cha1, 2)\), " ", \(SumOver(Neu1, 4)\), " ", 
            \(SumOver(Sfe1, 2)\), " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(Mat(F4)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", 
                        RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                          RowBox[{"(", 
                            RowBox[{
                            "Alfa2", " ", 
                              \(C0i(cc0, T, ME2, MHp2, MNeu2(Neu1), 
                                MSf2(1, 1, 1), MCha2(Cha1))\), " ", 
                              RowBox[{"(", 
                                RowBox[{
                                \(\((MW2\ S2B - ME2\ TB)\)\ 
                                    \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                  RowBox[{"ME", " ", 
                                    RowBox[{"(", 
                                      RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                    \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                              \(VChaC(Cha1, 1)\), " ", 
                              \((\(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\), 
                              " ", \((
                                2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 1)\) + 
                                  CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu1, 3)\))\)}], ")"}]}]}], "-", 
                      RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", "MHp2", " ", 
                            \(D0i(dd2, ME2, T, ME2, U, MHp2, MHp2, 
                              MSf2(Sfe1, 2, 1), MNeu2(Neu1), MSf2(1, 1, 1), 
                              MCha2(Cha1))\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                            \(VChaC(Cha1, 1)\), " ", 
                            \((\(\(VCha(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\), " ", 
                            \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 1)\) + 
                                CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 3)\))\)}], ")"}]}], "-", 
                      RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", "U", " ", 
                            \(D0i(dd3, ME2, T, ME2, U, MHp2, MHp2, 
                              MSf2(Sfe1, 2, 1), MNeu2(Neu1), MSf2(1, 1, 1), 
                              MCha2(Cha1))\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                      FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                  \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                            \(VChaC(Cha1, 1)\), " ", 
                            \((\(\(VCha(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\), " ", 
                            \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 1)\) + 
                                CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 3)\))\)}], ")"}]}], "+", 
                      RowBox[{
                      \(D0i(dd0, ME2, T, ME2, U, MHp2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MSf2(1, 1, 1), 
                          MCha2(Cha1))\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(2\ CB\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "SB", " ", \(MCha(Cha1)\), " ", 
                                  \(MNeu(Neu1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                  " ", \(VChaC(Cha1, 1)\), " ", 
                                  \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                      \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                    \), " ", 
                                  \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)}], ")"}]}], "-",
                             
                            RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", \(MSf2(Sfe1, 2, 1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                  " ", \(VChaC(Cha1, 1)\), " ", 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\),
                                   " ", 
                                  \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)}], ")"}]}]}], 
                          ")"}]}]}], ")"}]}], "+", 
                RowBox[{\(Mat(F3)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", 
                        RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                          RowBox[{"(", 
                            RowBox[{
                            "Alfa2", " ", 
                              \(C0i(cc0, T, ME2, MHp2, MSf2(1, 1, 1), 
                                MNeu2(Neu1), MSf2(Sfe1, 2, 1))\), " ", 
                              RowBox[{"(", 
                                RowBox[{
                                \(\((MW2\ S2B - ME2\ TB)\)\ 
                                    \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                  RowBox[{"ME", " ", 
                                    RowBox[{"(", 
                                      RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                    " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                              " ", \(VCha(Cha1, 1)\), " ", 
                              \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 1)\) + 
                                  CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\), " ", 
                              \((\(\(VChaC(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                  \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)}],
                             ")"}]}]}], "-", 
                      RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", "MHp2", " ", 
                            \(D0i(dd2, ME2, T, ME2, U, MHp2, MHp2, 
                              MCha2(Cha1), MSf2(1, 1, 1), MNeu2(Neu1), 
                              MSf2(Sfe1, 2, 1))\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                            " ", \(VCha(Cha1, 1)\), " ", 
                            \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 1)\) + 
                                CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 3)\))\), " ", 
                            \((\(\(VChaC(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)}], 
                          ")"}]}], "-", 
                      RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", "U", " ", 
                            \(D0i(dd3, ME2, T, ME2, U, MHp2, MHp2, 
                              MCha2(Cha1), MSf2(1, 1, 1), MNeu2(Neu1), 
                              MSf2(Sfe1, 2, 1))\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                            " ", \(VCha(Cha1, 1)\), " ", 
                            \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 1)\) + 
                                CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 3)\))\), " ", 
                            \((\(\(VChaC(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)}], 
                          ")"}]}], "+", 
                      RowBox[{
                      \(D0i(dd0, ME2, T, ME2, U, MHp2, MHp2, MCha2(Cha1), 
                          MSf2(1, 1, 1), MNeu2(Neu1), MSf2(Sfe1, 2, 1))\), 
                        " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(2\ CB\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "SB", " ", \(MCha(Cha1)\), " ", 
                                  \(MNeu(Neu1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], 
                                    ")"}], " ", \(VCha(Cha1, 1)\), " ", 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\),
                                   " ", 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                      CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)}], ")"}]}], "-", 
                            
                            RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", \(MCha2(Cha1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], 
                                    ")"}], " ", \(VCha(Cha1, 1)\), " ", 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 1)\) + 
                                      CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                  \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                      \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                    \)}], ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
                
                RowBox[{\(Mat(F1)\), " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"-", 
                        RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                          RowBox[{"(", 
                            RowBox[{
                            "Alfa2", " ", 
                              \(D0i(dd0, ME2, T, ME2, U, MHp2, MHp2, 
                                MSf2(Sfe1, 2, 1), MNeu2(Neu1), MSf2(1, 1, 1), 
                                MCha2(Cha1))\), " ", \(MNeu(Neu1)\), " ", 
                              RowBox[{"(", 
                                RowBox[{
                                \(\((MW2\ S2B - ME2\ TB)\)\ 
                                    \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                  RowBox[{"ME", " ", 
                                    RowBox[{"(", 
                                      RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                    \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], " ", 
                              \(VChaC(Cha1, 1)\), " ", 
                              \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\), " ", 
                              \((\(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)}], 
                            ")"}]}]}], "+", 
                      RowBox[{
                      \((D0i(dd2, ME2, T, ME2, U, MHp2, MHp2, 
                              MSf2(Sfe1, 2, 1), MNeu2(Neu1), MSf2(1, 1, 1), 
                              MCha2(Cha1)) + 
                            D0i(dd3, ME2, T, ME2, U, MHp2, MHp2, 
                              MSf2(Sfe1, 2, 1), MNeu2(Neu1), MSf2(1, 1, 1), 
                              MCha2(Cha1)))\), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(2\ CB\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "SB", " ", \(MCha(Cha1)\), " ", 
                                  
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                  " ", \(VChaC(Cha1, 1)\), " ", 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                  \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                      \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                    \)}], ")"}]}], "-", 
                            RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", \(MNeu(Neu1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USf(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{\(TB\ \(Af(2, 1)\)\), "+", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{"MUE", "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ MUE, 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"]}], ")"}], " ", 
                                        \(USf(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                                  " ", \(VChaC(Cha1, 1)\), " ", 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\), " ", 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)}], ")"}]}]}], ")"}]}], "-", 
                      RowBox[{\(1\/\(2\ CB\ CW\ MW2\ SW2\^2\)\), 
                        RowBox[{"(", 
                          RowBox[{
                          "Alfa2", " ", "SB", " ", 
                            \(D0i(dd0, ME2, T, ME2, U, MHp2, MHp2, 
                              MCha2(Cha1), MSf2(1, 1, 1), MNeu2(Neu1), 
                              MSf2(Sfe1, 2, 1))\), " ", \(MCha(Cha1)\), " ", 
                            RowBox[{"(", 
                              RowBox[{
                              \(\((MW2\ S2B - ME2\ TB)\)\ 
                                  \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                RowBox[{"ME", " ", 
                                  RowBox[{"(", 
                                    RowBox[{"MUE", "+", 
                                      RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                  " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], ")"}], 
                            " ", \(VCha(Cha1, 1)\), " ", 
                            \((\(\(UCha(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\), " ", 
                            \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeuC(Neu1, 1)\) + 
                                CW\ \((
                                  CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 2)\) - 
                                    ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)}], ")"}]}], 
                      "+", 
                      RowBox[{
                      \((D0i(dd2, ME2, T, ME2, U, MHp2, MHp2, MCha2(Cha1), 
                              MSf2(1, 1, 1), MNeu2(Neu1), MSf2(Sfe1, 2, 1)) + 
                            D0i(dd3, ME2, T, ME2, U, MHp2, MHp2, MCha2(Cha1), 
                              MSf2(1, 1, 1), MNeu2(Neu1), MSf2(Sfe1, 2, 1)))
                          \), " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{\(1\/\(2\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{"Alfa2", " ", \(MNeu(Neu1)\), " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], 
                                    ")"}], " ", \(VCha(Cha1, 1)\), " ", 
                                  \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\), " ", 
                                  \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                      \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                    \)}], ")"}]}], "-", 
                            RowBox[{\(1\/\(2\ CB\ CW\ MW2\ SW2\^2\)\), 
                              RowBox[{"(", 
                                RowBox[{
                                "Alfa2", " ", "SB", " ", \(MCha(Cha1)\), " ", 
                                  
                                  RowBox[{"(", 
                                    RowBox[{
                                    \(\((MW2\ S2B - ME2\ TB)\)\ 
                                        \(USfC(Sfe1, 1, 2, 1)\)\), "-", 
                                      RowBox[{"ME", " ", 
                                        RowBox[{"(", 
                                        RowBox[{"MUE", "+", 
                                        RowBox[{"TB", " ", 
                                        RowBox[{"(", 
                                        FormBox[
                                        InterpretationBox[
                                        RowBox[{
                                        \(Af(2, 1)\), "\[InvisibleSpace]", 
                                        InterpretationBox[\(\[Null]\^"*"\),
                                        Superscript[ "*"],
                                        Editable->False]}],
                                        SequenceForm[ 
                                        Af[ 2, 1], 
                                        Superscript[ "*"]],
                                        Editable->False],
                                        "TraditionalForm"], ")"}]}]}], ")"}], 
                                        " ", \(USfC(Sfe1, 2, 2, 1)\)}]}], 
                                    ")"}], " ", \(VCha(Cha1, 1)\), " ", 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\),
                                   " ", 
                                  \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)}], ")"}]}]}],
                           ")"}]}]}], ")"}]}]}], ")"}]}], ",", 
          \(\(SumOver(Cha1, 2)\)\ \(SumOver(Neu1, 4)\)\ \(SumOver(Neu2, 4)\)\ 
            \(SumOver(Sfe1, 2)\)\ 
            \((\(Mat(F3)\)\ 
                  \((\(D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, MSf2(Sfe1, 2, 1), 
                          MNeu2(Neu1), MNeu2(Neu2), MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ U\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\))\) + 
                      \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((S + U)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                      CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\))
                                \)\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu2)\)\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\))
                              \)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\))\) + 
                      \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ \((S + U)\)\ \(MNeu(Neu2)\)\ 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                  \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                      CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                      \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                    \))\)\)\) + 
                            \(1\/\(CW2\ MW2\ SW2\^2\)\((
                              2\ Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) + 
                            T\ \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                    \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                    \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) - 
                                \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ \(MNeu(Neu2)\)\ 
                                    \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                    \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                    \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                        \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\) + 
                      \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              3\ Alfa2\ MHp2\ SB2\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) + 
                            \((S + U)\)\ 
                              \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                      \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                        \)\ \((
                                        CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                      \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) - 
                                  \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ \(MNeu(Neu2)\)\ 
                                      \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                      \((2\ CB\ MW\ SW\ 
                                        \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                      \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                        \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                        \)\ \((
                                        \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\) + 
                            T\ \((\(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                    \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                    \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\) - 
                                \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ \(MNeu(Neu2)\)\ 
                                    \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                        CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                    \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                        CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                    \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                        \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                      \)\ \((
                                      \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\))\)\))\))\) + 
                      U\ \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              3\ Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((S + T + U\/2)\)\ \(MNeu(Neu1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                              \)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                  \(MNeu(Neu2)\)\ 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                      CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\))\)\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\))\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\))\)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\))\)\))\))\) + 
                \(Mat(F2)\)\ 
                  \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ SB2\ 
                          \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(UChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 - 
                              \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                        \)\) - 
                      \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ SB2\ 
                          \(C0i(cc2, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(UChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 - 
                              \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                        \)\) - 
                      \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ SB2\ 
                          \(D0i(dd00, ME2, S, MHp2, T, ME2, MHp2, 
                            MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(UChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 - 
                              \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                        \)\) + 
                      \(D0i(dd13, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ T\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\))\) + 
                      \(D0i(dd23, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((S + U)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\))\) + 
                      \(D0i(dd33, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((S + U)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                              2\ Alfa2\ MHp2\ SB2\ \((
                                \(\(UCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                  \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\) + 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\))\) - 
                      \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), 
                            MNeu2(Neu2), MCha2(Cha1))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                        \)\) + 
                      \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \(C0i(cc2, S, MHp2, MHp2, MNeu2(Neu1), 
                            MNeu2(Neu2), MCha2(Cha1))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                        \)\) + 
                      \(1\/\(CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \(D0i(dd00, ME2, S, MHp2, U, ME2, MHp2, 
                            MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                        \)\) + 
                      \(1\/\(CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ U\ 
                          \(D0i(dd33, ME2, S, MHp2, U, ME2, MHp2, 
                            MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu2, 1)\) + 
                              CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeu(Neu2, 3)\))\)\ 
                          \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 1)\) + 
                              CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                      ZNeuC(Neu2, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                        \)\) + 
                      \(D0i(dd13, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((S + T)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd23, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ U\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \((
                                \(\(VCha(Cha1, 2)\)\ 
                                      \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                  \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 1)\) + 
                                      CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeu(Neu2, 3)\))\)\ 
                                  \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\))\)\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((2\ CB\ MW\ SW\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 1)\) + 
                                    CW\ ME\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\))\)\))\))\) + 
                \(Mat(F1)\)\ 
                  \((\(-\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                          Alfa2\ SB2\ 
                            \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), 
                              MNeu2(Neu2), MCha2(Cha1))\)\ 
                            \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                  \((SW\ \(ZNeu(Neu1, 1)\) + 
                                      CW\ \(ZNeu(Neu1, 2)\))\) - 
                                CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                  \(ZNeu(Neu1, 3)\))\)\ 
                            \((\(\(UCha(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                            \((\(\(UChaC(Cha1, 2)\)\ 
                                    \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\)\ 
                            \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                  \(ZNeuC(Neu2, 1)\) + 
                                CW\ \((
                                  CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 2)\) - 
                                    ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                      \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ SB2\ 
                          \(C0i(cc2, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                      ZNeu(Neu2, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                          \((\(\(UChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 - 
                              \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu2, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                      \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ SB2\ 
                          \(D0i(dd00, ME2, S, MHp2, U, ME2, MHp2, 
                            MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                      ZNeu(Neu2, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                          \((\(\(UChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 - 
                              \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu2, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                      \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ SB2\ U\ 
                          \(D0i(dd33, ME2, S, MHp2, U, ME2, MHp2, 
                            MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                      ZNeu(Neu2, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                          \((\(\(UChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 - 
                              \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu2, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                      \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), 
                            MNeu2(Neu2), MCha2(Cha1))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                      ZNeu(Neu2, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu2, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                      \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \(C0i(cc2, S, MHp2, MHp2, MNeu2(Neu1), 
                            MNeu2(Neu2), MCha2(Cha1))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                      ZNeu(Neu2, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu2, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                      \(1\/\(CW2\ MW2\ SW2\^2\)\((
                        Alfa2\ \(D0i(dd00, ME2, S, MHp2, T, ME2, MHp2, 
                            MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                            MCha2(Cha1))\)\ 
                          \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                \((SW\ \(ZNeu(Neu1, 1)\) + 
                                    CW\ \(ZNeu(Neu1, 2)\))\) - 
                              CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(VCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                      ZNeu(Neu2, 2))\)\)\/\@2 + 
                              \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                          \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                \(ZNeuC(Neu2, 1)\) + 
                              CW\ \((
                                CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 2)\) - 
                                  ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                      \(D0i(dd13, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((S + T)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd23, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ U\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd13, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ T\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd23, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((S + U)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd33, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \((S + U)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(CW2\ MW2\ SW2\^2\)\((
                              2\ Alfa2\ MHp2\ \((
                                CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                    \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                  CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                    \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                  \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                      CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeu(Neu1, 3)\))\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))
                                    \)\ \((
                                    CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu2)\)\ 
                                \((CB\ MW\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \((SW\ \(ZNeu(Neu1, 1)\) + 
                                        CW\ \(ZNeu(Neu1, 2)\))\) - 
                                    CW\ ME\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeu(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  \(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\))\) + 
                \(Mat(F4)\)\ 
                  \((\(D0i(dd1, ME2, S, MHp2, U, ME2, MHp2, MSf2(Sfe1, 2, 1), 
                          MNeu2(Neu1), MNeu2(Neu2), MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              3\ Alfa2\ MHp2\ SB2\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((S + T + U\/2)\)\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                  \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd2, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \((S + U)\)\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                  \(MSf2(Sfe1, 2, 1)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    \(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                  \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu1)\)\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \(MNeu(Neu2)\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                  \)\ \((
                                  \(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB2\ \((S + U)\)\ \(MNeu(Neu2)\)\ 
                                  \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                  \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                      \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                    \)\ \((
                                    CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                              2\ Alfa2\ MHp2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            T\ \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ \(MNeu(Neu1)\)\ 
                                    \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                        \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                      \)\ \((
                                      2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                    \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                                \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                    \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\)\ 
                                    \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\))\) + 
                      \(D0i(dd1, ME2, S, MHp2, T, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                                3\ Alfa2\ MHp2\ \(MNeu(Neu1)\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                  \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                      \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))
                                    \)\ \((
                                    CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ SB2\ \(MNeu(Neu2)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 3)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \((S + U)\)\ 
                              \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ \(MNeu(Neu1)\)\ 
                                      \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                        \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                        \)\ \((
                                        2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                                  \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                    Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                      \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                        \)\ \((
                                        2\ CB\ MW\ SW\ 
                                        \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                      \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\)\ 
                                      \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                            T\ \((\(1\/\(CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ \(MNeu(Neu1)\)\ 
                                    \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 + 
                                        \(VCha(Cha1, 1)\)\ \(ZNeu(Neu2, 4)\))
                                      \)\ \((
                                      2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                    \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 + 
                                        \(VChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu1, 4)\))\)\ 
                                    \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                                \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                                  Alfa2\ SB2\ \(MNeu(Neu2)\)\ 
                                    \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 - 
                                        \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))
                                      \)\ \((
                                      2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                        CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                    \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 - 
                                        \(UChaC(Cha1, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\)\ 
                                    \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                        CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\))\) + 
                      U\ \(D0i(dd3, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(C0i(cc0, S, MHp2, MHp2, MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                  \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\) + 
                      \(D0i(dd2, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ U\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\) - 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ MHp2\ \(MNeu(Neu2)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))\))
                              \)\))\) + 
                      \(D0i(dd0, ME2, S, MHp2, U, ME2, MHp2, 
                          MSf2(Sfe1, 2, 1), MNeu2(Neu1), MNeu2(Neu2), 
                          MCha2(Cha1))\)\ 
                        \((\(-\(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                                Alfa2\ SB\ \(MCha(Cha1)\)\ 
                                  \(MSf2(Sfe1, 2, 1)\)\ 
                                  \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                      \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))
                                    \)\ \((
                                    \(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                      \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))
                                    \)\ \((
                                    2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 1)\) + 
                                      CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu1, 3)\))\)\ 
                                  \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 1)\) + 
                                      CW\ \((
                                        CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\)\) + 
                            \(1\/\(2\ CB2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB2\ \(MNeu(Neu1)\)\ 
                                \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(UCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu2, 1)\)\)\/CW + 
                                        ZNeu(Neu2, 2))\)\)\/\@2 - 
                                    \(UCha(Cha1, 1)\)\ \(ZNeu(Neu2, 3)\))\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) + 
                            \(1\/\(2\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ \(MNeu(Neu2)\)\ \(MSf2(Sfe1, 2, 1)\)\ 
                                \((\(\(VCha(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                        ZNeu(Neu1, 2))\)\)\/\@2 + 
                                    \(VCha(Cha1, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                                \((2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\) - 
                            \(1\/\(2\ CB\ CW2\ MW2\ SW2\^2\)\((
                              Alfa2\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                                \(MNeu(Neu2)\)\ 
                                \((\(\(UChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                        ZNeuC(Neu1, 2))\)\)\/\@2 - 
                                    \(UChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 3)\))
                                  \)\ \((
                                  2\ CB\ MW\ SW\ \(USfC(Sfe1, 2, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 1)\) + 
                                    CW\ ME\ \(USfC(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu1, 3)\))\)\ 
                                \((\(\(VChaC(Cha1, 2)\)\ 
                                        \((\(SW\ \(ZNeuC(Neu2, 1)\)\)\/CW + 
                                        ZNeuC(Neu2, 2))\)\)\/\@2 + 
                                    \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu2, 4)\))
                                  \)\ \((
                                  CB\ MW\ SW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                      \(ZNeuC(Neu2, 1)\) + 
                                    CW\ \((
                                      CB\ MW\ \(USf(Sfe1, 1, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 2)\) - 
                                        ME\ \(USf(Sfe1, 2, 2, 1)\)\ 
                                        \(ZNeuC(Neu2, 3)\))\))\))\)\))\))\))
              \)\), ",", 
          \(\(Mat(F1)\)\ \(SumOver(Cha1, 2)\)\ \(SumOver(Cha2, 2)\)\ 
            \(SumOver(Neu1, 4)\)\ 
            \((\(1\/SW2\^2\((
                  Alfa2\ CB2\ 
                    \(C0i(cc0, S, MHp2, MHp2, MCha2(Cha1), MCha2(Cha2), 
                      MNeu2(Neu1))\)\ \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                    \((\(\(VCha(Cha2, 2)\)\ 
                            \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + ZNeu(Neu1, 2))
                              \)\)\/\@2 + 
                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                    \((\(\(VChaC(Cha1, 2)\)\ 
                            \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                ZNeuC(Neu1, 2))\)\)\/\@2 + 
                        \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\) - 
                \(1\/SW2\^2\((
                  Alfa2\ CB2\ 
                    \(C0i(cc2, S, MHp2, MHp2, MCha2(Cha1), MCha2(Cha2), 
                      MNeu2(Neu1))\)\ \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                    \((\(\(VCha(Cha2, 2)\)\ 
                            \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + ZNeu(Neu1, 2))
                              \)\)\/\@2 + 
                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                    \((\(\(VChaC(Cha1, 2)\)\ 
                            \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                ZNeuC(Neu1, 2))\)\)\/\@2 + 
                        \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\) - 
                \(1\/SW2\^2\((
                  2\ Alfa2\ CB2\ 
                    \(D0i(dd00, ME2, S, MHp2, T, ME2, MHp2, MSf2(1, 1, 1), 
                      MCha2(Cha1), MCha2(Cha2), MNeu2(Neu1))\)\ 
                    \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                    \((\(\(VCha(Cha2, 2)\)\ 
                            \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + ZNeu(Neu1, 2))
                              \)\)\/\@2 + 
                        \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                    \((\(\(VChaC(Cha1, 2)\)\ 
                            \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                ZNeuC(Neu1, 2))\)\)\/\@2 + 
                        \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\) + 
                \(D0i(dd13, ME2, S, MHp2, T, ME2, MHp2, MSf2(1, 1, 1), 
                    MCha2(Cha1), MCha2(Cha2), MNeu2(Neu1))\)\ 
                  \((\(1\/SW2\^2\((
                        Alfa2\ CB2\ MHp2\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                        \)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ CB2\ T\ \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\))
                    \) + \(D0i(dd23, ME2, S, MHp2, T, ME2, MHp2, 
                    MSf2(1, 1, 1), MCha2(Cha1), MCha2(Cha2), MNeu2(Neu1))\)\ 
                  \((\(1\/SW2\^2\((
                        Alfa2\ CB2\ \((S + U)\)\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                        \)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ CB2\ MHp2\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\))
                    \) + \(D0i(dd33, ME2, S, MHp2, T, ME2, MHp2, 
                    MSf2(1, 1, 1), MCha2(Cha1), MCha2(Cha2), MNeu2(Neu1))\)\ 
                  \((\(1\/SW2\^2\((
                        2\ Alfa2\ CB2\ \((S + U)\)\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                        \)\) - 
                      \(1\/SW2\^2\((
                        4\ Alfa2\ CB2\ MHp2\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\))
                    \) + \(D0i(dd0, ME2, S, MHp2, T, ME2, MHp2, 
                    MSf2(1, 1, 1), MCha2(Cha1), MCha2(Cha2), MNeu2(Neu1))\)\ 
                  \((\(1\/SW2\^2\((
                        Alfa2\ CB2\ \(MSf2(1, 1, 1)\)\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))
                        \)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                          \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(UChaC(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 - 
                              \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))\)\))
                    \) + \(D0i(dd3, ME2, S, MHp2, T, ME2, MHp2, 
                    MSf2(1, 1, 1), MCha2(Cha1), MCha2(Cha2), MNeu2(Neu1))\)\ 
                  \((\(1\/SW2\^2\((
                        Alfa2\ CB\ SB\ \(MCha(Cha1)\)\ \(MNeu(Neu1)\)\ 
                          \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))\)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ CB2\ \(MSf2(1, 1, 1)\)\ \(VCha(Cha1, 1)\)\ 
                          \(VChaC(Cha2, 1)\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\)\ 
                          \((\(\(VCha(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 + 
                              \(VCha(Cha2, 1)\)\ \(ZNeu(Neu1, 4)\))\))\)\) - 
                      \(1\/SW2\^2\((
                        Alfa2\ SB2\ \(MCha(Cha1)\)\ \(MCha(Cha2)\)\ 
                          \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                          \((\(\(UCha(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeu(Neu1, 1)\)\)\/CW + 
                                      ZNeu(Neu1, 2))\)\)\/\@2 - 
                              \(UCha(Cha1, 1)\)\ \(ZNeu(Neu1, 3)\))\)\ 
                          \((\(\(UChaC(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 - 
                              \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\))
                        \)\) + 
                      \(1\/SW2\^2\((
                        Alfa2\ CB\ SB\ \(MCha(Cha2)\)\ \(MNeu(Neu1)\)\ 
                          \(VCha(Cha1, 1)\)\ \(VChaC(Cha2, 1)\)\ 
                          \((\(\(UChaC(Cha2, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 - 
                              \(UChaC(Cha2, 1)\)\ \(ZNeuC(Neu1, 3)\))\)\ 
                          \((\(\(VChaC(Cha1, 2)\)\ 
                                  \((\(SW\ \(ZNeuC(Neu1, 1)\)\)\/CW + 
                                      ZNeuC(Neu1, 2))\)\)\/\@2 + 
                              \(VChaC(Cha1, 1)\)\ \(ZNeuC(Neu1, 4)\))\))\)\))
                    \))\)\)}], ")"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Hel[_]\  = \ 0; \nhel\  = \ HelicityME[All, \ born]; \)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]8
          \[InvisibleSpace]" helicity matrix elements"\),
        SequenceForm[ "> ", 8, " helicity matrix elements"],
        Editable->False], TraditionalForm]], "Print"],

Cell["running FORM... ", "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[CellGroupData[{

Cell[BoxData[
    \(WriteSquaredME[born, \ {self, \ vert, \ box}, \ hel, \ Abbr[], \ 
      "\<fortran_mssm\>", \n\ \ Drivers\  -> FortranDir]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`{"C:\\Physics\\fortran_mssm\\2to2.F", 
      "C:\\Physics\\fortran_mssm\\2to2.h", 
      "C:\\Physics\\fortran_mssm\\2to3.F", 
      "C:\\Physics\\fortran_mssm\\2to3.h", 
      "C:\\Physics\\fortran_mssm\\abbr_angle.F", 
      "C:\\Physics\\fortran_mssm\\abbr_s.F", 
      "C:\\Physics\\fortran_mssm\\born.F", 
      "C:\\Physics\\fortran_mssm\\box_Cha1_Cha2_Neu1.F", 
      "C:\\Physics\\fortran_mssm\\box_Cha1_Neu1_Neu2_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\box_Cha1_Neu1_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\box_Cha1_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\box.F", 
      "C:\\Physics\\fortran_mssm\\box_Neu1_Sfe1_Sfe2.F", 
      "C:\\Physics\\fortran_mssm\\configure", 
      "C:\\Physics\\fortran_mssm\\gauss.F", 
      "C:\\Physics\\fortran_mssm\\GNUmakefile", 
      "C:\\Physics\\fortran_mssm\\GNUmakefile.in", 
      "C:\\Physics\\fortran_mssm\\GNUPlot.ini", 
      "C:\\Physics\\fortran_mssm\\model.h", 
      "C:\\Physics\\fortran_mssm\\mssm_ini.F", 
      "C:\\Physics\\fortran_mssm\\multigauss.F", 
      "C:\\Physics\\fortran_mssm\\num.F", 
      "C:\\Physics\\fortran_mssm\\prefix.h", 
      "C:\\Physics\\fortran_mssm\\process.h", 
      "C:\\Physics\\fortran_mssm\\rc_zero.F", 
      "C:\\Physics\\fortran_mssm\\renconst.F", 
      "C:\\Physics\\fortran_mssm\\renconst.h", 
      "C:\\Physics\\fortran_mssm\\run.exe", 
      "C:\\Physics\\fortran_mssm\\run.F", 
      "C:\\Physics\\fortran_mssm\\Run.out", 
      "C:\\Physics\\fortran_mssm\\runplt", 
      "C:\\Physics\\fortran_mssm\\run-tot.pol=UUSS.E=00100-00150", 
      "C:\\Physics\\fortran_mssm\\self_Cha1_Cha2.F", 
      "C:\\Physics\\fortran_mssm\\self_Cha1.F", 
      "C:\\Physics\\fortran_mssm\\self.F", 
      "C:\\Physics\\fortran_mssm\\self_Gen1.F", 
      "C:\\Physics\\fortran_mssm\\self_Gen1_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\self_Gen1_Sfe1_Sfe2.F", 
      "C:\\Physics\\fortran_mssm\\self_Neu1_Neu2.F", 
      "C:\\Physics\\fortran_mssm\\sm_ini.F", 
      "C:\\Physics\\fortran_mssm\\squared_me.a", 
      "C:\\Physics\\fortran_mssm\\squared_me.F", 
      "C:\\Physics\\fortran_mssm\\vars.h", 
      "C:\\Physics\\fortran_mssm\\vegas.F", 
      "C:\\Physics\\fortran_mssm\\vert_Cha1_Cha2.F", 
      "C:\\Physics\\fortran_mssm\\vert_Cha1_Cha2_Neu1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Cha1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Cha1_Neu1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Cha1_Neu1_Neu2.F", 
      "C:\\Physics\\fortran_mssm\\vert.F", 
      "C:\\Physics\\fortran_mssm\\vert_Gen1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Gen1_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Gen1_Sfe1_Sfe2.F", 
      "C:\\Physics\\fortran_mssm\\vert_Gen1_Sfe1_Sfe2_Sfe3.F", 
      "C:\\Physics\\fortran_mssm\\vert_Neu1_Neu2_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Neu1_Sfe1.F", 
      "C:\\Physics\\fortran_mssm\\vert_Neu1_Sfe1_Sfe2.F"}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\( (*\n
      InsertFieldsHook[tops_, \ f1_F\  -> \ f2_F]\  := \n\ \ 
        InsertFields[tops, \ f1\  -> \ f2, \ ExcludeParticles\  -> \ V[1]]
          \n*) \n (*\ dZGp1\ should\ really\ be\ dZHp1, \ 
      but\ is\ a\ temporary\ hack\n
      \t\ \ \ 
      until\ the\ "\<real\>"\ MSSM\ counterterms\ are\ implemented\ *) \)\)], 
  "Input"],

Cell[BoxData[
    \(RenConst[\ dZGp1\ ]\  := \ 
      \(-ReTilde[DSelfEnergy[S[5]\  -> \ S[5], \ MHp]]\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(WriteRenConst[counter, FortranDir]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 11 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 11 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 23 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 23 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 34 Classes insertions"\),
        SequenceForm[ "in total: ", "7 Generic, 34 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 11 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 11 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 23 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 23 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 34 Classes amplitudes"\),
        SequenceForm[ "in total: ", "7 Generic, 34 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]5
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 5, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]2
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 2, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell["\<\
running FORM... 
running FORM... 
running FORM... 
running FORM... 
running FORM... 
running FORM... \
\>", "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 6 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 6 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 14 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 14 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 20 Classes insertions"\),
        SequenceForm[ "in total: ", "7 Generic, 20 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 6 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 6 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 14 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 14 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 20 Classes amplitudes"\),
        SequenceForm[ "in total: ", "7 Generic, 20 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]4
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 4, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]3
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 3, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 6 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 6 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 14 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 14 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 20 Classes insertions"\),
        SequenceForm[ "in total: ", "7 Generic, 20 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 6 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 6 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 14 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 14 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 20 Classes amplitudes"\),
        SequenceForm[ "in total: ", "7 Generic, 20 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]4
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 4, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]3
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 3, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 13 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 13 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"3 Generic, 14 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "3 Generic, 14 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"5 Generic, 27 Classes insertions"\),
        SequenceForm[ "in total: ", "5 Generic, 27 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 13 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 13 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"3 Generic, 14 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "3 Generic, 14 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"5 Generic, 27 Classes amplitudes"\),
        SequenceForm[ "in total: ", "5 Generic, 27 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]5
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 5, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]0
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 0, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"0 Generic, 0 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "0 Generic, 0 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 5 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "2 Generic, 5 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"2 Generic, 5 Classes insertions"\),
        SequenceForm[ "in total: ", "2 Generic, 5 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 5 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 5 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"2 Generic, 5 Classes amplitudes"\),
        SequenceForm[ "in total: ", "2 Generic, 5 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]2
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 2, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]0
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 0, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Excluding "\[InvisibleSpace]18
          \[InvisibleSpace]" field point(s) (incl. charge conjugate ones)"\),
        SequenceForm[ 
        "Excluding ", 18, " field point(s) (incl. charge conjugate ones)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("inserting at level(s) "\[InvisibleSpace]{Generic, Classes}\),
        SequenceForm[ "inserting at level(s) ", {Generic, Classes}],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 13 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 13 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 21 Classes \
insertions"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 21 Classes insertions"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 34 Classes insertions"\),
        SequenceForm[ "in total: ", "7 Generic, 34 Classes insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("Restoring "\[InvisibleSpace]18\[InvisibleSpace]" field point(s)"\),
        
        SequenceForm[ "Restoring ", 18, " field point(s)"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`""\)], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("creating amplitudes at level(s) "\[InvisibleSpace]{Generic, 
            Classes}\),
        SequenceForm[ "creating amplitudes at level(s) ", {Generic, Classes}],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]1
          \[InvisibleSpace]": "\[InvisibleSpace]"2 Generic, 13 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 1, ": ", "2 Generic, 13 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> Top. "\[InvisibleSpace]2
          \[InvisibleSpace]": "\[InvisibleSpace]"5 Generic, 21 Classes \
amplitudes"\),
        SequenceForm[ "> Top. ", 2, ": ", "5 Generic, 21 Classes amplitudes"],
        
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("in total: "\[InvisibleSpace]"7 Generic, 34 Classes amplitudes"\),
        SequenceForm[ "in total: ", "7 Generic, 34 Classes amplitudes"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]7
          \[InvisibleSpace]" amplitudes with insertions"\),
        SequenceForm[ "> ", 7, " amplitudes with insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("> "\[InvisibleSpace]0
          \[InvisibleSpace]" amplitudes without insertions"\),
        SequenceForm[ "> ", 0, " amplitudes without insertions"],
        Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"Done"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`"ok"\)], "Print"],

Cell[BoxData[
    \(TraditionalForm\`{"renconst.F", "renconst.h"}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Run["\<CompileRun.bat \>"<>\ FortranDir\ <>\ "\< u u u u 100 150\>"]\)],
   "Input"],

Cell[BoxData[
    \(TraditionalForm\`0\)], "Output"],

Cell[CellGroupData[{

Cell["\<\
ReadList[OpenRead[\"CompileRun.out\", DOSTextFormat -> True], String]\
\>", "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`{"g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
abbr_s.F", "ar cr squared_me.a abbr_s.o", "rm -f abbr_s.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
abbr_angle.F", "ar cr squared_me.a abbr_angle.o", "rm -f abbr_angle.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
born.F", "ar cr squared_me.a born.o", "rm -f born.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self.F", "ar cr squared_me.a self.o", "rm -f self.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self_Gen1.F", "ar cr squared_me.a self_Gen1.o", "rm -f self_Gen1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self_Gen1_Sfe1.F", "ar cr squared_me.a self_Gen1_Sfe1.o", 
      "rm -f self_Gen1_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self_Cha1.F", "ar cr squared_me.a self_Cha1.o", "rm -f self_Cha1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self_Neu1_Neu2.F", "ar cr squared_me.a self_Neu1_Neu2.o", 
      "rm -f self_Neu1_Neu2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self_Gen1_Sfe1_Sfe2.F", "ar cr squared_me.a self_Gen1_Sfe1_Sfe2.o", 
      "rm -f self_Gen1_Sfe1_Sfe2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
self_Cha1_Cha2.F", "ar cr squared_me.a self_Cha1_Cha2.o", 
      "rm -f self_Cha1_Cha2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert.F", "ar cr squared_me.a vert.o", "rm -f vert.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Neu1_Sfe1.F", "ar cr squared_me.a vert_Neu1_Sfe1.o", 
      "rm -f vert_Neu1_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Gen1.F", "ar cr squared_me.a vert_Gen1.o", "rm -f vert_Gen1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Gen1_Sfe1.F", "ar cr squared_me.a vert_Gen1_Sfe1.o", 
      "rm -f vert_Gen1_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Gen1_Sfe1_Sfe2.F", "ar cr squared_me.a vert_Gen1_Sfe1_Sfe2.o", 
      "rm -f vert_Gen1_Sfe1_Sfe2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Cha1.F", "ar cr squared_me.a vert_Cha1.o", "rm -f vert_Cha1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Cha1_Neu1.F", "ar cr squared_me.a vert_Cha1_Neu1.o", 
      "rm -f vert_Cha1_Neu1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Neu1_Sfe1_Sfe2.F", "ar cr squared_me.a vert_Neu1_Sfe1_Sfe2.o", 
      "rm -f vert_Neu1_Sfe1_Sfe2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Neu1_Neu2_Sfe1.F", "ar cr squared_me.a vert_Neu1_Neu2_Sfe1.o", 
      "rm -f vert_Neu1_Neu2_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Gen1_Sfe1_Sfe2_Sfe3.F", "ar cr squared_me.a vert_Gen1_Sfe1_Sfe2_Sfe3.o", 
      "rm -f vert_Gen1_Sfe1_Sfe2_Sfe3.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Cha1_Neu1_Neu2.F", "ar cr squared_me.a vert_Cha1_Neu1_Neu2.o", 
      "rm -f vert_Cha1_Neu1_Neu2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Cha1_Cha2.F", "ar cr squared_me.a vert_Cha1_Cha2.o", 
      "rm -f vert_Cha1_Cha2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
vert_Cha1_Cha2_Neu1.F", "ar cr squared_me.a vert_Cha1_Cha2_Neu1.o", 
      "rm -f vert_Cha1_Cha2_Neu1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
box.F", "ar cr squared_me.a box.o", "rm -f box.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
box_Neu1_Sfe1_Sfe2.F", "ar cr squared_me.a box_Neu1_Sfe1_Sfe2.o", 
      "rm -f box_Neu1_Sfe1_Sfe2.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
box_Cha1_Sfe1.F", "ar cr squared_me.a box_Cha1_Sfe1.o", 
      "rm -f box_Cha1_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
box_Cha1_Neu1_Sfe1.F", "ar cr squared_me.a box_Cha1_Neu1_Sfe1.o", 
      "rm -f box_Cha1_Neu1_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
box_Cha1_Neu1_Neu2_Sfe1.F", "ar cr squared_me.a box_Cha1_Neu1_Neu2_Sfe1.o", 
      "rm -f box_Cha1_Neu1_Neu2_Sfe1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
box_Cha1_Cha2_Neu1.F", "ar cr squared_me.a box_Cha1_Cha2_Neu1.o", 
      "rm -f box_Cha1_Cha2_Neu1.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
renconst.F", "ar cr squared_me.a renconst.o", "rm -f renconst.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -c \
squared_me.F", "ar cr squared_me.a squared_me.o", "rm -f squared_me.o", 
      "g77 -O0 -ffixed-line-length-none -I. -I../LoopTool/include -g -o run \
run.F squared_me.a -L../LoopTool/ -looptools -ldcuhre \
-Lc:/physics/cernlib/2001/lib -lspdf -ltcgen -ltcgens -llapack -lblas"}\)], 
  "Output"],

Cell[CellGroupData[{

Cell["\<\
ReadList[OpenRead[FortranDir<>\"\\\\Run.out\", DOSTextFormat -> True], \
String]\
\>", "Input"],

Cell[BoxData[
    \(TraditionalForm
    \`{" output file: ", 
      " ./run-tot.pol=UUSS.E=00100-00150                                      \
                                                                              \
                                                    ", 
      " ====================================================", 
      "   FF 2.0, a package to evaluate one-loop integrals", 
      " written by G. J. van Oldenborgh, NIKHEF-H, Amsterdam", 
      " ====================================================", 
      " for the algorithms used see preprint NIKHEF-H 89/17,", 
      " 'New Algorithms for One-loop Integrals', by G.J. van", 
      " Oldenborgh and J.A.M. Vermaseren, published in ", 
      " Zeitschrift fuer Physik C46(1990)425.", 
      " ====================================================", "  ", 
      "  *****  CERN Computer Program Library     -     Reference:  W5051  \
*****", "  *****  PDFLIB Version:  8.04  Released on  2000-04-17  at  12.24  \
*****", "  PDFLIB  : Your selected PDF set has been retracted by the \
authors", "  Warning : ALL parameters set to default (Nucleon PDFs) !!", 
      " ============================", " SM parameters", 
      " ----------------------------", " 1/Alfa =  137.035989", 
      " Alfas(s) =  0.114681874", " GF =  1.16639E-05", 
      " SW2 =  0.222249945", " ----------------------------", 
      " Gauge-boson masses:", " MW =  80.419", " MZ =  91.1882", 
      " ----------------------------", " Fermion masses:", 
      " ME =  0.00051099907", " MM =  0.105658389", " ML =  1.777", 
      " MU =  0.0538", " MC =  1.5", " MT =  174.3", " MD =  0.0538", 
      " MS =  0.15", " MB =  4.7", " ----------------------------", 
      " Higgs mass:", " MH =  0.", " ============================", 
      " SUSY parameters", " ----------------------------", " TB =  1.5", 
      " M_1 =  47.6266857", " M_2 =  100.", " MUE = (-1000.,0.)", 
      " MSusy =  1000.", " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  97.0248606", 
      " MHH =  609.057162", " MHp =  605.365357", " MA0 =  600.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.35533078440        1000.44405439942    ", 
      " MSf(1|2, 2, 2) =    1000.26041670439        1000.53896221644    ", 
      " MSf(1|2, 2, 3) =    998.177999268835        1002.61961433290    ", 
      " MSf(1|2, 3, 1) =    999.431166073508        999.769101472152    ", 
      " MSf(1|2, 3, 2) =    998.339410501892        1000.86154268577    ", 
      " MSf(1|2, 3, 3) =    859.698063584938        1148.94783417565    ", 
      " MSf(1|2, 4, 1) =    1000.11053471694        1000.68877177096    ", 
      " MSf(1|2, 4, 2) =    1000.06171470305        1000.73758078559    ", 
      " MSf(1|2, 4, 3) =    994.514039637689        1006.27287490766    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1005.90069", " MCha(2) =  105.34811", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1007.30141", " MNeu(2) =  1000.35242", 
      " MNeu(3) =  105.492022", " MNeu(4) =  49.0836578", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1210.74", 
      "# TB=    1.50", "# MA0=  600.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " ffxdb1: IR divergent B1', using cutoff   1.", 
      " ffxdb0: IR divergent B0', using cutoff   1.", 
      " sqrt(s) =  1210.74071", 
      " ffxc0i: infra-red divergent threepoint function, working with a \
cutoff   1.", " ffxdbd: using IR cutoff lambda^2 =   1.", 
      " ffzdbd: using IR cutoff lambda^2 =   1.", 
      " ============================", " SM parameters", 
      " ----------------------------", " 1/Alfa =  137.035989", 
      " Alfas(s) =  0.114681874", " GF =  1.16639E-05", 
      " SW2 =  0.222249945", " ----------------------------", 
      " Gauge-boson masses:", " MW =  80.419", " MZ =  91.1882", 
      " ----------------------------", " Fermion masses:", 
      " ME =  0.00051099907", " MM =  0.105658389", " ML =  1.777", 
      " MU =  0.0538", " MC =  1.5", " MT =  174.3", " MD =  0.0538", 
      " MS =  0.15", " MB =  4.7", " ----------------------------", 
      " Higgs mass:", " MH =  97.0248606", " ============================", 
      " SUSY parameters", " ----------------------------", " TB =  1.5", 
      " M_1 =  47.6266857", " M_2 =  100.", " MUE = (-1000.,0.)", 
      " MSusy =  1000.", " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  97.3544281", 
      " MHH =  707.74342", " MHp =  704.604297", " MA0 =  700.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.35533078440        1000.44405439942    ", 
      " MSf(1|2, 2, 2) =    1000.26041670439        1000.53896221644    ", 
      " MSf(1|2, 2, 3) =    998.177999268835        1002.61961433290    ", 
      " MSf(1|2, 3, 1) =    999.431166073508        999.769101472152    ", 
      " MSf(1|2, 3, 2) =    998.339410501892        1000.86154268577    ", 
      " MSf(1|2, 3, 3) =    859.698063584938        1148.94783417565    ", 
      " MSf(1|2, 4, 1) =    1000.11053471694        1000.68877177096    ", 
      " MSf(1|2, 4, 2) =    1000.06171470305        1000.73758078559    ", 
      " MSf(1|2, 4, 3) =    994.514039637689        1006.27287490766    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1005.90069", " MCha(2) =  105.34811", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1007.30141", " MNeu(2) =  1000.35242", 
      " MNeu(3) =  105.492022", " MNeu(4) =  49.0836578", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1409.22", 
      "# TB=    1.50", "# MA0=  700.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1409.21859", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  97.3544281", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  1.5", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  97.579676", 
      " MHH =  806.764604", " MHp =  804.03185", " MA0 =  800.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    999.200133200856       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.35533078440        1000.44405439942    ", 
      " MSf(1|2, 2, 2) =    1000.26041670439        1000.53896221644    ", 
      " MSf(1|2, 2, 3) =    998.177999268835        1002.61961433290    ", 
      " MSf(1|2, 3, 1) =    999.431166073508        999.769101472152    ", 
      " MSf(1|2, 3, 2) =    998.339410501892        1000.86154268577    ", 
      " MSf(1|2, 3, 3) =    859.698063584938        1148.94783417565    ", 
      " MSf(1|2, 4, 1) =    1000.11053471694        1000.68877177096    ", 
      " MSf(1|2, 4, 2) =    1000.06171470305        1000.73758078559    ", 
      " MSf(1|2, 4, 3) =    994.514039637689        1006.27287490766    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1005.90069", " MCha(2) =  105.34811", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1007.30141", " MNeu(2) =  1000.35242", 
      " MNeu(3) =  105.492022", " MNeu(4) =  49.0836578", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1608.07", 
      "# TB=    1.50", "# MA0=  800.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1608.0737", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  97.579676", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  2.", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  103.177575", 
      " MHH =  606.17815", " MHp =  605.365357", " MA0 =  600.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.55426382998        1000.69263586675    ", 
      " MSf(1|2, 2, 2) =    1000.45060440550        1000.79628137690    ", 
      " MSf(1|2, 2, 3) =    997.956739235412        1003.28622457322    ", 
      " MSf(1|2, 3, 1) =    999.118749826311        999.633504434809    ", 
      " MSf(1|2, 3, 2) =    998.222574325835        1000.53066207914    ", 
      " MSf(1|2, 3, 3) =    876.175041403327        1136.03859550462    ", 
      " MSf(1|2, 4, 1) =    1000.17743449474        1001.06927415748    ", 
      " MSf(1|2, 4, 2) =    1000.13049114863        1001.11619306681    ", 
      " MSf(1|2, 4, 3) =    993.550246979090        1007.66893029088    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1005.98415", " MCha(2) =  104.54814", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1006.83861", " MNeu(2) =  1000.91547", 
      " MNeu(3) =  104.652766", " MNeu(4) =  48.8970612", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1210.74", 
      "# TB=    2.00", "# MA0=  600.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1210.74071", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  103.177575", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  2.", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  103.455749", 
      " MHH =  705.271168", " MHp =  704.604297", " MA0 =  700.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.55426382998        1000.69263586675    ", 
      " MSf(1|2, 2, 2) =    1000.45060440550        1000.79628137690    ", 
      " MSf(1|2, 2, 3) =    997.956739235412        1003.28622457322    ", 
      " MSf(1|2, 3, 1) =    999.118749826311        999.633504434809    ", 
      " MSf(1|2, 3, 2) =    998.222574325835        1000.53066207914    ", 
      " MSf(1|2, 3, 3) =    876.175041403327        1136.03859550462    ", 
      " MSf(1|2, 4, 1) =    1000.17743449474        1001.06927415748    ", 
      " MSf(1|2, 4, 2) =    1000.13049114863        1001.11619306681    ", 
      " MSf(1|2, 4, 3) =    993.550246979090        1007.66893029088    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1005.98415", " MCha(2) =  104.54814", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1006.83861", " MNeu(2) =  1000.91547", 
      " MNeu(3) =  104.652766", " MNeu(4) =  48.8970612", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1409.22", 
      "# TB=    2.00", "# MA0=  700.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1409.21859", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  103.455749", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  2.", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  103.645963", 
      " MHH =  804.598809", " MHp =  804.03185", " MA0 =  800.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    998.751927985237       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.55426382998        1000.69263586675    ", 
      " MSf(1|2, 2, 2) =    1000.45060440550        1000.79628137690    ", 
      " MSf(1|2, 2, 3) =    997.956739235412        1003.28622457322    ", 
      " MSf(1|2, 3, 1) =    999.118749826311        999.633504434809    ", 
      " MSf(1|2, 3, 2) =    998.222574325835        1000.53066207914    ", 
      " MSf(1|2, 3, 3) =    876.175041403327        1136.03859550462    ", 
      " MSf(1|2, 4, 1) =    1000.17743449474        1001.06927415748    ", 
      " MSf(1|2, 4, 2) =    1000.13049114863        1001.11619306681    ", 
      " MSf(1|2, 4, 3) =    993.550246979090        1007.66893029088    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1005.98415", " MCha(2) =  104.54814", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1006.83861", " MNeu(2) =  1000.91547", 
      " MNeu(3) =  104.652766", " MNeu(4) =  48.8970612", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1608.07", 
      "# TB=    2.00", "# MA0=  800.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1608.0737", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  103.645963", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  2.5", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  108.05117", 
      " MHH =  604.405089", " MHp =  605.365357", " MA0 =  600.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.66890111087        1000.83587965699    ", 
      " MSf(1|2, 2, 2) =    1000.54963007377        1000.95512773816    ", 
      " MSf(1|2, 2, 3) =    997.640600296989        1003.85768700337    ", 
      " MSf(1|2, 3, 1) =    998.937865639636        999.556120846408    ", 
      " MSf(1|2, 3, 2) =    998.152896799263        1000.34223491391    ", 
      " MSf(1|2, 3, 3) =    885.920067365982        1128.22688612688    ", 
      " MSf(1|2, 4, 1) =    1000.21471842658        1001.28978347450    ", 
      " MSf(1|2, 4, 2) =    1000.16155912037        1001.34290246479    ", 
      " MSf(1|2, 4, 3) =    992.493513491930        1008.96556598089    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1006.05842", " MCha(2) =  103.831097", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1006.42316", " MNeu(2) =  1001.4194", 
      " MNeu(3) =  103.905455", " MNeu(4) =  48.7249922", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1210.74", 
      "# TB=    2.50", "# MA0=  600.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1210.74071", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  108.05117", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  2.5", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  108.275026", 
      " MHH =  703.752977", " MHp =  704.604297", " MA0 =  700.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.66890111087        1000.83587965699    ", 
      " MSf(1|2, 2, 2) =    1000.54963007377        1000.95512773816    ", 
      " MSf(1|2, 2, 3) =    997.640600296989        1003.85768700337    ", 
      " MSf(1|2, 3, 1) =    998.937865639636        999.556120846408    ", 
      " MSf(1|2, 3, 2) =    998.152896799263        1000.34223491391    ", 
      " MSf(1|2, 3, 3) =    885.920067365982        1128.22688612688    ", 
      " MSf(1|2, 4, 1) =    1000.21471842658        1001.28978347450    ", 
      " MSf(1|2, 4, 2) =    1000.16155912037        1001.34290246479    ", 
      " MSf(1|2, 4, 3) =    992.493513491930        1008.96556598089    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1006.05842", " MCha(2) =  103.831097", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1006.42316", " MNeu(2) =  1001.4194", 
      " MNeu(3) =  103.905455", " MNeu(4) =  48.7249922", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1409.22", 
      "# TB=    2.50", "# MA0=  700.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1409.21859", " ============================", 
      " SM parameters", " ----------------------------", 
      " 1/Alfa =  137.035989", " Alfas(s) =  0.114681874", 
      " GF =  1.16639E-05", " SW2 =  0.222249945", 
      " ----------------------------", " Gauge-boson masses:", 
      " MW =  80.419", " MZ =  91.1882", " ----------------------------", 
      " Fermion masses:", " ME =  0.00051099907", " MM =  0.105658389", 
      " ML =  1.777", " MU =  0.0538", " MC =  1.5", " MT =  174.3", 
      " MD =  0.0538", " MS =  0.15", " MB =  4.7", 
      " ----------------------------", " Higgs mass:", " MH =  108.275026", 
      " ============================", " SUSY parameters", 
      " ----------------------------", " TB =  2.5", " M_1 =  47.6266857", 
      " M_2 =  100.", " MUE = (-1000.,0.)", " MSusy =  1000.", 
      " Af(top) = (1000.,0.)", " Af(bottom) = (1000.,0.)", 
      " ----------------------------", " Higgs masses:", 
      " (using 2-loop approximation for Higgs mass)", " Mh0 =  108.428273", 
      " MHH =  803.27126", " MHp =  804.03185", " MA0 =  800.", 
      " ----------------------------", " Sfermion masses:", 
      " (non-diagonal sfermion mass matrix: 1-2 basis)", 
      " MSf(1|2, 1, 1) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 1, 2) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 1, 3) =    998.493511416729       -1.00000000000000    ", 
      " MSf(1|2, 2, 1) =    1000.66890111087        1000.83587965699    ", 
      " MSf(1|2, 2, 2) =    1000.54963007377        1000.95512773816    ", 
      " MSf(1|2, 2, 3) =    997.640600296989        1003.85768700337    ", 
      " MSf(1|2, 3, 1) =    998.937865639636        999.556120846408    ", 
      " MSf(1|2, 3, 2) =    998.152896799263        1000.34223491391    ", 
      " MSf(1|2, 3, 3) =    885.920067365982        1128.22688612688    ", 
      " MSf(1|2, 4, 1) =    1000.21471842658        1001.28978347450    ", 
      " MSf(1|2, 4, 2) =    1000.16155912037        1001.34290246479    ", 
      " MSf(1|2, 4, 3) =    992.493513491930        1008.96556598089    ", 
      " ----------------------------", " Chargino masses:", 
      " MCha(1) =  1006.05842", " MCha(2) =  103.831097", 
      " ----------------------------", " Neutralino masses:", 
      " MNeu(1) =  1006.42316", " MNeu(2) =  1001.4194", 
      " MNeu(3) =  103.905455", " MNeu(4) =  48.7249922", 
      " ----------------------------", " Gluino mass:", " MGl =  325.746076", 
      " ============================", 
      "# WARNING: adjusted sqrt(s) to above-threshold value    1608.07", 
      "# TB=    2.50", "# MA0=  800.00", "# M_2=  100.00", "# MUE=-1000.00", 
      "# MSusy= 1000.00", "# Au= 1000.00", "# Ad= 1000.00", 
      " sqrt(s) =  1608.0737", " ", "total number of errors and warnings", 
      "===================================", "fferr: no errors", " "}\)], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Install["\<FormCalc\\ReadData.exe\>"]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`LinkObject[".\\FormCalc\\ReadData.exe", 4, 4]\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = ReadData[FortranDir\ <>\ "\<\\runplt\>"]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`9\)], "Output"],

Cell[CellGroupData[{

Cell["Table[Para[x],{x,n}]", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"TB", "\[Rule]", 
                StyleBox["1.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2]}], ",", \(MA0 \[Rule] 600.`\), ",", 
              \(M$2 \[Rule] 100.`\), ",", \(MUE \[Rule] \(-1000.`\)\), ",", 
              \(MSusy \[Rule] 1000.`\), ",", \(Au \[Rule] 1000.`\), ",", 
              \(Ad \[Rule] 1000.`\)}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"TB", "\[Rule]", 
                StyleBox["1.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2]}], ",", \(MA0 \[Rule] 700.`\), ",", 
              \(M$2 \[Rule] 100.`\), ",", \(MUE \[Rule] \(-1000.`\)\), ",", 
              \(MSusy \[Rule] 1000.`\), ",", \(Au \[Rule] 1000.`\), ",", 
              \(Ad \[Rule] 1000.`\)}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"TB", "\[Rule]", 
                StyleBox["1.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2]}], ",", \(MA0 \[Rule] 800.`\), ",", 
              \(M$2 \[Rule] 100.`\), ",", \(MUE \[Rule] \(-1000.`\)\), ",", 
              \(MSusy \[Rule] 1000.`\), ",", \(Au \[Rule] 1000.`\), ",", 
              \(Ad \[Rule] 1000.`\)}], "}"}], ",", 
          \({TB \[Rule] 2.`, MA0 \[Rule] 600.`, M$2 \[Rule] 100.`, 
            MUE \[Rule] \(-1000.`\), MSusy \[Rule] 1000.`, Au \[Rule] 1000.`, 
            Ad \[Rule] 1000.`}\), ",", 
          \({TB \[Rule] 2.`, MA0 \[Rule] 700.`, M$2 \[Rule] 100.`, 
            MUE \[Rule] \(-1000.`\), MSusy \[Rule] 1000.`, Au \[Rule] 1000.`, 
            Ad \[Rule] 1000.`}\), ",", 
          \({TB \[Rule] 2.`, MA0 \[Rule] 800.`, M$2 \[Rule] 100.`, 
            MUE \[Rule] \(-1000.`\), MSusy \[Rule] 1000.`, Au \[Rule] 1000.`, 
            Ad \[Rule] 1000.`}\), ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"TB", "\[Rule]", 
                StyleBox["2.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2]}], ",", \(MA0 \[Rule] 600.`\), ",", 
              \(M$2 \[Rule] 100.`\), ",", \(MUE \[Rule] \(-1000.`\)\), ",", 
              \(MSusy \[Rule] 1000.`\), ",", \(Au \[Rule] 1000.`\), ",", 
              \(Ad \[Rule] 1000.`\)}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"TB", "\[Rule]", 
                StyleBox["2.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2]}], ",", \(MA0 \[Rule] 700.`\), ",", 
              \(M$2 \[Rule] 100.`\), ",", \(MUE \[Rule] \(-1000.`\)\), ",", 
              \(MSusy \[Rule] 1000.`\), ",", \(Au \[Rule] 1000.`\), ",", 
              \(Ad \[Rule] 1000.`\)}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"TB", "\[Rule]", 
                StyleBox["2.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2]}], ",", \(MA0 \[Rule] 800.`\), ",", 
              \(M$2 \[Rule] 100.`\), ",", \(MUE \[Rule] \(-1000.`\)\), ",", 
              \(MSusy \[Rule] 1000.`\), ",", \(Au \[Rule] 1000.`\), ",", 
              \(Ad \[Rule] 1000.`\)}], "}"}]}], "}"}], TraditionalForm]], 
  "Output"],

Cell[CellGroupData[{

Cell["Table[Data[x],{x,n}]", "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        RowBox[{"(", GridBox[{
              {
                RowBox[{"{", 
                  RowBox[{
                    StyleBox["1210.74071417387996`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->15], ",", 
                    "1.2475592266984501`16.301*^-9", ",", 
                    "2.0020910693223799`*^-8"}], "}"}]},
             
               {\({1409.2185942982301`16.301, 
                  7.3299242260627798`16.9031*^-10, 1.19011054150947014`*^-8}
                  \)},
              {
                RowBox[{"{", 
                  RowBox[{
                    StyleBox["1608.0736996848099`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->15], ",", 
                    "4.6164894397469801`16.9031*^-10", ",", 
                    "7.5791881412403103`16.9031*^-9"}], "}"}]},
              {
                RowBox[{"{", 
                  RowBox[{
                    StyleBox["1210.74071417387996`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->15], ",", 
                    "1.2475592266984501`16.301*^-9", ",", 
                    "1.9974329951770801`16.301*^-8"}], "}"}]},
             
               {\({1409.2185942982301`16.301, 
                  7.3299242260627798`16.9031*^-10, 
                  1.1875631131328499`16.301*^-8}\)},
              {
                RowBox[{"{", 
                  RowBox[{
                    StyleBox["1608.0736996848099`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->15], ",", 
                    "4.6164894397469801`16.9031*^-10", ",", 
                    "7.5641468098170602`16.9031*^-9"}], "}"}]},
              {
                RowBox[{"{", 
                  RowBox[{
                    StyleBox["1210.74071417387996`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->15], ",", 
                    "1.2475592266984501`16.301*^-9", ",", 
                    "1.9886145523641499`16.301*^-8"}], "}"}]},
              {\({1409.2185942982301`16.301, 7.3299242260627798`16.9031*^-10, 
                  1.18248783433974002`*^-8}\)},
              {
                RowBox[{"{", 
                  RowBox[{
                    StyleBox["1608.0736996848099`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->15], ",", 
                    "4.6164894397469801`16.9031*^-10", ",", 
                    "7.5327484159917103`16.9031*^-9"}], "}"}]}
              }], ")"}],
        MatrixForm[
         {{{1210.74071417388, 1.24755922669845*^-9, .200209106932238*^-7}}, {{
          1409.21859429823, 7.32992422606278*^-10, .119011054150947*^-7}}, {{
          1608.07369968481, 4.61648943974698*^-10, 7.57918814124031*^-9}}, {{
          1210.74071417388, 1.24755922669845*^-9, 1.99743299517708*^-8}}, {{
          1409.21859429823, 7.32992422606278*^-10, 1.18756311313285*^-8}}, {{
          1608.07369968481, 4.61648943974698*^-10, 7.56414680981706*^-9}}, {{
          1210.74071417388, 1.24755922669845*^-9, 1.98861455236415*^-8}}, {{
          1409.21859429823, 7.32992422606278*^-10, .118248783433974*^-7}}, {{
          1608.07369968481, 4.61648943974698*^-10, 7.53274841599171*^-9}}}, 
          TableDepth -> 2]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[
"ScanPlot3D[Para[n]\[LeftDoubleBracket]1\[RightDoubleBracket]\
\[LeftDoubleBracket]1\[RightDoubleBracket], Para[n]\[LeftDoubleBracket]2\
\[RightDoubleBracket]\[LeftDoubleBracket]1\[RightDoubleBracket], n, \
MagnifyingFactor -> 1000]"], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.08426 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0468386 1.1982 0 1.1982 [
[-0.00989 -0.00751 -17.2578 -9 ]
[-0.00989 -0.00751 .7422 0 ]
[-0.00757 -0.00837 -23.0039 -9 ]
[-0.00757 -0.00837 .99615 0 ]
[-0.00524 -0.00923 -5.74933 -9 ]
[-0.00524 -0.00923 .25067 0 ]
[-0.00291 -0.0101 -22.9907 -9 ]
[-0.00291 -0.0101 1.00925 0 ]
[-0.00059 -0.01096 -17.2382 -9 ]
[-0.00059 -0.01096 .76185 0 ]
[-0.05584 -0.06444 -15.3315 -15 ]
[-0.05584 -0.06444 .66847 0 ]
[.05275 .02645 0 -6.15426 ]
[.05275 .02645 18 2.84574 ]
[.32552 .32465 0 -6.04011 ]
[.32552 .32465 18 2.95989 ]
[.57105 .59308 0 -5.94069 ]
[.57105 .59308 18 3.05931 ]
[.79322 .83598 0 -5.85334 ]
[.79322 .83598 18 3.14666 ]
[.99522 1.05683 0 -5.77597 ]
[.99522 1.05683 18 3.22403 ]
[.64237 .57025 0 -9.90116 ]
[.64237 .57025 22 5.09884 ]
[-0.01405 .00882 -42 -2.83339 ]
[-0.01405 .00882 0 6.16661 ]
[-0.01405 .00882 -48 -2.83339 ]
[-0.01405 .00882 0 6.16661 ]
[-0.01405 .00882 -42 -2.83339 ]
[-0.01405 .00882 0 6.16661 ]
[-0.08427 .03482 -34 -4.72232 ]
[-0.08427 .03482 0 10.2777 ]
[ 0 0 0 0 ]
[ 1 1.08426 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
0 .00361 m
.00976 0 L
s
.00023 .00353 m
.0053 .00905 L
s
[(1.5)] -0.00989 -0.00751 .91753 1 Mshowa
.00255 .00267 m
.00762 .00819 L
s
[(1.75)] -0.00757 -0.00837 .91699 1 Mshowa
.00488 .00181 m
.00994 .00733 L
s
[(2)] -0.00524 -0.00923 .91644 1 Mshowa
.0072 .00095 m
.01226 .00647 L
s
[(2.25)] -0.00291 -0.0101 .9159 1 Mshowa
.00953 9e-005 m
.01458 .00561 L
s
[(2.5)] -0.00059 -0.01096 .91535 1 Mshowa
.125 Mabswid
.0007 .00336 m
.00373 .00667 L
s
.00116 .00318 m
.0042 .0065 L
s
.00163 .00301 m
.00466 .00632 L
s
.00209 .00284 m
.00513 .00615 L
s
.00302 .0025 m
.00606 .00581 L
s
.00348 .00232 m
.00652 .00564 L
s
.00395 .00215 m
.00698 .00546 L
s
.00441 .00198 m
.00745 .00529 L
s
.00534 .00164 m
.00838 .00495 L
s
.00581 .00146 m
.00884 .00478 L
s
.00627 .00129 m
.00931 .0046 L
s
.00674 .00112 m
.00977 .00443 L
s
.00767 .00077 m
.0107 .00409 L
s
.00813 .0006 m
.01117 .00392 L
s
.0086 .00043 m
.01163 .00374 L
s
.00906 .00026 m
.01209 .00357 L
s
gsave
-0.05584 -0.06444 -76.3315 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(TB) show
75.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentNeededFonts: Courier
%%DocumentFonts: font Courier
grestore
.25 Mabswid
.00976 0 m
1 1.08188 L
s
.0387 .03162 m
.03167 .0342 L
s
[(600)] .05275 .02645 -1 .36761 Mshowa
.31135 .3295 m
.30427 .33193 L
s
[(650)] .32552 .32465 -1 .34225 Mshowa
.55678 .59765 m
.54965 .59993 L
s
[(700)] .57105 .59308 -1 .32015 Mshowa
.77888 .8403 m
.77171 .84245 L
s
[(750)] .79322 .83598 -1 .30074 Mshowa
.98081 1.06092 m
.97361 1.06296 L
s
[(800)] .99522 1.05683 -1 .28355 Mshowa
.125 Mabswid
.09562 .09381 m
.0914 .09534 L
s
.15131 .15465 m
.14708 .15616 L
s
.2058 .21419 m
.20156 .21568 L
s
.25914 .27246 m
.25489 .27393 L
s
.36248 .38536 m
.35822 .3868 L
s
.41256 .44007 m
.40829 .44149 L
s
.46161 .49367 m
.45734 .49507 L
s
.50968 .54618 m
.5054 .54757 L
s
.60296 .6481 m
.59868 .64945 L
s
.64823 .69756 m
.64394 .6989 L
s
.69262 .74606 m
.68833 .74738 L
s
.73617 .79363 m
.73187 .79494 L
s
.82079 .88608 m
.81648 .88736 L
s
.86191 .93102 m
.8576 .93228 L
s
.90228 .97512 m
.89796 .97637 L
s
.9419 1.01841 m
.93758 1.01965 L
s
gsave
.64237 .57025 -61 -13.9012 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(MA0) show
81.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentNeededFonts: Courier
%%DocumentFonts: font Courier
grestore
.25 Mabswid
0 .00361 m
0 .00361 L
s
0 .00361 m
.00702 .00101 L
s
[(0.00001)] -0.01405 .00882 1 -0.37036 Mshowa
0 .00361 m
.00702 .00101 L
s
[(0.000015)] -0.01405 .00882 1 -0.37036 Mshowa
0 .00361 m
.00702 .00101 L
s
[(0.00002)] -0.01405 .00882 1 -0.37036 Mshowa
.125 Mabswid
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
0 .00361 m
.00421 .00205 L
s
gsave
-0.08427 .03482 -95 -8.72232 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(sigma) show
93.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentNeededFonts: Courier
%%DocumentFonts: font Courier
grestore
.25 Mabswid
0 .00361 m
0 .00361 L
s
0 .00361 m
.99158 1.08426 L
s
.99158 1.08426 m
.99158 1.08426 L
s
.99158 1.08426 m
0 .00361 L
s
.00976 0 m
1 1.08188 L
s
1 1.08188 m
1 1.08188 L
s
1 1.08188 m
.00976 0 L
s
.00976 0 m
.00976 0 L
s
0 .00361 m
0 .00361 L
s
0 .00361 m
.00976 0 L
s
.00976 0 m
.00976 0 L
s
.00976 0 m
0 .00361 L
s
.99158 1.08426 m
1 1.08188 L
s
1 1.08188 m
1 1.08188 L
s
1 1.08188 m
.99158 1.08426 L
s
.99158 1.08426 m
.99158 1.08426 L
s
0 0 m
1 0 L
1 1.08426 L
0 1.08426 L
closepath
clip
newpath
.5 Mabswid
.97659 1.06211 m .97257 1.06325 L .54795 .60048 L p .656 .751 .92 r
F P
0 g
s
.98061 1.06097 m .97659 1.06211 L .55226 .5991 L p .656 .751 .92 r
F P
0 g
s
.54795 .60048 m .55226 .5991 L .97659 1.06211 L p .656 .751 .92 r
F P
0 g
s
.55226 .5991 m .55657 .59772 L .98061 1.06097 L p .656 .751 .92 r
F P
0 g
s
.55226 .5991 m .54795 .60048 L .02921 .03511 L p .656 .751 .92 r
F P
0 g
s
.55657 .59772 m .55226 .5991 L .03383 .0334 L p .656 .751 .92 r
F P
0 g
s
.02921 .03511 m .03383 .0334 L .55226 .5991 L p .656 .751 .92 r
F P
0 g
s
.03383 .0334 m .03846 .0317 L .55657 .59772 L p .656 .751 .92 r
F P
0 g
s
.25 Mabswid
.00976 0 m
1 1.08188 L
s
1 1.08188 m
1 1.08188 L
s
1 1.08188 m
.00976 0 L
s
.00976 0 m
.00976 0 L
s
0 .00361 m
0 .00361 L
s
0 .00361 m
.00976 0 L
s
.00976 0 m
.00976 0 L
s
.00976 0 m
0 .00361 L
s
0 .00361 m
.00976 0 L
s
.00023 .00353 m
.0053 .00905 L
s
[(1.5)] -0.00989 -0.00751 .91753 1 Mshowa
.00255 .00267 m
.00762 .00819 L
s
[(1.75)] -0.00757 -0.00837 .91699 1 Mshowa
.00488 .00181 m
.00994 .00733 L
s
[(2)] -0.00524 -0.00923 .91644 1 Mshowa
.0072 .00095 m
.01226 .00647 L
s
[(2.25)] -0.00291 -0.0101 .9159 1 Mshowa
.00953 9e-005 m
.01458 .00561 L
s
[(2.5)] -0.00059 -0.01096 .91535 1 Mshowa
.125 Mabswid
.0007 .00336 m
.00373 .00667 L
s
.00116 .00318 m
.0042 .0065 L
s
.00163 .00301 m
.00466 .00632 L
s
.00209 .00284 m
.00513 .00615 L
s
.00302 .0025 m
.00606 .00581 L
s
.00348 .00232 m
.00652 .00564 L
s
.00395 .00215 m
.00698 .00546 L
s
.00441 .00198 m
.00745 .00529 L
s
.00534 .00164 m
.00838 .00495 L
s
.00581 .00146 m
.00884 .00478 L
s
.00627 .00129 m
.00931 .0046 L
s
.00674 .00112 m
.00977 .00443 L
s
.00767 .00077 m
.0107 .00409 L
s
.00813 .0006 m
.01117 .00392 L
s
.0086 .00043 m
.01163 .00374 L
s
.00906 .00026 m
.01209 .00357 L
s
gsave
-0.05584 -0.06444 -76.3315 -19 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(TB) show
75.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentNeededFonts: Courier
%%DocumentFonts: font Courier
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{265.563, 288},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`400049000182000`400?l00000o`00003oo`3oool:
0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:
0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:
0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:
0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:
0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:
0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`006`3oool3000000030?ooo`00
0000000000800000iP3oool001`0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000003o
ool0oooo0><0oooo000L0?ooo`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?oo
o`3S0?ooo`00703oool010000000oooo0?ooo`3oool300000>H0oooo000J0?ooo`070000003oool0
00000?ooo`000000oooo000000020?ooo`030000003oool0oooo0><0oooo000J0?ooo`T00000iP3o
ool00?l0oooo2P3oool00?l0oooo2P3oool00?l0oooo2P3oool00?l0oooo2P3oool00?l0oooo2P3o
ool00?l0oooo2P3oool00200oooo0`0000020?ooo`<000000P3oool700000=P0oooo000N0?ooo`T0
00000P3oool200000080oooo0`0000000`3oool000000?ooo`3F0?ooo`00803oool200000080oooo
00<000000?ooo`0000000`3oool2000000030?ooo`000000000000<00000e`3oool00200oooo0P00
00020?ooo`030000003oool0000000<0oooo0P0000020?ooo`<00000f03oool00200oooo00@00000
0?ooo`000000oooo0P000000103oool000000?ooo`3oool600000=T0oooo000P0?ooo`8000000P3o
ool00`000000oooo000000020?ooo`040000003oool000000?ooo`D00000e`3oool001l0oooo0P00
00020?ooo`800000103oool3000000030?ooo`000000000000800000f03oool00?l0oooo2P3oool0
0?l0oooo2P3oool00?l0oooo2P3oool003<0oooo0`00003C0?ooo`00<P3oool4000000P0oooo0P00
00040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0_@3oool000X0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X000000`3oool2
000000030?ooo`000000oooo00H0oooo00<000000?ooo`0000000P3oool00`000000oooo00000002
0?ooo`030000003oool000000;h0oooo00090?ooo`030000003oool000000080oooo00<000000?oo
o`0000000P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo00<000000?oo
o`0000000P3oool00`000000oooo000000020?ooo`<000000P3oool3000000D0oooo00@000000?oo
o`3oool000001P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo00<00000
0?ooo`000000_P3oool000T0oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?oo
o`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?oo
o`030000003oool000000080oooo0`0000030?ooo`8000001P3oool010000000oooo0?ooo`000005
0?ooo`8000000`3oool00`000000oooo000000020?ooo`030000003oool000000;h0oooo00090?oo
o`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?oo
o`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?oo
o`<000000P3oool2000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`02
0?ooo`030000003oool000000080oooo00<000000?ooo`000000_P3oool000T0oooo00<000000?oo
o`0000000P3oool300000080oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?oo
o`030000003oool000000080oooo00<000000?ooo`0000000P3oool300000080oooo0`0000080?oo
o`040000003oool00000000000@0oooo0P0000030?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0_@3oool000X0oooo00<000000?ooo`3oool00P3oool2000000<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?oo
o`3oool0oooo0P0000030?ooo`<000002@3oool400000<d0oooo00030?ooo`8000000`3oool30000
00<0oooo100000001@3oool000000?ooo`000000oooo00@000007@3oool00`000000Yl3[0:O0j`02
00000<`0oooo00050?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`02
000000050?ooo`000000oooo0000003oool00`00000O0?ooo`030000002W`>/0Yl3[00800000b`3o
ool000@0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080oooo0P00
0000103oool000000?ooo`0000030?ooo`030000003oool0oooo01h0oooo00<000000:O0j`2W`>/0
0P00003:0?ooo`00103oool200000080oooo0P0000040?ooo`D0000000<0oooo0000003oool00P3o
ool200000240oooo00<000000:O0j`2W`>/00P0000390?ooo`00?03oool010000000Yl3[0:O0j`00
00390?ooo`002@3oool00`000000oooo0?ooo`0a0?ooo`040000002W`>/0Yl3[00000<P0oooo000n
0?ooo`80000000<0Yl3[0000003oool0aP3oool003l0oooo0P0000000`2W`>/000000?ooo`350?oo
o`00@03oool2000000030:O0j`000000oooo0<@0oooo00110?ooo`80000000<0Yl3[0000003oool0
``3oool00480oooo100000330?ooo`00@`3oool400000<80oooo00130?ooo`D00000`@3oool004@0
oooo00<000000:O0j`2W`>/00P0000300?ooo`00A@3oool00`000000Yl3[0:O0j`0200000;l0oooo
00160?ooo`040000002W`>/0Yl3[00000;l0oooo00170?ooo`040000002W`>/0Yl3[00000;h0oooo
00180?ooo`040000002W`>/0Yl3[00000;d0oooo00190?ooo`80000000<0Yl3[0000003oool0^`3o
ool004X0oooo0P0000000`2W`>/000000?ooo`2j0?ooo`00B`3oool2000000030:O0j`000000oooo
0;T0oooo001<0?ooo`80000000<0Yl3[0000003oool0^03oool004d0oooo1000002h0?ooo`00CP3o
ool400000;L0oooo001?0?ooo`@00000]P3oool004l0oooo1@00002e0?ooo`00D03oool400000;D0
oooo001A0?ooo`040000002W`>/0Yl3[00000;@0oooo001B0?ooo`040000002W`>/0Yl3[00000;<0
oooo001C0?ooo`040000002W`>/0Yl3[00000;80oooo001D0?ooo`80000000<0Yl3[0000003oool0
/03oool005D0oooo0P0000000`2W`>/000000?ooo`2_0?ooo`00EP3oool2000000030:O0j`000000
oooo0:h0oooo001G0?ooo`80000000<0Yl3[0000003oool0[@3oool005P0oooo1000002]0?ooo`00
F@3oool400000:`0oooo001J0?ooo`@00000Z`3oool005X0oooo1@00002Z0?ooo`00F`3oool40000
0:X0oooo001L0?ooo`@00000Z@3oool005d0oooo1000002X0?ooo`00GP3oool010000000Yl3[0:O0
j`00002W0?ooo`00G`3oool010000000Yl3[0:O0j`00002V0?ooo`00H03oool2000000030:O0j`00
0000oooo0:@0oooo001Q0?ooo`80000000<0Yl3[0000003oool0X`3oool00680oooo0P0000000`2W
`>/000000?ooo`2R0?ooo`00H`3oool400000:80oooo001T0?ooo`@00000X@3oool006D0oooo1000
002P0?ooo`00IP3oool300000:00oooo001V0?ooo`@00000W`3oool006L0oooo1000002N0?ooo`00
J03oool4000000L0oooo0P0000040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
R03oool006T0oooo100000060?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3o
ool00`000000oooo000000290?ooo`00JP3oool4000000D0oooo00<000000?ooo`000000103oool0
1P000000oooo0?ooo`000000oooo000008T0oooo001[0?ooo`80000000<0Yl3[0000003oool00`3o
ool2000000<0oooo0P0000030?ooo`030000003oool0000008T0oooo001/0?ooo`80000000<0Yl3[
0000003oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`000000R@3oool006d0oooo0P0000000`2W`>/000000?ooo`020?ooo`8000000P3oool30000
00<0oooo00<000000?ooo`3oool0R03oool006h0oooo1000002G0?ooo`00K`3oool4000009H0oooo
001`0?ooo`<00000UP3oool00740oooo0`00002E0?ooo`00L@3oool4000009@0oooo001b0?ooo`@0
0000T`3oool007<0oooo1000002B0?ooo`00M03oool400000940oooo001e0?ooo`@00000T03oool0
07H0oooo0P0000000`2W`>/000000?ooo`2>0?ooo`00M`3oool2000000030:O0j`000000oooo08d0
oooo001h0?ooo`@00000S@3oool007T0oooo1000002<0?ooo`00NP3oool3000008`0oooo001k0?oo
o`<00000R`3oool007`0oooo0`00002:0?ooo`00O@3oool3000008T0oooo001m0?ooo`@00000R03o
ool007h0oooo100000270?ooo`00O`3oool4000008H0oooo00200?ooo`@00000Q@3oool00840oooo
100000240?ooo`00PP3oool2000000030:O0j`000000oooo0880oooo00230?ooo`@00000PP3oool0
08@0oooo0`0000220?ooo`00Q@3oool300000840oooo00260?ooo`<00000P03oool008L0oooo0`00
001o0?ooo`00R03oool3000007h0oooo00280?ooo`@00000O@3oool008T0oooo1000001l0?ooo`00
RP3oool4000007/0oooo002;0?ooo`@00000NP3oool008`0oooo1000001i0?ooo`00S@3oool20000
00030:O0j`000000oooo07L0oooo002>0?ooo`@00000M`3oool008l0oooo0`00001g0?ooo`00T03o
ool3000007H0oooo002A0?ooo`<00000M@3oool00980oooo0`00001d0?ooo`00T`3oool3000007<0
oooo002C0?ooo`@00000LP3oool009@0oooo1000000K0?ooo`80000000<0oooo0000000000000P00
00000`3oool00000000000020?ooo`030000003oool0oooo04L0oooo002E0?ooo`@000006P3oool0
10000000oooo0?ooo`3oool2000000<0oooo00D000000?ooo`000000oooo000000180?ooo`00UP3o
ool4000001T0oooo00H000000?ooo`000000oooo0000003oool300000080oooo00<000000?ooo`00
0000B03oool009L0oooo1000000H0?ooo`80000000L0oooo000000000000oooo0000003oool00000
0080oooo00<000000?ooo`000000B03oool009P0oooo0P0000000`2W`>/000000?ooo`0F0?ooo`80
000000<0oooo0000000000000P3oool01`000000oooo0?ooo`3oool000000?ooo`000000B03oool0
09T0oooo0`0000090?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool01000
0000oooo0?ooo`3oool2000000040?ooo`00000000000?ooo`800000103oool00`000000oooo0?oo
o`170?ooo`00VP3oool3000000P0oooo00L000000?ooo`3oool0oooo0000003oool000000080oooo
00<000000?ooo`000000F03oool009/0oooo0`0000070?ooo`070000003oool0oooo0?ooo`000000
oooo000000020?ooo`030000003oool0000005P0oooo002L0?ooo`<000001`3oool01P000000oooo
0?ooo`000000oooo00000080oooo00<000000?ooo`000000F03oool009d0oooo0`0000040?ooo`03
0000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0000001H0?ooo`00
WP3oool3000000<0oooo0`0000030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
E`3oool009l0oooo0P2W`>/00`000000oooo0?ooo`1U0?ooo`00W`3oool010000000Yl3[0:O0j`00
001V0?ooo`00X03oool010000000Yl3[0:O0j`00001U0?ooo`00X@3oool010000000Yl3[0:O0j`00
001T0?ooo`00XP3oool010000000Yl3[0:O0j`00001S0?ooo`00X`3oool400000680oooo002T0?oo
o`<00000HP3oool00:D0oooo0`00001Q0?ooo`00YP3oool300000600oooo002W0?ooo`<00000G`3o
ool00:P0oooo0`00001N0?ooo`00Z@3oool3000005d0oooo002Z0?ooo`80Yl3[00<000000?ooo`3o
ool0FP3oool00:X0oooo00@000000:O0j`2W`>/00000F`3oool00:/0oooo00@000000:O0j`2W`>/0
0000FP3oool00:`0oooo00@000000:O0j`2W`>/00000F@3oool00:d0oooo00@000000:O0j`2W`>/0
0000F03oool00:h0oooo1000001G0?ooo`00[`3oool4000005H0oooo002`0?ooo`<00000EP3oool0
0;40oooo0`00001E0?ooo`00/P3oool3000005@0oooo002c0?ooo`<00000D`3oool00;@0oooo0`00
001B0?ooo`00]@3oool300000540oooo002f0?ooo`80Yl3[00<000000?ooo`3oool0CP3oool00;H0
oooo00@000000:O0j`2W`>/00000C`3oool00;L0oooo00@000000:O0j`2W`>/00000CP3oool00;P0
oooo1000001=0?ooo`00^@3oool3000004d0oooo002j0?ooo`<00000C03oool00;/0oooo0`00001;
0?ooo`00_03oool3000004X0oooo002m0?ooo`<00000B@3oool00;h0oooo0`0000180?ooo`00_`3o
ool3000004L0oooo00300?ooo`<00000AP3oool00<40oooo0`0000150?ooo`00`@3oool010000000
Yl3[0:O0j`0000140?ooo`00`P3oool4000004<0oooo00330?ooo`@00000@P3oool00<@0oooo0`00
00120?ooo`00a@3oool3000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo02`0oooo00360?ooo`<000001`3oool01`000000oooo0?ooo`3oool0
00000?ooo`0000000P3oool00`000000oooo0000000]0?ooo`00a`3oool3000000H0oooo00<00000
0?ooo`3oool00`3oool01P000000oooo0?ooo`000000oooo000002d0oooo00380?ooo`<000001P3o
ool00`000000oooo0?ooo`02000000<0oooo00<000000?ooo`000000;@3oool00<T0oooo10000002
0?ooo`030000003oool000000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0000000]
0?ooo`00bP3oool300000080oooo0`0000020?ooo`<000000`3oool00`000000oooo0?ooo`0/0?oo
o`00b`3oool3000003/0oooo003<0?ooo`<00000>P3oool00<d0oooo0`00000i0?ooo`00c@3oool4
000003P0oooo003>0?ooo`@00000=`3oool00<l0oooo0`00000g0?ooo`00d03oool3000003H0oooo
003A0?ooo`<00000=@3oool00=80oooo0`00000d0?ooo`00d`3oool3000003<0oooo003D0?ooo`<0
0000<P3oool00=D0oooo0`00000a0?ooo`00eP3oool300000300oooo003G0?ooo`<00000;`3oool0
0=P0oooo0`00000^0?ooo`00f03oool4000002d0oooo003I0?ooo`@00000;03oool00=X0oooo0`00
000/0?ooo`00f`3oool3000002/0oooo003L0?ooo`<00000:P3oool00=d0oooo0`00000Y0?ooo`00
gP3oool3000002P0oooo003O0?ooo`<000009`3oool00>00oooo0`00000V0?ooo`00h@3oool30000
02D0oooo003R0?ooo`<00000903oool00><0oooo0`00000S0?ooo`00i03oool300000280oooo003T
0?ooo`@000008@3oool00>D0oooo0`00000Q0?ooo`00iP3oool300000200oooo003W0?ooo`<00000
7`3oool00>P0oooo0`00000N0?ooo`00j@3oool3000001d0oooo003Z0?ooo`<00000703oool00>/0
oooo0`00000K0?ooo`00k03oool3000001X0oooo003]0?ooo`<00000203oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool00>h0oooo0`000006
0?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo00000005
0?ooo`00k`3oool3000000D0oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?oo
o`030000003oool0000000D0oooo003_0?ooo`@000001@3oool01`000000oooo0?ooo`3oool00000
0?ooo`0000000P3oool00`000000oooo000000050?ooo`00l03oool3000000@0oooo00<000000?oo
o`0000000P3oool00`000000oooo000000020?ooo`030000003oool0000000D0oooo003a0?ooo`<0
0000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0103oool00?80oooo00<000000?ooo`000000503oool00?<0oooo00<000000?ooo`0000004`3o
ool00?@0oooo00<000000?ooo`0000004P3oool00?D0oooo0`00000A0?ooo`00o`3oool:0?ooo`00
o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00
o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00
o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00
o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00o`3oool:0?ooo`00
o`3oool:0?ooo`00o`3oool:0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 264.563}, {287, 0}} -> {-0.176262, -0.215, 
  0.00424214, 0.00424214}}],

Cell[BoxData[
    FormBox[
      TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
        False,
        Editable->False], TraditionalForm]], "Output"],

Cell[CellGroupData[{

Cell["$MissingPoints", "Input"],

Cell[BoxData[
    \(TraditionalForm\`{}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[
"ScanDensityPlot[Para[n]\[LeftDoubleBracket]1\[RightDoubleBracket]\
\[LeftDoubleBracket]1\[RightDoubleBracket], Para[n]\[LeftDoubleBracket]2\
\[RightDoubleBracket]\[LeftDoubleBracket]1\[RightDoubleBracket], n, \
RenderMissing -> RGBColor[0, 0, 1]]"], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.40476 0.952381 -1.7511 0.00294302 [
[.11905 -0.0125 -9 -9 ]
[.11905 -0.0125 9 0 ]
[.30952 -0.0125 -9 -9 ]
[.30952 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.69048 -0.0125 -9 -9 ]
[.69048 -0.0125 9 0 ]
[.88095 -0.0125 -9 -9 ]
[.88095 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -8 -24 ]
[.5 -0.0125 8 -9 ]
[-0.0125 .01472 -18 -4.5 ]
[-0.0125 .01472 0 4.5 ]
[-0.0125 .16187 -18 -4.5 ]
[-0.0125 .16187 0 4.5 ]
[-0.0125 .30902 -18 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .45617 -18 -4.5 ]
[-0.0125 .45617 0 4.5 ]
[-0.0125 .60332 -18 -4.5 ]
[-0.0125 .60332 0 4.5 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -33 -11 ]
[-0.01875 .30902 -18 11 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.11905 0 m
.11905 .00625 L
s
[(1.6)] .11905 -0.0125 0 1 Mshowa
.30952 0 m
.30952 .00625 L
s
[(1.8)] .30952 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(2)] .5 -0.0125 0 1 Mshowa
.69048 0 m
.69048 .00625 L
s
[(2.2)] .69048 -0.0125 0 1 Mshowa
.88095 0 m
.88095 .00625 L
s
[(2.4)] .88095 -0.0125 0 1 Mshowa
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -69 -28 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(TB) show
75.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentNeededFonts: Courier
%%DocumentFonts: font Courier
grestore
0 .01472 m
.00625 .01472 L
s
[(600)] -0.0125 .01472 1 0 Mshowa
0 .16187 m
.00625 .16187 L
s
[(650)] -0.0125 .16187 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(700)] -0.0125 .30902 1 0 Mshowa
0 .45617 m
.00625 .45617 L
s
[(750)] -0.0125 .45617 1 0 Mshowa
0 .60332 m
.00625 .60332 L
s
[(800)] -0.0125 .60332 1 0 Mshowa
.125 Mabswid
0 .04415 m
.00375 .04415 L
s
0 .07358 m
.00375 .07358 L
s
0 .10301 m
.00375 .10301 L
s
0 .13244 m
.00375 .13244 L
s
0 .1913 m
.00375 .1913 L
s
0 .22073 m
.00375 .22073 L
s
0 .25016 m
.00375 .25016 L
s
0 .27959 m
.00375 .27959 L
s
0 .33845 m
.00375 .33845 L
s
0 .36788 m
.00375 .36788 L
s
0 .39731 m
.00375 .39731 L
s
0 .42674 m
.00375 .42674 L
s
0 .4856 m
.00375 .4856 L
s
0 .51503 m
.00375 .51503 L
s
0 .54446 m
.00375 .54446 L
s
0 .57389 m
.00375 .57389 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -97.5 -11.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
72 11.5 translate
90 rotate
-72 -11.5 translate
0 23 translate 1 -1 scale
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(MA0) show
81.000000 14.625000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentNeededFonts: Courier
%%DocumentFonts: font Courier
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .01472 m
1 .01472 L
s
.99375 .16187 m
1 .16187 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .45617 m
1 .45617 L
s
.99375 .60332 m
1 .60332 L
s
.125 Mabswid
.99625 .04415 m
1 .04415 L
s
.99625 .07358 m
1 .07358 L
s
.99625 .10301 m
1 .10301 L
s
.99625 .13244 m
1 .13244 L
s
.99625 .1913 m
1 .1913 L
s
.99625 .22073 m
1 .22073 L
s
.99625 .25016 m
1 .25016 L
s
.99625 .27959 m
1 .27959 L
s
.99625 .33845 m
1 .33845 L
s
.99625 .36788 m
1 .36788 L
s
.99625 .39731 m
1 .39731 L
s
.99625 .42674 m
1 .42674 L
s
.99625 .4856 m
1 .4856 L
s
.99625 .51503 m
1 .51503 L
s
.99625 .54446 m
1 .54446 L
s
.99625 .57389 m
1 .57389 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.673 g
.02381 .01472 m
.02381 .30902 L
.5 .30902 L
.5 .01472 L
F
.176 g
.02381 .30902 m
.02381 .60332 L
.5 .60332 L
.5 .30902 L
F
.669 g
.5 .01472 m
.5 .30902 L
.97619 .30902 L
.97619 .01472 L
F
.173 g
.5 .30902 m
.5 .60332 L
.97619 .60332 L
.97619 .30902 L
F
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00W@3oool3
000000030?ooo`000000000000800000N`3oool009h0oooo00D000000?ooo`3oool0oooo00000002
0?ooo`030000003oool0oooo07P0oooo002N0?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo0?ooo`1h0?ooo`00WP3oool010000000oooo0?ooo`3oool3000007/0oooo002L0?oo
o`070000003oool000000?ooo`000000oooo000000020?ooo`030000003oool0oooo07P0oooo002L
0?ooo`T00000N`3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3o
ool00?l0oooo8@3oool003h0oooo1@0000020?ooo`040000003oool0oooo0?ooo`8000008@3oool5
00000080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0U0?ooo`<000009P3oool3
000000<0oooo00@000000?ooo`3oool0oooo0`00000Q0?ooo`<000000`3oool00`000000oooo0?oo
o`020?ooo`<000006`3oool00400oooo00<000000?ooo`3oool01P3oool00`000000oooo0000000R
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`0000009P3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo01X0oooo00100?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`0000008P3oool00`000000oooo0?ooo`060?ooo`030000003oool0000002L0oooo00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool01P3oool4000001/0oooo00100?ooo`030000003oool0oooo00H0oooo0P00000S0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0V0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0000001`0oooo00100?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`060?ooo`030000003oool0000002H0oooo00<000000?ooo`000000
9P3oool00`000000oooo000000070?ooo`030000003oool000000240oooo00<000000?ooo`000000
203oool2000001`0oooo000o0?ooo`8000002@3oool200000240oooo0P0000090?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0J0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00:@3oooob
000000D0oooo000H0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool00`3oool001P0oooo00<000000?ooo`0000000P3oool00`000000
oooo000000020?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool0kP3oool00`000000
oooo0?ooo`030?ooo`00603oool00`000000oooo000000020?ooo`030000003oool000000080oooo
00<000000?ooo`000000103oool300000>d0oooo0P0000050?ooo`00603oool2000000<0oooo00<0
00000?ooo`0000000P3oool00`000000oooo000000040?ooo`030000003oool0oooo00<0ooooL`2/
[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool001P0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo000000020?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool00`3ooomc
0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`006@3oool2000000<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[
Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780
Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/
LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/
[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0[:b/LP2[Zj/5
0?ooo`8000001@3oool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`00
0000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`03
0000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo
00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3o
ool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/6
0?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[
Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0[:b/LP2[Zj/50?ooo`80
00001@3oool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo
0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003o
ool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<00000
0?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`00
0000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`03
0000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo
00<000000?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0[:b/LP2[Zj/50?ooo`8000001@3o
ool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`03
0?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo
00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3o
ool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo
0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003o
ool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<00000
0?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0[:b/LP2[Zj/50?ooo`8000001@3oool002T0
oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00
:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo
000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3o
ool002T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`03
0?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo
00<0oooo000H0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00
603oool00`000000oooo000000020?ooo`030000003oool000000080oooo00<000000?ooo`000000
103oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo
000H0?ooo`030000003oool0000000@0oooo00H000000?ooo`3oool000000?ooo`0000040?ooo`<0
00000`3ooomc0:b/[780Zj^[1@3oool2000000D0oooo000H0?ooo`8000000`3oool2000000<0oooo
00<000000?ooo`000000103oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000
003oool0oooo00<0oooo000H0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool00`000000oooo000000040?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<0
00000?ooo`3oool00`3oool001T0oooo0P0000020?ooo`<000000`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool0
02T0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?oo
o`00:@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0
oooo000Y0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool0
0`3oool002T0oooo0P0000040?ooog<0[:b/LP2[Zj/50?ooo`8000001@3oool002T0oooo00<00000
0?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`00
0000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`03
0000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo
00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3o
ool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000Y
0?ooo`030000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool0
02T0oooo0P0000040?ooog<0[:b/LP2[Zj/50?ooo`8000001@3oool002T0oooo00<000000?ooo`3o
ool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo
0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003o
ool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<00000
0?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`00
0000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`03
0000003oool0oooo00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo
0P0000040?ooog<0[:b/LP2[Zj/50?ooo`8000001@3oool002T0oooo00<000000?ooo`3oool00`3o
oomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`03
0?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo
00<0ooooL`2/[:ab0:^[Z`H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3o
ool00`3ooomc0:b/[780Zj^[1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo
0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo00070?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`030?ooog<0[:b/LP2[
Zj/60?ooo`030000003oool0oooo00<0oooo00070?ooo`L000006`3oool2000000@0ooooL`2/[:ab
0:^[Z`D0oooo0P0000050?ooo`001`3oool3000000<0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000:0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool00`3ooomc0:b/[780Zj^[1P3oool00`000000
oooo0?ooo`030?ooo`001`3oool3000000<0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo00070?ooo`L000006`3o
ool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo0007
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`03
0?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000=0?ooo`030000003oool0oooo
00T0oooo00L000000?ooo`3oool0oooo0000003oool000000080oooo00<000000?ooo`000000103o
ool00`000000oooo0?ooo`030?ooog<0[:b/LP2[Zj/60?ooo`030000003oool0oooo00<0oooo000<
0?ooo`8000002`3oool01`000000oooo0?ooo`3oool000000?ooo`0000000P3oool00`000000oooo
000000040?ooo`<000000`3ooomc0:b/[780Zj^[1@3oool2000000D0oooo00070?ooo`030000003o
ool000000080000000<0oooo0000003oool02`3oool01P000000oooo0?ooo`000000oooo00000080
oooo00<000000?ooo`000000103oool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`03
0000003oool0oooo00<0oooo00070?ooo`8000000P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo000000040?ooo`030000
003oool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool000T0oooo0`00
00000`3oool000000?ooo`090?ooo`<000000`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?oo
o`030?ooo`00303oool2000001/0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/1P3oool0
0`000000oooo0?ooo`030?ooo`003@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3o
ool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00203oool5000001`0oooo
0P0000040?ooog<0;Bd]LP0/;2`50?ooo`8000001@3oool000L0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<0
00000?ooo`3oool00`3oool000L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool0
00L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool000P0oooo1@00000L0?ooo`03
0000003oool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo
00<000000?ooo`3oool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3o
ool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y
0?ooo`800000103ooomc02d];G80;2`/1@3oool2000000D0oooo000Y0?ooo`030000003oool0oooo
00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3o
ool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo
0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003o
ool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<00000
0?ooo`3oool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`00
0000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`80
0000103ooomc02d];G80;2`/1@3oool2000000D0oooo000Y0?ooo`030000003oool0oooo00<0oooo
L`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3o
oomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`03
0?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo
00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3o
ool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo
0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`800000103o
oomc02d];G80;2`/1@3oool2000000D0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`0];Beb
02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d]
;G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0
;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0oooo
L`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3o
oomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`006@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`03
0?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000I0?ooo`070000003oool0oooo
0?ooo`000000oooo000000020?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool00`3o
oomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`006@3oool00`000000oooo0?ooo`03
0?ooo`060000003oool0oooo0000003oool00000103oool3000000<0ooooL`0];Beb02`/;0D0oooo
0P0000050?ooo`006P3oool00`000000oooo0?ooo`02000000<0oooo00<000000?ooo`000000103o
ool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000H
0?ooo`030000003oool000000080oooo00<000000?ooo`3oool00P3oool00`000000oooo00000004
0?ooo`030000003oool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool0
01P0oooo0`0000020?ooo`<000000`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3o
ool00`3ooomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo
0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003o
ool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo0P000004
0?ooog<0;Bd]LP0/;2`50?ooo`8000001@3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d]
;G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0
;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0oooo
L`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3o
oomc02d];G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`03
0?ooog<0;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo
00<0ooooL`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0
;Bd]LP0/;2`50?ooo`8000001@3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/
1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/
;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`0];Beb
02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d]
;G80;2`/1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0
;Bd]LP0/;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0oooo
L`0];Beb02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0;Bd]LP0/
;2`50?ooo`8000001@3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/1P3oool0
0`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?oo
o`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`0];Beb02`/;0H0
oooo00<000000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/
1P3oool00`000000oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/
;2`60?ooo`030000003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`0];Beb
02`/;0H0oooo00<000000?ooo`3oool00`3oool002T0oooo0P0000040?ooog<0;Bd]LP0/;2`50?oo
o`8000001@3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/1P3oool00`000000
oooo0?ooo`030?ooo`00:@3oool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?ooo`030000
003oool0oooo00<0oooo000Y0?ooo`030000003oool0oooo00<0ooooL`0];Beb02`/;0H0oooo00<0
00000?ooo`3oool00`3oool002T0oooo00<000000?ooo`3oool00`3ooomc02d];G80;2`/1P3oool0
0`000000oooo0?ooo`030?ooo`006@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooog<0;Bd]LP0/;2`60?oo
o`030000003oool0oooo00<0oooo000H0?ooo`030000003oool000000080oooo00<000000?ooo`00
00000P3oool00`000000oooo000000040?ooo`030000003oool0oooo00<0ooooL`0];Beb02`/;0H0
oooo00<000000?ooo`3oool00`3oool001P0oooo00<000000?ooo`0000000P3oool00`000000oooo
000000020?ooo`030000003oool0000000@0oooo0`0000030?ooog<0;Bd]LP0/;2`50?ooo`800000
1@3oool001T0oooo00L000000?ooo`3oool0oooo0000003oool000000080oooo00<000000?ooo`00
0000103oool00`000000oooo0?ooo`3^0?ooo`030000003oool0oooo00<0oooo000H0?ooo`030000
003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo000000040?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0[0?ooo`030000
003oool0oooo02/0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo00<0oooo000I0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3o
ool002T0oooolP0000050?ooo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {1.29344, 556.92, 0.00435616, 
  1.40968}}],

Cell[BoxData[
    FormBox[
      TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
        False,
        Editable->False], TraditionalForm]], "Output"],

Cell[CellGroupData[{

Cell["$MissingPoints", "Input"],

Cell[BoxData[
    \(TraditionalForm\`{}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Clear[DataRow]
DataRow[x_, tree_, loop_] := {x, (loop - tree)/tree}\
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Install["\<LoopTool\\LoopTools.exe\>"]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`LinkObject[".\\LoopTool\\LoopTools.exe", 5, 5]\)], 
  "Output"],

Cell[CellGroupData[{

Cell[TextData[
"B0[Para[n]\[LeftDoubleBracket]1\[RightDoubleBracket]\[LeftDoubleBracket]2\
\[RightDoubleBracket], Para[n]\[LeftDoubleBracket]2\[RightDoubleBracket]\
\[LeftDoubleBracket]2\[RightDoubleBracket],Para[n]\[LeftDoubleBracket]3\
\[RightDoubleBracket]\[LeftDoubleBracket]2\[RightDoubleBracket]]"], "Input"],

Cell[BoxData[
    \(TraditionalForm\`\(-5.9805906651744651`16.9031\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[
"InputForm[Cget[Para[n]\[LeftDoubleBracket]1\[RightDoubleBracket]\
\[LeftDoubleBracket]2\[RightDoubleBracket], Para[n]\[LeftDoubleBracket]2\
\[RightDoubleBracket]\[LeftDoubleBracket]2\[RightDoubleBracket],Para[n]\
\[LeftDoubleBracket]3\[RightDoubleBracket]\[LeftDoubleBracket]2\
\[RightDoubleBracket], \n\tPara[n]\[LeftDoubleBracket]2\[RightDoubleBracket]\
\[LeftDoubleBracket]2\[RightDoubleBracket], Para[n]\[LeftDoubleBracket]5\
\[RightDoubleBracket]\[LeftDoubleBracket]2\[RightDoubleBracket],Para[n]\
\[LeftDoubleBracket]6\[RightDoubleBracket]\[LeftDoubleBracket]2\
\[RightDoubleBracket]]]"], "Input"],

Cell["\<\
{cc0 -> -0.0005837057022050522, cc1 -> 0.000193162358717713, 
  cc2 -> 0.0001942032362018333, cc00 -> -1.688461181995094, 
  cc11 -> -0.000095344112229626, cc12 -> -0.00004928429090380491, 
  cc22 -> -0.00009602981393856189, cc001 -> 0.56344303156057, 
  cc002 -> 0.5629835551549397, cc111 -> 0.00005648105094822179, 
  cc112 -> 0.00001969604667739627, cc122 -> 0.00001974678366878063, 
  cc222 -> 0.00005691590350123856}\
\>", "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Run[\ "\<pnuglot.bat \>"<>\ FortranDir]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Print["\<time used: \>", \ SessionTime[]\  - \ time1]\)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[
        \("time used: "\[InvisibleSpace]871.692999999999962`\),
        SequenceForm[ "time used: ", 871.69299999999998],
        Editable->False], TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
CellGrouping->Manual,
WindowSize->{718, 613},
WindowMargins->{{114, Automatic}, {Automatic, -6}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 80, 2, 46, "Input"],
Cell[1814, 55, 52, 1, 25, "Print"],
Cell[1869, 58, 62, 1, 25, "Print"],
Cell[1934, 61, 100, 2, 25, "Print"],
Cell[2037, 65, 74, 1, 25, "Print"],

Cell[CellGroupData[{
Cell[2136, 70, 93, 2, 46, "Input"],
Cell[2232, 74, 52, 1, 25, "Print"],
Cell[2287, 77, 64, 1, 25, "Print"],
Cell[2354, 80, 66, 1, 25, "Print"],
Cell[2423, 83, 73, 1, 25, "Print"]
}, Open  ]],
Cell[2511, 87, 106, 2, 46, "Input"],

Cell[CellGroupData[{
Cell[2642, 93, 58, 1, 30, "Input"],
Cell[2703, 96, 71, 1, 29, "Output"],

Cell[CellGroupData[{
Cell[2799, 101, 53, 1, 30, "Input"],
Cell[2855, 104, 61, 1, 29, "Output"]
}, Open  ]],
Cell[2931, 108, 69, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[3025, 113, 120, 2, 30, "Input"],
Cell[3148, 117, 291, 5, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3476, 127, 444, 8, 170, "Input"],
Cell[3923, 137, 376, 7, 150, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4348, 150, 71, 1, 30, "Input"],
Cell[4422, 153, 52, 1, 29, "Output"],

Cell[CellGroupData[{
Cell[4499, 158, 114, 3, 30, "Input"],
Cell[4616, 163, 120, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4773, 171, 136, 2, 50, "Input"],
Cell[4912, 175, 290, 5, 48, "Output"]
}, Open  ]],
Cell[5217, 183, 85, 1, 30, "Input"],
Cell[5305, 186, 168, 4, 50, "Input"],

Cell[CellGroupData[{
Cell[5498, 194, 279, 6, 70, "Input"],
Cell[5780, 202, 52, 1, 25, "Print"],
Cell[5835, 205, 421, 10, 25, "Print"],
Cell[6259, 217, 70, 1, 25, "Print"],
Cell[6332, 220, 262, 6, 25, "Print"],
Cell[6597, 228, 52, 1, 25, "Print"],
Cell[6652, 231, 413, 10, 25, "Print"],
Cell[7068, 243, 354, 8, 25, "Print"],
Cell[7425, 253, 74, 1, 25, "Print"],
Cell[7502, 256, 216, 5, 25, "Print"],
Cell[7721, 263, 261, 6, 25, "Print"],
Cell[7985, 271, 257, 6, 25, "Print"],
Cell[8245, 279, 254, 6, 25, "Print"],
Cell[8502, 287, 52, 1, 25, "Print"],
Cell[8557, 290, 324, 7, 25, "Print"],
Cell[8884, 299, 52, 1, 25, "Print"],
Cell[8939, 302, 247, 5, 25, "Print"],
Cell[9189, 309, 308, 7, 25, "Print"],
Cell[9500, 318, 308, 7, 25, "Print"],
Cell[9811, 327, 308, 7, 25, "Print"],
Cell[10122, 336, 308, 7, 25, "Print"],
Cell[10433, 345, 308, 7, 25, "Print"],
Cell[10744, 354, 308, 7, 25, "Print"],
Cell[11055, 363, 308, 7, 25, "Print"],
Cell[11366, 372, 308, 7, 25, "Print"],
Cell[11677, 381, 308, 7, 25, "Print"],
Cell[11988, 390, 319, 8, 25, "Print"],
Cell[12310, 400, 253, 5, 25, "Print"],
Cell[12566, 407, 52, 1, 25, "Print"],
Cell[12621, 410, 253, 6, 25, "Print"],

Cell[CellGroupData[{
Cell[12899, 420, 68, 1, 30, "Input"],
Cell[12970, 423, 155, 4, 25, "Message"],
Cell[13128, 429, 264, 6, 25, "Print"],
Cell[13395, 437, 264, 6, 25, "Print"],
Cell[13662, 445, 264, 6, 25, "Print"],
Cell[13929, 453, 89781, 3309, 296, 68990, 3047, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103747, 3767, 90, 1, 30, "Input"],
Cell[103840, 3770, 52, 1, 25, "Print"],
Cell[103895, 3773, 289, 7, 25, "Print"],
Cell[104187, 3782, 308, 7, 25, "Print"],
Cell[104498, 3791, 308, 7, 25, "Print"],
Cell[104809, 3800, 308, 7, 25, "Print"],
Cell[105120, 3809, 253, 5, 25, "Print"],
Cell[105376, 3816, 58105, 1158, 1334, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[163530, 4980, 83, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[163638, 4985, 66, 1, 30, "Input"],
Cell[163707, 4988, 52, 1, 25, "Print"],
Cell[163762, 4991, 415, 10, 25, "Print"],
Cell[164180, 5003, 354, 8, 25, "Print"],
Cell[164537, 5013, 74, 1, 25, "Print"],
Cell[164614, 5016, 218, 5, 25, "Print"],
Cell[164835, 5023, 256, 6, 25, "Print"],
Cell[165094, 5031, 52, 1, 25, "Print"],
Cell[165149, 5034, 324, 7, 25, "Print"],
Cell[165476, 5043, 52, 1, 25, "Print"],
Cell[165531, 5046, 247, 5, 25, "Print"],
Cell[165781, 5053, 308, 7, 25, "Print"],
Cell[166092, 5062, 308, 7, 25, "Print"],
Cell[166403, 5071, 308, 7, 25, "Print"],
Cell[166714, 5080, 308, 7, 25, "Print"],
Cell[167025, 5089, 253, 5, 25, "Print"],
Cell[167281, 5096, 52, 1, 25, "Print"],
Cell[167336, 5099, 253, 6, 25, "Print"],
Cell[167592, 5107, 264, 6, 25, "Print"],
Cell[167859, 5115, 27696, 959, 296, 20042, 860, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[195592, 6079, 89, 1, 30, "Input"],
Cell[195684, 6082, 52, 1, 25, "Print"],
Cell[195739, 6085, 289, 7, 25, "Print"],
Cell[196031, 6094, 308, 7, 25, "Print"],
Cell[196342, 6103, 253, 5, 25, "Print"],
Cell[196598, 6110, 261, 6, 25, "Print"],
Cell[196862, 6118, 267, 6, 25, "Print"],
Cell[197132, 6126, 33, 0, 25, "Print"],
Cell[197168, 6128, 56, 1, 25, "Print"],
Cell[197227, 6131, 54, 1, 25, "Print"],
Cell[197284, 6134, 495, 9, 103, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[197816, 6148, 71, 1, 30, "Input"],
Cell[197890, 6151, 58108, 1158, 1334, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[256047, 7315, 198, 5, 50, "Input"],

Cell[CellGroupData[{
Cell[256270, 7324, 66, 1, 30, "Input"],
Cell[256339, 7327, 52, 1, 25, "Print"],
Cell[256394, 7330, 324, 7, 25, "Print"],
Cell[256721, 7339, 52, 1, 25, "Print"],
Cell[256776, 7342, 247, 5, 25, "Print"],
Cell[257026, 7349, 319, 8, 25, "Print"],
Cell[257348, 7359, 308, 7, 25, "Print"],
Cell[257659, 7368, 308, 7, 25, "Print"],
Cell[257970, 7377, 323, 8, 25, "Print"],
Cell[258296, 7387, 308, 7, 25, "Print"],
Cell[258607, 7396, 308, 7, 25, "Print"],
Cell[258918, 7405, 268, 6, 25, "Print"],
Cell[259189, 7413, 52, 1, 25, "Print"],
Cell[259244, 7416, 253, 6, 25, "Print"],
Cell[259500, 7424, 266, 6, 25, "Print"],
Cell[259769, 7432, 268, 6, 25, "Print"],
Cell[260040, 7440, 248702, 9037, 296, 192178, 8334, "GraphicsData", 
"PostScript", "Graphics"],
Cell[508745, 16479, 254473, 9254, 296, 197625, 8547, "GraphicsData", 
"PostScript", "Graphics"],
Cell[763221, 25735, 250995, 8688, 296, 192556, 7962, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1014219, 34425, 253775, 9142, 296, 198381, 8453, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1267997, 43569, 253028, 9010, 296, 196998, 8313, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1521028, 52581, 260021, 9187, 296, 202270, 8469, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1781052, 61770, 265030, 9652, 296, 208996, 8955, "GraphicsData", 
"PostScript", "Graphics"],
Cell[2046085, 71424, 86392, 3137, 296, 67485, 2899, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2132514, 74566, 150, 3, 70, "Input"],
Cell[2132667, 74571, 52, 1, 25, "Print"],
Cell[2132722, 74574, 289, 7, 25, "Print"],
Cell[2133014, 74583, 319, 8, 25, "Print"],
Cell[2133336, 74593, 323, 8, 25, "Print"],
Cell[2133662, 74603, 268, 6, 25, "Print"],
Cell[2133933, 74611, 263, 6, 25, "Print"],
Cell[2134199, 74619, 267, 6, 25, "Print"],
Cell[2134469, 74627, 33, 0, 25, "Print"],
Cell[2134505, 74629, 56, 1, 25, "Print"],
Cell[2134564, 74632, 54, 1, 25, "Print"],
Cell[2134621, 74635, 146328, 2373, 24949, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2280998, 77014, 201, 5, 50, "Input"],

Cell[CellGroupData[{
Cell[2281224, 77023, 196, 5, 50, "Input"],
Cell[2281423, 77030, 52, 1, 25, "Print"],
Cell[2281478, 77033, 324, 7, 25, "Print"],
Cell[2281805, 77042, 52, 1, 25, "Print"],
Cell[2281860, 77045, 247, 5, 25, "Print"],
Cell[2282110, 77052, 319, 8, 25, "Print"],
Cell[2282432, 77062, 308, 7, 25, "Print"],
Cell[2282743, 77071, 308, 7, 25, "Print"],
Cell[2283054, 77080, 308, 7, 25, "Print"],
Cell[2283365, 77089, 308, 7, 25, "Print"],
Cell[2283676, 77098, 319, 8, 25, "Print"],
Cell[2283998, 77108, 319, 8, 25, "Print"],
Cell[2284320, 77118, 319, 8, 25, "Print"],
Cell[2284642, 77128, 319, 8, 25, "Print"],
Cell[2284964, 77138, 319, 8, 25, "Print"],
Cell[2285286, 77148, 319, 8, 25, "Print"],
Cell[2285608, 77158, 319, 8, 25, "Print"],
Cell[2285930, 77168, 319, 8, 25, "Print"],
Cell[2286252, 77178, 319, 8, 25, "Print"],
Cell[2286574, 77188, 319, 8, 25, "Print"],
Cell[2286896, 77198, 319, 8, 25, "Print"],
Cell[2287218, 77208, 319, 8, 25, "Print"],
Cell[2287540, 77218, 319, 8, 25, "Print"],
Cell[2287862, 77228, 319, 8, 25, "Print"],
Cell[2288184, 77238, 319, 8, 25, "Print"],
Cell[2288506, 77248, 319, 8, 25, "Print"],
Cell[2288828, 77258, 319, 8, 25, "Print"],
Cell[2289150, 77268, 319, 8, 25, "Print"],
Cell[2289472, 77278, 319, 8, 25, "Print"],
Cell[2289794, 77288, 268, 6, 25, "Print"],
Cell[2290065, 77296, 52, 1, 25, "Print"],
Cell[2290120, 77299, 253, 6, 25, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2290410, 77310, 65, 1, 30, "Input"],
Cell[2290478, 77313, 266, 6, 25, "Print"],
Cell[2290747, 77321, 266, 6, 25, "Print"],
Cell[2291016, 77329, 266, 6, 25, "Print"],
Cell[2291285, 77337, 266, 6, 25, "Print"],
Cell[2291554, 77345, 266, 6, 25, "Print"],
Cell[2291823, 77353, 252766, 9049, 296, 198454, 8374, "GraphicsData", 
"PostScript", "Graphics"],
Cell[2544592, 86404, 265087, 9782, 296, 211026, 9110, "GraphicsData", 
"PostScript", "Graphics"],
Cell[2809682, 96188, 270259, 10157, 296, 216611, 9490, "GraphicsData", 
"PostScript", "Graphics"],
Cell[3079944, 106347, 258231, 9332, 296, 204530, 8664, "GraphicsData", 
"PostScript", "Graphics"],
Cell[3338178, 115681, 263818, 10127, 296, 212000, 9482, "GraphicsData", 
"PostScript", "Graphics"],
Cell[3601999, 125810, 204617, 7424, 296, 157742, 6840, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3806653, 133239, 150, 3, 70, "Input"],
Cell[3806806, 133244, 52, 1, 25, "Print"],
Cell[3806861, 133247, 289, 7, 25, "Print"],
Cell[3807153, 133256, 319, 8, 25, "Print"],
Cell[3807475, 133266, 319, 8, 25, "Print"],
Cell[3807797, 133276, 319, 8, 25, "Print"],
Cell[3808119, 133286, 319, 8, 25, "Print"],
Cell[3808441, 133296, 319, 8, 25, "Print"],
Cell[3808763, 133306, 268, 6, 25, "Print"],
Cell[3809034, 133314, 263, 6, 25, "Print"],
Cell[3809300, 133322, 267, 6, 25, "Print"],
Cell[3809570, 133330, 33, 0, 25, "Print"],
Cell[3809606, 133332, 56, 1, 25, "Print"],
Cell[3809665, 133335, 54, 1, 25, "Print"],
Cell[3809722, 133338, 633774, 9962, 88172, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4443545, 143306, 308, 7, 70, "Input"],

Cell[CellGroupData[{
Cell[4443878, 143317, 53, 1, 30, "Input"],
Cell[4443934, 143320, 52, 1, 25, "Print"],
Cell[4443989, 143323, 324, 7, 25, "Print"],
Cell[4444316, 143332, 52, 1, 25, "Print"],
Cell[4444371, 143335, 247, 5, 25, "Print"],
Cell[4444621, 143342, 308, 7, 25, "Print"],
Cell[4444932, 143351, 308, 7, 25, "Print"],
Cell[4445243, 143360, 308, 7, 25, "Print"],
Cell[4445554, 143369, 308, 7, 25, "Print"],
Cell[4445865, 143378, 308, 7, 25, "Print"],
Cell[4446176, 143387, 308, 7, 25, "Print"],
Cell[4446487, 143396, 319, 8, 25, "Print"],
Cell[4446809, 143406, 308, 7, 25, "Print"],
Cell[4447120, 143415, 308, 7, 25, "Print"],
Cell[4447431, 143424, 319, 8, 25, "Print"],
Cell[4447753, 143434, 319, 8, 25, "Print"],
Cell[4448075, 143444, 319, 8, 25, "Print"],
Cell[4448397, 143454, 257, 5, 25, "Print"],
Cell[4448657, 143461, 52, 1, 25, "Print"],
Cell[4448712, 143464, 253, 6, 25, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4449002, 143475, 64, 1, 30, "Input"],
Cell[4449069, 143478, 264, 6, 25, "Print"],
Cell[4449336, 143486, 266, 6, 25, "Print"],
Cell[4449605, 143494, 264, 6, 25, "Print"],
Cell[4449872, 143502, 264, 6, 25, "Print"],
Cell[4450139, 143510, 215678, 9466, 296, 215584, 9463, "GraphicsData", 
"PostScript", "Graphics",
  ImageCacheValid->False],
Cell[4665820, 152978, 60098, 2600, 296, 60004, 2597, "GraphicsData", 
"PostScript", "Graphics",
  ImageCacheValid->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[4725955, 155583, 149, 3, 70, "Input"],
Cell[4726107, 155588, 52, 1, 25, "Print"],
Cell[4726162, 155591, 289, 7, 25, "Print"],
Cell[4726454, 155600, 308, 7, 25, "Print"],
Cell[4726765, 155609, 319, 8, 25, "Print"],
Cell[4727087, 155619, 308, 7, 25, "Print"],
Cell[4727398, 155628, 308, 7, 25, "Print"],
Cell[4727709, 155637, 257, 5, 25, "Print"],
Cell[4727969, 155644, 261, 6, 25, "Print"],
Cell[4728233, 155652, 267, 6, 25, "Print"],
Cell[4728503, 155660, 33, 0, 25, "Print"],
Cell[4728539, 155662, 56, 1, 25, "Print"],
Cell[4728598, 155665, 54, 1, 25, "Print"],
Cell[4728655, 155668, 268482, 4257, 43364, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4997186, 159931, 86, 1, 50, "Input"],
Cell[4997275, 159934, 257, 6, 25, "Print"],
Cell[4997535, 159942, 33, 0, 25, "Print"],
Cell[4997571, 159944, 56, 1, 25, "Print"],
Cell[4997630, 159947, 54, 1, 25, "Print"],

Cell[CellGroupData[{
Cell[4997709, 159952, 155, 2, 50, "Input"],
Cell[4997867, 159956, 2922, 58, 447, "Output"]
}, Open  ]],
Cell[5000804, 160017, 364, 8, 130, "Input"],
Cell[5001171, 160027, 117, 2, 30, "Input"],

Cell[CellGroupData[{
Cell[5001313, 160033, 67, 1, 30, "Input"],
Cell[5001383, 160036, 52, 1, 25, "Print"],
Cell[5001438, 160039, 324, 7, 25, "Print"],
Cell[5001765, 160048, 52, 1, 25, "Print"],
Cell[5001820, 160051, 247, 5, 25, "Print"],
Cell[5002070, 160058, 319, 8, 25, "Print"],
Cell[5002392, 160068, 319, 8, 25, "Print"],
Cell[5002714, 160078, 255, 5, 25, "Print"],
Cell[5002972, 160085, 52, 1, 25, "Print"],
Cell[5003027, 160088, 253, 6, 25, "Print"],
Cell[5003283, 160096, 52, 1, 25, "Print"],
Cell[5003338, 160099, 289, 7, 25, "Print"],
Cell[5003630, 160108, 319, 8, 25, "Print"],
Cell[5003952, 160118, 319, 8, 25, "Print"],
Cell[5004274, 160128, 255, 5, 25, "Print"],
Cell[5004532, 160135, 261, 6, 25, "Print"],
Cell[5004796, 160143, 267, 6, 25, "Print"],
Cell[5005066, 160151, 126, 7, 110, "Print"],
Cell[5005195, 160160, 56, 1, 25, "Print"],
Cell[5005254, 160163, 54, 1, 25, "Print"],
Cell[5005311, 160166, 52, 1, 25, "Print"],
Cell[5005366, 160169, 324, 7, 25, "Print"],
Cell[5005693, 160178, 52, 1, 25, "Print"],
Cell[5005748, 160181, 247, 5, 25, "Print"],
Cell[5005998, 160188, 308, 7, 25, "Print"],
Cell[5006309, 160197, 319, 8, 25, "Print"],
Cell[5006631, 160207, 255, 5, 25, "Print"],
Cell[5006889, 160214, 52, 1, 25, "Print"],
Cell[5006944, 160217, 253, 6, 25, "Print"],
Cell[5007200, 160225, 52, 1, 25, "Print"],
Cell[5007255, 160228, 289, 7, 25, "Print"],
Cell[5007547, 160237, 308, 7, 25, "Print"],
Cell[5007858, 160246, 319, 8, 25, "Print"],
Cell[5008180, 160256, 255, 5, 25, "Print"],
Cell[5008438, 160263, 261, 6, 25, "Print"],
Cell[5008702, 160271, 267, 6, 25, "Print"],
Cell[5008972, 160279, 56, 1, 25, "Print"],
Cell[5009031, 160282, 54, 1, 25, "Print"],
Cell[5009088, 160285, 52, 1, 25, "Print"],
Cell[5009143, 160288, 324, 7, 25, "Print"],
Cell[5009470, 160297, 52, 1, 25, "Print"],
Cell[5009525, 160300, 247, 5, 25, "Print"],
Cell[5009775, 160307, 308, 7, 25, "Print"],
Cell[5010086, 160316, 319, 8, 25, "Print"],
Cell[5010408, 160326, 255, 5, 25, "Print"],
Cell[5010666, 160333, 52, 1, 25, "Print"],
Cell[5010721, 160336, 253, 6, 25, "Print"],
Cell[5010977, 160344, 52, 1, 25, "Print"],
Cell[5011032, 160347, 289, 7, 25, "Print"],
Cell[5011324, 160356, 308, 7, 25, "Print"],
Cell[5011635, 160365, 319, 8, 25, "Print"],
Cell[5011957, 160375, 255, 5, 25, "Print"],
Cell[5012215, 160382, 261, 6, 25, "Print"],
Cell[5012479, 160390, 267, 6, 25, "Print"],
Cell[5012749, 160398, 56, 1, 25, "Print"],
Cell[5012808, 160401, 54, 1, 25, "Print"],
Cell[5012865, 160404, 52, 1, 25, "Print"],
Cell[5012920, 160407, 324, 7, 25, "Print"],
Cell[5013247, 160416, 52, 1, 25, "Print"],
Cell[5013302, 160419, 247, 5, 25, "Print"],
Cell[5013552, 160426, 319, 8, 25, "Print"],
Cell[5013874, 160436, 319, 8, 25, "Print"],
Cell[5014196, 160446, 255, 5, 25, "Print"],
Cell[5014454, 160453, 52, 1, 25, "Print"],
Cell[5014509, 160456, 253, 6, 25, "Print"],
Cell[5014765, 160464, 52, 1, 25, "Print"],
Cell[5014820, 160467, 289, 7, 25, "Print"],
Cell[5015112, 160476, 319, 8, 25, "Print"],
Cell[5015434, 160486, 319, 8, 25, "Print"],
Cell[5015756, 160496, 255, 5, 25, "Print"],
Cell[5016014, 160503, 261, 6, 25, "Print"],
Cell[5016278, 160511, 267, 6, 25, "Print"],
Cell[5016548, 160519, 56, 1, 25, "Print"],
Cell[5016607, 160522, 54, 1, 25, "Print"],
Cell[5016664, 160525, 52, 1, 25, "Print"],
Cell[5016719, 160528, 324, 7, 25, "Print"],
Cell[5017046, 160537, 52, 1, 25, "Print"],
Cell[5017101, 160540, 247, 5, 25, "Print"],
Cell[5017351, 160547, 308, 7, 25, "Print"],
Cell[5017662, 160556, 308, 7, 25, "Print"],
Cell[5017973, 160565, 253, 5, 25, "Print"],
Cell[5018229, 160572, 52, 1, 25, "Print"],
Cell[5018284, 160575, 253, 6, 25, "Print"],
Cell[5018540, 160583, 52, 1, 25, "Print"],
Cell[5018595, 160586, 289, 7, 25, "Print"],
Cell[5018887, 160595, 308, 7, 25, "Print"],
Cell[5019198, 160604, 253, 5, 25, "Print"],
Cell[5019454, 160611, 261, 6, 25, "Print"],
Cell[5019718, 160619, 267, 6, 25, "Print"],
Cell[5019988, 160627, 56, 1, 25, "Print"],
Cell[5020047, 160630, 54, 1, 25, "Print"],
Cell[5020104, 160633, 52, 1, 25, "Print"],
Cell[5020159, 160636, 324, 7, 25, "Print"],
Cell[5020486, 160645, 52, 1, 25, "Print"],
Cell[5020541, 160648, 247, 5, 25, "Print"],
Cell[5020791, 160655, 319, 8, 25, "Print"],
Cell[5021113, 160665, 319, 8, 25, "Print"],
Cell[5021435, 160675, 255, 5, 25, "Print"],
Cell[5021693, 160682, 52, 1, 25, "Print"],
Cell[5021748, 160685, 253, 6, 25, "Print"],
Cell[5022004, 160693, 52, 1, 25, "Print"],
Cell[5022059, 160696, 289, 7, 25, "Print"],
Cell[5022351, 160705, 319, 8, 25, "Print"],
Cell[5022673, 160715, 319, 8, 25, "Print"],
Cell[5022995, 160725, 255, 5, 25, "Print"],
Cell[5023253, 160732, 261, 6, 25, "Print"],
Cell[5023517, 160740, 267, 6, 25, "Print"],
Cell[5023787, 160748, 56, 1, 25, "Print"],
Cell[5023846, 160751, 54, 1, 25, "Print"],
Cell[5023903, 160754, 79, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5024043, 160762, 104, 2, 30, "Input"],
Cell[5024150, 160766, 52, 1, 29, "Output"],

Cell[CellGroupData[{
Cell[5024227, 160771, 94, 2, 30, "Input"],
Cell[5024324, 160775, 5252, 86, 1017, "Output"],

Cell[CellGroupData[{
Cell[5029601, 160865, 105, 3, 30, "Input"],
Cell[5029709, 160870, 26496, 394, 5083, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5056254, 161270, 70, 1, 30, "Input"],
Cell[5056327, 161273, 99, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5056463, 161280, 77, 1, 30, "Input"],
Cell[5056543, 161283, 52, 1, 29, "Output"],

Cell[CellGroupData[{
Cell[5056620, 161288, 37, 0, 30, "Input"],
Cell[5056660, 161290, 3293, 67, 181, "Output"],

Cell[CellGroupData[{
Cell[5059978, 161361, 37, 0, 30, "Input"],
Cell[5060018, 161363, 3408, 73, 197, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5063487, 161443, 259, 4, 30, "Input"],
Cell[5063749, 161449, 22151, 721, 296, 9725, 563, "GraphicsData", 
"PostScript", "Graphics"],
Cell[5085903, 162172, 169, 4, 29, "Output"],

Cell[CellGroupData[{
Cell[5086097, 162180, 31, 0, 30, "Input"],
Cell[5086131, 162182, 53, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5086221, 162188, 274, 4, 30, "Input"],
Cell[5086498, 162194, 24035, 659, 186, 6272, 435, "GraphicsData", 
"PostScript", "Graphics"],
Cell[5110536, 162855, 167, 4, 29, "Output"],

Cell[CellGroupData[{
Cell[5110728, 162863, 31, 0, 30, "Input"],
Cell[5110762, 162865, 53, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5110876, 162873, 92, 3, 48, "Input"],

Cell[CellGroupData[{
Cell[5110993, 162880, 71, 1, 30, "Input"],
Cell[5111067, 162883, 100, 2, 29, "Output"],

Cell[CellGroupData[{
Cell[5111192, 162889, 314, 4, 30, "Input"],
Cell[5111509, 162895, 82, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5111628, 162901, 619, 9, 48, "Input"],
Cell[5112250, 162912, 447, 8, 145, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5112758, 162927, 72, 1, 30, "Input"],
Cell[5112833, 162930, 52, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5112922, 162936, 86, 1, 30, "Input"],
Cell[5113011, 162939, 227, 5, 25, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

